
(function (factory)
{
	// UMD module (AMD, CommonJS, and Global).
	// Good documentation on modules:
	// http://2ality.com/2015/12/babel-commonjs.html
	// http://davidbcalhoun.com/2014/what-is-amd-commonjs-and-umd

	// CommonJS module.
	// Enables support for bundlers such as WebPack.
	if (typeof module === "object" && typeof module.exports === "object")
	{
		// Register module the same way Babel does:
		// http://2ality.com/2015/12/babel-commonjs.html#how-babel-compiles-es6-modules-to-commonjs

		var fit = factory();

		// Export as one large module (Fit)
		// module.exports.Fit = fit;

		// Export each namespace of Fit.UI as individual modules (Array, Core, Dom, etc.)
		// This way we can do this: import { Controls as c, Dom as d } from "fit-ui"
		module.exports = fit;
		// module.exports.default = fit; // DISABLED, breaks IE8

		Object.defineProperty(module.exports, "__esModule", { value: true });
	}

	// AMD module for runtime loading.
	// Usage: require(["libs/Fit.UI/Fit.UI.js"], function(fit) { console.log("Loaded", fit); });
	// Make sure to load RequireJS first - demo: https://codepen.io/anon/pen/KvMGNa?editors=0010
	else if (typeof define === "function" && define.amd)
	{
		define(["require", "exports"], factory);
	}

	// Global instance
	else if (window)
	{
		var src = document.scripts[document.scripts.length-1].src;
		var paramSeparatorPos = src.indexOf("?");

		if (paramSeparatorPos > -1)
		{
			// Argument(s) found in src reference - add instance to custom global variable name.
			// First argument is expected to be the name of the Fit instance.

			var name = src.substring(paramSeparatorPos + 1, (src.indexOf("&") === -1 ? src.length : src.indexOf("&")));

			if (name.indexOf("=") === -1) // Make sure name was defined in the form of e.g. ?Fit and not ?arg=val
			{
				window[name] = factory();
			}
			else // Name was not valid, it was in the form of ?arg=val
			{
				window.Fit = factory();
			}
		}
		else
		{
			window.Fit = factory();
		}
	}
})(function()
{
	//"use strict"; // Not supported by IE9 and below - may cause different runtime behaviour - https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Strict_mode
	var fitInstance =

(function(){

;(function() // Terminate script if browser is not capable of running Fit.UI
{
	if (!window.JSON || !window.NodeList) // JSON and NodeList are not available on IE7 and older
	{
		if (navigator.userAgent.indexOf("MSIE") > -1)
			throw new Error("Browser not supported - Internet Explorer 8 or newer is required - make sure Compatibility View is not enabled");
		else
			throw new Error("Browser not supported");
	}

	return true;
})();

;(function() // Prevent Legacy IE from choking if e.g. console.log(..) is called without developer tools open
{
	if (!window.console)
		window.console = {};

	var shims = [ "log", "debug", "info", "error", "warn", "trace" ];

	for (var i = 0 ; i < shims.length ; i++)
	{
		if (!console[shims[i]])
			console[shims[i]] = function() {};
	}
})();


/// <container name="Fit.Core">
/// 	Core features extending the capabilities of native JS
/// </container>
var Fit = {};
Fit.Core = {};

/// <function container="Fit.Core" name="Extend" access="public" static="true" returns="object">
/// 	<description>
/// 		Extend any object with the public members of a super class.
///
/// 		MyClass = function(controlId)
/// 		{
/// 			&#160;&#160;&#160;&#160; Fit.Core.Extend(this, MySuperClass).Apply();
/// 		}
///
/// 		The code above defines a class called MyClass which extends from MySuperClass.
/// 		Use Apply() to pass variables to the super class constructor as shown below:
///
/// 		Male = function(name, age)
/// 		{
/// 			&#160;&#160;&#160;&#160; Fit.Core.Extend(this, Person).Apply("Male", name, age);
/// 		}
///
/// 		Notice that calling just Extend(..) without calling Apply() on the object returned,
/// 		will not cause extension to occure. Apply() must be called, with or without parameters.
///
/// 		Notice that Fit.UI supports multiple inheritance. Be careful not to extend from multiple
/// 		classes implementing functions with identical names, or at least be aware that the last
/// 		class from which the derivative extends, takes precedence.
/// 	</description>
/// 	<param name="subInstance" type="object"> Instance of sub class to extend </param>
/// 	<param name="superType" type="function"> Class (function) to extend from </param>
/// </function>
Fit.Core.Extend = function(subInstance, superType)
{
	Fit.Validation.ExpectIsSet(subInstance);
	Fit.Validation.ExpectFunction(superType);

	// Notice that we support multiple inheritance. For that reason we
	// cannot do something like the code below to support instanceof:
	// Human.prototype = Object.create(Creature.prototype);
	// Human.prototype.constructor = Creature;
	// As an alternative to instanceof, use Fit.Core.InstanceOf(..).

	var binder =
	{
		Apply: function()
		{
			superType.apply(subInstance, arguments);

			// Support for Fit.Core.Extends(..)

			if (!subInstance._internal)
				subInstance._internal = {};

			if (!subInstance._internal.Extends)
				subInstance._internal.Extends = [];

			Fit.Array.Add(subInstance._internal.Extends, superType);
		}
	}
	return binder;
}

/// <function container="Fit.Core" name="Extends" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Returns boolean indicating whether given object is an extension of a given super type - see Fit.Core.Extend(..).
/// 		Also look into Fit.Core.InstanceOf(..) which may provide the desired behaviour.
/// 	</description>
/// 	<param name="instance" type="object"> Object instance </param>
/// 	<param name="superType" type="function"> Reference to super class (function) </param>
/// </function>
Fit.Core.Extends = function(instance, superType)
{
	Fit.Validation.ExpectIsSet(instance);
	Fit.Validation.ExpectFunction(superType);

	return (instance._internal !== undefined && instance._internal.Extends !== undefined && Fit.Array.Contains(instance._internal.Extends, superType) === true);
}

/// <function container="Fit.Core" name="InstanceOf" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Returns boolean indicating whether given object is an instance or extension of a given class type - see Fit.Core.Extend(..).
/// 		This is equivalent of: var result = (obj instanceof MyType || Fit.Core.Extends(obj, MyType));
/// 	</description>
/// 	<param name="instance" type="object"> Object instance </param>
/// 	<param name="type" type="function"> Reference to class (function) </param>
/// </function>
Fit.Core.InstanceOf = function(instance, type)
{
	return (instance instanceof type || Fit.Core.Extends(instance, type) === true);
}

/// <function container="Fit.Core" name="CreateOverride" access="public" static="true" returns="function">
/// 	<description>
/// 		Create a function override for any given function using the approach below.
///
/// 		this.SayHello = function(name) { alert("Hello " + name); }
/// 		this.SayHello = Fit.Core.CreateOverride(this.SayHello, function(name)
/// 		{
/// 			console.log(name + " logged in");
/// 			console.log(name + " is using the following browser: " + navigator.userAgent);
///
/// 			base(name); // Call original SayHello function
/// 		});
///
/// 		Notice how base(..) allows us to call the original function.
/// 	</description>
/// 	<param name="originalFunction" type="function"> Reference to function to override </param>
/// 	<param name="newFunction" type="function"> Reference to replacement function </param>
/// </function>
Fit.Core.CreateOverride = function(originalFunction, newFunction)
{
	Fit.Validation.ExpectFunction(originalFunction);
	Fit.Validation.ExpectFunction(newFunction);

	return function()
	{
		var orgBase = window.base; // May already exist
		window.base = originalFunction; // Globally accessible base function

		var error = null;
		var result = undefined;

		try // Make sure we can clean up globally accessible base function in case of errors
		{
			// The arguments variable is actually not an ordinary array
			// (see https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Functions/arguments).
			// Browsers implementing ECMAScript prior to version 5 (e.g. IE8) require apply(..) to be
			// called with an ordinary array containing the arguments.
			var args = [];

			for (var i = 0 ; i < arguments.length ; i++)
				args[i] = arguments[i];

			if (args.length > 0)
				result = newFunction.apply(this, args);
			else
				result = newFunction.apply(this);
		}
		catch (err)
		{
			error = err;
		}

		if (orgBase)
		{
			window.base = orgBase;
		}
		else
		{
			try
			{
				delete window.base; // Fails in IE8 with "Object doesn't support this action"
			}
			catch (err)
			{
				window.base = undefined;
			}
		}

		if (error !== null)
			Fit.Validation.ThrowError(error);

		if (result !== undefined)
			return result;
	}
}

/// <function container="Fit.Core" name="CreateDebouncer" access="public" static="true" returns="Fit.CoreTypeDefs.DebounceFunction">
/// 	<description>
/// 		Create a debouncing function that delays execution the specified number of milliseconds.
/// 		Invoking function multiple times merely postpone execution the specified number of milliseconds.
/// 		This can greatly increase performance for expensive operations invoked often.
/// 	</description>
/// 	<param name="func" type="function"> Reference to function to invoke </param>
/// 	<param name="timeout" type="integer"> Number of milliseconds to postpone execution </param>
/// 	<param name="thisArg" type="any" default="undefined"> The value 'this' resolves to within debounced function </param>
/// </function>
Fit.Core.CreateDebouncer = function(func, timeoutMilliseconds, thisArg)
{
	var timeout = -1;
	var lastArgs = [];

	/// <container name="Fit.CoreTypeDefs.DebounceFunction">
	/// </container>
	var d =
	{
		/// <function container="Fit.CoreTypeDefs.DebounceFunction" name="Cancel" access="public">
		/// 	<description> Cancel debounced function if scheduled for execution </description>
		/// </function>
		Cancel: function()
		{
			if (timeout !== -1)
			{
				clearTimeout(timeout);
				lastArgs = [];
				timeout = -1;
			}
		},

		/// <member container="Fit.CoreTypeDefs.DebounceFunction" name="Invoke" type="function">
		/// 	<description> Schedule or re-schedule execution of function </description>
		/// </member>
		Invoke: function() // Defined as a <member> of type function to allow variable arguments which <function> does not allow
		{
			if (timeout !== -1)
			{
				clearTimeout(timeout);
			}

			lastArgs = arguments;

			timeout = setTimeout(function()
			{
				var args = lastArgs;

				lastArgs = [];
				timeout = -1;

				func.apply(thisArg || this, args); // Might call Invoke again, so do not do cleanup below this line!
			}, timeoutMilliseconds);
		},

		/// <function container="Fit.CoreTypeDefs.DebounceFunction" name="Flush" access="public">
		/// 	<description> Force execution of debounced function if scheduled for execution </description>
		/// </function>
		Flush: function()
		{
			if (timeout !== -1)
			{
				var args = lastArgs; // Cancel() clears lastArgs
				d.Cancel();
				func.apply(thisArg || this, args);
			}
		}
	};

	return d;
}

/// <function container="Fit.Core" name="IsEqual" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Compare two JavaScript objects to determine whether they are identical.
/// 		Returns True if objects are identical (equal), otherwise False.
/// 		Functions are compared by reference, not by value.
/// 		Custom properties set on native JS objects (e.g. Array.XYZ) are not compared, only
/// 		values are. Naturally JSON objects will be fully compared, including all properties.
/// 		Be aware of self referencing variables and circular structures, which
/// 		will cause an infinite loop, and eventually a stack overflow exception.
/// 		DOM objects and window/frame instances are not supported.
/// 	</description>
/// 	<param name="jsObj1" type="object"> JS object to compare agains second JS object </param>
/// 	<param name="jsObj2" type="object"> JS object to compare agains first JS object </param>
/// </function>
Fit.Core.IsEqual = function(jsObj1, jsObj2)
{
	// TEST CASE: Example below is supposed to return: TRUE!
	/*var f1 = function() { alert("Hello"); }
	var f2 = f1;
	Fit.Core.IsEqual(
	{
		str: "Hello world",
		num: 123,
		dec: 123.321,
		date: new Date("2014-12-01 13:02:23"),
		bool: true,
		bool2: false,
		arr: [100, 200, 250, 400],
		arr2: ["Hello", "world"],
		arr3: [123, "hello", true, false, new Date("1990-01-20"), [1,2,3], { x: { "hapsen": f1, "hello": new Array(1,2,3) } }],
		obj: { a: 123, b: 123.321, c: true, d: false, e: new Date("1993-06-25"), f: "hello", g: null, h: undefined }
	},
	{
		str: "Hello world",
		num: 123,
		dec: 123.321,
		date: new Date("2014-12-01 13:02:23"),
		bool: true,
		bool2: false,
		arr: [100, 200, 250, 400],
		arr2: ["Hello", "world"],
		arr3: [123, "hello", true, false, new Date("1990-01-20"), [1,2,3], { x: { "hapsen": f2, "hello": new Array(1,2,3) } }],
		obj: { a: 123, b: 123.321, c: true, d: false, e: new Date("1993-06-25"), f: "hello", g: null, h: undefined }
	});*/

	if (typeof(jsObj1) !== typeof(jsObj2))
		return false;

	if ((jsObj1 === undefined && jsObj2 === undefined) || (jsObj1 === null && jsObj2 === null))
	{
		return true;
	}
	else if (typeof(jsObj1) === "string" || typeof(jsObj1) === "boolean")
	{
		return (jsObj1 === jsObj2);
	}
	else if (typeof(jsObj1) === "number")
	{
		if (isNaN(jsObj1) === true && isNaN(jsObj2) === true) // NaN variables are not comparable!
			return true;
		else
			return (jsObj1 === jsObj2);
	}
	else if (jsObj1 instanceof Date && jsObj2 instanceof Date)
	{
		return (jsObj1.getTime() === jsObj2.getTime());
	}
	else if (jsObj1 instanceof Array && jsObj2 instanceof Array)
	{
		if (jsObj1.length !== jsObj2.length)
			return false;

		for (var i = 0 ; i < jsObj1.length ; i++)
		{
			if (Fit.Core.IsEqual(jsObj1[i], jsObj2[i]) === false)
				return false;
		}

		return true;
	}
	else if (typeof(jsObj1) === "object" && typeof(jsObj2) === "object" && jsObj1 !== null && jsObj2 !== null) // typeof(null) returns "object"
	{
		var identical = true;
		var keys = Fit.Array.Merge(Fit.Array.GetKeys(jsObj1), Fit.Array.GetKeys(jsObj2));

		Fit.Array.ForEach(keys, function(k)
		{
			if (Fit.Core.IsEqual(jsObj1[k], jsObj2[k]) === false)
			{
				identical = false;
				return false;
			}
		});

		return identical;
	}
	else if (typeof(jsObj1) === "function" && typeof(jsObj2) === "function")
	{
		// Returns True in the following situation:
		// var f1 = function() { alert("Hello"); }
		// var f2 = f1;
		// Fit.Core.IsEqual(f1, f2);

		// Returns False in the following situation:
		// var f1 = function() { alert("Hello"); }
		// var f2 = function() { alert("Hello"); }
		// Fit.Core.IsEqual(f1, f2);

		return (jsObj1 === jsObj2);
	}

	return false;
}

/// <container name="Fit.Core.MergeOverwriteBehaviour">
/// 	Merge behaviour
/// </container>
Fit.Core.MergeOverwriteBehaviour = // Enums must exist runtime
{
	/// <member container="Fit.Core.MergeOverwriteBehaviour" name="Always" access="public" static="true" type="string" default="Always">
	/// 	<description> Always overwrite property values from target object with property values from merge object (default behaviour) </description>
	/// </member>
	Always: "Always",

	/// <member container="Fit.Core.MergeOverwriteBehaviour" name="SkipNullAndUndefined" access="public" static="true" type="string" default="SkipNullAndUndefined">
	/// 	<description> Always overwrite property values from target object with property values from merge object, except values from merge object that are Null or Undefined </description>
	/// </member>
	SkipNullAndUndefined: "SkipNullAndUndefined",

	/// <member container="Fit.Core.MergeOverwriteBehaviour" name="Never" access="public" static="true" type="string" default="Never">
	/// 	<description>
	/// 		Never overwrite property values from target object - only add missing property values from merge object
	/// 	</description>
	/// </member>
	Never: "Never"
};

/// <function container="Fit.Core" name="Merge" access="public" static="true" returns="$ObjectTypeA + $ObjectTypeB">
/// 	<description>
/// 		Deep merges two objects and returns the resulting object.
/// 		Take notice of the behaviour and restriction of Fit.Core.Clone(..) since
/// 		the target object is first cloned using that function. The resulting object is
/// 		then enriched with the data from the merge object.
/// 		Property values on the merge object takes precedence over property values on the
/// 		target object. Arrays are not merged but merely replaced if defined on the merge object.
/// 	</description>
/// 	<param name="targetObject" type="$ObjectTypeA"> Target object </param>
/// 	<param name="mergeObject" type="$ObjectTypeB"> Merge object </param>
/// 	<param name="mergeObjectOverwriteBehaviour" type="Fit.Core.MergeOverwriteBehaviour" default="undefined"> Overwrite behaviour for merge object </param>
/// </function>
Fit.Core.Merge = function(targetObject, mergeObject, mergeObjectOverwriteBehaviour)
{
	Fit.Validation.ExpectObject(targetObject);
	Fit.Validation.ExpectObject(mergeObject);
	Fit.Validation.ExpectStringValue(mergeObjectOverwriteBehaviour, true);

	/* // Test data
	f1 = function() { alert("Hello"); };
	f2 = function() { alert("Hello2"); };
	x = {
		str: "Hello world",
		num: 123,
		dec: 123.321,
		num2: 1,
		num3: parseFloat("abc"),
		date: new Date("2014-12-01 13:02:23"),
		bool: true,
		bool2: false,
		arr: [100, 200, 250, 400],
		arr2: ["Hello", "world"],
		arr3: [123, "hello", true, false, new Date("1990-01-20"), [1,2,3], { x: { "hapsen": f1, "hello": new Array(1,2,3) } }],
		obj: { a: 123, b: 123.321, c: true, d: false, e: new Date("1993-06-25"), f: "hello", g: null, h: undefined, propNotFoundOnMergeObj: { name: "one", age: 11 } },
		specialProp: { x: true, y: undefined, z: null, date: new Date() },
		oneUn: undefined
	};
	y = {
		str: "Hello world 2",
		num: 222,
		dec: 222.222,
		num2: parseInt("abc"),
		num3: 22.2,
		date: new Date("2222-02-02 22:22:22"),
		bool: false,
		bool2: true,
		arr: [2, 22, 222, 2222, 22222, 2222222222],
		arr2: ["Hello2", "world2"],
		arr3: [222, "hello2", false, true, new Date("2002-02-02"), [2], { x: { "hapsen2": f2, "hello2": new Array(2,2,2,2,2,2,2,2,2,2) } }],
		obj: { a: 2, b: 2.2, c: false, d: true, e: new Date("2202-02-22"), f: "hello2", g: {}, h: null, newProp: { name: "two", age: 22, gender: "female" } },
		two: { x: 2, y: true, z: undefined },
		twoUn: undefined
	};
	var backupX = Fit.Core.Clone(x);
	var backupY = Fit.Core.Clone(y);
	var merged1 = Fit.Core.Merge(x, y);
	var merged2 = Fit.Core.Merge(y, x);
	console.log(merged1);
	console.log(merged2);
	console.log("Merges equal:", Fit.Core.IsEqual(merged1, merged2)); // Expecting False
	console.log("X untouched:", Fit.Core.IsEqual(x, backupX)); // Expecting True
	console.log("Y untouched:", Fit.Core.IsEqual(y, backupY)); // Expecting True*/

	var isObject = function(val)
	{
		return (val !== undefined && val !== null && typeof(val) === "object" && (val instanceof Date) === false && (val instanceof Array) === false);
	}

	var newObject = Fit.Core.Clone(targetObject);

	Fit.Array.ForEach(mergeObject, function(prop)
	{
		if (isObject(newObject[prop]) && isObject(mergeObject[prop]))
		{
			newObject[prop] = Fit.Core.Merge(newObject[prop], mergeObject[prop], mergeObjectOverwriteBehaviour);
		}
		else
		{
			if (mergeObjectOverwriteBehaviour === Fit.Core.MergeOverwriteBehaviour.SkipNullAndUndefined && (mergeObject[prop] === null || mergeObject[prop] === undefined))
			{
				return; // Skip - ignore values of Null and Undefined from merge object
			}
			else if (mergeObjectOverwriteBehaviour === Fit.Core.MergeOverwriteBehaviour.Never && prop in newObject)
			{
				return; // Skip - never update values from target object, only add missing values
			}

			newObject[prop] = mergeObject[prop];
		}
	});

	return newObject;
}

/// <function container="Fit.Core" name="Clone" access="public" static="true" returns="$ObjectType">
/// 	<description>
/// 		Clone JavaScript object. Supported object types and values:
/// 		String, Number, Boolean, Date, Array, (JSON) Object, Function, Undefined, Null, NaN,
/// 		Infinity.
/// 		Variables defined as undefined are left out of clone,
/// 		since an undefined variable is equal to a variable defined as undefined.
/// 		Notice that Arrays and Objects can contain supported object types and values only.
/// 		Functions are considered references, and as such the cloned object will reference
/// 		the same functions.
/// 		Custom properties set on native JS objects (e.g. Array.XYZ) are not cloned, only
/// 		values are. Naturally custom (JSON) objects will be fully cloned, including all
/// 		properties. Both arrays and custom (JSON) objects are cloned recursively.
/// 		Be aware of self referencing variables and circular structures, which
/// 		will cause an infinite loop, and eventually a stack overflow exception.
/// 		DOM objects and window/frame instances are not supported.
/// 	</description>
/// 	<param name="obj" type="$ObjectType"> JS object to clone </param>
/// </function>
Fit.Core.Clone = function(obj)
{
	// TODO - Known problem:
	// var a = new SomeClass();
	// var b = (a instanceOf SomeClass);
	// var c = (SMCore.Clone(a) instanceOf SomeClass);
	// Variable b is True as expected, while variable c is False!
	// TODO: Restore/preserve support for instanceof!

	// TEST CASE: Example below is supposed to return: TRUE!
	/*var f1 = function() { alert("Hello"); }
	var x =
	{
		str: "Hello world",
		num: 123,
		dec: 123.321,
		date: new Date("2014-12-01 13:02:23"),
		bool: true,
		bool2: false,
		arr: [100, 200, 250, 400],
		arr2: ["Hello", "world"],
		arr3: [123, "hello", true, false, new Date("1990-01-20"), [1,2,3], { x: { "hapsen": f1, "hello": new Array(1,2,3) } }],
		obj: { a: 123, b: 123.321, c: true, d: false, e: new Date("1993-06-25"), f: "hello", g: null, h: undefined }
	};
	var y = SMCore.Clone(x);
	console.log("Is equal: " + SMCore.IsEqual(x, y));*/

	// Clone object by serializing it into a JSON string, and parse it back into a JS object

	var serialized = JSON.stringify(obj); // Returns undefined if obj is either undefined or a function (these are not serialized)
	var clone = ((serialized !== undefined) ? JSON.parse(serialized) : serialized); // parse(..) throws error if argument is undefined

	// Fixes
	//  - Dates are serialized into strings - turn back into Date instances.
	//  - Functions are not serialized (discarded) - add function reference to clone
	//  - Number variables with a value of NaN is serialized into Null - convert to NaN

	var fixClone = null;
	fixClone = function(org, clo)
	{
		if (org instanceof Date) // Dates are turned into string representations - turn back into Date instances
		{
			return new Date(org.getTime());
		}
		else if (typeof(org) === "function") // Functions are not serialized - use same reference as original object
		{
			return org;
		}
		else if (typeof(org) === "number" && isNaN(org) === true) // NaN is turned into Null - turn back into NaN
		{
			return parseInt("");
		}
		else if (typeof(org) === "number" && org === Infinity) // Infinity is turned into Null - turn back into Infinity
		{
			return Infinity;
		}
		else if (org instanceof RegExp)
		{
			var flags = "";

			if (org.ignoreCase)
				flags += "i";
			if (org.global)
				flags += "g";
			if (org.multiline)
				flags += "m";
			if (org.sticky) // Notice that sticky is not supported in legacy IE
				flags += "y";

			return new RegExp(org.source, flags);
		}
		else if (org && typeof(org) === "object") // Recursively fix children (object/array)
		{
			for (var p in org)
				clo[p] = fixClone(org[p], clo[p]);
		}

		return clo;
	};

	clone = fixClone(obj, clone);

	// Done, clone is now identical to original object - SMCore.IsEqual(obj, clone) should return True

	return clone;
}


// INTERNAL

Fit._internal =
{
	Core:
	{
		VersionInfo: { Major: 2, Minor: 18, Patch: 2 } // Do NOT modify format - version numbers are programmatically changed when releasing new versions - MUST be on a separate line!
	}
};

Fit._internal.Core.EnsureStyles = function()
{
	if (Fit._internal.Core.StylesEnsured === true)
		return;

	Fit._internal.Core.StylesEnsured = true;

	Fit.Events.OnDomReady(function() // In case function is called too early in which case document.body will be null
	{
		var elm = Fit.Dom.CreateElement("<div class='FitUiStyleCheck'></div>");
		Fit.Dom.Add(document.body, elm);

		if (Fit.Dom.GetComputedStyle(elm, "width") !== "20px")
		{
			Fit.Browser.Log("Lazy loading Fit.UI stylesheet. It is recommended to add a stylesheet reference to Fit.UI.min.css to prevent temporarily unstyled content.");
			Fit.Loader.LoadStyleSheet(Fit.GetUrl() + "/Fit.UI.min.css?cacheKey=" + Fit.GetVersion().Version);
		}

		Fit.Dom.Remove(elm);
	});
}

;(function()
{
	// Find Base URL - e.g. http://server.com/libs/fitui

	// Trying to aquire Fit.UI script by ID first, as this is most reliable. Obtaining the last script from the
	// scripts collection is less reliable as some browser extensions manipulate the collection while scripts are loading.
	var script = document.querySelector("script#FitUI") || document.querySelector("script#fitui") || document.scripts[document.scripts.length - 1];

	var src = script && script.src || null;

	if (!src)
	{
		// Fit.UI was loaded dynamically as a module or bundled with e.g. WebPack.
		src = location.href;
	}

	Fit._internal.BaseUrl = src.substring(0, src.lastIndexOf("/"));

	// Calculate Base Path - e.g. / (unlikely scenario having Fit.UI located at the root though) or /libs/fitui

	var path = Fit._internal.BaseUrl.replace("http://", "").replace("https://", "");
	Fit._internal.BasePath = ((path.indexOf("/") !== -1) ? path.substring(path.indexOf("/")) : "/");
})();

/// <container name="Fit.CoreTypeDefs.VersionInfo">
/// 	<description> Version information </description>
/// 	<member name="Major" type="integer"> Major version number </member>
/// 	<member name="Minor" type="integer"> Minor version number </member>
/// 	<member name="Patch" type="integer"> Patch version number </member>
/// 	<member name="Version" type="string"> String version number in the format Major.Minor.Patch </member>
/// </container>

/// <function container="Fit" name="GetVersion" access="public" static="true" returns="Fit.CoreTypeDefs.VersionInfo">
/// 	<description>
/// 		Get Fit.UI version object containing the following properties:
///			Major (integer), Minor (integer), Patch (integer), Version (string representing Major.Minor.Patch).
/// 	</description>
/// </function>
Fit.GetVersion = function()
{
	var info = Fit.Core.Clone(Fit._internal.Core.VersionInfo);
	info.Version = info.Major + "." + info.Minor + "." + info.Patch;

	return info;
}

/// <function container="Fit" name="GetUrl" access="public" static="true" returns="string">
/// 	<description> Get fully qualified URL to Fit.UI on server - e.g. http://server.com/libs/fitui </description>
/// </function>
Fit.GetUrl = function()
{
	if (Fit._internal.BaseUrlOverride !== undefined)
		return Fit._internal.BaseUrlOverride;

	return Fit._internal.BaseUrl;
}

/// <function container="Fit" name="GetPath" access="public" static="true" returns="string">
/// 	<description> Get absolute path to Fit.UI on server - e.g. /libs/fitui </description>
/// </function>
Fit.GetPath = function()
{
	if (Fit._internal.BasePathOverride !== undefined)
		return Fit._internal.BasePathOverride;

	return Fit._internal.BasePath;
}

/// <function container="Fit" name="SetPath" access="public" static="true">
/// 	<description>
/// 		Set path to Fit.UI on server - e.g. libs/fitui.
/// 		This may be necessary if Fit.UI is loaded dynamically
/// 		using RequireJS or bundled using e.g. WebPack.
/// 		Changing the path affects the return value of both
/// 		GetUrl() and GetPath(), and from where Fit.UI will
/// 		load resources dynamically.
/// 	</description>
/// 	<param name="basePath" type="string"> Absolute or relative path to folder containing Fit.UI </param>
/// </function>
Fit.SetPath = function(basePath)
{
	Fit.Validation.ExpectStringValue((basePath === null ? "-" : basePath));

	if (basePath === null)
	{
		delete Fit._internal.BasePathOverride;
		delete Fit._internal.BaseUrlOverride;

		return;
	}

	// Remove trailing slash if found
	if (basePath !== "/" && basePath.lastIndexOf("/") === basePath.length - 1)
	{
		basePath = basePath.substring(0, basePath.length - 1);
	}

	var rootUrl = location.protocol + "//" + location.host; // E.g. http://my-domain.com

	// Both GetPath() and GetUrl() return values without trailing slashes.
	// E.g. libs/fitui and http://host/libs/fitui.
	// However, when installed to the root, this is indicated by a slash returned
	// from GetPath() while GetUrl() keeps returning a URL without a trailing slash.
	// E.g. / and http://my-domain.com.

	Fit._internal.BasePathOverride = basePath;

	if (basePath === "/")
	{
		Fit._internal.BaseUrlOverride = rootUrl; //curUrl;
	}
	else if (basePath.indexOf("/") === 0) // Absolute path
	{
		Fit._internal.BaseUrlOverride = rootUrl + basePath;
	}
	else // Relative path
	{
		var curPath = location.pathname.substring(0, location.pathname.lastIndexOf("/")); // E.g. "" (empty for root) or /path/to/folder
		Fit._internal.BaseUrlOverride = rootUrl + curPath + "/" + basePath;
	}
}

/// <function container="Fit" name="SetUrl" access="public" static="true">
/// 	<description>
/// 		Set URL to Fit.UI on server - e.g. http://cdn/libs/fitui/.
/// 		This may be necessary if Fit.UI is loaded dynamically
/// 		from a foreign domain such as a CDN (Content Delivery Network).
/// 		Changing the URL affects the return value of both
/// 		GetUrl() and GetPath(), and from where Fit.UI will
/// 		load resources dynamically.
/// 	</description>
/// 	<param name="baseUrl" type="string"> Full URL to folder containing Fit.UI </param>
/// </function>
Fit.SetUrl = function(baseUrl) // E.g. http://foreign-host/path/to/Fit.UI/
{
	Fit.Validation.ExpectStringValue((baseUrl === null ? "-" : baseUrl));

	if (baseUrl === null)
	{
		Fit.SetPath(null); // Reset Path and URL overriding
		return;
	}

	var url = Fit.Browser.ParseUrl(baseUrl);

	Fit.SetPath(url.FullPath); // Using FullPath rather than Path in case URL has been specified using http://host/path/to/Fit.UI (without trailing slash which makes Fit.UI a resource and not a folder, resulting in it being excluded from Path)

	var newUrl = (url.Url.lastIndexOf("/") === url.Url.length - 1 ? url.Url.substring(0, url.Url.length - 1) : url.Url); // Both GetPath() and GetUrl() return values without trailing slashes - e.g. libs/fitui and http://host/libs/fitui - so make sure no trailing slash is present
	Fit._internal.BaseUrlOverride = newUrl;
}

/// <container name="Fit.Validation">
/// 	Validation logic
/// </container>
Fit.Validation = {};

Fit._internal.Validation = {};
Fit._internal.Validation.DebugMode = true;
Fit._internal.Validation.Clone = null;

// Ensure object types not found in all browsers
(function()
{
	// window.event is of type MSEventObj in IE9 and above.
	// Type used by Fit.Validation.ExpectEvent.
	if (!window.MSEventObj)
		window.MSEventObj = function() {};

	// StaticNodeList type is produced by document.querySelectorAll(..) in Legacy IE.
	// FileList type is not defined in Legacy IE.
	// These types are used by Fit._internal.Validation.IsCollectionType.
	if (!window.StaticNodeList)
		window.StaticNodeList = function() {};
	if (!window.FileList)
		window.FileList = function() {};

	// File type not defined in Legacy IE.
	// Make sure we can use Fit.Validation.ExpectInstance(selectedFileFromInput, File, true)
	if (!window.File)
		window.File = function() {};

	// Some versions of Firefox temporarily removed NamedNodeMap.
	// They renamed it to MozNamedAttrMap, but it was later restored.
	// https://bugzilla.mozilla.org/show_bug.cgi?id=858344
	if (!window.NamedNodeMap)
		window.NamedNodeMap = function() {};
})();

// ==========================================================
// Type checking
// ==========================================================

/// <function container="Fit.Validation" name="ExpectObject" access="public" static="true">
/// 	<description> Throws error if passed object is not a valid object such as { Name: 'Jimmy', Age: 34 } </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectObject = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (!val || val.constructor !== Object)
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid object");
}

/// <function container="Fit.Validation" name="ExpectNumber" access="public" static="true">
/// 	<description> Throws error if passed object is not a number </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectNumber = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(val) !== "number" || isNaN(val) === true)
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid number");
}

/// <function container="Fit.Validation" name="ExpectInteger" access="public" static="true">
/// 	<description> Throws error if passed object is not an integer </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectInteger = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(val) !== "number" || val % 1 !== 0)
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid integer");
}

/// <function container="Fit.Validation" name="ExpectString" access="public" static="true">
/// 	<description> Throws error if passed object is not a string </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectString = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(val) !== "string")
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid string");
}

/// <function container="Fit.Validation" name="ExpectStringValue" access="public" static="true">
/// 	<description> Same as Fit.Validation.ExpectString(..), but string must contain an actual value if set (not be empty) </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectStringValue = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	Fit.Validation.ExpectString(val);

	if (val === "")
		Fit.Validation.ThrowError("String cannot be empty");
}

/// <function container="Fit.Validation" name="ExpectBoolean" access="public" static="true">
/// 	<description> Throws error if passed object is not a boolean </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectBoolean = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(val) !== "boolean")
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid boolean");
}

/// <function container="Fit.Validation" name="ExpectDate" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of Date </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectDate = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Date) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Date");
}

/// <function container="Fit.Validation" name="ExpectArray" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of Array </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectArray = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Array) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Array");
}

/// <function container="Fit.Validation" name="ExpectTypeArray" access="public" static="true">
/// 	<description>
/// 		Throws error if passed object is not an instance of Array
/// 		contaning only objects/values of type given by validation callback.
/// 		Example: Fit.Validation.ExpectTypeArray(arr, Fit.Validation.ExpectString)
/// 	</description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="typeValCallback" type="function"> Value validation callback </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectTypeArray = function(val, typeValCallback, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Array) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Array");

	// Validate types within array

	Fit.Validation.ExpectFunction(typeValCallback); // Make sure callback is valid

	Fit.Array.ForEach(val, function(v)
	{
		typeValCallback(v);
	});
}

/// <function container="Fit.Validation" name="ExpectInstanceArray" access="public" static="true">
/// 	<description>
/// 		Throws error if passed object is not an instance of Array
/// 		contaning only instances of specified type. Example:
/// 		Fit.Validation.ExpectInstanceArray(arr, Fit.Controls.TreeViewNode)
/// 	</description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="instanceType" type="object"> Instance type (constructor, e.g. Fit.Http.Request) </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectInstanceArray = function(val, instanceType, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Array) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Array");

	// Validate types within array

	Fit.Array.ForEach(val, function(v)
	{
		Fit.Validation.ExpectInstance(v, instanceType);
	});
}

/// <function container="Fit.Validation" name="ExpectDictionary" access="public" static="true">
/// 	<description>
/// 		Throws error if passed object is not a dictionary (associative array / object array),
/// 		contaning only objects/values of type given by validation callback.
/// 		Example: Fit.Validation.ExpectDictionary(dict, Fit.Validation.ExpectString)
/// 	</description>
/// 	<param name="val" type="object"> Dictionary to validate </param>
/// 	<param name="typeValCallback" type="function"> Value validation callback </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectDictionary = function(val, typeValCallback, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	Fit.Validation.ExpectObject(val);
	Fit.Validation.ExpectFunction(typeValCallback);

	Fit.Array.ForEach(val, function(key)
	{
		Fit.Validation.ExpectStringValue(key);
		typeValCallback(val[key]);
	});
}

/// <function container="Fit.Validation" name="ExpectCollection" access="public" static="true">
/// 	<description> Throws error if passed object is not a collection that can be iterated </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectCollection = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (Fit._internal.Validation.IsCollectionType(val) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid collection");
}

/// <function container="Fit.Validation" name="ExpectRegExp" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of RegExp </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectRegExp = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (val instanceof RegExp === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of RegExp");
}

/// <function container="Fit.Validation" name="ExpectElement" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of HTMLElement </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectElement = function(val, allowNotSet) // DOMElement (actually HTMLElement)
{
	// All elements within the HTML DOM tree inherits from Element except for Comments, CDATA, and Text nodes.
	// But most elements from an XML document (also excluding Comments, CDATA, and Text nodes) also
	// inherit from Element.
	// (new DOMParser()).parseFromString("<root></root>", "text/xml").getElementsByTagName("root")[0] instanceof Element; // Returns true
	// However, Fit.UI is for building HTML applications, and is therefore not expected to handle XML elements.
	// Therefore, despite the function name ExpectElement, it actually ensures that the passed element is an instance of HTMLElement.
	// (new DOMParser()).parseFromString("<root></root>", "text/xml").getElementsByTagName("root")[0] instanceof HTMLElement; // Returns false

	// More about HTMLElement vs Element vs Node, as well as the nodeType property:
	// https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement
	// https://developer.mozilla.org/en-US/docs/Web/API/Element
	// https://developer.mozilla.org/en-US/docs/Web/API/Node
	// https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType
	// The inheritance chain looks like this: EventTarget <= Node <= Element <= HTMLElement

	if (allowNotSet === true && (val === undefined || val === null))
		return;

	// The HTMLElement interface is not available in legacy IE, in which case we use feature detection instead
	if ((window.HTMLElement && (val instanceof HTMLElement) === false) || (val instanceof Element) === false || (val.style instanceof CSSStyleDeclaration) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an HTMLElement");
}
Fit.Validation.ExpectDomElement = Fit.Validation.ExpectElement;		// Backward compatibility
Fit.Validation.ExpectElementNode = Fit.Validation.ExpectElement;	// Backward compatibility

// DISABLED - not in use internally, and not supported by legacy IE
// <function container="Fit.Validation" name="ExpectTextNode" access="public" static="true">
// 	<description> Throws error if passed object is not an instance of Text </description>
// 	<param name="val" type="object"> Object to validate </param>
// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
// </function>
/*Fit.Validation.ExpectTextNode = function(val, allowNotSet) // DOMText
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Text) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not a Text node");
}*/

// DISABLED - not in use internally, and not supported by legacy IE
// <function container="Fit.Validation" name="ExpectCommentNode" access="public" static="true">
// 	<description> Throws error if passed object is not an instance of Comment </description>
// 	<param name="val" type="object"> Object to validate </param>
// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
// </function>
/*Fit.Validation.ExpectCommentNode = function(val, allowNotSet) // DOMComment
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Comment) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not a Comment node");
}*/

/// <function container="Fit.Validation" name="ExpectNode" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of Element, Text, or Comment </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectNode = function(val, allowNotSet) // DOMNode
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	// The Text and Comment interfaces are not available in legacy IE so we use feature detection instead.
	// We assume we have a valid Node instance if it exposes common functions and properties of a Node instance,
	// and has the appropriate nodeType: 1 = Element, 3 = Text, 8 = Comment. More on node types here:
	// https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType
	if (!val.cloneNode || !val.nodeName || !val.nodeType || (val.nodeType !== 1 && val.nodeType !== 3 && val.nodeType !== 8))
		Fit.Validation.ThrowError("Value '" + val + "' is not a Node (Element, Text, or Comment)");
}

// <function container="Fit.Validation" name="ExpectTextNode" access="public" static="true">
// 	<description> Throws error if passed object is not an instance of Text </description>
// 	<param name="val" type="object"> Object to validate </param>
// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
// </function>
Fit.Validation.ExpectTextNode = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	// The Text interface is not available in legacy IE so we use feature detection instead.
	// We assume we have a valid Node instance if it exposes common functions and properties of a Node instance,
	// and has the appropriate nodeType: 1 = Element, 3 = Text, 8 = Comment. More on node types here:
	// https://developer.mozilla.org/en-US/docs/Web/API/Node/nodeType
	if (!val.cloneNode || !val.nodeName || val.nodeType !== 3)
		Fit.Validation.ThrowError("Value '" + val + "' is not a Text node");
}

// <function container="Fit.Validation" name="ExpectContentNode" access="public" static="true">
// 	<description> Throws error if passed object is not an instance of Element or Text </description>
// 	<param name="val" type="object"> Object to validate </param>
// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
// </function>
/*Fit.Validation.ExpectContentNode = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Element) === false && (val instanceof Text) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not a Content Node (Element or Text)");
}*/

/// <function container="Fit.Validation" name="ExpectWindow" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of Window </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectWindow = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof Window) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Window");
}

/// <function container="Fit.Validation" name="ExpectFunction" access="public" static="true">
/// 	<description> Throws error if passed object is not a valid function </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectFunction = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(val) !== "function")
		Fit.Validation.ThrowError("Value '" + val + "' is not a valid function");
}

/// <function container="Fit.Validation" name="ExpectEventTarget" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of EventTarget </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectEventTarget = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if (typeof(EventTarget) !== "undefined" && (val instanceof EventTarget) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of EventTarget");
	else if (!val.removeEventListener && !val.attachEvent) // IE8
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of EventTarget");
}

/// <function container="Fit.Validation" name="ExpectEvent" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of Event </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectEvent = function(val, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	// IE9 and above: window.event is now of type MSEventObj (legacy),
	// while an actual Event instance is passed to handlers as specified by W3C.
	if ((val instanceof Event) === false && (val instanceof MSEventObj) === false)
		Fit.Validation.ThrowError("Value '" + val + "' is not an instance of Event");
}

/// <function container="Fit.Validation" name="ExpectInstance" access="public" static="true">
/// 	<description> Throws error if passed object is not an instance of specified type </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// 	<param name="instanceType" type="object"> Instance type (constructor, e.g. Fit.Http.Request) </param>
/// 	<param name="allowNotSet" type="boolean" default="false"> Set True to allow object to be Null or Undefined </param>
/// </function>
Fit.Validation.ExpectInstance = function(val, instanceType, allowNotSet)
{
	if (allowNotSet === true && (val === undefined || val === null))
		return;

	if ((val instanceof instanceType) === true || Fit.Core.Extends(val, instanceType) === true)
		return;

	Fit.Validation.ThrowError("Unsupported object type passed");
}

/// <function container="Fit.Validation" name="ExpectIsSet" access="public" static="true">
/// 	<description> Throws error if passed object is either Null or Undefined </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// </function>
Fit.Validation.ExpectIsSet = function(val)
{
	if (Fit.Validation.IsSet(val) === false)
		Fit.Validation.ThrowError("Value not set");
}

// ==========================================================
// Misc.
// ==========================================================

/// <function container="Fit.Validation" name="IsSet" access="public" static="true" returns="boolean">
/// 	<description> Returns True if specified object is set (not Null or Undefined), otherwise False </description>
/// 	<param name="val" type="object"> Object to validate </param>
/// </function>
Fit.Validation.IsSet = function(obj)
{
	return (obj !== null && obj !== undefined);
}

/// <function container="Fit.Validation" name="ThrowError" access="public" static="true">
/// 	<description> Throw error and provide stack trace to browser console </description>
/// 	<param name="msg" type="string"> Error message </param>
/// </function>
Fit.Validation.ThrowError = function(msg)
{
	if (Fit._internal.Validation.DebugMode === true)
	{
		/*var stackTrace = Fit.Validation.GetStackTrace();
		alert("ThrowError: " + msg + ((stackTrace !== "") ? "\n\n" : "") + stackTrace);

		if (window.console && console.trace)
			console.trace();*/

		if (window.console && console.log) console.log("ThrowError: " + msg);
		if (window.console && console.trace) console.trace(); // Not all browsers produce a stack trace when an error is thrown (below)
	}

	throw new Error(msg); // Never change this behaviour - we should always re-throw the error to terminate execution
}

Fit.Validation.GetStackTrace = function() // NOTICE: captureStackTrace is a feature of V8/Chromium!
{
	if (window.Error === undefined || Error.captureStackTrace === undefined)
		return "";

	var obj = {};
	Error.captureStackTrace(obj, Fit.Validation.GetStackTrace);
	return obj.stack;
}

// ==========================================================
// DebugMode (enable/disable type checking)
// ==========================================================

/// <function container="Fit.Validation" name="Enabled" access="public" static="true">
/// 	<description> Set or get flag indicating whether type checking should take place (DebugMode) </description>
/// 	<param name="val" type="boolean"> True enables DebugMode, False disables DebugMode </param>
/// </function>
Fit.Validation.Enabled = function(val)
{
	if (Fit.Validation.IsSet(val) === true)
	{
		if (val === true)
		{
			Fit._internal.Validation.DebugMode = true;

			if (Fit._internal.Validation.Clone !== null) // Null if Debug Mode is already enabled
				Fit.Validation = Fit.Core.Clone(Fit._internal.Validation.Clone);
		}
		else
		{
			Fit._internal.Validation.DebugMode = false;

			if (Fit._internal.Validation.Clone === null)
				Fit._internal.Validation.Clone = Fit.Core.Clone(Fit.Validation);

			// Replace all type checking functions when Debug Mode is disabled
			for (var f in Fit.Validation)
				if (f.indexOf("Expect") === 0)
					Fit.Validation[f] = function() {}
		}
	}

	return (Fit._internal.Validation.DebugMode === true);
}

// ==========================================================
// Internal
// ==========================================================

Fit._internal.Validation.IsCollectionType = function(val) // Used by Fit.Validation and Fit.Array
{
	if (val === null || val === undefined)
		return false;
	else if (val instanceof Array)
		return true;
	else if (val instanceof NodeList)
		return true;
	else if (val instanceof StaticNodeList)
		return true;
	else if (val instanceof HTMLCollection)
		return true;
	else if (val instanceof NamedNodeMap)
		return true;
	else if (val instanceof FileList)
		return true;
	else if (val instanceof StyleSheetList)
		return true;
	else if (val instanceof CSSRuleList)
		return true;
	else if (val.callee && val.length !== null && val.length !== undefined) // Arguments array - https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Functions/arguments
		return true;

	return false;

	/*if (val === null || val === undefined)
		return false;
	if ((val instanceof NodeList) === false && (val instanceof StaticNodeList) === false && (val instanceof FileList) === false && (val instanceof HTMLCollection) === false && (val instanceof Array) === false && (!val.callee && (val.length === undefined || val.length === null)))
		return false;
	return true;*/
}

/// <container name="Fit.Array">
/// 	Functionality extending the capabilities of native JS arrays
/// </container>
Fit.Array = {};

/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="$Type[]"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:$Type) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="HTMLCollection"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:HTMLElement) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="NodeList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:Node) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="NamedNodeMap"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:Attr) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="FileList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:File) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="StyleSheetList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:StyleSheet) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="CSSRuleList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:CSSRule) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="object[]"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:object) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="ForEach" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through object properties and passes each property name to the provided callback function.
/// 		Returns boolean indicating whether iteration was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="obj" type="object"> Object containing properties to iterate through </param>
/// 	<param name="callback" type="(key:string) => boolean | void">
/// 		Callback function accepting properties from the object, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
Fit.Array.ForEach = function(obj, callback) // obj not validated - passing null/undefined is allowed - no iteration is performed in this case
{
	Fit.Validation.ExpectFunction(callback);

	if (Fit._internal.Validation.IsCollectionType(obj) === true)
	{
		var count = obj.length;
		var res = false;

		for (var i = 0 ; i < obj.length ; i++)
		{
			res = callback(obj[i]);

			if (obj.length !== count)
				Fit.Validation.ThrowError("Collection was modified while iterating objects");

			if (res === false)
				return false;
		}
	}
	else if (typeof(obj) === "object")
	{
		for (var i in obj) // Object array is not vulnerable to changes in callback - properties removed or added are not iterated
			if (callback(i) === false)
				return false;
	}

	return true;
}

/// <function container="Fit.Array" name="Count" access="public" static="true" returns="integer">
/// 	<description> Returns number of elements in collection </description>
/// 	<param name="arr" type="array | HTMLCollection | NodeList | NamedNodeMap | FileList | StyleSheetList | CSSRuleList">
/// 		Collection to count elements within
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Count" access="public" static="true" returns="integer">
/// 	<description> Returns number of elements in object array </description>
/// 	<param name="obj" type="object"> Object array to count elements within </param>
/// </function>
Fit.Array.Count = function(obj)
{
	if (Fit._internal.Validation.IsCollectionType(obj) === true)
	{
		return obj.length;
	}
	else if (typeof(obj) === "object")
	{
		var count = 0;

		for (var i in obj)
			count++;

		return count;
	}

	Fit.Validation.ThrowError("Unsupported collection type passed - unable to determine number of contained elements");
}

/// <function container="Fit.Array" name="HasItems" access="public" static="true" returns="boolean">
/// 	<description> Returns True if collection has items, otherwise False </description>
/// 	<param name="arr" type="array | HTMLCollection | NodeList | NamedNodeMap | FileList | StyleSheetList | CSSRuleList">
/// 		Collection to investigate
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="HasItems" access="public" static="true" returns="boolean">
/// 	<description> Returns True if object array has items, otherwise False </description>
/// 	<param name="obj" type="object"> Object array to investigate </param>
/// </function>
Fit.Array.HasItems = function(obj)
{
	if (Fit._internal.Validation.IsCollectionType(obj) === true)
	{
		return obj.length > 0;
	}
	else if (typeof(obj) === "object")
	{
		for (var i in obj)
		{
			return true;
		}

		return false;
	}

	Fit.Validation.ThrowError("Unsupported collection type passed - unable to determine number of contained elements");
}

/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="$Type[]"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:$Type) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="HTMLCollection"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:HTMLElement) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="NodeList"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:Node) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="NamedNodeMap"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:Attr) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="FileList"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:File) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="StyleSheetList"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:StyleSheet) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Recurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Recursively iterates through objects in collection and passes each object to the provided callback function.
/// 		Returns boolean indicating whether recursion was carried through (True) or interrupted (False).
/// 	</description>
/// 	<param name="arr" type="CSSRuleList"> Collection containing objects to iterate through </param>
/// 	<param name="childrenProperty" type="string">
/// 		Name of property or argumentless getter function returning children (e.g. "Children" or "GetChildren")
/// 	</param>
/// 	<param name="callback" type="(obj:CSSRule) => boolean | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Return False from callback to break loop.
/// 	</param>
/// </function>
Fit.Array.Recurse = function(arr, childrenProperty, callback)
{
	Fit.Validation.ExpectCollection(arr, true);
	Fit.Validation.ExpectStringValue(childrenProperty);
	Fit.Validation.ExpectFunction(callback);

	if (Fit.Validation.IsSet(arr) === false)
		return true;

	var count = arr.length;

	for (var i = 0 ; i < arr.length ; i++)
	{
		if (arr.length !== count)
			Fit.Validation.ThrowError("Collection was modified while iterating objects");

		if (callback(arr[i]) === false)
			return false;

		if (Fit.Validation.IsSet(arr[i][childrenProperty]) === false)
			continue;

		if (arr[i][childrenProperty] instanceof Function)
		{
			if (Fit.Array.Recurse(arr[i][childrenProperty](), childrenProperty, callback) === false)
				return false;
		}
		else
		{
			if (Fit.Array.Recurse(arr[i][childrenProperty], childrenProperty, callback) === false)
				return false;
		}
	}

	return true;
}

/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="$Type[]"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:$Type) => $Type[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="HTMLCollection"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:HTMLElement) => HTMLCollection | HTMLElement[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="NodeList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:Node) => NodeList | Node[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="NamedNodeMap[]"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:Attr) => NamedNodeMap | Attr[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="FileList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:File) => FileList | File[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="StyleSheetList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:StyleSheet) => StyleSheetList | StyleSheet[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="CustomRecurse" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Iterates through objects in collection and passes each object to provided callback.
/// 		Callback is expected to return any children supposed to be iterated too, or Null
/// 		if further/deeper iteration is not required.
/// 	</description>
/// 	<param name="arr" type="CSSRuleList"> Collection containing objects to iterate through </param>
/// 	<param name="callback" type="(obj:CSSRule) => CSSRuleList | CSSRule[] | null | void">
/// 		Callback function accepting objects from the collection, passed in turn.
/// 		Function must return children collection to continue recursive operation,
/// 		or Null (or nothing) to stop further processing.
/// 	</param>
/// </function>
Fit.Array.CustomRecurse = function(arr, callback) // arr not validated - passing null/undefined is allowed - no iteration is performed in this case
{
	Fit.Validation.ExpectFunction(callback);

	if (arr !== undefined && arr !== null && Fit._internal.Validation.IsCollectionType(arr) === false)
		Fit.Validation.ThrowError("Unexpected collection type passed"); // CustomRecurse does not support iterating object arrays like ForEach does

	Fit.Array.ForEach(arr, function(o)
	{
		Fit.Array.CustomRecurse(callback(o), callback);
	});
}

/// <function container="Fit.Array" name="Add" access="public" static="true">
/// 	<description> Add object to array </description>
/// 	<param name="arr" type="array"> Array to which object is added </param>
/// 	<param name="obj" type="object"> Object to add to array </param>
/// </function>
Fit.Array.Add = function(arr, obj) // obj not validated - passing any object or undefined/null is allowed
{
	Fit.Validation.ExpectArray(arr);
    arr.push(obj);
}

/// <function container="Fit.Array" name="Insert" access="public" static="true">
/// 	<description> Insert object into array at specified index </description>
/// 	<param name="arr" type="array"> Array into which object is inserted </param>
/// 	<param name="idx" type="integer"> Index to insert object at </param>
/// 	<param name="obj" type="object"> Object to insert into array </param>
/// </function>
Fit.Array.Insert = function(arr, idx, obj) // obj not validated - passing any object or undefined/null is allowed
{
	Fit.Validation.ExpectArray(arr);
	Fit.Validation.ExpectInteger(idx);
    arr.splice(idx, 0, obj);
}

/// <function container="Fit.Array" name="Move" access="public" static="true">
/// 	<description> Move object within array from one position to another </description>
/// 	<param name="arr" type="array"> Array to manipulate </param>
/// 	<param name="fromIdx" type="integer"> Position of object to move </param>
/// 	<param name="ToIdx" type="integer"> New object position </param>
/// </function>
Fit.Array.Move = function(arr, fromIdx, toIdx)
{
	Fit.Validation.ExpectArray(arr);
	Fit.Validation.ExpectInteger(fromIdx);
	Fit.Validation.ExpectInteger(toIdx);

	if (fromIdx < arr.length)
	{
		// If toIdx is below 0 (zero) the object will be added at the beginning of the array.
		// If toIdx is greater than the length of the array, the object will be added at the end of the array.

		var obj = arr[fromIdx];
		Fit.Array.RemoveAt(arr, fromIdx);
		Fit.Array.Insert(arr, toIdx > 0 ? toIdx : 0, obj);
	}
}

/// <function container="Fit.Array" name="Merge" access="public" static="true" returns="($TypeA | $TypeB)[]">
/// 	<description> Merge/join two arrays </description>
/// 	<param name="arr1" type="$TypeA[]"> Array 1 to merge with array 2 </param>
/// 	<param name="arr2" type="$TypeB[]"> Array 2 to merge with array 1 </param>
/// </function>
Fit.Array.Merge = function(arr1, arr2)
{
	Fit.Validation.ExpectArray(arr1);
	Fit.Validation.ExpectArray(arr2);
    return arr1.concat(arr2);
}

/// <function container="Fit.Array" name="Remove" access="public" static="true">
/// 	<description> Remove object from array </description>
/// 	<param name="arr" type="array"> Array from which object is remove </param>
/// 	<param name="obj" type="object"> Object to remove from array </param>
/// </function>
Fit.Array.Remove = function(arr, obj) // obj not validated - passing any object or undefined/null is allowed
{
	Fit.Validation.ExpectArray(arr);

    var idx = Fit.Array.GetIndex(arr, obj);
    if (idx !== -1)
        arr.splice(idx, 1);
}

/// <function container="Fit.Array" name="RemoveAt" access="public" static="true">
/// 	<description> Remove object from array at specified index </description>
/// 	<param name="arr" type="array"> Array from which object is remove </param>
/// 	<param name="idx" type="integer"> Object index in array </param>
/// </function>
Fit.Array.RemoveAt = function(arr, idx)
{
	Fit.Validation.ExpectArray(arr);
	Fit.Validation.ExpectInteger(idx);
    arr.splice(idx, 1);
}

/// <function container="Fit.Array" name="Clear" access="public" static="true">
/// 	<description> Clear all items from array </description>
/// 	<param name="arr" type="array"> Array from which all objects are remove </param>
/// </function>
Fit.Array.Clear = function(arr)
{
	Fit.Validation.ExpectArray(arr);
    arr.length = 0; // http://jsperf.com/array-destroy/151
}

/// <function container="Fit.Array" name="GetIndex" access="public" static="true" returns="integer">
/// 	<description> Returns index of object in collection if found, otherwise a value of -1 is returned </description>
/// 	<param name="arr" type="array | HTMLCollection | NodeList | NamedNodeMap | FileList | StyleSheetList | CSSRuleList">
/// 		Collection to search through
/// 	</param>
/// 	<param name="obj" type="object"> Object to obtain index for </param>
/// </function>
Fit.Array.GetIndex = function(arr, obj) // obj not validated - passing any object or undefined/null is allowed
{
	Fit.Validation.ExpectCollection(arr);

    for (var i = 0 ; i < arr.length ; i++)
        if (arr[i] === obj)
            return i;

    return -1;
}

/// <function container="Fit.Array" name="GetKeys" access="public" static="true" returns="integer[]">
/// 	<description> Returns all keys in collection (indices) - 0, 1, 2, 3, ... </description>
/// 	<param name="arr" type="array | HTMLCollection | NodeList | NamedNodeMap | FileList | StyleSheetList | CSSRuleList">
/// 		Collection to get keys from
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="GetKeys" access="public" static="true" returns="string[]">
/// 	<description> Returns all keys (property names) in object </description>
/// 	<param name="obj" type="object"> Object to get keys from </param>
/// </function>
Fit.Array.GetKeys = function(obj)
{
	var keys = [];

	if (Fit._internal.Validation.IsCollectionType(obj) === true)
	{
		for (var i = 0 ; i < obj.length ; i++)
		{
			keys.push(i);
		}
	}
	else if (typeof(obj) === "object")
	{
		for (var i in obj)
		{
			keys.push(i);
		}
	}

	return keys;
}

/// <function container="Fit.Array" name="Contains" access="public" static="true" returns="boolean">
/// 	<description> Returns True if given object is contained in collection, otherwise False </description>
/// 	<param name="arr" type="array | HTMLCollection | NodeList | NamedNodeMap | FileList | StyleSheetList | CSSRuleList">
/// 		Collection to search through
/// 	</param>
/// 	<param name="obj" type="object"> Object to look for </param>
/// </function>
Fit.Array.Contains = function(arr, obj) // obj not validated - passing any object or undefined/null is allowed
{
	Fit.Validation.ExpectCollection(arr);
    return (Fit.Array.GetIndex(arr, obj) > -1);
}

/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="$Type[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="$Type[]"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="HTMLElement[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="HTMLCollection"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="Node[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="NodeList"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="Attr[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="NamedNodeMap"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="File[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="FileList"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="StyleSheet[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="StyleSheetList"> Collection to copy </param>
/// </function>
/// <function container="Fit.Array" name="Copy" access="public" static="true" returns="CSSRule[]">
/// 	<description> Returns a shallow copy of the collection. For a deep copy see Fit.Core.Clone(..). </description>
/// 	<param name="arr" type="CSSRuleList"> Collection to copy </param>
/// </function>
Fit.Array.Copy = function(arr)
{
	/*Fit.Validation.ExpectCollection(arr);

	// Notice that collection type is not preserved, meaning e.g. a NodeList or FileList will be returned
	// as a traditional array holding references to the objects - just like Fit.Array.ToArray(..) does.

	var newArr = [];
	Fit.Array.ForEach(arr, function(item)
	{
		newArr.push(item);
	});
	return newArr;*/

	return Fit.Array.ToArray(arr);
}

/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="HTMLElement[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="HTMLCollection"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="Node[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="NodeList"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="Attr[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="NamedNodeMap"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="File[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="FileList"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="StyleSheet[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="StyleSheetList"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="CSSRule[]">
/// 	<description> Convert collection to a traditional JavaScript array </description>
/// 	<param name="coll" type="CSSRuleList"> Collection to convert to array </param>
/// </function>
/// <function container="Fit.Array" name="ToArray" access="public" static="true" returns="$Type[]">
/// 	<description> Copy array to a new array </description>
/// 	<param name="coll" type="$Type[]"> Array to copy to a new array </param>
/// </function>
Fit.Array.ToArray = function(coll)
{
	Fit.Validation.ExpectCollection(coll);

	var arr = [];

	for (var i = 0 ; i < coll.length ; i++)
		arr.push(coll[i]);

	return arr;
}

/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="$Type[]"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:$Type) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="HTMLCollection"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:HTMLElement) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="NodeList"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:Node) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="NamedNodeMap"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:Attr) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="FileList"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:File) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="StyleSheetList"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:StyleSheet) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
/// <function container="Fit.Array" name="Join" access="public" static="true" returns="string">
/// 	<description> Join objects from a collection into a string </description>
/// 	<param name="arr" type="CSSRuleList"> Collection containing objects </param>
/// 	<param name="separator" type="string"> String used to glue values together </param>
/// 	<param name="callback" type="(obj:CSSRule) => string">
/// 		Callback returning string representation of objects passed from collection in turn
/// 	</param>
/// </function>
Fit.Array.Join = function(arr, separator, callback)
{
	Fit.Validation.ExpectCollection(arr);
	Fit.Validation.ExpectString(separator);
	Fit.Validation.ExpectFunction(callback);

	var result = "";

	Fit.Array.ForEach(arr, function(obj)
	{
		result += (result !== "" ? separator : "") + callback(obj);
	});

	return result;
}

/*
// Difference between using ordinary for loop and Fit.Array.ForEach
// can be easily demonstrated with the code below.
// Fit.Array.ForEach creates a new execution scope (closure) which
// ensure the expected array value is used, while an ordinary loop
// results in the same index (i) variable being used, which will
// have the value 3 when all setTimeout callbacks finally fire.

// Will output 1, 3, 6, 8 as expected
Fit.Array.ForEach([1,3,6,8], function(val)
{
	setTimeout(function() { console.log(val); }, 100);
});

// Will output "8" 4 times
for (var i in vals = [1,3,6,8])
{
	setTimeout(function() { console.log(vals[i]); }, 100);
}
*/

/// <container name="Fit.Browser">
/// 	Provides access to various browser information.
///
/// 	// Example code
///
/// 	var browserName = Fit.Browser.GetBrowser();
/// 	var browserVersion = Fit.Browser.GetVersion();
/// 	var browserLanguage = Fit.Browser.GetLanguage();
///
/// 	if (browserName === &quot;MSIE&quot; &amp;&amp; browserVersion &lt; 7)
/// 	{
/// 		&#160;&#160;&#160;&#160; if (browserLanguage === &quot;da&quot;)
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; alert(&quot;Opgrader venligst til IE7 eller nyere&quot;);
/// 		&#160;&#160;&#160;&#160; else
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; alert(&quot;Please upgrade to IE7 or newer&quot;);
/// 	}
/// </container>
Fit.Browser = {};
Fit._internal.Browser = {};

// Allow Fit.Browser unit tests to override user agent information
// to test parsing of a range of different user agent strings.
Fit._internal.Browser.UserAgent = navigator.userAgent;

/// <function container="Fit.Browser" name="GetBrowser" access="public" static="true" returns='"Edge" | "Chrome" | "Safari" | "MSIE" | "Firefox" | "Opera" | "Unknown"'>
/// 	<description>
/// 		Returns name of browser useful for adjusting behaviour based on render engine.
/// 		Possible values are: Chrome (both WebKit and Blink based - also returned for modern versions of
/// 		Opera and Edge), Safari (also returned for Edge, Chrome, Opera, and Firefox on iOS), Edge (version 12-18),
/// 		MSIE (version 8-11), Firefox, Opera (version 1-12), and Unknown.
/// 	</description>
/// 	<param name="returnAppId" type="false" default="false"> Set True to have app specific identifier returned </param>
/// </function>
/// <function container="Fit.Browser" name="GetBrowser" access="public" static="true" returns='"Edge" | "Chrome" | "Safari" | "MSIE" | "Firefox" | "Opera" | "Unknown"'>
/// 	<description>
/// 		Returns browser app name useful for adjusting behaviour based on actual
/// 		application, regardless of render engine and platform. Possible values are:
/// 		Chrome (both Webkit and Blink based), Safari, Edge (both legacy and modern),
/// 		MSIE (version 8-11), Firefox, Opera (both legacy and modern), and Unknown.
/// 		Be careful not to check against browser app name and app version alone.
/// 		For instance Opera 3 on a touch device is newer than Opera 60 on a Desktop
/// 		device, as they are two completely different browsers. Check whether the browser runs on
/// 		a tablet or phone using e.g. Fit.Browser.IsMobile(true) or Fit.Browser.GetInfo(true).IsMobile.
/// 	</description>
/// 	<param name="returnAppId" type="true"> Set True to have app specific identifier returned </param>
/// </function>
Fit.Browser.GetBrowser = function(returnAppId)
{
	Fit.Validation.ExpectBoolean(returnAppId, true);

	var agent = Fit._internal.Browser.UserAgent;

	// IMPORTANT: The order in which browsers are detected matters! For instance, several browsers define both Chrome and Safari as part of their user agent string. Examples:
	// Firefox 46	"Mozilla/5.0 (Windows NT 5.2; rv:46.0) Gecko/20100101 Firefox/46.0"
	// Firefox 81	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:81.0) Gecko/20100101 Firefox/81.0"
	// Chrome 85	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36"
	// Safari 13	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.1 Safari/605.1.15"
	// Opera 12		"Opera/9.80 (X11; Linux i686; Ubuntu/14.10) Presto/2.12.388 Version/12.16.2"
	// Opera 71		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 OPR/71.0.3770.228"
	// IE 8			"Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)"
	// IE 11		"Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; rv:11.0) like Gecko"
	// Edge 18		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.102 Safari/537.36 Edge/18.18363"
	// Edge 85		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36 Edg/85.0.564.41"

	if (agent.indexOf("Edge/") > -1 || (returnAppId === true && (agent.indexOf("Edg/") > -1 || agent.indexOf("EdgiOS/") > -1 || agent.indexOf("EdgA/") > -1)))
		return "Edge"; // Legacy Edge is identified by "Edge/", Chromium based Edge is identified by "Edg/", Edge on iOS is identified by "EdgiOS/", and Edge on Android is identified by "EdgA/"
	if (agent.indexOf("MSIE") > -1 || agent.indexOf("Trident") > -1)
		return "MSIE";
	if (agent.indexOf("Opera") > -1 || (returnAppId === true && (agent.indexOf("OPR/") > -1 || agent.indexOf("OPT/") > -1)))
		return "Opera"; // Legacy Opera is identified by "Opera", Chromium based Opera is identified by "OPR/", and Opera on iOS is identified by "OPT/"
	if (agent.indexOf("OPT/") > -1 && (agent.indexOf("iPhone;") > -1 || agent.indexOf("iPad;") > -1))
		return "Safari"; // Opera on iOS does not contain "Opera", nor "Safari" to identify the engine like other browsers using WebView - we can use "OPT/" plus "iPhone;" or "iPad;" instead
	if (agent.indexOf("Firefox") > -1 || (returnAppId === true && agent.indexOf("FxiOS/") > -1))
		return "Firefox"; // Firefox is identified by "Firefox" except on iOS where "FxiOS/" identifies it instead
	if (agent.indexOf("Chrome") > -1 || (returnAppId === true && agent.indexOf("CriOS/") > -1))
		return "Chrome"; // Chrome is identified by "Chrome" except on iOS where "CriOS/" identifies it instead
	if (agent.indexOf("Safari") > -1)
		return "Safari";

	return "Unknown";
}

/// <function container="Fit.Browser" name="GetVersion" access="public" static="true" returns="integer">
/// 	<description> Returns major version number for known browsers, -1 for unknown browsers </description>
/// 	<param name="returnAppVersion" type="boolean" default="false">
/// 		Set True to have app specific version number returned, rather than version number of browser engine.
/// 		If version number is used in combination with the result from Fit.Browser.GetBrowser(true), then
/// 		this argument should be True as well, otherwise False (default).
/// 	</param>
/// </function>
Fit.Browser.GetVersion = function(returnAppVersion)
{
	Fit.Validation.ExpectBoolean(returnAppVersion, true);

	// Firefox 46	"Mozilla/5.0 (Windows NT 5.2; rv:46.0) Gecko/20100101 Firefox/46.0"
	// Firefox 81	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10.13; rv:81.0) Gecko/20100101 Firefox/81.0"
	// Chrome 85	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36"
	// Safari 13	"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.1.1 Safari/605.1.15"
	// Opera 12		"Opera/9.80 (X11; Linux i686; Ubuntu/14.10) Presto/2.12.388 Version/12.16.2"
	// Opera 71		"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.121 Safari/537.36 OPR/71.0.3770.228"
	// IE 8			"Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.2; Trident/4.0; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.648; .NET CLR 3.5.21022)"
	// IE 11		"Mozilla/5.0 (Windows NT 10.0; WOW64; Trident/7.0; .NET4.0C; .NET4.0E; .NET CLR 2.0.50727; .NET CLR 3.0.30729; .NET CLR 3.5.30729; rv:11.0) like Gecko"
	// Edge 18		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.102 Safari/537.36 Edge/18.18363"
	// Edge 85		"Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36 Edg/85.0.564.41"

	var browser = Fit.Browser.GetBrowser(returnAppVersion);

	var start = 0;
	var end = 0;
	var agent = Fit._internal.Browser.UserAgent;

	if (browser === "Edge")
	{
		var search = agent.indexOf("Edge/") > -1 && "Edge/" || agent.indexOf("Edg/") > -1 && "Edg/" || agent.indexOf("EdgiOS/") > -1 && "EdgiOS/" || agent.indexOf("EdgA/") > -1 && "EdgA/" || null;

		start = search !== null && agent.indexOf(search) || -1;
		start = (start !== -1 ? start + search.length : 0);
		end = agent.indexOf(".", start);
		end = (end !== -1 ? end : 0);
	}
	else if (browser === "Chrome")
	{
		var search = agent.indexOf("Chrome/") > -1 && "Chrome/" || agent.indexOf("CriOS/") > -1 && "CriOS/" || null;

		start = search !== null && agent.indexOf(search) || -1;
		start = (start !== -1 ? start + search.length : 0);
		end = agent.indexOf(".", start);
		end = (end !== -1 ? end : 0);
	}
	else if (browser === "Safari")
	{
		var search = agent.indexOf("CriOS/") > -1 && "CriOS/" || agent.indexOf("FxiOS/") > -1 && "FxiOS/" || agent.indexOf("EdgiOS/") > -1 && "EdgiOS/" || agent.indexOf("OPT/") > -1 && "OPT/" || null;

		if (search !== null) // Browser based on WebView on iOS - Chrome, Firefox, Edge, or Opera
		{
			if (returnAppVersion !== true) // Return Safari version parsed from OS version
			{
				start = agent.indexOf(" OS ");
				start = (start !== -1 ? start + 4 : 0);
				end = agent.indexOf("_", start);
				end = (end !== -1 ? end : 0);
			}
			else // Return application version
			{
				start = agent.indexOf(search);
				start = (start !== -1 ? start + search.length : 0);
				end = agent.indexOf(".", start);
				end = (end !== -1 ? end : 0);
			}
		}
		else // Real Safari or Firefox on iPad which does not identify itself as Firefox
		{
			start = agent.indexOf("Version/");
			start = (start !== -1 ? start + 8 : 0);
			end = agent.indexOf(".", start);
			end = (end !== -1 ? end : 0);
		}
	}
	else if (browser === "MSIE")
	{
		if (agent.indexOf("MSIE") > -1)
		{
			start = agent.indexOf("MSIE ");
			start = (start !== -1 ? start + 5 : 0);
			end = agent.indexOf(".", start);
			end = (end !== -1 ? end : 0);
		}
		else if (agent.indexOf("Trident") > -1) // IE11+
		{
			start = agent.indexOf("rv:");
			start = (start !== -1 ? start + 3 : 0);
			end = agent.indexOf(".", start);
			end = (end !== -1 ? end : 0);
		}
	}
	else if (browser === "Firefox")
	{
		var search = agent.indexOf("Firefox/") > -1 && "Firefox/" || agent.indexOf("FxiOS/") > -1 && "FxiOS/" || null;

		start = search !== null && agent.indexOf(search) || -1;
		start = (start !== -1 ? start + search.length : 0);
		end = agent.indexOf(".", start);
		end = (end !== -1 ? end : 0);
	}
	else if (browser === "Opera")
	{
		var search = agent.indexOf("Version/") > -1 && "Version/" || agent.indexOf("Opera ") > -1 && "Opera " || agent.indexOf("Opera/") > -1 && "Opera/" || agent.indexOf("OPR/") > -1 && "OPR/" || agent.indexOf("OPT/") > -1 && "OPT/" || null;

		start = search !== null && agent.indexOf(search) || -1;
		start = (start !== -1 ? start + search.length : 0);
		end = agent.indexOf(".", start);
		end = (end !== -1 ? end : 0);
	}

	if (start !== 0 && start !== 0)
		return parseInt(agent.substring(start, end));

	return -1;
}

/// <container name="Fit.BrowserTypeDefs.QueryString">
/// 	<description> Object representing query string </description>
/// 	<member name="Url" type="string"> Full URL address </member>
/// 	<member name="Parameters" type="{[key:string]: string | undefined}">
/// 		Associative array with key value pairs representing URL parameters
/// 	</member>
/// 	<member name="Hash" type="string | null"> URL hash value if specified, otherwise Null </member>
/// </container>

/// <function container="Fit.Browser" name="GetQueryString" access="public" static="true" returns="Fit.BrowserTypeDefs.QueryString">
/// 	<description> Returns query string information </description>
/// 	<param name="alternativeUrl" type="string" default="undefined"> Alternative URL to parse </param>
/// </function>
Fit.Browser.GetQueryString = function(alternativeUrl)
{
	Fit.Validation.ExpectString(alternativeUrl, true);

	var qs = { Url: null, Parameters: {}, Hash: null, Anchor: null }; // Anchor is there for backwards compatibility

	var url = ((alternativeUrl !== undefined) ? alternativeUrl : location.href);
	var params = ((url.indexOf("?") > -1) ? url.split("?")[1] : "");
	var hash = null;

	params = ((params.indexOf("#") > -1) ? params.split("#")[0] : params);
	hash = ((url.indexOf("#") > -1) ? url.split("#")[1] : null);

	qs.Url = url;
	qs.Hash = hash;
	qs.Anchor = hash; // Backwards compatibility

	Fit.Array.ForEach(((params !== "") ? params.split("&") : []), function(p)
	{
		var keyval = p.split("=");

		try
		{
			// Notice: decodeURIComponent(..) throws an error in case invalid encoding is used
			qs.Parameters[decodeURIComponent(keyval[0])] = ((keyval.length > 1) ? decodeURIComponent(keyval[1]) : "");
		}
		catch (err)
		{
			// An error occurred decoding parameter - use raw value instead
			qs.Parameters[keyval[0]] = ((keyval.length > 1) ? keyval[1] : "");
		}
	});

	return qs;
}

/// <container name="Fit.BrowserTypeDefs.ParsedUrl">
/// 	<description> Object representing components of a URL </description>
/// 	<member name="Url" type="string"> Full URL address </member>
/// 	<member name="Protocol" type='"ftp" | "http" | "https"'> Full URL address </member>
/// 	<member name="Port" type="integer"> Port number - returns -1 if not defined in URL </member>
/// 	<member name="Auth" type="string | null"> Authentication token or user:pass if specified, otherwise Null </member>
/// 	<member name="Host" type="string"> Hostname, e.g. localhost or domain name </member>
/// 	<member name="Path" type="string"> Path to folder containing resources, e.g. / or /folder </member>
/// 	<member name="Resource" type="string"> Name of resource, e.g. resource.php </member>
/// 	<member name="FullPath" type="string"> Path and Resource combined, e.g. /folder/resource.php </member>
/// 	<member name="Parameters" type="{[key:string]: string | undefined}">
/// 		Associative array with key value pairs representing URL parameters
/// 	</member>
/// 	<member name="Hash" type="string | null"> URL hash value if specified, otherwise Null </member>
/// </container>

/// <function container="Fit.Browser" name="ParseUrl" access="public" static="true" returns="Fit.BrowserTypeDefs.ParsedUrl">
/// 	<description>
/// 		Parses well-formed URLs and returns an object containing the various components.
/// 	</description>
/// 	<param name="url" type="string"> Well-formed URL to parse </param>
/// </function>
Fit.Browser.ParseUrl = function(url)
{
	Fit.Validation.ExpectString(url);

	// URLs are often concatenated incorrectly with double forward slashes.
	// E.g. http://localhost//path/to//file.php
	// Compensate for this problem to reduce risk of runtime parse errors.
	// https://regex101.com/r/DMQs9A/1/
	var doubleSlashes = /(^.*:\/\/.*?)\/\/(.*)/;
	while (doubleSlashes.test(url) === true)
	{
		url = url.replace(doubleSlashes, "$1/$2");
	}

	// This function is not perfect.
	// It will parse well-formed URLs properly
	// but may be fooled by incorrecly formatted URLs,
	// and does not currently support IPv6 addresses.
	// But for a solution using Regular Expression it
	// works quite well and is very simple in its
	// implementation.

	// Using JS is possible although known to give slightly
	// different results across different browsers - example:
	//    var a = document.createElement("a");
	//    console.log(a.pathname, a.host, a.port, a.protocol, a.username, a.password); // etc.
	// On top of differences across browsers it also depends on DOM which is not available in Workers,
	// it will automatically canonicalize the URL to the page (relative to domain, protocol, etc.),
	// and some browsers (e.g. Legacy IE) does not parse IPv6 addresses either.
	// See comments here: https://gist.github.com/jlong/2428561
	// Solution must be cross browser and support Legacy IE !

	// It is impossible to produce a perfect solution using a one-liner
	// regular expression. Inventer of WWW, Tim Burners-Lee (TimBL / TBL),
	// among many others, solves parsing programmatically: https://www.w3.org/wiki/UriTesting
	// Such solutions tend to be much more complex though.
	// Another example: https://github.com/unshiftio/url-parse

	/* RegEx fiddle: https://regex101.com/r/poWR2S/14
	0 = Full match
	1 = Protocol (http/https/ftp/etc)
	2 = Group used to exclude @ from auth in #3
	3 = Auth
	5 = Host followed by port in #7
	6 = Group used to exclude / from port in #7
	7 = Port
	8 = Host NOT followed by port
	9 = Host NOT followed by anything (end of string)
	10 = Group used to wrap #11 #12 #13 #14 #15
	11 = Path followed by QS (and possibly #hash) in #12
	12 = QS (and possibly #hash)
	13 = Path followed by #hash in #14
	14 = #hash value
	15 = Path NOT followed by anything (end of string) */
	var regEx = /(.+):\/\/((.+)@)?((.+):((\d+)\/?)?|(.+?)\/|(.+?)$)((.*)\?(.+)|(.*)#(.*)|(.*)$)/;
	var match = regEx.exec(url);
	var result = { Url: null, Protocol: null, Port: null, Auth: null, Host: null, FullPath: null, Path: null, Resource: null, Parameters: {}, Hash: null, Anchor: null }; // Anchor is there for backwards compatibility

	if (match === null)
	{
		Fit.Validation.ThrowError("Unable to parse invalid URL - valid example: schema://[auth@]host[:port][/path/resource[?parm=val[#hash]]]");
	}

	if (match.length !== 16) // RegExp should produce this amount of capture group results!
	{
		Fit.Validation.ThrowError("Unexpected parse error - internal error"); // Happens if RegExp is changed without adjusting code using it
	}

	var fullPath = (match[11] || match[13] || match[15] || ""); //var fullPath = (match[11] || match[13] || (match[15] && match[15] !== "?" ? match[15] : "") || "");
	var qs = Fit.Browser.GetQueryString(url);

	result.Url = url;
	result.Protocol = match[1].toLowerCase();
	result.Auth = match[3] || null;
	result.Host = match[5] || match[8] || match[9];
	result.Port = (match[7] ? parseInt(match[7]) : -1);
	result.FullPath = "/" + fullPath;
	result.Parameters = qs.Parameters;
	result.Hash = qs.Hash;
	result.Anchor = qs.Hash; // Backwards compatibility

	result.Path = "/";

	if (fullPath !== "")
	{
		// Separate path from resource - e.g. path/to/index.php is separated into path/to/ and index.php

		var pathInfo = fullPath.split("/"); // E.g. index.php or path/to/ or path/to/index.php

		if (pathInfo.length === 1)
		{
			// Only resource contained, no path - e.g. index.php
			result.Resource = fullPath;
		}
		else
		{
			// Path and maybe resource contained - e.g. path/to/ or path/to/index.php

			result.Path += pathInfo.slice(0, -1).join("/") + "/"; // Slice to remove last portion of array which is either the resource or an empty string if path is something like path/to/

			// Add resource (e.g. resource.php) if found in URL - last item in array is an empty string for a path such as path/to/
			if (pathInfo[pathInfo.length - 1] !== "")
			{
				result.Resource = pathInfo[pathInfo.length - 1];
			}
		}
	}

	return result;
}

/// <function container="Fit.Browser" name="GetLanguage" access="public" static="true" returns="string">
/// 	<description> Returns browser language - e.g. &quot;da&quot; (Danish), &quot;en&quot; (English) etc. </description>
/// </function>
Fit.Browser.GetLanguage = function()
{
	var lang = null;

	if (navigator.language)
		lang = navigator.language.toLowerCase();
	else if (navigator.browserLanguage)
		lang = navigator.browserLanguage.toLowerCase();

	if (lang === null || lang === "")
		return "en";

	if (lang.length === 2)
		return lang;

	if (lang.length === 5)
		return lang.substring(0, 2);

	return "en";
}

/// <function container="Fit.Browser" name="GetPageWidth" access="public" static="true" returns="integer">
/// 	<description> Returns page width in pixels </description>
/// </function>
Fit.Browser.GetPageWidth = function() // GetViewPortWidth would be a more accurate name - TODO: deprecate this, we now have GetViewPortDimensions(..)
{
	var w = 0;

	if (window.innerWidth) // W3C
	{
		w = window.innerWidth;
	}
	else if (document.documentElement && document.documentElement.clientWidth) // IE 6-8 (not quirks mode)
	{
		w = document.documentElement.clientWidth;

		if (document.documentElement.scrollHeight > document.documentElement.clientHeight)
		{
			w += Fit.Browser.GetScrollBarSize(); // Width affected by vertical scrollbar - return potential viewport size, just like window.innerWidth
		}
	}

	return w;
}

/// <function container="Fit.Browser" name="GetPageHeight" access="public" static="true" returns="integer">
/// 	<description> Returns page height in pixels </description>
/// </function>
Fit.Browser.GetPageHeight = function() // GetViewPortHeight would be a more accurate name - TODO: deprecate this, we now have GetViewPortDimensions(..)
{
	var h = 0;

	if (window.innerHeight) // W3C
	{
		h = window.innerHeight;
	}
	else if (document.documentElement && document.documentElement.clientHeight) // IE 6-8 (not quirks mode)
	{
		h = document.documentElement.clientHeight;

		if (document.documentElement.scrollWidth > document.documentElement.clientWidth)
		{
			h += Fit.Browser.GetScrollBarSize(); // Height affected by horizontal scrollbar - return potential viewport size, just like window.innerHeight
		}
	}

	return h;
}

/// <function container="Fit.Browser" name="GetViewPortDimensions" access="public" static="true" returns="Fit.TypeDefs.Dimension">
/// 	<description> Returns object with Width and Height properties specifying dimensions of viewport </description>
/// 	<param name="includeScrollbars" type="boolean" default="false">
/// 		Include scrollbars if present to get all potential space available if scrollbars are removed.
/// 	</param>
/// </function>
Fit.Browser.GetViewPortDimensions = function(includeScrollbars)
{
	Fit.Validation.ExpectBoolean(includeScrollbars, true);

	// GetPageWidth() and GetPageHeight() returns width/height of viewport including scrollbars.
	// The space consumed by the scrollbars are not available for content, so by default we substract
	// scrollbars from the dimensions, unless includeScrollbars is True.

	var dim = { Width: Fit.Browser.GetPageWidth(), Height: Fit.Browser.GetPageHeight() };

	if (includeScrollbars !== true)
	{
		if (Fit.Browser.GetScrollDocument().scrollWidth > dim.Width)
		{
			dim.Height = dim.Height - Fit.Browser.GetScrollBarSize(); // Height affected by horizontal scrollbar
		}

		if (Fit.Browser.GetScrollDocument().scrollHeight > dim.Height)
		{
			dim.Width = dim.Width - Fit.Browser.GetScrollBarSize(); // Width affected by vertical scrollbar
		}
	}

	return dim;
}

/// <function container="Fit.Browser" name="GetScrollPosition" access="public" static="true" returns="Fit.TypeDefs.Position">
/// 	<description> Returns object with X and Y properties specifying scroll position </description>
/// </function>
Fit.Browser.GetScrollPosition = function()
{
	var x = document.body.scrollLeft || document.documentElement.scrollLeft || window.pageXOffset || 0;
	var y = document.body.scrollTop || document.documentElement.scrollTop || window.pageYOffset || 0;

	return { X: x, Y: y };
}

/// <function container="Fit.Browser" name="GetScrollDocument" access="public" static="true" returns="DOMElement">
/// 	<description>
/// 		Get scrolling document element. This is the cross browser
/// 		equivalent of document.scrollingElement.
/// 	</description>
/// </function>
Fit.Browser.GetScrollDocument = function()
{
	if (Fit._internal.Browser.ScrollDocument === undefined)
	{
		if (document.scrollingElement)
		{
			Fit._internal.Browser.ScrollDocument = document.scrollingElement;
		}
		else
		{
			var iframe = document.createElement("iframe");
			iframe.style.cssText = "height: 1px; position: fixed; top: -100px; left: -100px;";

			document.documentElement.appendChild(iframe);

			var doc = iframe.contentWindow.document;
			doc.write("<!DOCTYPE html><div style='height: 100px'>&nbsp;</div>");
			doc.close();

			if (doc.documentElement.scrollHeight > doc.body.scrollHeight)
			{
				Fit._internal.Browser.ScrollDocument = document.documentElement;
			}
			else
			{
				Fit._internal.Browser.ScrollDocument = document.body;
			}

			iframe.parentNode.removeChild(iframe);
		}
	}

	return Fit._internal.Browser.ScrollDocument;
}

/// <function container="Fit.Browser" name="GetScrollBars" access="public" static="true" returns="Fit.TypeDefs.ScrollBarsPresent">
/// 	<description>
/// 		Get information about scrollbars in viewport.
/// 		Returns an object with Vertical and Horizontal properties, each containing
/// 		Enabled and Size properties, which can be used to determine whether scrolling is enabled,
/// 		and the size of the scrollbar. The size remains 0 when scrolling is not enabled.
/// 		To determine whether a DOM element has scrolling enabled, use Fit.Dom.GetScrollBars(..).
/// 	</description>
/// </function>
Fit.Browser.GetScrollBars = function()
{
	var res = { Vertical: { Enabled: false, Size: 0 }, Horizontal: { Enabled: false, Size: 0 } };

	// NOTICE: clientWidth on <html> (scroll document) does not behave
	// identical to clientWidth on other elements. For the document element
	// the size of the viewport is returned. For other DOM elements the
	// inner width is returned:
	// https://developer.mozilla.org/en-US/docs/Web/API/Element/clientWidth

	var doc = Fit.Browser.GetScrollDocument(); // <html> in standards mode, otherwise <body>

	// Notice that overflow:hidden set on <body> propagates to the viewport as
	// defined by the specification (https://www.w3.org/TR/CSS22/visufx.html#propdef-overflow):
	// "UAs must apply the 'overflow' property set on the root element to the viewport. When the root element
	//  is an HTML "HTML" element or an XHTML "html" element, and that element has an HTML "BODY" element or an
	//  XHTML "body" element as a child, user agents must instead apply the 'overflow' property from the first
	//  such child element to the viewport, if the value on the root element is 'visible'. The 'visible' value
	//  when used for the viewport must be interpreted as 'auto'. The element from which the value is propagated
	//  must have a used value for 'overflow' of 'visible'."
	// Therefore, if overflow:hidden is applied to <body> or <html>, we do not consider the scrollbars enabled,
	// even though scrollWidth might be greater than clientWidth, and/or scrollHeight might be greater than clientHeight,
	// which will be the case when content is hidden due to overflow. Scrollbars are hidden!
	// Also notice that even if <html> or <body> was manipulated to display as a box within the viewport,
	// scrollbars would still be attached to the sides of the viewport area of the browser. Example:
	// <body style="width: 100px; height: 100px; border: 1px solid red; overflow: scroll;">

	if (doc.clientWidth < doc.scrollWidth && Fit.Browser.GetComputedStyle(document.body, "overflow-x") !== "hidden" && Fit.Browser.GetComputedStyle(document.documentElement, "overflow-x") !== "hidden")
	{
		res.Horizontal.Enabled = true;
		res.Horizontal.Size = Fit.Browser.GetScrollBarSize();
	}

	if (doc.clientHeight < doc.scrollHeight && Fit.Browser.GetComputedStyle(document.body, "overflow-y") !== "hidden" && Fit.Browser.GetComputedStyle(document.documentElement, "overflow-y") !== "hidden")
	{
		res.Vertical.Enabled = true;
		res.Vertical.Size = Fit.Browser.GetScrollBarSize();
	}

	return res;
}

/// <function container="Fit.Browser" name="GetScrollBarSize" access="public" static="true" returns="integer">
/// 	<description>
/// 		Get thickness of browser scrollbars. The function works even when
/// 		no scrollbars are currently present. For browsers and operating systems
/// 		hiding scrollbars for scrollable objects, the value returned will be 0 (zero).
/// 	</description>
/// </function>
Fit.Browser.GetScrollBarSize = function()
{
	if (Fit._internal.Browser.ScrollBarWidth === undefined)
	{
		var outer = document.createElement("div");
		outer.style.cssText = "visibility: hidden; overflow: scroll; position: fixed; top: -100px; left: -100px;";
		document.documentElement.appendChild(outer); //document.body.appendChild(outer);

		var inner = document.createElement("div");
		outer.appendChild(inner);

		Fit._internal.Browser.ScrollBarWidth = outer.offsetWidth - inner.offsetWidth;

		outer.parentNode.removeChild(outer);
	}

	return Fit._internal.Browser.ScrollBarWidth;
}

/// <function container="Fit.Browser" name="GetComputedStyle" access="public" static="true" returns="string | null">
/// 	<description>
/// 		Get style value applied after stylesheets have been loaded.
/// 		An empty string or null may be returned if style has not been defined or does not exist.
/// 		Make sure not to use shorthand properties (e.g. border-color or padding) as some browsers are
/// 		not capable of calculating these - use the fully qualified property name (e.g. border-left-color
/// 		or padding-left).
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element which contains desired CSS style value </param>
/// 	<param name="style" type="string"> CSS style property name </param>
/// </function>
Fit.Browser.GetComputedStyle = function(elm, style)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(style);

	var res = null;

	if (window.getComputedStyle) // W3C
	{
		res = window.getComputedStyle(elm)[style];
	}
	else if (elm.currentStyle)
	{
		if (style.indexOf("-") !== -1) // Turn e.g. border-bottom-style into borderBottomStyle which is required by legacy browsers
		{
			var items = style.split("-");
			style = "";

			Fit.Array.ForEach(items, function(i)
			{
				if (style === "")
					style = i;
				else
					style += i[0].toUpperCase() + i.slice(1); // First character is uppercased (from Fit.String.UpperCaseFirst(..))
			});
		}

		res = elm.currentStyle[style]; // Might return strings rather than useful values - e.g. "3em" or "medium"

		// IE Computed Style fix by Dean Edwards - http://disq.us/p/myl99x
		// Transform values such as 2em or 4pt to actual pixel values.

		if (res !== undefined && res !== null && /^\d+/.test(res) === true && res.toLowerCase().indexOf("px") === -1) // Non-pixel numeric value
		{
			// Save original value
			var orgLeft = elm.style.left;

			// Calculate pixel value
			var runtimeStyle = elm.runtimeStyle.left;
			elm.runtimeStyle.left = elm.currentStyle.left;
			elm.style.left = ((style === "fontSize") ? "1em" : res || 0); // Throws error for a value such as "medium"
			res = elm.style.pixelLeft + "px";

			// Restore value
			elm.style.left = orgLeft;
			elm.runtimeStyle.left = runtimeStyle;
		}
	}

	return (res !== undefined ? res : null);
}

/// <function container="Fit.Browser" name="GetScreenWidth" access="public" static="true" returns="integer">
/// 	<description> Get screen width </description>
/// 	<param name="onlyAvailable" type="boolean" default="false">
/// 		Set True to return only available space (may be reduced by e.g. start menu (Windows) or Dock (Linux/OSX)
/// 	</param>
/// </function>
Fit.Browser.GetScreenWidth = function(onlyAvailable)
{
	Fit.Validation.ExpectBoolean(onlyAvailable, true);

	if (onlyAvailable === true)
		return window.screen.availWidth;

	return window.screen.width;
}

/// <function container="Fit.Browser" name="GetScreenHeight" access="public" static="true" returns="integer">
/// 	<description> Get screen height </description>
/// 	<param name="onlyAvailable" type="boolean" default="false">
/// 		Set True to return only available space (may be reduced by e.g. start menu (Windows) or Dock (Linux/OSX)
/// 	</param>
/// </function>
Fit.Browser.GetScreenHeight = function(onlyAvailable)
{
	Fit.Validation.ExpectBoolean(onlyAvailable, true);

	if (onlyAvailable === true)
		return window.screen.availHeight;

	return window.screen.height;
}

/// <function container="Fit.Browser" name="GetScreenDimensions" access="public" static="true" returns="Fit.TypeDefs.Dimension">
/// 	<description> Returns object with Width and Height properties specifying screen dimensions </description>
/// 	<param name="onlyAvailable" type="boolean" default="false">
/// 		Set True to return only available space (may be reduced by e.g. Start menu (Windows) or Dock (Linux/OSX)
/// 	</param>
/// </function>
Fit.Browser.GetScreenDimensions = function(onlyAvailable)
{
	Fit.Validation.ExpectBoolean(onlyAvailable, true);
	return { Width: Fit.Browser.GetScreenWidth(onlyAvailable), Height: Fit.Browser.GetScreenHeight(onlyAvailable) };
}

/// <function container="Fit.Browser" name="IsMobile" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Returns value indicating whether device is a mobile device or not.
/// 		Notice that some phones and tablets may identify as desktop devices,
/// 		in which case IsMobile(..) will return False. In this case consider
/// 		using Fit.Browser.IsTouchEnabled(), e.g. in combination with
/// 		Fit.Browser.GetBrowser(), or a check against the size of the viewport,
/// 		which will provide some indication as to whether device should be treated
/// 		as a mobile device or not. As an example, Safari on iPad identifies as
/// 		a Mac computer by default (it has &quot;Request Desktop Website&quot; enabled by default).
/// 		To always detect iPad and iPhone as a mobile device, no matter the configuration
/// 		of &quot;Request Desktop Website&quot;), simply use:
/// 		var isMobile = Fit.Browser.IsMobile(true) || (Fit.Browser.GetBrowser() === &quot;Safari&quot; &amp;&amp; Fit.Browser.IsTouchEnabled())
/// 		We will not be able to distinguish between an iPhone and an iPad though, not even
/// 		by looking at the viewport size, since this approach is unreliable with high resolution
/// 		iPhones and support for split screen which affects the viewport size.
/// 	</description>
/// 	<param name="includeTablets" type="boolean" default="true"> Value indicating whether tablets are considered mobile devices or not </param>
/// </function>
Fit.Browser.IsMobile = function(includeTablets)
{
	Fit.Validation.ExpectBoolean(includeTablets, true);

	// Based on http://detectmobilebrowsers.com
	// See About section for Tablet support: http://detectmobilebrowsers.com/about

	var nav = Fit._internal.Browser.UserAgent;

	if (includeTablets !== false && /android|ipad|playbook|silk/i.test(nav))
		return true;

	return (/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(nav) || /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(nav.substr(0,4)));
}

/// <function container="Fit.Browser" name="IsTouchEnabled" access="public" static="true" returns="boolean">
/// 	<description> Returns value indicating whether device is touch enabled or not </description>
/// </function>
Fit.Browser.IsTouchEnabled = function()
{
	return ("ontouchstart" in window);
}

/// <function container="Fit.Browser" name="Log" access="public" static="true">
/// 	<description> Log message or object </description>
/// 	<param name="msg" type="object"> Message or object to log </param>
/// </function>
Fit.Browser.Log = function(msg) // msg not validated - any object or value (including null/undefined) can be logged
{
	if (window.console)
		console.log(msg);
}

Fit.Browser.Debug = function(msg) // msg not validated - any object or value (including null/undefined) can be logged
{
	if (window.console && Fit._internal.Validation.DebugMode === true)
		console.log(msg);
}

/// <function container="Fit.Browser" name="LogDeprecated" access="public" static="true">
/// 	<description> Log message about use of deprecated feature </description>
/// 	<param name="msg" type="string"> Message to log </param>
/// </function>
Fit.Browser.LogDeprecated = function(msg)
{
	Fit.Validation.ExpectString(msg);

	if (window.console && console.warn)
		console.warn(msg);
	else
		Fit.Browser.Log(msg);

	//if (window.console && console.trace)
		//console.trace();
}

/// <container name="Fit.BrowserTypeDefs.BrowserDetails">
/// 	<description> Object representing browser details </description>
/// 	<member name="Version" type="integer"> Browser version </member>
/// 	<member name="Language" type="string"> Browser language, e.g. en, da, de, etc. </member>
/// 	<member name="IsMobile" type="boolean"> Boolean indicating whether this is a mobile device (tablet or phone) </member>
/// 	<member name="IsPhone" type="boolean"> Boolean indicating whether this is a phone </member>
/// 	<member name="IsTablet" type="boolean"> Boolean indicating whether this is a tablet device </member>
/// 	<member name="IsTouchEnabled" type="boolean"> Boolean indicating whether this is a touch enabled device </member>
/// </container>

/// <container name="Fit.BrowserTypeDefs.BrowserInfo" extends="Fit.BrowserTypeDefs.BrowserDetails">
/// 	<description> Object representing browser environment </description>
/// 	<member name="Name" type='"Edge" | "Chrome" | "Safari" | "MSIE" | "Firefox" | "Opera" | "Unknown"'> Browser name </member>
/// </container>

/// <container name="Fit.BrowserTypeDefs.BrowserAppInfo" extends="Fit.BrowserTypeDefs.BrowserDetails">
/// 	<description> Object representing browser app environment </description>
/// 	<member name="Name" type='"Edge" | "EdgeChromium" | "Chrome" | "Safari" | "MSIE" | "Firefox" | "Opera" | "OperaChromium" | "Unknown"'> Browser app name </member>
/// </container>

/// <function container="Fit.Browser" name="GetInfo" access="public" static="true" returns="Fit.BrowserTypeDefs.BrowserInfo">
/// 	<description> Returns cached object with browser information </description>
/// 	<param name="returnAppInfo" type="false" default="false"> Set True to have app specific browser information returned - see GetBrowser(..) for details </param>
/// </function>
/// <function container="Fit.Browser" name="GetInfo" access="public" static="true" returns="Fit.BrowserTypeDefs.BrowserAppInfo">
/// 	<description> Returns cached object with browser app information </description>
/// 	<param name="returnAppInfo" type="true"> Set True to have app specific browser information returned - see GetBrowser(..) for details </param>
/// </function>
Fit.Browser.GetInfo = function(returnAppInfo)
{
	Fit.Validation.ExpectBoolean(returnAppInfo, true);

	if (returnAppInfo === true)
	{
		if (!Fit._internal.Browser.AppInfo)
		{
			Fit._internal.Browser.AppInfo = Fit.Browser.GetInfo();
			Fit._internal.Browser.AppInfo.Name = Fit.Browser.GetBrowser(true);		// Get app specific browser identifier (e.g. EdgeChromium for the chromed based Edge browser)
			Fit._internal.Browser.AppInfo.Version = Fit.Browser.GetVersion(true);	// Get app specific version number rather than browser engine version number
		}

		return Fit.Core.Clone(Fit._internal.Browser.AppInfo); // Clone to ensure values are not shared and potentially changed
	}

	if (!Fit._internal.Browser.Info)
	{
		Fit._internal.Browser.Info = {};

		Fit._internal.Browser.Info.Name = Fit.Browser.GetBrowser();
		Fit._internal.Browser.Info.Version = Fit.Browser.GetVersion();
		Fit._internal.Browser.Info.Language = Fit.Browser.GetLanguage();
		Fit._internal.Browser.Info.IsMobile = Fit.Browser.IsMobile();
		Fit._internal.Browser.Info.IsPhone = Fit.Browser.IsMobile(false); // Phone only
		Fit._internal.Browser.Info.IsTablet = (Fit.Browser.IsMobile() === true && Fit.Browser.IsMobile(false) === false); // Tablet only
		Fit._internal.Browser.Info.IsTouchEnabled = Fit.Browser.IsTouchEnabled();
	}

	return Fit.Core.Clone(Fit._internal.Browser.Info); // Clone to ensure values are not shared and potentially changed
}

/// <function container="Fit.Browser" name="IsStorageSupported" access="public" static="true" returns="boolean">
/// 	<description> Returns value indicating whether Session and Local storage is supported or not </description>
/// </function>
Fit.Browser.IsStorageSupported = function()
{
	if (Fit._internal.Browser.StorageSupported === undefined)
	{
		Fit._internal.Browser.StorageSupported = false;

		try
		{
			if (window.localStorage && window.sessionStorage)
			{
				var x = "__FITUITEST__";

				localStorage.setItem(x, x);
				localStorage.removeItem(x);

				sessionStorage.setItem(x, x);
				sessionStorage.removeItem(x);

				Fit._internal.Browser.StorageSupported = true;
			}
		}
		catch (err)
		{
		}
	}

	return Fit._internal.Browser.StorageSupported;
}

Fit.Controls = {};
Fit._internal.Controls = {}
Fit._internal.ControlBase = { Controls: {} };

/// <container name="Fit.Controls.Component">
/// 	Class from which all UI components extend
/// </container>

/// <function container="Fit.Controls.Component" name="Component" access="public">
/// 	<description> Component constructor </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique component ID that can be used to access component using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.Component = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);

	var me = this;
	var id = null;
	var container = null;

	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);

	function init()
	{
		Fit._internal.Core.EnsureStyles();

		id = (Fit.Validation.IsSet(controlId) === true ? controlId : "Ctl" + Fit.Data.CreateGuid());

		if (Fit._internal.ControlBase.Controls[id] !== undefined)
			Fit.Validation.ThrowError("Control with ID '" + id + "' has already been defined - Control IDs must be unique!");

		Fit._internal.ControlBase.Controls[id] = me;

		container = document.createElement("div");
		container.id = id;
		container._internal = { Instance: me };

		//Fit.Dom.Data(container, "device", ((Fit.Browser.GetInfo().IsMobile === false) ? "Desktop" : (Fit.Browser.GetInfo().IsPhone === true) ? "Phone" : "Tablet"));
		Fit.Dom.Data(container, "device", Fit.Device.OptimizeForTouch === false ? "Desktop" : "Touch");
	}

	/// <function container="Fit.Controls.Component" name="GetId" access="public" returns="string">
	/// 	<description> Get unique Control ID </description>
	/// </function>
	this.GetId = function()
	{
		return id;
	}

   	/// <function container="Fit.Controls.Component" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description> Get DOMElement representing control </description>
	/// </function>
	this.GetDomElement = function()
	{
		return container;
	}

	/// <function container="Fit.Controls.Component" name="Render" access="public">
	/// 	<description> Render control, either inline or to element specified </description>
	/// 	<param name="toElement" type="DOMElement" default="undefined"> If defined, control is rendered to this element </param>
	/// </function>
	this.Render = function(toElement)
	{
		Fit.Validation.ExpectDomElement(toElement, true);

		if (Fit.Validation.IsSet(toElement) === true)
		{
			Fit.Dom.Add(toElement, me.GetDomElement()); // Using GetDomElement() which may have been overridden, e.g. by ControlBase which does some validation when GetDomElement() is called, or by ContextMenu which returns a different element
		}
		else
		{
			var script = document.scripts[document.scripts.length - 1];
			Fit.Dom.InsertBefore(script, me.GetDomElement()); // Using GetDomElement() which may have been overridden, e.g. by ControlBase which does some validation when GetDomElement() is called, or by ContextMenu which returns a different element
		}
	}

	/// <function container="Fit.Controls.Component" name="Dispose" access="public">
	/// 	<description>
	/// 		Destroys control to free up memory.
	/// 		Make sure to call Dispose() on Component which can be done like so:
	/// 		this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	/// 		{
	/// 			&#160;&#160;&#160;&#160; // Add control specific dispose logic here
	/// 			&#160;&#160;&#160;&#160; base(); // Call Dispose on Component
	/// 		});
	/// 	</description>
	/// </function>
	this.Dispose = function()
	{
		// This will destroy control - it will no longer work!

		Fit.Dom.Remove(container); // Dispose 'container' rather than object returned from GetDomElement() which may have been overridden and potentially returning a different object, in which case the derivative should dispose the object
		delete container._internal; // Removed in case external code holds a reference to DOMElement. Also allows us to internally determine whether control has been disposed or not (e.g. in Fit.Template), since it will always be presented unless disposed.
		delete Fit._internal.ControlBase.Controls[id];
		me = id = container = isIe8 = null;
	}

	// Internals

	this._internal = (this._internal ? this._internal : {});

	this._internal.Repaint = function(f) // Use callback function if a repaint is required both before and after a given operation, which often requires JS thread to be released on IE
	{
		Fit.Validation.ExpectFunction(f, true);

		var cb = ((Fit.Validation.IsSet(f) === true) ? f : function() {});

		if (isIe8 === false)
		{
			cb();
		}
		else
		{
			// Flickering may occure on IE8 when updating UI over time
			// (UI update + JS thread released + UI updates again "later").

			Fit.Dom.AddClass(me.GetDomElement(), "FitUi_Non_Existing_ControlBase_Class");
			Fit.Dom.RemoveClass(me.GetDomElement(), "FitUi_Non_Existing_ControlBase_Class");

			setTimeout(function()
			{
				if (me === null)
					return; // Control has been disposed

				cb();

				Fit.Dom.AddClass(me.GetDomElement(), "FitUi_Non_Existing_ControlBase_Class");
				Fit.Dom.RemoveClass(me.GetDomElement(), "FitUi_Non_Existing_ControlBase_Class");
			}, 0);
		}
	}

	this._internal.IsIe8 = function()
	{
		return isIe8;
	}

	init();
}

/// <container name="Fit.Controls.ControlBase" extends="Fit.Controls.Component">
/// 	Class from which all editable controls extend
/// </container>

/// <function container="Fit.Controls.ControlBase" name="ControlBase" access="public">
/// 	<description> ControlBase constructor </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.ControlBase = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	// ============================================
	// Interface - must be overridden
	// ============================================

	/// <function container="Fit.Controls.ControlBase" name="Value" access="public" returns="string">
	/// 	<description>
	/// 		Get/set control value.
	/// 		For controls supporting multiple selections: Set value by providing a string in one the following formats:
	/// 		title1=val1[;title2=val2[;title3=val3]] or val1[;val2[;val3]].
	/// 		If Title or Value contains reserved characters (semicolon or equality sign), these most be URIEncoded.
	/// 		Selected items are returned in the first format described, also with reserved characters URIEncoded.
	/// 		Providing a new value to this function results in OnChange being fired.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is inserted into control </param>
	/// 	<param name="preserveDirtyState" type="boolean" default="false">
	/// 		If defined, True prevents dirty state from being reset, False (default) resets the dirty state.
	/// 		If dirty state is reset (default), the control value will be compared against the value passed,
	/// 		to determine whether it has been changed by the user or not, when IsDirty() is called.
	/// 	</param>
	/// </function>
	this.Value = function(val, preserveDirtyState)
	{
		// Function MUST remember to fire OnChange event when
		// value is changed, both programmatically and by user.

		Fit.Validation.ThrowError("Not implemented");
	}

	/// <function container="Fit.Controls.ControlBase" name="UserValue" access="public" returns="string">
	/// 	<description>
	/// 		Get/set value as if it was changed by the user. Contrary to Value(..), this function will never reset the dirty state.
	/// 		Restrictions/filtering/modifications may be enforced just as the UI control might do, e.g. prevent the use of certain
	/// 		characters, or completely ignore input if not allowed. It may also allow invalid values such as a partially entered date
	/// 		value. The intention with UserValue(..) is to mimic the behaviour of what the user can do with the user interface control.
	/// 		For picker controls the value format is equivalent to the one dictated by the Value(..) function.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is inserted into control </param>
	/// </function>
	this.UserValue = function(val)
	{
		//Fit.Validation.ThrowError("Not implemented");
		return me.Value(val, true); // Default implementation - change value but do not reset dirty state
	}

	/// <function container="Fit.Controls.ControlBase" name="IsDirty" access="public" returns="boolean">
	/// 	<description> Get value indicating whether user has changed control value </description>
	/// </function>
	this.IsDirty = function()
	{
		Fit.Validation.ThrowError("Not implemented");
	}

	/// <function container="Fit.Controls.ControlBase" name="Clear" access="public">
	/// 	<description> Clear control value </description>
	/// </function>
	this.Clear = function()
	{
		// Function MUST remember to fire OnChange event when
		// value is clear, both programmatically and by user.

		Fit.Validation.ThrowError("Not implemented");
	}

	/// <function container="Fit.Controls.ControlBase" name="Focused" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control has focus.
	/// 		Control must be rooted in DOM and be visible for control to gain focus.
	/// 	</description>
	/// 	<param name="value" type="boolean" default="undefined"> If defined, True assigns focus, False removes focus (blur) </param>
	/// </function>
	this.Focused = function(val)
	{
		Fit.Validation.ThrowError("Not implemented");
	}

	// ============================================
	// Init
	// ============================================

	var me = this;
	var container = me.GetDomElement();
	var width = { Value: 200, Unit: "px" }; // Any changes to this line must be dublicated to Width(..)
	var height = { Value: -1, Unit: "px" };
	var scope = null;
	var required = false;
	var orgDirtyFunction = null;
	var validationExpr = null;			// Obsolete - used by SetValidationExpression
	var validationError = null;			// Obsolete - used by SetValidationExpression
	var validationErrorType = -1;		// 0 = Required, 1 = RegEx validation via SetValidationExpression(..), 2 = Callback validation via SetValidationHandler(..), 3 = Callback validation via SetValidationCallback(..), 4 = RegEx or Callback validation via AddValidationRule(..)
	var validationCallbackFunc = null;	// Obsolete - used by SetValidationCallback
	var validationCallbackError = null;	// Obsolete - used by SetValidationCallback
	var validationHandlerFunc = null;	// Obsolete - used by SetValidationHandler
	var validationHandlerError = null;	// Obsolete - used by SetValidationHandler
	var validationRules = [];
	var validationRuleError = null;
	var lazyValidation = false;
	var lazyValidationDisabled = false;
	var hasValidated = false;
	var blockAutoPostBack = false; // Used by AutoPostBack mechanism to prevent multiple postbacks, e.g on double click
	var onChangeHandlers = [];
	var onFocusHandlers = [];
	var onBlurHandlers = [];
	var hasFocus = false;			// Used by OnFocusIn and OnFocusOut handlers
	var onBlurTimeout = null;		// Used by OnFocusIn and OnFocusOut handlers
	var ensureFocusFires = false;	// Used by OnFocusIn and OnFocusOut handlers
	var waitingForFocus = false;	// Used by OnFocusIn and OnFocusOut handlers
	var focusStateLocked = false;	// Used by OnFocusIn and OnFocusOut handlers
	var txtValue = null;
	var txtDirty = null;
	var txtValid = null;
	var txtEnabled = null;
	var baseControlDisabled = false;	// True if BaseControl's implementation of Enabled(..) disabled the control
	var ie8DisabledLayer = null;		// Layer used to block clicks in IE8 when control is disabled

	function init()
	{
		//container.style.width = width.Value + width.Unit;
		Fit.Dom.AddClass(container, "FitUiControl");

		me._internal.Data("focused", "false");
		me._internal.Data("valid", "true");
		me._internal.Data("dirty", "false");
		me._internal.Data("enabled", "true");

		// Add hidden inputs which are automatically populated with
		// control value and state information when control is updated.

		txtValue = document.createElement("textarea"); // Using Textarea to allow HTML content
		txtValue.style.display = "none";
		txtValue.name = "FitUIValue" + me.GetId();
		Fit.Dom.Add(container, txtValue);

		txtDirty = document.createElement("input");
		txtDirty.type = "hidden";
		txtDirty.name = "FitUIDirty" + me.GetId();
		Fit.Dom.Add(container, txtDirty);

		txtValid = document.createElement("input");
		txtValid.type = "hidden";
		txtValid.name = "FitUIValid" + me.GetId();
		Fit.Dom.Add(container, txtValid);

		txtEnabled = document.createElement("input");
		txtEnabled.type = "hidden";
		txtEnabled.name = "FitUIEnabled" + me.GetId();
		Fit.Dom.Add(container, txtEnabled);

		me.OnChange(function(sender)
		{
			if (blockAutoPostBack === false && me.AutoPostBack() === true && document.forms.length > 0)
			{
				setTimeout(function() // Postpone to allow other handlers to execute before postback
				{
					blockAutoPostBack = true;
					document.forms[0].submit();

					setTimeout(function() { blockAutoPostBack = false; }, 500); // Enable AutoPostBack again in case OnBeforeUnload handler was used to cancel postback
				}, 0);
			}
		});

		if (me._internal.IsIe8() === false)
		{
			// Notice: Using Capture (true argument) for these handlers,
			// meaning they are fired before the event reach its target.
			Fit.Events.AddHandler(container, "focus", true, onFocusIn);
			Fit.Events.AddHandler(container, "blur", true, onFocusOut);
		}
		else // Legacy IE does not support event capturing used above
		{
			container.onfocusin = onFocusIn;
			container.onfocusout = onFocusOut;
		}

		me.OnBlur(function(sender)
		{
			if (lazyValidation === true && lazyValidationDisabled === false)
			{
				lazyValidationDisabled = true;
				me._internal.Validate();
			}
		});

		Fit.Internationalization.OnLocaleChanged(localize);
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.ControlBase" name="AutoPostBack" access="public" returns="boolean">
	/// 	<description> Set flag indicating whether control should post back changes automatically when value is changed </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables auto post back, False disables it </param>
	/// </function>
	this.AutoPostBack = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			me._internal.Data("autopost", val.toString());
		}

		return (me._internal.Data("autopost") === "true");
	}

	this.GetDomElement = Fit.Core.CreateOverride(this.GetDomElement, function()
	{
		if (hasValidated === false)
		{
			hasValidated = true;
			me._internal.Validate();
			updateInternalState(); // Make sure state posted to server is up to date (Dirty and Valid flags) in case a control value has not been assigned
		}

		return base();
	});

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		Fit.Internationalization.RemoveOnLocaleChanged(localize);
		me = container = width = height = scope = required = validationExpr = validationError = validationErrorType = validationCallbackFunc = validationCallbackError = validationHandlerFunc = validationHandlerError = validationRules = validationRuleError = lazyValidation = lazyValidationDisabled = hasValidated = blockAutoPostBack = onChangeHandlers = onFocusHandlers = onBlurHandlers = hasFocus = onBlurTimeout = ensureFocusFires = waitingForFocus = focusStateLocked = txtValue = txtDirty = txtValid = txtEnabled = baseControlDisabled = ie8DisabledLayer = null;
		base();
	});


	/// <function container="Fit.Controls.ControlBase" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control width is updated to specified value. A value of -1 resets control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control width is updated to specified CSS unit </param>
	/// </function>
	this.Width = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				width = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				container.style.width = width.Value + width.Unit;
			}
			else
			{
				width = { Value: 200, Unit: "px" }; // Any changes to this line must be dublicated to line declaring the width variable !
				container.style.width = "";
			}
		}

		return width;
	}

	/// <function container="Fit.Controls.ControlBase" name="Height" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control height is updated to specified value. A value of -1 resets control height. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control height is updated to specified CSS unit </param>
	/// </function>
	this.Height = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			height = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true && val !== -1) ? unit : "px") };

			if (height.Value > -1)
				container.style.height = height.Value + height.Unit;
			else
				container.style.height = "";
		}

		return height;
	}

	/// <function container="Fit.Controls.ControlBase" name="AddCssClass" access="public">
	/// 	<description> Add CSS class to DOMElement representing control </description>
	/// 	<param name="val" type="string"> CSS class to add </param>
	/// </function>
	this.AddCssClass = function(val)
	{
		Fit.Validation.ExpectStringValue(val);
		Fit.Dom.AddClass(container, val);
	}

	/// <function container="Fit.Controls.ControlBase" name="RemoveCssClass" access="public">
	/// 	<description> Remove CSS class from DOMElement representing control </description>
	/// 	<param name="val" type="string"> CSS class to remove </param>
	/// </function>
	this.RemoveCssClass = function(val)
	{
		Fit.Validation.ExpectStringValue(val);
		Fit.Dom.RemoveClass(container, val);
	}

	/// <function container="Fit.Controls.ControlBase" name="HasCssClass" access="public" returns="boolean">
	/// 	<description> Check whether CSS class is found on DOMElement representing control </description>
	/// 	<param name="val" type="string"> CSS class to check for </param>
	/// </function>
	this.HasCssClass = function(val)
	{
		Fit.Validation.ExpectStringValue(val);
		return Fit.Dom.HasClass(container, val);
	}

	/// <function container="Fit.Controls.ControlBase" name="Enabled" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control is enabled or disabled.
	/// 		A disabled control's value and state is still included on postback, if part of a form.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined">
	/// 		If defined, True enables control (default), False disables control.
	/// 	</param>
	/// </function>
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== me.Enabled())
		{
			baseControlDisabled = val === false;

			var disableSelector = "input, textarea, select, button";
			var disableEvents = {
				"contextmenu": preventEventDefault,
				"click": stopEventPropagation, // For a link it will also suppress navigation (prevent default) - see stopEventPropagation
				"dblclick": stopEventPropagation,
				"mousedown": stopEventPropagation,
				"mouseup": stopEventPropagation,
				"mousemove": stopEventPropagation,
				"keydown": stopEventPropagation,
				"keypress": stopEventPropagation,
				"keyup": stopEventPropagation,
				"touchstart": stopEventPropagation,
				"touchmove": stopEventPropagation,
				"touchend": stopEventPropagation,
				"touchcancel": stopEventPropagation
			};

			var dom = me.GetDomElement();

			me._internal.Data("enabled", val === true ? "true" : "false");

			if (val === false) // Disable control
			{
				me.Focused(false);

				// Prevent interaction with controls

				if (me._internal.IsIe8() === false)
				{
					// Register events handlers suppressing user interactions such as clicks and keystrokes
					Fit.Array.ForEach(disableEvents, function(eventName)
					{
						Fit.Events.AddHandler(dom, eventName, true, disableEvents[eventName]);
					});
				}
				else // IE8
				{
					ie8DisabledLayer = document.createElement("div");
					ie8DisabledLayer.className = "FitUiControlDisabledLayer";
					Fit.Dom.Add(me.GetDomElement(), ie8DisabledLayer);
				}

				// Disable HTML controls

				Fit.Array.ForEach(dom.querySelectorAll(disableSelector), function(elm)
				{
					// Disabled vs ReadOnly: https://stackoverflow.com/questions/7730695/whats-the-difference-between-disabled-disabled-and-readonly-readonly-for-ht

					if (elm === txtValue || elm === txtDirty || elm === txtValid || elm === txtEnabled)
					{
						return; // Skip controls holding state so we can retain state across postbacks - these controls are hidden and have no event handlers attached
					}

					elm._fitDisabled = elm.disabled;
					elm.disabled = true; // Element will not have events fired (e.g. OnFocus and OnClick), it does not receive focus on TAB navigation, and its value is excluded on form submit - most browsers grays out the element

					elm._fitReadOnly = elm.readOnly;
					elm.readOnly = true; // Merely makes it read only, but without changing the appearance to reflect the read only state - not supported by all element types (e.g. <select> and <button>)
				});

				// Prevent focusable elements from gaining focus via click and tab navigation

				Fit.Array.CustomRecurse([dom], function(elm)
				{
					if (Fit.Dom.Attribute(elm, "tabindex") !== null) // Since .tabIndex always has a value, we have to use Attribute(..) to determine whether it has been explicitely set or not
					{
						elm._fitTabIndex = elm.tabIndex;
						Fit.Dom.Attribute(elm, "tabindex", null); // Remove tabindex - a value of -1 only takes it our of tab flow, but the object remains focusable on click

						if (elm.tagName === "A")
						{
							// Links by default have tabindex set to 0 when tabindex attribute
							// is not explicitely declared. For onFocusIn() to be able to determine
							// whether links can be ignored, we assign -1. See onFocusIn() for details.
							elm.tabIndex = -1;
						}
					}

					return elm.children;
				});
			}
			else if (val === true)
			{
				// Allow user interactions again

				if (me._internal.IsIe8() === false)
				{
					// Remove events handlers suppressing user interactions such as clicks and keystrokes
					Fit.Array.ForEach(disableEvents, function(eventName)
					{
						Fit.Events.RemoveHandler(dom, eventName, true, disableEvents[eventName]);
					});
				}
				else
				{
					Fit.Dom.Remove(ie8DisabledLayer);
					ie8DisabledLayer = null;
				}

				// Re-enable HTML controls

				Fit.Array.ForEach(dom.querySelectorAll(disableSelector), function(elm)
				{
					elm.disabled = elm._fitDisabled;
					elm._fitDisabled = undefined;

					elm.readOnly = elm._fitReadOnly;
					elm._fitReadOnly = undefined;
				});

				// Allow focusable elements to gain focus again

				Fit.Array.CustomRecurse([dom], function(elm)
				{
					if (elm._fitTabIndex !== undefined)
					{
						elm.tabIndex = elm._fitTabIndex;
						elm._fitTabIndex = undefined;
					}

					return elm.children;
				});
			}

			updateInternalState();
			me._internal.Repaint();
		}

		return me._internal.Data("enabled") === "true";
	}

	/// <function container="Fit.Controls.ControlBase" name="Visible" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is visible </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, control visibility is updated </param>
	/// </function>
	this.Visible = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
			container.style.display = ((val === true) ? "" : "none");

		return (container.style.display !== "none");
	}

	/// <function container="Fit.Controls.ControlBase" name="Required" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is required to be set </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, control required feature is enabled/disabled </param>
	/// </function>
	this.Required = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			required = val;
			me._internal.Validate(); // Update error indicator
		}

		return required;
	}

	/// <function container="Fit.Controls.ControlBase" name="Scope" access="public" returns="string">
	/// 	<description>
	/// 		Get/set scope to which control belongs - this is used to validate multiple
	/// 		controls at once using Fit.Controls.ValidateAll(scope) or Fit.Controls.DirtyCheckAll(scope).
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, control scope is updated </param>
	/// </function>
	this.Scope = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === "")
				scope = null;
			else
				scope = val;
		}

		return scope;
	}

	/// <function container="Fit.Controls.ControlBase" name="AlwaysDirty" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control is always considered dirty. This
	/// 		comes in handy when programmatically changing a value of a control on behalf
	/// 		of the user. Some applications may choose to only save values from dirty controls.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, Always Dirty is enabled/disabled </param>
	/// </function>
	this.AlwaysDirty = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && orgDirtyFunction === null)
			{
				orgDirtyFunction = this.IsDirty;
				this.IsDirty = function() { return true; };
				updateInternalState();
			}
			else if (val === false && orgDirtyFunction !== null)
			{
				this.IsDirty = orgDirtyFunction;
				orgDirtyFunction = null;
				updateInternalState();
			}
		}

		return (orgDirtyFunction !== null);
	}

	/// <function container="Fit.Controls.ControlBase" name="SetValidationExpression" access="public">
	/// 	<description>
	/// 		DEPRECATED! Please use AddValidationRule(..) instead.
	/// 		Set regular expression used to perform on-the-fly validation against control value.
	/// 	</description>
	/// 	<param name="regEx" type="RegExp | null"> Regular expression to validate against </param>
	/// 	<param name="errorMsg" type="string" default="undefined">
	/// 		If defined, specified error message is displayed when user clicks or hovers validation error indicator
	/// 	</param>
	/// </function>
	this.SetValidationExpression = function(regEx, errorMsg)
	{
		Fit.Validation.ExpectRegExp(regEx, true); // Allow Null/undefined which disables validation
		Fit.Validation.ExpectString(errorMsg, true);

		Fit.Browser.LogDeprecated("Use of deprecated function SetValidationExpression - please use AddValidationRule instead");

		validationExpr = (regEx ? regEx : null);
		validationError = (errorMsg ? errorMsg : null);

		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBase" name="SetValidationCallback" access="public">
	/// 	<description>
	/// 		DEPRECATED! Please use AddValidationRule(..) instead.
	/// 		Set callback function used to perform on-the-fly validation against control value.
	/// 	</description>
	/// 	<param name="cb" type="function | null"> Function receiving control value - must return True if value is valid, otherwise False </param>
	/// 	<param name="errorMsg" type="string" default="undefined">
	/// 		If defined, specified error message is displayed when user clicks or hovers validation error indicator
	/// 	</param>
	/// </function>
	this.SetValidationCallback = function(cb, errorMsg)
	{
		Fit.Validation.ExpectFunction(cb, true); // Allow Null/undefined which disables validation
		Fit.Validation.ExpectString(errorMsg, true);

		Fit.Browser.LogDeprecated("Use of deprecated function SetValidationCallback - please use AddValidationRule instead");

		validationHandlerFunc = null;
		validationCallbackFunc = (cb ? cb : null);;
		validationCallbackError = (errorMsg ? errorMsg : null);

		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBase" name="SetValidationHandler" access="public">
	/// 	<description>
	/// 		DEPRECATED! Please use AddValidationRule(..) instead.
	/// 		Set callback function used to perform on-the-fly validation against control value
	/// 	</description>
	/// 	<param name="cb" type="function | null">
	/// 		Function receiving an instance of the control and its value.
	/// 		An error message string must be returned if value is invalid,
	/// 		otherwise Null or an empty string if the value is valid.
	/// 	</param>
	/// </function>
	this.SetValidationHandler = function(cb)
	{
		Fit.Validation.ExpectFunction(cb, true); // Allow Null/undefined which disables validation

		Fit.Browser.LogDeprecated("Use of deprecated function SetValidationHandler - please use AddValidationRule instead");

		validationCallbackFunc = null;
		validationHandlerFunc = (cb ? cb : null);
		validationHandlerError = null;

		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBaseTypeDefs" name="ValidationCallback" returns="boolean | string | void">
	/// 	<description> Validation callback used with AddValidationRule(..) inherited from Fit.Controls.ControlBase </description>
	/// 	<param name="sender" type="$TypeOfThis"> Control to validate </param>
	/// </function>

	/// <function container="Fit.Controls.ControlBase" name="AddValidationRule" access="public">
	/// 	<description> Set callback function used to perform on-the-fly validation against control </description>
	/// 	<param name="validator" type="Fit.Controls.ControlBaseTypeDefs.ValidationCallback">
	/// 		Function receiving an instance of the control.
	/// 		A value of False or a non-empty string with an
	/// 		error message must be returned if value is invalid.
	/// 	</param>
	/// </function>
	/// <function container="Fit.Controls.ControlBase" name="AddValidationRule" access="public">
	/// 	<description> Set regular expression used to perform on-the-fly validation against control value, as returned by the Value() function </description>
	/// 	<param name="validator" type="RegExp"> Regular expression to validate value against </param>
	/// 	<param name="errorMessage" type="string" default="undefined"> Optional error message displayed if value validation fails </param>
	/// </function>
	this.AddValidationRule = function(validator, errorMessage)
	{
		Fit.Validation.ExpectIsSet(validator);
		Fit.Validation.ExpectString(errorMessage, true);

		if (typeof(validator) === "function")
		{
			Fit.Validation.ExpectFunction(validator);
			validationRules.push( { Type: "Callback", Validator: validator, ErrorMessage: null } );
		}
		else
		{
			Fit.Validation.ExpectRegExp(validator);
			validationRules.push( { Type: "RegExp", Validator: validator, ErrorMessage: errorMessage || null } );
		}

		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBase" name="RemoveValidationRule" access="public">
	/// 	<description> Remove validation function used to perform on-the-fly validation against control </description>
	/// 	<param name="validator" type="Fit.Controls.ControlBaseTypeDefs.ValidationCallback"> Validation function registered using AddValidationRule(..) </param>
	/// </function>
	/// <function container="Fit.Controls.ControlBase" name="RemoveValidationRule" access="public">
	/// 	<description> Remove regular expression used to perform on-the-fly validation against control value </description>
	/// 	<param name="validator" type="RegExp"> Regular expression registered using AddValidationRule(..) </param>
	/// </function>
	this.RemoveValidationRule = function(validator) // Function or RegExp
	{
		var found = null;

		Fit.Array.ForEach(validationRules, function(rule)
		{
			if (rule.Validator === validator)
			{
				found = rule;
				return false; // Break loop
			}
		});

		if (found !== null)
		{
			Fit.Array.Remove(validationRules, found);
		}

		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBase" name="RemoveAllValidationRules" access="public">
	/// 	<description> Remove all validation rules </description>
	/// </function>
	this.RemoveAllValidationRules = function()
	{
		validationRules = [];
		me._internal.Validate();
	}

	/// <function container="Fit.Controls.ControlBase" name="IsValid" access="public" returns="boolean">
	/// 	<description>
	/// 		Get value indicating whether control value is valid.
	/// 		Control value is considered invalid if control is required, but no value is set,
	/// 		or if control value does not match regular expression set using SetValidationExpression(..).
	/// 	</description>
	/// </function>
	this.IsValid = function()
	{
		validationErrorType = -1;
		validationHandlerError = null;
		validationRuleError = null;

		if (validationExpr === null && validationCallbackFunc === null && validationHandlerFunc === null && required === false && validationRules.length === 0)
			return true;

		var obj = me.Value();
		var val = ((Fit.Validation.IsSet(obj) === true) ? obj.toString() : "");

		if (required === true && val === "")
		{
			validationErrorType = 0;
			return false;
		}

		if (validationExpr !== null && validationExpr.test(val) === false)
		{
			validationErrorType = 1;
			return false;
		}

		if (validationHandlerFunc !== null)
		{
			var errorMessage = validationHandlerFunc(me, val);

			if (errorMessage !== null && errorMessage !== "" && typeof(errorMessage) === "string")
			{
				validationErrorType = 2;
				validationHandlerError = errorMessage;
				return false;
			}
		}
		else if (validationCallbackFunc !== null && validationCallbackFunc(val) === false)
		{
			validationErrorType = 3;
			return false;
		}

		if (validationRules.length > 0)
		{
			Fit.Array.ForEach(validationRules, function(rule)
			{
				if (rule.Type === "Callback")
				{
					var result = rule.Validator(me);

					if (result === false || (typeof(result) === "string" && result !== ""))
					{
						validationErrorType = 4;
						validationRuleError = result || null;
						return false; // Break loop
					}
				}
				else // RegExp
				{
					if (rule.Validator.test(me.Value()) === false)
					{
						validationErrorType = 4;
						validationRuleError = rule.ErrorMessage;
						return false; // Break loop
					}
				}
			});

			if (validationErrorType === 4)
			{
				return false;
			}
		}

		return true;
	}

	/// <function container="Fit.Controls.ControlBase" name="LazyValidation" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control initially appears as valid, even
	/// 		though it is not. It will appear invalid once the user touches the control,
	/// 		or when control value is validated using Fit.Controls.ValidateAll(..).
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, Lazy Validation is enabled/disabled </param>
	/// </function>
	this.LazyValidation = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			lazyValidation = val;
			lazyValidationDisabled = false;

			if (lazyValidation === true)
			{
				me._internal.Data("valid", "true");
				me._internal.Data("errormessage", null);
				Fit.Dom.Attribute(me.GetDomElement(), "title", null); // Title attribute holds error message in IE8
			}
			else
			{
				this._internal.Validate();
			}
		}

		return lazyValidation;
	}

	// ============================================
	// Events
	// ============================================

	/// <function container="Fit.Controls.ControlBaseTypeDefs" name="BaseEvent">
	/// 	<description> Event handler receiving an instance of the control firing the event </description>
	/// 	<param name="sender" type="$TypeOfThis"> Control instance </param>
	/// </function>

	/// <function container="Fit.Controls.ControlBase" name="OnChange" access="public">
	/// 	<description> Register OnChange event handler which is invoked when control value is changed either programmatically or by user </description>
	/// 	<param name="cb" type="Fit.Controls.ControlBaseTypeDefs.BaseEvent"> Event handler function which accepts Sender (ControlBase) </param>
	/// </function>
	this.OnChange = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onChangeHandlers, cb);
	}

	/// <function container="Fit.Controls.ControlBase" name="OnFocus" access="public">
	/// 	<description> Register OnFocus event handler which is invoked when control gains focus </description>
	/// 	<param name="cb" type="Fit.Controls.ControlBaseTypeDefs.BaseEvent"> Event handler function which accepts Sender (ControlBase) </param>
	/// </function>
	this.OnFocus = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onFocusHandlers, cb);
	}

	/// <function container="Fit.Controls.ControlBase" name="OnBlur" access="public">
	/// 	<description> Register OnBlur event handler which is invoked when control loses focus </description>
	/// 	<param name="cb" type="Fit.Controls.ControlBaseTypeDefs.BaseEvent"> Event handler function which accepts Sender (ControlBase) </param>
	/// </function>
	this.OnBlur = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onBlurHandlers, cb);
	}

	// ============================================
	// Protected
	// ============================================

	this._internal = (this._internal ? this._internal : {});

	// Disable control but keep it focused if already focused, so OnBlur doesn't fire
	this._internal.DisableAndKeepFocus = function()
	{
		if (me.Focused() === true)
		{
			// Disable control.
			// Disabling the control removes focus from it. OnBlur will fire in a moment (async.
			// via onFocusOut which fires immediately). We prevent OnBlur from firing below.
			me.Enabled(false);

			// Enabled(false) above scheduled OnBlur for execution since onFocusOut fires immediately, and since control is
			// now disabled, reassigning focus below will not cancel upcoming OnBlur event invocation. Therefore we cancel it here.
			clearTimeout(onBlurTimeout);
			onBlurTimeout = null;

			// Assign focus to control's outer container to keep focus and prevents OnBlur from firing.
			// After this point we have prevented OnBlur from firing. However, OnBlur will still fire if
			// focus is removed from the outer container, which is the intended behaviour. If this happens,
			// focus will not be restored back to the control when it is enabled again via restore function below.
			var dom = me.GetDomElement();
			var orgTabIndex = Fit.Dom.Attribute(dom, "tabindex");
			Fit.Dom.Attribute(dom, "tabindex", "-1");
			dom.focus({ preventScroll: true });

			// Return restore function
			return function()
			{
				me.Enabled(true);

				// Move focus from outer container back to control's intended focus target,
				// but only if user did not remove focus from control while control was disabled.
				if (hasFocus === true)
				{
					me.Focused(true);
				}

				Fit.Dom.Attribute(dom, "tabindex", orgTabIndex);
			};
		}
		else
		{
			// Disable control
			me.Enabled(false);

			// Return restore function
			return function() { me.Enabled(true); };
		}
	}

	// ============================================
	// Private
	// ============================================

	function stopEventPropagation(e) // Used to suppress user interaction when control is disabled
	{
		Fit.Events.StopPropagation(e);

		if (e.type === "click" && Fit.Events.GetTarget(e).tagName === "A")
		{
			preventEventDefault(e); // Prevent navigation
		}
	}

	function preventEventDefault(e) // Used to suppress user interaction when control is disabled
	{
		Fit.Events.PreventDefault(e);
	}

	// NOTICE - IMPORTANT: Regarding invocation of onFocusIn and onFocusOut:
	// We can't trust onFocusIn and onFocusOut to be fired in turn like in/out/in/out.
	// Events might be fired like in/out/out/in/in like this example demonstrates:
	// https://codepen.io/JemtDK/pen/abqRqNy?editors=0010
	// More information in this issue: https://github.com/Jemt/Fit.UI/issues/164

	function onFocusIn(e)
	{
		if (baseControlDisabled === true && Fit.Dom.GetFocused() !== me.GetDomElement())
		{
			// An element in control gained focus, even though control is disabled. This might happen
			// if data is loaded async and control's DOM is populated after control has been disabled.

			var focused = Fit.Dom.GetFocused(); // Element that gained focus

			if (me._internal.IsIe8() === true && focused === ie8DisabledLayer)
			{
				return; // User clicked on IE8's Disabled Layer - IE fires OnFocusIn, even for elements that are not focusable
			}

			// Enable and re-disable control if an element within the control received focus,
			// which will ensure focusable elements within control lose their ability to receive focus.
			// Notice: the focus event always fires for hyperlinks, no matter what, since they are always
			// focusable. So unless the hyperlink is explicitely set to be focusable on tab navigation
			// (tabIndex >= 0), there is no need to enable and re-disable the control. Enabled(false) will
			// make sure to set tabIndex to -1 for links so we do not trigger the code below again and again.
			if (focused.tagName !== "A" || focused.tabIndex >= 0)
			{
				// Display warning so we don't get confused as to why tab navigation suddently acts weird.
				// If this gets annoying, the control triggering this warning should override Enabled(..) to make its
				// own specific implementation, rather than relying on ControlBase's very generic (but free) approach.
				var msg = "";
				msg += "Disabled control '" + me.GetId() + "' received focus because an object was introduced in the control ";
				msg += "that is focusable, which did not exist when the control was disabled. Control's disabled state will ";
				msg += "now be updated to prevent this from happening again with the control's current state. Since the control ";
				msg += "has already received focus, and we do not know the exact tab navigation order of the page, focus will ";
				msg += "now be returned to the component previously focused. The user will experience this as TAB navigation "
				msg += "being ignored for this keystroke, but it will work the next time. To avoid this, make sure to disable ";
				msg += "the control once it is ready - e.g. fully populated.";
				console.warn(msg);

				me.Enabled(true);
				me.Enabled(false);
			}

			// Since control is not focusable when disabled, and we don't know
			// which element is the next/previous focusable one, we return focus
			// to the element from which focus was handed over (previously focused).
			var related = Fit.Events.GetRelatedTarget(e);
			if (related !== null && related.tabIndex >= 0)
			{
				//console.warn("Returning focus to previously focused element");
				related.focus();
			}
			else
			{
				//console.warn("Removing focus from disabled control");
				Fit.Dom.GetFocused().blur(); // There is either no related element (first focus on page), or it has tabIndex -1 (e.g. a hyperlink which had its tabIndex set to -1 when Enabled(false) was called)
			}
		}

		if (me.Enabled() === false)
			return;

		if (focusStateLocked === true)
			return;

		// Note on how OnFocus and OnBlur is handled:
		// OnFocus in JS fires for focusable elements only, meaning
		// elements with tabIndex set.
		// If an element within a control is clicked, and that
		// particular element is not focusable, <body> will be
		// focused by the browser, causing the control to lose focus
		// and fire OnBlur.
		// Programmatically re-assigning focus within a control
		// will not cause OnBlur to fire since onFocusIn cancels
		// onFocusOut.
		// Notice that onFocusOut is triggered for an element losing
		// focus before onFocusIn is triggered for an element gaining focus.
		// See https://jsfiddle.net/34q5n7xm/8/ to understand how
		// onFocusIn/Out fires internally and how OnFocus and OnBlur
		// fires externally.

		// Cancel OnBlur
		if (onBlurTimeout !== null)
		{
			clearTimeout(onBlurTimeout);
			onBlurTimeout = null;
		}

		if (waitingForFocus === true) // Ensure FIFO execution of OnFocus and OnBlur - see onFocusOut handler for details
		{
			// Control regained focus while waiting for OnFocus and OnBlur to fire.
			// OnBlur timer was canceled above, so we'll just let the initial FocusIn timer fire.
			// This happens if FIFO execution for timers are not honored.
			// See onFocusOut handler for details.

			waitingForFocus = false;
			return;
		}

		// Make sure OnFocus only fires once when control is initially given
		// focus - prevent it from firing when focus is changed internally.
		if (hasFocus === true)
			return;

		hasFocus = true;

		// Make sure control does in fact fire initial OnFocus event,
		// in case control is only assigned focus shortly - otherwise
		// OnBlur may fire without OnFocus firing first if FIFO execution
		// of timers are not honored. See onFocusOut handler for details.
		ensureFocusFires = true;

		// Queue, event has not reached target yet (using Capture instead of event bubbling).
		// Target must have focus before firing OnFocus. This approach also allow for control
		// to internally change focus (e.g. in TreeView where focus may change from one node
		// to another) which may fire both Focus and Blur multiple times.
		setTimeout(function()
		{
			if (me === null)
				return; // Control was disposed

			ensureFocusFires = false;

			me._internal.FireOnFocus();
		}, 0);
	}

	function onFocusOut(e)
	{
		if (me === null) // Disposed while focused (e.g. from an onscroll event handler)
			return;

		// Make sure OnBlur does not fire unless OnFocus was fired first. This prevents OnBlur from firing
		// if a disabled control allow some elements to gain focus such as links via tab navigation.
		if (hasFocus === false)
			return;

		if (focusStateLocked === true)
			return;

		// Browsers might fire onFocusOut multiple times without firing onFocusIn in between.
		// See issue for more information: https://github.com/Jemt/Fit.UI/issues/164
		if (onBlurTimeout !== null)
		{
			clearTimeout(onBlurTimeout);
			onBlurTimeout = null;
		}

		// See comments in onFocusIn(..)

		var fireBlur = null;
		fireBlur = function()
		{
			if (me === null)
				return; // Control was disposed

			if (ensureFocusFires === true) // Make absolutely sure initial OnFocus has fired before firing OnBlur
			{
				// Browsers can, in some situations, disregard FIFO execution for timers, for instance when attaching
				// a debugger, or if the browser suspends JS execution due to power management, especially on mobile devices.
				// Since the control has already lost focus at this point, it's safe to schedule another
				// timer to fire OnBlur once OnFocus has been fired. We will keep checking to make sure OnFocus fires first.
				// OnFocusOut should not be called again since the control no longer has focus, and if the control regains focus
				// while waiting for OnFocus and OnBlur to fire, then onFocusIn will cancel the OnBlur timer and let the
				// initial OnFocus timer execute (see the use of waitingForFocus in onFocusIn).
				// To simulate the FIFO problem, set the OnFocus timer to timeout late, e.g. after 1500 ms.
				// Then focus the control and immediately remove focus from it again. Register an OnFocus
				// and OnBlur handler to observe that OnFocus does in fact fire before OnBlur.
				waitingForFocus = true;

				onBlurTimeout = setTimeout(fireBlur, 0);
				return;
			}

			onBlurTimeout = null;
			waitingForFocus = false;
			hasFocus = false; // Control has lost focus, allow OnFocus to fire again when it regains focus

			me._internal.FireOnBlur();
		};

		onBlurTimeout = setTimeout(fireBlur, 0);
	}

	function updateInternalState()
	{
		txtValue.value = me.Value().toString(); // TBD: Why .toString() ? It always returns a string!
		txtDirty.value = ((me.IsDirty() === true) ? "1" : "0");
		txtValid.value = ((me.IsValid() === true) ? "1" : "0");
		txtEnabled.value = ((me.Enabled() === true) ? "1" : "0");

		me._internal.Data("dirty", ((me.IsDirty() === true) ? "true" : "false"));
	}

	function localize()
	{
		if (validationErrorType === 0 /*&& me._internal.Data("errormessage") !== null*/)
		{
			var locale = Fit.Internationalization.GetSystemLocale();
			me._internal.Data("errormessage", locale.Translations.Required);

			if (me._internal.IsIe8() === true)
			{
				Fit.Dom.Attribute(me.GetDomElement(), "title", locale.Translations.Required);
			}
		}
	}

	// Private members (must be public in order to be accessible to controls extending from ControlBase)

	this._internal.FireOnChange = function()
	{
		hasValidated = true;
		me._internal.Validate();
		updateInternalState();

		Fit.Array.ForEach(onChangeHandlers, function(cb)
		{
			cb(me);
		});
	}

	this._internal.FireOnFocus = function()
	{
		ensureFocusFires = false; // Usually set to False in onfocusin event handler, but specialized controls may fire OnFocus as well, in which case we assume control has in fact gained focus

		me._internal.Data("focused", "true");
		me._internal.Repaint();

		Fit.Array.ForEach(onFocusHandlers, function(cb)
		{
			cb(me);
		});
	}

	this._internal.FireOnBlur = function()
	{
		me._internal.Data("focused", "false");
		me._internal.Repaint();

		Fit.Array.ForEach(onBlurHandlers, function(cb)
		{
			cb(me);
		});
	}

	this._internal.FocusStateLocked = function(value)
	{
		Fit.Validation.ExpectBoolean(value, true);

		// Prevent control from firing OnFocus (onfocusin) and OnBlur (onfocusout) automatically.
		// Specialized controls can use this to either suppress OnFocus and OnBlur invocation temporarily,
		// or take over the responsibility of handling invocation of OnBlur and OnFocus.
		// This is useful if a control for instance opens a modal dialog and gives it focus, in which case the
		// control would lose focus and fire OnBlur. But since the dialog is considered part of the control, we
		// do not want OnBlur to fire. We can use FocusStateLocked(..) to make the control preserve its current
		// focused state, and let the specialized control handle invocation of focus events when needed, and hand
		// back control to ControlBase when the modal dialog closes.

		if (Fit.Validation.IsSet(value) === true)
		{
			if (value !== focusStateLocked)
			{
				focusStateLocked = value;

				// Make sure ControlBase can handle focus in/out properly when focus state is unlocked again

				var meElement = me.GetDomElement();
				var focusElement = Fit.Dom.GetFocused();
				hasFocus = meElement === focusElement || Fit.Dom.Contained(meElement, focusElement) === true;
			}
		}

		return focusStateLocked;
	}

	this._internal.ExecuteWithNoOnChange = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);

		var onChangeHandler = me._internal.FireOnChange;
		me._internal.FireOnChange = function() {};
		me._internal.FireOnChangeSuppressed = true; // Allow specialized controls to detect when OnChange event will be suppressed for performance optimizations

		var error = null;

		try // Try/catch to make absolutely sure OnChange handler is restored!
		{
			cb();
		}
		catch (err)
		{
			error = err.message;
		}

		me._internal.FireOnChange = onChangeHandler;
		me._internal.FireOnChangeSuppressed = false;

		if (error !== null)
			Fit.Validation.ThrowError(error);
	}
	this._internal.FireOnChangeSuppressed = false;

	this._internal.Data = function(key, val)
	{
		Fit.Validation.ExpectStringValue(key);
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true || val === null)
			Fit.Dom.Data(container, key, val);

		return Fit.Dom.Data(container, key);
	}

	this._internal.AddDomElement = function(elm)
	{
		Fit.Validation.ExpectDomElement(elm);
		Fit.Dom.InsertBefore(txtValue, elm); //Fit.Dom.Add(container, elm);
	}

	this._internal.RemoveDomElement = function(elm)
	{
		Fit.Validation.ExpectDomElement(elm);
		Fit.Dom.Remove(elm);
	}

	this._internal.Validate = function(force)
	{
		Fit.Validation.ExpectBoolean(force, true);

		if (force === true)
		{
			lazyValidationDisabled = true;
		}

		// For LazyValidation the UI is only updated if control has focus, unless force is true (see above)
		if (lazyValidation === true && lazyValidationDisabled === false && me.Focused() === false)
			return;

		var valid = me.IsValid();

		me._internal.Data("valid", valid.toString());

		if (valid === false)
		{
			me._internal.Data("errormessage", null);

			if (validationErrorType === 0)
				me._internal.Data("errormessage", Fit.Internationalization.GetSystemLocale().Translations.Required);
			else if (validationErrorType === 1 && validationError !== null)
				me._internal.Data("errormessage", validationError.replace("\r", "").replace(/<br.*>/i, "\n"));
			else if (validationErrorType === 2 && validationHandlerError !== null)
				me._internal.Data("errormessage", validationHandlerError.replace("\r", "").replace(/<br.*>/i, "\n"));
			else if (validationErrorType === 3 && validationCallbackError !== null)
				me._internal.Data("errormessage", validationCallbackError.replace("\r", "").replace(/<br.*>/i, "\n"));
			else if (validationErrorType === 4 && validationRuleError !== null)
				me._internal.Data("errormessage", validationRuleError.replace("\r", "").replace(/<br.*>/i, "\n"));
		}
		else
		{
			me._internal.Data("errormessage", null);
		}

		if (me._internal.IsIe8() === true)
		{
			Fit.Dom.Attribute(me.GetDomElement(), "title", me._internal.Data("errormessage"));
		}

		me._internal.Repaint();
	}

	this._internal.UpdateInternalState = function()
	{
		updateInternalState();
	}

	init();
}

// ============================================
// Public static
// ============================================

/// <function container="Fit.Controls" name="Find" access="public" static="true" returns="Fit.Controls.Component | null">
/// 	<description> Get control by unique Control ID - returns Null if not found </description>
/// 	<param name="id" type="string"> Unique Control ID </param>
/// </function>
/// <function container="Fit.Controls" name="Find" access="public" static="true" returns="$ExpectedControlType | null">
/// 	<description> Get control by unique Control ID - returns Null if not found </description>
/// 	<param name="id" type="string"> Unique Control ID </param>
/// 	<param name="expectedType" type="$ExpectedControlType">
/// 		For development environments supporting JSDoc and generics (e.g. VSCode), make Find(..) return found component
/// 		as specified type. For instance to return a type as Fit.Controls.DropDown, specify Fit.Controls.DropDown.prototype.
/// 	</param>
/// </function>
Fit.Controls.Find = function(id, expectedType) // The expectedType argument is there only to add support for a generic return type via typings
{
	Fit.Validation.ExpectStringValue(id);
	return ((Fit._internal.ControlBase.Controls[id] !== undefined) ? Fit._internal.ControlBase.Controls[id] : null);
}

/// <function container="Fit.Controls" name="ValidateAll" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Validate all controls - either all controls on page, or within specified scope.
/// 		Returns True if all controls contain valid values, otherwise False.
/// 	</description>
/// 	<param name="scope" type="string" default="undefined">
/// 		If specified, validate controls only within this scope.
/// 		See Fit.Controls.ControlBase.Scope(..)
/// 		for details on configurering scoped validation.
/// 	</param>
/// </function>
Fit.Controls.ValidateAll = function(scope)
{
	Fit.Validation.ExpectStringValue(scope, true);

	var result = true;

	Fit.Array.ForEach(Fit._internal.ControlBase.Controls, function(controlId)
	{
		var control = Fit._internal.ControlBase.Controls[controlId];

		if (Fit.Core.InstanceOf(control, Fit.Controls.ControlBase) === false)
			return;

		if (Fit.Validation.IsSet(scope) === true && control.Scope() !== scope)
			return;

		control._internal.Validate(true);

		if (control.IsValid() === false)
			result = false;
	});

	return result;
}

/// <function container="Fit.Controls" name="DirtyCheckAll" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Check dirty state for all controls - either all controls on page, or within specified scope.
/// 		Returns True if one or more controls are dirty, otherwise False.
/// 	</description>
/// 	<param name="scope" type="string" default="undefined">
/// 		If specified, dirty check controls only within this scope.
/// 		See Fit.Controls.ControlBase.Scope(..)
/// 		for details on configurering scoped validation.
/// 	</param>
/// </function>
Fit.Controls.DirtyCheckAll = function(scope)
{
	Fit.Validation.ExpectStringValue(scope, true);

	var result = false;

	Fit.Array.ForEach(Fit._internal.ControlBase.Controls, function(controlId)
	{
		var control = Fit._internal.ControlBase.Controls[controlId];

		if (Fit.Core.InstanceOf(control, Fit.Controls.ControlBase) === false)
			return;

		if (Fit.Validation.IsSet(scope) === true && control.Scope() !== scope)
			return;

		if (control.IsDirty() === true)
		{
			result = true;
			return false; // Break loop
		}
	});

	return result;
}

// <container name="Fit._internal.ControlBase">
// 	<member name="ReduceDocumentRootPollution" static="true" type="boolean">
// 		Reduce use of document root to host elements such as
// 		context menus, dialogs, widgets, etc., which are temporarily
// 		visible when interacting with controls on the screen.
// 	</member>
// </container>
Fit._internal.ControlBase.ReduceDocumentRootPollution = false;
/// <container name="Fit.Cookies">
/// 	Cookie functionality.
/// 	Set/Get/Remove functions can be invoked as static members, or an instance of Fit.Cookies
/// 	can be created to isolate cookies to either the current path or a custom path.
/// </container>

/// <container name="Fit.CookiesDefs.CookieIdentity">
/// 	<description> Cookie identity </description>
/// 	<member name="Name" type="string"> See Fit.Cookies.Set(..) function's description for its name argument </member>
/// 	<member name="Path" type="string" default="undefined"> See Fit.Cookies.Set(..) function's description for its path argument </member>
/// 	<member name="Domain" type="string" default="undefined"> See Fit.Cookies.Set(..) function's description for its domain argument </member>
/// </container>

/// <container name="Fit.CookiesDefs.Cookie" extends="Fit.CookiesDefs.CookieIdentity">
/// 	<description> New cookie </description>
/// 	<member name="Value" type="string"> See Fit.Cookies.Set(..) function's description for its value argument </member>
/// 	<member name="Seconds" type="integer" default="undefined"> See Fit.Cookies.Set(..) function's description for its seconds argument </member>
/// 	<member name="SameSite" type='"None" | "Lax" | "Strict"' default="undefined"> See Fit.Cookies.Set(..) function's description for its sameSite argument </member>
/// 	<member name="Secure" type="boolean" default="undefined"> See Fit.Cookies.Set(..) function's description for its secure argument </member>
/// </container>

/// <function container="Fit.Cookies" name="Cookies" access="public">
/// 	<description>
/// 		Create instance of cookie container isolated to either current path (default)
/// 		or a custom path, and optionally an alternative part of the domain (by default
/// 		cookies are available only on the current domain, while defining a domain makes
/// 		cookies available to that particular domain and subdomains).
/// 	</description>
/// </function>
Fit.Cookies = function()
{
	var me = this;
	var path = location.pathname.match(/^.*\//)[0]; // Examples: / OR /sub/ OR /sub/sub/sub/
	var domain = null;
	var sameSite = null;
	var secure = false;
	var prefix = null;

	function init()
	{
		// Remove trailing slash for path determined automatically,
		// to prevent double slashes when doing this: cookieContainer.Path() + "/sub"
		// Actually a trailing slash should be used for the path, but fortunately
		// Fit.Cookies.Set(..) makes sure to add it if missing.
		path = path.substring(0, path.length - 1);
	}

	/// <function container="Fit.Cookies" name="Path" access="public" returns="string">
	/// 	<description> Get/set path to which cookies are isolated </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes isolation to specified path </param>
	/// </function>
	this.Path = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			path = val;
		}

		return path;
	}

	/// <function container="Fit.Cookies" name="Domain" access="public" returns="string | null">
	/// 	<description> Get/set portion of domain to which cookies are isolated </description>
	/// 	<param name="val" type="string | null" default="undefined">
	/// 		If defined, changes isolation to specified domain portion, including subdomains - pass
	/// 		Null to unset it to make cookies available to current domain only (excluding subdomains).
	/// 	</param>
	/// </function>
	this.Domain = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (val !== undefined)
		{
			domain = val;
		}

		return domain;
	}

	/// <function container="Fit.Cookies" name="SameSite" access="public" returns="string | null">
	/// 	<description> Get/set SameSite policy </description>
	/// 	<param name="val" type='"None" | "Lax" | "Strict" | null' default="undefined">
	/// 		If defined, changes SameSite policy - pass Null to unset it
	/// 	</param>
	/// </function>
	this.SameSite = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (val === "None" || val === "Lax" || val === "Strict" || val === null)
		{
			sameSite = val;
		}

		return sameSite;
	}

	/// <function container="Fit.Cookies" name="Secure" access="public" returns="boolean">
	/// 	<description> Get/set Secure flag </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, changes Secure flag </param>
	/// </function>
	this.Secure = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			secure = val;
		}

		return secure;
	}

	/// <function container="Fit.Cookies" name="Prefix" access="public" returns="string | null">
	/// 	<description>
	/// 		Get/set prefix added to all cookies - useful for grouping related cookies and to avoid naming conflicts.
	/// 		Notice that Set/Get/Remove functions automatically apply the prefix to cookie names, so the use of a prefix
	/// 		is completely transparent.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes cookie prefix to specified value - pass Null to unset it </param>
	/// </function>
	this.Prefix = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (val !== undefined)
		{
			prefix = val;
		}

		return prefix;
	}

	/// <function container="Fit.Cookies" name="Set" access="public">
	/// 	<description> Create or update cookie </description>
	/// 	<param name="name" type="string"> Unique cookie name </param>
	/// 	<param name="value" type="string"> Cookie value (cannot contain semicolon!) </param>
	/// 	<param name="seconds" type="integer" default="undefined">
	/// 		Optional expiration time in seconds. Creating a cookie with
	/// 		no expiration time will cause it to expire when session ends.
	/// 	</param>
	/// </function>
	this.Set = function(name, value, seconds)
	{
		Fit.Validation.ExpectStringValue(name);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectInteger(seconds, true);

		Fit.Cookies.Set((prefix || "") + name, value, seconds, path || undefined, domain || undefined, sameSite || undefined, secure || undefined);
	}

	/// <function container="Fit.Cookies" name="Get" access="public" returns="string | null">
	/// 	<description> Returns cookie value if found, otherwise Null </description>
	/// 	<param name="name" type="string"> Unique cookie name </param>
	/// </function>
	this.Get = function(name)
	{
		Fit.Validation.ExpectStringValue(name);
		return Fit.Cookies.Get((prefix || "") + name);
	}

	/// <function container="Fit.Cookies" name="Remove" access="public">
	/// 	<description> Remove cookie </description>
	/// 	<param name="name" type="string"> Unique cookie name </param>
	/// </function>
	this.Remove = function(name)
	{
		Fit.Validation.ExpectStringValue(name);
		Fit.Cookies.Set((prefix || "") + name, "", -1, path || undefined, domain || undefined);
	}

	init();
}

/// <function container="Fit.Cookies" name="Set" access="public" static="true">
/// 	<description> Create or update cookie </description>
/// 	<param name="name" type="string"> Unique cookie name </param>
/// 	<param name="value" type="string"> Cookie value (cannot contain semicolon!) </param>
/// 	<param name="seconds" type="integer" default="undefined">
/// 		Optional expiration time in seconds. Creating a cookie with
/// 		no expiration time will cause it to expire when session ends.
/// 	</param>
/// 	<param name="path" type="string" default="undefined">
/// 		Optional cookie path.
/// 		Specifying no path makes cookie accessible to entire domain.
/// 		Specifying a path makes the cookie accessible to that particular path only.
/// 		However, this is not a security feature! A page can read cookies from any path
/// 		by creating an iframe with the path of the cookies, and read them through
/// 		the iframe's contentDocument.cookie property.
/// 	</param>
/// 	<param name="domain" type="string" default="undefined">
/// 		Optional cookie domain.
/// 		Not specifying a domain restricts the cookie to the host portion of the page currently loaded.
/// 		Specifying a domain makes the cookies accessible to the domain and subdomains.
/// 		It is not possible to specify a foreign domain name - this will be silently ignored.
/// 		Example: domain.com or sub.domain.com
/// 	</param>
/// 	<param name="sameSite" type='"None" | "Lax" | "Strict"' default="undefined">
/// 		Optional Same-Site policy determining whether to accept cookie and send it along with HTTP requests.
/// 		Different browsers (and versions) default to different values/behaviour, and a lot of different versions
/// 		are known to incorrectly handle SameSite, so don't expect this to work reliably across browsers.
/// 		Some browers may even work differently across platforms, despite being the same version, such as Safari on macOS and iOS.
/// 		Furthermore the behaviour may vary depending on browser configuration (configuration flags).
/// 		See https://www.chromium.org/updates/same-site/incompatible-clients
/// 		and https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie/SameSite#browser_compatibility
/// 		and https://caniuse.com/same-site-cookie-attribute
/// 		None = Cookie is included with cross-site requests, but only when Secure is True.
/// 		Lax = Cookie is included with all types of Same-Site requests, but only for top-level
/// 		navigation for cross-site requests (GET/HEAD requests only, and only in main window/frame).
/// 		Strict = Cookie is included with all types of Same-Site requests, never for cross-site requests.
/// 		More recent versions of Chrome (and Chrome based browsers) default to Lax.
/// 	</param>
/// 	<param name="secure" type="boolean" default="undefined">
/// 		Optional flag determining whether to accept and send along cookie on secure connections only.
/// 	</param>
/// </function>
/// <function container="Fit.Cookies" name="Set" access="public" static="true">
/// 	<description> Create or update cookie </description>
/// 	<param name="newCookie" type="Fit.CookiesDefs.Cookie"> New or updated cookie </param>
/// </function>
Fit.Cookies.Set = function()
{
	// Support for SameSite is an absolute mess with lots of bugs and varying browser/platform support
	// https://www.chromium.org/updates/same-site/incompatible-clients
	// https://bugs.webkit.org/show_bug.cgi?id=198181
	// https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/Set-Cookie/SameSite
	// More recent versions of Chrome (and Chrome based browsers) default to Lax,
	// while SameSite=None with Secure=true theoretically provides the best cross browser compatibility,
	// as it makes cookies behave as they did before the introduction of SameSite (on HTTPS) - had it not
	// been for the massive amount of bugs in various browsers, which unfortunately prevents
	// us from defaulting to this configuration.

	var name = undefined;
	var value = undefined;
	var seconds = undefined;
	var path = undefined;
	var domain = undefined;
	var sameSite = undefined;
	var secure = undefined;

	if (arguments.length === 1)
	{
		Fit.Validation.ExpectObject(arguments[0]);

		name = arguments[0].Name;
		value = arguments[0].Value;
		seconds = arguments[0].Seconds;
		path = arguments[0].Path;
		domain = arguments[0].Domain;
		sameSite = arguments[0].SameSite;
		secure = arguments[0].Secure;
	}
	else
	{
		name = arguments[0];
		value = arguments[1];
		seconds = arguments[2];
		path = arguments[3];
		domain = arguments[4];
		sameSite = arguments[5];
		secure = arguments[6];
	}

	Fit.Validation.ExpectStringValue(name);
	Fit.Validation.ExpectString(value);
	Fit.Validation.ExpectInteger(seconds, true);
	Fit.Validation.ExpectStringValue(path, true);
	Fit.Validation.ExpectStringValue(domain, true);
	Fit.Validation.ExpectStringValue(sameSite, true);
	Fit.Validation.ExpectBoolean(secure, true);

	if (value.indexOf(';') > -1)
		Fit.Validation.ThrowError("Unable to set cookie - value contains illegal character: ';'");

	value = value.replace(/\n/g, "\\n"); // Preserve line breaks which would otherwise break cookie value

	var newCookie = name + "=" + value;

	if (Fit.Validation.IsSet(seconds) === true)
	{
		var date = new Date();
		date.setTime(date.getTime() + (seconds * 1000));

		newCookie += "; expires=" + date.toGMTString();
	}

	if (Fit.Validation.IsSet(path) === true)
	{
		newCookie += "; path=" + path + ((path[path.length-1] !== "/") ? "/" : "");
	}
	else
	{
		newCookie += "; path=/";
	}

	if (Fit.Validation.IsSet(domain) === true)
	{
		newCookie += "; domain=" + domain;
	}

	if (sameSite === "None" || sameSite === "Lax" || sameSite === "Strict")
	{
		newCookie += "; samesite=" + sameSite.toLowerCase();
	}

	if (secure === true)
	{
		newCookie += "; secure";
	}

	document.cookie = newCookie;

	//document.cookie = name + "=" + value + ((date !== null) ? "; expires=" + date.toGMTString() : "") + "; path=" + ((Fit.Validation.IsSet(path) === true) ? path + ((path[path.length-1] !== "/") ? "/" : "") : "/");
}

/// <function container="Fit.Cookies" name="Get" access="public" static="true" returns="string | null">
/// 	<description> Returns cookie value if found, otherwise Null </description>
/// 	<param name="name" type="string"> Unique cookie name </param>
/// </function>
Fit.Cookies.Get = function(name)
{
	Fit.Validation.ExpectStringValue(name);

	var cookieName = name + "=";
	var cookies = document.cookie.split(";");
	var cookie = null;

	for (var i = 0 ; i < cookies.length ; i++)
	{
		cookie = cookies[i];

		while (cookie.charAt(0) === " ")
			cookie = cookie.substring(1, cookie.length);

		if (cookie.indexOf(cookieName) === 0)
			return cookie.substring(cookieName.length, cookie.length).replace(/\\n/g, "\n");
	}

	return null;
}

/// <function container="Fit.Cookies" name="Remove" access="public" static="true">
/// 	<description> Remove cookie </description>
/// 	<param name="name" type="string"> Unique cookie name </param>
/// 	<param name="path" type="string" default="undefined">
/// 		Optional cookie path.
/// 		If cookie was defined on a custom path, the
/// 		same path must be specified to remove the cookie.
/// 	</param>
/// 	<param name="domain" type="string" default="undefined">
/// 		Optional cookie domain.
/// 		If cookie was defined on a specific domain, the
/// 		same domain must be specified to remove the cookie.
/// 	</param>
/// </function>
/// <function container="Fit.Cookies" name="Remove" access="public" static="true">
/// 	<description> Remove cookie </description>
/// 	<param name="cookie" type="Fit.CookiesDefs.CookieIdentity"> Cookie to remove </param>
/// </function>
Fit.Cookies.Remove = function()
{
	var name = undefined;
	var path = undefined;
	var domain = undefined;

	if (arguments.length === 1 && typeof(arguments[0]) === "object")
	{
		name = arguments[0].Name;
		path = arguments[0].Path;
		domain = arguments[0].Domain;
	}
	else
	{
		name = arguments[0];
		path = arguments[1];
		domain = arguments[2];
	}

	Fit.Validation.ExpectStringValue(name);
	Fit.Validation.ExpectStringValue(path, true);
	Fit.Validation.ExpectStringValue(domain, true);

	Fit.Cookies.Set(name, "", -1, path, domain);
}

// =====================================
// Data
// =====================================

Fit.Data = {};

/// <function container="Fit.Data" name="CreateGuid" access="public" static="true" returns="string">
/// 	<description> Returns Version 4 compliant GUID </description>
/// 	<param name="dashFormat" type="boolean" default="true">
/// 		Flag indicating whether to use format with or without dashes.
/// 		True = With dashes (default): 57df75f2-3d09-48ca-9c94-f64a5986518f (length = 36)
/// 		False = Without dashes: 57df75f23d0948ca9c94f64a5986518f (length = 32)
/// 	</param>
/// </function>
Fit.Data.CreateGuid = function(dashFormat)
{
	Fit.Validation.ExpectBoolean(dashFormat, true);

	/*
	// Test case proving that unique GUIDs are generated every time.
	// Use a powerful computer to run this test, and a fast browser (e.g. Safari).

	var guids = {};
	var minors = 0;

	for (var i = 0 ; i < 25000000 ; i++) // 25 million
	{
		var g = Fit.Data.CreateGuid();

		minors++;
		if (minors === 5000)
		{
			console.log("5000 more GUIDs generated");
			minors = 0;
		}

		if (guids[g])
		{
			alert("GUID already in use: " + g);
			break;
		}
		else
		{
			guids[g] = 1;
		}
	}
	*/

	var chars = "0123456789abcdef".split("");

	var uuid = new Array();
	var rnd = Math.random;
	var r = -1;

	if (dashFormat !== false) uuid[8] = "-";
	if (dashFormat !== false) uuid[13] = "-";
	uuid[14] = "4"; // version 4 compliant
	if (dashFormat !== false) uuid[18] = "-";
	if (dashFormat !== false) uuid[23] = "-";

	var length = 32 + ((dashFormat !== false) ? 4 : 0);

	for (var i = 0 ; i < length ; i++)
	{
		if (uuid[i] !== undefined)
			continue;

		r = 0 | rnd() * 16;

		uuid[i] = chars[((i === 19) ? (r & 0x3) | 0x8 : r & 0xf)];
	}

	return uuid.join("");
}

/// <function container="Fit.Data" name="Base64ToBlob" access="public" static="true">
/// 	<description> Converts base64 value into a blob </description>
/// 	<param name="base64Data" type="string"> Base64 data to convert into a blob </param>
/// 	<param name="mimeType" type="string | null">
/// 		Mime type such as image/png. If Null is passed, then mime
/// 		type will be extracted from base64 value if possible. If mime type
/// 		cannot be determined, then application/octet-stream will be used.
/// 	</param>
/// 	<param name="async" type="boolean"> Perform conversion asynchronously on supported browsers </param>
/// 	<param name="resolve" type="(result: { Success: boolean, Blob: Blob | null, Error: string | null }) => void">
/// 		Function receiving blob once it is ready. Blob is null if an error occurred, in which case the Error property is set.
/// 	</param>
/// </function>
Fit.Core.Base64ToBlob = function(base64Data, mimeType, async, resolve)
{
	Fit.Validation.ExpectStringValue(base64Data);
	Fit.Validation.ExpectStringValue(mimeType || "unknown"); // Allow null to sniff mime type from base64 data URL
	Fit.Validation.ExpectBoolean(async);
	Fit.Validation.ExpectFunction(resolve);

	// Prepare data - handle base64 image URL format

	var base64 = base64Data;
	var contentType = mimeType || "";

	var dataUrlRegEx = /^data:(.+);base64,/; // 0 = Full match, 1 = mime type

	if (dataUrlRegEx.test(base64) === true)
	{
		contentType = contentType || base64.match(dataUrlRegEx)[1];
		base64 = base64.replace(dataUrlRegEx, ""); // Remove e.g. "data:image/png;base64," which leaves us with just the raw base64 data
	}

	contentType = contentType || "application/octet-stream";

	// Convert base64 to blob

	var createResult = function(success, blob, error)
	{
		return { Success: success, Blob: blob, Error: error };
	};

	if (async === true && window.fetch) // Chrome 42+, Safari 10.1+, Firefox 39+, MSEdge 14+
	{
		fetch("data:" + contentType + ";base64," + base64).then(function(response)
		{
			response.blob().then(function(blob)
			{
				resolve(createResult(true, blob, null));

			})["catch"](function(rejectReason) // Using ["catch"](..) rather than .catch(..) to avoid parse error in IE8
			{
				resolve(createResult(false, null, "Unable to read blob"));
			});

		})["catch"](function(rejectReason) // Using ["catch"](..) rather than .catch(..) to avoid parse error in IE8
		{
			resolve(createResult(false, null, "Unable to fetch blob, possibly due to invalid base64 value"));
		});
	}
	else if (window.atob) // Almost all browsers (not IE8 and IE9 though)
	{
		var byteCharacters = null;
		var byteArrays = [];
		var sliceSize = 512;

		try
		{
			byteCharacters = atob(base64);
		}
		catch (err)
		{
			resolve(createResult(false, null, "Invalid base64 value"));
		}

		for (var offset = 0; offset < byteCharacters.length; offset += sliceSize)
		{
			var slice = byteCharacters.slice(offset, offset + sliceSize);
			var byteNumbers = new Array(slice.length);

			for (var i = 0; i < slice.length; i++)
			{
				byteNumbers[i] = slice.charCodeAt(i);
			}

			var byteArray = new Uint8Array(byteNumbers);
			byteArrays.push(byteArray);
		}

		resolve(createResult(true, new Blob(byteArrays, { type: contentType }), null));
	}
	else // Unsupported browser
	{
		resolve(createResult(false, null, "Unsupported browser"));
	}
}


// =====================================
// Math
// =====================================

/// <container name="Fit.Math">
/// 	Math related functionality.
/// </container>
Fit.Math = {};

/// <container name="Fit.Math.MidpointRounding">
/// 	Enum values determining how a rounding mechanism processes a number that is midway between two numbers.
/// </container>
Fit.Math.MidpointRounding =
{
	/// <member container="Fit.Math.MidpointRounding" name="Up" access="public" static="true" type="string" default="Up">
	/// 	<description>
	/// 		When a number is midway between two others, the value is rounded up towards a positive value. Examples:
	/// 		1.4 becomes 1, 1.5 (midway) becomes 2, and 1.6 becomes 2.
	/// 		-1,4 becomes -1, -1,5 (midway) becomes -1, and -1.6 becomes -2.
	/// 		This is the default behaviour for JavaScript's Math.round(..) function.
	/// 	</description>
	/// </member>
	Up: "Up",

	/// <member container="Fit.Math.MidpointRounding" name="Down" access="public" static="true" type="string" default="Down">
	/// 	<description>
	/// 		When a number is midway between two others, the value is rounded down towards a negative value. Examples:
	/// 		1.4 becomes 1, 1.5 (midway) becomes 1, and 1.6 becomes 2.
	/// 		-1,4 becomes -1, -1,5 (midway) becomes -2, and -1.6 becomes -2.
	/// 		This is the reverse behaviour of Fit.Math.MidpointRounding.Up.
	/// 	</description>
	/// </member>
	Down: "Down",

	/// <member container="Fit.Math.MidpointRounding" name="AwayFromZero" access="public" static="true" type="string" default="AwayFromZero">
	/// 	<description>
	/// 		When a number is midway between two others, the value is rounded towards the nearest number away from zero. Examples:
	/// 		1.4 becomes 1, 1.5 (midway) becomes 2, and 1.6 becomes 2.
	/// 		-1,4 becomes -1, -1,5 (midway) becomes -2, and -1.6 becomes -2.
	/// 		This is the default behaviour of PHP's round(..) function and .NET's Math.Round(..) function.
	/// 	</description>
	/// </member>
	AwayFromZero: "AwayFromZero"
}

/// <function container="Fit.Math" name="Round" access="public" static="true" returns="number">
/// 	<description> Round off value to a number with the specified precision </description>
/// 	<param name="value" type="number"> Number to round off </param>
/// 	<param name="precision" type="integer"> Desired precision </param>
/// 	<param name="midpointRounding" type="Fit.Math.MidpointRounding" default="undefined">
/// 		Argument specifies how the function processes a number that is midway between
/// 		two numbers - defaults to Fit.Math.MidpointRounding.AwayFromZero if not specified.
/// 	</param>
/// </function>
Fit.Math.Round = function(value, precision, midpointRounding)
{
	Fit.Validation.ExpectNumber(value);
	Fit.Validation.ExpectInteger(precision, true);

	if (Fit.Validation.IsSet(midpointRounding) === true && Fit.Validation.IsSet(Fit.Math.MidpointRounding[midpointRounding]) === false)
		Fit.Validation.ThrowError("Unsupported MidpointRounding specified - use e.g. Fit.Math.MidpointRounding.Up");

	var decimals = ((Fit.Validation.IsSet(precision) === true) ? precision : 0);
	var mpr = (midpointRounding ? midpointRounding : Fit.Math.MidpointRounding.AwayFromZero);

    var factor = 1;
	for (var i = 0 ; i < decimals ; i++) factor = factor * 10;

	var res = null;

	if (mpr === Fit.Math.MidpointRounding.Up)
	{
		// Math.round(..) always round up numbers from midpoint (.5) towards a positive number
		//  2.5 = 3
		// -2.5 = -2

		res = Math.round(value * factor) / factor;
	}
	else if (mpr === Fit.Math.MidpointRounding.AwayFromZero)
	{
		// To round away from zero simply make sure rounding is always performed
		// on a positive number, and then turn it back into a negative number if
		// the value was initially negative.
		//  2.5 = 3
		// -2.5 = -3

		res = (value < 0 ? -1 : 1) * Math.round(Math.abs(value * factor)) / factor;
	}
	else //if (mpr === Fit.Math.MidpointRounding.Down)
	{
		// To round down simply reverse positive to negative and
		// negative to positive before rounding, and then back to its initial sign.
		//  2.5 = 2
		// -2.5 = -3

		res = -1 * Math.round(-1 * value * factor) / factor;
	}

	return res;
}

/// <function container="Fit.Math" name="Format" access="public" static="true" returns="string">
/// 	<description>
/// 		Format value to produce a number with the specified number of decimals.
/// 		Value is properly rounded off to ensure best precision.
/// 	</description>
/// 	<param name="value" type="number"> Number to format </param>
/// 	<param name="decimals" type="integer"> Desired number of decimals </param>
/// 	<param name="decimalSeparator" type="string" default="undefined">
/// 		If defined, the specified decimal separator will be used
/// 	</param>
/// </function>
Fit.Math.Format = function(value, decimals, decimalSeparator)
{
	Fit.Validation.ExpectNumber(value);
	Fit.Validation.ExpectInteger(decimals, true);
	Fit.Validation.ExpectString(decimalSeparator, true);

	var res = Fit.Math.Round(value, decimals);

    if (decimals <= 0)
        return res.toString();

    var str = ((res % 1 === 0) ? res.toString() + ".0" : res.toString());

    for (var i = str.split(".")[1].length ; i < decimals ; i++)
        str += "0";

    return ((Fit.Validation.IsSet(decimalSeparator) === true) ? str.replace(".", decimalSeparator) : str);
}

/// <function container="Fit.Math" name="Random" access="public" static="true" returns="integer">
/// 	<description> Get random integer value </description>
/// 	<param name="min" type="integer" default="undefined"> Minimum value - assumes 0 if not defined </param>
/// 	<param name="max" type="integer" default="undefined"> Maximum value - assumes Number.MAX_SAFE_INTEGER (9007199254740991) if not defined </param>
/// </function>
Fit.Math.Random = function(min, max)
{
	Fit.Validation.ExpectInteger(min, true);
	Fit.Validation.ExpectInteger(max, true);

	var mn = Fit.Validation.IsSet(min) === true ? min : 0;
	var mx = Fit.Validation.IsSet(max) === true ? max : 9007199254740991; // Number.MAX_SAFE_INTEGER not supported by IE

	return Math.floor(Math.random() * (mx - mn + 1)) + mn;
}


// =====================================
// String
// =====================================

Fit.String = {};

/// <function container="Fit.String" name="Trim" access="public" static="true" returns="string">
/// 	<description> Removes any whitespaces in beginning and end of string passed, and returns the new string </description>
/// 	<param name="str" type="string"> String to trim </param>
/// </function>
Fit.String.Trim = function(str)
{
	Fit.Validation.ExpectString(str);
	return str.replace(/^\s+|\s+$/g, "");
}

/// <function container="Fit.String" name="StripHtml" access="public" static="true" returns="string">
/// 	<description> Removes any HTML contained in string, and returns the raw text value </description>
/// 	<param name="str" type="string"> String to strip HTML from </param>
/// </function>
Fit.String.StripHtml = function(str)
{
	Fit.Validation.ExpectString(str);

	return str.replace(/<\/?[a-zA-Z][^>]*>/g, "");

	// Disabled - a value such as "A < B > C" will have "< B >" removed which is not desirable
	//return str.replace(/(<([^>]+)>)/g, "");

	// Disabled - whitespaces are not preserved!
	/*var span = document.createElement("span");
	span.innerHTML = str;
	return Fit.String.Trim(Fit.Dom.Text(span));*/
}

/// <function container="Fit.String" name="EncodeHtml" access="public" static="true" returns="string">
/// 	<description> Encode special characters into HTML entities </description>
/// 	<param name="str" type="string"> String to encode </param>
/// </function>
Fit.String.EncodeHtml = function(str)
{
	Fit.Validation.ExpectString(str);
	return str.replace(/&/g, "&amp;").replace(/"/g, "&quot;").replace(/'/g, "&#39;").replace(/</g, "&lt;").replace(/>/g, "&gt;")
}

/// <function container="Fit.String" name="DecodeHtml" access="public" static="true" returns="string">
/// 	<description> Decode special characters represented as HTML entities back into their actual characters </description>
/// 	<param name="str" type="string"> String to decode </param>
/// </function>
Fit.String.DecodeHtml = function(str)
{
	Fit.Validation.ExpectString(str);
	return str.replace(/&quot;/g, "\"").replace(/&#39;/g, "'").replace(/&#039;/g, "'").replace(/&lt;/g, "<").replace(/&gt;/g, ">").replace(/&amp;/g, "&");
}

/// <function container="Fit.String" name="ParseImageBlobUrls" access="public" static="true" returns="string[]">
/// 	<description> Parse and return image blob URLs from image tags </description>
/// 	<param name="str" type="string"> String to parse </param>
/// </function>
Fit.String.ParseImageBlobUrls = function(str)
{
	var imageBlobUrls = [];
	var blobImages = str.match(/<img .*?src=(["'])blob:.+?\1.*?>/gi) || [];

	Fit.Array.ForEach(blobImages, function(img)
	{
		var blobUrl = img.match(/src=(["'])(blob:.*?)\1/i)[2]; // 0 = Full match, 1 = double quote or ping character, 2 = blob URL

		if (Fit.Array.Contains(imageBlobUrls, blobUrl) === false)
		{
			Fit.Array.Add(imageBlobUrls, blobUrl);
		}
	});

	return imageBlobUrls;
}

/// <function container="Fit.String" name="Hash" access="public" static="true" returns="integer">
/// 	<description> Get Java compatible Hash Code from string </description>
/// 	<param name="str" type="string"> String to get hash code from </param>
/// </function>
Fit.String.Hash = function(str)
{
	Fit.Validation.ExpectString(str);

	if (str.length == 0) return 0;

	var hash = 0;
	var chr = '';

	for (var i = 0 ; i < str.length ; i++)
	{
		chr = str.charCodeAt(i);
		hash = ((hash << 5) - hash) + chr;
		hash = hash & hash;
	}

	return hash;
}

/// <function container="Fit.String" name="UpperCaseFirst" access="public" static="true" returns="string">
/// 	<description> Returns string with first letter upper cased </description>
/// 	<param name="str" type="string"> String to turn first letter into upper case </param>
/// </function>
Fit.String.UpperCaseFirst = function(str)
{
	Fit.Validation.ExpectString(str);

	if (str === "")
		return str;

	return str[0].toUpperCase() + str.slice(1);
}


// =====================================
// Date
// =====================================

Fit.Date = {};

/// <function container="Fit.Date" name="Format" access="public" static="true" returns="string">
/// 	<description>
/// 		Format instance of Date.
/// 		Example: Fit.Date.Format(new Date(), &quot;YYYY-MM-DD hh:mm:ss&quot;);
/// 		Result: 2016-06-18 15:23:48
///
/// 		Date can be formatted using the following variables:
/// 		YYYY: 4 digits year (e.g. 2016)
/// 		YY: 2 digits year (e.g. 16)
/// 		MM: 2 digits month (e.g. 04)
/// 		M: Prefer 1 digit month if possible (e.g. 4)
/// 		DD: 2 digits day (e.g. 09)
/// 		D: Prefer 1 digit day if possible (e.g. 9)
/// 		W: 1 digit week number (e.g. 35)
/// 		hh: 2 digits hours value (e.g. 08)
/// 		h: 1 digit hours value if possible (e.g. 8)
/// 		mm: 2 digits minutes value (e.g. 02)
/// 		m: 1 digit minutes value if possible (e.g. 2)
/// 		ss: 2 digits seconds value (e.g. 05)
/// 		s: 1 digit seconds value if possible (e.g. 5)
/// 	</description>
/// 	<param name="date" type="Date"> Date to format as string </param>
/// 	<param name="format" type="string"> Date format </param>
/// </function>
Fit.Date.Format = function(date, format)
{
	Fit.Validation.ExpectDate(date);
	Fit.Validation.ExpectString(format);

	format = format.replace(/YYYY/g, date.getFullYear().toString());
	format = format.replace(/YY/g, date.getFullYear().toString().substring(2));
	format = format.replace(/Y/g, parseInt(date.getFullYear().toString().substring(2), 10).toString());
	format = format.replace(/MM/g, ((date.getMonth() + 1 <= 9) ? "0" : "") + (date.getMonth() + 1));
	format = format.replace(/M/g, (date.getMonth() + 1).toString());
	format = format.replace(/DD/g, ((date.getDate() <= 9) ? "0" : "") + date.getDate());
	format = format.replace(/D/g, date.getDate().toString());
	format = format.replace(/W/g, Fit.Date.GetWeek(date).toString());
	format = format.replace(/hh/g, ((date.getHours() <= 9) ? "0" : "") + date.getHours());
	format = format.replace(/h/g, date.getHours().toString());
	format = format.replace(/mm/g, ((date.getMinutes() <= 9) ? "0" : "") + date.getMinutes());
	format = format.replace(/m/g, date.getMinutes().toString());
	format = format.replace(/ss/g, ((date.getSeconds() <= 9) ? "0" : "") + date.getSeconds());
	format = format.replace(/s/g, date.getSeconds().toString());

	return format;
}

/// <function container="Fit.Date" name="Parse" access="public" static="true" returns="Date">
/// 	<description>
/// 		Parse date as string into an instance of Date - example: 18-09/2016 17:03:21
/// 	</description>
/// 	<param name="strDate" type="string"> Date as string </param>
/// 	<param name="format" type="string">
/// 		Specify date format used to allow parser to determine which parts of the string
/// 		is Year, Month, Day, etc. The same variables used for Fit.Date.Format
/// 		can be used, except for W (week).
/// 		Since the parser do not need to know the length of the different parts that
/// 		makes up the Date, one can simply use the shorter variable format: Y, M, D, h, m, s.
/// 		Be aware that the Parse function does not support a Year represented by a
/// 		2 digit value, since it will be impossible to determine which century it belongs to.
/// 		Example: D-M/Y h:m:s
/// 	</param>
/// </function>
Fit.Date.Parse = function(strDate, format)
{
	Fit.Validation.ExpectString(strDate);
	Fit.Validation.ExpectString(format);

	// Format validation

	var regEx = format; // Example: D-M/Y h:m:s
	regEx = regEx.replace(/Y{1,4}/, "\\d{4}");
	regEx = regEx.replace(/M{1,2}/, "\\d{1,2}");
	regEx = regEx.replace(/D{1,2}/, "\\d{1,2}");
	regEx = regEx.replace(/h{1,2}/, "\\d{1,2}");
	regEx = regEx.replace(/m{1,2}/, "\\d{1,2}");
	regEx = regEx.replace(/s{1,2}/, "\\d{1,2}");
	regEx = regEx.replace(/\//g, "\\/"); // Allow use of slash by escaping it (/ => \/)
	regEx = "^" + regEx + "$";
	var regExp = new RegExp(regEx);

	if (regExp.test(strDate) === false)
		throw "InvalidDateFormat - Value '" + strDate + "' does not match format '" + format + "'";

	// Extract numbers from string date

	var regex = /\d+/g;
	var match = null;
	var matches = [];

	while ((match = regex.exec(strDate)) !== null)
	{
		Fit.Array.Add(matches, match[0]);
	}

	// Construct array with supported date parts

	var date = new Date(); //new Date(0); // Jan 01 1970 01:00:00

	var parts =
	[
		{ Name: "Year", Key: "Y", Index: -1, Value: date.getFullYear() },
		{ Name: "Month", Key: "M", Index: -1, Value: date.getMonth() + 1 },
		{ Name: "Day", Key: "D", Index: -1, Value: date.getDate() },
		{ Name: "Hours", Key: "h", Index: -1, Value: date.getHours() },
		{ Name: "Minutes", Key: "m", Index: -1, Value: date.getMinutes() },
		{ Name: "Seconds", Key: "s", Index: -1, Value: date.getSeconds() }
	];

	// Create getter that allows us to retrieve date part value by its Key

	parts.getVal = function(key/*, asString*/)
	{
		var res = null;

		Fit.Array.ForEach(this, function(part)
		{
			if (key === part.Key)
			{
				res = part.Value;
				return false;
			}
		});

		return res;
	}

	// Determine where various date parts are found in date string

	Fit.Array.ForEach(parts, function(part)
	{
		part.Index = format.indexOf(part.Key);
	});

	// Sort date parts accordingly to order in date string

	parts.sort(function(a, b)
	{
		return ((a.Index !== -1) ? a.Index : 999999) - ((b.Index !== -1) ? b.Index : 999999);
	});

	// Update date part values previously parsed out from date string using RegEx

	var idx = -1;

	Fit.Array.ForEach(parts, function(part)
	{
		if (part.Index !== -1)
		{
			idx++;

			if (idx > matches.length - 1)
				throw "InvalidDateFormat - " + part.Name + " not found in value";

			part.Value = parseInt(matches[idx], 10); // Radix (10) set to prevent some implementations of ECMAScript (e.g. on IE8) to intepret the value as octal (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/parseInt)

			if (part.Key === "Y" && part.Value < 1000) // In case value was specified as e.g. 0001 or 0111. The Date(year, month, day, ...) implementation requires the year to be at least 3 digits which is odd, so let's restrict it to 4 which is more intuitive. More on the topic here: https://stackoverflow.com/questions/29968116/javascript-date-01-01-0001.
				throw "InvalidDataFormat - value '" + part.Value + "' is not valid for '" + part.Name + "'";
			if (part.Key === "M" && (part.Value < 0 || part.Value > 12))
				throw "InvalidDataFormat - value '" + part.Value + "' is not valid for '" + part.Name + "'";
			if (part.Key === "D" && (part.Value < 0 || part.Value > 31))
				throw "InvalidDataFormat - value '" + part.Value + "' is not valid for '" + part.Name + "'";
			if (part.Key === "h" && (part.Value < 0 || part.Value > 23))
				throw "InvalidDataFormat - value '" + part.Value + "' is not valid for '" + part.Name + "'";
			if ((part.Key === "m" || part.Key === "s") && (part.Value < 0 || part.Value > 59))
				throw "InvalidDataFormat - value '" + part.Value + "' is not valid for '" + part.Name + "'";

			if (part.Key === "M")
				part.Value = part.Value - 1; // Month is zero-based
		}
	});

	var dt = new Date(parts.getVal("Y"), parts.getVal("M"), parts.getVal("D", true), 0, 0, 0);

	dt.setHours(parseInt(parts.getVal("h"), 10));
	dt.setMinutes(parseInt(parts.getVal("m"), 10));
	dt.setSeconds(parseInt(parts.getVal("s"), 10));

	return dt;
}

/// <function container="Fit.Date" name="GetWeek" access="public" static="true" returns="integer">
/// 	<description> Get ISO 8601 week number from date </description>
/// 	<param name="date" type="Date"> Date to get week number from </param>
/// </function>
Fit.Date.GetWeek = function(date) // ISO 8601 - use MomentJS for wider support!
{
	Fit.Validation.ExpectDate(date);

	// https://en.wikipedia.org/wiki/Date_and_time_representation_by_country
	// https://en.wikipedia.org/wiki/Week#Week_numbering
	// http://stackoverflow.com/questions/6117814/get-week-of-year-in-javascript-like-in-php/6117889#6117889 (ISO 8601)

	var d = new Date(date.getTime()); // Copy date - do not modify original

	d.setHours(0, 0, 0); // Set hours, minutes, and seconds
	d.setDate(d.getDate() + 4 - (d.getDay() || 7)); // Set to nearest Thursday (make Sunday day no. 7 instead of 0)

	var yearStart = new Date(d.getFullYear(), 0, 1);
	var week = Math.ceil((((d.getTime() - yearStart.getTime()) / (24 * 60 * 60 * 1000)) + 1) / 7);

	return week;

	// http://javascript.about.com/library/blweekyear.htm
	//var janFirst = new Date(date.getFullYear(), 0, 1);
	//return Math.ceil((((date - janFirst) / 86400000) + janFirst.getDay() + ((useIso8601 !== true) ? 1 : 0) ) / 7);
}


// =====================================
// Color
// =====================================

Fit.Color = {};

/// <function container="Fit.Color" name="RgbToHex" access="public" static="true" returns="string">
/// 	<description> Convert RGB colors into HEX color string </description>
/// 	<param name="r" type="integer"> Color index for red </param>
/// 	<param name="g" type="integer"> Color index for green </param>
/// 	<param name="b" type="integer"> Color index for blue </param>
/// </function>
Fit.Color.RgbToHex = function(r, g, b)
{
	Fit.Validation.ExpectNumber(r);
	Fit.Validation.ExpectNumber(g);
	Fit.Validation.ExpectNumber(b);

	if (r < 0 || r > 255 || g < 0 || g > 255 || b < 0 || b > 255)
		throw "RGB parameters must have a value between 0 and 255"; //return null;

    var rHex = r.toString(16);
    var gHex = g.toString(16);
    var bHex = b.toString(16);

    return ("#" + ((rHex.length === 1) ? "0" : "") + rHex + ((gHex.length === 1) ? "0" : "") + gHex + ((bHex.length === 1) ? "0" : "") + bHex).toUpperCase();
}

/// <function container="Fit.Color" name="HexToRgb" access="public" static="true" returns="string">
/// 	<description> Convert HEX color string into RGB color string </description>
/// 	<param name="hex" type="string"> HEX color string, e.g. #C0C0C0 (hash symbol is optional) </param>
/// </function>
Fit.Color.HexToRgb = function(hex)
{
	Fit.Validation.ExpectString(hex);

	var rgb = Fit.Color.ParseHex(hex);

	// if (rgb === null)
	// 	return null;

	return "rgb(" + rgb.Red + ", " + rgb.Green + ", " + rgb.Blue + ")";
}

/// <container name="Fit.ColorTypeDefs.RgbColor">
/// 	<description> RGB color object </description>
/// 	<member name="Red" type="integer"> </member>
/// 	<member name="Green" type="integer"> </member>
/// 	<member name="Blue" type="integer"> </member>
/// </container>

/// <container name="Fit.ColorTypeDefs.RgbaColor" extends="Fit.ColorTypeDefs.RgbColor">
/// 	<description> RGBA color object </description>
/// 	<member name="Alpha" type="integer"> Alpha channel (opacity) </member>
/// </container>

/// <function container="Fit.Color" name="ParseHex" access="public" static="true" returns="Fit.ColorTypeDefs.RgbColor">
/// 	<description> Convert HEX color string into RGB color object, e.g. { Red: 150, Green: 30, Blue: 185 } </description>
/// 	<param name="hex" type="string"> HEX color string, e.g. #C0C0C0 (hash symbol is optional) </param>
/// </function>
Fit.Color.ParseHex = function(hex)
{
	Fit.Validation.ExpectString(hex);

	var result = hex.match(/^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i);

	if (result === null)
		throw "HEX color is in an invalid format"; //return null;

	return { Red: parseInt(result[1], 16), Green: parseInt(result[2], 16), Blue: parseInt(result[3], 16) };
}

/// <function container="Fit.Color" name="ParseRgb" access="public" static="true" returns="Fit.ColorTypeDefs.RgbaColor">
/// 	<description>
/// 		Parses RGB(A) string and turns result into a RGB(A) color object, e.g. { Red: 100, Green: 100, Blue: 100, Alpha: 0.3 }.
/// 	</description>
/// 	<param name="val" type="string"> RGB(A) color string, e.g. rgba(100, 100, 100, 0.3) or simply 100,100,200,0.3 </param>
/// </function>
Fit.Color.ParseRgb = function(val)
{
	Fit.Validation.ExpectString(val);

	// Parse colors from rgb[a](r, g, b[, a]) string
	var result = val.match(/(\d+)\s*,\s*(\d+)\s*,\s*(\d+)(\s*,\s*(\d*.*\d+))*/); // http://regex101.com/r/rZ7rO2/9

	if (result === null)
		throw "RGB(A) color is in an invalid format"; //return null;

	var c = {};
	c.Red = parseInt(result[1], 10);
	c.Green = parseInt(result[2], 10);
	c.Blue = parseInt(result[3], 10);
	c.Alpha = ((result[5] !== undefined) ? parseFloat(result[5]) : 1.00);

	return c;
}

/// <function container="Fit.Color" name="IsHex" access="public" static="true" returns="boolean">
/// 	<description> Returns True if value is a valid HEX color value, otherwise False </description>
/// 	<param name="val" type="string"> Value to validate </param>
/// </function>
Fit.Color.IsHex = function(val)
{
	Fit.Validation.ExpectString(val);

	try
	{
		Fit.Color.ParseHex(val); // Throws exception if value is in an invalid format
	}
	catch (err)
	{
		return false;
	}

	return true;
}

/// <function container="Fit.Color" name="IsRgb" access="public" static="true" returns="boolean">
/// 	<description> Returns True if value is a valid RGB(A) color value, otherwise False </description>
/// 	<param name="val" type="string"> Value to validate </param>
/// </function>
Fit.Color.IsRgb = function(val)
{
	Fit.Validation.ExpectString(val);

	try
	{
		Fit.Color.ParseRgb(val); // Throws exception if value is in an invalid format
	}
	catch (err)
	{
		return false;
	}

	return true;
}

/// <container name="Fit.Device">
/// 	Provides access to device information.
/// </container>
Fit.Device =
{
	// Assuming ordinary desktop device by default - touch support is determined further down

	/// <member container="Fit.Device" name="HasMouse" static="true" type="boolean"> Flag indicating whether device has a mouse (or another type of precision pointer) attached </member>
	HasMouse: true,

	/// <member container="Fit.Device" name="HasTouch" static="true" type="boolean"> Flag indicating whether device has touch support </member>
	HasTouch: false,

	/// <member container="Fit.Device" name="OptimizeForTouch" static="true" type="boolean"> Flag indicating whether user experience should be optimized for touch interaction </member>
	OptimizeForTouch: false
};

(function()
{
	// Inspired by https://github.com/rafgraph/detect-it

	// Browser support:
	// matchMedia(..)				Chrome 9+	IE 10+	Safari 5.1+		https://developer.mozilla.org/en-US/docs/Web/API/Window/matchMedia
	// maxTouchPoints				Chrome 35+	IE 11+	Safari 13+		https://developer.mozilla.org/en-US/docs/Web/API/Navigator/maxTouchPoints
	// Pointer media queries		Chrome 41+	Edge12	Safari 9+		https://developer.mozilla.org/en-US/docs/Web/CSS/@media/pointer

	var pointerQueriesSupported = window.matchMedia && (window.matchMedia("(any-pointer:coarse)").matches === true || window.matchMedia("(any-pointer:fine)").matches === true);

	// Older touch enabled browsers such as IE11 running on early hybrid devices like Surface tablets, will not have
	// OptimizeForTouch become true because IE11 does not support pointer media queries (e.g. pointer:fine). Furthermore such
	// devices detached from mouse and keyboard (tablet mode) will not have HasMouse become false, also due to the lack of pointer
	// media query support. Too bad - these devices are old and not very common - they will have to rely on the desktop experience.
	// If such devices are required to work with Fit.UI with a touch optimized experience, the UI should allow the user to configure
	// the desired experience, which must ensure that the Fit.Device flags are set appropriately after Fit.UI has loaded, but before
	// any widgets or controls are created.

	// Detect touch support using fairly new maxTouchPoints property (Safari 13+) if possible. If not, then try alternatives.
	// Some browsers on modern devices with support for both touch and mouse returns false for ("ontouchstart" in window),
	// since it historically has been used to detect "touch only" devices. Therefore we also check against ("TouchEvent" in window).
	// However, TouchEvent is present in Chrome, even on "mouse only" devices, so we also need to check against the precision of
	// pointer devices. If any of the available pointers (any-pointer) are coarse, then assume touch support.
	if ((navigator.maxTouchPoints || 0) > 0 || "ontouchstart" in window || ("TouchEvent" in window && window.matchMedia && matchMedia("(any-pointer:coarse)").matches === true))
	{
		Fit.Device.HasTouch = true;
	}

	// Detect lack of high precision pointer device (mouse).
	// If pointer queries are not supported, then assume a mouse is not present if touch capabilities are detected, which
	// will be the case on old touch devices (e.g. Safari v. 8 and below on iOS or Chrome v. 40 and below on Android).
	// IMPORTANT: any-pointer:fine will match on an iPad with a pen attached (and possibily other devices with pens
	// as well) since a pen is a fine precision pointer. In this case HasMouse will therefore become (remain) True!
	if ((pointerQueriesSupported === true && matchMedia("(any-pointer:fine)").matches === false) ||
		(pointerQueriesSupported === false && Fit.Device.HasTouch === true))
	{
		Fit.Device.HasMouse = false;
	}

	// Determine whether user experience should be optimized for touch
	if ((pointerQueriesSupported === true && matchMedia("(pointer:coarse)").matches === true) ||				// Optimize for touch if this is the primary pointing device
		(pointerQueriesSupported === false && Fit.Device.HasTouch === true && Fit.Device.HasMouse === false))	// Optimize for touch if this is supported and no mouse is detected
	{
		Fit.Device.OptimizeForTouch = true;
	}
})();

/// <container name="Fit.Dom">
/// 	DOM (Document Object Model) manipulation and helper functionality
/// </container>
Fit.Dom = {};
Fit._internal.Dom = {};

// ==========================================================
// CSS
// ==========================================================

/// <function container="Fit.Dom" name="AddClass" access="public" static="true">
/// 	<description> Add CSS class to element if not already found </description>
/// 	<param name="elm" type="DOMElement"> Element on which CSS class is to be added </param>
/// 	<param name="cls" type="string"> CSS class name </param>
/// </function>
Fit.Dom.AddClass = function(elm, cls)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(cls);

	if (Fit.Dom.HasClass(elm, cls) === false)
		elm.className += ((elm.className !== "") ? " " : "") + cls;
}

/// <function container="Fit.Dom" name="RemoveClass" access="public" static="true">
/// 	<description> Remove CSS class from element if found </description>
/// 	<param name="elm" type="DOMElement"> Element from which CSS class is to be removed </param>
/// 	<param name="cls" type="string"> CSS class name </param>
/// </function>
Fit.Dom.RemoveClass = function(elm, cls)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(cls);

	var arr = elm.className.split(" ");
	var newCls = "";

	Fit.Array.ForEach(arr, function(item)
	{
		if (item !== cls)
			newCls += ((newCls !== "") ? " " : "") + item;
	});

	elm.className = newCls;
}

/// <function container="Fit.Dom" name="HasClass" access="public" static="true" returns="boolean">
/// 	<description> Check whether given DOMElement has specified CSS class registered - returns True if found, otherwise False </description>
/// 	<param name="elm" type="DOMElement"> Element for which CSS class may be registered </param>
/// 	<param name="cls" type="string"> CSS class name </param>
/// </function>
Fit.Dom.HasClass = function(elm, cls)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(cls);

	var arr = elm.className.split(" ");
	var found = false;

	Fit.Array.ForEach(arr, function(item)
	{
		if (item === cls)
		{
			found = true;
			return false; // Stop loop
		}
	});

	return found;
}

/// <function container="Fit.Dom" name="GetComputedStyle" access="public" static="true" returns="string | null">
/// 	<description>
/// 		Get style value applied after stylesheets have been loaded.
/// 		An empty string or null may be returned if style has not been defined or does not exist.
/// 		Make sure not to use shorthand properties (e.g. border-color or padding) as some browsers are
/// 		not capable of calculating these - use the fully qualified property name (e.g. border-left-color
/// 		or padding-left).
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element which contains desired CSS style value </param>
/// 	<param name="style" type="string"> CSS style property name </param>
/// </function>
Fit.Dom.GetComputedStyle = function(elm, style)
{
	// Functionality has been moved to Fit.Browser where it was needed,
	// and we do not want Fit.Browser to depend on Fit.Dom, as Fit.Browser
	// is used quite a bit in Fit.Dom - we risk creating circular dependencies.
	return Fit.Browser.GetComputedStyle(elm, style);
}

/// <function container="Fit.Dom" name="GetInnerDimensions" access="public" static="true" returns="Fit.TypeDefs.Dimension">
/// 	<description>
/// 		Returns object with Width and Height properties (integers) with inner dimensions of specified
/// 		container. Inner dimensions are width and height with padding and borders substracted.
/// 		Result returned will be as expected no matter the box-sizing model being used.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get inner dimensions for </param>
/// </function>
Fit.Dom.GetInnerDimensions = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	if (elm === document.documentElement) // If document element (<html>) is passed
	{
		// Measuring document.documentElement or anything else outside of document.body makes no sense.
		// Content is added to document.body or children within, not within the document element. So the
		// result returned for the document element would not give us any idea as to how much space is
		// available in the document (body). Throw error to prevent programmers from creating unreliable apps.
		Fit.Validation.ThrowError("Unable to determine inner dimensions for document element (<html>). Maybe you wanted Fit.Dom.GetInnerDimensions(document.body) or Fit.Browser.GetViewPortDimensions().");
	}

	var width = elm.offsetWidth;
	var height = elm.offsetHeight;

	if (width !== 0) // Width is 0 if element is either not visible, or truly 0px
	{
		width -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "padding-left"));
		width -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "padding-right"));

		width -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "border-left-width"));
		width -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "border-right-width"));
	}

	if (height !== 0) // Height is 0 if element is either not visible, or truly 0px
	{
		height -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "padding-top"));
		height -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "padding-bottom"));

		height -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "border-top-width"));
		height -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "border-bottom-width"));
	}

	var res = { Width: Math.floor(width), Height: Math.floor(height) };

	// Backwards compatibility
	res.X = res.Width;
	res.Y = res.Height;

	return res;
}

Fit.Dom.GetInnerWidth = function(elm) // Backward compatibility
{
	Fit.Validation.ExpectDomElement(elm);
	return Fit.Dom.GetInnerDimensions(elm).X;
}


// ==========================================================
// DOM
// ==========================================================

/// <function container="Fit.Dom" name="IsRooted" access="public" static="true" returns="boolean">
/// 	<description> Returns True if element is rooted in document (appended to body), otherwise False </description>
/// 	<param name="elm" type="DOMNode"> Element to check </param>
/// </function>
Fit.Dom.IsRooted = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	var parent = elm.parentElement;
	while (parent !== null)
	{
		if (parent === document.body)
			return true;

		parent = parent.parentElement;
	}

	return false;
}

/// <function container="Fit.Dom" name="InsertBefore" access="public" static="true">
/// 	<description> Insert DOMNode before another DOMNode </description>
/// 	<param name="target" type="DOMNode"> Element to insert new element before </param>
/// 	<param name="newElm" type="DOMNode"> Element to insert before target element </param>
/// </function>
Fit.Dom.InsertBefore = function(target, newElm)
{
	Fit.Validation.ExpectNode(target);
	Fit.Validation.ExpectNode(newElm);

	if (target.parentElement === null)
		Fit.Validation.ThrowError("Unable to insert element - target is not rooted");

	target.parentElement.insertBefore(newElm, target);
}

/// <function container="Fit.Dom" name="InsertAfter" access="public" static="true">
/// 	<description> Insert DOMNode after another DOMNode </description>
/// 	<param name="target" type="DOMNode"> Element to insert new element after </param>
/// 	<param name="newElm" type="DOMNode"> Element to insert after target element </param>
/// </function>
Fit.Dom.InsertAfter = function(target, newElm)
{
	Fit.Validation.ExpectNode(target);
	Fit.Validation.ExpectNode(newElm);

	if (target.parentElement === null)
		Fit.Validation.ThrowError("Unable to insert element - target is not rooted");

	if (target.nextSibling)
		target.parentElement.insertBefore(newElm, target.nextSibling);
	else
		target.parentElement.appendChild(newElm);
}

/// <function container="Fit.Dom" name="InsertAt" access="public" static="true">
/// 	<description>
/// 		Insert DOMNode at given position.
/// 		Notice that position is relative to contained DOM Elements.
/// 		Text and Comment nodes are ignored.
/// 	</description>
/// 	<param name="container" type="DOMElement"> Container to insert element into </param>
/// 	<param name="position" type="integer"> Position (index) to insert element at </param>
/// 	<param name="newElm" type="DOMNode"> Element to insert </param>
/// </function>
Fit.Dom.InsertAt = function(container, position, newElm)
{
	Fit.Validation.ExpectDomElement(container);
	Fit.Validation.ExpectInteger(position);
	Fit.Validation.ExpectNode(newElm);

	// Notice: InsertAt does not allow insertion based on childNodes as it is unreliable on Legacy IE.

	if (container.children.length === 0 || container.children.length - 1 < position)
	{
		container.appendChild(newElm);
	}
	else
	{
		var before = container.children[position];

		if (before)
			Fit.Dom.InsertBefore(before, newElm);
	}
}

/// <function container="Fit.Dom" name="Replace" access="public" static="true">
/// 	<description> Replace element with another one </description>
/// 	<param name="oldElm" type="DOMNode"> Element to replace (Element, Text, or Comment) </param>
/// 	<param name="newElm" type="DOMNode"> Replacement element (Element, Text, or Comment) </param>
/// </function>
Fit.Dom.Replace = function(oldElm, newElm) // http://jsfiddle.net/Jemt/eu74o984/
{
	Fit.Validation.ExpectNode(oldElm);
	Fit.Validation.ExpectNode(newElm);

	if (oldElm.parentElement === null)
		Fit.Validation.ThrowError("Unable to replace element - not rooted");

	var container = oldElm.parentElement;
	container.replaceChild(newElm, oldElm);
}

/// <function container="Fit.Dom" name="Add" access="public" static="true">
/// 	<description> Add element to container </description>
/// 	<param name="container" type="DOMElement"> Add element to this container </param>
/// 	<param name="elm" type="DOMNode"> Element, Text, or Comment to add to container </param>
/// </function>
Fit.Dom.Add = function(container, elm)
{
	Fit.Validation.ExpectDomElement(container);
	Fit.Validation.ExpectNode(elm);

	container.appendChild(elm);
}

/// <function container="Fit.Dom" name="Remove" access="public" static="true">
/// 	<description> Remove DOMNode from its container element </description>
/// 	<param name="elm" type="DOMNode"> DOMNode to remove </param>
/// </function>
Fit.Dom.Remove = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	// Remove element if mounted

	if (elm.parentElement === undefined)
	{
		// Remove TextNode in IE
		// https://developer.mozilla.org/en-US/docs/Web/API/Node/parentElement
		// "On some browsers, the parentElement property is only defined on nodes that
		//  are themselves an Element. In particular, it is not defined on text nodes."

		if (elm.parentNode !== null) // Notice: might be null even though TextNode is added to a parent node, if entire document was cleared using document.body.innerHTML="";
		{
			elm.parentNode.removeChild(elm);
		}
	}
	else if (elm.parentElement !== null)
	{
		elm.parentElement.removeChild(elm);
	}
}

/// <function container="Fit.Dom" name="Attribute" access="public" static="true" returns="string | null">
/// 	<description> Get/set attribute on DOMElement - returns Null if attribute does not exist </description>
/// 	<param name="elm" type="DOMElement"> DOMElement to which attribute is set and/or returned from </param>
/// 	<param name="name" type="string"> Name of attribute to set or retrieve </param>
/// 	<param name="value" type="string | null" default="undefined">
/// 		If defined, attribute is updated with specified value.
/// 		Passing Null results in attribute being removed.
/// 	</param>
/// </function>
Fit.Dom.Attribute = function(elm, name, value)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(name);
	Fit.Validation.ExpectString(value, true);

	if (Fit.Validation.IsSet(value) === true)
		elm.setAttribute(name, value);
	else if (value === null)
		elm.removeAttribute(name);

	return elm.hasAttribute(name) === true ? elm.getAttribute(name) : null;
}

/// <function container="Fit.Dom" name="Data" access="public" static="true" returns="string | null">
/// 	<description> Get/set data attribute on DOMElement - returns Null if data attribute does not exist </description>
/// 	<param name="elm" type="DOMElement"> DOMElement to which data attribute is set and/or returned from </param>
/// 	<param name="name" type="string"> Name of data attribute to set or retrieve </param>
/// 	<param name="value" type="string | null" default="undefined">
/// 		If defined, data attribute is updated with specified value.
/// 		Passing Null results in data attribute being removed.
/// 	</param>
/// </function>
Fit.Dom.Data = function(elm, name, value)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectStringValue(name);
	Fit.Validation.ExpectString(value, true);

	// Modern browsers can read data-attributes from elm.dataset.ATTRIBUTE.
	// Notice that data-title-value="XYZ" becomes elm.dataset.titleValue.

	return Fit.Dom.Attribute(elm, "data-" + name, value);
}

/// <function container="Fit.Dom" name="CreateElement" access="public" static="true" returns="DOMNode">
/// 	<description>
/// 		Create element with the specified HTML content.
/// 		HTML content is (by default) wrapped in a <![CDATA[ &lt;div&gt; ]]> if it produced multiple elements.
/// 		If content on the other hand produces only one outer element, that particular element is returned.
/// 		It is possible to construct DOM objects of type Element, Text, and Comment.
/// 		The container type used to wrap multiple elements can be changed using the containerTagName argument.
/// 	</description>
/// 	<param name="html" type="string"> HTML element to create DOMElement from </param>
/// 	<param name="containerTagName" type="string" default="div">
/// 		If defined, and html argument produces multiple element, the result is wrapped in a container of
/// 		the specified type. If not set, multiple elements will be wrapped in a <![CDATA[ &lt;div&gt; ]]> container.
/// 	</param>
/// </function>
Fit.Dom.CreateElement = function(html, containerTagName)
{
	Fit.Validation.ExpectString(html);
	Fit.Validation.ExpectStringValue(containerTagName, true);

	if (Fit.String.Trim(html).toLowerCase().indexOf("<tr") === 0) // <tr> can only be rooted in a <table> container
	{
		html = Fit.String.Trim(html);

		var container = document.createElement("div"); // Using <div> rather than <tbody> because tbody.innerHTML is read-only in Legacy IE
		container.innerHTML = "<table><tbody>" + html + "</tbody></table>";

		if (container.firstChild.firstChild.children.length === 1)
			return container.firstChild.firstChild.firstChild;

		return container.firstChild;
	}
	else if (Fit.String.Trim(html).toLowerCase().indexOf("<td") === 0) // <td> can only be rooted in a <tr> container
	{
		html = Fit.String.Trim(html); // Make sure container is not returned if e.g. "  <td>test</td>" is passed

		var container = document.createElement("div"); // Using <div> rather than <tr> because tr.innerHTML is read-only in Legacy IE
		container.innerHTML = "<table><tbody><tr>" + html + "</tr></tbody></table>";

		if (container.firstChild.firstChild.firstChild.children.length === 1)
			return container.firstChild.firstChild.firstChild.firstChild;

		return container.firstChild.firstChild.firstChild;
	}
	else
	{
		var container = document.createElement(((Fit.Validation.IsSet(containerTagName) === true) ? containerTagName : "div"));
		container.innerHTML = html;

		// Using childNodes property instead of children property to include text nodes,
		// which the DOM functions in Fit.UI usually do not take into account.
		// If text nodes are not included, a call like the following would exclude
		// the " world" portion. We do not want to throw away data, naturally! Example:
		// Fit.Dom.CreateElement("<span>Hello</span> world"); // Returns <div><span>Hello</span> world</div>
		if (container.childNodes.length === 1)
			return container.firstChild;

		return container;
	}
}

/// <function container="Fit.Dom" name="Text" access="public" static="true" returns="string">
/// 	<description> Get/set inner text of DOMElement </description>
/// 	<param name="elm" type="DOMNode"> Node to update and/or get text value from </param>
/// 	<param name="value" type="string" default="undefined"> If defined, inner text is updated with specified value </param>
/// </function>
Fit.Dom.Text = function(elm, value)
{
	Fit.Validation.ExpectNode(elm);
	Fit.Validation.ExpectString(value, true);

	if (Fit.Validation.IsSet(value) === true)
	{
		if (elm.textContent !== undefined) // Use textContent if supported
		{
			elm.textContent = value;
		}
		else if (elm.innerText !== undefined) // Set content with innerText which does not interpret HTML
		{
			// IE8 does not support textContent.
			// https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent
			// "Altering innerText in Internet Explorer (version 11 and below) removes child nodes from the element
			// and permanently destroys all descendant text nodes. It is impossible to insert the nodes again into
			// any other element or into the same element anymore."
			// We therefore remove all nodes prior to changing the text value in Internet Explorer.
			if (elm.children.length > 0)
			{
				Fit.Array.ForEach(Fit.Array.Copy(elm.children), function(c)
				{
					Fit.Dom.Remove(c);
				});
			}

			elm.innerText = value;
		}
		else if (elm.nodeValue !== undefined) // Text or Comment nodes do not expose innerText, nor does it expose textContent on legacy browsers
		{
			elm.nodeValue = value;
		}
	}

	// NOTICE: The properties textContent (requires IE9+) and innerText return very different values.
	// https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent
	// Code from script and style blocks are included with textContent, and while innerText
	// does not, it also doesn't include text from elements hidden using CSS.
	// But fortunately hidden elements are included in IE8, probably due to incorrect implementation.
	// Since pretty much all browsers except for IE8 supports textContent,
	// we allow the inconsistency; that innerText does not include code from script and style blocks.
	// If we at some point realize that such behaviour is completely unacceptable, we can get a result
	// very similar to textContent with IE8 using the code below:
	// Fit.String.StripHtml(el.innerHTML).replace(/&nbsp;/g, " ");
	// The only major difference is that line breaks and indentation is not identical.
	// If we need even better consistency, we would need to make all browsers use the following code instead:
	// Fit.String.StripHtml(el.innerHTML).replace(/^\s*([^\s].*?)\s*$/gm, "$1").replace(/\r/g, "").replace(/\n/g, " ").replace(/&nbsp;/g, " ");
	// It takes innerHTML, removes all HTML, trims every line, remove line breaks between lines, and make sure non-breaking spaces are preserved.
	// Both approaches come with a performance penelty, and frankly it's not worth it when IE9+ and all other browsers support textContent.

	return (elm.textContent ? elm.textContent : elm.innerText);
}

/// <function container="Fit.Dom" name="GetIndex" access="public" static="true" returns="integer">
/// 	<description>
/// 		Get element position within parent element.
/// 		Notice that Text and Comment nodes are ignored.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get index for </param>
/// </function>
Fit.Dom.GetIndex = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	if (!elm.parentElement)
		return -1;

	var parent = elm.parentElement;

	for (var i = 0 ; i < parent.children.length ; i++)
		if (parent.children[i] === elm)
			return i;

	return -1; // Should not happen
}

/// <function container="Fit.Dom" name="GetDepth" access="public" static="true" returns="integer">
/// 	<description>
/// 		Get number of levels specified element is nested in DOM.
/// 		HTMLElement is at level 0, HTMLBodyElement is at level 1,
/// 		first element in HTMLBodyElement is at level 2, and so forth.
/// 	</description>
/// 	<param name="elm" type="DOMNode"> Element to get depth in DOM for </param>
/// </function>
Fit.Dom.GetDepth = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	var i = 0;
	var parent = elm.parentElement;

	while (parent)
	{
		i++;
		parent = parent.parentElement;
	}

	return i;
}

/// <function container="Fit.Dom" name="Contained" access="public" static="true" returns="boolean">
/// 	<description> Check whether given element is found in given container at any given level in object hierarchy </description>
/// 	<param name="container" type="DOMElement"> Container expected to contain element </param>
/// 	<param name="elm" type="DOMNode"> Element expected to be found in container's object hierarchy </param>
/// </function>
Fit.Dom.Contained = function(container, elm)
{
	Fit.Validation.ExpectDomElement(container);
	Fit.Validation.ExpectNode(elm);

	var parent = elm.parentElement;

	while (parent)
	{
		if (parent === container)
			return true;

		parent = parent.parentElement;
	}

	return false;
}

/// <function container="Fit.Dom" name="IsVisible" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Check whether given element is visible. Returns True if element has been rooted
/// 		in DOM and is visible. Returns False if not rooted, or display:none has been set
/// 		on element or any of its ancestors.
/// 	</description>
/// 	<param name="elm" type="DOMNode"> Element to check visibility for </param>
/// </function>
Fit.Dom.IsVisible = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	if (elm.nodeType === 8)
		return false; // Comments are not visual, hence not visible

	if (elm.nodeType === 3)
	{
		// Check parent element if a Text element is passed

		if (elm.parentElement === null)
			return false; // Not rooted

		elm = elm.parentElement;
	}

	// Determine visibility quickly using offsetParent if possible.
	// Notice that offsetParent is always Null for an element with
	// position:fixed, in which case this check will not suffice.
	// Also <html> and <body> have no offsetParent (null).
	if (Fit._internal.Dom.IsOffsetParentSupported() === true && Fit.Dom.GetComputedStyle(elm, "position") !== "fixed" && elm !== document.body && elm !== document.body.parentElement)
	{
		return (elm.offsetParent !== null);
	}
	/*if (Fit._internal.Dom.IsOffsetParentSupported() === true)
	{
		if (elm.offsetParent !== null)
		{
			return true;
		}
	}*/

	// Traverse DOM bottom-up to determine whether element or any ancestors have display:none set

	var element = elm;
	var previous = null;

	while (element !== null)
	{
		if (Fit.Dom.GetComputedStyle(element, "display") === "none")
			return false; // Element is not visible

		previous = element;
		element = element.parentElement;
	}

	return (previous === document.documentElement); // If last parent reached is not <html>, then element is not rooted in DOM yet
}

/// <function container="Fit.Dom" name="GetConcealer" access="public" static="true" returns="DOMElement | null">
/// 	<description>
/// 		Get container responsible for hiding given element.
/// 		Element passed will be returned if hidden itself.
/// 		Returns Null if element is visible, or has not been rooted in DOM yet.
/// 	</description>
/// 	<param name="elm" type="DOMNode"> Element to get concealer for </param>
/// </function>
Fit.Dom.GetConcealer = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	if (elm.nodeType === 8)
		return elm; // Comments are not visual, hence not visible

	if (Fit.Dom.IsVisible(elm) === true)
		return null; // Element is not concealed - it is visible and rooted in DOM

	if (elm.nodeType === 3)
		elm = elm.parentElement; // Check parent element if a Text element is passed

	// Element is hidden or not rooted in DOM.
	// Traverse DOM bottom-up to find container hiding element.

	var element = elm;
	while (element !== null)
	{
		if (Fit.Dom.GetComputedStyle(element, "display") === "none")
			return element;

		element = element.parentElement;
	}

	return null; // Not rooted in DOM yet
}

/// <function container="Fit.Dom" name="GetFocused" access="public" static="true" returns="DOMElement | null">
/// 	<description>
/// 		Returns element currently focused. If no element is focused, the document body is returned.
/// 		Null will be returned if the document has not been loaded yet.
/// 	</description>
/// </function>
Fit.Dom.GetFocused = function()
{
	var focused = null;

	try // Legacy IE is buggy (https://developer.mozilla.org/en-US/docs/Web/API/Document/activeElement)
	{
		focused = document.activeElement;

		if (!focused.nodeType) // IE11 returns an empty object when running in an iFrame (http://fiddle.jshell.net/Jemt/dL9q6b2d/6/embedded/result,js)
			focused = document.body;
	}
	catch (err)
	{
		focused = document.body;
	}

	return focused;
}

/// <function container="Fit.Dom" name="GetParentOfType" access="public" static="true" returns="DOMElement | null">
/// 	<description> Returns first parent of specified type for a given element if found, otherwise Null </description>
/// 	<param name="element" type="DOMNode"> Element to find parent for </param>
/// 	<param name="parentType" type="string"> Tagname of parent element to look for </param>
/// </function>
Fit.Dom.GetParentOfType = function(element, parentType)
{
	Fit.Validation.ExpectNode(element);
	Fit.Validation.ExpectStringValue(parentType);

	var parent = element.parentElement;

	while (parent !== null)
	{
		if (parent.tagName.toLowerCase() === parentType.toLowerCase())
			return parent;

		parent = parent.parentElement;
	}

	return null;
}

/// <function container="Fit.Dom" name="Wrap" access="public" static="true">
/// 	<description> Wraps element in container element while preserving position in DOM if rooted </description>
/// 	<param name="elementToWrap" type="DOMNode"> Element to wrap </param>
/// 	<param name="container" type="DOMElement"> Container to wrap element within </param>
/// </function>
Fit.Dom.Wrap = function(elementToWrap, container)
{
	Fit.Validation.ExpectNode(elementToWrap);
	Fit.Validation.ExpectDomElement(container);

	var parent = elementToWrap.parentElement;
	var nextSibling = elementToWrap.nextSibling;

	container.appendChild(elementToWrap); // Causes elementToWrap to be removed from existing container

	if (parent === null)
		return; // Not rooted

	if (nextSibling === null)
		parent.appendChild(container);
	else
		parent.insertBefore(container, nextSibling);
}


// ==========================================================
// Position
// ==========================================================

// Notice: Browser vendors are changing the way coordinates
// and dimensions are reported. W3C previously define these as
// integers/longs, but browsers are moving to floats for smoother
// animation and scrolling, and for more accurate positioning.
// https://code.google.com/p/chromium/issues/detail?id=323935
// For consistency we use Math.round/floor to make sure integers are
// always returned on both modern and legacy browsers.

/// <function container="Fit.Dom" name="SetCaretPosition" access="public" static="true">
/// 	<description> Set caret position for input control </description>
/// 	<param name="input" type="DOMElement"> Input element </param>
/// 	<param name="pos" type="integer"> Integer value specifying caret position in input control </param>
/// </function>
Fit.Dom.SetCaretPosition = function(input, pos)
{
	Fit.Validation.ExpectDomElement(input);
	Fit.Validation.ExpectInteger(pos);

	// Notice: This will - unfortunately - not make input field scroll
	// its content if cursor is positioned "outside" of input's viewport.
	// In Chrome this can be solved by blurring and re-focusing control
	// after setting caret position - but that will not work across all browsers.

	if (input.setSelectionRange) // Modern browsers
	{
		input.focus();
		input.setSelectionRange(pos, pos);
	}
	else if (input.createTextRange) // IE8 and below
	{
		var range = input.createTextRange();
		range.collapse(true);
		range.moveEnd("character", pos);
		range.moveStart("character", pos);
		range.select();
	}
}

/// <function container="Fit.Dom" name="GetBoundingPosition" access="public" static="true" returns="Fit.TypeDefs.Position | null">
/// 	<description>
/// 		Get position for visible element within viewport.
/// 		Object returned contains an X and Y property
/// 		with the desired integer values (pixels).
/// 		Contrary to Fit.Dom.GetPosition(elm, true) which returns
/// 		the position to the margin edge, this function returns the
/// 		position of the element's border edge, and is the recommended
/// 		approach.
/// 		Null will be returned if element is not visible.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get position for </param>
/// </function>
Fit.Dom.GetBoundingPosition = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	if (Fit.Dom.IsVisible(elm) === false)
		return null;

	var bcr = elm.getBoundingClientRect();

	return { X: Math.round(bcr.x || bcr.left), Y: Math.round(bcr.y || bcr.top) }; // Several legacy browsers use top/left instead of x/y
}

/// <function container="Fit.Dom" name="GetPosition" access="public" static="true" returns="Fit.TypeDefs.Position | null">
/// 	<description>
/// 		Get position for visible element.
/// 		Object returned contains an X and Y property
/// 		with the desired integer values (pixels).
/// 		The position returned is where the margin edge
/// 		starts, if such is applied.
/// 		Null will be returned if element is not visible.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get position for </param>
/// 	<param name="relativeToViewport" type="boolean" default="false">
/// 		Set True to get element position relative to viewport rather than to document which may exceed the viewport.
/// 		Contrary to Fit.Dom.GetBoundingPosition(elm), the position returned is where the margin edge starts, if such is applied.
/// 	</param>
/// </function>
Fit.Dom.GetPosition = function(elm, relativeToViewport) { return Fit._internal.Dom.GetPosition(elm, relativeToViewport); }
Fit._internal.Dom.GetPosition = function(elm, relativeToViewport, internalKeepMargin)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectBoolean(relativeToViewport, true);
	Fit.Validation.ExpectBoolean(internalKeepMargin, true);

	if (Fit.Dom.IsVisible(elm) === false)
		return null;

	var pos = { X: 0, Y: 0 };

	// Get position relative to viewport
	var rect = elm.getBoundingClientRect(); // Might contain floating point values
	pos.X = rect.left;
	pos.Y = rect.top;

	// Substract margin as positioning is done where margin starts
	if (internalKeepMargin !== true)
	{
		pos.X -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "margin-left"));
		pos.Y -= Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(elm, "margin-top"));
	}

	if (relativeToViewport === true)
	{
		pos.X = Math.round(pos.X);
		pos.Y = Math.round(pos.Y);
		return pos;
	}

	// Add scroll to get position relative to document
	pos.X += window.pageXOffset || document.documentElement.scrollLeft || document.body.scrollLeft || 0;
	pos.Y += window.pageYOffset || document.documentElement.scrollTop || document.body.scrollTop || 0;

	pos.X = Math.round(pos.X);
	pos.Y = Math.round(pos.Y);

	return pos;
}

/// <function container="Fit.Dom" name="GetRelativePosition" access="public" static="true" returns="Fit.TypeDefs.Position | null">
/// 	<description>
/// 		Get element position relative to a positioned parent or ancestor (offsetParent).
/// 		Coordinates returned are relative to document if no positioned parent or ancestor is found.
/// 		For an element with position:fixed coordinates relative to the document is returned.
/// 		Object returned contains an X and Y property with the desired integer values (pixels).
/// 		Notice that Null is returned in case element is not rooted yet (added to DOM) or invisible.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get position for </param>
/// </function>
Fit.Dom.GetRelativePosition = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	if (Fit.Dom.GetComputedStyle(elm, "position") === "fixed")
	{
		return Fit.Dom.GetPosition(elm); // Page scroll and element margin has already been added and substracted
	}

	// Make sure offsetParent is available, which will not be the cause if element is not
	// rooted, hidden using display:none, or has position:fixed (the latter is handled above).
	if (Fit.Dom.IsVisible(elm) === false)
		return null;

	var pos = Fit.Dom.GetPosition(elm); // Page scroll and element margin has already been added and substracted

	// Get actual offsetParent.
	// For an element without a positioned parent, the offsetParent
	// is most often the body element, but for elements in e.g. a table,
	// the immediate offsetParent is the td element, and the td's offsetParent
	// is the table element. So we need to walk up the hierarchy until
	// we either find a positioned parent or reach the body element.
	// https://developer.mozilla.org/en-US/docs/Web/API/HTMLElement/offsetParent

	var offsetParent = elm.offsetParent;
	while (offsetParent !== null && Fit.Dom.GetComputedStyle(offsetParent, "position") === "static" /*&& offsetParent !== document.body && offsetParent !== document.documentElement*/)
	{
		offsetParent = offsetParent.offsetParent;
	}
	offsetParent = offsetParent || document.documentElement;

	if (offsetParent !== document.documentElement)
	{
		var offsetParentPos = Fit._internal.Dom.GetPosition(offsetParent, false, true);

		if ((Fit.Browser.GetInfo().Name === "Firefox" && offsetParent.tagName === "TABLE")
			|| (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version >= 10 && offsetParent.tagName === "TABLE")
			|| (Fit.Browser.GetInfo().Name === "Edge" && offsetParent.tagName === "TABLE"))
		{
			pos.X = pos.X - offsetParentPos.X;
			pos.Y = pos.Y - offsetParentPos.Y;
		}
		else
		{
			pos.X = pos.X - (offsetParentPos.X + Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(offsetParent, "border-left-width")));
			pos.Y = pos.Y - (offsetParentPos.Y + Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(offsetParent, "border-top-width")));
		}

		//pos.X = pos.X - (offsetParent.offsetLeft + Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(offsetParent, "border-left-width")));
		//pos.Y = pos.Y - (offsetParent.offsetTop + Fit._internal.Dom.GetPx(Fit.Dom.GetComputedStyle(offsetParent, "border-top-width")));
	}

	return pos;
}

/// <function container="Fit.Dom" name="GetScrollBars" access="public" static="true" returns="Fit.TypeDefs.ScrollBarsPresent">
/// 	<description>
/// 		Get information about scrollbars within a given DOM element.
/// 		Returns an object with Vertical and Horizontal properties, each containing
/// 		Enabled and Size properties, which can be used to determine whether scrolling is enabled,
/// 		and the size of the scrollbar. The size remains 0 when scrolling is not enabled.
/// 		To determine whether the browser's viewport has scrolling enabled, use Fit.Browser.GetScrollBars().
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get scrollbar information for </param>
/// </function>
Fit.Dom.GetScrollBars = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	if (elm === document.documentElement || elm === document.body)
	{
		// For <html> and <body> clientWidth behaves differently:
		// https://developer.mozilla.org/en-US/docs/Web/API/Element/clientWidth
		// Therefore, make sure Fit.Browser.GetScrollBars() is used to determine scroll for the viewport.
		//Fit.Validation.ThrowError("Unintended use of Fit.Dom.GetScrollBars(..) - please use Fit.Browser.GetScrollBars() to detect scrollbars for the viewport");

		// Scrollbars for <html> and <body> is equivalent to the viewport's scrollbars.
		// Therefore, we return scrollbar information for the viewport when <html> or <body> is passed.

		return Fit.Browser.GetScrollBars();
	}

	var res = { Vertical: { Enabled: false, Size: 0 }, Horizontal: { Enabled: false, Size: 0 } };

	// NOTICE:
	// It has been observed that on Chrome 86 on macOS High Sierra with scrollbars set to be always shown,
	// scrollWidth and scrollHeight sometimes are reported to have a value of +1 higher than
	// clientWidth and clientHeight, but no scrollbars are shown until the difference between
	// the scrollWidth/Height and clientWidth/Height is increased a bit more. Decreasing the value again
	// to just +1 does not remove the scrollbars again, so this works as expected. But even though
	// the scrollbars do not show with a difference of just 1px initially, the area is in fact scrollable
	// by the 1px difference. The missing scrollbars must be a bug in macOS or Chrome.

	if (elm.scrollWidth > elm.clientWidth) // Has horizontal scrollbar
	{
		res.Horizontal.Enabled = true;
		res.Horizontal.Size = Fit.Browser.GetScrollBarSize();
	}

	if (elm.scrollHeight > elm.clientHeight) // Has vertical scrollbar
	{
		res.Vertical.Enabled = true;
		res.Vertical.Size = Fit.Browser.GetScrollBarSize();
	}

	return res;
}

/// <function container="Fit.Dom" name="GetScrollPosition" access="public" static="true" returns="Fit.TypeDefs.Position">
/// 	<description>
/// 		Get number of pixels specified element's container(s)
/// 		have been scrolled. This gives us the total scroll value
/// 		for nested scrollable elements.
/// 		Object returned contains an X and Y property with the desired integer values (pixels).
/// 	</description>
/// 	<param name="elm" type="DOMNode"> Element to get scroll position for </param>
/// </function>
Fit.Dom.GetScrollPosition = function(elm)
{
	Fit.Validation.ExpectNode(elm);

	// Get number of pixels specified element's container(s)
	// have been scrolled. This gives us the total scroll value
	// for nested scrollable elements which in combination with
	// X,Y mouse coordinates can be used to determine the mouse
	// position in the document rather than in the viewport:
	// scrollX = mouseXviewport + GetScrollPosition(elm).X;
	// scrollY = mouseYviewport + GetScrollPosition(elm).Y;

	var pos = { X: 0, Y: 0 };

	if (elm.nodeType !== 1) // Text or Comment element which do not have scrollLeft and scrollTop properties
		elm = elm.parentElement;

	while (elm)
	{
		pos.X += elm.scrollLeft;
		pos.Y += elm.scrollTop;
		elm = elm.parentElement;
	}

	pos.X = Math.floor(pos.X);
	pos.Y = Math.floor(pos.Y);

	return pos;
}

/// <function container="Fit.Dom" name="GetScrollParent" access="public" static="true" returns="DOMElement | null">
/// 	<description>
/// 		Get element's scroll parent (parent that has overflow:auto or overflow:scroll,
/// 		and that affects the position of the element if scrolled). This may not necessarily
/// 		be the first parent with overflow set - it also depends on the element's offsetParent.
/// 		For a parent to be considered the scroll parent, it must be scrollable, and the element
/// 		passed to this function must be relatively positioned against this scroll parent, or
/// 		relatively positioned against an element within the scroll parent that is statically
/// 		positioned, or in turn relatively positioned against the scroll parent.
/// 		Returns null if element passed is placed on its own stacking context with position:fixed.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get scroll parent for </param>
/// </function>
Fit.Dom.GetScrollParent = function(elm)
{
	Fit.Validation.ExpectDomElement(elm);

	var pos = Fit.Dom.GetComputedStyle(elm, "position");

	if (pos === "fixed")
	{
		return null; // No scroll parent when element has its own stacking context with position:fixed
	}

	var offsetParent = elm.offsetParent;

	while ((elm = elm.parentElement))
	{
		var regEx = /scroll|auto/;
		var overflow = Fit.Dom.GetComputedStyle(elm, "overflow");
		var overflowX = Fit.Dom.GetComputedStyle(elm, "overflow-x");
		var overflowY = Fit.Dom.GetComputedStyle(elm, "overflow-y");

		if (regEx.test(overflow) === true || regEx.test(overflowX) === true || regEx.test(overflowY) === true)
		{
			// Parent found with overflow:scroll or overflow:auto.
			// However, it may not be this parent that affects the position of the element when scrolled.
			// If the element passed has position:absolute, and the current parent is not itself positioned
			// (which makes it the element's offsetParent), and the element's offsetParent is located outside
			// of the parent with overflow set, then we ignore this parent and continue the search.
			if (pos === "absolute" && elm !== offsetParent && Fit.Dom.Contained(elm, offsetParent) === false)
			{
				continue; // Skip parent if element is floating over it or outside of it using absolute positioning
			}

			return elm;
		}

		if (Fit.Dom.GetComputedStyle(elm, "position") === "fixed")
		{
			// Parent with position:fixed reached. It no longer matters if an element further up the hierarchy
			// is scrollable, since position:fixed creates a new stacking context independent from the document.
			return null;
		}
	}

	// Return document

	return Fit.Dom.GetScrollDocument();
}

/// <function container="Fit.Dom" name="GetOverflowingParent" access="public" static="true" returns="DOMElement | null">
/// 	<description>
/// 		Get parent that has overflow set to auto, scroll, or hidden (hidden is optional).
/// 		To get the parent that actually affects the position of the element when scrolled,
/// 		use GetScrollParent(..) instead, as it may not be the first parent with overflow set.
/// 		Which parent affects the position of an element when scrolled depends on which element
/// 		is the offsetParent - it might not be an element within the overflowing parent, or
/// 		the overflowing parent itself, which would be required to affect the element's position
/// 		when the overflowing parent is scrolled.
/// 		A scrollable container will only affect an element's position if it is statically positioned
/// 		within the scrollable parent, or if it is positioned relative to another statically positioned element
/// 		within the parent with overflow, or if the parent with overflow itself is positioned, which makes it the
/// 		offsetParent, hence causing the element to scroll along.
/// 		In most cases GetScrollParent(..) will be the correct function to use.
/// 		Exceptions to this is when we also want to include overflow:hidden, or if we have an
/// 		element with position:absolute that is positioned relative to the document or an element
/// 		outside of the overflowing/scrollable parent, and we still want a reference to the nearest
/// 		parent with overflow set.
/// 		Returns null if element passed is placed on its own stacking context with position:fixed.
/// 	</description>
/// 	<param name="elm" type="DOMElement"> Element to get overflowing parent for </param>
/// 	<param name="scrollableOnly" type="boolean" default="false">
/// 		Flag indicating whether to only consider parents with overflow:auto or overflow:scroll.
/// 		Parents with overflow:hidden will be ignored. As such, only a parent with the ability to
/// 		scroll overflowing content into view can be returned.
/// 	</param>
/// </function>
Fit.Dom.GetOverflowingParent = function(elm, scrollableOnly)
{
	Fit.Validation.ExpectDomElement(elm);

	// Code below is similar to the code for GetScrollParent(..), except it includes overflow:hidden

	var pos = Fit.Dom.GetComputedStyle(elm, "position");

	if (pos === "fixed")
	{
		return null; // No overflowing parent when element has its own stacking context with position:fixed
	}

	var regEx = scrollableOnly === true ? /scroll|auto/ : /scroll|auto|hidden/;
	var current = elm;

	while ((current = current.parentElement) !== null)
	{
		var overflow = Fit.Dom.GetComputedStyle(current, "overflow");
		var overflowX = Fit.Dom.GetComputedStyle(current, "overflow-x");
		var overflowY = Fit.Dom.GetComputedStyle(current, "overflow-y");

		if (regEx.test(overflow) === true || regEx.test(overflowX) === true || regEx.test(overflowY) === true)
		{
			return current;
		}

		if (Fit.Dom.GetComputedStyle(current, "position") === "fixed")
		{
			// Parent with position:fixed reached. It no longer matters if an element further up the hierarchy
			// has overflow, since position:fixed creates a new stacking context independent from the document.
			return null;
		}
	}

	return null;
}

/// <function container="Fit.Dom" name="GetScrollDocument" access="public" static="true" returns="DOMElement">
/// 	<description>
/// 		Get scrolling document element. This is the cross browser
/// 		equivalent of document.scrollingElement.
/// 	</description>
/// </function>
Fit.Dom.GetScrollDocument = function()
{
	// Functionality has been moved to Fit.Browser where it was needed,
	// and we do not want Fit.Browser to depend on Fit.Dom, as Fit.Browser
	// is used quite a bit in Fit.Dom - we risk creating circular dependencies.
	return Fit.Browser.GetScrollDocument();
}

// Internal members

Fit._internal.Dom.IsOffsetParentSupported = function() // Returns True if offsetParent can be used to determine whether an element is visible or not
{
	if (Fit._internal.Dom.OffsetParentSupported === undefined)
	{
		var parent = document.createElement("div");
		var child = document.createElement("div");

		parent.style.display = "none";

		Fit.Dom.Add(document.body, parent);
		Fit.Dom.Add(parent, child);

		Fit._internal.Dom.OffsetParentSupported = (child.offsetParent === null); // If supported, offsetParent should return Null (will return <body> on e.g. IE9-10)

		Fit.Dom.Remove(parent);
	}

	return Fit._internal.Dom.OffsetParentSupported;
}

Fit._internal.Dom.GetPx = function(val)
{
	val = (val ? parseInt(val) : 0);

	if (isNaN(val) === true) // E.g. if val was "auto"
		val = 0;

	return val;
}

Fit.DragDrop = {};

// Draggable

/// <function container="Fit.DragDrop.Draggable" name="Draggable" access="public">
/// 	<description> Constructor - create instance of Draggable class </description>
/// 	<param name="domElm" type="DOMElement"> Element to turn into draggable object </param>
/// 	<param name="domTriggerElm" type="DOMElement" default="undefined"> Element that triggers dragging (optional) </param>
/// </function>
Fit.DragDrop.Draggable = function(domElm, domTriggerElm)
{
	Fit.Validation.ExpectElementNode(domElm);
	Fit.Validation.ExpectElementNode(domTriggerElm, true);

	// Private properties

	var me = this;
	var elm = domElm;
	var posState = null; // { position: "", left: "", top: "" };
	var trgElm = (domTriggerElm ? domTriggerElm : null);
	var bringToFrontOnActivation = false;
	var returnFocus = false;

	var onDragStart = null;
	var onDragging = null;
	var onDragStop = null;

	var activationEventId = -1;
	var mouseDownEventId = -1;

	// Construct

	function init()
	{
		Fit._internal.Core.EnsureStyles();

		Fit.Dom.AddClass(elm, "FitDragDropDraggable");
		Fit.Dom.AddClass((trgElm !== null ? trgElm : elm), "FitDragDropDraggableHandle");

		// Bring to front on activation

		activationEventId = Fit.Events.AddHandler(elm, (Fit.Browser.IsTouchEnabled() === true ? "touchstart" : "mousedown"), function(e)
		{
			if (bringToFrontOnActivation === true)
			{
				me.BringToFront();
			}
		});

		// Mouse down

		var focusedBeforeDrag = null;

		mouseDownEventId = Fit.Events.AddHandler(((trgElm !== null) ? trgElm : elm), (Fit.Browser.IsTouchEnabled() === true ? "touchstart" : "mousedown"), function(e)
		{
			if (Fit.DragDrop.Draggable._internal.active !== null)
				return; // Skip - current element is a draggable parent to which event propagated

			var ev = Fit.Events.GetEvent(e);

			if (onDragStart)
			{
				if (onDragStart(elm) === false)
				{
					return;
				}
			}

			focusedBeforeDrag = returnFocus === true ? Fit.Dom.GetFocused() : null;

			Fit.Dom.AddClass(elm, "FitDragDropDragging");

			// Initial positioning (used by Reset())
			if (posState === null)
			{
				// Element's positioning is only read on first drag attempt,
				// and again after Reset() has been called (which sets posState null),
				// as we can imagine an implementation that lets the user drag
				// an element and temporarily release it outside of a DropZone, and
				// then move it again, before finally dropping it into an actual DropZone,
				// where the positioning set by Draggable is canceled via Reset(), and
				// thereby reverted to the initial positioning defined by the element.

				posState =
				{
					position: elm.style.position,
					left: elm.style.left,
					top: elm.style.top
				};
			}

			// Mouse position in viewport
			var pp = Fit.Events.GetPointerState().Coordinates.ViewPort;
			var mouseXviewport = pp.X;
			var mouseYviewport = pp.Y;

			// Create state information object for draggable currently being dragged
			var state =
			{
				Draggable: me,
				Positioning: "relative",
				Mouse: {Viewport: {X: -1, Y: -1}, Document: {X: -1, Y: -1}},
				Position: {Viewport: {X: -1, Y: -1}, Document: {X: -1, Y: -1}, Offset: {X: -1, Y: -1}},
				Events: { OnDragStart: onDragStart, OnDragging: onDragging, OnDragStop: onDragStop }/*,
				OnSelectStart : document.onselectstart*/
			};

			var positioning = Fit.Dom.GetComputedStyle(elm, "position");
			if (positioning === "absolute" || positioning === "fixed")
			{
				state.Positioning = positioning;
			}

			// Disable text selection for legacy browsers.
			// DISABLED: Now handled using Fit.Events.PreventDefault(ev).
			//document.onselectstart = function() { return false; }

			// Find mouse position in viewport
			state.Mouse.Viewport.X = mouseXviewport;
			state.Mouse.Viewport.Y = mouseYviewport;

			// Find mouse position in document (which may have been scrolled)
			//state.Mouse.Document.X = mouseXviewport + window.scrollX;
			//state.Mouse.Document.Y = mouseYviewport + window.scrollY;
			var scrollPos = Fit.Dom.GetScrollPosition(elm);
			state.Mouse.Document.X = mouseXviewport + scrollPos.X;
			state.Mouse.Document.Y = mouseYviewport + scrollPos.Y;

			// Find draggable position in viewport and document
			state.Position.Viewport = Fit.Dom.GetPosition(elm, true);
			state.Position.Document = Fit.Dom.GetPosition(elm);

			// Find X,Y position already set for draggable (by previous drag operation or using CSS positioning)
			var offsetX = Fit.Dom.GetComputedStyle(elm, "left");
			var offsetY = Fit.Dom.GetComputedStyle(elm, "top");
			state.Position.Offset.X = (offsetX.indexOf("px") > -1 ? parseInt(offsetX) : 0);
			state.Position.Offset.Y = (offsetY.indexOf("px") > -1 ? parseInt(offsetY) : 0);

			Fit.DragDrop.Draggable._internal.active = state;

			// If draggable is also a dropzone, move it to the end of internal dropzones collection
			// to make sure this dropzone is considered "on top" of other dropzones.
			// NOTICE: This only works reliable if draggable dropzones are placed on top of each
			// other by the user. If the dropzones are programmatically floated on top of each other,
			// the dropzones must be added (turned into dropzones) in the same order as they appear
			// visually.

			if (Fit.Dom.HasClass(elm, "FitDragDropDropZone") === true)
			{
				// Find draggable dropzone in dropzones collection
				var draggableDropZone = null;
				Fit.Array.ForEach(Fit.DragDrop.DropZone._internal.dropzones, function(dzState)
				{
					if (dzState.DropZone.GetElement() === elm)
					{
						draggableDropZone = dzState;
						return false;
					}
				});

				// Move dropzone to end of collection
				Fit.Array.Remove(Fit.DragDrop.DropZone._internal.dropzones, draggableDropZone);
				Fit.Array.Add(Fit.DragDrop.DropZone._internal.dropzones, draggableDropZone);
			}

			if (ev.type === "touchstart")
			{
				Fit.Events.PreventDefault(ev); // Stop page scrolling on iOS (required in both ontouchstart and ontouchmove handlers)
			}
		});

		// Mouse Up

		if (Fit.DragDrop.Draggable._internal.mouseUpRegistered === false)
		{
			Fit.DragDrop.Draggable._internal.mouseUpRegistered = true;

			Fit.Events.AddHandler(document, (Fit.Browser.IsTouchEnabled() === true ? "touchend" : "mouseup"), function(e)
			{
				if (Fit.DragDrop.Draggable._internal.active === null)
					return;

				var draggableState = Fit.DragDrop.Draggable._internal.active;
				var draggable = Fit.DragDrop.Draggable._internal.active.Draggable;

				var dropzoneState = Fit.DragDrop.DropZone._internal.active
				var dropzone = (dropzoneState !== null ? dropzoneState.DropZone : null);

				// Handle draggable

				Fit.Dom.RemoveClass(draggable.GetElement(), "FitDragDropDragging");
				Fit.DragDrop.Draggable._internal.active = null;

				//if (draggableState.Events.OnDragStop)
				//    draggableState.Events.OnDragStop(draggable);

				// Handle active dropzone

				if (dropzoneState !== null)
				{
					Fit.Dom.RemoveClass(dropzone.GetElement(), "FitDragDropDropZoneActive");
					Fit.DragDrop.DropZone._internal.active = null;

					if (dropzoneState.OnDrop)
						dropzoneState.OnDrop(dropzone, draggable);
				}

				// Restore focus

				if (focusedBeforeDrag !== null)
				{
					focusedBeforeDrag.focus();
					focusedBeforeDrag = null;
				}

				// Fire OnDragStop after OnDrop to make sure OnDragStop can act
				// depending on whether draggable was dropped on a dropzone or not.

				if (draggableState.Events.OnDragStop)
					draggableState.Events.OnDragStop(draggable);

				// Restore OnSelectStart event
				//document.onselectstart = draggableState.OnSelectStart;
			});
		}

		Fit.Browser.IsTouchEnabled() === true && Fit.Events.AddHandler(document, "touchcancel", function(e)
		{
			focusedBeforeDrag = null;
		});

		// Mouse move

		if (Fit.DragDrop.Draggable._internal.mouseMoveRegistered === false)
		{
			Fit.DragDrop.Draggable._internal.mouseMoveRegistered = true;

			Fit.Events.AddHandler(document, (Fit.Browser.IsTouchEnabled() === true ? "touchmove" : "mousemove"), { passive: false /* https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener */ }, function(e)
			{
				if (Fit.DragDrop.Draggable._internal.active === null)
					return;

				var ev = Fit.Events.GetEvent(e);

				// Handle draggable

				var state = Fit.DragDrop.Draggable._internal.active;
				var draggable = state.Draggable;
				var elm = draggable.GetElement();

				// Mouse position in viewport
				var pp = Fit.Events.GetPointerState().Coordinates.ViewPort;
				var mouseXviewport = pp.X;
				var mouseYviewport = pp.Y;

				// Prevent user from moving object out of viewport
				var vpDim = Fit.Browser.GetViewPortDimensions();
				mouseXviewport = mouseXviewport >= 0 ? mouseXviewport : 0;
				mouseXviewport = mouseXviewport <= vpDim.Width ? mouseXviewport : vpDim.Width;
				mouseYviewport = mouseYviewport >= 0 ? mouseYviewport : 0;
				mouseYviewport = mouseYviewport <= vpDim.Height ? mouseYviewport : vpDim.Height;

				// Positioning (fixed, absolute, or relative)
				var positioning = Fit.DragDrop.Draggable._internal.active.Positioning;

				// Mouse position in viewport (fixed) or document (absolute or relative).
				// Potential performance optimization: Only call GetScrollPosition if element
				// is contained in scrollable element. If not, use window.scrollX/Y instead!
				//var mouseXdocument = mouseXviewport + window.scrollX;
				//var mouseYdocument = mouseYviewport + window.scrollY;
				var scrollPos = (positioning !== "fixed" ? Fit.Dom.GetScrollPosition(elm) : { X: 0, Y: 0 });
				var mouseXdocument = mouseXviewport + scrollPos.X;
				var mouseYdocument = mouseYviewport + scrollPos.Y;

				if (positioning !== "relative")
				{
					// Mouse position within draggable
					var mouseFromLeft = state.Mouse.Viewport.X - state.Position.Viewport.X;
					var mouseFromTop = state.Mouse.Viewport.Y - state.Position.Viewport.Y;

					elm.style.position = positioning; // absolute or fixed
					elm.style.left = (mouseXdocument - mouseFromLeft) + "px";
					elm.style.top = (mouseYdocument - mouseFromTop) + "px";
				}
				else // relative
				{
					// Number of pixels mouse moved
					var mouseXmoved = mouseXdocument - state.Mouse.Document.X;
					var mouseYmoved = mouseYdocument - state.Mouse.Document.Y;

					// Element have been initially positioned or previously moved
					var elementLeft = state.Position.Offset.X;
					var elementTop = state.Position.Offset.Y;

					elm.style.position = "relative";
					elm.style.left = elementLeft + mouseXmoved + "px";
					elm.style.top = elementTop + mouseYmoved + "px";
				}

				if (state.Events.OnDragging)
					state.Events.OnDragging(elm);

				// Handle dropzones

				var dropzone = null;
				var dropzoneElement = null;
				var pos = null;
				var dropZoneX = -1;
				var dropZoneY = -1;

				// State objects
				var previouslyActiveDropZone = Fit.DragDrop.DropZone._internal.active;
				var dropzoneActive = null;

				// Find active dropzone.
				// NOTICE: If dropzones are floated on top of each other, make sure they are added to the
				// internal dropzones collection in order of appearance to make sure the correct dropzone
				// is turned active.
				Fit.Array.ForEach(Fit.DragDrop.DropZone._internal.dropzones, function(dzState)
				{
					dropzone = dzState.DropZone;
					dropzoneElement = dropzone.GetElement();

					if (Fit.Dom.IsVisible(dropzoneElement) === false)
						return; // Skip - dropzone not currently visible

					pos = Fit.Dom.GetPosition(dropzoneElement, true);
					dropZoneX = pos.X;
					dropZoneY = pos.Y;

					var dropzoneFound = (dropzoneActive === null);
					var dropzoneDeeperThanPreviouslyFound = (dropzoneActive === null || Fit.Dom.GetDepth(dropzoneElement) > Fit.Dom.GetDepth(dropzoneActive.DropZone.GetElement()));
					var dropzoneCurrentlyBeingDragged = (Fit.Dom.HasClass(dropzoneElement, "FitDragDropDragging") === true);
					var draggableHoveringDropZone = (mouseXviewport > dropZoneX && mouseXviewport < (dropZoneX + dropzoneElement.offsetWidth) && mouseYviewport > dropZoneY && mouseYviewport < (dropZoneY + dropzoneElement.offsetHeight));

					if ((dropzoneFound === false || dropzoneDeeperThanPreviouslyFound === true) && dropzoneCurrentlyBeingDragged === false && draggableHoveringDropZone === true)
					{
						dropzoneActive = dzState;
					}
				});

				// Leave previously active dropzone if no longer active
				if (previouslyActiveDropZone !== null && dropzoneActive !== previouslyActiveDropZone)
				{
					Fit.Dom.RemoveClass(previouslyActiveDropZone.DropZone.GetElement(), "FitDragDropDropZoneActive");
					Fit.DragDrop.DropZone._internal.active = null;

					if (previouslyActiveDropZone.OnLeave)
						previouslyActiveDropZone.OnLeave(previouslyActiveDropZone.DropZone);
				}

				// Mark dropzone active if not already done
				if (dropzoneActive !== null && dropzoneActive !== previouslyActiveDropZone)
				{
					Fit.Dom.AddClass(dropzoneActive.DropZone.GetElement(), "FitDragDropDropZoneActive");
					Fit.DragDrop.DropZone._internal.active = dropzoneActive;

					if (dropzoneActive.OnEnter)
						dropzoneActive.OnEnter(dropzoneActive.DropZone);
				}

				// Stop page scrolling on iOS (required in both ontouchstart and ontouchmove handlers).
				// Ontouchmove must be registered with { passive: false } for this to work in browsers
				// which defaults to passive:true. This also prevents text selection on older browsers
				// not supporting user-select:none.

				Fit.Events.PreventDefault(ev);
			});
		}
	}

	// Public

	/// <function container="Fit.DragDrop.Draggable" name="BringToFrontOnActivation" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether to bring draggable to front when activated </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, a value of True enables functionality, False disables it (default) </param>
	/// </function>
	this.BringToFrontOnActivation = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			bringToFrontOnActivation = val;

			// NOTICE: z-index is not reset when BringToFrontOnActivation is disabled.
			// It may have been enabled at some point, but we keep the z-index to make
			// sure the element remains below other elements that have BringToFrontOnActivation
			// enabled. Resetting the z-index style attribute will cause the z-index value of
			// 99999 from CSS to be applied when dragging the element, causing it to temporarily
			// be positioned above elements with BringToFrontOnActivation enabled while dragging it.
			// That becomes bad UX when the user stops dragging it and releases it, at which point
			// the temporary z-index value of 99999 from CSS is no longer in effect and z-index:1200
			// takes effect again, which causes the element to be hidden behind elements with
			// BringToFrontOnActivation enabled.
		}

		return bringToFrontOnActivation;
	}

	/// <function container="Fit.DragDrop.Draggable" name="ReturnFocus" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether focus is returned/restored after drag operation </description>
	/// 	<param name="val" type="boolean" default="undefined">
	/// 		A value of True causes draggable to return focus to previously
	/// 		focused element when drag operation is completed - defaults to False
	/// 	</param>
	/// </function>
	this.ReturnFocus = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			returnFocus = val;
		}

		return returnFocus;
	}

	/// <function container="Fit.DragDrop.Draggable" name="BringToFront" access="public">
	/// 	<description> Bring draggable to front </description>
	/// </function>
	this.BringToFront = function()
	{
		elm.style.zIndex = Fit.DragDrop.Draggable._internal.getNextZindex();
	}

	/// <function container="Fit.DragDrop.Draggable" name="Reset" access="public">
	/// 	<description> Reset draggable to initial position </description>
	/// </function>
	this.Reset = function()
	{
		if (posState === null)
			return;

		elm.style.position = posState.position;
		elm.style.left = posState.left;
		elm.style.top = posState.top;
		elm.style.zIndex = "";

		posState = null;
	}

	/// <function container="Fit.DragDrop.Draggable" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description> Get draggable DOM element </description>
	/// </function>
	this.GetDomElement = function()
	{
		return elm;
	}

	this.GetElement = this.GetDomElement; // Backward compatibility

	/// <function container="Fit.DragDrop.Draggable" name="Dispose" access="public">
	/// 	<description> Free resources and disable dragging support for DOM element </description>
	/// </function>
	this.Dispose = function()
	{
		// Dispose should not be called while dragging element!
		// To support this we would need to fire events, invoke Reset(),
		// remove FitDragDropDragging class, unset Fit.DragDrop.Draggable._internal.active etc.

		// For elements removed from DOM and memory, calling Dispose is not necessary to
		// ensure memory is freed. There are no global resources associated with the DOM element.

		Fit.Dom.RemoveClass(elm, "FitDragDropDraggable");
		Fit.Dom.RemoveClass((trgElm !== null ? trgElm : elm), "FitDragDropDraggableHandle");

		Fit.Events.RemoveHandler(elm, activationEventId);
		Fit.Events.RemoveHandler(((trgElm !== null) ? trgElm : elm), mouseDownEventId);

		if (Fit.DragDrop.Draggable._internal.active !== null && Fit.DragDrop.Draggable._internal.active.Draggable === me)
		{
			Fit.DragDrop.Draggable._internal.active = null;
		}

		me = elm = posState = trgElm = bringToFrontOnActivation = returnFocus = onDragStart = onDragging = onDragStop = activationEventId = mouseDownEventId = null;
	}

	// Event handling

	/// <function container="Fit.DragDrop.DraggableTypeDefs" name="DragEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="dom" type="DOMElement"> DOM element to which event is associated </param>
	/// </function>

	/// <function container="Fit.DragDrop.DraggableTypeDefs" name="DragStopEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="dom" type="DOMElement"> DOM element to which event is associated </param>
	/// 	<param name="draggable" type="Fit.DragDrop.Draggable"> Instance of Draggable to which event is associated </param>
	/// </function>

	/// <function container="Fit.DragDrop.Draggable" name="OnDragStart" access="public">
	/// 	<description> Add event handler which gets fired when dragging starts </description>
	/// 	<param name="cb" type="Fit.DragDrop.DraggableTypeDefs.DragEventHandler"> Callback (event handler) function - draggable DOM element is passed to function </param>
	/// </function>
	this.OnDragStart = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		onDragStart = cb;
	}

	/// <function container="Fit.DragDrop.Draggable" name="OnDragging" access="public">
	/// 	<description> Add event handler which constantly gets fired when dragging takes place </description>
	/// 	<param name="cb" type="Fit.DragDrop.DraggableTypeDefs.DragEventHandler"> Callback (event handler) function - draggable DOM element is passed to function </param>
	/// </function>
	this.OnDragging = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		onDragging = cb;
	}

	/// <function container="Fit.DragDrop.Draggable" name="OnDragStop" access="public">
	/// 	<description> Add event handler which gets fired when dragging stops </description>
	/// 	<param name="cb" type="Fit.DragDrop.DraggableTypeDefs.DragStopEventHandler"> Callback (event handler) function - instance of Draggable is passed to function </param>
	/// </function>
	this.OnDragStop = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		onDragStop = cb;
	}

	init();
}
Fit.DragDrop.Draggable._internal =
{
	/* Shared */
	_zIndex : 10000,
	getNextZindex: function()
	{
		Fit.DragDrop.Draggable._internal._zIndex++;
		return Fit.DragDrop.Draggable._internal._zIndex + (Fit.Dom.Data(document.documentElement, "fitui-companion") === "fluent-ui" ? 2000000 : 0);
	},
	mouseUpRegistered : false,
	mouseMoveRegistered : false,

	/* Draggable state object */
	active: null
}

/// <function container="Fit.DragDrop.Draggable" name="GetNextZindex" access="public" static="true" returns="integer">
/// 	<description> Get next z-index which will place an element above any draggable element </description>
/// </function>
Fit.DragDrop.Draggable.GetNextZindex = function()
{
	return Fit.DragDrop.Draggable._internal.getNextZindex();
}

// DropZone

/// <function container="Fit.DragDrop.DropZone" name="DropZone" access="public">
/// 	<description> Constructor - create instance of DropZone class </description>
/// 	<param name="domElm" type="DOMElement"> Element to turn into dropzone object </param>
/// </function>
Fit.DragDrop.DropZone = function(domElm)
{
	Fit.Validation.ExpectElementNode(domElm);

	var elm = domElm;

	var cfg =
	{
		DropZone: this,
		OnEnter: null,
		OnDrop: null,
		OnLeave: null
	};

	function init()
	{
		Fit._internal.Core.EnsureStyles();
		Fit.Dom.AddClass(elm, "FitDragDropDropZone");
		Fit.DragDrop.DropZone._internal.dropzones.push(cfg);
	}

	/// <function container="Fit.DragDrop.DropZone" name="Dispose" access="public">
	/// 	<description> Free resources and disable DropZone support for DOM element </description>
	/// </function>
	this.Dispose = function()
	{
		// Dispose should not be called while DropZone is active!
		// To support this we would need to fire OnLeave and unset
		// Fit.DragDrop.DropZone._internal.active. Draggable
		// does not support being disposed while being dragged either.

		Fit.Dom.RemoveClass(elm, "FitDragDropDropZone");
		Fit.Array.Remove(Fit.DragDrop.DropZone._internal.dropzones, cfg);
		elm = cfg = null;
	}

	/// <function container="Fit.DragDrop.DropZone" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description> Get dropzone DOM element </description>
	/// </function>
	this.GetDomElement = function()
	{
		return elm;
	}

	this.GetElement = this.GetDomElement; // Backward compatibility

	// Event handling

	/// <function container="Fit.DragDrop.DropZoneTypeDefs" name="EnterEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="dropZone" type="Fit.DragDrop.DropZone"> Instance of DropZone to which draggable can potentially be dropped now </param>
	/// </function>

	/// <function container="Fit.DragDrop.DropZoneTypeDefs" name="DropEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="dropZone" type="Fit.DragDrop.DropZone"> Instance of DropZone to which draggable is dropped </param>
	/// 	<param name="draggable" type="Fit.DragDrop.Draggable"> Instance of Draggable being dropped </param>
	/// </function>

	/// <function container="Fit.DragDrop.DropZoneTypeDefs" name="LeaveEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="dropZone" type="Fit.DragDrop.DropZone"> Instance of DropZone from which draggable is moved away </param>
	/// </function>

	/// <function container="Fit.DragDrop.DropZone" name="OnEnter" access="public">
	/// 	<description> Add event handler which gets fired when draggable enters dropzone, ready to be dropped </description>
	/// 	<param name="cb" type="Fit.DragDrop.DropZoneTypeDefs.EnterEventHandler"> Callback (event handler) function - instance of DropZone is passed to function </param>
	/// </function>
	this.OnEnter = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		cfg.OnEnter = cb;
	}

	/// <function container="Fit.DragDrop.DropZone" name="OnDrop" access="public">
	/// 	<description> Add event handler which gets fired when draggable is dropped on dropzone </description>
	/// 	<param name="cb" type="Fit.DragDrop.DropZoneTypeDefs.DropEventHandler"> Callback (event handler) function - instance of DropZone and Draggable is passed to function (in that order) </param>
	/// </function>
	this.OnDrop = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		cfg.OnDrop = cb;
	}

	/// <function container="Fit.DragDrop.DropZone" name="OnLeave" access="public">
	/// 	<description> Add event handler which gets fired when draggable leaves dropzone </description>
	/// 	<param name="cb" type="Fit.DragDrop.DropZoneTypeDefs.LeaveEventHandler"> Callback (event handler) function - instance of DropZone is passed to function </param>
	/// </function>
	this.OnLeave = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		cfg.OnLeave = cb;
	}

	init();
}
Fit.DragDrop.DropZone._internal =
{
	dropzones: [],
	active: null
}

/// <container name="Fit.EventManager">
/// </container>

/// <function container="Fit.EventManager" name="EventManager" access="public">
/// 	<description> Create instance of EventManager </description>
/// </function>
Fit.EventManager = function()
{
	var registrations = []; // { Dom: DOMElement, Event: string, Handler: function, Options?: { Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean } }[]


	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??
	// TODO: Create "proxies" for function such as PreventDefault, StopProgation, etc. so we don't need to use both Fit.EventManager and Fit.Events ??


	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"keydown" | "keyup" | "keypress"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackKeyboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"click" | "contextmenu" | "dblclick" | "mousedown" | "mouseenter" | "mouseleave" | "mousemove" | "mouseout" | "mouseover" | "mouseup" | "mousewheel"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackMouse"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"popstate"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackPopState"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"hashchange"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackHashChange"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"focus" | "focusin" | "focusout" | "blur"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackFocus"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"beforeunload"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackBeforeUnload"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"cut" | "copy" | "paste"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackClipboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"storage"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackStorage"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
	/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
	/// 	<param name="event" type='"#rooted"'> Event name </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
	/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
	/// 	<param name="event" type='"#mutation"'> Event name </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.MutationObserverCallback"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type="string"> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackGeneric"> JavaScript function to register </param>
	/// </function>
	//  ------------------------------------------------------------------------------------------
	//  Overloads with options argument
	//  ------------------------------------------------------------------------------------------
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"keydown" | "keyup" | "keypress"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackKeyboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"click" | "contextmenu" | "dblclick" | "mousedown" | "mouseenter" | "mouseleave" | "mousemove" | "mouseout" | "mouseover" | "mouseup" | "mousewheel"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackMouse"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"popstate"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackPopState"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"hashchange"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackHashChange"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"focus" | "focusin" | "focusout" | "blur"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackFocus"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"beforeunload"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackBeforeUnload"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"cut" | "copy" | "paste"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackClipboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type='"storage"'> Event name without the 'on' prefix </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackStorage"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
	/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
	/// 	<param name="event" type='"#rooted"'> Event name </param>
	/// 	<param name="options" type="{ Id?: string }">
	/// 		This argument is ignored for the specialized #rooted event.
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
	/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
	/// 	<param name="event" type='"#mutation"'> Event name </param>
	/// 	<param name="options" type="{ Id?: String, Deep?: boolean }">
	/// 		This argument is ignored for the specialized #rooted event.
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.EventManager" name="Add" access="public">
	/// 	<description> Registers event handler for specified event on given EventTarget </description>
	/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
	/// 	<param name="event" type="string"> Event name without 'on' prefix (e.g. 'load', 'mouseover', 'click' etc.) </param>
	/// 	<param name="options" type="{ Id?: string, Capture?: boolean, Once?: boolean, Passive?: boolean }">
	/// 		Set additional options - not guaranteed to be cross browser compatible!
	/// 	</param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackGeneric"> JavaScript function to register </param>
	/// </function>
	this.Add = function(elmArg, eventArg, eventFunctionArg)
	{
		var element = elmArg;
		var event = eventArg;
		var eventFunction = arguments[3] !== undefined ? arguments[3] : eventFunctionArg;
		var options = arguments.length === 4 ? arguments[2] : {};

		Fit.Validation.ExpectEventTarget(element);
		Fit.Validation.ExpectStringValue(event);
		Fit.Validation.ExpectFunction(eventFunction);
		Fit.Validation.ExpectObject(options);
		Fit.Validation.ExpectString(options && options.Id, true);
		Fit.Validation.ExpectBoolean(options && options.Capture, true);
		Fit.Validation.ExpectBoolean(options && options.Once, true);
		Fit.Validation.ExpectBoolean(options && options.Passive, true);
		Fit.Validation.ExpectBoolean(options && options.Deep, true);

		for (var prop in options || {})
		{
			if (Fit.Array.Contains(["Id", "Capture", "Once", "Passive", "Deep"], prop) === false)
			{
				throw "Unsupported options passed to Fit.EventManager.Add(..): " + prop;
			}
		}

		var reg =
		{
			Dom: element,
			Event: event,
			Handler: eventFunction,
			Options: options
		};

		if (reg.event === "#mutation")
		{
			Fit.Events.AddMutationObserver(reg.Dom, reg.Handler, reg.Options.Deep)
		}
		else
		{
			Fit.Events.AddHandler(reg.Dom, reg.Event, reg.Options, reg.Handler);
		}

		Fit.Array.Add(registrations, reg);
	}

	/// <function container="Fit.EventManager" name="Move" access="public">
	/// 	<description> Move event handler by event ID to another DOMElement </description>
	/// 	<param name="id" type="integer"> Event ID identifying handler to move </param>
	/// 	<param name="newElm" type="DOMElement"> DOM element to which event handler is moved </param>
	/// </function>
	this.Move = function(id, newElm)
	{
		Fit.Validation.ExpectString(id);
		Fit.Validation.ExpectEventTarget(newElm);

		Fit.Array.ForEach(registrations, function(reg)
		{
			if (reg.Options.Id === id)
			{
				if (reg.Event === "#mutation")
				{
					Fit.Events.RemoveMutationObserver(reg.Dom, reg.Handler, reg.Options.Deep);

					reg.Dom = newElm;
					Fit.Events.AddMutationObserver(reg.Dom, reg.Handler, reg.Options.Deep);
				}
				else
				{
					Fit.Events.RemoveHandler(reg.Dom, reg.Event, reg.Options, reg.Handler);

					reg.Dom = newElm;
					Fit.Events.AddHandler(reg.Dom, reg.Event, reg.Options, reg.Handler);
				}
			}
		});
	}

	/// <function container="Fit.EventManager" name="Remove" access="public">
	/// 	<description> Removes event handler by event ID </description>
	/// 	<param name="id" type="integer"> Event ID identifying handler to remove </param>
	/// </function>
	this.Remove = function(id)
	{
		Fit.Validation.ExpectString(id);

		var newRegistrations = [];

		Fit.Array.ForEach(registrations, function(reg)
		{
			if (reg.Options.Id === id)
			{
				if (reg.Event === "#mutation")
				{
					Fit.Events.RemoveMutationObserver(reg.Dom, reg.Handler, reg.Options.Deep);
				}
				else
				{
					Fit.Events.RemoveHandler(reg.Dom, reg.Event, reg.Options, reg.Handler);
				}
			}
			else
			{
				Fit.Array.Add(newRegistrations, reg);
			}
		});

		registrations = newRegistrations;
	}

	/// <function container="Fit.EventManager" name="RemoveAll" access="public">
	/// 	<description> Remove all event handlers registered </description>
	/// </function>
	this.RemoveAll = function()
	{
		Fit.Array.ForEach(registrations, function(reg)
		{
			if (reg.Event === "#mutation")
			{
				Fit.Events.RemoveMutationObserver(reg.Dom, reg.Handler, reg.Options.Deep);
			}
			else
			{
				Fit.Events.RemoveHandler(reg.Dom, reg.Event, reg.Options, reg.Handler);
			}
		});

		registrations = [];
	}

	/// <function container="Fit.EventManager" name="Move" access="public">
	/// 	<description> Invoke event handler by event ID </description>
	/// 	<param name="id" type="integer"> Event ID identifying handler to invoke </param>
	/// 	<param name="args" type="any[]" default="undefined"> Event handler arguments </param>
	/// 	<param name="thisArg" type="any" default="undefined"> The value 'this' resolves to within event handler </param>
	/// </function>
	this.Invoke = function(id, args, thisArg)
	{
		Fit.Validation.ExpectString(id);
		Fit.Validation.ExpectArray(args, true);

		Fit.Array.ForEach(registrations, function(reg)
		{
			if (reg.Options.Id === id)
			{
				reg.Handler.apply(thisArg || reg.Dom, args || [])
			}
		});
	}

	// this.dump = function()
	// {
	// 	console.log(registrations);
	// }
}
/// <container name="Fit.Events">
/// 	Event handler functionality
/// </container>
Fit.Events = {};

// See TypeScript's GlobalEventHandlersEventMap (lib.dom.d.ts) which reveals how named events map to event types.

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackKeyboard" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="KeyboardEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackMouse" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="MouseEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackPopState" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="PopStateEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackHashChange" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="HashChangeEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackFocus" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="FocusEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackBeforeUnload" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="BeforeUnloadEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackClipboard" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="ClipboardEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackStorage" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="StorageEvent"> Event argument </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackRooted" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="dom" type="DOMElement"> DOM element </param>
/// </function>

/// <function container="Fit.EventsTypeDefs" name="EventHandlerCallbackGeneric" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddHandler(..) </description>
/// 	<param name="e" type="Event"> Event argument </param>
/// </function>

/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"keydown" | "keyup" | "keypress"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackKeyboard"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"click" | "contextmenu" | "dblclick" | "mousedown" | "mouseenter" | "mouseleave" | "mousemove" | "mouseout" | "mouseover" | "mouseup" | "mousewheel"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackMouse"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"popstate"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackPopState"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"hashchange"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackHashChange"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"focus" | "focusin" | "focusout" | "blur"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackFocus"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"beforeunload"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackBeforeUnload"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"cut" | "copy" | "paste"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackClipboard"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"storage"'> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackStorage"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
/// 	<param name="event" type='"#rooted"'> Event name </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type="string"> Event name without the 'on' prefix </param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackGeneric"> JavaScript function to register </param>
/// </function>
//  ------------------------------------------------------------------------------------------
//  Overloads with useCapture or options argument
//  ------------------------------------------------------------------------------------------
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"keydown" | "keyup" | "keypress"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackKeyboard"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"click" | "contextmenu" | "dblclick" | "mousedown" | "mouseenter" | "mouseleave" | "mousemove" | "mouseout" | "mouseover" | "mouseup" | "mousewheel"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackMouse"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"popstate"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackPopState"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"hashchange"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackHashChange"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"focus" | "focusin" | "focusout" | "blur"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackFocus"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"beforeunload"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackBeforeUnload"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"cut" | "copy" | "paste"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackClipboard"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type='"storage"'> Event name without the 'on' prefix </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackStorage"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given DOMElement and returns Event ID </description>
/// 	<param name="element" type="DOMElement"> DOM element on to which event handler is registered </param>
/// 	<param name="event" type='"#rooted"'> Event name </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		This argument is ignored for the specialized #rooted event.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
/// </function>
/// <function container="Fit.Events" name="AddHandler" access="public" static="true" returns="integer">
/// 	<description> Registers handler for specified event on given EventTarget and returns Event ID </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) on to which event handler is registered </param>
/// 	<param name="event" type="string"> Event name without 'on' prefix (e.g. 'load', 'mouseover', 'click' etc.) </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Set True to capture event before it reaches target, False to catch event when it bubbles out from target.
/// 		Alternatively pass options object to control behaviour on a more fine-grained scale.
/// 		NOTICE: This feature will be ignored by Internet Explorer 8 and below.
/// 	</param>
/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackGeneric"> JavaScript function to register </param>
/// </function>
Fit.Events.AddHandler = function()
{
	var element = null;
	var event = null;
	var useCaptureOrOptions = false; // false = event bubbling (reverse of event capturing)
	var eventFunction = null;

	if (arguments.length === 3)
	{
		element = arguments[0];
		event = arguments[1];
		eventFunction = arguments[2];
	}
	else if (arguments.length === 4)
	{
		element = arguments[0];
		event = arguments[1];
		useCaptureOrOptions = arguments[2];
		eventFunction = arguments[3];
	}

	Fit.Validation.ExpectEventTarget(element);
	Fit.Validation.ExpectStringValue(event);
	Fit.Validation.ExpectBoolean(typeof(useCaptureOrOptions) !== "object" ? useCaptureOrOptions : true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Capture || true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Once || true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Passive || true);
	Fit.Validation.ExpectFunction(eventFunction);

	if (event.toLowerCase() === "#rooted") // Custom event
	{
		if (useCaptureOrOptions)
			Fit.Validation.ThrowError("Event capturing or event options not supported for Rooted event");

		Fit.Array.Add(Fit._internal.Events.OnRootedHandlers, { Element: element, Parent: element.parentElement, Event: event, Callback: eventFunction });
	}
	else if (element.addEventListener) // W3C
	{
		var options = false;

		if (useCaptureOrOptions === true)
		{
			options = true;
		}
		else if (useCaptureOrOptions)
		{
			options = {};

			if (useCaptureOrOptions.Capture !== undefined)
			{
				options.capture = useCaptureOrOptions.Capture;
			}
			if (useCaptureOrOptions.Once !== undefined)
			{
				options.once = useCaptureOrOptions.Once;
			}
			if (useCaptureOrOptions.Passive !== undefined)
			{
				options.passive = useCaptureOrOptions.Passive;
			}
		}

		element.addEventListener(event, eventFunction, options);
	}
	else if (element.attachEvent) // IE
	{
		if (event.toLowerCase() === "domcontentloaded" && Fit._internal.Events.Browser.Name === "MSIE" && Fit._internal.Events.Browser.Version === 8)
		{
			// DOMContentLoaded not supported on IE8.
			// Using OnReadyStateChange to achieve similar behaviour.

			element.attachEvent("onreadystatechange", function(e)
			{
				if (element.readyState === "complete")
				{
					eventFunction(e); // NOTICE: Event argument not identical to argument passed to modern browsers using the real DOMContentLoaded event!
				}
			});
		}
		else
		{
			element.attachEvent("on" + event, eventFunction);
		}
	}

	element._internal = element._internal || {};
	element._internal.Events = element._internal.Events || { Handlers: [] };

	var eventId = element._internal.Events.Handlers.length;

	Fit.Array.Add(element._internal.Events.Handlers, { Event: event, Handler: eventFunction, UseCaptureOrOptions: useCaptureOrOptions, Id: eventId });

	// Fire event function for onload event if document in window/iframe has already been loaded.
	// Notice that no event argument is passed to function since we don't have one.
	// BE AWARE that contentDocument is replaced when loading another document (URL) within a frame!
	// Do not register onload on iFrame.contentDocument prior to starting loading content - it will never fire.
	// Instead register OnLoad on contentDocument afterwards, or register OnLoad on the iFrame element. The latter
	// will always fire, every time the document is replaced (URL changed / navigation occur).
	// Also be aware that a frame without a src attribute will always be considered loaded (readyState === "complete")
	// because it already has a contentDocument ready to be used. Therefore we only immediately fire onload for a
	// frame that has readyState "complete" IF it also has a src attribute set. If no src attribute is set, we
	// assume it will be set later, in which case the onload event will fire normally. It simply doesn't make sense to
	// register an OnLoad handler for a frame that is dynamically populated (which has no src attribute) - it will always be ready ("complete").
	if (event.toLowerCase() === "load" && element.nodeType === 9 && element.readyState === "complete") // Element is a Document (window.document or iframe.contentDocument)
	{
		eventFunction();
	}
	else if (event.toLowerCase() === "load" && (typeof(element.contentDocument) === "object" && element.contentDocument !== null) && element.src && element.contentDocument.readyState === "complete") // Element is an iFrame
	{
		// MSIE 8 requires use of typeof(element.contentDocument) - accessing element.contentDocument without typeof results in an "unspecified error" if
		// the iFrame is not rooted in DOM. This has been fixed in MSIE 9 where contentDocument on the other hand remains Null until rooted in DOM.

		eventFunction();
	}
	else if (event.toLowerCase() === "load" && element === window && Fit._internal.Events.OnReadyFired === true) // Element is the current Window instance
	{
		eventFunction();
	}

	return eventId;
}

/// <function container="Fit.Events" name="RemoveHandler" access="public" static="true">
/// 	<description> Remove event handler given by Event ID returned from Fit.Events.AddHandler(..) </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) from which event handler is removed </param>
/// 	<param name="eventId" type="integer"> Event ID identifying handler to remove </param>
/// </function>
/// <function container="Fit.Events" name="RemoveHandler" access="public" static="true">
/// 	<description> Remove event handler for specified event on given EventTarget </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) from which event handler is removed </param>
/// 	<param name="event" type="string"> Event name without 'on' prefix (e.g. 'load', 'mouseover', 'click' etc.) </param>
/// 	<param name="eventFunction" type="function"> JavaScript function to remove </param>
/// </function>
/// <function container="Fit.Events" name="RemoveHandler" access="public" static="true">
/// 	<description> Remove event handler for specified event on given EventTarget </description>
/// 	<param name="element" type="EventTarget"> EventTarget (e.g. Window or DOMElement) from which event handler is removed </param>
/// 	<param name="event" type="string"> Event name without 'on' prefix (e.g. 'load', 'mouseover', 'click' etc.) </param>
/// 	<param name="useCaptureOrOptions" type="boolean | { Capture?: boolean, Once?: boolean, Passive?: boolean }">
/// 		Value indicating whether event handler was registered using event capturing (True) or event bubbling (False).
/// 		If event handler was registered with event options for more fine-grained control, these options are passed instead.
/// 	</param>
/// 	<param name="eventFunction" type="function"> JavaScript function to remove </param>
/// </function>
Fit.Events.RemoveHandler = function()
{
	var element = null;
	var event = null;
	var useCaptureOrOptions = false; // false = event bubbling (reverse of event capturing)
	var eventFunction = null;

	if (arguments.length === 2)
	{
		Fit.Validation.ExpectEventTarget(arguments[0]);
		Fit.Validation.ExpectInteger(arguments[1]);

		element = arguments[0];

		var handler = ((element._internal && element._internal.Events && element._internal.Events.Handlers) ? element._internal.Events.Handlers[arguments[1]] : undefined);

		if (handler === undefined)
			Fit.Validation.Throw("No event handler with ID '" + arguments[0] + "' exists for this element");

		if (handler === null)
			return; // Already removed

		event = handler.Event;
		eventFunction = handler.Handler;
		useCaptureOrOptions = handler.UseCaptureOrOptions;
	}
	else if (arguments.length === 3)
	{
		element = arguments[0];
		event = arguments[1];
		eventFunction = arguments[2];
	}
	else if (arguments.length === 4)
	{
		element = arguments[0];
		event = arguments[1];
		useCaptureOrOptions = arguments[2];
		eventFunction = arguments[3];
	}

	Fit.Validation.ExpectEventTarget(element);
	Fit.Validation.ExpectStringValue(event);
	Fit.Validation.ExpectBoolean(typeof(useCaptureOrOptions) !== "object" ? useCaptureOrOptions : true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Capture || true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Once || true);
	Fit.Validation.ExpectBoolean(useCaptureOrOptions && useCaptureOrOptions.Passive || true);
	Fit.Validation.ExpectFunction(eventFunction);

	if (event.toLowerCase() === "#rooted")
	{
		var handlers = Fit._internal.Events.OnRootedHandlers;

		Fit.Array.ForEach(handlers, function(handler)
		{
			if (element === handler.Element && event.toLowerCase() === handler.Event.toLowerCase() && eventFunction === handler.Callback && useCaptureOrOptions === false)
			{
				handler.Removed = true; // Allow OnRooted handler to remove element without causing a "Collection was modified" exception - properly removed in timer responsible for observing elements
				return false; // Break loop
			}
		});
	}
	else if (element.removeEventListener)
	{
		// Notice regarding useCaptureOrOptions:
		// Only the capture flag is used when removing event handlers.
		// This can be either a boolean or an object like this: { capture: true|false }.
		// Other flags are not considered - or at least should not have been. But unfortunately
		// some browser versions are known to be byggy in this regard.
		// https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/removeEventListener
		// "It's worth noting that some browser releases have been inconsistent on this, and
		//  unless you have specific reasons otherwise, it's probably wise to use the same values
		//  used for the call to addEventListener() when calling removeEventListener()".
		// Nothing in the documentation states that the same object reference must be used.
		// In fact it states "it's probably wise to use the same VALUES ..." in the quote above.
		// This has been verified using unit tests on various browsers (see Tests/Core/Events.js).

		var options = false;

		if (useCaptureOrOptions === true)
		{
			options = true;
		}
		else if (useCaptureOrOptions)
		{
			options = {};

			if (useCaptureOrOptions.Capture !== undefined)
			{
				options.capture = useCaptureOrOptions.Capture;
			}
			if (useCaptureOrOptions.Once !== undefined)
			{
				options.once = useCaptureOrOptions.Once;
			}
			if (useCaptureOrOptions.Passive !== undefined)
			{
				options.passive = useCaptureOrOptions.Passive;
			}
		}

		element.removeEventListener(event, eventFunction, options);
	}
	else if (element.detachEvent)
	{
		element.detachEvent("on" + event, eventFunction);
	}

	if (arguments.length === 2)
	{
		element._internal.Events.Handlers[arguments[1]] = null;
	}
	else
	{
		for (var i = 0 ; i < element._internal.Events.Handlers.length ; i++)
		{
			var handler = element._internal.Events.Handlers[i];

			if (handler !== null && event.toLowerCase() === handler.Event.toLowerCase() && eventFunction === handler.Handler && Fit.Core.IsEqual(useCaptureOrOptions, handler.UseCaptureOrOptions) === true)
			{
				element._internal.Events.Handlers[i] = null;
				break;
			}
		}
	}
}

/// <function container="Fit.Events" name="PreventDefault" access="public" static="true" returns="boolean">
/// 	<description> Prevent default behaviour triggered by given event - returns False </description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.PreventDefault = function(e)
{
	Fit.Validation.ExpectEvent(e, true);

	var ev = e || window.event;

	if ((ev.type === "click" || ev.type === "mousedown" || ev.type === "mouseup") && ev.button === 2) // Right click = suppress context menu
	{
		var preventContextMenu = Fit.Events.PreventDefault;

		Fit.Events.AddHandler(document, "contextmenu", true, preventContextMenu);
		Fit.Events.AddHandler(document, "contextmenu", preventContextMenu);

		setTimeout(function()
		{
			Fit.Events.RemoveHandler(document, "contextmenu", true, preventContextMenu);
			Fit.Events.RemoveHandler(document, "contextmenu", preventContextMenu);
		}, 0);
	}

	if (ev.preventDefault)
		ev.preventDefault();
	ev.returnValue = false;
	return false;
}

/// <function container="Fit.Events" name="StopPropagation" access="public" static="true" returns="boolean">
/// 	<description> Prevent given event from propagating (bubble up) - returns False </description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.StopPropagation = function(e)
{
	Fit.Validation.ExpectEvent(e, true);

	var ev = e || window.event;

	if (ev.stopPropagation)
		ev.stopPropagation();
	ev.cancelBubble = true;
	return false;
}

/// <function container="Fit.Events" name="Stop" access="public" static="true" returns="boolean">
/// 	<description>
/// 		Completely suppress event which is equivalent of
/// 		calling both PreventDefault(e) and StopPropagation(e).
/// 		Returns False.
/// 	</description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.Stop = function(e) // e not validated, done by PreventDefault(..) and StopPropagation(..)
{
	Fit.Events.PreventDefault(e);
	Fit.Events.StopPropagation(e);
	return false;
}

/// <function container="Fit.Events" name="GetTarget" access="public" static="true" returns="DOMElement">
/// 	<description> Get a reference to the object that is affected by an event </description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.GetTarget = function(e)
{
	Fit.Validation.ExpectEvent(e, true);

	var ev = e || window.event;
	var target = ev.srcElement || ev.target;
	return (target ? target : null); // Should never return null
}

/// <function container="Fit.Events" name="GetRelatedTarget" access="public" static="true" returns="DOMElement | null">
/// 	<description>
/// 		Get a reference to the secondary object related to an event - e.g. the element losing
/// 		focus in a focus event handler. Returns Null if there is no related event object, or if
/// 		not supported by the browser.
/// 	</description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.GetRelatedTarget = function(e)
{
	Fit.Validation.ExpectEvent(e, true);

	var ev = e || window.event;
	return ev.relatedTarget || null;
}

/// <function container="Fit.Events" name="GetEvent" access="public" static="true" returns="Event">
/// 	<description> Get event argument related to event just fired in a cross browser compatible manner </description>
/// 	<param name="e" type="Event" default="undefined"> Event argument </param>
/// </function>
Fit.Events.GetEvent = function(e)
{
	Fit.Validation.ExpectEvent(e, true);
	return e || window.event;
}

/// <container name="Fit.EventTypeDefs.ModifierKeys">
/// 	<description> Modifier keys </description>
/// 	<member name="Shift" type="boolean"> True if Shift key is being held down, otherwise False </member>
/// 	<member name="Ctrl" type="boolean"> True if Ctrl key is being held down, otherwise False </member>
/// 	<member name="Alt" type="boolean"> True if Alt key is being held down, otherwise False </member>
/// 	<member name="Meta" type="boolean"> True if Meta key (Windows key on a PC, Cmd key on a Mac) is being held down, otherwise False </member>
/// 	<member name="KeyUp" type="integer"> Key code if key is released, otherwise -1 </member>
/// 	<member name="KeyDown" type="integer"> Key code if key is pressed, otherwise -1 </member>
/// </container>

/// <function container="Fit.Events" name="GetModifierKeys" access="public" static="true" returns="Fit.EventTypeDefs.ModifierKeys">
/// 	<description>
/// 		Get object containing information about modifier keys currently being active.
/// 	</description>
/// </function>
Fit.Events.GetModifierKeys = function()
{
	if (window.event && (window.event.type === "keypress" || window.event.type === "keydown" || window.event.type === "keyup")) // Make sure state is current on IE8 which does not support event capturing
	{
		Fit._internal.Events.KeysDown.Shift = window.event.shiftKey === true;
		Fit._internal.Events.KeysDown.Ctrl = window.event.ctrlKey === true;
		Fit._internal.Events.KeysDown.Alt = window.event.altKey === true;
		Fit._internal.Events.KeysDown.Meta = window.event.metaKey === true;
		Fit._internal.Events.KeysDown.KeyUp = window.event.type === "keyup" ? window.event.keyCode : -1;
		Fit._internal.Events.KeysDown.KeyDown = window.event.type === "keypress" || window.event.type === "keydown" ? window.event.keyCode : -1;
	}

	// Cloning to prevent external code from manipulating the object
	return Fit.Core.Clone(Fit._internal.Events.KeysDown);
}

/// <container name="Fit.EventTypeDefs.PointerState">
/// 	<description> Pointer state </description>
/// 	<member name="Buttons" type="{ Primary: boolean, Secondary: boolean, Touch: boolean }"> Pointer buttons currently activated </member>
/// 	<member name="Coordinates" type="{ ViewPort: Fit.TypeDefs.Position, Document: Fit.TypeDefs.Position }"> Pointer position within viewport and document, which might have been scrolled </member>
/// 	<member name="Target" type="DOMElement | null"> Pointer target </member>
/// </container>

/// <function container="Fit.Events" name="GetPointerState" access="public" static="true" returns="Fit.EventTypeDefs.PointerState">
/// 	<description>
/// 		Get object containing information about pointer.
/// 		Object contains the following properties:
/// 		Target: Is the DOMElement affected by click/touch.
/// 		Buttons.Primary/Secondary: Is True if given button is being held down.
/// 		Coordinates.ViewPort.X/Y: Mouse coordinates within viewport.
/// 		Coordinates.Document.X/Y: Mouse coordinates within document which might have been scrolled.
/// 	</description>
/// </function>
Fit.Events.GetPointerState = function()
{
	if (window.event && (window.event.type === "click" || window.event.type === "mousedown" || window.event.type === "mouseup")) // Make sure state is current on IE8 which does not support event capturing
	{
		if ((Fit._internal.Events.Browser.Name === "MSIE" && Fit._internal.Events.Browser.Version === 8 && window.event.button === 1) || window.event.button === 0)
		{
			Fit._internal.Events.Mouse.Buttons.Primary = true;
			Fit._internal.Events.Mouse.Buttons.Secondary = false;
		}
		else if (window.event.button === 2)
		{
			Fit._internal.Events.Mouse.Buttons.Secondary = true;
			Fit._internal.Events.Mouse.Buttons.Primary = false;
		}
	}

	// Cloning to prevent external code from manipulating the object

	var target = Fit._internal.Events.Mouse.Target;
	Fit._internal.Events.Mouse.Target = null; // It is not possible to clone DOMElements with Fit.Core.Clone

	var obj = Fit.Core.Clone(Fit._internal.Events.Mouse);

	obj.Target = target;
	Fit._internal.Events.Mouse.Target = target;

	return obj;
}

// ==============================================
// Internals
// ==============================================

Fit._internal.Events = {};
Fit._internal.Events.Browser = Fit.Browser.GetInfo();
Fit._internal.Events.KeysDown = { Shift: false, Ctrl: false, Alt: false, Meta: false, KeyDown: -1, KeyUp: -1 };
Fit._internal.Events.Mouse = { Buttons: { Primary: false, Secondary: false, Touch: false }, Coordinates: { ViewPort: { X: -1, Y: -1 }, Document: { X: -1, Y: -1 } }, Target: null };
Fit._internal.Events.OnReadyHandlers = [];
Fit._internal.Events.OnDomReadyHandlers = [];
Fit._internal.Events.OnRootedHandlers = [];

// ==============================================
// Observing DOM rooting
// ==============================================

setInterval(function()
{
	var remove = [];

	Fit.Array.ForEach(Fit._internal.Events.OnRootedHandlers, function(handler)
	{
		if (handler.Removed === true)
		{
			Fit.Array.Add(remove, handler);
			return; // Skip to next
		}

		if (Fit.Dom.IsRooted(handler.Element) === true && handler.Parent !== handler.Element.parentElement)
		{
			// Element was rooted or moved to another parent

			handler.Parent = handler.Element.parentElement;
			handler.Callback(handler.Element);
		}
	});

	Fit.Array.ForEach(remove, function(handler)
	{
		Fit.Array.Remove(Fit._internal.Events.OnRootedHandlers, handler);
	});

}, 100);

// ==============================================
// Keyboard tracking
// ==============================================

// Using event capturing to make sure event is registered before target is reached.
// This is not supported by MSIE 8, but we solve this by updating the state when
// Fit.Events.GetModifierKeys() is invoked.

Fit.Events.AddHandler(document, "keydown", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	Fit._internal.Events.KeysDown.Shift = ev.shiftKey === true;
	Fit._internal.Events.KeysDown.Ctrl = ev.ctrlKey === true;
	Fit._internal.Events.KeysDown.Alt = ev.altKey === true;
	Fit._internal.Events.KeysDown.Meta = ev.metaKey === true;
	Fit._internal.Events.KeysDown.KeyUp = -1;
	Fit._internal.Events.KeysDown.KeyDown = ev.keyCode;
});
Fit.Events.AddHandler(document, "keyup", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	Fit._internal.Events.KeysDown.Shift = ev.shiftKey === true;
	Fit._internal.Events.KeysDown.Ctrl = ev.ctrlKey === true;
	Fit._internal.Events.KeysDown.Alt = ev.altKey === true;
	Fit._internal.Events.KeysDown.Meta = ev.metaKey === true;
	Fit._internal.Events.KeysDown.KeyUp = ev.keyCode;
	Fit._internal.Events.KeysDown.KeyDown = -1;
});

// ==============================================
// Mouse and touch tracking
// ==============================================

// Using event capturing to make sure event is registered before target is reached.
// This is not supported by MSIE 8, but we solve this by updating the state when
// Fit.Events.GetPointerState() is invoked.

// Notice that MouseUp is not fired when using the secondary mouse button (right click)
// if context menu is not suppressed. Therefore we always assume the secondary button
// is released when MouseDown or MouseUp is fired.
// And for consistency, we do the same for the primary button.
// Therefore, the primary and secondary buttons are never considered pressed or held down
// simultaneously.

// http://www.quirksmode.org/js/events_properties.html

Fit.Events.AddHandler(document, "mousedown", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	if ((Fit._internal.Events.Browser.Name === "MSIE" && Fit._internal.Events.Browser.Version === 8 && ev.button === 1) || ev.button === 0)
	{
		Fit._internal.Events.Mouse.Buttons.Primary = true;
		Fit._internal.Events.Mouse.Buttons.Secondary = false;
	}
	else if (ev.button === 2)
	{
		Fit._internal.Events.Mouse.Buttons.Secondary = true;
		Fit._internal.Events.Mouse.Buttons.Primary = false;
	}

	Fit._internal.Events.Mouse.Target = Fit.Events.GetTarget(ev);
});
Fit.Events.AddHandler(document, "mouseup", true, function(e)
{
	setTimeout(function()
	{
		// OnContextMenu event usually fires before MouseUp, but on Firefox it fires after, so the
		// button states are preserved a little longer to make them accessible to OnContextMenu handlers on Firefox.
		Fit._internal.Events.Mouse.Buttons.Primary = false;
		Fit._internal.Events.Mouse.Buttons.Secondary = false;

		// OnClick event might fire after MouseUp (if mouse was not moved away from element on which
		// MouseDown fired), so keep Target a little longer to make it accessible to OnClick handlers.
		Fit._internal.Events.Mouse.Target = null;
	}, 0);
});
Fit.Events.AddHandler(document, "mouseout", true, function(e)
{
	Fit._internal.Events.Mouse.Buttons.Primary = false;
	Fit._internal.Events.Mouse.Buttons.Secondary = false;
	Fit._internal.Events.Mouse.Target = null;
});
Fit.Events.AddHandler(document, "mousemove", function(e)
{
	var ev = Fit.Events.GetEvent(e);

	if (document.body === null) // Not ready yet
		return;

	// Notice: Browser vendors are changing the way coordinates
	// and dimensions are reported. W3C previously define these as
	// integers/longs, but browsers are moving to floats for smoother
	// animation and scrolling, and for more accurate positioning.
	// https://code.google.com/p/chromium/issues/detail?id=323935
	// For consistency we use Math.floor to make sure integers are
	// always returned on both modern and legacy browsers.

	// Mouse position in viewport
	Fit._internal.Events.Mouse.Coordinates.ViewPort.X = Math.floor(ev.clientX);
	Fit._internal.Events.Mouse.Coordinates.ViewPort.Y = Math.floor(ev.clientY);

	// Mouse position in document which may have been scrolled
	var scrollPos = Fit.Dom.GetScrollPosition(document.body); // Object with integer values returned
	Fit._internal.Events.Mouse.Coordinates.Document.X = Fit._internal.Events.Mouse.Coordinates.ViewPort.X + scrollPos.X;
	Fit._internal.Events.Mouse.Coordinates.Document.Y = Fit._internal.Events.Mouse.Coordinates.ViewPort.Y + scrollPos.Y;
});
Fit.Events.AddHandler(document, "touchstart", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	Fit._internal.Events.Mouse.Buttons.Touch = true;

	// Touch position in viewport
	Fit._internal.Events.Mouse.Coordinates.ViewPort.X = Math.floor(ev.touches[0].clientX);
	Fit._internal.Events.Mouse.Coordinates.ViewPort.Y = Math.floor(ev.touches[0].clientY);

	// Touch position in document which may have been scrolled
	var scrollPos = Fit.Dom.GetScrollPosition(document.body); // Object with integer values returned
	Fit._internal.Events.Mouse.Coordinates.Document.X = Fit._internal.Events.Mouse.Coordinates.ViewPort.X + scrollPos.X;
	Fit._internal.Events.Mouse.Coordinates.Document.Y = Fit._internal.Events.Mouse.Coordinates.ViewPort.Y + scrollPos.Y;

	Fit._internal.Events.Mouse.Target = Fit.Events.GetTarget(ev);
});
Fit.Events.AddHandler(document, "touchend", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	// Postpone reset to allow touchend handlers registered to fire in bubbling phase,
	// as well as OnClick handlers which might fire on touch devices, to access pointer state.
	setTimeout(function()
	{
		Fit._internal.Events.Mouse.Buttons.Touch = false;

		/*Fit._internal.Events.Mouse.Coordinates.ViewPort.X = -1;
		Fit._internal.Events.Mouse.Coordinates.ViewPort.Y = -1;
		Fit._internal.Events.Mouse.Coordinates.Document.X = -1;
		Fit._internal.Events.Mouse.Coordinates.Document.Y = -1;*/

		Fit._internal.Events.Mouse.Target = null;
	}, 0);
});
Fit.Events.AddHandler(document, "touchcancel", true, function(e)
{
	var ev = Fit.Events.GetEvent(e);

	// Postpone reset to allow touchcancel handlers registered to fire in bubbling phase to access pointer state
	setTimeout(function()
	{
		Fit._internal.Events.Mouse.Buttons.Touch = false;

		/*Fit._internal.Events.Mouse.Coordinates.ViewPort.X = -1;
		Fit._internal.Events.Mouse.Coordinates.ViewPort.Y = -1;
		Fit._internal.Events.Mouse.Coordinates.Document.X = -1;
		Fit._internal.Events.Mouse.Coordinates.Document.Y = -1;*/

		Fit._internal.Events.Mouse.Target = null;
	}, 0);
});
Fit.Events.AddHandler(document, "touchmove", function(e)
{
	var ev = Fit.Events.GetEvent(e);

	// Touch position in viewport
	Fit._internal.Events.Mouse.Coordinates.ViewPort.X = Math.floor(ev.touches[0].clientX);
	Fit._internal.Events.Mouse.Coordinates.ViewPort.Y = Math.floor(ev.touches[0].clientY);

	// Touch position in document which may have been scrolled
	var scrollPos = Fit.Dom.GetScrollPosition(document.body); // Object with integer values returned
	Fit._internal.Events.Mouse.Coordinates.Document.X = Fit._internal.Events.Mouse.Coordinates.ViewPort.X + scrollPos.X;
	Fit._internal.Events.Mouse.Coordinates.Document.Y = Fit._internal.Events.Mouse.Coordinates.ViewPort.Y + scrollPos.Y;
});

// ==============================================
// Simple mutation tracking
// ==============================================

// This is a very simple substitution for native mutation observers,
// which unfortunately requires recent versions of some of the major
// browsers (e.g. IE 11, Safari 6 and Chrome on Android 4.4).

Fit._internal.Events.MutationObservers = [];
Fit._internal.Events.MutationObserverIds = -1;
Fit._internal.Events.MutationObserverIntervalId = -1;
Fit._internal.Events.MutationCheckExecuting = false;
Fit._internal.Events.MutationRegisterPostponed = [];

/// <function container="Fit.EventsTypeDefs" name="MutationObserverCallback" returns="any">
/// 	<description> Event handler callback used with Fit.Events.AddMutationObserver(..) </description>
/// 	<param name="dom" type="DOMElement"> DOM element </param>
/// </function>

/// <function container="Fit.Events" name="AddMutationObserver" access="public" static="true" returns="integer">
/// 	<description>
/// 		Registers mutation observer which is invoked when a DOMElement is updated. By default
/// 		only attributes and dimensions are observed. Use deep flag to have children and character data observed too.
/// 		An observer ID is returned which can be used to remove mutation observer.
/// 		Important: Mutation observers should be removed when no longer needed for better performance!
/// 		To remove an observer from within the observer function itself, simply call disconnect().
/// 	</description>
/// 	<param name="elm" type="DOMElement"> DOMElement to observe </param>
/// 	<param name="obs" type="Fit.EventsTypeDefs.MutationObserverCallback"> JavaScript observer function to register - receives reference to DOMElement being observed when updated </param>
/// 	<param name="deep" type="boolean" default="false"> Flag indicating whether to check for modifications within element (children and character data) - this could potentially be expensive </param>
/// </function>
Fit.Events.AddMutationObserver = function(elm, obs, deep)
{
	Fit.Validation.ExpectDomElement(elm);
	Fit.Validation.ExpectFunction(obs);
	Fit.Validation.ExpectBoolean(deep, true);

	// Postpone if check is running

	if (Fit._internal.Events.MutationCheckExecuting === true)
	{
		Fit.Array.Add(Fit._internal.Events.MutationRegisterPostponed, { Task: "Add", Element: elm, Observer: obs, Deep: deep === true });
		return;
	}

	// Configure event handlers responsible for triggering mutation check

	if (Fit._internal.Events.MutationObservers.length === 0)
	{
		Fit.Events.AddHandler(document, "click", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "mousedown", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "mouseup", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "keypress", Fit._internal.Events.CheckMutations); // Not using keydown, it fires continuously
		Fit.Events.AddHandler(document, "keyup", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "touchstart", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "touchend", Fit._internal.Events.CheckMutations);
		Fit.Events.AddHandler(document, "touchcancel", Fit._internal.Events.CheckMutations);

		var checkInterval = Fit._internal.Events.Browser.Name !== "MSIE" ? 500 : 1000;
		Fit._internal.Events.MutationObserverIntervalId = setInterval(Fit._internal.Events.CheckMutations, checkInterval);
	}

	// Add mutation observer

	var hashCode = 0;
	var dimensions = elm.offsetWidth + "x" + elm.offsetHeight;

	if (deep === true)
	{
		hashCode = Fit.String.Hash(elm.outerHTML + dimensions);
	}
	else
	{
		var clone = elm.cloneNode(false);
		hashCode = Fit.String.Hash(clone.outerHTML + dimensions)
	}

	Fit._internal.Events.MutationObserverIds++;
	Fit.Array.Add(Fit._internal.Events.MutationObservers, { Element: elm, Observer: obs, Hash: hashCode, Deep: (deep === true), Id: Fit._internal.Events.MutationObserverIds });

	return Fit._internal.Events.MutationObserverIds;
}

/// <function container="Fit.Events" name="RemoveMutationObserver" access="public" static="true">
/// 	<description> Remove mutation observer </description>
/// 	<param name="elm" type="DOMElement"> DOMElement being observed </param>
/// 	<param name="obs" type="Fit.EventsTypeDefs.MutationObserverCallback"> JavaScript observer function to remove </param>
/// 	<param name="deep" type="boolean" default="undefined"> If defined, observer must have been registered with the same deep value to be removed </param>
/// </function>
/// <function container="Fit.Events" name="RemoveMutationObserver" access="public" static="true">
/// 	<description> Remove mutation observer by ID </description>
/// 	<param name="id" type="integer"> Observer ID returned from AddMutationObserver(..) function </param>
/// </function>
Fit.Events.RemoveMutationObserver = function()
{
	var elm, obs, deep, id = undefined;

	if (arguments.length > 1)
	{
		elm = arguments[0];
		obs = arguments[1];
		deep = arguments[2];

		Fit.Validation.ExpectDomElement(elm);
		Fit.Validation.ExpectFunction(obs);
		Fit.Validation.ExpectBoolean(deep, true);
	}
	else
	{
		id = arguments[0];

		Fit.Validation.ExpectInteger(id);
	}

	if (Fit._internal.Events.MutationCheckExecuting === true)
	{
		if (Fit.Validation.IsSet(id) === true)
		{
			Fit.Array.Add(Fit._internal.Events.MutationRegisterPostponed, { Task: "Remove", Id: id });
		}
		else
		{
			Fit.Array.Add(Fit._internal.Events.MutationRegisterPostponed, { Task: "Remove", Element: elm, Observer: obs, Deep: deep === true });
		}

		return;
	}

	var found = null;

	Fit.Array.ForEach(Fit._internal.Events.MutationObservers, function(mo)
	{
		if ((Fit.Validation.IsSet(id) === true && mo.Id === id) || (mo.Element === elm && mo.Observer === obs && mo.Deep === ((Fit.Validation.IsSet(deep) === true) ? deep : false)))
		{
			found = mo;
			return false; // Break loop
		}
	});

	if (found !== null)
		Fit.Array.Remove(Fit._internal.Events.MutationObservers, found);

	// Remove event handlers if all mutation observers have been removed

	if (Fit._internal.Events.MutationObservers.length === 0)
	{
		Fit.Events.RemoveHandler(document, "click", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "mousedown", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "mouseup", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "keypress", Fit._internal.Events.CheckMutations); // Not using keydown, it fires continuously
		Fit.Events.RemoveHandler(document, "keyup", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "touchstart", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "touchend", Fit._internal.Events.CheckMutations);
		Fit.Events.RemoveHandler(document, "touchcancel", Fit._internal.Events.CheckMutations);
		clearInterval(Fit._internal.Events.MutationObserverIntervalId);
	}
}

Fit._internal.Events.CheckMutations = function()
{
	// Guard against changes to mutation observer collection while running CheckMutations().
	// This is necessary in case a running mutation observer, triggered from CheckMutations(),
	// results in another mutation observer being added/removed. We do not want to change a
	// collection being iterated, and we do not want to risk invoking mutation observers that
	// have been removed.
	Fit._internal.Events.MutationCheckExecuting = true;

	var toRemove = [];

	Fit.Array.ForEach(Fit._internal.Events.MutationObservers, function(mo)
	{
		// Skip mutation observers removed while CheckMutations() was running

		var skipRemovedObserver = false;

		Fit.Array.ForEach(Fit._internal.Events.MutationRegisterPostponed, function(mop)
		{
			if (mop.Task === "Add")
			{
				return; // Skip, only check for removals
			}

			if ((Fit.Validation.IsSet(mop.Id) === true && mop.Id === mo.Id) || (mop.Element === mo.Element && mop.Observer === mo.Observer && mo.Deep === ((Fit.Validation.IsSet(mo.Deep) === true) ? mo.Deep : false)))
			{
				// Observer has been scheduled for removal - do not invoke it

				skipRemovedObserver = true;
				return false; // Break loop
			}

		});

		if (skipRemovedObserver === true)
		{
			return; // Skip, scheduled for removal
		}

		// Calculate and compare hashes revealing changes to element

		var newHash = 0;
		var dimensions = mo.Element.offsetWidth + "x" + mo.Element.offsetHeight;

		if (mo.Deep === true)
		{
			newHash = Fit.String.Hash(mo.Element.outerHTML + dimensions);
		}
		else
		{
			var clone = mo.Element.cloneNode(false);
			newHash = Fit.String.Hash(clone.outerHTML + dimensions)
		}

		// Trigger mutation observer if element has changed

		if (mo.Hash !== newHash)
		{
			// Element has mutated

			mo.Hash = newHash;

			// Create global disconnect function allowing observer to remove itself by simply invoking disconnect()

			var orgDisconnect = window.disconnect;
			window.disconnect = function() { Fit.Array.Add(toRemove, mo); };

			// Call observer

			var error = null;

			try
			{
				mo.Observer(mo.Element);
			}
			catch (err)
			{
				error = err;
			}

			// Remove global disconnect function

			if (orgDisconnect)
			{
				window.disconnect = orgDisconnect;
			}
			else
			{
				try
				{
					delete window.disconnect; // Fails in IE8 with "Object doesn't support this action"
				}
				catch (err)
				{
					window.disconnect = undefined;
				}
			}

			// Re-throw error if observer failed

			if (error !== null)
				Fit.Validation.Throw(error);
		}
	});

	// Remove observers that called disconnect()

	Fit.Array.ForEach(toRemove, function(mo)
	{
		Fit.Events.RemoveMutationObserver(mo.Element, mo.Observer, mo.Deep);
	});

	// Handle mutation observers that were added/removed
	// while CheckMutations() invoked existing mutation observers.

	Fit._internal.Events.MutationCheckExecuting = false;

	Fit.Array.ForEach(Fit._internal.Events.MutationRegisterPostponed, function(mo)
	{
		if (mo.Task === "Add")
		{
			Fit.Events.AddMutationObserver(mo.Element, mo.Observer, mo.Deep);
		}
		else // if (mo.Task === "Remove")
		{
			if (Fit.Validation.IsSet(mo.Id) === true)
			{
				Fit.Events.RemoveMutationObserver(mo.Id);
			}
			else
			{
				Fit.Events.RemoveMutationObserver(mo.Element, mo.Observer, mo.Deep);
			}
		}
	});

	Fit._internal.Events.MutationRegisterPostponed = [];
}

// ==============================================
// OnReady handling
// ==============================================

/// <function container="Fit.Events" name="OnReady" access="public" static="true">
/// 	<description> Registers OnReady handler which gets fired when document is ready, or if it is already ready </description>
/// 	<param name="callback" type="function"> JavaScript function to register </param>
/// </function>
Fit.Events.OnReady = function(callback)
{
	Fit.Validation.ExpectFunction(callback);

	if (Fit._internal.Events.OnReadyFired === true)
	{
		callback();
	}
	else
	{
		Fit._internal.Events.OnReadyHandlers.push(callback);
	}
}

Fit.Events.AddHandler(window, "load", function()
{
	Fit._internal.Events.OnReadyFired = true;

	Fit.Array.ForEach(Fit._internal.Events.OnReadyHandlers, function(handler)
	{
		handler();
	});

	Fit.Array.Clear(Fit._internal.Events.OnReadyHandlers);
});

/// <function container="Fit.Events" name="OnDomReady" access="public" static="true">
/// 	<description> Registers OnReady handler which gets fired when DOM is ready for manipulation, or if it is already ready </description>
/// 	<param name="callback" type="function"> JavaScript function to register </param>
/// </function>
Fit.Events.OnDomReady = function(callback)
{
	Fit.Validation.ExpectFunction(callback);

	if (Fit._internal.Events.OnDomReadyFired === true)
	{
		callback();
	}
	else
	{
		Fit._internal.Events.OnDomReadyHandlers.push(callback);
	}
}

Fit.Events.AddHandler(document, "DOMContentLoaded", function()
{
	Fit._internal.Events.OnDomReadyFired = true;

	Fit.Array.ForEach(Fit._internal.Events.OnDomReadyHandlers, function(handler)
	{
		handler();
	});

	Fit.Array.Clear(Fit._internal.Events.OnDomReadyHandlers);
});

Fit.Http = {};

Fit._internal.Http = {};
Fit._internal.Http.JsonpRequest = {};
Fit._internal.Http.JsonpRequest.Callbacks = {};

// Use http://www.jsontest.com for testing

/// <container name="Fit.Http.Request">
/// 	Asynchronous HTTP request functionality (AJAX/WebService).
///
/// 	// Example code
///
/// 	var http = new Fit.Http.Request(&quot;CreateUser.php&quot;);
///
/// 	http.SetData(&quot;username=Jack&amp;password=Secret&quot;);
/// 	http.OnStateChange(function(r)
/// 	{
/// 		&#160;&#160;&#160;&#160; if (http.GetCurrentState() === 4 &amp;&amp; http.GetHttpStatus() === 200)
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; alert(&quot;User created - server said: &quot; + http.GetResponseText());
/// 	});
///
/// 	http.Start();
/// </container>

/// <function container="Fit.Http.Request" name="Request" access="public">
/// 	<description> Constructor - creates instance of Request class </description>
/// 	<param name="uri" type="string"> URL to request </param>
/// </function>
Fit.Http.Request = function(uri)
{
	/*
	// Test case
	req = new Fit.Http.Request("./post.php");
	req.AddData("Name", "Jimmy");
	req.AddData("Age", "31");
	req.AddData("Gender", "Male");
	req.OnStateChange(function(r) { console.log("State listener", r, r.GetCurrentState(), r.GetHttpStatus()); });
	req.OnStateChange(function(r) { console.log("OnStateChange ", r, r.GetCurrentState(), r.GetHttpStatus()); });
	req.OnSuccess(function(r) { console.log("OnSuccess ", r, r.GetResponseText()); });
	req.OnSuccess(function(r) { console.log("Finally done!"); });
	req.OnFailure(function(r) { console.log("OnFailure ", r, r.GetHttpStatus()); });
	req.Start();*/

	Fit.Validation.ExpectStringValue(uri);

	var me = this;
	var url = uri;
	var async = -1; // -1 = not defined / enforced, 0 = sync, 1 = async
	var httpRequest = getHttpRequestObject();
	var customHeaders = {};
	var customProperties = {};
	var formData = null;
	var data = null;
	var method = null;

	var onStateChange = [];
	var onRequestHandlers = [];
	var onSuccessHandlers = [];
	var onFailureHandlers = [];
	var onAbortHandlers = [];

	// Init

	function init()
	{
		httpRequest.onreadystatechange = function()
		{
			me._internal.FireOnStateChange();

			if (httpRequest.readyState === 4)
			{
				if (httpRequest.status === 0)
				{
					me._internal.FireOnAbort();
				}
				else if (httpRequest.status >= 200 && httpRequest.status <= 299) // Entire 2xx range indicates success
				{
					me._internal.FireOnSuccess();
				}
				else
				{
					me._internal.FireOnFailure();
				}
			}
		}

		// Cross Site Request Forgery (CSRF) protection
		// https://markitzeroday.com/x-requested-with/cors/2017/06/29/csrf-mitigation-for-ajax-requests.html
		me.AddHeader("X-Requested-With", "XMLHttpRequest");
	}

	// Public

	/// <function container="Fit.Http.Request" name="AddHeader" access="public">
	/// 	<description> Add header to request </description>
	/// 	<param name="key" type="string"> Header key </param>
	/// 	<param name="value" type="string"> Header value </param>
	/// </function>
	this.AddHeader = function(key, value)
	{
		Fit.Validation.ExpectStringValue(key);
		Fit.Validation.ExpectString(value);
		customHeaders[key.toLowerCase()] = { Key: key, Value: value };
	}

	/// <function container="Fit.Http.Request" name="GetHeader" access="public" returns="string | null">
	/// 	<description> Get request header - returns Null if not found </description>
	/// 	<param name="key" type="string"> Header name </param>
	/// </function>
	this.GetHeader = function(key)
	{
		Fit.Validation.ExpectString(key);
		return (Fit.Validation.IsSet(customHeaders[key.toLowerCase()]) === true ? customHeaders[key.toLowerCase()].Value : null);
	}

	/// <function container="Fit.Http.Request" name="RemoveHeader" access="public">
	/// 	<description> Remove request header </description>
	/// 	<param name="key" type="string"> Header name </param>
	/// </function>
	this.RemoveHeader = function(key)
	{
		Fit.Validation.ExpectString(key);
		delete customHeaders[key.toLowerCase()];
	}

	/// <function container="Fit.Http.Request" name="GetHeaders" access="public" returns="string[]">
	/// 	<description> Get all request header names </description>
	/// </function>
	this.GetHeaders = function()
	{
		var keys = [];

		Fit.Array.ForEach(customHeaders, function(key)
		{
			Fit.Array.Add(keys, customHeaders[key].Key);
		})

		return keys;
	}

	/// <function container="Fit.Http.Request" name="ClearHeaders" access="public">
	/// 	<description> Remove all request headers </description>
	/// </function>
	this.ClearHeaders = function()
	{
		customHeaders = {};
	}

	/// <function container="Fit.Http.Request" name="SetData" access="public">
	/// 	<description> Set data to post - this will change the request method from GET to POST </description>
	/// 	<param name="dataObj" type="object"> Data to send </param>
	/// </function>
	this.SetData = function(dataObj)
	{
		me.ClearFormData(); // Clears both form data and removes content-type header if ensured in conjunction with form data previously set
		data = null;

		if (typeof(dataObj) === "string" && (me.GetHeader("content-type") === null || me.GetHeader("content-type").toLowerCase() === "application/x-www-form-urlencoded"))
		{
			// Backward compatibility - Fit.Http.Request previously only allowed "key=val&key2=val2&etc" being passed to SetData.
			// Now accepting both the old "form data string mode" and any kind of object which will be stringified upon request.
			// Unfortunately this means that passing a "clean string" will not be possible without triggering "form data mode",
			// unless the Content-Type header is set prior to calling SetData("My string").
			// So request.SetData("Hello world") without setting the Content-Type header first will be considered form data
			// (Hello world=) and the Content-Type header will be set to "application/x-www-form-urlencoded".
			Fit.Browser.Log("Deprecated use of SetData(..) to set form data on instance of Fit.Http.Request - please use AddFormData(..) instead! If data added is not to be considered form data then make sure to set Content-Type header prior to calling SetData(..)!");

			formData = {};
			ensureContentTypeHeaderForFormData();

			var coll = parseKeyValuePairs(dataObj);

			Fit.Array.ForEach(coll, function(key)
			{
				formData[key] = { Value: coll[key], Encode: true };
			});
		}
		else
		{
			data = dataObj;
		}
	}

	/// <function container="Fit.Http.Request" name="GetData" access="public" returns="object">
	/// 	<description> Get data set to be posted </description>
	/// </function>
	this.GetData = function()
	{
		if (formData !== null)
		{
			return getFormDataString(false); // False = do not encode values (returns string as provided to SetData(..))
		}

		return data;
	}

	/// <function container="Fit.Http.Request" name="AddFormData" access="public">
	/// 	<description>
	/// 		Add form data - this will change the request method from GET to POST
	/// 		and cause the following header to be added to the request, unless already
	/// 		defined: Content-type: application/x-www-form-urlencoded
	/// 	</description>
	/// 	<param name="key" type="string"> Data key </param>
	/// 	<param name="value" type="string"> Data value </param>
	/// 	<param name="uriEncode" type="boolean" default="true">
	/// 		Set False to prevent value from being URI encoded to preserve special characters
	/// 	</param>
	/// </function>
	this.AddFormData = function(key, value, uriEncode)
	{
		Fit.Validation.ExpectStringValue(key);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(uriEncode, true);

		if (formData === null)
		{
			formData = {};
			ensureContentTypeHeaderForFormData();
		}

		formData[key] = { Value: value, Encode: (uriEncode !== false) };
	}

	this.AddData = function(key, value, uriEncode) // Backward compatibility
	{
		Fit.Validation.ExpectStringValue(key);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(uriEncode, true);

		Fit.Browser.Log("Use of deprecated AddData(..) function to set form data on instance of Fit.Http.Request - please use AddFormData(..) instead!")

		me.AddFormData(key, value, uriEncode);
	}

	/// <function container="Fit.Http.Request" name="GetFormData" access="public" returns="string | null">
	/// 	<description> Get form value added to form data collection - returns Null if not found </description>
	/// 	<param name="key" type="string"> Data key </param>
	/// </function>
	this.GetFormData = function(key)
	{
		Fit.Validation.ExpectString(key);
		return (formData !== null && Fit.Validation.IsSet(formData[key]) === true ? formData[key].Value : null);
	}

	/// <function container="Fit.Http.Request" name="RemoveFormData" access="public">
	/// 	<description> Remove form value from form data collection </description>
	/// 	<param name="key" type="string"> Data key </param>
	/// </function>
	this.RemoveFormData = function(key)
	{
		Fit.Validation.ExpectString(key);

		if (formData !== null)
			delete formData[key];
	}

	/// <function container="Fit.Http.Request" name="ClearFormData" access="public">
	/// 	<description> Remove all form values from form data collection </description>
	/// </function>
	this.ClearFormData = function()
	{
		formData = null;

		if (me.GetHeader("content-type") !== null && customHeaders["content-type"]._ensured === true)
			me.RemoveHeader("content-type");
	}

	/// <function container="Fit.Http.Request" name="Method" access="public" returns='"GET" | "POST" | "HEAD" | "PUT" | "DELETE" | "OPTIONS"'>
	/// 	<description> Get/set request method (e.g. GET, POST, HEAD, PUT, DELETE, OPTIONS) </description>
	/// 	<param name="val" type='"GET" | "POST" | "HEAD" | "PUT" | "DELETE" | "OPTIONS"' default="undefined"> If defined, changes HTTP request method to specified value </param>
	/// </function>
	this.Method = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			method = val;
		}

		return (method !== null ? method : (data !== null || formData !== null ? "POST" : "GET"));
	}

	/// <function container="Fit.Http.Request" name="Async" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether request is made asynchronously or synchronously </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, enforces an async or sync request based on the boolean value provided </param>
	/// </function>
	this.Async = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			async = (val === true ? 1 : 0);
		}

		if (async !== -1) // Enforced (value is 0 or 1)
			return (async === 1);

		// Not enforced - request will become async if event handlers are registered
		return (onStateChange.length > 0 || onSuccessHandlers.length > 0 || onFailureHandlers.length > 0);
	}

	/// <function container="Fit.Http.Request" name="Url" access="public" returns="string">
	/// 	<description> Get/set request URL </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes request URL to specified value </param>
	/// </function>
	this.Url = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			url = val;
		}

		return url;
	}

	/// <function container="Fit.Http.Request" name="Start" access="public">
	/// 	<description>
	/// 		Invoke request. An asynchroneus request is performed if an
	/// 		OnStateChange, OnSuccess, or OnFailure event handler has been set.
	/// 		If no event handlers have been set, a synchronous request will be performed,
	/// 		causing the client to wait (and freeze) until data is received.
	/// 	</description>
	/// </function>
	this.Start = function()
	{
		if (formData !== null && data !== null)
		{
			Fit.Validation.ThrowError("Unable to send both (JSON) Data and Form Data simultaneously - specify only one type of data to send");
		}

		// Fire OnRequest

		var cancel = me._internal.FireOnRequest();

		if (cancel === true)
			return;

		// Perform request

		var httpMethod = me.Method();
		var async = me.Async();
		httpRequest.open(httpMethod, url, async);

		for (var header in customHeaders)
		{
			httpRequest.setRequestHeader(header, customHeaders[header].Value);
		}

		Fit.Array.ForEach(customProperties, function(key)
		{
			httpRequest[key] = customProperties[key];
		});

		// Monitor request to catch communication problems (e.g. DNS errors)
		var checkerId = -1;
		checkerId = setInterval(function()
		{
			if (me.GetCurrentState() === 4) // Request done
			{
				clearInterval(checkerId);

				if (me.GetHttpStatus() === 0)
				{
					// Request is done (ready state === 4) and failed (HTTP status === 0),
					// possibly due to DNS problem (ERR_NAME_NOT_RESOLVED) - fire OnFailure.

					me._internal.FireOnFailure();
				}
			}
		}, 1000);

		try // Using try/catch to catch any error that may occur when initializing connection
		{
			if (formData !== null)
				httpRequest.send(getFormDataString(true));
			else
				httpRequest.send((data !== null ? (typeof(data) === "string" ? data : JSON.stringify(data)) : ""));
		}
		catch (err)
		{
			me._internal.FireOnFailure();
		}
	}

	/// <function container="Fit.Http.Request" name="Abort" access="public">
	/// 	<description> Abort asynchroneus request </description>
	/// </function>
	this.Abort = function()
	{
		httpRequest.abort();
	}

	/// <function container="Fit.Http.Request" name="GetResponseXml" access="public" returns="Document">
	/// 	<description>
	/// 		Returns result from request as XML or HTML document.
	/// 		Return value will only be as expected if GetCurrentState() returns a value of 4
	/// 		(request done) and GetHttpStatus() returns a value of 2xx (request successful).
	/// 	</description>
	/// </function>
	this.GetResponseXml = function()
	{
		return httpRequest.responseXML;
	}

	/// <function container="Fit.Http.Request" name="GetResponseText" access="public" returns="string">
	/// 	<description>
	/// 		Returns text result from request.
	/// 		Return value will only be as expected if GetCurrentState() returns a value of 4
	/// 		(request done) and GetHttpStatus() returns a value of 2xx (request successful).
	/// 	</description>
	/// </function>
	this.GetResponseText = function()
	{
		return httpRequest.responseText;
	}

	/// <function container="Fit.Http.Request" name="GetResponseJson" access="public" returns="object | null">
	/// 	<description>
	/// 		Returns result from request as JSON object, Null if no response was returned.
	/// 		Return value will only be as expected if GetCurrentState() returns a value of 4
	/// 		(request done) and GetHttpStatus() returns a value of 2xx (request successful).
	/// 	</description>
	/// </function>
	this.GetResponseJson = function()
	{
		return ((httpRequest.responseText !== "") ? JSON.parse(httpRequest.responseText) : null);
	}

	/// <function container="Fit.Http.Request" name="GetResponse" access="public" returns="object">
	/// 	<description>
	/// 		Returns result from request. Use this to obtain e.g. binary data on supported browsers.
	/// 		For requests without custom RequestProperties set the return value will be the response text.
	/// 	</description>
	/// </function>
	this.GetResponse = function()
	{
		return (((!httpRequest.responseType || httpRequest.responseType === "text") && typeof(httpRequest.responseText) === "string") ? httpRequest.responseText : httpRequest.response);
	}

	/// <function container="Fit.Http.Request" name="GetCurrentState" access="public" returns="integer">
	/// 	<description>
	/// 		Get current request state.
	/// 		0 = Unsent
	/// 		1 = Opened
	/// 		2 = Headers received
	/// 		3 = Loading
	/// 		4 = Done (response is ready for processing)
	/// 	</description>
	/// </function>
	this.GetCurrentState = function() // 0 = unsent, 1 = opened, 2 = headers received, 3 = loading, 4 = done
	{
		return httpRequest.readyState;
	}

	/// <function container="Fit.Http.Request" name="GetHttpStatus" access="public" returns="integer">
	/// 	<description>
	/// 		Returns HTTP status. Common return values are:
	/// 		200 = OK (successful request)
	/// 		304 = Forbidden (access denied)
	/// 		404 = Not found
	/// 		408 = Request time out
	/// 		500 = Internal server error
	/// 		503 = Service unavailable
	/// 	</description>
	/// </function>
	this.GetHttpStatus = function()
	{
		return httpRequest.status;
	}

	/// <function container="Fit.Http.Request" name="GetResponseHeaders" access="public" returns="{ Key: string, Value: string }[]">
	/// 	<description> Get response headers - returned array contain objects with Key (string) and Value (string) properties </description>
	/// </function>
	this.GetResponseHeaders = function()
	{
		var headers = [];

		if (httpRequest.getAllResponseHeaders() === "")
			return headers;

		Fit.Array.ForEach(Fit.String.Trim(httpRequest.getAllResponseHeaders()).split("\n"), function(h)
		{
			var info = h.split(":");
			Fit.Array.Add(headers, { Key: Fit.String.Trim(info[0]), Value: Fit.String.Trim(info[1]) });
		});

		return headers;
	}

	/// <function container="Fit.Http.Request" name="GetResponseHeader" access="public" returns="string | null">
	/// 	<description> Get response header (e.g. text/html) - returns Null if not found </description>
	/// 	<param name="key" type="string"> Header key (e.g. Content-Type) </param>
	/// </function>
	this.GetResponseHeader = function(key)
	{
		Fit.Validation.ExpectString(key);

		var headerValue = httpRequest.getResponseHeader(key);

		if (headerValue === "" && Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() < 10) // Legacy IE returns an empty string for non-existing headers - make sure it exists or return Null if it does not
		{
			var found = false;

			Fit.Array.ForEach(me.GetResponseHeaders(), function(header)
			{
				if (header.Key.toLowerCase() === key.toLowerCase())
				{
					found = true;
					return false; // Break loop
				}
			});

			headerValue = (found === true ? headerValue : null);
		}

		return headerValue;
	}

	// Events

	/// <function container="Fit.Http.RequestTypeDefs" name="EventHandler">
	/// 	<description> Request event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of request which triggered event </param>
	/// </function>

	/// <function container="Fit.Http.Request" name="OnStateChange" access="public">
	/// 	<description>
	/// 		Add function to invoke when request state is changed.
	/// 		Use GetCurrentState() to read the state at the given time.
	/// 	</description>
	/// 	<param name="func" type="Fit.Http.RequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when state changes.
	/// 		Fit.Http.Request instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnStateChange = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onStateChange, func);
	}

	this.SetStateListener = this.OnStateChange; // Backward compatibility

	/// <function container="Fit.Http.Request" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add function to invoke when request is initiated.
	/// 		Request can be canceled by returning False.
	/// 	</description>
	/// 	<param name="func" type="Fit.Http.RequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request is initiated.
	/// 		Fit.Http.Request instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnRequest = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onRequestHandlers, func);
	}

	/// <function container="Fit.Http.Request" name="OnSuccess" access="public">
	/// 	<description> Add function to invoke when request is successful </description>
	/// 	<param name="func" type="Fit.Http.RequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request finished successfully.
	/// 		Fit.Http.Request instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnSuccess = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onSuccessHandlers, func);
	}

	/// <function container="Fit.Http.Request" name="OnFailure" access="public">
	/// 	<description> Add function to invoke when request is unsuccessful </description>
	/// 	<param name="func" type="Fit.Http.RequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request finished, but not successfully.
	/// 		Fit.Http.Request instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnFailure = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onFailureHandlers, func);
	}

	/// <function container="Fit.Http.Request" name="OnAbort" access="public">
	/// 	<description> Add function to invoke when request is canceled </description>
	/// 	<param name="func" type="Fit.Http.RequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request is canceled.
	/// 		Fit.Http.Request instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnAbort = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onAbortHandlers, func);
	}

	/// <function container="Fit.Http.Request" name="RequestProperties" access="public" returns="object">
	/// 	<description>
	/// 		Set/get custom XHR request properties.
	/// 		Example of property object: { withCredentials: true, responseType: 'blob' }.
	/// 		How different browsers and versions support and handle custom properties differ:
	/// 		https://developer.mozilla.org/en-US/docs/Web/API/XMLHttpRequest
	/// 		Full cross browser support is therefore not guaranteed.
	/// 	</description>
	/// 	<param name="propertyObject" type="object" default="undefined"> If specified, properties will be applied to XHR request when Start() is invoked </param>
	/// </function>
	this.RequestProperties = function(propertyObject)
	{
		Fit.Validation.ExpectObject(propertyObject, true);

		if (Fit.Validation.IsSet(propertyObject) === true)
		{
			customProperties = propertyObject;
		}

		return customProperties;
	}

	// Protected

	this._internal =
	{
		FireOnStateChange: function()
		{
			Fit.Array.ForEach(onStateChange, function(handler) { handler(me); });
		},

		FireOnSuccess: function()
		{
			Fit.Array.ForEach(onSuccessHandlers, function(handler) { handler(me); });
		},

		FireOnFailure: function()
		{
			Fit.Array.ForEach(onFailureHandlers, function(handler) { handler(me); });
		},

		FireOnAbort: function()
		{
			Fit.Array.ForEach(onAbortHandlers, function(handler) { handler(me); });
		},

		FireOnRequest: function()
		{
			var cancel = false;

			Fit.Array.ForEach(onRequestHandlers, function(handler)
			{
				if (handler(me) === false)
					cancel = true;
			});

			return cancel;
		},
	}

	// Private

	function getHttpRequestObject()
	{
		if (window.XMLHttpRequest) // Firefox, IE7, Chrome, Opera, Safari
			return new XMLHttpRequest();
		else if (window.ActiveXObject) // IE5, IE6
			return new ActiveXObject("Microsoft.XMLHTTP");

		throw new Error("Http Request object not supported");
	}

	function parseKeyValuePairs(formDataStr) // formDataStr = key1=val1&name=james&keyOnly&more=data
	{
		Fit.Validation.ExpectString(formDataStr);

		if (formDataStr === "")
			return {};

		var obj = {};
		var entries = formDataStr.split("&");

		Fit.Array.ForEach(entries, function(entry)
		{
			var kv = entry.split("=");
			obj[kv[0]] = (kv.length > 1 ? kv[1] : "");
		});

		return obj;
	}

	function getFormDataString(encode) // Returns form data in query string format: key1=val1&name=james&keyOnly=&more=data
	{
		Fit.Validation.ExpectBoolean(encode);

		var formDataStr = "";

		Fit.Array.ForEach((formData !== null ? formData : {}), function(key)
		{
			formDataStr += (formDataStr !== "" ? "&" : "") + key + "=" + (encode === true && formData[key].Encode === true ? encodeURIComponent(formData[key].Value) : formData[key].Value);
		});

		return formDataStr;
	}

	function ensureContentTypeHeaderForFormData() // Add content-type header for Form Data if content-type header has not already been defined
	{
		if (me.GetHeader("content-type") === null)
		{
			// https://www.w3.org/Protocols/rfc2616/rfc2616-sec7.html#sec7.2.1
			me.AddHeader("content-type", "application/x-www-form-urlencoded");
			customHeaders["content-type"]._ensured = true;
		}
	}

	init();
}

/// <container name="Fit.Http.JsonRequest" extends="Fit.Http.Request">
/// 	Asynchronous HTTP request functionality (AJAX/WebService)
/// 	optimized for exchanging data with the server in JSON format.
/// 	Extending from Fit.Http.Request.
///
/// 	// Example code
///
/// 	var http = new Fit.Http.JsonRequest(&quot;WebService.asmx/AddUser&quot;);
///
/// 	http.SetData({ Username: &quot;Jack&quot;, Password: &quot;Secret&quot; });
/// 	http.OnSuccess(function(sender)
/// 	{
/// 		&#160;&#160;&#160;&#160; var json = http.GetResponseJson();
/// 		&#160;&#160;&#160;&#160; alert(&quot;User created - server response: &quot; + json.Message);
/// 	});
///
/// 	http.Start();
/// </container>

/// <function container="Fit.Http.JsonRequest" name="JsonRequest" access="public">
/// 	<description>
/// 		Constructor - creates instance of JSON Request class.
/// 	</description>
/// 	<param name="url" type="string">
/// 		URL to request, e.g.
/// 		http://server/_layouts/15/Company/MyWebService.asmx/MyMethod
/// 	</param>
/// </function>
Fit.Http.JsonRequest = function(url)
{
	Fit.Validation.ExpectStringValue(url);
	Fit.Core.Extend(this, Fit.Http.Request).Apply(url);

	var me = this;

	function init()
	{
		me.AddHeader("Content-Type", "application/json; charset=UTF-8");
	}

	this.AddFormData = function(key, value, uriEncode)
	{
		Fit.Validation.ThrowError("Use SetData(..) to set JSON request data for JSON WebService");
	}

	/// <function container="Fit.Http.JsonRequest" name="GetResponseJson" access="public" returns="object | null">
	/// 	<description>
	/// 		Returns result from request as JSON object, Null if no response was returned.
	/// 		Return value will only be as expected if GetCurrentState() returns a value of 4
	/// 		(request done) and GetHttpStatus() returns a value of 2xx (request successful).
	/// 		Notice: .NET usually wraps data in a .d property. Data is automatically extracted
	/// 		from this property, hence contained data is returned as the root object.
	/// 	</description>
	/// </function>
	this.GetResponseJson = Fit.Core.CreateOverride(this.GetResponseJson, function()
	{
		var resp = base();

		if (url.toLowerCase().indexOf(".asmx/") !== -1 && resp && resp.d)
			resp = resp.d; // Extract .NET response data

		return resp;
	});

	init();
}

/// <container name="Fit.Http.JsonpRequest">
/// 	JSONP (JSON with Padding) request functionality allowing for cross-domain data exchange.
///
/// 	// Example code
///
/// 	var http = new Fit.Http.JsonpRequest(&quot;GetUsers.php&quot;);
/// 	http.OnSuccess(function(r)
/// 	{
/// 		&#160;&#160;&#160;&#160; var data = http.GetResponse();
/// 		&#160;&#160;&#160;&#160; console.log(data);
/// 	});
/// 	http.Start();
/// </container>

/// <function container="Fit.Http.JsonpRequest" name="JsonpRequest" access="public">
/// 	<description> Constructor - creates instance of JsonpRequest class </description>
/// 	<param name="uri" type="string"> URL to request </param>
/// 	<param name="jsonpCallbackName" type="string" default="callback">
/// 		Name of URL parameter receiving name of JSONP callback function
/// 	</param>
/// </function>
Fit.Http.JsonpRequest = function(uri, jsonpCallbackName)
{
	Fit.Validation.ExpectStringValue(uri);
	Fit.Validation.ExpectStringValue(jsonpCallbackName, true);

	var me = this;
	var url = uri;
	var cbKey = ((Fit.Validation.IsSet(jsonpCallbackName) === true) ? jsonpCallbackName : "callback");
	var cbId = null;
	var data = {};
	var timeout = 30000;
	var timer = -1;
	var response = null;
	var aborted = false;

	var onRequestHandlers = [];
	var onSuccessHandlers = [];
	var onTimeoutHandlers = [];

	/// <function container="Fit.Http.JsonpRequest" name="Url" access="public" returns="string">
	/// 	<description> Get/set request URL </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes request URL to specified value </param>
	/// </function>
	this.Url = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			url = val;
		}

		return url;
	}

	/// <function container="Fit.Http.JsonpRequest" name="Callback" access="public" returns="string">
	/// 	<description> Get/set name of URL parameter receiving the name of the JSONP callback function </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes the name of the URL parameter to specified value </param>
	/// </function>
	this.Callback = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			cbKey = val;
		}

		return cbKey;
	}

	/// <function container="Fit.Http.JsonpRequest" name="Timeout" access="public" returns="integer">
	/// 	<description>
	/// 		Get/set request timeout in milliseconds.
	/// 		OnTimeout is fired if response is not received
	/// 		within specified amount of time. Defaults to 30000 ms.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, changes timeout to specified value </param>
	/// </function>
	this.Timeout = function(val)
	{
		Fit.Validation.ExpectInteger(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			timeout = val;
		}

		return timeout;
	}

	/// <function container="Fit.Http.JsonpRequest" name="Abort" access="public">
	/// 	<description> Abort request </description>
	/// </function>
	this.Abort = function()
	{
		aborted = true;
	}

	/// <function container="Fit.Http.JsonpRequest" name="SetParameter" access="public">
	/// 	<description> Set URL parameter </description>
	/// 	<param name="key" type="string"> URL parameter key </param>
	/// 	<param name="value" type="string"> URL parameter value </param>
	/// 	<param name="uriEncode" type="boolean" default="true">
	/// 		Set False to prevent value from being URI encoded to preserve special characters
	/// 	</param>
	/// </function>
	this.SetParameter = function(key, value, uriEncode)
	{
		Fit.Validation.ExpectStringValue(key);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(uriEncode, true);

		data[key] = ((uriEncode === false) ? value : encodeURIComponent(value).replace(/%20/g, "+"));
	}

	/// <function container="Fit.Http.JsonpRequest" name="GetParameter" access="public" returns="string | null">
	/// 	<description> Get URL parameter value - returns Null if parameter is not defined </description>
	/// 	<param name="key" type="string"> URL parameter key </param>
	/// </function>
	this.GetParameter = function(key)
	{
		Fit.Validation.ExpectString(key);

		var val = data[key];
		return ((val !== undefined) ? val : null);
	}

	/// <function container="Fit.Http.JsonpRequest" name="GetParameters" access="public" returns="string[]">
	/// 	<description> Get all URL parameter keys </description>
	/// </function>
	this.GetParameters = function()
	{
		var keys = [];

		Fit.Array.ForEach(data, function(key)
		{
			Fit.Array.Add(keys, key);
		});

		return keys;
	}

	/// <function container="Fit.Http.JsonpRequest" name="Start" access="public">
	/// 	<description> Invoke asynchroneus request </description>
	/// </function>
	this.Start = function()
	{
		aborted = false;

		// Fire OnRequest handlers

		if (fireEvent(onRequestHandlers) === false)
			return;

		response = null;

		// Handle situation where Start() is invoked multiple times before initial request is done processing.
		// Behave like Fit.Http.Request which cancels previous request.

		if (timer !== -1)
		{
			clearTimeout(timer);
			timer = -1;

			// Prevent OnSuccess handlers from executing if data is received after being canceled.
			// Request is not actually canceled, only ignored.
			// NOTICE: Do not remove callback - it would cause a JavaScript error if the situation described above occures.
			Fit._internal.Http.JsonpRequest.Callbacks[cbId] = function(response) { };
		}

		// Register callback

		cbId = "CB" + Fit.Data.CreateGuid(false);

		Fit._internal.Http.JsonpRequest.Callbacks[cbId] = function(resp)
		{
			clearTimeout(timer);
			timer = -1;

			response = resp;
			delete Fit._internal.Http.JsonpRequest.Callbacks[cbId];

			if (aborted === false)
			{
				fireEvent(onSuccessHandlers);
			}
		}

		// Configure timeout

		timer = setTimeout(function()
		{
			timer = -1;

			// Prevent OnSuccess handlers from executing if data is received after timeout is reached.
			// NOTICE: Do not remove callback - it would cause a JavaScript error if the situation described above occures.
			Fit._internal.Http.JsonpRequest.Callbacks[cbId] = function(response) { };

			if (aborted === false)
			{
				fireEvent(onTimeoutHandlers);
			}
		}, timeout);

		// Construct request URL

		var newUrl = url;
		var cbName = "Fit._internal.Http.JsonpRequest.Callbacks." + cbId;

		newUrl += ((newUrl.indexOf("?") === -1) ? "?" : "&") + cbKey + "=" + cbName;

		Fit.Array.ForEach(data, function(key)
		{
			newUrl += "&" + key + "=" + data[key];
		});

		// Inject script to invoke JSONP request

		var script = document.createElement("script");
		script.type = "text/javascript";
		script.src = newUrl;
		document.getElementsByTagName("head")[0].appendChild(script);
	}

	/// <function container="Fit.Http.JsonpRequest" name="GetResponse" access="public" returns="object">
	/// 	<description> Get response returned from server </description>
	/// </function>
	this.GetResponse = function()
	{
		return response;
	}

	// Events

	/// <function container="Fit.Http.JsonpRequestTypeDefs" name="EventHandler">
	/// 	<description> JsonpRequest event handler </description>
	/// 	<param name="sender" type="Fit.Http.JsonpRequest"> Instance of JsonpRequest which triggered event </param>
	/// </function>

	/// <function container="Fit.Http.JsonpRequest" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add function to invoke when request is initiated.
	/// 		Request can be canceled by returning False.
	/// 	</description>
	/// 	<param name="func" type="Fit.Http.JsonpRequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request is initiated.
	/// 		Fit.Http.JsonpRequest instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnRequest = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onRequestHandlers, func);
	}

	/// <function container="Fit.Http.JsonpRequest" name="OnSuccess" access="public">
	/// 	<description> Add function to invoke when request is successful </description>
	/// 	<param name="func" type="Fit.Http.JsonpRequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request finished successfully.
	/// 		Fit.Http.JsonpRequest instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnSuccess = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onSuccessHandlers, func);
	}

	/// <function container="Fit.Http.JsonpRequest" name="OnTimeout" access="public">
	/// 	<description> Add function to invoke when request is unsuccessful due to timeout </description>
	/// 	<param name="func" type="Fit.Http.JsonpRequestTypeDefs.EventHandler">
	/// 		JavaScript function invoked when request timed out.
	/// 		Fit.Http.JsonpRequest instance is passed to function.
	/// 	</param>
	/// </function>
	this.OnTimeout = function(func)
	{
		Fit.Validation.ExpectFunction(func);
		Fit.Array.Add(onTimeoutHandlers, func);
	}

	function fireEvent(handlers)
	{
		var canceled = false; // Only OnRequest can be canceled!

		Fit.Array.ForEach(handlers, function(handler)
		{
			if (handler(me) === false)
				canceled = true;
		});

		return !canceled;
	}
}

Fit.Http.DotNetJsonRequest = Fit.Http.JsonRequest;

Fit.Internationalization = { _internal: { Locale: "en", Locales: {}, Additions: {}, OnChangeHandlers: [] } };

// ====================================================
// Locales
// ====================================================

Fit.Internationalization._internal.AddSystemLocalization = function(locales) // May be called multiple times, e.g. if splitting languages up into several files
{
	Fit.Validation.ExpectObject(locales);

	// Make sure all locales inherit from "en", and make sure country specific
	// overrides (e.g. de_AT) inherits from their primary locale (e.g. de).

	Fit.Array.ForEach(locales, function(localeKey)
	{
		var key = localeKey.toLowerCase();
		var locale = null;

		if (key !== "en") // E.g. da or de_AT
		{
			// Merge everything from "en" to current locale

			var english = locales["en"] || Fit.Internationalization._internal.Locales["en"] || null;

			if (english === null)
				Fit.Validation.ThrowError("Unexpected error - English not defined in System Locales");

			locale = Fit.Core.Merge(english, locales[localeKey]);
		}
		else // en
		{
			locale = locales[localeKey];
		}

		// For country specific locales, merge everything from their parent (e.g. de_AT inherits from de)
		if (key.indexOf("_") !== -1) // E.g. de_AT
		{
			// Merge e.g. de_AT with de

			var primaryLangKey = key.split("_")[0]; // E.g. de_AT => de
			var primaryLocale = locales[primaryLangKey] || Fit.Internationalization._internal.Locales[primaryLangKey] || null;

			if (primaryLocale === null)
				Fit.Validation.ThrowError("Unexpected error - '" + primaryLangKey + "' not defined in System Locales");

			locale = Fit.Core.Merge(primaryLocale, locales[localeKey]);
		}

		Fit.Internationalization._internal.Locales[key] = locale;
	});
}

/// <function container="Fit.Internationalization" name="GetSystemLocale" access="public" static="true" returns="object">
/// 	<description>
/// 		Get locale object such as:
/// 		{
/// 		&#8239;&#8239;&#8239;&#8239; Formatting:
/// 		&#8239;&#8239;&#8239;&#8239; {
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; DecimalSeparator: '.',
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; ThousandsSeparator: ',',
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; DateFormat: 'MM/DD/YYYY',
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; TimeFormat: 'hh:mm',
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; TimeFormatLong: 'hh:mm:ss',
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; ClockHours: 12 // 24 or 12 (AM/PM)
/// 		&#8239;&#8239;&#8239;&#8239; },
/// 		&#8239;&#8239;&#8239;&#8239; Translations:
/// 		&#8239;&#8239;&#8239;&#8239; {
/// 		&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239;&#8239; Required: 'Field is required'
/// 		&#8239;&#8239;&#8239;&#8239; }
/// 		}
/// 	</description>
/// 	<param name="localeKey" type="string" default="undefined">
/// 		If defined, specified locale such as en, en_GB, or da is returned. If not
/// 		found, en (en_US) is returned. If omitted, current locale is returned.
/// 	</param>
/// </function>
Fit.Internationalization.GetSystemLocale = function(localeKey)
{
	Fit.Validation.ExpectString(localeKey, true);

	var locale = localeKey ? localeKey.toLowerCase() : Fit.Internationalization._internal.Locale;

	// Get locale

	var locales = Fit.Internationalization._internal.Locales;
	var found = locales[locale] || null;

	// In case e.g. de_AT is loaded, but not found, try to load de instead

	if (found === null && locale.indexOf("_") !== -1) // E.g. de_AT
	{
		var key = locale.split("_")[0]; // E.g. de_AT => de
		found = locales[key] || null;
	}

	// If specified locale is not found then fall back to English

	if (found === null)
	{
		found = locales["en"] || null; // English MUST exist!
	}

	if (found === null)
	{
		Fit.Validation.ThrowError("Unexpected error - English not defined in System Locales");
	}

	return Fit.Core.Clone(found); // Clone to avoid changes
}

/// <function container="Fit.Internationalization" name="Locale" access="public" static="true" returns="string">
/// 	<description>
/// 		Get/set active locale. Value returned is a lower cased string such as
/// 		&quot;en&quot;, &quot;en_us&quot;, &quot;de&quot;, etc. Changing locale results in OnLocaleChanged being fired.
/// 	</description>
/// 	<param name="locale" type="string" default="undefined"> If defined, locale is updated with specified value (e.g. en, en_GB, da, etc.) </param>
/// </function>
Fit.Internationalization.Locale = function(locale)
{
	Fit.Validation.ExpectString(locale, true);

	if (Fit.Validation.IsSet(locale) === true)
	{
		var key = locale.toLowerCase();

		if (Fit.Core.IsEqual(Fit.Internationalization.GetSystemLocale(key), Fit.Internationalization.GetSystemLocale()) === false)
		{
			Fit.Internationalization._internal.Locale = key;

			Fit.Array.ForEach(Fit.Array.Copy(Fit.Internationalization._internal.OnChangeHandlers), function(onChangeHandler)
			{
				if (onChangeHandler._fitUiInternationalizationRemoved === undefined)
				{
					onChangeHandler();
				}
			});
		}
	}

	return Fit.Internationalization._internal.Locale;
}

// ====================================================
// Localization - internationalization of apps
// ====================================================

/// <function container="Fit.Internationalization" name="AddLocalization" access="public" static="true">
/// 	<description>
/// 		Register information such as translations related to a specific language and country.
/// 	</description>
/// 	<param name="type" type="function">
/// 		Object type associated with localization information - e.g. MyApp.ContactForm
/// 	</param>
/// 	<param name="translations" type="Object">
/// 		Object array containing language and country specific information such as translations.
/// 		The object array must be indexed using locale keys, and &quot;en&quot; must be defined first.
/// 		Example: { &quot;en&quot;: {}, &quot;en_GB&quot;: {}, &quot;da&quot;: {} }
/// 		Every language inherits all the information from &quot;en&quot;, and country specific information
/// 		such as &quot;de_AT&quot; automatically inherits everything from &quot;de&quot; - in which case
/// 		&quot;de&quot; must be declared first.
/// 		Information can be obtained from within instances of the given type using:
/// 		Fit.Internationalization.GetLocale(this); // Translations for current locale
/// 		Fit.Internationalization.GetLocale(this, &quot;de_AT&quot;); // Translations for specific locale
/// 		Naturally &quot;this&quot; is an instance of MyApp.ContactForm in this example.
/// 	</param>
/// </function>
Fit.Internationalization.AddLocalization = function(type, translations) // May be called multiple times, e.g. if splitting languages up into several files
{
	Fit.Validation.IsSet(type);
	Fit.Validation.ExpectObject(translations);

	// Assign ID to type which can be used to resolve translations later

	type._internal = type._internal || {};
	type._internal.LocalizationKey = type._internal.LocalizationKey || Fit.Data.CreateGuid();
	var typeId = type._internal.LocalizationKey;

	// Register translations

	Fit.Internationalization._internal.Additions[typeId] = Fit.Internationalization._internal.Additions[typeId] || {};
	var existingTranslations = Fit.Internationalization._internal.Additions[typeId];

	Fit.Array.ForEach(translations, function(key)
	{
		var localeKey = key.toLowerCase();
		var translationSet = translations[key];

		if (localeKey !== "en")
		{
			// Not primary english - merge with en which MUST exist

			var english = translations["en"] || existingTranslations["en"] || null;

			if (english === null)
				Fit.Validation.ThrowError("Unexpected error - English (en) not defined in locales");

			translationSet = Fit.Core.Merge(english, translationSet);
		}

		if (localeKey.indexOf("_") !== -1) // E.g. de_AT
		{
			// Merge e.g. de_AT with de

			var primaryLangKey = localeKey.split("_")[0];
			var primaryLocale = translations[primaryLangKey] || existingTranslations[primaryLangKey] || null;

			if (primaryLocale === null)
				Fit.Validation.ThrowError("Unexpected error - '" + primaryLangKey + "' not defined in locales");

			translationSet = Fit.Core.Merge(primaryLocale, translationSet);
		}

		existingTranslations[localeKey] = translationSet;
	});
}

/// <function container="Fit.Internationalization" name="GetLocale" access="public" static="true" returns="object">
/// 	<description> Get type specific locale information registered using Fit.Internationalization.AddLocalization(..) </description>
/// 	<param name="instance" type="object"> Instance of type used to register locale information </param>
/// 	<param name="locale" type="string" default="undefined">
/// 		If defined, information for specified locale such as en, en_GB, or da is returned.
/// 		If not found, en (en_US) is returned. If omitted, information for current locale is returned.
/// 	</param>
/// </function>
Fit.Internationalization.GetLocale = function(instance, locale)
{
	Fit.Validation.ExpectIsSet(instance);
	Fit.Validation.ExpectString(locale, true);

	var found = null;

	if (instance.constructor._internal && instance.constructor._internal.LocalizationKey)
	{
		var langKey = (locale ? locale.toLowerCase() : null) || Fit.Internationalization._internal.Locale;

		found = Fit.Internationalization._internal.Additions[instance.constructor._internal.LocalizationKey][langKey] || null;

		if (found === null && langKey.indexOf("_") !== -1) // E.g. de_AT
		{
			var primaryLangKey = langKey.split("_")[0]; // E.g. de_AT => de
			found = Fit.Internationalization._internal.Additions[instance.constructor._internal.LocalizationKey][primaryLangKey] || null;
		}

		if (found === null)
		{
			found = Fit.Internationalization._internal.Additions[instance.constructor._internal.LocalizationKey]["en"] || null;
		}
	}

	return found;
}

/// <function container="Fit.Internationalization" name="OnLocaleChanged" access="public" static="true">
/// 	<description> Add event handler which is called if locale is changed </description>
/// 	<param name="cb" type="function"> Event handler which takes no arguments </param>
/// </function>
Fit.Internationalization.OnLocaleChanged = function(cb)
{
	Fit.Validation.ExpectFunction(cb);
	Fit.Array.Add(Fit.Internationalization._internal.OnChangeHandlers, cb);
	delete cb._fitUiInternationalizationRemoved; // In case it was removed and added again
}

/// <function container="Fit.Internationalization" name="RemoveOnLocaleChanged" access="public" static="true">
/// 	<description> Remove event handler to avoid it being called when locale is changed </description>
/// 	<param name="cb" type="function"> Event handler to remove </param>
/// </function>
Fit.Internationalization.RemoveOnLocaleChanged = function(cb)
{
	Fit.Validation.ExpectFunction(cb);
	Fit.Array.Remove(Fit.Internationalization._internal.OnChangeHandlers, cb);
	cb._fitUiInternationalizationRemoved = true; // In case it is removed while OnLocaleChanged is firing, in which case it should be ignored
}
// ==============================================================
// WARNING: THIS FILE IS OBSOLETE!
// File will be removed in a future version of Fit.UI!
// ==============================================================

Fit.Language = {};
Fit.Language.Translations = {};

// Simply override translation object to change language

// ControlBase
Fit.Language.Translations.Required = "Field is required";

// DropDown
Fit.Language.Translations.InvalidSelection = "Invalid selection";

// FilePicker
Fit.Language.Translations.SelectFile = "Select file";
Fit.Language.Translations.SelectFiles = "Select file(s)";

// Dialog
Fit.Language.Translations.Ok = "OK";
Fit.Language.Translations.Cancel = "Cancel";

// The order of processing scripts and stylesheets:
// http://www.html5rocks.com/en/tutorials/internals/howbrowserswork/#The_order_of_processing_scripts_and_style_sheets

/// <container name="Fit.Loader">
/// 	Loader is a useful mechanism for loading styleheets and JavaScript on demand in a non blocking manner.
/// </container>
Fit.Loader = {};

/// <function container="Fit.LoaderTypeDefs" name="LoadSingleEventHandler">
/// 	<description> Callback invoked when resource is loaded </description>
/// 	<param name="src" type="string"> Resource loaded </param>
/// </function>

/// <function container="Fit.Loader" name="ExecuteScript" access="public" static="true">
/// 	<description>
/// 		Load client script on demand in a non-blocking manner.
///
/// 		ExecuteScript differs from LoadScript in the way code is loaded
/// 		and executed. ExecuteScript loads the script using AJAX, meaning
/// 		the script file either has to be located on the same domain as
/// 		the application invoking ExecuteScript, or have the
/// 		Access-Control-Allow-Origin header configured to allow file to be
/// 		loaded from a foreign domain.
/// 		The code loaded is injected into the virtual machine meaning debugging
/// 		will not reveal from which file the code originated, although this
/// 		information can be retrived by inspecting the call stack.
/// 		ExecuteScript has the advantages that is allows for both an OnSuccess
/// 		and OnFailure handler, and it allows for objects to be passed to
/// 		the executing script using the context parameter, which will
/// 		cause the code to execute within its own contained scope (closure).
/// 		The script will execute in the global scope if no context object is provided.
///
/// 		// Example of loading a JavaScript file, passing variables to it:
///
/// 		Fit.Loader.ExecuteScript(&quot;extensions/test/test.js&quot;, function(src)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;JavaScript &quot; + src + &quot; loaded and ready to be used!&quot;);
/// 		},
/// 		function(src)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;JavaScript &quot; + src + &quot; could NOT be loaded;);
///
/// 		}, { $: window.jQuery, mode: &quot;advanced&quot;, showData: &quot;users&quot; });
/// 	</description>
/// 	<param name="src" type="string"> Script source (path or URL) </param>
/// 	<param name="onSuccess" type="Fit.LoaderTypeDefs.LoadSingleEventHandler" default="undefined">
/// 		Callback function fired when script has been successfully loaded and executed.
/// 		The function takes the script source requested as an argument.
/// 	</param>
/// 	<param name="onFailure" type="Fit.LoaderTypeDefs.LoadSingleEventHandler" default="undefined">
/// 		Callback function fired if script could not be loaded or executed successfully.
/// 		The function takes the script source requested as an argument.
/// 	</param>
/// 	<param name="context" type="object" default="undefined">
/// 		Properties registered on the context object will be
/// 		exposed as globally accessible objects for the executing script.
/// 		Example: { jQuery: window.jQuery, $: window.jQuery }
/// 		The script will execute in the global window scope if no
/// 		context object is defined.
/// 	</param>
/// </function>
Fit.Loader.ExecuteScript = function(src, onSuccess, onFailure, context)
{
	// Loading JS using AJAX and Eval versus a traditional script block
	// like Fit.Loader.LoadScript(..) uses:
	// The approach with AJAX allows us to expose both OnSuccess and
	// OnFailure, even on legacy browsers. However, if a script fails
	// or need debugging, it is not immediately obvious what file is
	// being debugged since the code runs as "anonymous" code in the VM.
	// One would have to investigate the call stack to reveal the source file.
	// Also, the AJAX request will not allow scripts from remote locations
	// to be loaded, unless the Access-Control-Allow-Origin header is configured.
	// The best thing about using eval however, is the possibility of passing
	// variables to the file being loaded using the context object.

	Fit.Validation.ExpectStringValue(src);
	Fit.Validation.ExpectFunction(onSuccess, true);
	Fit.Validation.ExpectFunction(onFailure, true);

	var r = new Fit.Http.Request(src);
	r.OnSuccess(function(sender)
	{
		var js = r.GetResponseText();
		var error = null;

		if (Fit.Validation.IsSet(context) === false || context === window)
		{
			try
			{
				eval(js);
			}
			catch (err)
			{
				error = err;
			}
		}
		else
		{
			// Execute in closure (execution scope).
			// All variables registered on context object is made globally available to JS file.

			var args = "";
			var parms = "";
			for (var prop in (context ? context : {}))
			{
				args += (args !== "" ? ", " : "") + prop;
				parms += (parms !== "" ? ", " : "") + "this." + prop;
			}

			try
			{
				eval("(function() {  return (function() { (function(" + args + ") { " + js + " })(" + parms + "); });  })();").call(context ? context : this); // https://jsfiddle.net/yut8fptm/
			}
			catch (err)
			{
				error = err;
			}
		}

		if (error === null && Fit.Validation.IsSet(onSuccess) === true)
		{
			onSuccess(src);
		}
		else if (error !== null)
		{
			if (window.console)
			{
				console.log(error.message);
				console.log(error.stack);
				console.log(error);
			}

			if (Fit.Validation.IsSet(onFailure) === true)
				onFailure(src);
		}
	});
	r.OnFailure(function(sender)
	{
		if (Fit.Validation.IsSet(onFailure) === true)
		{
			onFailure(src);
		}
	});
	r.Start();
}

/// <function container="Fit.Loader" name="LoadScript" access="public" static="true">
/// 	<description>
/// 		Load client script on demand in a non-blocking manner.
///
/// 		// Example of loading a JavaScript file
///
/// 		Fit.Loader.LoadScript(&quot;extensions/test/test.js&quot;, function(src)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;JavaScript &quot; + src + &quot; loaded and ready to be used!&quot;);
/// 		});
/// 	</description>
/// 	<param name="source" type="string"> Script source (path or URL) </param>
/// 	<param name="cb" type="Fit.LoaderTypeDefs.LoadSingleEventHandler" default="undefined">
/// 		Callback function fired when script loading is complete - takes the script source requested as an argument.
/// 		Be aware that a load error will also trigger the callback to make sure control is always returned.
/// 		Consider using feature detection within callback function for most reliable execution - example:
/// 		if (expectedObjectOrFunction) { /* Successfully loaded, continue.. */ }
/// 	</param>
/// </function>
/// <function container="Fit.Loader" name="LoadScript" access="public" static="true">
/// 	<description>
/// 		Load client script on demand in a non-blocking manner.
///
/// 		// Example of loading a JavaScript file
///
/// 		Fit.Loader.LoadScript(&quot;extensions/test/test.js&quot;, { id: &quot;my-script&quot;, charset: &quot;UTF-8&quot; }, function(src)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;JavaScript &quot; + src + &quot; loaded and ready to be used!&quot;);
/// 		});
/// 	</description>
/// 	<param name="source" type="string"> Script source (path or URL) </param>
/// 	<param name="attributes" type="{[key:string]: string}"> Attributes registered with script block </param>
/// 	<param name="cb" type="Fit.LoaderTypeDefs.LoadSingleEventHandler" default="undefined">
/// 		Callback function fired when script loading is complete - takes the script source requested as an argument.
/// 		Be aware that a load error will also trigger the callback to make sure control is always returned.
/// 		Consider using feature detection within callback function for most reliable execution - example:
/// 		if (expectedObjectOrFunction) { /* Successfully loaded, continue.. */ }
/// 	</param>
/// </function>
Fit.Loader.LoadScript = function(source, cb)
{
	var src = null;
	var attributes = null;
	var callback = null;

	if (arguments.length === 1 || (arguments.length === 2 && typeof(arguments[1]) === "function"))
	{
		src = arguments[0];
		callback = arguments[1] || null;
	}
	else if (arguments.length >= 2)
	{
		src = arguments[0];
		attributes = arguments[1] || null;
		callback = arguments[2] || null;
	}

	Fit.Validation.ExpectStringValue(src);
	Fit.Validation.ExpectDictionary(attributes, Fit.Validation.ExpectString, true);
	Fit.Validation.ExpectFunction(callback, true);

	// if (arguments.length > 2)
	// {
	// 	// One could easily confuse LoadScript with ExecuteScript - help the developer to choose the right function
	// 	Fit.Validation.ThrowError("Argument(s) do not match function signature - did you intend to use Fit.Loader.ExecuteScript instead ?");
	// }

	// Scripts injected always load as if they had the async attribute
	// set, so we will not cause the parser to stall using this approach.
	// It is completely non-blocking while the script loads.
	// https://developer.mozilla.org/en/docs/Web/HTML/Element/script

	var script = document.createElement("script");
	script.type = "text/javascript";
	script.charset = "UTF-8";

	if (attributes !== null)
	{
		Fit.Array.ForEach(attributes, function(key)
		{
			script.setAttribute(key, attributes[key]);
		});
	}

	if (callback !== null && (Fit.Browser.GetBrowser() !== "MSIE" || (Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() >= 9)))
	{
		script.onload = function() { callback(src); };

		// Terrible, but we need same behaviour for all browsers, and IE8 (and below) does not distinguish between success and failure.
		// Also, we need to make sure control is returned no matter what - just like using ordinary <script src=".."> elements which
		// doesn't halt execution on 404 or syntax errors.
		script.onerror = function() { callback(src); };
	}
	else if (callback !== null && Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() <= 8)
	{
		script.onreadystatechange = function()
		{
			if (this.readyState === "complete" || this.readyState === "loaded") // loaded = initial load, complete = from cache
				callback(src);
		}
	}

	script.src = src;

	// NOTICE: Scripts loaded this way will NOT be able to reliably resolve their own
	// script block using something like this: document.scripts[document.scripts.length - 1].
	// The reason is of couse that scripts are loaded async., and multiple script blocks
	// may have been defined after LoadScript(..) was initially invoked.

	document.getElementsByTagName("head")[0].appendChild(script);
}

/// <container name="Fit.LoaderTypeDefs.ResourceConfiguration">
/// 	<description> Resource configuration </description>
/// 	<member name="source" type="string"> Path to resource </member>
/// 	<member name="attributes" type="{[key:string]: string}" default="undefined"> Attributes registered with script block </member>
/// 	<member name="loaded" type="Fit.LoaderTypeDefs.LoadSingleConfigurationEventHandler" default="undefined">
/// 		Callback invoked when resource is loaded
/// 	</member>
/// </container>

/// <function container="Fit.LoaderTypeDefs" name="LoadSingleConfigurationEventHandler">
/// 	<description> Callback invoked when resource is loaded </description>
/// 	<param name="cfg" type="Fit.LoaderTypeDefs.ResourceConfiguration"> Resource loaded </param>
/// </function>

/// <function container="Fit.LoaderTypeDefs" name="LoadMultiConfigurationsEventHandler">
/// 	<description> Callback invoked when all resources are loaded </description>
/// 	<param name="cfgs" type="Fit.LoaderTypeDefs.ResourceConfiguration[]"> Resources loaded </param>
/// </function>

/// <function container="Fit.Loader" name="LoadScripts" access="public" static="true">
/// 	<description>
/// 		Chain load multiple client scripts on demand in a non-blocking manner.
///
/// 		// Example of loading multiple JavaScript files in serial:
///
/// 		Fit.Loader.LoadScripts(
/// 		[
/// 			&#160;&#160;&#160;&#160; {
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; source: &quot;extensions/test/menu.js&quot;,
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; attributes: { id: &quot;my-script&quot; },
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; loaded: function(cfg) { alert(&quot;JavaScript &quot; + cfg.source + &quot; loaded&quot;); }
/// 			&#160;&#160;&#160;&#160; },
/// 			&#160;&#160;&#160;&#160; {
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; source: &quot;http://cdn.domain.com/chat.js&quot;,
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; attributes: { id: &quot;another-script&quot; },
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; loaded: function(cfg) { alert(&quot;JavaScript &quot; + cfg.source + &quot; loaded&quot;); }
/// 			&#160;&#160;&#160;&#160; }
/// 		],
/// 		function(cfgs)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;All files loaded&quot;);
/// 		});
///
/// 		First argument is an array of script configurations:
/// 		source:string (required): Script source (path or URL)
/// 		loaded:function (optional): Callback function to execute when file has loaded (takes file configuration as argument)
/// 		Be aware that loaded callback is invoked even if a load error occures, to make sure control is returned to your code.
///
/// 		Second argument is the callback function fired when all files have finished loading - takes configuration array as argument.
/// 		This too may be invoked even if a load error occured, to make sure control is returned to your code.
///
/// 		Consider using feature detection within callback functions for super reliable execution - example:
/// 		if (expectedObjectOrFunction) { /* Successfully loaded, continue.. */ }
/// 	</description>
/// 	<param name="cfg" type="Fit.LoaderTypeDefs.ResourceConfiguration[]"> Configuration array (see function description for details) </param>
/// 	<param name="callback" type="Fit.LoaderTypeDefs.LoadMultiConfigurationsEventHandler" default="undefined">
/// 		Callback function fired when all scripts have finished loading (see function description for details)
/// 	</param>
/// </function>
Fit.Loader.LoadScripts = function(cfg, callback, skipValidation)
{
	Fit.Validation.ExpectArray(cfg);
	Fit.Validation.ExpectFunction(callback, true);
	Fit.Validation.ExpectBoolean(skipValidation, true);

	// Verify configuration

	if (skipValidation !== true)
	{
		for (var i = 0 ; i < cfg.length ; i++)
		{
			Fit.Validation.ExpectStringValue(cfg[i].source);
			Fit.Validation.ExpectDictionary(cfg[i].attributes, Fit.Validation.ExpectString, true);
			Fit.Validation.ExpectFunction(cfg[i].loaded, true);
		}
	}

	// Find next unhandled script to load

	var toLoad = null;

	for (var i = 0 ; i < cfg.length ; i++)
	{
		if (cfg[i].handled !== true)
		{
			toLoad = cfg[i];
			break;
		}
	}

	// Break out if no more scripts need handling

	if (toLoad === null)
	{
		if (Fit.Validation.IsSet(callback) === true)
			callback(cfg);

		return;
	}

	// Load script

	toLoad.handled = true;

	Fit.Loader.LoadScript(toLoad.source, toLoad.attributes || {}, function()
	{
		if (Fit.Validation.IsSet(toLoad.loaded) === true)
		{
			try // Use try/catch to prevent buggy code from stopping the chain
			{
				toLoad.loaded(toLoad);
			}
			catch (err)
			{
				if (window.console)
				{
					console.log(err.message);
					console.log(err.stack);
					console.log(err);
				}
			}
		}

		// Continue chain - load next script from configuration

		Fit.Loader.LoadScripts(cfg, callback, true);
	});
}

/// <function container="Fit.Loader" name="LoadStyleSheet" access="public" static="true">
/// 	<description>
/// 		Load CSS stylesheet on demand in a non-blocking manner.
/// 		It is recommended to load stylesheets before rendering items using
/// 		the CSS classes to avoid FOUC (Flash Of Unstyled Content).
///
/// 		// Example of loading a CSS file
///
/// 		Fit.Loader.LoadStyleSheet(&quot;extensions/test/layout.css&quot;, function(src)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;CSS file &quot; + src + &quot; loaded!&quot;);
/// 		});
/// 	</description>
/// 	<param name="src" type="string"> CSS file source (path or URL) </param>
/// 	<param name="callback" type="Fit.LoaderTypeDefs.LoadSingleEventHandler" default="undefined">
/// 		Callback function fired when CSS file loading is complete - takes the file source requested as an argument.
/// 		Be aware that a load error will also trigger the callback to make sure control is always returned.
/// 	</param>
/// </function>
Fit.Loader.LoadStyleSheet = function(src, callback)
{
	Fit.Validation.ExpectStringValue(src);
	Fit.Validation.ExpectFunction(callback, true);

	// OnError event could likely be supported using the following
	// lines of code which allows us to check the number of loaded CSS rules:
	// W3C browsers: var success = (cssLinkNode.sheet.cssRules.length > 0);
	// Internet Explorer: var success = (cssLinkNode.styleSheet.rules.length > 0);
	// For consistency this approach is not currently being used - we need same
	// behaviour for both LoadStyleSheet(..), and LoadScript(..) which doesn't support OnError.

	var link = document.createElement("link");
	link.type = "text/css";
	link.rel = "stylesheet";

	if (Fit.Validation.IsSet(callback) === true && (Fit.Browser.GetBrowser() !== "MSIE" || (Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() >= 9)))
	{
		link.onload = function() { callback(src); };
		link.onerror = function() { callback(src); }; // Same behaviour as LoadScript(..)
	}
	else if (Fit.Validation.IsSet(callback) === true && Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() <= 8)
	{
		link.onreadystatechange = function()
		{
			if (this.readyState === "complete" || this.readyState === "loaded") // loaded = initial load, complete = from cache
				callback(src);
		}
	}

	link.href = src;
	document.getElementsByTagName("head")[0].appendChild(link);
}

/// <function container="Fit.Loader" name="LoadStyleSheets" access="public" static="true">
/// 	<description>
/// 		Load multiple stylesheets in parrallel in a non-blocking manner.
///
/// 		// Example of loading multiple CSS files:
///
/// 		Fit.Loader.LoadStyleSheets(
/// 		[
/// 			&#160;&#160;&#160;&#160; {
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; source: &quot;extensions/test/menu.css&quot;,
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; loaded: function(cfg) { alert(&quot;Stylesheet &quot; + cfg.source + &quot; loaded&quot;); }
/// 			&#160;&#160;&#160;&#160; },
/// 			&#160;&#160;&#160;&#160; {
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; source: &quot;http://cdn.domain.com/chat.css&quot;,
/// 			&#160;&#160;&#160;&#160; &#160;&#160;&#160;&#160; loaded: function(cfg) { alert(&quot;Stylesheet &quot; + cfg.source + &quot; loaded&quot;); }
/// 			&#160;&#160;&#160;&#160; }
/// 		],
/// 		function(cfgs)
/// 		{
/// 			&#160;&#160;&#160;&#160; alert(&quot;All stylesheets loaded&quot;);
/// 		});
///
/// 		First argument is an array of stylesheet configurations:
/// 		source:string (required): Stylesheet source (path or URL)
/// 		loaded:function (optional): Callback function to execute when stylesheet has loaded (takes stylesheet configuration as argument)
/// 		Be aware that loaded callback is invoked even if a load error occures, to make sure control is returned to your code.
///
/// 		Second argument is the callback function fired when all stylesheets have finished loading - takes configuration array as argument.
/// 		This too may be invoked even if a load error occured, to make sure control is returned to your code.
/// 	</description>
/// 	<param name="cfg" type="Fit.LoaderTypeDefs.ResourceConfiguration[]"> Configuration array (see function description for details) </param>
/// 	<param name="callback" type="Fit.LoaderTypeDefs.LoadMultiConfigurationsEventHandler" default="undefined"> Callback function fired when all stylesheets have finished loading (see function description for details) </param>
/// </function>
Fit.Loader.LoadStyleSheets = function(cfg, callback)
{
	Fit.Validation.ExpectArray(cfg);
	Fit.Validation.ExpectFunction(callback, true);

	// Verify configuration

	for (var i = 0 ; i < cfg.length ; i++)
	{
		Fit.Validation.ExpectStringValue(cfg[i].source);
		Fit.Validation.ExpectFunction(cfg[i].loaded, true);
	}

	// Invoke callback if nothing to load

	if (cfg.length === 0)
	{
		if (Fit.Validation.IsSet(callback) === true)
			callback(cfg);

		return;
	}

	// Batch load all stylesheets

	for (var i = 0 ; i < cfg.length ; i++)
	{
		// Load stylesheet

		Fit.Loader.LoadStyleSheet(cfg[i].source, function(src)
		{
			// Fire stylesheet callback function when completed

			for (var j = 0 ; j < cfg.length ; j++)
			{
				if (cfg[j].source === src)
				{
					cfg[j].handled = true;

					if (Fit.Validation.IsSet(cfg[j].loaded) === true)
					{
						try // Use try/catch to make sure a buggy callback function does not prevent "all completed" callback to be reached
						{
							cfg[j].loaded(cfg[j]);
						}
						catch (err)
						{
							if (window.console)
							{
								console.log(err.message);
								console.log(err.stack);
								console.log(err);
							}
						}
					}

					break;
				}
			}

			// Fire "all completed" callback if all stylesheets have finished loading

			for (var j = 0 ; j < cfg.length ; j++)
			{
				if (cfg[j].handled !== true)
					return;
			}

			if (Fit.Validation.IsSet(callback) === true)
				callback(cfg);
		});
	}
}

/// <container name="Fit.Template">
/// 	<![CDATA[
/// 	Templating engine allowing for separation between layout and logic.
///
/// 	// Example code
///
/// 	// Load template (asynchronously)
/// 	var tpl = new Fit.Template(true);
/// 	tpl.LoadUrl(&quot;UserListView.html&quot;, function(sender)
/// 	{
/// 		&#160;&#160;&#160;&#160; // Populate placeholders
/// 		&#160;&#160;&#160;&#160; tpl.Content.Title = &quot;User list&quot;;
/// 		&#160;&#160;&#160;&#160; tpl.Content.Description = &quot;List of users created in system&quot;;
///
/// 		&#160;&#160;&#160;&#160; // Load user data
/// 		&#160;&#160;&#160;&#160; var users = getUsers();
///
/// 		&#160;&#160;&#160;&#160; // Populate user list with data
/// 		&#160;&#160;&#160;&#160; Fit.Array.ForEach(users, function(userData)
/// 		&#160;&#160;&#160;&#160; {
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; var user = tpl.Content.Users.AddItem();
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; user.Name = userData.Name;
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; user.Role = userData.Role;
/// 		&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; user.Email = userData.Email;
/// 		&#160;&#160;&#160;&#160; });
///
/// 		&#160;&#160;&#160;&#160; // Push changes to DOM
/// 		&#160;&#160;&#160;&#160; tpl.Update();
/// 	});
/// 	tpl.Render(document.getElementById("UserList"));
///
/// 	// HTML template example (UserListView.html)
///
/// 	&lt;h1&gt;{[Title]}&lt;/h1&gt;
/// 	&lt;p&gt;{[Description]}&lt;/p&gt;
/// 	&lt;table&gt;
/// 	&#160;&#160;&#160;&#160; &lt;tr&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;&lt;b&gt;User ID&lt;/b&gt;&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;&lt;b&gt;Name&lt;/b&gt;&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;&lt;b&gt;E-mail&lt;/b&gt;&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;&lt;b&gt;Department&lt;/b&gt;&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160; &lt;/tr&gt;
/// 	&#160;&#160;&#160;&#160; &lt;!-- LIST Users --&gt;
/// 	&#160;&#160;&#160;&#160; &lt;tr&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;{[Name]}&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;{[Role]}&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; &lt;td&gt;{[Email]}&lt;/td&gt;
/// 	&#160;&#160;&#160;&#160; &lt;/tr&gt;
/// 	&#160;&#160;&#160;&#160; &lt;!-- /LIST Users --&gt;
/// 	&lt;/table&gt;
/// 	]]>
/// </container>

/// <function container="Fit.Template" name="Template" access="public">
/// 	<description> Constructor - creates instance of Template class </description>
/// 	<param name="refreshable" type="boolean" default="false">
/// 		Flag indicating whether template can be updated after being rendered
/// 		to DOM. A value of True results in template being wrapped in a div container
/// 		controlled by the templating system.
/// 	</param>
/// 	<param name="autoDispose" type="boolean" default="true">
/// 		Flag indicating whether Fit.UI controls should be automatically disposed
/// 		when removed from view. Controls are disposed once changes are pushed to
/// 		DOM using the Update() function.
/// 	</param>
/// </function>
Fit.Template = function(refreshable, autoDispose) // http://fiddle.jshell.net/5sb97qtn/28/  --  http://fiddle.jshell.net/3rbq1r13/3/
{
	Fit.Validation.ExpectBoolean(refreshable, true);
	Fit.Validation.ExpectBoolean(autoDispose, true);

	var me = this;
	var allowUnsafe = -1;		// -1 = default behaviour (see code) which may change in the future, 0 = protect against code injection, 1 = allow code injection
	var unsafeWarned = false;
	var htmlContent = "";
	var container = null;
	var pendingElements = [];	// Holds references to all DOMElements to be rendered: { Id:string, Element:DOMElements }
	var nodesRendered = [];		// Holds references to all DOMElements rendered to DOM
	var eventHandlers = []; 	// Holds references to all event handler functions associated with DOM elements: { ElementId:string, EventType:string, Callback:function }
	var controls = [];			// Holds references to all Fit.UI controls (DOMElements) rendered to DOM

	function init()
	{
		if (refreshable === true)
			container = document.createElement("div");
	}

	/// <member container="Fit.Template" name="Content" access="public" type="object">
	/// 	<description>
	/// 		Once template is loaded, any placeholder or list will be accessible
	/// 		through the Content property. A placeholder identified as UserRole
	/// 		will be accessible as templateInstance.Content.UserRole.
	/// 		UserRole is a property that can be set with either a string,
	/// 		an instance of a Fit.UI control, or a DOMElement.
	/// 		A list identified as Users is accessible using
	/// 		templateInstance.Content.Users. See Fit.TemplateList for
	/// 		additional information.
	/// 	</description>
	/// </member>
	this.Content = null;

	/// <function container="Fit.Template" name="LoadHtml" access="public">
	/// 	<description> Load HTML string into template </description>
	/// 	<param name="htmlSource" type="string"> HTML string </param>
	/// </function>
	this.LoadHtml = function(htmlSource)
	{
		Fit.Validation.ExpectString(htmlSource);
		parse(htmlSource);
	}

	/// <function container="Fit.TemplateTypeDefs" name="LoadUrlCallback">
	/// 	<description> Callback invoked when template is loaded </description>
	/// 	<param name="sender" type="Fit.Template"> Instance of Template </param>
	/// 	<param name="html" type="string"> Template's HTML content </param>
	/// </function>

	/// <function container="Fit.Template" name="LoadUrl" access="public">
	/// 	<description> Load HTML from URL asynchronously </description>
	/// 	<param name="url" type="string"> URL to HTML template </param>
	/// 	<param name="cb" type="Fit.TemplateTypeDefs.LoadUrlCallback">
	/// 		Callback function invoked once HTML template has
	/// 		been loaded. Template instance and raw HTML data
	/// 		is passed to function. At this point placeholders
	/// 		and lists are accessible from the Content propery.
	/// 	</param>
	/// </function>
	this.LoadUrl = function(url, cb)
	{
		Fit.Validation.ExpectString(url);
		Fit.Validation.ExpectFunction(cb);

		var r = new Fit.Http.Request(url);
		r.OnSuccess(function(sender)
		{
			parse(r.GetResponseText());
			cb(me, r.GetResponseText());
		});
		r.Start();
	}

	/// <function container="Fit.Template" name="AllowUnsafeContent" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set flag indicating whether unsafe string values are handled or not.
	/// 		If AllowUnsafeContent is True, arbitrary code can be added to the template
	/// 		and will be intepreted by the browser. If AllowUnsafeContent is False,
	/// 		potentially unsafe code will be encoded and displayed as is without interpretation.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, handling of string encoding is changed to reflect value </param>
	/// </function>
	this.AllowUnsafeContent = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			allowUnsafe = (val === true ? 1 : 0);
		}

		return (allowUnsafe !== 0);
	}

	/// <function container="Fit.Template" name="Reset" access="public">
	/// 	<description> Resets template - all data and event handlers are removed. Change is not pushed to DOM automatically. </description>
	/// </function>
	this.Reset = function()
	{
		// Do NOT clear e.g. 'nodesRendered' or 'controls' as they are needed if template is disposed.
		// The purpose of this function is to clear data added externally, not internal state.

		parse(htmlContent); // Re-initializes Content property
		eventHandlers = [];
	}

	/// <function container="Fit.Template" name="Dispose" access="public">
	/// 	<description> Dispose template to free up memory </description>
	/// </function>
	this.Dispose = function()
	{
		if (autoDispose !== false)
		{
			Fit.Array.ForEach(controls, function(controlElm)
			{
				// Dispose control unless manually disposed by external code in which case the _internal property no longer exists
				if (controlElm._internal !== undefined)
				{
					controlElm._internal.Instance.Dispose();
				}
			});
		}

		if (container !== null)
		{
			Fit.Dom.Remove(container);
		}
		else
		{
			Fit.Array.ForEach(nodesRendered, function(node)
			{
				Fit.Dom.Remove(node);
			});
		}

		this.Content = null;
		me = htmlContent = container = pendingElements = nodesRendered = eventHandlers = controls = null;
	}

	/// <function container="Fit.Template" name="GetHtml" access="public" returns="string">
	/// 	<description> Get HTML from template including any changes made </description>
	/// </function>
	this.GetHtml = function()
	{
		return me.toString();
	}

	/// <function container="Fit.Template" name="Render" access="public">
	/// 	<description> Render template, either inline or to element specified </description>
	/// 	<param name="toElement" type="DOMElement" default="undefined"> If defined, template is rendered to this element </param>
	/// </function>
	this.Render = function(toElement)
	{
		Fit.Validation.ExpectDomElement(toElement, true);

		if (nodesRendered.length > 0)
			return; // Skip - already rendered - Update() should be used to push changes!

		// Turn Template into DOM element

		var html = me.toString(); // Also populates 'pendingElements' array containing DOMElements to be added further down
		var dom = Fit.Dom.CreateElement("<div>" + html + "</div>");

		// Inject DOMElements

		Fit.Array.ForEach(pendingElements, function(elm)
		{
			var element = dom.querySelector("var.FitTemplate[id='PH" + elm.Id + "']");

			if (element !== null) // Null if element was added with non-existing placeholder key
			{
				Fit.Dom.Replace(element, elm.Element);
			}

			if (elm.Element._internal !== undefined && elm.Element._internal.Instance !== undefined) // Fit.UI control inheriting from Fit.Controls.Component which is disposable
			{
				Fit.Array.Add(controls, elm.Element);
			}
		});

		Fit.Array.Clear(pendingElements); // No longer needed - do not create new object - it will break references to collection on lists

		// Register event handlers

		Fit.Array.ForEach(eventHandlers, function(ev)
		{
			if (ev === null) // Event handler was removed
				return;

			var elm = dom.querySelector("#" + ev.ElementId);

			if (elm !== null)
				Fit.Events.AddHandler(elm, ev.EventType, ev.Callback);
		});

		// Render to DOM

		var renderTarget = ((container !== null) ? container : toElement);
		var nodes = Fit.Array.Copy(dom.childNodes); // Copy to prevent "Collection was modified" error when rendering elements, which moves them to a new parent

		nodesRendered = [];

		if (Fit.Validation.IsSet(renderTarget) === true)
		{
			Fit.Array.ForEach(nodes, function(n)
			{
				Fit.Dom.Add(renderTarget, n);
				Fit.Array.Add(nodesRendered, n);
			});
		}
		else
		{
			var script = document.scripts[document.scripts.length - 1];

			Fit.Array.ForEach(nodes, function(n)
			{
				Fit.Dom.InsertBefore(script, n);
				Fit.Array.Add(nodesRendered, n);
			});
		}

		if (container !== null)
		{
			if (Fit.Validation.IsSet(toElement) === true)
			{
				Fit.Dom.Add(toElement, container);
			}
			else
			{
				var script = document.scripts[document.scripts.length - 1];
				Fit.Dom.InsertBefore(script, container);
			}
		}
	}

	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"keydown" | "keyup" | "keypress"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackKeyboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"click" | "contextmenu" | "dblclick" | "mousedown" | "mouseenter" | "mouseleave" | "mousemove" | "mouseout" | "mouseover" | "mouseup" | "mousewheel"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackMouse"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"popstate"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackPopState"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"hashchange"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackHashChange"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"focus" | "focusin" | "focusout" | "blur"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackFocus"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"beforeunload"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackBeforeUnload"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"cut" | "copy" | "paste"'> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackClipboard"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type='"#rooted"'> Event name </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackRooted"> JavaScript function to register </param>
	/// </function>
	/// <function container="Fit.Template" name="AddEventHandler" access="public" returns="integer">
	/// 	<description> Add event handler to element within template given by element ID - event handler ID is returned </description>
	/// 	<param name="elmId" type="string"> Element ID </param>
	/// 	<param name="event" type="string"> Event name without the 'on' prefix </param>
	/// 	<param name="eventFunction" type="Fit.EventsTypeDefs.EventHandlerCallbackGeneric"> JavaScript function to register </param>
	/// </function>
	this.AddEventHandler = function(elmId, event, eventFunction)
	{
		Fit.Validation.ExpectStringValue(elmId);
		Fit.Validation.ExpectStringValue(event);
		Fit.Validation.ExpectFunction(eventFunction);

		Fit.Array.Add(eventHandlers, { ElementId: elmId, EventType: event, Callback: eventFunction });
		return (eventHandlers.length - 1);
	}

	/// <function container="Fit.Template" name="RemoveEventHandler" access="public">
	/// 	<description> Remove event handler by event ID returned by AddEventHandler(..) </description>
	/// 	<param name="eventId" type="integer"> Event ID </param>
	/// </function>
	this.RemoveEventHandler = function(eventId)
	{
		Fit.Validation.ExpectInteger(eventId);

		if (eventHandlers.length - 1 >= eventId)
			eventHandlers[eventId] = null;
	}

	/// <function container="Fit.Template" name="Update" access="public">
	/// 	<description> Push changes to DOM to make them visible through the user interface </description>
	/// </function>
	this.Update = function()
	{
		if (container === null)
			throw "UpdateTemplateError: Template has not been configured to allow updates";

		// Turn Template into DOM element

		var html = me.toString(); // Also populates 'pendingElements' array containing DOMElements to be added further down
		var dom = Fit.Dom.CreateElement("<div>" + html + "</div>");

		// Inject DOMElements

		var oldControls = controls;
		controls = [];

		Fit.Array.ForEach(pendingElements, function(elm)
		{
			var element = dom.querySelector("var.FitTemplate[id='PH" + elm.Id + "']");

			if (element !== null) // Null if element was added with non-existing placeholder key
			{
				Fit.Dom.Replace(element, elm.Element);
			}

			if (elm.Element._internal !== undefined && elm.Element._internal.Instance !== undefined) // Fit.UI control inheriting from Fit.Controls.Component which is disposable
			{
				Fit.Array.Add(controls, elm.Element);
			}
		});

		Fit.Array.Clear(pendingElements); // No longer needed - do not create new object - it will break references to collection on lists

		// Auto dispose controls previously added to template if now left out

		if (autoDispose !== false)
		{
			Fit.Array.ForEach(oldControls, function(oldControl)
			{
				// Dispose control if no longer found in view, unless manually disposed by external code in which case the _internal property no longer exists
				if (Fit.Array.Contains(controls, oldControl) === false && oldControl._internal !== undefined)
				{
					oldControl._internal.Instance.Dispose();
				}
			});
		}

		// Register event handlers

		Fit.Array.ForEach(eventHandlers, function(ev)
		{
			if (ev === null) // Event handler was removed
				return;

			var elm = dom.querySelector("#" + ev.ElementId);

			if (elm !== null)
				Fit.Events.AddHandler(elm, ev.EventType, ev.Callback);
		});

		// Clear view

		container.innerHTML = "";

		// Render to DOM

		var nodes = Fit.Array.Copy(dom.childNodes); // Copy to prevent "Collection was modified" error when rendering elements, which moves them to a new parent
		nodesRendered = [];

		Fit.Array.ForEach(nodes, function(n)
		{
			Fit.Dom.Add(container, n);
			Fit.Array.Add(nodesRendered, n);
		});
	}

	this.toString = function()
	{
		// Turn populated template into HTML string.
		// DOMElements added to template are added to DOM later - they are temporarily represented using <var> elements.
		// Corresponding, event handlers are registered later when template is rendered to real DOM.

		if (allowUnsafe === -1 && unsafeWarned === false)
		{
			unsafeWarned = true;
			Fit.Browser.Log("WARNING: An instance of Fit.Template allows for unsafe content which could potentially lead to code injection (XSS attacks). The default behaviour for handling unsafe content will change in the future from allowing unsafe content to NOT allowing unsafe content which may break this application. Please explicitely set AllowUnsafeContent(bool) to ensure compatibility with future versions of Fit.UI.");
		}

		var newHtml = htmlContent;
		Fit.Array.Clear(pendingElements); // Do not create new object - it will break references to collection on lists

		Fit.Array.ForEach(me.Content, function(key)
		{
			var obj = me.Content[key];

			if (!obj)
				return;

			if (obj._internal !== undefined && obj._internal.IsFitTemplate === true) // List
			{
				newHtml = newHtml.replace(obj._internal.Block, obj.toString());
			}
			else
			{
				if (Fit.Core.InstanceOf(obj, Fit.Controls.Component) === true) // Fit.UI Control
				{
					var id = Fit.Data.CreateGuid();
					Fit.Array.Add(pendingElements, { Id: id, Element: obj.GetDomElement() });

					newHtml = newHtml.replace("{[" + key + "]}", "<var class='FitTemplate' id='PH" + id + "'></var>");
				}
				else if (typeof(obj) === "object" && (obj instanceof Element || obj instanceof Text)) // DOM
				{
					var id = Fit.Data.CreateGuid();
					Fit.Array.Add(pendingElements, { Id: id, Element: obj });

					// Notice: Placeholders to be replaced by DOM elements should only
					// be declared once - a DOM element cannot be added multiple times!
					// Placeholders for string values can on the other hand be declared multiple times.
					newHtml = newHtml.replace("{[" + key + "]}", "<var class='FitTemplate' id='PH" + id + "'></var>");
				}
				else // String value
				{
					var sVal = (me.AllowUnsafeContent() === true ? obj.toString() : Fit.String.EncodeHtml(obj.toString()));
					newHtml = newHtml.replace(new RegExp("{\\[" + key + "\\]}", "g"), sVal);
				}
			}
		});

		// Remove placeholders that have not been replaced
		newHtml = newHtml.replace(/\{\[(\S+?)\]\}/g, "");

		return newHtml;
	}

	function parse(sourceHtml)
	{
		Fit.Validation.ExpectString(sourceHtml);

		htmlContent = sourceHtml;

		// Parse lists

		var result = {};

		var lists = handleLists(htmlContent);
		Fit.Array.ForEach(lists, function(l)
		{
			result[l] = lists[l];
		});

		// Parse placeholders

		var placeHolders = handlePlaceHolders(htmlContent);
		Fit.Array.ForEach(placeHolders, function(ph)
		{
			result[ph] = placeHolders[ph];
		});

		// Expose lists and placeholders

		me.Content = result;
	}

	function handleLists(html)
	{
		Fit.Validation.ExpectString(html);

		var regEx = /<!-- LIST (\S+) -->[\S\s]*<!-- \/LIST \1 -->/g; // https://regex101.com/r/uL4vM9/2
		var match;

		var res = {};

		while ((match = regEx.exec(html)))
		{
			res[match[1]] = createListObject(match[1], match[0]); // 0 = HTML, 1 = List name
		}

		return res;
	}

	function handlePlaceHolders(html)
	{
		Fit.Validation.ExpectString(html);

		// Remove lists and their associated placeholders.
		// We only want to extract the global placeholders,
		// not the placeholders within lists.

		var regExLists = /<!-- LIST (\S+) -->[\S\s]*<!-- \/LIST \1 -->/g;
		html = html.replace(regExLists, "");

		// Parse global placeholders

		var regEx = /\{\[(\S+?)\]\}/g; // https://regex101.com/r/aM5cV4/1
		var match;
		var res = {};

		while ((match = regEx.exec(html)))
		{
			res[match[1]] = null; // String or DOMNode
		}

		return res;
	}

	function createListObject(name, html)
	{
		Fit.Validation.ExpectString(name);
		Fit.Validation.ExpectString(html);

		// The design of the List object is not ideal.
		// Each time an item is added, the object structure representing
		// that item is parsed from HTML. Caching the parsed item and cloning it
		// the next time AddItem is called will not work since functions
		// on the 'res' object is not cloned - they keep pointing to the original
		// functions, hence they will continue using the orignal 'res' object
		// part of their closure.
		// This can be fixed by referring to 'this' rather than 'res' within the
		// functions, but we also need to "fix" clones which holds a reference to
		// 'pendingElements' on the template instance, and needs to keep this reference
		// rather than having its own copy.
		// Another issue is that the List object depends heavily on
		// functionality on the Template object (e.g. handlePlaceHolders(..)
		// and handleLists(..)). It all works - and works well - but it is a bit
		// difficult to maintain and understand, and could potentially perform better
		// if designed differently, although such performance gain would only be
		// noticable on pretty large and complex applications which is why refactoring
		// is not prioritized at the moment.

		/// <container name="Fit.TemplateList">
		/// 	A template list is a dynamically created object representing
		/// 	a variable number of items containing placeholders. An example
		/// 	could be a list of Users containing information such as Name,
		/// 	Phone number, and E-mail address about each user.
		/// </container>

		var res = { _internal: {} };
		res._internal.Id = name;
		res._internal.Block = html;
		res._internal.Html = html.replace("<!-- LIST " + name + " -->", "").replace("<!-- /LIST " + name + " -->", "");
		res._internal.Items = [];
		res._internal.PendingElements = pendingElements;
		res._internal.IsFitTemplate = true;

		/// <function container="Fit.TemplateList" name="AddItem" access="public" returns="object">
		/// 	<description>
		/// 		Create list item represented by associative object
		/// 		array containing either strings or DOMElements.
		/// 		Example:
		/// 		var item = templateInstance.Content.MyList.AddItem();
		/// 		item[&quot;Name&quot;] = &quot;James Thompson&quot;;
		/// 		item[&quot;Email&quot;] = &quot;james@server.com&quot;;
		/// 		It is also possible to assign data using properties:
		/// 		item.ProfilePicture = document.createElement(&quot;img&quot;);
		/// 		item.ProfilePicture.src = &quot;james.png&quot;;
		/// 	</description>
		/// </function>
		res.AddItem = function()
		{
			var item = handlePlaceHolders(res._internal.Html);

			var nestedLists = handleLists(res._internal.Html);
			Fit.Array.ForEach(nestedLists, function(l)
			{
				item[l] = nestedLists[l];
			});

			Fit.Array.Add(res._internal.Items, item);
			return item;
		}

		/// <function container="Fit.TemplateList" name="GetItems" access="public" returns="object[]">
		/// 	<description> Get all list items added using AddItem() </description>
		/// </function>
		res.GetItems = function()
		{
			return res._internal.Items;
		}

		/// <function container="Fit.TemplateList" name="RemoveItem" access="public">
		/// 	<description> Remove list item (associative object array) </description>
		/// 	<param name="item" type="object"> Item to remove </param>
		/// </function>
		res.RemoveItem = function(item)
		{
			Fit.Array.Remove(res._internal.Items, item);
		}

		/// <function container="Fit.TemplateList" name="Clear" access="public">
		/// 	<description> Clear list </description>
		/// </function>
		res.Clear = function()
		{
			Fit.Array.Clear(res._internal.Items);
		}

		res.toString = function()
		{
			var listHtml = "";

			function populateItem(itemHtml, item)
			{
				Fit.Array.ForEach(item, function(prop)
				{
					var obj = item[prop];

					if (!obj)
						return;

					if (obj._internal !== undefined && obj._internal.IsFitTemplate === true) // Nested list
					{
						var allSubItems = "";

						Fit.Array.ForEach(obj._internal.Items, function(subItem)
						{
							allSubItems += populateItem(obj._internal.Html, subItem);
						});

						itemHtml = itemHtml.replace(obj._internal.Block, allSubItems);
					}
					else if (Fit.Core.InstanceOf(obj, Fit.Controls.Component) === true) // Fit.UI Control
					{
						var id = Fit.Data.CreateGuid();
						Fit.Array.Add(res._internal.PendingElements, { Id: id, Element: obj.GetDomElement() });

						// Notice: Placeholders to be replaced by DOM elements should only
						// be declared once - a DOM element cannot be added multiple times!
						// Theoretically placeholders for string values could be used multiple times, but
						// for the sake of consistency we only replace the first occurrence in this case too.
						itemHtml = itemHtml.replace("{[" + prop + "]}", "<var class='FitTemplate' id='PH" + id + "'></var>");
					}
					else if (typeof(obj) === "object" && (obj instanceof Element || obj instanceof Text)) // DOM
					{
						var id = Fit.Data.CreateGuid();
						Fit.Array.Add(res._internal.PendingElements, { Id: id, Element: obj });

						// Notice: Placeholders to be replaced by DOM elements should only
						// be declared once - a DOM element cannot be added multiple times!
						// Theoretically placeholders for string values could be used multiple times, but
						// for the sake of consistency we only replace the first occurrence in this case too.
						itemHtml = itemHtml.replace("{[" + prop + "]}", "<var class='FitTemplate' id='PH" + id + "'></var>");
					}
					else // String value
					{
						var sVal = (me.AllowUnsafeContent() === true ? obj.toString() : Fit.String.EncodeHtml(obj.toString()));
						itemHtml = itemHtml.replace(new RegExp("{\\[" + prop + "\\]}", "g"), sVal);
					}
				});

				return itemHtml;
			}

			var itmHtml = res._internal.Html;

			Fit.Array.ForEach(res._internal.Items, function(item)
			{
				listHtml += populateItem(itmHtml, item);
			});

			return listHtml;
		}

		return res;
	}

	init();
}

// ======================================
// Common type defintions
// ======================================

Fit.TypeDefs = {};

/// <container name="Fit.TypeDefs.Position">
/// 	<description> Position of a visual object from top and left </description>
/// 	<member name="X" type="integer"> Position from left in pixels </member>
/// 	<member name="Y" type="integer"> Position from top in pixels </member>
/// </container>

/// <container name="Fit.TypeDefs.Dimension">
/// 	<description> Represents the size of a visual object </description>
/// 	<member name="Width" type="integer"> Object width in pixels </member>
/// 	<member name="Height" type="integer"> Object height in pixels </member>
/// </container>

/// <container name="Fit.TypeDefs.CssUnit">
/// 	<description> Represents a CSS unit </description>
/// 	<member name="cm" access="public" static="true" type="string" default="cm"> </member>
/// 	<member name="mm" access="public" static="true" type="string" default="mm"> </member>
/// 	<member name="in" access="public" static="true" type="string" default="in"> </member>
/// 	<member name="px" access="public" static="true" type="string" default="px"> </member>
/// 	<member name="pt" access="public" static="true" type="string" default="pt"> </member>
/// 	<member name="pc" access="public" static="true" type="string" default="pc"> </member>
/// 	<member name="em" access="public" static="true" type="string" default="em"> </member>
/// 	<member name="ex" access="public" static="true" type="string" default="ex"> </member>
/// 	<member name="ch" access="public" static="true" type="string" default="ch"> </member>
/// 	<member name="rem" access="public" static="true" type="string" default="rem"> </member>
/// 	<member name="vw" access="public" static="true" type="string" default="vw"> </member>
/// 	<member name="vh" access="public" static="true" type="string" default="vh"> </member>
/// 	<member name="vmin" access="public" static="true" type="string" default="vmin"> </member>
/// 	<member name="vmax" access="public" static="true" type="string" default="vmax"> </member>
/// 	<member name="%" access="public" static="true" type="string" default="%"> </member>
/// </container>
Fit.TypeDefs.CssUnit = // Enums must exist runtime
{
	"cm":"cm", "mm":"mm", "in":"in", "px":"px", "pt":"pt", "pc":"pc", "em":"em", "ex":"ex",
	"ch":"ch", "rem":"rem", "vw":"vw", "vh":"vh", "vmin":"vmin", "vmax":"vmax", "%":"%"
};

/// <container name="Fit.TypeDefs.CssValue">
/// 	<description> Represents a CSS value </description>
/// 	<member name="Value" type="number"> Value </member>
/// 	<member name="Unit" type="Fit.TypeDefs.CssUnit"> Unit </member>
/// </container>

/// <container name="Fit.TypeDefs.ScrollBarInfo">
/// 	<member name="Enabled" type="boolean"> True if scrollbar is enabled, otherwise False </member>
/// 	<member name="Size" type="integer"> Size of scrollbar - always returns 0 if Enabled is False </member>
/// </container>

/// <container name="Fit.TypeDefs.ScrollBarsPresent">
/// 	<member name="Vertical" type="Fit.TypeDefs.ScrollBarInfo"> Vertical scrollbar information </member>
/// 	<member name="Horizontal" type="Fit.TypeDefs.ScrollBarInfo"> Horizontal scrollbar information </member>
/// </container>
Fit.Internationalization._internal.AddSystemLocalization(
{
	// Primary locales must be registered before country specific overrides.
	// Example order: en, en_US, en_GB, de, de_AT, etc.
	// All locales inherit from en. All country specific overrides inherit
	// from their primary locale (e.g. de_AT inherits from de).
	// English (en) MUST be defined!

	"en": // US
	{
		Formatting:
		{
			DecimalSeparator		: ".",
			ThousandsSeparator		: ",",
			DateFormat				: "MM/DD/YYYY",
			TimeFormat				: "hh:mm",
			TimeFormatLong			: "hh:mm:ss",
			ClockHours				: 12 // 24 or 12 (AM/PM)
		},
		Translations:
		{
			Required				: "Field is required"
		}
	},
	"en_GB":
	{
		Formatting:
		{
			DateFormat				: "DD/MM/YYYY"
		}
	},
	"da":
	{
		Formatting:
		{
			DecimalSeparator		: ",",
			ThousandsSeparator		: ".",
			DateFormat				: "DD-MM-YYYY",
			ClockHours				: 24
		},
		Translations:
		{
			Required				: "Feltet er påkrævet"
		}
	},
	"de":
	{
		Formatting:
		{
			DecimalSeparator		: ",",
			ThousandsSeparator		: ".",
			DateFormat				: "DD.MM.YYYY",
			ClockHours				: 24
		},
		Translations:
		{
			Required				: "Feld ist erforderlich"
		}
	},
	"no":
	{
		Formatting:
		{
			DecimalSeparator		: ",",
			ThousandsSeparator		: ".",
			DateFormat				: "DD-MM-YYYY",
			ClockHours				: 24
		},
		Translations:
		{
			Required				: "Feltet er påkrevd"
		}
	}
});

/// <container name="Fit.Controls.Button" extends="Fit.Controls.Component">
/// 	Button control with support for Font Awesome icons
/// </container>

// http://fiddle.jshell.net/05q0tLt6/14/

/// <function container="Fit.Controls.Button" name="Button" access="public">
/// 	<description> Create instance of Button control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.Button = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	// Internals

	var me = this;
	var id = me.GetId();
	var element = me.GetDomElement();
	var wrapper = null;
	var icon = null;
	var label = null;
	var width = { Value: -1, Unit: "px" };	// Initial width - a value of -1 indicates that size adjusts to content
	var height = { Value: -1, Unit: "px" };	// Initial height - a value of -1 indicates that size adjusts to content
	var returnFocus = false;
	var onClickHandlers = [];

	function init()
	{
		var invokeClick = true;
		var focusedBeforeClick = null;

		Fit.Events.AddHandler(element, Fit.Browser.IsTouchEnabled() === true ? "touchstart" : "mousedown", function(e)
		{
			invokeClick = true;
			focusedBeforeClick = returnFocus === true ? Fit.Dom.GetFocused() : null;
		});
		Fit.Browser.IsTouchEnabled() === true && Fit.Events.AddHandler(element, "touchmove", function(e)
		{
			invokeClick = false;
		});
		Fit.Events.AddHandler(element, Fit.Browser.IsTouchEnabled() === true ? "touchend" : "click", function(e)
		{
			if (invokeClick === true && me.Enabled() === true)
				me.Click();

			if (focusedBeforeClick !== null)
			{
				focusedBeforeClick.focus();
				focusedBeforeClick = null;
			}
		});
		Fit.Browser.IsTouchEnabled() === true && Fit.Events.AddHandler(element, "touchcancel", function(e)
		{
			focusedBeforeClick = null;
		});
		Fit.Events.AddHandler(element, "keydown", function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			if (me.Enabled() === true && (ev.keyCode === 13 || ev.keyCode === 32)) // Enter or Spacebar
			{
				me.Click();
				Fit.Events.PreventDefault(ev);
			}
		});

		Fit.Dom.AddClass(element, "FitUiControl");
		Fit.Dom.AddClass(element, "FitUiControlButton");

		wrapper = document.createElement("div");
		Fit.Dom.Add(element, wrapper);

		icon = document.createElement("span");
		Fit.Dom.Add(wrapper, icon);

		label = document.createElement("span");
		Fit.Dom.Add(wrapper, label);

		me.Enabled(true);
		me.Type(Fit.Controls.ButtonType.Default);
	}

	/// <function container="Fit.Controls.Button" name="Title" access="public" returns="string">
	/// 	<description> Get/set button title </description>
	/// 	<param name="val" type="string" default="undefined"> If specified, button title will be set to specified value </param>
	/// </function>
	this.Title = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			Fit.Dom.Data(element, "title", ((val !== "") ? val : null));
			label.innerHTML = val;
		}

		return ((Fit.Dom.Data(element, "title") !== null) ? Fit.Dom.Data(element, "title") : "");
	}

	/// <function container="Fit.Controls.Button" name="Icon" access="public" returns="string">
	/// 	<description> Get/set button icon (Font Awesome icon name, e.g. fa-check-circle-o - https://fontawesome.com/v4.7.0/icons) </description>
	/// 	<param name="val" type="string" default="undefined"> If specified, button icon will be set to specified value </param>
	/// </function>
	this.Icon = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			Fit.Dom.Data(element, "icon", ((val !== "") ? val : null));
			icon.className = ((val !== "") ? "fa " + ((val.indexOf("fa-") !== 0) ? "fa-" : "") + val : "");
		}

		return ((Fit.Dom.Data(element, "icon") !== null) ? Fit.Dom.Data(element, "icon") : "");
	}

	/// <function container="Fit.Controls.Button" name="Type" access="public" returns="Fit.Controls.ButtonType">
	/// 	<description>
	/// 		Get/set button type producing specific look and feel.
	/// 		Possible values are:
	/// 		 - Fit.Controls.ButtonType.Default (white)
	/// 		 - Fit.Controls.ButtonType.Primary (blue)
	/// 		 - Fit.Controls.ButtonType.Success (green)
	/// 		 - Fit.Controls.ButtonType.Info (turquoise)
	/// 		 - Fit.Controls.ButtonType.Warning (orange)
	/// 		 - Fit.Controls.ButtonType.Danger (red)
	/// 	</description>
	/// 	<param name="val" type="Fit.Controls.ButtonType" default="undefined"> If specified, button type will be set to specified value </param>
	/// </function>
	this.Type = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (Fit.Validation.IsSet(Fit.Controls.ButtonType[val]) === false)
				Fit.Validation.ThrowError("Unsupported button type specified - use e.g. Fit.Controls.ButtonType.Default");

			Fit.Dom.Data(element, "type", val);
		}

		return Fit.Dom.Data(element, "type");
	}

	/// <function container="Fit.Controls.Button" name="Enabled" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether button is enabled or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables button, False disables it </param>
	/// </function>
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== me.Enabled())
		{
			Fit.Dom.Data(element, "enabled", ((val === true) ? "true" : "false"));

			if (val === true)
			{
				element.tabIndex = 0;
			}
			else
			{
				me.Focused(false);
				Fit.Dom.Attribute(element, "tabindex", null);
			}

			me._internal.Repaint();
		}

		return (Fit.Dom.Data(element, "enabled") === "true");
	}

	/// <function container="Fit.Controls.Button" name="Focused" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control has focus </description>
	/// 	<param name="focus" type="boolean" default="undefined"> If defined, True assigns focus, False removes focus (blur) </param>
	/// </function>
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (Fit.Validation.IsSet(focus) === true)
		{
			if (focus === true)
				element.focus();
			else
				element.blur();
		}

		// Fit.Dom.Contained(..) portion added to support IE which incorrectly assigns focus to contained elements, even though tabIndex is not set
		return (Fit.Dom.GetFocused() === element || (Fit.Dom.GetFocused() && Fit.Dom.Contained(element, Fit.Dom.GetFocused())));
	}

	/// <function container="Fit.Controls.Button" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control width is updated to specified value. A value of -1 resets control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control width is updated to specified CSS unit </param>
	/// </function>
	this.Width = function(val, unit) // Differs from ControlBase.Width(..) when -1 is passed - this control resets to width:auto while ControlBase resets to width:200px
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				width = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				element.style.width = width.Value + width.Unit;
			}
			else
			{
				width = { Value: -1, Unit: "px" };
				element.style.width = ""; // Notice: width:auto is applied in Button.css
			}
		}

		return width;
	}

	/// <function container="Fit.Controls.Button" name="Height" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control height is updated to specified value. A value of -1 resets control height. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control height is updated to specified CSS unit </param>
	/// </function>
	this.Height = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				height = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				element.style.height = height.Value + height.Unit;

				// Work around bug in WebKit/Chrome:
				// https://code.google.com/p/chromium/issues/detail?id=573715
				// Also see Button.css (div.FitUiControlButton[style*="height"] > div)
				wrapper.style.position = "relative";
			}
			else
			{
				height = { Value: -1, Unit: "px" };
				element.style.height = "";

				// Undo WebKit/Chrome bug fix (see condition above)
				wrapper.style.position = "";
			}
		}

		return height;
	}

	/// <function container="Fit.Controls.Button" name="ReturnFocus" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether button returns focus after click </description>
	/// 	<param name="val" type="boolean" default="undefined">
	/// 		A value of True causes button to return focus to previously focused element after click - defaults to False
	/// 	</param>
	/// </function>
	this.ReturnFocus = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			returnFocus = val;
		}

		return returnFocus;
	}

	/// <function container="Fit.Controls.ButtonTypeDefs" name="ClickEventHandler">
	/// 	<description> OnClick event handler </description>
	/// 	<param name="sender" type="Fit.Controls.Button"> Instance of Button </param>
	/// </function>

	/// <function container="Fit.Controls.Button" name="OnClick" access="public">
	/// 	<description> Set callback function invoked when button is clicked </description>
	/// 	<param name="cb" type="Fit.Controls.ButtonTypeDefs.ClickEventHandler">
	/// 		Callback function invoked when button is clicked - takes button instance as argument
	/// 	</param>
	/// </function>
	this.OnClick = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onClickHandlers, cb);
	}

	/// <function container="Fit.Controls.Button" name="Click" access="public">
	/// 	<description> Programmatically trigger a button click </description>
	/// </function>
	this.Click = function()
	{
		Fit.Array.ForEach(onClickHandlers, function(handler)
		{
			handler(me);
		});
	}

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		me = id = element = wrapper = icon = label = width = height = returnFocus = onClickHandlers = null;
		base();
	});

	init();
}

/// <container name="Fit.Controls.ButtonType">
/// 	Enum values determining visual appearance of button controls
/// </container>
Fit.Controls.ButtonType =
{
	/// <member container="Fit.Controls.ButtonType" name="Default" access="public" static="true" type="string" default="Default">
	/// 	<description> White unless styled differently - default look and feel </description>
	/// </member>
	Default: "Default",

	/// <member container="Fit.Controls.ButtonType" name="Primary" access="public" static="true" type="string" default="Primary">
	/// 	<description> Blue unless styled differently </description>
	/// </member>
	Primary: "Primary",

	/// <member container="Fit.Controls.ButtonType" name="Success" access="public" static="true" type="string" default="Success">
	/// 	<description> Green unless styled differently </description>
	/// </member>
	Success: "Success",

	/// <member container="Fit.Controls.ButtonType" name="Info" access="public" static="true" type="string" default="Info">
	/// 	<description> Turquoise unless styled differently </description>
	/// </member>
	Info: "Info",

	/// <member container="Fit.Controls.ButtonType" name="Warning" access="public" static="true" type="string" default="Warning">
	/// 	<description> Orange unless styled differently </description>
	/// </member>
	Warning: "Warning",

	/// <member container="Fit.Controls.ButtonType" name="Danger" access="public" static="true" type="string" default="Danger">
	/// 	<description> Red unless styled differently </description>
	/// </member>
	Danger: "Danger"
}

Fit.Controls.Button.Type = Fit.Controls.ButtonType; // Backward compatibility

/// <container name="Fit.Controls.CheckBox" extends="Fit.Controls.ControlBase">
/// 	Simple CheckBox control.
/// 	Extending from Fit.Controls.ControlBase.
/// </container>

/// <function container="Fit.Controls.CheckBox" name="CheckBox" access="public">
/// 	<description> Create instance of CheckBox control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.CheckBox = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;
	var checkbox = null;
	var label = null;
	var width = { Value: -1, Unit: "px" };	// Initial width - a value of -1 indicates that size adjusts to content
	var orgChecked = false;
	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);

	// ============================================
	// Init
	// ============================================

	function init()
	{
		me.AddCssClass("FitUiControlCheckBox");
		me.GetDomElement().tabIndex = 0;

		checkbox = document.createElement("div");
		Fit.Dom.AddClass(checkbox, "fa");
		Fit.Dom.AddClass(checkbox, "fa-check");

		Fit.Events.AddHandler(me.GetDomElement(), "click", function(e)
		{
			if (me.Enabled() === true)
			{
				//var orgVal = orgChecked;
				me.Checked(!me.Checked(), true);
				//orgChecked = orgVal;
			}
		});
		Fit.Events.AddHandler(me.GetDomElement(), "keydown", function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			if (me.Enabled() === true && ev.keyCode === 32) // Spacebar
			{
				//var orgVal = orgChecked;
				me.Checked(!me.Checked(), true);
				//orgChecked = orgVal;

				Fit.Events.PreventDefault(ev); // Prevent scroll
			}
		});

		label = document.createElement("span");

		me._internal.AddDomElement(checkbox);
		me._internal.AddDomElement(label);

		me.Enabled(true);
		me.Checked(false);
		me.Width(-1);

		Fit.Internationalization.OnLocaleChanged(localize);
	}

	// ============================================
	// Public - overrides
	// ============================================

	// See documentation on ControlBase
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (Fit.Validation.IsSet(focus) === true)
		{
			if (focus === true)
				me.GetDomElement().focus();
			else
				me.GetDomElement().blur();
		}

		// Fit.Dom.Contained(..) portion added to support IE which incorrectly assigns focus to contained elements, even though tabIndex is not set
		return (Fit.Dom.GetFocused() === me.GetDomElement() || (Fit.Dom.GetFocused() && Fit.Dom.Contained(me.GetDomElement(), Fit.Dom.GetFocused())));
	}

	// See documentation on ControlBase
	this.Value = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			var valStr = val.toLowerCase();

			if (valStr === "true")
			{
				me.Checked(true, preserveDirtyState);
			}
			else if (valStr === "false")
			{
				me.Checked(false, preserveDirtyState);
			}
		}

		return ((me.Checked() === true) ? "true" : "false");
	}

	/// <function container="Fit.Controls.CheckBox" name="Checked" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is checked </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, control's checked state is updated to specified value </param>
	/// 	<param name="preserveDirtyState" type="boolean" default="false">
	/// 		If defined, True prevents dirty state from being reset, False (default) resets the dirty state.
	/// 		If dirty state is reset (default), the control value will be compared against the value passed,
	/// 		to determine whether it has been changed by the user or not, when IsDirty() is called.
	/// 	</param>
	/// </function>
	this.Checked = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			var before = (Fit.Dom.Data(me.GetDomElement(), "checked") === "true");

			Fit.Dom.Data(me.GetDomElement(), "checked", val.toString());

			if (preserveDirtyState !== true)
				orgChecked = (Fit.Dom.Data(me.GetDomElement(), "checked") === "true");

			if (before !== val)
			{
				repaint();
				me._internal.FireOnChange();
			}
		}

		return (Fit.Dom.Data(me.GetDomElement(), "checked") === "true");
	}

	/// <function container="Fit.Controls.CheckBox" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control width is updated to specified value. A value of -1 resets control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control width is updated to specified CSS unit </param>
	/// </function>
	this.Width = function(val, unit) // Differs from ControlBase.Width(..) when -1 is passed - this control resets to width:auto while ControlBase resets to width:200px
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				width = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				me.GetDomElement().style.width = width.Value + width.Unit;
			}
			else
			{
				width = { Value: -1, Unit: "px" };
				me.GetDomElement().style.width = ""; // Notice: width:auto is applied in CheckBox.css
			}
		}

		return width;
	}

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		return (orgChecked !== me.Checked());
	}

	// See documentation on ControlBase
	this.Clear = function()
	{
		me.Checked(false);
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		// This will destroy control - it will no longer work!

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		me = checkbox = label = width = orgChecked = isIe8 = null;

		base();
	});

	// Support for required checkbox - See documentation on ControlBase
	this.IsValid = Fit.Core.CreateOverride(this.IsValid, function()
	{
		if (me.Required() === true && me.Checked() === false)
		{
			return false;
		}

		return base();
	});

	// Support for required checkbox - See documentation on ControlBase
	this._internal.Validate = Fit.Core.CreateOverride(this._internal.Validate, function()
	{
		if (me.Required() === true && me.Checked() === false)
		{
			var locale = Fit.Internationalization.GetSystemLocale();
			me._internal.Data("errormessage", locale.Translations.Required);
		}

		base();
	});

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.CheckBox" name="Label" access="public" returns="string">
	/// 	<description> Get/set label associated with checkbox </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, label is updated to specified value </param>
	/// </function>
	this.Label = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			label.innerHTML = val;
		}

		return label.innerHTML;
	}

	/// <function container="Fit.Controls.CheckBox" name="Enabled" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is enabled or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables control, False disables it </param>
	/// </function>
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== me.Enabled())
		{
			Fit.Dom.Data(me.GetDomElement(), "enabled", val.toString());

			if (val === true)
			{
				me.GetDomElement().tabIndex = 0;
			}
			else
			{
				me.Focused(false);
				Fit.Dom.Attribute(me.GetDomElement(), "tabindex", null);
			}

			me._internal.UpdateInternalState();
			me._internal.Repaint();
		}

		return (Fit.Dom.Data(me.GetDomElement(), "enabled") === "true");
	}

	// ============================================
	// Private
	// ============================================

	function localize()
	{
		me._internal.Validate();
	}

	function repaint()
	{
		if (isIe8 === true)
		{
			// IE8 does not update pseudo elements properly.
			// Changing CSS classes or content within the control
			// is not sufficient - we actually have to remove the
			// control temporarily from the DOM to make it update.
			// Unfortunately this results in focus being lost if
			// control had focus, so we have to restore it as well.

			var focused = Fit.Dom.GetFocused();

			var elm = document.createElement("span");
			Fit.Dom.Replace(checkbox, elm);
			Fit.Dom.Replace(elm, checkbox);

			if (Fit.Dom.GetFocused() !== focused)
				focused.focus();
		}
	}

	init();
}

/// <container name="Fit.Controls.ContextMenu" extends="Fit.Controls.Component">
/// 	ContextMenu control allowing for quick access to select features.
/// </container>

/// <function container="Fit.Controls.ContextMenu" name="ContextMenu" access="public">
/// 	<description> Create instance of ContextMenu control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.ContextMenu = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	var me = this;
	var tree = new Fit.Controls.TreeView("ContextMenuTreeView_" + Fit.Data.CreateGuid());
	var prevFocused = null;
	var detectBoundaries = true;
	var highlightOnInitKeyStroke = true;
	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);

	var onShowing = [];
	var onShown = [];
	var onHide = [];
	var onSelect = [];

	// ============================================
	// Init
	// ============================================

	function init()
	{
		Fit.Dom.Data(tree.GetDomElement(), "keynav", "false");				// True when navigating using keyboard
		Fit.Dom.Data(tree.GetDomElement(), "sticky", "false");				// True when user toggles node
		Fit.Dom.Data(tree.GetDomElement(), "viewportcollision", "false");	// True when context menu collides with viewport boundaries

		tree.AddCssClass("FitUiControlContextMenu");
		tree.Selectable(true);

		// Custom keyboard navigation

		tree.KeyboardNavigation(false);

		Fit.Events.AddHandler(tree.GetDomElement(), "keydown", function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			if (ev.keyCode === 27) // Escape
			{
				// me.Hide(); // Moved below prevFocused.focus() to allow OnHide handlers to reliably react on current focus state

				// Return focus to previously focused element
				if (prevFocused !== null)
				{
					prevFocused.focus();
					prevFocused = null;
				}

				me.Hide();

				return;
			}

			var node = tree.GetNodeFocused();

			if (node === null) // In case context menu has no children
				return;

			if (highlightOnInitKeyStroke === true)
			{
				highlightOnInitKeyStroke = false;
				Fit.Dom.Data(tree.GetDomElement(), "keynav", "true"); // Requires repaint in Legacy IE
				repaint();

				// Make sure the first item is highlighted instead of
				// the 2nd item, in case initial keystroke was arrow down.

				if (ev.keyCode === 40 && tree.GetChildren().length > 0)
				{
					tree.GetChildren()[0].Focused(true);

					Fit.Events.PreventDefault(ev); // Prevent scrolling
					return;
				}
			}

			if (ev.keyCode === 37) // Arrow left
			{
				if (node.Expanded() === true)
				{
					node.Expanded(false);
				}
				else if (node.GetParent() !== null)
				{
					node.GetParent().Focused(true);
				}

				Fit.Events.PreventDefault(ev); // Prevent scrolling
				return;
			}

			if (ev.keyCode === 39) // Arrow right
			{
				if (node.GetChildren().length > 0)
				{
					node.Expanded(true);
					node.GetChildren()[0].Focused(true);
				}

				Fit.Events.PreventDefault(ev); // Prevent scrolling
				return;
			}

			if (ev.keyCode === 38) // Arrow up
			{
				var newNode = tree.GetNodeAbove(node);

				if (newNode !== null)
					newNode.Focused(true);

				Fit.Events.PreventDefault(ev); // Prevent scrolling
				return;
			}

			if (ev.keyCode === 40) // Arrow down
			{
				var newNode = tree.GetNodeBelow(node);

				if (newNode !== null)
					newNode.Focused(true);

				Fit.Events.PreventDefault(ev); // Prevent scrolling
				return;
			}

			if (ev.keyCode === 13) // Enter
			{
				if (node.Selectable() === true)
					node.Selected(true);

				Fit.Events.PreventDefault(ev); // Prevent insertion of a line break in an editor if focus is returned to such OnSelect

				return;
			}
		});

		// Have OnSelect fire when TreeView node is selected

		tree.OnSelected(function(sender, node)
		{
			if (node.Selected() === true) // OnSelected fires when both selecting and deselecting nodes
			{
				node.Selected(false); // Notice: Fires OnSelected again
				node.Expanded(!node.Expanded());

				// Navigate link contained in item, or fire ContextMenu.OnSelect

				var links = node.GetDomElement().getElementsByTagName("a");

				if (links.length === 1 && Fit.Dom.GetParentOfType(links[0], "li") === node.GetDomElement())
				{
					if (links[0] !== Fit.Events.GetPointerState().Target)
						links[0].click();

					me.Hide();
				}
				else
				{
					fireEventHandlers(onSelect, node.GetDomElement()._internal.ContextMenuItem);
				}
			}
		});

		// Support for sticky nodes

		tree.OnToggled(function(sender, node)
		{
			if (node.Expanded() === false) // Node collapsed
			{
				// Collapse all children
				Fit.Array.ForEach(node.GetChildren(), function(c)
				{
					c.Expanded(false); // Notice: Fires OnToggled again if node is expanded, causing all children to be collapsed recursively
				});
			}
			else // Node expanded
			{
				if (node.Focused() === true) // Prevent this from executing when recursively expanding parent nodes (see further down)
				{
					Fit.Dom.Data(tree.GetDomElement(), "sticky", "true");
					highlightOnInitKeyStroke = false;

					// Collapse previously expanded nodes

					var expanded = null;
					var currentNode = node;

					while (currentNode !== null)
					{
						if (currentNode.GetParent() !== null)
							expanded = getExpandedChild(currentNode.GetParent().GetChildren(), currentNode);
						else
							expanded = getExpandedChild(tree.GetChildren(), currentNode);

						if (expanded !== null)
						{
							expanded.Expanded(false); // Notice: Fires OnToggled again, causing all children to be collapsed recursively
							break;
						}

						currentNode = currentNode.GetParent();
					}
				}

				// Expand parent nodes to make the currently hovered hierarchy/path sticky
				if (node.GetParent() !== null)
					node.GetParent().Expanded(true); // Notice: Fires OnToggled again causing all parents to expand recursively

				// Boundary detection - detect and handle viewport collisions
				if (detectBoundaries === true)
					handleViewPortCollision(node.GetDomElement());
			}
		});

		// Boundary detection - detect and handle viewport collisions when hovering items to open submenus

		Fit.Events.AddHandler(tree.GetDomElement(), "mouseover", function(e)
		{
			if (detectBoundaries === true && Fit.Dom.Data(tree.GetDomElement(), "sticky") === "false") // Viewport collision is handled by OnToggled handler when items are sticky
			{
				var elm = Fit.Events.GetTarget(e);
				elm = ((elm.tagName === "LI") ? elm : Fit.Dom.GetParentOfType(elm, "li")); // Null if user is hovering context menu border or any margin/padding applied to context menu container

				if (elm !== null)
					handleViewPortCollision(elm);
			}
		});

		// Contain clicks - prevents e.g. drop down from closing when context menu is used within picker control

		Fit.Events.AddHandler(tree.GetDomElement(), "click", function(e)
		{
			Fit.Events.StopPropagation(e);
		});
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.ContextMenu" name="Show" access="public">
	/// 	<description> Make context menu show up. If X,Y coordinates are not specified, the position of the mouse pointer will be used. </description>
	/// 	<param name="x" type="integer" default="undefined"> If defined, context menu will open at specified horizontal position </param>
	/// 	<param name="y" type="integer" default="undefined"> If defined, context menu will open at specified vertical position </param>
	/// </function>
	this.Show = function(x, y)
	{
		Fit.Validation.ExpectInteger(x, true);
		Fit.Validation.ExpectInteger(y, true);

		if (me._internal.ExecuteBeforeShowBehaviour() === false)
			return;

		me._internal.ExecuteShowBehaviour(x, y);
	}

	/// <function container="Fit.Controls.ContextMenu" name="Hide" access="public">
	/// 	<description> Hide context menu </description>
	/// </function>
	this.Hide = function()
	{
		if (me.IsVisible() === true)
		{
			Fit.Dom.Remove(tree.GetDomElement());
			fireEventHandlers(onHide);

			Fit.Array.ForEach(tree.GetChildren(), function(n) // OnToggled handler makes sure to collapse nodes recursively
			{
				n.Expanded(false);
			});

			highlightOnInitKeyStroke = true;
			Fit.Dom.Data(tree.GetDomElement(), "keynav", "false");
			Fit.Dom.Data(tree.GetDomElement(), "viewportcollision", "false");
			Fit.Dom.Data(tree.GetDomElement(), "sticky", "false");

			Fit.Array.ForEach(tree.GetDomElement().getElementsByTagName("ul"), function(ul)
			{
				Fit.Dom.Data(ul, "viewportcollision", null);
			});
		}
	}

	/// <function container="Fit.Controls.ContextMenu" name="DetectBoundaries" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether boundary/collision detection is enabled or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables collision detection (default), False disables it </param>
	/// </function>
	this.DetectBoundaries = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === false)
			{
				Fit.Dom.Data(tree.GetDomElement(), "viewportcollision", "false");

				Fit.Array.ForEach(tree.GetDomElement().getElementsByTagName("ul"), function(ul)
				{
					Fit.Dom.Data(ul, "viewportcollision", null);
				});
			}

			detectBoundaries = val;
		}

		return detectBoundaries;
	}

	/// <function container="Fit.Controls.ContextMenu" name="IsVisible" access="public" returns="boolean">
	/// 	<description> Get value indicating whether context menu is visible or not </description>
	/// </function>
	this.IsVisible = function()
	{
		return (tree.GetDomElement().parentElement !== null);
	}

	this.GetDomElement = function() // Override GetDomElement() on Fit.Controls.Component
	{
		return tree.GetDomElement();
	}

	this.Render = function(toElement) // Override Render() on Fit.Controls.Component
	{
		Fit.Validation.ThrowError("Use Show function to open ContextMenu");
	}

	/// <function container="Fit.Controls.ContextMenu" name="AddChild" access="public">
	/// 	<description> Add item to ContextMenu </description>
	/// 	<param name="item" type="Fit.Controls.ContextMenuItem"> Item to add </param>
	/// </function>
	this.AddChild = function(item)
	{
		Fit.Validation.ExpectInstance(item, Fit.Controls.ContextMenuItem);

		var tvNode = item.GetDomElement()._internal.Node;
		tree.AddChild(tvNode);
	}

	/// <function container="Fit.Controls.ContextMenu" name="RemoveChild" access="public">
	/// 	<description> Remove item from ContextMenu </description>
	/// 	<param name="item" type="Fit.Controls.ContextMenuItem"> Item to remove </param>
	/// </function>
	this.RemoveChild = function(item)
	{
		Fit.Validation.ExpectInstance(item, Fit.Controls.ContextMenuItem);

		var tvNode = tree.GetChild(item.Value());

		if (tvNode !== null)
			tree.RemoveChild(tvNode);
	}

	/// <function container="Fit.Controls.ContextMenu" name="RemoveAllChildren" access="public">
	/// 	<description> Remove all items contained in ContextMenu </description>
	/// 	<param name="dispose" type="boolean" default="false"> Set True to dispose items </param>
	/// </function>
	this.RemoveAllChildren = function(dispose)
	{
		Fit.Validation.ExpectBoolean(dispose, true);
		tree.RemoveAllChildren(dispose);
	}

	/// <function container="Fit.Controls.ContextMenu" name="GetChild" access="public" returns="Fit.Controls.ContextMenuItem | null">
	/// 	<description> Get item by value - returns Null if not found </description>
	/// 	<param name="val" type="string"> Item value </param>
	/// 	<param name="recursive" type="boolean" default="false"> If defined, True enables recursive search </param>
	/// </function>
	this.GetChild = function(val, recursive)
	{
		Fit.Validation.ExpectString(val);
		Fit.Validation.ExpectBoolean(recursive, true);

		var tvNode = tree.GetChild(val, recursive);

		if (tvNode === null)
			return null;

		return tvNode.GetDomElement()._internal.ContextMenuItem;
	}

	/// <function container="Fit.Controls.ContextMenu" name="GetChildren" access="public" returns="Fit.Controls.ContextMenuItem[]">
	/// 	<description> Get all children </description>
	/// </function>
	this.GetChildren = function()
	{
		var items = [];

		Fit.Array.ForEach(tree.GetChildren(), function(tvNode)
		{
			Fit.Array.Add(items, tvNode.GetDomElement()._internal.ContextMenuItem);
		});

		return items;
	}

	/// <function container="Fit.Controls.ContextMenu" name="GetAllChildren" access="public" returns="Fit.Controls.ContextMenuItem[]">
	/// 	<description> Get all children across entire hierarchy in a flat collection </description>
	/// </function>
	this.GetAllChildren = function()
	{
		var items = [];

		Fit.Array.ForEach(tree.GetAllNodes(), function(tvNode)
		{
			Fit.Array.Add(items, tvNode.GetDomElement()._internal.ContextMenuItem);
		});

		return items;
	}

	/// <function container="Fit.Controls.ContextMenu" name="Focused" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control has focus </description>
	/// 	<param name="value" type="boolean" default="undefined"> If defined, True assigns focus, False removes focus (blur) </param>
	/// </function>
	this.Focused = function(value)
	{
		Fit.Validation.ExpectBoolean(value, true);

		if (Fit.Validation.IsSet(value) === true)
		{
			prevFocused = Fit.Dom.GetFocused();
		}

		return tree.Focused(value);
	}

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		if (me === Fit._internal.ContextMenu.Current) // In case ContextMenu is being disposed while being used
			Fit._internal.ContextMenu.Current = null;

		tree.Dispose();
		me = tree = prevFocused = detectBoundaries = highlightOnInitKeyStroke = isIe8 = onShowing = onShown = onHide = onSelect = null;

		base();
	});

	// ============================================
	// Events
	// ============================================

	/// <function container="Fit.Controls.ContextMenuTypeDefs" name="CancelableEventHandler" returns="boolean | void">
	/// 	<description> Event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.ContextMenuTypeDefs" name="EventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.ContextMenuTypeDefs" name="SelectEventHandler">
	/// 	<description> OnSelect event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="item" type="Fit.Controls.ContextMenuItem"> Instance of ContextMenuItem </param>
	/// </function>

	/// <function container="Fit.Controls.ContextMenu" name="OnShowing" access="public">
	/// 	<description>
	/// 		Add event handler fired before context menu is shown.
	/// 		This event can be canceled by returning False.
	/// 		Function receives one argument: Sender (Fit.Controls.ContextMenu).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ContextMenuTypeDefs.CancelableEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnShowing = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onShowing, cb);
	}

	/// <function container="Fit.Controls.ContextMenu" name="OnShown" access="public">
	/// 	<description>
	/// 		Add event handler fired when context menu is shown.
	/// 		Function receives one argument: Sender (Fit.Controls.ContextMenu).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ContextMenuTypeDefs.EventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnShown = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onShown, cb);
	}

	/// <function container="Fit.Controls.ContextMenu" name="OnHide" access="public">
	/// 	<description>
	/// 		Add event handler fired when context menu is hidden.
	/// 		Function receives one argument: Sender (Fit.Controls.ContextMenu).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ContextMenuTypeDefs.EventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnHide = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onHide, cb);
	}

	/// <function container="Fit.Controls.ContextMenu" name="OnSelect" access="public">
	/// 	<description>
	/// 		Add event handler fired when an item is selected in context menu.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.ContextMenu) and Item (Fit.Controls.ContextMenuItem).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ContextMenuTypeDefs.SelectEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSelect = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelect, cb);
	}

	// ============================================
	// Private
	// ============================================

	function fireEventHandlers(handlers, item) // Notice: item variable only provided for OnSelect event
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (cb(me, item) === false)
				cancel = true; // Do NOT cancel loop though! All handlers must be fired!
		});

		return !cancel;
	}

	function getExpandedChild(children, childToIgnore)
	{
		Fit.Validation.ExpectInstanceArray(children, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectInstance(childToIgnore, Fit.Controls.TreeViewNode);

		var found = null;

		Fit.Array.ForEach(children, function(c)
		{
			if (c.Expanded() === true && c !== childToIgnore)
			{
				found = c;
				return false; // Break loop
			}
		});

		return found;
	}

	function handleViewPortCollision(nodeElm)
	{
		Fit.Validation.ExpectDomElement(nodeElm);

		if (nodeElm.getElementsByTagName("ul").length > 0)
		{
			var ul = nodeElm.getElementsByTagName("ul")[0];
			Fit.Dom.Data(ul, "viewportcollision", null); // Requires repaint in Legacy IE

			repaint(function()
			{
				var pos = Fit.Dom.GetPosition(ul, true); // Null if not visible, which might happen if parent is hovered while user brings up e.g. a screenshot tool using keyboard shortcuts

				if (pos !== null && Fit.Browser.GetViewPortDimensions().Height < pos.Y + ul.offsetHeight)
					Fit.Dom.Data(ul, "viewportcollision", "true");
			});
		}
	}

	function repaint(f)
	{
		Fit.Validation.ExpectFunction(f, true);

		var cb = ((Fit.Validation.IsSet(f) === true) ? f : function() {});

		if (isIe8 === false)
		{
			cb();
		}
		else
		{
			// Flickering may occure on IE8 when updating UI over time
			// (UI update + JS thread released + UI updates again "later").

			Fit.Dom.AddClass(tree.GetDomElement(), "FitUi_Non_Existing_ContextMenu_Class");
			Fit.Dom.RemoveClass(tree.GetDomElement(), "FitUi_Non_Existing_ContextMenu_Class");

			setTimeout(function()
			{
				cb();

				Fit.Dom.AddClass(tree.GetDomElement(), "FitUi_Non_Existing_ContextMenu_Class");
				Fit.Dom.RemoveClass(tree.GetDomElement(), "FitUi_Non_Existing_ContextMenu_Class");
			}, 0);
		}
	}

	this._internal = (this._internal ? this._internal : {});

	this._internal.ExecuteBeforeShowBehaviour = function()
	{
		// Fire OnShowing event

		if (fireEventHandlers(onShowing) === false)
			return false;

		// Close context menu if one is already open

		if (Fit._internal.ContextMenu.Current !== null && Fit._internal.ContextMenu.Current !== me && Fit._internal.ContextMenu.Current.IsVisible() === true)
		{
			Fit._internal.ContextMenu.Current.Hide();
			Fit._internal.ContextMenu.Current = null;
		}

		return true;
	}

	this._internal.ExecuteShowBehaviour = function(x, y)
	{
		Fit.Validation.ExpectInteger(x, true);
		Fit.Validation.ExpectInteger(y, true);

		// Set position

		var pos = Fit.Events.GetPointerState().Coordinates.Document;

		var posX = ((Fit.Validation.IsSet(x) === true) ? x : pos.X);
		var posY = ((Fit.Validation.IsSet(y) === true) ? y : pos.Y);

		tree.GetDomElement().style.left = posX + "px";
		tree.GetDomElement().style.top = posY + "px";
		tree.GetDomElement().style.width = "auto"; // TreeView.Width(val, unit) cannot be used to set width:auto

		// Add to DOM

		if (me.IsVisible() === false) // Only append to DOM once - ContextMenu may have been rooted elsewhere by external code
		{
			Fit.Dom.Add(document.body, tree.GetDomElement());
			Fit._internal.ContextMenu.Current = me;
		}

		// Update all items to let them know if they have children that contains children.
		// This lets us use CSS ( li[data-deep="true"] > ul > li { /* ... */ } ) to indent all children
		// within an item if just one of them has children (hence we need to make room for an expand/collapse icon).

		Fit.Array.Recurse(tree.GetChildren(), "GetChildren", function(child)
		{
			// Multiple children in the same level will cause their parent to get updated multiple times which is acceptable

			Fit.Dom.Data(child.GetDomElement(), "deep", "false");

			if (child.GetChildren().length > 0)
			{
				if (child.GetParent() !== null)
					Fit.Dom.Data(child.GetParent().GetDomElement(), "deep", "true");
				else
					Fit.Dom.Data(child.GetTreeView().GetDomElement().children[0].children[0] /* ul > li */, "deep", "true");
			}
		});

		// Boundary detection

		if (detectBoundaries === true)
		{
			var treeElm = tree.GetDomElement();
			Fit.Dom.Data(treeElm, "viewportcollision", "false");

			if (Fit.Browser.GetViewPortDimensions().Height < (posY - Fit.Dom.GetScrollPosition(document.body).Y) + treeElm.offsetHeight)
			{
				Fit.Dom.Data(treeElm, "viewportcollision", "true");
				treeElm.style.top = (posY - treeElm.offsetHeight) + "px";
			}
		}

		// Focus context menu to allow keyboard navigation

		me.Focused(true);

		// Fire OnShown event

		fireEventHandlers(onShown);

		// Make sure OnClick handler on document does not close
		// Context Menu if triggered using left button.
		Fit._internal.ContextMenu.SkipClickClose = true;
		setTimeout(function() { Fit._internal.ContextMenu.SkipClickClose = false; }, 0);
	}

	init();
}

/// <function container="Fit.Controls.ContextMenuItem" name="ContextMenuItem" access="public">
/// 	<description> Create instance of ContextMenu Item </description>
/// 	<param name="displayTitle" type="string"> Item title </param>
/// 	<param name="itemValue" type="string"> Item value </param>
/// </function>
Fit.Controls.ContextMenuItem = function(displayTitle, itemValue)
{
	Fit.Validation.ExpectString(displayTitle);
	Fit.Validation.ExpectString(itemValue);

	var me = this;
	var node = new Fit.Controls.TreeViewNode(displayTitle, itemValue);

	// ============================================
	// Init
	// ============================================

	function init()
	{
		node.GetDomElement()._internal.ContextMenuItem = me;
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.ContextMenuItem" name="Title" access="public" returns="string">
	/// 	<description> Get/set item title </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, item title is updated </param>
	/// </function>
	this.Title = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);
		return node.Title(val);
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="Value" access="public" returns="string">
	/// 	<description> Get item value </description>
	/// </function>
	this.Value = function()
	{
		return node.Value();
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="Selectable" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether item is selectable or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables item, False disables it </param>
	/// </function>
	this.Selectable = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);
		return node.Selectable(val);
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description> Get DOMElement representing context menu item </description>
	/// </function>
	this.GetDomElement = function()
	{
		return node.GetDomElement();
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="AddChild" access="public">
	/// 	<description> Add child item </description>
	/// 	<param name="item" type="Fit.Controls.ContextMenuItem"> Item to add </param>
	/// </function>
	this.AddChild = function(item)
	{
		Fit.Validation.ExpectInstance(item, Fit.Controls.ContextMenuItem);

		var tvNode = item.GetDomElement()._internal.Node;
		node.AddChild(tvNode);
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="RemoveChild" access="public">
	/// 	<description> Remove child item </description>
	/// 	<param name="item" type="Fit.Controls.ContextMenuItem"> Item to remove </param>
	/// </function>
	this.RemoveChild = function(item)
	{
		Fit.Validation.ExpectInstance(item, Fit.Controls.ContextMenuItem);

		var tvNode = node.GetChild(item.Value());

		if (tvNode !== null)
			node.RemoveChild(tvNode);
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="GetChild" access="public" returns="Fit.Controls.ContextMenuItem | null">
	/// 	<description> Get item by value - returns Null if not found </description>
	/// 	<param name="val" type="string"> Item value </param>
	/// 	<param name="recursive" type="boolean" default="false"> If defined, True enables recursive search </param>
	/// </function>
	this.GetChild = function(val, recursive)
	{
		Fit.Validation.ExpectString(val);
		Fit.Validation.ExpectBoolean(recursive, true);

		var tvNode = node.GetChild(val, recursive);

		if (tvNode === null)
			return null;

		return tvNode.GetDomElement()._internal.ContextMenuItem;
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="GetChildren" access="public" returns="Fit.Controls.ContextMenuItem[]">
	/// 	<description> Get all children </description>
	/// </function>
	this.GetChildren = function()
	{
		var items = [];

		Fit.Array.ForEach(node.GetChildren(), function(tvNode)
		{
			Fit.Array.Add(items, tvNode.GetDomElement()._internal.ContextMenuItem);
		});

		return items;
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="GetParent" access="public" returns="Fit.Controls.ContextMenuItem | null">
	/// 	<description> Get parent item - returns Null for a root item </description>
	/// </function>
	this.GetParent = function()
	{
		var parent = node.GetParent();
		return ((parent !== null) ? parent.GetDomElement()._internal.ContextMenuItem : null);
	}

	/// <function container="Fit.Controls.ContextMenuItem" name="Dispose" access="public">
	/// 	<description> Destroys item to free up memory </description>
	/// </function>
	this.Dispose = function()
	{
		node.Dispose();
		node = null;
	}

	init();
}

Fit.Controls.ContextMenu.Item = Fit.Controls.ContextMenuItem; // Backward compatibility

// Internals

Fit._internal.ContextMenu = {};
Fit._internal.ContextMenu.SkipClickClose = false;
Fit._internal.ContextMenu.Current = null;

// Event handler responsible for closing context menu when clicking outside of context menu

Fit.Events.OnReady(function()
{
	Fit.Events.AddHandler(document, "click", function(e)
	{
		if (Fit._internal.ContextMenu.SkipClickClose === true)
			return;

		var target = Fit.Events.GetTarget(e);
		var ctx = Fit._internal.ContextMenu.Current;

		if (ctx === null || target === ctx.GetDomElement() || Fit.Dom.Contained(ctx.GetDomElement(), target) === true)
			return;

		ctx.Hide();
	});

	/*Fit.Events.AddHandler(document, "mousewheel", function(e) // Close ContextMenu when scrolling
	{
		if (Fit._internal.ContextMenu.Current !== null)
			Fit._internal.ContextMenu.Current.Hide();
	});*/
});

/// <container name="Fit.Controls.WSContextMenu" extends="Fit.Controls.ContextMenu">
/// 	ContextMenu control allowing for quick access to select features provided by a WebService.
/// 	Extending from Fit.Controls.ContextMenu.
/// </container>

/// <function container="Fit.Controls.WSContextMenu" name="WSContextMenu" access="public">
/// 	<description> Create instance of WSContextMenu control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.WSContextMenu = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.ContextMenu).Apply(controlId);

	var me = this;
	var url = null;
	var jsonpCallback = null;
	var dataLoading = false;
	var onDataLoadedCallback = [];

	var onRequestHandlers = [];
	var onResponseHandlers = [];
	var onPopulatedHandlers = [];

	// ============================================
	// Init
	// ============================================

	function init()
	{
	}

	// ============================================
	// Public
	// ============================================

	// WSContextMenu has its own implementation of Show() because it is async.
	// We want to fire OnShowing, load and WAIT for the data, and THEN open the context menu and call OnShown.
	// We don't want to show the context menu or fire OnShown before it has been populated with data.
	this.Show = function(x, y)
	{
		Fit.Validation.ExpectInteger(x, true);
		Fit.Validation.ExpectInteger(y, true);

		if (dataLoading === true)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.Show(x, y); });
			return;
		}

		// Fire OnShowing event

		if (me._internal.ExecuteBeforeShowBehaviour() === false)
			return;

		// Load data

		dataLoading = true;

		getData(function(eventArgs)
		{
			// Populate data received

			me.RemoveAllChildren();

			Fit.Array.ForEach(eventArgs.Children, function(c)
			{
				me.AddChild(createItemFromJson(c));
			});

			// Show context menu

			me._internal.ExecuteShowBehaviour(x, y);

			// Invoke onDataLoaded callbacks

			dataLoading = false;
			fireOnDataLoaded();
		});
	}

	/// <function container="Fit.Controls.WSContextMenu" name="Url" access="public" returns="string">
	/// 	<description>
	/// 		Get/set URL to WebService responsible for providing data to ContextMenu.
	/// 		WebService must deliver all data at once in the following JSON format:
	/// 		[
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 1", Value: "1001", Selectable: true, Selected: true, Children: [] },
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 2", Value: "1002", Selectable: false, Selected: false, Children: [] }
	/// 		]
	/// 		Only Value is required. Children is a collection of items with the same format as described above.
	/// 	</description>
	/// 	<param name="wsUrl" type="string" default="undefined"> If defined, updates WebService URL (e.g. http://server/ws/data.asxm/GetItems) </param>
	/// </function>
	this.Url = function(wsUrl)
	{
		Fit.Validation.ExpectString(wsUrl, true);

		if (Fit.Validation.IsSet(wsUrl) === true)
		{
			url = wsUrl;
		}

		return url;
	}

	/// <function container="Fit.Controls.WSContextMenu" name="JsonpCallback" access="public" returns="string">
	/// 	<description>
	/// 		Get/set name of JSONP callback argument. Assigning a value will enable JSONP communication.
	/// 		Often this argument is simply &quot;callback&quot;. Passing Null disables JSONP communication again.
	/// 	</description>
	/// 	<param name="val" type="string | null" default="undefined"> If defined, enables JSONP and updates JSONP callback argument </param>
	/// </function>
	this.JsonpCallback = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true || val === null)
		{
			jsonpCallback = val;
		}

		return jsonpCallback;
	}

	// ============================================
	// Events
	// ============================================

	/// <container name="Fit.Controls.WSContextMenuTypeDefs.RequestEventArgs">
	/// 	<description> Request event handler arguments </description>
	/// 	<member name="Sender" type="$TypeOfThis"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonpRequest | Fit.Http.JsonRequest"> Instance of JsonpRequest or JsonRequest </member>
	/// </container>

	/// <function container="Fit.Controls.WSContextMenuTypeDefs" name="RequestEventHandler" returns="boolean | void">
	/// 	<description> OnRequest event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSContextMenuTypeDefs.RequestEventArgs"> Event arguments </param>
	/// </function>

	/// <container name="Fit.Controls.WSContextMenuTypeDefs.DataEventArgs">
	/// 	<description> Data event handler arguments </description>
	/// 	<member name="Sender" type="$TypeOfThis"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonpRequest | Fit.Http.JsonRequest"> Instance of JsonpRequest or JsonRequest </member>
	/// 	<member name="Children" type="object"> JSON items received from web service </member>
	/// </container>

	/// <function container="Fit.Controls.WSContextMenuTypeDefs" name="DataEventHandler">
	/// 	<description> Data event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSContextMenuTypeDefs.DataEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSContextMenu" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is being requested.
	/// 		Request can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSContextMenu) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSContextMenu instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSContextMenuTypeDefs.RequestEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnRequest = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onRequestHandlers, cb);
	}

	/// <function container="Fit.Controls.WSContextMenu" name="OnResponse" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is received,
	/// 		allowing for data transformation to occure before
	/// 		ContextMenu is populated. Function receives two arguments:
	/// 		Sender (Fit.Controls.WSContextMenu) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSContextMenu instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Children: JSON items received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSContextMenuTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnResponse = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onResponseHandlers, cb);
	}

	/// <function container="Fit.Controls.WSContextMenu" name="OnPopulated" access="public">
	/// 	<description>
	/// 		Add event handler fired when ContextMenu has been populated with items.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSContextMenu) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSContextMenu instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Children: JSON items received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSContextMenuTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnPopulated = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onPopulatedHandlers, cb);
	}

	// ============================================
	// Private
	// ============================================

	function getData(cb)
	{
		Fit.Validation.ExpectFunction(cb);

		if (url === null)
			Fit.Validation.ThrowError("Unable to get data, no WebService URL has been specified");

		// Determine request method

		var request = null;

		if (jsonpCallback !== null)
		{
			request = new Fit.Http.JsonpRequest(url, jsonpCallback);
		}
		/*else if (url.toLowerCase().indexOf(".asmx/") === -1)
		{
			request = new Fit.Http.Request(url);
		}*/
		else
		{
			request = new Fit.Http.JsonRequest(url)
		}

		// Fire OnRequest

		var eventArgs = { Sender: null, Request: null, Children: null };
		eventArgs.Sender = me;
		eventArgs.Request = request;

		if (fireEventHandlers(onRequestHandlers, eventArgs) === false)
			return;

		if (eventArgs.Request !== request)
		{
			// Support for changing request instans to
			// take control over webservice communication.

			// Restrict to support for Fit.Http.Request or classes derived from this
			Fit.Validation.ExpectInstance(eventArgs.Request, Fit.Http.Request);

			request = eventArgs.Request;
		}

		// Define request callbacks

		var onSuccess = function(children)
		{
			// Fire OnResponse

			eventArgs.Children = ((children instanceof Array) ? children : []);
			fireEventHandlers(onResponseHandlers, eventArgs);

			// Fire getData callback

			cb(eventArgs); // Callback is responsible for populating Context Menu

			// Fire OnPopulated

			fireEventHandlers(onPopulatedHandlers, eventArgs);
		};

		var onFailure = function(httpStatusCode)
		{
			Fit.Validation.ThrowError("Unable to load data for context menu - request failed with HTTP Status Code " + httpStatusCode)
		};

		// Register request callbacks

		if (Fit.Core.InstanceOf(request, Fit.Http.JsonpRequest) === false)
		{
			request.OnSuccess(function(req)
			{
				var children = request.GetResponseJson();
				onSuccess(children);
			});

			request.OnFailure(function(req)
			{
				onFailure(request.GetHttpStatus());
			});
		}
		else
		{
			request.OnSuccess(function(req)
			{
				var children = req.GetResponse();
				onSuccess(children);
			});

			request.OnTimeout(function()
			{
				onFailure("UNKNOWN (JSONP)");
			});
		}

		// Invoke request

		request.Start();
	}

	function onDataLoaded(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onDataLoadedCallback, cb);
	}

	function fireOnDataLoaded()
	{
		// Copied from WSTreeView.
		// Immediately clear collection. If multiple callbacks are registered,
		// chances are that only the first will run, and the remaining will be
		// re-scheduled again - so we need the collection to be cleared before
		// invoking callbacks.
		var orgOnDataLoadedCallback = onDataLoadedCallback;
		onDataLoadedCallback = [];

		Fit.Array.ForEach(orgOnDataLoadedCallback, function(cb)
		{
			cb();
		});
	}

	function createItemFromJson(jsonNode)
	{
		Fit.Validation.ExpectIsSet(jsonNode);

		// Convert JSON to ContextMenu item, including all contained children

		var item = new Fit.Controls.ContextMenuItem((jsonNode.Title ? jsonNode.Title : jsonNode.Value), jsonNode.Value);

		if (jsonNode.Selectable !== undefined)
			item.Selectable((jsonNode.Selectable === true));

		if (jsonNode.Children instanceof Array)
		{
			Fit.Array.ForEach(jsonNode.Children, function(c)
			{
				item.AddChild(createItemFromJson(c));
			});
		}

		return item;
	}

	function fireEventHandlers(handlers, eventArgs)
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (cb(me, eventArgs) === false)
				cancel = true; // Do NOT cancel loop though! All handlers must be fired!
		});

		return !cancel;
	}

	init();
}

/// <container name="Fit.Controls.DatePicker" extends="Fit.Controls.ControlBase">
/// 	DatePicker control allowing user to easily pick a date and optionally time.
/// 	On mobile devices (phones and tablets) the native date and time pickers are used.
/// 	Extending from Fit.Controls.ControlBase.
/// </container>

/// <function container="Fit.Controls.DatePicker" name="DatePicker" access="public">
/// 	<description> Create instance of DatePicker control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.DatePicker = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;
	var input = null;			// Input field for date
	var inputTime = null;		// Input field for time (Null if not enabled)
	var orgVal = null;			// Date object containing control value set using Value(..) or Date(..) - used to determine Dirty state (holds both date and time portion)
	var preVal = "";			// Previous valid date value as string (without time portion)
	var prevTimeVal = "";		// Previous valid time value as string (without date portion)
	var locale = "en";			// Default ("", "en", and "en-US" is the same) - see this.regional[""] decleration in jquery-ui.js
	var localeEnforced = false;	// Whether locale was set by external code which takes precedence over locale set using Fit.Internationalization.Locale(..)
	var format = "MM/DD/YYYY";	// Default format for "en" locale (specified using Fit.UI format - jQuery UI DataPicker uses "mm/dd/yy" - see this.regional[""] decleration in jquery-ui.js)
	var formatEnforced = false;	// Whether format was set by external code which takes precedence over locale set using DatePicker.Locale(..) and Fit.Internationalization.Locale(..)
	var placeholderDate = null;	// Placeholder value for date
	var placeholderTime = null;	// Placeholder value for time
	var weeks = false;			// Whether to display week numbers or not
	var jquery = undefined;		// jQuery instance
	var datepicker = null;		// jQuery UI calendar widget
	var datepickerElm = null;	// jQuery UI calendar widget element (remains null until shown for the first time - element shared among all DatePicker instances)
	var startDate = null;		// Which year/month to display in calendar widget if view needs to be restored (related to restoreView variable)
	var open = false;			// Whether calendar widget is currently open
	var focused = false;		// Whether control is currently focused
	var restoreView = false;	// Whether to keep calendar widget on given year/month when temporarily closing and opening it again
	var updateCalConf = true;	// Whether to set/update calendar widget settings - true on initial load
	var detectBoundaries = false;				// Flag indicating whether calendar widget should detect viewport collision and open upwards when needed
	var detectBoundariesRelToViewPort = false;	// Flag indicating whether calendar widget should be positioned relative to viewport (true) or scroll parent (false)

	//var isMobile = Fit.Browser.GetInfo().IsMobile || (Fit.Browser.GetInfo().IsTouchEnabled && Fit.Browser.GetInfo().Name === "Safari"); // More recent versions of Safari on iPad identifies as a Mac computer by default ("Request desktop website" is enabled by default)
	var isMobile = Fit.Device.OptimizeForTouch;
	var inputMobile = null;		// Native date picker on mobile devices - value selected is synchronized to input field defined above (remains Null on desktop devices)
	var inputTimeMobile = null;	// Native time picker on mobile devices - value selected is synchronized to inputTime field defined above (remains Null on desktop devices)

	// ============================================
	// Init
	// ============================================

	function init()
	{
		input = document.createElement("input");
		input.type = "text";
		input.autocomplete = "off";
		input.spellcheck = false;
		input.placeholder = getDatePlaceholder();
		input.tabIndex = ((isMobile === true) ? -1 : 0);

		input.onkeydown = function(e)
		{
			// Open calendar when arrow down key is pressed

			var ev = Fit.Events.GetEvent(e);

			if (ev.keyCode === 40) // Arrow down
			{
				me.Show();
			}
		}

		input.onkeyup = function()
		{
			// Fire OnChange when a valid value is entered

			var curDateTime = me.Date();

			if (input.value !== "" && curDateTime === null)
				return; // Invalid value

			var curDateStr = ((curDateTime !== null) ? Fit.Date.Format(curDateTime, me.Format()) : ""); // Not using Value() which includes time if enabled

			if (curDateStr !== preVal)
			{
				if (input.value === "" && open === true) // Value cleared while calendar widget is open
				{
					// Fix: DatePicker keeps selection when value is
					// cleared - hide and show again to update picker
					restoreView = true;
					me.Hide();
					me.Show();
					restoreView = false;
				}

				preVal = curDateStr;
				me._internal.FireOnChange();
			}
		}

		input.onchange = function() // OnKeyUp does not catch changes made using mouse (e.g. cut)
		{
			if (me === null)
			{
				// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
				// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
				// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
				return;
			}

			input.onkeyup();
		}

		input.onpaste = function()
		{
			setTimeout(function() // Postpone - OnPaste fires before pasted value is set
			{
				input.onkeyup();

				// The calendar widget is not updated when pasting a value
				// using the context menu. CTRL/CMD + V updates it just fine though.
				// Force it to always update when pasting a value.
				if (open === true)
				{
					me.Hide();
					me.Show();
				}

				startDate = null;
			}, 0);
		}

		input.onclick = function()
		{
			me.Show();
		}

		// Prevent OnFocus from firing when user interacts with calendar widget which
		// is not contained in div.FitUiControlDatePicker (changing month/year and selecting date).
		// Focus is returned to input almost immediately after interacting with calendar widget.
		//  - Also see FireOnBlur override further down.
		me._internal.FireOnFocus = Fit.Core.CreateOverride(me._internal.FireOnFocus, function()
		{
			if (focused === false)
			{
				focused = true;
				base();
			}
		});

		// Prevent OnBlur from firing when user interacts with calendar widget which
		// is not contained in div.FitUiControlDatePicker (changing month/year).
		// Focus is returned to input almost immediately after interacting with calendar widget.
		//  - Also see FireOnFocus override above.
		me._internal.FireOnBlur = Fit.Core.CreateOverride(me._internal.FireOnBlur, function()
		{
			var pointerState = Fit.Events.GetPointerState();
			var userAction = pointerState.Buttons.Primary === true || pointerState.Buttons.Touch === true;

			if (open === true && userAction === true) // Skipping invocation of OnBlur event when user interacts with calendar widget
			{
				return;
			}

			focused = false;
			base();
		});

		input.onblur = function()
		{
			if (me === null)
			{
				// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
				// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
				// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
				return;
			}

			// Make sure date entered is properly formatted (e.g. 1-1-2012 transforms to 01-01-2012)

			if (open === true)
				return; // Do not execute code below when input looses focus when user interacts with calendar widget

			// The input.onchange/onpaste/onkeyup events constantly updates preVal,
			// so it always contains the last valid (and properly formatted) value.

			// Notice that the calendar widget is temporarily closed when clearing the
			// control value using e.g. CTRL+X or CMD+X. This causes OnBlur to fire and
			// reach the code below BEFORE preVal is updated, because this value is not
			// updated until Key UP. This results in the previous value being restored
			// every time the user tries to clear the input field. The fix is a simple
			// as making sure preVal is only used when the input field actually contains
			// a value.

			if (input.value !== "")
				input.value = preVal;
		}

		me.OnBlur(function(sender)
		{
			if (input.value === "" && inputTime !== null)
			{
				inputTime.value = "";
				prevTimeVal = "00:00";
			}

			if (input.value !== "" && inputTime !== null && inputTime.value === "")
			{
				inputTime.value = "00:00";
				prevTimeVal = "00:00";
			}

			if (open === true) // Calendar might be left open if focus is "stolen" from control, e.g. using document.activeElement.blur(), or using datePicker.Focused(false)
			{
				me.Hide();
			}
		});

		me._internal.AddDomElement(input);
		me.AddCssClass("FitUiControlDatePicker");

		me._internal.Data("weeks", "false");
		me._internal.Data("time", "false");

		if (isMobile === true)
		{
			inputMobile = document.createElement("input");
			inputMobile.type = "date";
			inputMobile.onchange = function()
			{
				if (inputMobile.valueAsDate !== null)
					input.value = Fit.Date.Format(inputMobile.valueAsDate, me.Format());
				else
					input.value = "";

				input.onchange();
			}

			if (Fit.Device.HasMouse === true)
			{
				// This might be an iPad with a pen (precision pointer) attached, or a PC/laptop with a touch screen
				// attached, where touch is considered the primary input method and mouse/pen the secondary input method.

				// Using OnMouseDown event below since OnClick on iOS/Safari 15.6.1 prevents native calendar
				// widget from showing, and OnMouseUp sometimes causes calendar widget to stop emerging on screen.
				// Both mouse and touch events fire, even on pure touch devices.
				inputMobile.onmousedown = function(e)
				{
					// NOTICE: On traditional desktop browsers Show() only works if the native
					// showPicker() function is supported: https://github.com/Jemt/Fit.UI/issues/169
					me.Show();
				}
			}

			me._internal.AddDomElement(inputMobile);
		}

		Fit.Internationalization.OnLocaleChanged(localize);
		localize();
	}

	// ============================================
	// Public - overrides
	// ============================================

	// See documentation on ControlBase
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (Fit.Validation.IsSet(focus) === true)
		{
			if (focus === true && me.Focused() === false)
			{
				((inputMobile === null) ? input : inputMobile).focus();
			}
			else if (focus === false && me.Focused() === true)
			{
				Fit.Dom.GetFocused().blur();
			}
		}

		var hasFocus = Fit.Array.Contains([input, inputTime, inputMobile, inputTimeMobile], Fit.Dom.GetFocused()) === true;

		if (hasFocus === false && isMobile === false)
		{
			// Make sure Focused() returns true while interacting with calendar widget - https://github.com/Jemt/Fit.UI/issues/194
			var calendarWidget = datepickerElm; // Null if not loaded yet
			hasFocus = calendarWidget !== null && calendarWidget.style.display === "block" && calendarWidget._associatedFitUiControl === me.GetId();
		}

		return hasFocus;
	}

	/// <function container="Fit.Controls.DatePicker" name="Value" access="public" returns="string">
	/// 	<description>
	/// 		Get/set control value in the following format: YYYY-MM-DD[ hh:mm]
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is inserted into control </param>
	/// 	<param name="preserveDirtyState" type="boolean" default="false">
	/// 		If defined, True prevents dirty state from being reset, False (default) resets the dirty state.
	/// 		If dirty state is reset (default), the control value will be compared against the value passed,
	/// 		to determine whether it has been changed by the user or not, when IsDirty() is called.
	/// 	</param>
	/// </function>
	this.Value = function(val, preserveDirtyState) // ONLY accepts YYYY-MM-DD[ hh:mm]
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			var fireOnChange = (me.Value() !== val);
			var wasOpen = open;

			if (wasOpen === true)
			{
				restoreView = true;
				me.Hide();
			}

			if (val !== "")
			{
				if (val.indexOf(":") === -1)
					val += " 00:00";

				var date = Fit.Date.Parse(val, "YYYY-MM-DD hh:mm");
				date.setSeconds(0);
				date.setMilliseconds(0);

				input.value = Fit.Date.Format(date, me.Format());
				preVal = input.value;

				if (preserveDirtyState !== true)
				{
					orgVal = date;
				}

				if (inputTime !== null)
				{
					inputTime.value = Fit.Date.Format(date, "hh:mm");
					prevTimeVal = inputTime.value;
				}

				if (inputMobile !== null)
					inputMobile.valueAsDate = new Date(Date.UTC(date.getFullYear(), date.getMonth(), date.getDate()));

				if (inputTimeMobile !== null)
					inputTimeMobile.value = inputTime.value;
			}
			else
			{
				input.value = "";
				preVal = "";

				if (preserveDirtyState !== true)
				{
					orgVal = null;
				}

				if (inputTime !== null)
				{
					inputTime.value = "";
					prevTimeVal = "00:00";
				}

				if (inputMobile !== null)
					inputMobile.valueAsDate = null;

				if (inputTimeMobile !== null)
					inputTimeMobile.value = "";
			}

			if (wasOpen === true)
			{
				me.Show();
				restoreView = false;
			}

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		if (input.value === "")
			return "";

		try
		{
			var value = input.value + ((inputTime !== null) ? " " + ((inputTime.value !== "") ? inputTime.value : "00:00") : "");
			var dtFormat = me.Format() + ((value.indexOf(":") > -1) ? " hh:mm" : "");

			var d = Fit.Date.Parse(value, dtFormat); // May throw exception if value is invalid - this may happen if user is currently entering an invalid value while Value() is being called at the same time async.
			return Fit.Date.Format(d, "YYYY-MM-DD" + ((value.indexOf(":") > -1) ? " hh:mm" : ""));
		}
		catch (err)
		{
		}

		return "";
	}

	/// <function container="Fit.Controls.DatePicker" name="UserValue" access="public" returns="string">
	/// 	<description>
	/// 		Get/set value as if it was changed by the user.
	/// 		Contrary to Value(..), this function allows for a partial (or invalid) date value.
	/// 		If the time picker is enabled (see Time(..)) then both the date and time portion can
	/// 		be set, separated by a space (e.g. 2020-10-25 14:30).
	/// 		OnChange fires if value provided is valid. See ControlBase.UserValue(..) for more details.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is inserted into control </param>
	/// </function>
	this.UserValue = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			// Current state

			var curDate = input.value;
			var curTime = (inputTime !== null ? inputTime.value : null);
			var onChangeFired = false;

			// Parse date and time value

			var datePortion = val;
			var timePortion = null;

			if (inputTime !== null && val.indexOf(" ") !== -1)
			{
				datePortion = val.substring(0, val.indexOf(" "));
				timePortion = val.substring(val.indexOf(" ") + 1);
			}

			if (val === "" && inputTime !== null)
			{
				timePortion = "";
			}

			// Update input fields and fire OnChange

			input.value = datePortion;

			if (inputTime !== null)
			{
				inputTime.value = (timePortion !== null ? timePortion : "");

				if (curTime !== inputTime.value)
				{
					inputTime.onchange(); // Does not fire DatePicker's OnChange if datatime value is invalid, or if value is identical to previously valid value assigned
					onChangeFired = true;
				}
			}

			if (onChangeFired === false && curDate !== input.value)
			{
				input.onchange(); // Does not fire DatePicker's OnChange if date value is invalid, or if value is identical to previously valid value assigned
			}
		}

		return input.value + (inputTime !== null && inputTime.value !== "" ? " " + inputTime.value : "");
	}

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		if (inputTime !== null)
		{
			return (Fit.Core.IsEqual(orgVal, me.Date()) === false);
		}
		else
		{
			var curDate = me.Date();

			if ((orgVal === null && curDate !== null) || (orgVal !== null && curDate === null))
				return true;
			else if (orgVal === null && curDate === null)
				return false;
			else
				return (Fit.Date.Format(orgVal, "YYYY-MM-DD") !== Fit.Date.Format(me.Date(), "YYYY-MM-DD"));
		}
	}

	// See documentation on ControlBase
	this.Clear = function()
	{
		me.Value("");
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		// This will destroy control - it will no longer work!

		if (datepicker !== null)
		{
			me.Hide();
			datepicker.datepicker("destroy");
		}

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		me = input = inputTime = orgVal = preVal = prevTimeVal = locale = localeEnforced = format = formatEnforced = placeholderDate = placeholderTime = weeks = jquery = datepicker = datepickerElm = startDate = open = focused = restoreView = updateCalConf = detectBoundaries = detectBoundariesRelToViewPort = isMobile = inputMobile = inputTimeMobile = null;
		base();
	});

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.DatePicker" name="Date" access="public" returns="Date | null">
	/// 	<description>
	/// 		Get/set control value.
	/// 		The function works the same as the Value function, expect it
	/// 		accepts and returns a Date object instead of a string.
	/// 	</description>
	/// 	<param name="val" type="Date | null" default="undefined"> If defined, date is selected </param>
	/// </function>
	this.Date = function(val)
	{
		Fit.Validation.ExpectDate(val, true);

		// Set value if supplied

		if (val !== undefined) // Allow Null
		{
			if (val !== null)
				me.Value(val.getFullYear() + "-" + (val.getMonth() + 1) + "-" + val.getDate() + " " + val.getHours() + ":" + val.getMinutes());
			else
				me.Clear();
		}

		// Get current value

		var value = me.Value();
		var date = null;

		if (value !== "")
		{
			try
			{
				date = Fit.Date.Parse(value, "YYYY-MM-DD" + ((inputTime !== null) ? " hh:mm" : ""));
			}
			catch (err)
			{
			}
		}

		if (date !== null)
		{
			if (inputTime === null)
			{
				date.setHours(0);
				date.setMinutes(0);
			}

			date.setSeconds(0);
			date.setMilliseconds(0);
		}

		return date;
	}

	/// <function container="Fit.Controls.DatePicker" name="GetText" access="public" returns="string">
	/// 	<description>
	/// 		Get control value as a string. Opposite to the Value function GetText returns the
	/// 		selected Date/DateTime in the format configured (see Format function). The Value
	/// 		function always returns the value in a fixed format, which is YYYY-MM-DD[ hh:mm].
	/// 		The time portion is only appended if time is enabled (see Time function).
	/// 	</description>
	/// </function>
	this.GetText = function()
	{
		var date = me.Date();
		return ((date !== null) ? Fit.Date.Format(date, me.Format() + ((inputTime !== null) ? " hh:mm" : "")) : "");
	}

	/// <function container="Fit.Controls.DatePicker" name="Locale" access="public" returns="string">
	/// 	<description>
	/// 		Get/set locale used by the DatePicker control. This will affect the
	/// 		date format, unless format has been set explicitely, as well as the language used by the calendar widget.
	/// 		DatePicker locale takes precedence over locale set using Fit.Internationalization.Locale(..).
	/// 		Call the GetLocales function to get a complete list of supported locales.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined"> If defined, locale is changed </param>
	/// </function>
	this.Locale = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			localeEnforced = true;
			setLocale(val);
		}

		return locale;
	}

	/// <function container="Fit.Controls.DatePicker" name="Format" access="public" returns="string">
	/// 	<description>
	/// 		Get/set format used by the DatePicker control. This will affect the format
	/// 		in which the date is presented, as well as the value returned by the GetText function.
	/// 		Format takes precedence over locale.
	/// 	</description>
	/// 	<param name="val" type="string" default="undefined">
	/// 		If defined, format is changed.
	///			The following tokens can be used to construct the format:
	///			YYYY = Year with four digits (e.g. 2016)
	///			M = Month with one digit if possible (e.g. 1 or 12)
	///			MM = Month with two digits (e.g. 01 or 12)
	///			D = Day with one digit if possible (e.g. 1 or 24)
	///			DD = Day with two digits (e.g. 01 or 24)
	/// 		Examples: YYYY-MM-DD or D/M-YYYY
	/// 	</param>
	/// </function>
	this.Format = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			formatEnforced = true;
			setFormat(val);
		}

		return format;
	}

	/// <function container="Fit.Controls.DatePicker" name="DatePlaceholder" access="public" returns="string | null">
	/// 	<description> Get/set date placeholder value. Returns Null if not set. </description>
	/// 	<param name="val" type="string | null" default="undefined">
	/// 		If defined, placeholder is updated. Pass Null to use default
	/// 		placeholder, or an empty string to remove placeholder.
	/// 	</param>
	/// </function>
	this.DatePlaceholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (val !== undefined) // Allow Null
		{
			placeholderDate = val;
			input.placeholder = getDatePlaceholder();
		}

		return placeholderDate;
	}

	/// <function container="Fit.Controls.DatePicker" name="TimePlaceholder" access="public" returns="string | null">
	/// 	<description> Get/set time placeholder value. Returns Null if not set. </description>
	/// 	<param name="val" type="string | null" default="undefined">
	/// 		If defined, placeholder is updated. Pass Null to use default
	/// 		placeholder, or an empty string to remove placeholder.
	/// 	</param>
	/// </function>
	this.TimePlaceholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (val !== undefined) // Allow Null
		{
			placeholderTime = val;

			if (inputTime !== null)
				inputTime.placeholder = getTimePlaceholder();
		}

		return placeholderTime;
	}

	this.WeekNumbers = function(val) // Not supported on mobile, and jQuery calendar is buggy: https://bugs.jqueryui.com/ticket/14907
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			var wasOpen = open;

			if (wasOpen === true)
			{
				restoreView = true;
				me.Hide();
			}

			weeks = val;
			updateCalConf = true; // Update calendar widget settings

			if (wasOpen === true)
			{
				me.Show();
				restoreView = false;
			}

			me._internal.Data("weeks", val.toString());
		}

		return weeks;
	}

	/// <function container="Fit.Controls.DatePicker" name="Time" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether DatePicker should allow a time portion to be set.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, time is changed </param>
	/// </function>
	this.Time = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && inputTime === null)
			{
				prevTimeVal = "00:00";

				inputTime = document.createElement("input");
				inputTime.type = "text";
				inputTime.autocomplete = "off";
				inputTime.spellcheck = false;
				inputTime.placeholder = getTimePlaceholder();
				inputTime.value = ((me.Date() !== null) ? "00:00" : "");
				inputTime.tabIndex = ((isMobile === true) ? -1 : 0);

				if (me.Date() !== null && orgVal !== null) // DatePicker has a date selected, and had a DateTime value assigned programmatically when created
				{
					inputTime.value = Fit.Date.Format(orgVal, "hh:mm");
					prevTimeVal = inputTime.value;
				}

				inputTime.onkeydown = function(e)
				{
					// Suppress invalid characters

					var ev = Fit.Events.GetEvent(e);
					var mod = Fit.Events.GetModifierKeys();

					// Allow Tab/Backspace/ArrowLeft/ArrowRightDelete/Ctrl/Cmd

					if (ev.keyCode === 9 || ev.keyCode === 8 || ev.keyCode === 37 || ev.keyCode === 39 || ev.keyCode === 46 || mod.Ctrl === true || mod.Meta === true)
						return;

					// Allow colon and 0-9

					var tv = inputTime.value;
					var isColon = (ev.keyCode === 186 || (mod.Shift === true && ev.keyCode === 190)); // Colon (keycode 186 or Shift+190 - depends on browser)
					var valid = (/*tv.length < 5 &&*/ (isColon === true || (ev.keyCode >= 48 && ev.keyCode <= 58) || (ev.keyCode >= 96 && ev.keyCode <= 105))); // Colon or 0-9 (numrow or numpad) - disabled check on length which prevents value from being overwritten when selected

					if (isColon === true && (tv.length === 0 || tv.substring(tv.length - 1) === ":"))
					{
						valid = false; // Skip colon if it is already present - DatePicker adds it automatically which many users overlook
					}

					if (valid === false)
					{
						Fit.Events.PreventDefault(ev);
						return;
					}
				}

				inputTime.onkeyup = function(e)
				{
					// Automatically add colon when hours have been entered

					if (e !== null) // Null when called from inputTime.onchange handler
					{
						var ev = Fit.Events.GetEvent(e);

						if (inputTime.value.length === 2 && inputTime.value.substring(1) !== ":" && ev.keyCode !== 8 && ev.keyCode !== 46) // KeyCodes: Backspace and Delete
						{
							inputTime.value += ":";
							return;
						}
					}

					// Update prevTimeVal which holds the last valid time value entered, and fire OnChange

					var dt = null;

					try
					{
						dt = Fit.Date.Parse(((inputTime.value !== "") ? inputTime.value : "00:00"), "hh:mm");
					}
					catch (err)
					{
					}

					if (dt === null) // Invalid value entered
						return;

					var newVal = Fit.Date.Format(dt, "hh:mm");

					if (newVal !== prevTimeVal)
					{
						prevTimeVal = newVal;

						if (input.value !== "")
							me._internal.FireOnChange();
					}
				}

				inputTime.onchange = function()
				{
					if (me === null)
					{
						// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
						// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
						// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
						return;
					}

					inputTime.onkeyup(null); // In case mouse was used to cut a value
				}

				inputTime.onpaste = function() // In case mouse was used to paste a value
				{
					setTimeout(function() // Postpone - OnPaste fires before pasted value is set
					{
						inputTime.onkeyup(null);
					}, 0);
				}

				inputTime.onblur = function()
				{
					if (me === null)
					{
						// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
						// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
						// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
						return;
					}

					if (inputTime.value === "")
					{
						inputTime.value = ((input.value !== "") ? "00:00" : "");
						prevTimeVal = "00:00";
					}
					else
					{
						// Use prevTimeVal which is properly formatted with leading zeros (see inputTime.onkeyup handler)
						inputTime.value = prevTimeVal;
					}
				}
				Fit.Dom.InsertAfter(input, inputTime);

				// Use native time picker on mobile devices (where touch is the primary pointing method).
				// However, do not do so on a hybrid device with support for both touch and mouse/pen.
				// On a device with a precision pointer, we most likely want to be able to accurately
				// place the text cursor in the time portion of the control to correct or enter a value
				// using the keyboard (psysical keyboard or virtual on-screen keyboard).
				if (isMobile === true && Fit.Device.HasMouse === false)
				{
					inputTimeMobile = document.createElement("input");
					inputTimeMobile.type = "time";
					inputTimeMobile.onchange = function()
					{
						if (inputTimeMobile.value === "" && inputMobile.value !== "")
							inputTimeMobile.value = "00:00";

						inputTime.value = inputTimeMobile.value; // Value is 24 hours hh:mm format (e.g. 22:43), even for devices with AM/PM picker
						inputTime.onchange();
					}
					me._internal.AddDomElement(inputTimeMobile);
				}

				me._internal.FireOnChange(); // Enabling Time causes Value() and Date() to return Date with timestamp - invoke event listeners
			}
			else if (val === false && inputTime !== null)
			{
				Fit.Dom.Remove(inputTime);

				if (inputTimeMobile !== null)
				{
					Fit.Dom.Remove(inputTimeMobile);
				}

				inputTime = null;
				inputTimeMobile = null;
				prevTimeVal = "";

				me._internal.FireOnChange(); // Disabling Time causes Value() and Date() to return Date without timestamp - invoke event listeners
			}

			me._internal.Data("time", (inputTime !== null).toString());
		}

		return (inputTime !== null);
	}

	/// <function container="Fit.Controls.DatePicker" name="DetectBoundaries" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether boundary/collision detection is enabled or not (off by default).
	/// 		This may cause calendar to open upwards if sufficient space is not available below control.
	/// 		If control is contained in a scrollable parent, this will be considered the active viewport,
	/// 		and as such define the active boundaries - unless relativeToViewport is set to True, in which
	/// 		case the actual browser viewport will be used.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables collision detection, False disables it (default) </param>
	/// 	<param name="relativeToViewport" type="boolean" default="false">
	/// 		If defined, True results in viewport being considered the container to which available space is determined.
	/// 		This also results in calendar widget being positioned with position:fixed, allowing it to escape a container
	/// 		with overflow (e.g. overflow:auto|hidden|scroll). Be aware though that this does not work reliably in combination
	/// 		with CSS animation and CSS transform as it creates a new stacking context to which position:fixed becomes relative.
	/// 		A value of False (default) results in available space being determined relative to the boundaries of the
	/// 		control's scroll parent. The calendar widget will stay within its container and not overflow it.
	/// 	</param>
	/// </function>
	this.DetectBoundaries = function(val, relativeToViewport)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(relativeToViewport, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			detectBoundaries = val;
			detectBoundariesRelToViewPort = val === true && relativeToViewport === true;
		}

		return detectBoundaries;
	}

	/// <function container="Fit.Controls.DatePicker" name="Show" access="public">
	/// 	<description>
	/// 		Calling this function will open the calendar widget.
	/// 		Whether this results in picker being displayed on mobile depends on implementation.
	/// 		Often it will only work if Show() or Focused(true) is triggered by a user action such as a button click.
	/// 	</description>
	/// </function>
	this.Show = function()
	{
		if (isMobile === false)
		{
			initializeOnDemand(function()
			{
				if (Fit.Dom.IsVisible(me.GetDomElement()) === true)
				{
					var focused = Fit.Dom.GetFocused();

					datepicker.datepicker("show"); // Fails if not visible (part of render tree)

					var calendarWidget = datepickerElm;
					calendarWidget._associatedFitUiControl = me.GetId();

					// Allow light dismissable panels/callouts to prevent close/dismiss
					// when interacting with calendar widget hosted outside of panels/callouts,
					// by detecting the presence of a data-disable-light-dismiss="true" attribute.
					Fit.Dom.Data(calendarWidget, "disable-light-dismiss", "true");

					moveCalenderWidgetLocally();

					if (focused === inputTime)
						inputTime.focus();
				}
			});
		}
		else
		{
			if (inputMobile.showPicker !== undefined) // Chrome 99+, Safari 16+, Firefox 101+, Edge 99+
			{
				inputMobile.showPicker();
			}
			else // This will bring up calendar widget in older browsers on mobile devices - doesn't work on desktop browsers running on hybrid computers (laptops with a touch screen)
			{
				me.Focused(false);
				me.Focused(true);
			}
		}
	}

	/// <function container="Fit.Controls.DatePicker" name="Hide" access="public">
	/// 	<description>
	/// 		Calling this function will close the calendar widget.
	/// 		Whether this results in picker being hidden on mobile depends on implementation.
	/// 		Often it will only work if Hide() or Focused(false) is triggered by a user action such as a button click.
	/// 	</description>
	/// </function>
	this.Hide = function()
	{
		if (isMobile === false)
		{
			if (datepicker !== null)
				datepicker.datepicker("hide");
		}
		else
		{
			me.Focused(false);
		}
	}

	/// <function container="Fit.Controls.DatePicker" name="GetLocales" access="public" returns="string[]">
	/// 	<description> Returns a string array containing supported locales </description>
	/// </function>
	this.GetLocales = function()
	{
		var locales = []

		Fit.Array.ForEach(getLocales(), function(locale)
		{
			if (locale === "") // Skip "" which in jQuery UI DatePicker is equivalent to "en" and "en-US"
				return;

			Fit.Array.Add(locales, locale);
		});

		return locales;
	}

	// ============================================
	// Private
	// ============================================

	function initializeOnDemand(cb) // Callback is invoked once datepicker is loaded
	{
		Fit.Validation.ExpectFunction(cb);

		if (datepicker !== null)
		{
			cb();
			return;
		}

		// jQUery instance is shared between multiple instances of DatePicker.
		// Testing multiple instances loading simultaneously: http://fiddle.jshell.net/se7qt8fj/15/
		jquery = Fit._internal.Controls.DatePicker.jQuery;

		if (jquery === undefined)
		{
			// jQuery has not been loaded yet

			jquery = null;
			Fit._internal.Controls.DatePicker.jQuery = null;

			// jQuery UI CSS has been added to Fit.UI.css to make overriding easier.
			// To load the stylesheet dynamically, consider moving overrides from
			// Controls/DatePicker/DatePicker.css to the bottom of jquery-ui.css.
			// Fit.Loader.LoadStyleSheet(Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/jquery-ui.css");

			// Load files

			Fit.Loader.ExecuteScript(Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/external/jquery/jquery.js?cacheKey=" + Fit.GetVersion().Version, function(src)
			{
				jquery = $;
				Fit._internal.Controls.DatePicker.jQuery = $; // jQuery instance is shared between multiple instances of DatePicker
				$.noConflict(true);

				Fit.Loader.ExecuteScript(Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/jquery-ui.js?cacheKey=" + Fit.GetVersion().Version, function(src)
				{
					if (me === null)
						return; // Control was disposed while waiting for jQuery UI to load

					createDatePicker();
					cb();

				}, null, { jQuery: jquery, $: jquery }); // Make jQuery accessible to jquery-ui.js
			});
		}
		else if (jquery && jquery.datepicker)
		{
			// jQuery and UI has already been loaded

			createDatePicker();
			cb();
		}
	}

	function createDatePicker()
	{
		var jq = jquery;

		datepicker = jq(input).datepicker(
		{
			showOn: "none", // Prevents datepicker from showing when input gains focus
			showAnim: false,
			showWeek: weeks, // Buggy - does not work if week starts on sunday: https://bugs.jqueryui.com/ticket/14907
			//firstDay: 1,
			changeMonth: true,
			changeYear: true,
			dateFormat: getJqueryUiDatePickerFormat(me.Format()),
			defaultDate: null,
			onChangeMonthYear: function(year, month, dp) // Fires when changing year/month but also when simply opening calendar widget
			{
				// Retain focus when changing month or year using calendar
				// widget, which causes its DOM to be removed and replaced.
				// Focus is later returned to DatePicker, but not in time for
				// OnFocusOut. Related issue: https://github.com/Jemt/Fit.UI/issues/194
				input.focus(); // Do not use Focused(true) as it will not re-focus input, since control is already considered focused.

				if (open === true) // Remember which year and month the user navigated to
				{
					try
					{
						// Fit.Date.Parse will throw an exception if the year is below 1000
						startDate = Fit.Date.Parse(year + "-" + month + "-" + "01", "YYYY-MM-DD");
					}
					catch (err)
					{
					}
				}
			},
			onSelect: function(dateText, dp) // Notice: jQuery UI DatePicker no longer fires input.OnChange when OnSelect is registered
			{
				startDate = null;
				input.focus(); // Do not use Focused(true) as it will not re-focus input, since control is already considered focused
				input.onchange();
			},
			beforeShow: function(elm, dp)
			{
				datepickerElm = dp.dpDiv[0] || null;

				// Load locale if not already loaded

				if (datepicker.jq.datepicker.regional[locale] === null)
					return false; // Do not display - locale is currently loading (potentially started loading from another instance of DatePicker)

				if (datepicker.jq.datepicker.regional[locale] === undefined)
				{
					datepicker.jq.datepicker.regional[locale] = null; // Prevent multiple instances from loading the same locale

					Fit.Loader.ExecuteScript(Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/i18n/datepicker-" + locale + ".js?cacheKey=" + Fit.GetVersion().Version, function(src)
					{
						if (me === null)
						{
							return; // Control was disposed while waiting for locale to load
						}

						if (datepicker.jq.datepicker.regional[locale])
						{
							me.Show(); // Causes beforeShow to be fired again
						}

					}, null, { jQuery: datepicker.jq, $: datepicker.jq } // Make jQuery accessible to datepicker-xx.js
					);

					return false; // Do not display - wait for locale to load
				}

				// Retain focus when interacting with calendar widget.
				// Focus is later returned to DatePicker, but not in time for
				// OnFocusOut. Related issue: https://github.com/Jemt/Fit.UI/issues/194
				jquery(dp.dpDiv).attr("tabindex", "-1");

				// Update settings in case they were changed

				var val = me.Value();			// Always returns value in the YYYY-MM-DD[ hh:mm] format, but returns an empty string if value entered is invalid
				var actualValue = input.value;	// Since me.Value() returns an empty string if value is invalid, which might be the case if date is only partially entered, we keep the actual value entered as well

				if (updateCalConf === true) // Only update settings if actually changed, as this results in input value being updated, causing cursor position to change in IE
				{
					updateCalConf = false;

					datepicker.datepicker("option", datepicker.jq.datepicker.regional[locale]);
					datepicker.datepicker("option", "dateFormat", getJqueryUiDatePickerFormat(me.Format()));
					datepicker.datepicker("option", "showWeek", weeks);
				}

				if (startDate !== null && restoreView === true) // Restore year and month the user previously navigated to
				{
					setTimeout(function()
					{
						var year = Fit.Date.Format(startDate, "YYYY");
						var month = Fit.Date.Format(startDate, "M");

						var yearPicker = document.querySelector("select.fitui-datepicker-year");
						yearPicker.value = year;
						datepicker.jq(yearPicker).trigger("change");

						var monthPicker = document.querySelector("select.fitui-datepicker-month");
						monthPicker.value = (parseInt(month) - 1).toString();
						datepicker.jq(monthPicker).trigger("change");
					}, 0);
				}

				open = true;

				// The calendar widget clears the input value if it is invalid, which will be the case if user is in the process of entering a
				// value while the calendar widget is loading, or if a new locale is loaded with a format different from the one previously loaded.

				if (val === "" && actualValue !== "" && input.value === "")
				{
					// An invalid (possibly partial) date value was entered. The 'val' variable is empty because me.Value() returns an empty string for invalid
					// values, while 'actualValue' is not, as it originates from the input field directly. Since input.value is now empty, the calendar widget
					// has cleared the input field due to the invalid format. We therefore restore it to allow the user to proceed entering the value.
					input.value = actualValue;
				}
				else if (val !== me.Value())
				{
					// The calendar widget cleared the input field because locale loaded has a format different from the one previously loaded.
					// We restore it by re-assigning the 'val' value using me.Value(..) which uses the generic YYYY-MM-DD[ hh:mm] format.

					me._internal.ExecuteWithNoOnChange(function()
					{
						me.Value(val);
					});
				}
			},
			onClose: function(dateText, dp)
			{
				open = false;

				moveCalenderWidgetGlobally();

				// Make sure properly formatted value is used if user entered
				// the value manually and pressed ESC to close the calendar.
				// Calendar widget returns focus if closed using ESC key.
				input.onblur();

				// Calendar remains open if control lose focus while year or month picker is open (has focus).
				// In this case OnBlur does not fire for the control since elements within the calendar
				// widget is not part of the control (it lives in the root of the document), and therefore
				// does not inherit the automatic invocation of OnBlur and OnFocus.
				// This is unlikely to happen since it would require focus to programmatically be changed
				// while user is interacting with the year or month pickers. But if it does happen, we make
				// sure to fire OnBlur when the calendar widget closes, which happens when the user interacts
				// with something else on the page.
				// If the user decides to return focus to the control, it will not cause OnFocus to fire again,
				// without firing OnBlur in-between, thanks to the 'focused' flag.
				if (focused === true && Fit.Dom.GetFocused() !== input)
				{
					me._internal.FireOnBlur();
				}
			}
		});

		datepicker.jq = jq;
	}

	function moveCalenderWidgetLocally()
	{
		if (me._internal.ReduceDocumentRootPollution !== true && Fit._internal.ControlBase.ReduceDocumentRootPollution !== true)
		{
			return;
		}

		// We want the benefits of a connected calendar control (connected to input control),
		// but do not want it rendered in the root of the document. It pollutes the global scope,
		// and it doesn't work with dialogs with light dismiss, since interacting with the calendar
		// in the document root will close/dismiss such callouts.
		// Ideally we should move the calendar widget inside the control's container, but then it
		// would be affected by Fit.UI's styling which would cause issues with the visual appearance.
		// Instead we move it below the DatePicker control and make sure it is still properly positioned.
		// For more details see https://github.com/Jemt/Fit.UI/issues/116

		var selfDom = me.GetDomElement();
		var calendarWidget = datepickerElm;

		Fit.Dom.InsertAfter(selfDom, calendarWidget);

		var below = true;
		var forceFixed = false;

		var overflowParent = (detectBoundariesRelToViewPort === false ? Fit.Dom.GetOverflowingParent(selfDom) : null); // Contrary to GetScrollParent(..), GetOverflowingParent(..) also take overflow:hidden into account
		var offsetParent = selfDom.offsetParent;

		if (overflowParent !== null && overflowParent !== offsetParent && Fit.Dom.Contained(overflowParent, offsetParent) === false)
		{
			// Control is contained in a parent with overflow:hidden|scroll|auto, but offsetParent,
			// which calendar widget is positioned against, it outside of overflow/scroll parent.
			// With position:absolute this would result in calendar widget scrolling along with the document,
			// but not when scrolling the scroll container, which would confuse the user. Force position:fixed.
			forceFixed = true;
		}

		if (detectBoundaries === true)
		{
			// Determine whether to position calendar widget below or above control

			var visualBoundaryBottomPosition = overflowParent !== null && forceFixed === false ? Fit.Dom.GetBoundingPosition(overflowParent).Y + overflowParent.offsetHeight : Fit.Browser.GetViewPortDimensions().Height;
			var controlBottomPosition = Fit.Dom.GetBoundingPosition(selfDom).Y + selfDom.offsetHeight;
			var spaceBelowControl = visualBoundaryBottomPosition - controlBottomPosition;
			var verticalScrollbarHeight = overflowParent !== null && detectBoundariesRelToViewPort === false && forceFixed === false ? Fit.Dom.GetScrollBars(overflowParent).Horizontal.Size : 0; // No need to compensate for scrollbars in viewport as Fit.Browser.GetViewPortDimensions() exclude these by default

			below = (spaceBelowControl >= calendarWidget.offsetHeight + verticalScrollbarHeight);
		}

		if (below === true) // Position calendar widget below control
		{
			if (detectBoundariesRelToViewPort === true || forceFixed === true) // Position relative to viewport (position:fixed) - can escape a container with overflow:hidden|scroll|auto
			{
				calendarWidget.style.position = "fixed";
				calendarWidget.style.left =  Fit.Dom.GetPosition(selfDom, true).X + "px";
				calendarWidget.style.top = (Fit.Dom.GetPosition(selfDom, true).Y + selfDom.offsetHeight) + "px";
			}
			else // Position relative to offsetParent (position:absolute)
			{
				calendarWidget.style.position = "absolute";
				calendarWidget.style.left = selfDom.offsetLeft + "px";
				calendarWidget.style.top = (selfDom.offsetTop + selfDom.offsetHeight) + "px";
			}
		}
		else // Position calendar widget above control
		{
			if (detectBoundariesRelToViewPort === true || forceFixed === true) // Position relative to viewport (position:fixed) - can escape a container with overflow:hidden|scroll|auto
			{
				calendarWidget.style.position = "fixed";
				calendarWidget.style.left =  Fit.Dom.GetPosition(selfDom, true).X + "px";
				calendarWidget.style.top = (Fit.Dom.GetPosition(selfDom, true).Y - calendarWidget.offsetHeight) + "px"
			}
			else // Position relative to offsetParent (position:absolute)
			{
				calendarWidget.style.position = "absolute";
				calendarWidget.style.left = selfDom.offsetLeft + "px";
				calendarWidget.style.top = (selfDom.offsetTop - calendarWidget.offsetHeight) + "px";
			}
		}
	}

	function moveCalenderWidgetGlobally() // Undo everything done in moveCalenderWidgetLocally()
	{
		if (me._internal.ReduceDocumentRootPollution !== true && Fit._internal.ControlBase.ReduceDocumentRootPollution !== true)
		{
			return;
		}

		var calendarWidget = datepickerElm;
		Fit.Dom.Add(document.body, calendarWidget);

		calendarWidget.style.position = ""; // "absolute"
		calendarWidget.style.left = "";
		calendarWidget.style.top = "";
	}

	function getJqueryUiDatePickerFormat()
	{
		return me.Format().replace(/D/g, "d").replace(/M/g, "m").replace(/YY/g, "y");
	}

	function getFitUiDateFormat(jqueryFormat) // jquery.datepicker.regional["da"].dateFormat
	{
		Fit.Validation.ExpectString(jqueryFormat);
		return jqueryFormat.replace(/d/g, "D").replace(/m/g, "M").replace(/y/g, "YY");
	}

	function getJqueryDateFormatFromLocale(lc)
	{
		Fit.Validation.ExpectString(lc);

		var locales = getLocales();

		if (locales[lc] !== undefined)
			return locales[lc];

		return null;
	}

	function getLocales()
	{
		// List of locales was generated this way:
		// 1) Make sure to prepare locales first like described in README-FitUiCustomJqueryUiBuild.txt
		// 2) Merge all i18n files using Bash:
		//    # cd Resources/JqueryUI-1.11.4.custom/i18n
		//    # echo "" > merged.js && for file in `ls datepicker-*js` ; do cat $file >> merged.js ; done
		// 3) Execute JS code in a page running Fit.UI:
		//    # var jq = Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/external/jquery/jquery.js";
		//    # var ui = Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/jquery-ui.js?FitJquery";
		//    # var ln = Fit.GetUrl() + "/Resources/JqueryUI-1.11.4.custom/i18n/merged.js?FitJquery";
		//    # Fit.Loader.LoadScripts([ { source: jq, loaded: function(src) { FitJquery = $; $.noConflict(true); } }, { source: ui }, { source: ln } ], function(cfgs)
		//    # {
		//    # 	var regional = FitJquery.datepicker.regional;
		//    # 	var res = {};
		//    # 	for (lang in regional)
		//    # 	{
		//    # 		if (regional[lang].dateFormat) // Make sure property is a language
		//    # 			res[lang] = regional[lang].dateFormat;
		//    # 	}
		//    #
		//    # 	console.log(JSON.stringify(res));
		//    # });
		// 4) Insert JSON written to console into variable below.

		return {"":"mm/dd/yy","en":"mm/dd/yy","en-US":"mm/dd/yy","da":"dd-mm-yy","af":"dd/mm/yy","ar-DZ":"dd/mm/yy","ar":"dd/mm/yy","az":"dd.mm.yy","be":"dd.mm.yy","bg":"dd.mm.yy","bs":"dd.mm.yy","ca":"dd/mm/yy","cs":"dd.mm.yy","cy-GB":"dd/mm/yy","de":"dd.mm.yy","el":"dd/mm/yy","en-AU":"dd/mm/yy","en-GB":"dd/mm/yy","en-NZ":"dd/mm/yy","eo":"dd/mm/yy","es":"dd/mm/yy","et":"dd.mm.yy","eu":"yy-mm-dd","fa":"yy/mm/dd","fi":"d.m.yy","fo":"dd-mm-yy","fr-CA":"yy-mm-dd","fr-CH":"dd.mm.yy","fr":"dd/mm/yy","gl":"dd/mm/yy","he":"dd/mm/yy","hi":"dd/mm/yy","hr":"dd.mm.yy.","hu":"yy.mm.dd.","hy":"dd.mm.yy","id":"dd/mm/yy","is":"dd.mm.yy","it-CH":"dd.mm.yy","it":"dd/mm/yy","ja":"yy/mm/dd","ka":"dd-mm-yy","kk":"dd.mm.yy","km":"dd-mm-yy","ko":"yy. m. d.","ky":"dd.mm.yy","lb":"dd.mm.yy","lt":"yy-mm-dd","lv":"dd.mm.yy","mk":"dd.mm.yy","ml":"dd/mm/yy","ms":"dd/mm/yy","nb":"dd.mm.yy","nl-BE":"dd/mm/yy","nl":"dd-mm-yy","nn":"dd.mm.yy","no":"dd.mm.yy","pl":"dd.mm.yy","pt-BR":"dd/mm/yy","pt":"dd/mm/yy","rm":"dd/mm/yy","ro":"dd.mm.yy","ru":"dd.mm.yy","sk":"dd.mm.yy","sl":"dd.mm.yy","sq":"dd.mm.yy","sr":"dd.mm.yy","sr-SR":"dd.mm.yy","sv":"yy-mm-dd","ta":"dd/mm/yy","th":"dd/mm/yy","tj":"dd.mm.yy","tr":"dd.mm.yy","uk":"dd.mm.yy","vi":"dd/mm/yy","zh-CN":"yy-mm-dd","zh-HK":"dd-mm-yy","zh-TW":"yy/mm/dd"};
	}

	function getDatePlaceholder()
	{
		return (placeholderDate !== null ? placeholderDate : Fit.Date.Format(new Date(), format));
	}

	function getTimePlaceholder()
	{
		return (placeholderTime !== null ? placeholderTime : Fit.Date.Format(new Date(), "hh:mm"));
	}

	function setLocale(val)
	{
		Fit.Validation.ExpectString(val);

		var newFormat = getJqueryDateFormatFromLocale(val); // Null if locale does not exist

		if (newFormat === null)
			Fit.Validation.ThrowError("Unknown locale '" + val + "'");

		var wasOpen = open;

		if (wasOpen === true)
		{
			restoreView = true;
			me.Hide();
		}

		locale = val;
		updateCalConf = true; // Update calendar widget settings

		if (formatEnforced === false)
		{
			setFormat(getFitUiDateFormat(newFormat));
		}

		if (wasOpen === true)
		{
			me.Show();
			restoreView = false;
		}
	}

	function setFormat(val)
	{
		Fit.Validation.ExpectString(val);

		// Validate format

		try
		{
			// Notice: Fit.Date.Parse(..) produces a DateTime object that contains
			// current Hours:Minutes:Seconds unless explicitly set, so we need to set
			// those to prevent 'date' and 'parsed' variables from differing one second.

			var date = Fit.Date.Parse("2016-06-24 00:00:00", "YYYY-MM-DD hh:mm:ss");
			var dateStr = Fit.Date.Format(date, val + " hh:mm:ss");
			var parsed = Fit.Date.Parse(dateStr, val + " hh:mm:ss");

			if (date.getTime() !== parsed.getTime())
				throw "Invalid"; // Catched below and re-thrown
		}
		catch (err)
		{
			Fit.Validation.ThrowError("Invalid date format '" + val + "'");
		}

		// Set format

		var wasOpen = open;

		if (wasOpen === true)
		{
			restoreView = true;
			me.Hide();
		}

		var curDate = me.Date(); // The format is used by Value(), which is called by Date(), to parse the date entered - get DateTime value before changing format

		format = val;
		updateCalConf = true; // Update calendar widget settings
		input.placeholder = getDatePlaceholder();

		// Update input with new format
		me._internal.ExecuteWithNoOnChange(function()
		{
			if (curDate !== null)
			{
				// Update input directly rather than using Value(..) since the input field still contains
				// the old value which is formatted using the old format. This will cause a call to Value(..)
				// to fail since it cannot parse the old string value using the new format applied.
				// Notice that we only change the format of the DateTime value - not the actual date or time.
				// Therefore, there is no need to update curVal.

				input.value = Fit.Date.Format(curDate, format);
				preVal = input.value;
			}
		});

		if (wasOpen === true)
		{
			me.Show();
			restoreView = false;
		}
	}

	function localize()
	{
		if (localeEnforced === true)
			return;

		var newLocale = Fit.Internationalization.Locale();
		var key = null;

		// Transform Fit.UI's locale format (e.g. en or en_gb) to jQuery's locale format (e.g. en or en-GB)

		if (newLocale.indexOf("_") !== -1) // E.g. de_AT
		{
			var info = newLocale.split("_");
			key = info[0].toLowerCase() + "-" + info[1].toUpperCase();
		}
		else // e.g. de
		{
			key = newLocale.toLowerCase();
		}

		// Update locale

		// Make sure locale exists - otherwise setLocale throws an error
		if (Fit.Array.Contains(Fit.Array.GetKeys(getLocales()), key) === true)
		{
			setLocale(key);
		}
	}

	init();
}

Fit._internal.Controls.DatePicker = {};

/// <container name="Fit.Controls.Dialog" extends="Fit.Controls.Component">
/// 	Simple Dialog control with support for Fit.UI buttons.
/// </container>

/// <function container="Fit.Controls.Dialog" name="Dialog" access="public">
/// 	<description> Create instance of Dialog control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.Dialog = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	var me = this;
	var dialog = me.GetDomElement();
	var focusTrapStart = null;
	var focusTrapEnd = null;
	var titleContainer = null;
	var titleText = null;
	var titleButtons = null;
	var cmdMaximize = null;
	var cmdDismiss = null;
	var content = null;
	var iframe = null;
	var resizer = null;
	var buttons = null;
	var modal = false;
	var layer = null;

	var draggable = null;
	var suppressPositioning = false;

	var width = null;
	var minWidth = null;
	var maxWidth = null;

	var height = null;
	var minHeight = null;
	var maxHeight = null;
	var mutationObserverId = -1;
	var resizeHandlerId = -1;

	var onDismissHandlers = [];
	var onCloseHandlers = [];
	var isClosing = false;

	// ============================================
	// Init
	// ============================================

	function init()
	{
		Fit.Dom.AddClass(dialog, "FitUiControl");
		Fit.Dom.AddClass(dialog, "FitUiControlDialog");
		Fit.Dom.Data(dialog, "framed", "false");
		Fit.Dom.Data(dialog, "maximized", "false");
		Fit.Dom.Data(dialog, "resizable", "false");

		focusTrapStart = document.createElement("div");
		focusTrapStart.tabIndex = 0;
		Fit.Dom.Add(dialog, focusTrapStart);

		focusTrapEnd = document.createElement("div");
		focusTrapEnd.tabIndex = 0;
		Fit.Dom.Add(dialog, focusTrapEnd);

		content = createContentElement();
		Fit.Dom.InsertAfter(focusTrapStart, content);

		layer = document.createElement("div");
		Fit.Dom.AddClass(layer, "FitUiControlDialogModalLayer");

		// Focus first button when clicking dialog or modal layer

		Fit.Events.AddHandler(dialog, "click", function(e)
		{
			if (me === null)
				return; // Dialog was disposed when a button was clicked

			var target = Fit.Events.GetTarget(e);

			// Do not focus first button when a (any) button within the button area was clicked.
			// Button clicked might have been e.g. second button, which perhaps caused dialog to remain open,
			// in which case we should not change focus. The button clicked could also have changed focus in
			// which case we should not intervene. In case button did not change focus, it will remain focused
			// of course since it was clicked.
			if (buttons !== null && Fit.Dom.Contained(buttons, target))
				return;

			// Do not focus first button if maximize button or dismiss button was clicked
			if ((cmdMaximize !== null && (target === cmdMaximize.GetDomElement() || Fit.Dom.Contained(cmdMaximize.GetDomElement(), target) === true)) ||
				(cmdDismiss !== null && (target === cmdDismiss.GetDomElement() || Fit.Dom.Contained(cmdDismiss.GetDomElement(), target) === true)))
			{
				return;
			}

			if (buttons !== null && (Fit.Dom.GetFocused() === null || Fit.Dom.Contained(dialog, Fit.Dom.GetFocused()) === false))
				buttons.children[0].focus();
		});

		Fit.Events.AddHandler(layer, "click", function(e)
		{
			if (buttons !== null && (Fit.Dom.GetFocused() === null || Fit.Dom.Contained(dialog, Fit.Dom.GetFocused()) === false))
				buttons.children[0].focus();
		});

		// Keep tab navigation within modal dialog

		Fit.Events.AddHandler(dialog, "keydown", function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			if (modal === true && ev.keyCode === 9) // Tab key
			{
				if (ev.shiftKey === true && Fit.Events.GetTarget(ev) === focusTrapStart) // Tabbing backwards
				{
					Fit.Events.PreventDefault(ev);
				}
				else if (ev.shiftKey === false && Fit.Events.GetTarget(ev) === focusTrapEnd) // Tabbing forward
				{
					Fit.Events.PreventDefault(ev);
				}
			}
		});
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.Dialog" name="Title" access="public" returns="string | null">
	/// 	<description> Get/set title - returns null if not set, and null can be passed to remove title </description>
	/// 	<param name="val" type="string | null" default="undefined"> If specified, dialog title is updated with specified value </param>
	/// </function>
	this.Title = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (val !== undefined) // Allow null to remove title
		{
			if (val === null && titleContainer !== null) // Remove title
			{
				if (titleButtons !== null || draggable !== null) // Clear title but keep title bar used for buttons and drag support
				{
					Fit.Dom.Text(titleText, "");
				}
				else // Remove title bar
				{
					Fit.Dom.Remove(titleContainer);
					titleContainer = null;
					titleText = null;

					setContentHeight();
					updatePosition();
				}
			}
			else // Set title
			{
				if (titleContainer === null)
				{
					titleContainer = document.createElement("div");
					Fit.Dom.AddClass(titleContainer, "FitUiControlDialogTitle");
					Fit.Dom.InsertAfter(focusTrapStart, titleContainer);

					titleText = document.createElement("div");
					Fit.Dom.AddClass(titleText, "FitUiControlDialogTitleText");
					Fit.Dom.Add(titleContainer, titleText);
				}

				Fit.Dom.Text(titleText, val);

				setContentHeight();
				updatePosition();
			}
		}

		return (titleText !== null ? Fit.Dom.Text(titleText) : null);
	}

	/// <function container="Fit.Controls.Dialog" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog width is updated to specified value. A value of -1 resets width (auto sizing). </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog width is updated to specified CSS unit </param>
	/// </function>
	this.Width = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		var defaultValue = { Value: -1, Unit: "px" };

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				width = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.width = width.Value + width.Unit;

				// Disable default min-width and default max-width from CSS when width alone is set, unless Resizable is
				// enabled, in which case we must prevent dialog from "collapsing" completely due to resizing, or become
				// larger than the viewport. When resizing is enabled, appropriate values for min-width and max-width are
				// set from CSS, but can be overridden using MinimumWidth(..) and MaximumWidth(..)

				if (minWidth === null && me.Resizable() === false)
				{
					dialog.style.minWidth = "0";
				}

				if (maxWidth === null && me.Resizable() === false)
				{
					dialog.style.maxWidth = "none";
				}
			}
			else
			{
				width = null;
				dialog.style.width = "";

				if (minWidth === null)
				{
					dialog.style.minWidth = "";
				}

				if (maxWidth === null)
				{
					dialog.style.maxWidth = "";
				}
			}

			updatePosition();
		}

		return (width !== null ? width : defaultValue);
	}

	/// <function container="Fit.Controls.Dialog" name="MinimumWidth" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog minimum width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog minimum width is updated to specified value. A value of -1 resets minimum width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog minimum width is updated to specified CSS unit </param>
	/// </function>
	this.MinimumWidth = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		// defaultValue must match min-width in Dialog.css
		var defaultValue = { Value: 280, Unit: "px" };
		var defaultValueResizable = { Value: 15, Unit: "em" };

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				minWidth = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.minWidth = minWidth.Value + minWidth.Unit;
			}
			else
			{
				minWidth = null;
				dialog.style.minWidth = (width !== null && me.Resizable() === false ? "0" : ""); // Apply "0" (no min-width) if width is set, to override min-width from CSS - similar logic found in Width(..)
			}

			updatePosition();
		}

		if (minWidth === null && me.Resizable() === true)
		{
			return defaultValueResizable;
		}

		return (minWidth !== null ? minWidth : (width !== null ? width : defaultValue));
	}

	/// <function container="Fit.Controls.Dialog" name="MaximumWidth" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog maximum width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog maximum width is updated to specified value. A value of -1 resets maximum width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog maximum width is updated to specified CSS unit </param>
	/// </function>
	this.MaximumWidth = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		// defaultValue must match max-width in Dialog.css
		var defaultValue = { Value: 800, Unit: "px" };
		var defaultValueResizable = { Value: 100, Unit: "%" };

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				maxWidth = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.maxWidth = maxWidth.Value + maxWidth.Unit;
			}
			else
			{
				maxWidth = null;
				dialog.style.maxWidth = (width !== null && me.Resizable() === false ? "none" : ""); // Apply "none" (no max-width) if width is set, to override max-width from CSS - similar logic found in Width(..)
			}

			updatePosition();
		}

		if (maxWidth === null && me.Resizable() === true)
		{
			return defaultValueResizable;
		}

		return (maxWidth !== null ? maxWidth : (width !== null ? width : defaultValue));
	}

	/// <function container="Fit.Controls.Dialog" name="Height" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog height is updated to specified value. A value of -1 resets height to default. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog width is updated to specified CSS unit </param>
	/// </function>
	this.Height = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		// defaultValue must match height in Dialog.css
		var defaultValue = (iframe !== null ? { Value: 40, Unit: "%" } : { Value: -1, Unit: "px" });

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				height = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.height = height.Value + height.Unit;
			}
			else
			{
				height = null;
				dialog.style.height = "";
			}

			setContentHeight();
			updatePosition();
		}

		return (height !== null ? height : defaultValue);
	}

	/// <function container="Fit.Controls.Dialog" name="MinimumHeight" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog minimum height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog minimum height is updated to specified value. A value of -1 resets minimum height. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog minimum height is updated to specified CSS unit </param>
	/// </function>
	this.MinimumHeight = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		// defaultValue must match min-height in Dialog.css (which is not defined)
		var defaultValue = { Value: -1, Unit: "px" };
		var defaultValueResizable = { Value: 10, Unit: "em" };

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				minHeight = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.minHeight = minHeight.Value + minHeight.Unit;
			}
			else
			{
				minHeight = null;
				dialog.style.minHeight = "";
			}

			setContentHeight();
			updatePosition();
		}

		if (minHeight === null && me.Resizable() === true)
		{
			return defaultValueResizable;
		}

		return (minHeight !== null ? minHeight : defaultValue);
	}

	/// <function container="Fit.Controls.Dialog" name="MaximumHeight" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set dialog maximum height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, dialog maximum height is updated to specified value. A value of -1 resets maximum height. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, dialog maximum height is updated to specified CSS unit </param>
	/// </function>
	this.MaximumHeight = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		// defaultValue must match max-height in Dialog.css (which is not defined)
		var defaultValue = { Value: -1, Unit: "px" };
		var defaultValueResizable = { Value: 100, Unit: "%" };

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				maxHeight = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				dialog.style.maxHeight = maxHeight.Value + maxHeight.Unit;
			}
			else
			{
				maxHeight = null;
				dialog.style.maxHeight = "";
			}

			setContentHeight();
			updatePosition();
		}

		if (maxHeight === null && me.Resizable() === true)
		{
			return defaultValueResizable;
		}

		return (maxHeight !== null ? maxHeight : defaultValue);
	}

	/// <function container="Fit.Controls.Dialog" name="Modal" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether dialog is modal or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables modal mode, False disables it </param>
	/// </function>
	this.Modal = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val !== modal && me.IsOpen() === true)
			{
				if (val === true)
				{
					layer.style.zIndex = dialog.style.zIndex; // Update in case dialog was/is draggable and it was brought to front (z-index set by Fit.DragDrop.Draggable when Draggable is enabled)
					Fit.Dom.InsertBefore(dialog, layer);

					if (me.IsOpen() === true)
					{
						me.BringToFront();
					}
				}
				else
				{
					Fit.Dom.Remove(layer);
				}
			}

			modal = val;
		}

		return modal;
	}

	/// <function container="Fit.Controls.Dialog" name="Content" access="public" returns="string">
	/// 	<description> Get/set dialog content </description>
	/// 	<param name="val" type="string" default="undefined"> If specified, dialog content is updated with specified value </param>
	/// </function>
	this.Content = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			content.innerHTML = val;

			if (iframe !== null)
			{
				Fit.Dom.Data(dialog, "framed", "false");
				iframe = null;
			}

			setContentHeight();
			updatePosition();
		}

		return content.innerHTML;
	}

	/// <function container="Fit.Controls.Dialog" name="GetContentDomElement" access="public" returns="DOMElement">
	/// 	<description> Get dialog content element </description>
	/// </function>
	this.GetContentDomElement = function()
	{
		return content;
	}

	/// <function container="Fit.Controls.Dialog" name="ContentUrl" access="public" returns="string | null">
	/// 	<description> Get/set content URL - returns null if not set </description>
	/// 	<param name="url" type="string" default="undefined"> If specified, dialog is updated with content from specified URL </param>
	/// 	<param name="onLoadHandler" type="Fit.Controls.DialogTypeDefs.DialogEventHandler" default="undefined">
	/// 		If specified, callback is invoked when content has been loaded.
	/// 		Instance of Dialog is passed as an argument.
	/// 	</param>
	/// </function>
	this.ContentUrl = function(url, onLoadHandler)
	{
		Fit.Validation.ExpectString(url, true);
		Fit.Validation.ExpectFunction(onLoadHandler, true);

		if (Fit.Validation.IsSet(url) === true)
		{
			iframe = Fit.Dom.CreateElement("<iframe src='" + url + "' scrolling='yes' frameBorder='0' allowtransparency='true'></iframe>");

			if (Fit.Validation.IsSet(onLoadHandler) === true)
			{
				Fit.Events.AddHandler(iframe, "load", function(e)
				{
					onLoadHandler(me);
				});
			}

			content.innerHTML = "";
			Fit.Dom.Add(content, iframe);
			Fit.Dom.Data(dialog, "framed", "true");
		}

		return (iframe !== null ? iframe.src : null);
	}

	/// <function container="Fit.Controls.Dialog" name="Maximized" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether dialog is maximized or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True maximizes dialog while False restores it </param>
	/// </function>
	this.Maximized = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			Fit.Dom.Data(dialog, "maximized", (val === true ? "true" : "false"));

			if (cmdMaximize !== null)
			{
				if (Fit.Dom.Data(dialog, "maximized") === "true")
				{
					cmdMaximize.Icon("compress");
				}
				else
				{
					cmdMaximize.Icon("expand");
				}
			}

			me._internal.Repaint(function()
			{
				// Safari on iOS: Scroll does not work after resizing the
				// dialog. Temporarily hiding the iframe solves the problem.
				var b = Fit.Browser.GetInfo();
				if (iframe !== null && b.Name === "Safari" && b.IsMobile === true)
				{
					iframe.style.display = "none";
					setTimeout(function() { iframe.style.display = ""; }, 0);
				}
			});
		}

		return (Fit.Dom.Data(dialog, "maximized") === "true");
	}

	/// <function container="Fit.Controls.Dialog" name="Maximizable" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether dialog is maximizable or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, a value of True makes dialog maximizable by adding a maximize button while False disables it </param>
	/// </function>
	this.Maximizable = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && cmdMaximize === null)
			{
				cmdMaximize = new Fit.Controls.Button();
				cmdMaximize.Icon((me.Maximized() === false ? "expand" : "compress"));
				cmdMaximize.ReturnFocus(true);
				cmdMaximize.OnClick(function(sender)
				{
					if (me.Maximized() === false)
					{
						me.Maximized(true); // Also updates button icon
					}
					else
					{
						me.Maximized(false); // Also updates button icon
					}
				});

				updateTitleButtons();
			}
			else if (val === false && cmdMaximize !== null)
			{
				cmdMaximize.Dispose();
				cmdMaximize = null;
				updateTitleButtons();
			}
		}

		return (cmdMaximize !== null);
	}

	/// <function container="Fit.Controls.Dialog" name="Dismissible" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether dialog is dismissible or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, a value of True makes dialog dismissible by adding a close button while False disables it </param>
	/// 	<param name="disposeOnDismiss" type="boolean" default="undefined"> If defined, a value of True results in dialog being disposed (destroyed) when closed using dismiss/close button </param>
	/// </function>
	this.Dismissible = function(val, disposeOnDismiss)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(disposeOnDismiss, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && cmdDismiss === null)
			{
				cmdDismiss = new Fit.Controls.Button();
				cmdDismiss.Icon("close");
				cmdDismiss.OnClick(function(sender)
				{
					var canceled = false;

					Fit.Array.ForEach(onDismissHandlers, function(cb)
					{
						if (cb(me) === false)
						{
							canceled = true;
							return false; // Break loop
						}
					});

					if (canceled === true)
					{
						return;
					}

					if (me !== null) // me is null if dialog was disposed from within OnDismiss handler
					{
						if (disposeOnDismiss === true)
						{
							me.Dispose();
						}
						else
						{
							me.Close();
						}
					}
				});

				updateTitleButtons();
			}
			else if (val === false && cmdDismiss !== null)
			{
				cmdDismiss.Dispose();
				cmdDismiss = null;
				updateTitleButtons();
			}
		}

		return (cmdDismiss !== null);
	}

	/// <function container="Fit.Controls.Dialog" name="Draggable" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether dialog can be moved around on screen </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, a value of True enables dragging, False disables it (default) </param>
	/// </function>
	this.Draggable = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && draggable === null)
			{
				if (me.Title() === null) // Ensure title element
				{
					me.Title("");
				}

				draggable = new Fit.DragDrop.Draggable(me.GetDomElement(), titleText);
				draggable.BringToFrontOnActivation(true);
				draggable.ReturnFocus(true);
				draggable.OnDragStart(function(elm)
				{
					if (me.Maximized() === true)
					{
						return false; // Prevent dragging while maximized
					}

					suppressPositioning = true;
				});
			}
			else if (val === false && draggable !== null)
			{
				draggable.Dispose();
				draggable = null;
				suppressPositioning = false;

				if (me.Title() === "")
				{
					me.Title(null); // Remove title element - will remain enabled if buttons (dismiss/maximize) are present though
				}

				updatePosition();
			}
		}

		return (draggable !== null);
	}

	/// <function container="Fit.Controls.Dialog" name="Resizable" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether dialog can be resized by the user </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, a value of True enables resizing, False disables it (default) </param>
	/// </function>
	this.Resizable = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && resizer === null)
			{
				resizer = createResizerElement();
				Fit.Dom.InsertAfter(content, resizer);
				Fit.Dom.Data(dialog, "resizable", "true");
			}
			else if (val === false && resizer !== null)
			{
				Fit.Dom.Remove(resizer);
				resizer = null;
				Fit.Dom.Data(dialog, "resizable", "false");

				// Undo width/height set by user
				me.Width(me.Width().Value, me.Width().Unit);
				me.Height(me.Height().Value, me.Height().Unit);
			}
		}

		return (resizer !== null);
	}

	/// <function container="Fit.Controls.Dialog" name="Reset" access="public">
	/// 	<description> Reset custom size (Resizable enabled) and position (Draggable enabled) </description>
	/// </function>
	this.Reset = function()
	{
		if (draggable !== null)
		{
			suppressPositioning = false;
			updatePosition();
		}

		if (me.Resizable() === true)
		{
			me.Width(me.Width().Value, me.Width().Unit);
			me.Height(me.Height().Value, me.Height().Unit);
		}
	}

	/// <function container="Fit.Controls.Dialog" name="AddButton" access="public">
	/// 	<description> Add button to dialog </description>
	/// 	<param name="btn" type="Fit.Controls.Button"> Instance of Fit.Controls.Button </param>
	/// </function>
	this.AddButton = function(btn)
	{
		Fit.Validation.ExpectInstance(btn, Fit.Controls.Button);

		if (buttons === null)
		{
			buttons = document.createElement("div");
			Fit.Dom.AddClass(buttons, "FitUiControlDialogButtons");
			Fit.Dom.InsertBefore(focusTrapEnd, buttons);
		}

		Fit.Dom.Add(buttons, btn.GetDomElement());

		setContentHeight();
		updatePosition();
	}

	/// <function container="Fit.Controls.Dialog" name="RemoveButton" access="public">
	/// 	<description> Remove button from dialog </description>
	/// 	<param name="btn" type="Fit.Controls.Button"> Instance of Fit.Controls.Button </param>
	/// 	<param name="dispose" type="boolean" default="undefined"> If defined, a value of True results in button being disposed (destroyed) </param>
	/// </function>
	this.RemoveButton = function(btn, dispose)
	{
		Fit.Validation.ExpectInstance(btn, Fit.Controls.Button);
		Fit.Validation.ExpectBoolean(dispose, true);

		if (buttons === null)
			return;

		Fit.Array.ForEach(buttons.children, function(elm)
		{
			if (elm === btn.GetDomElement())
			{
				if (dispose === true)
				{
					btn.Dispose();
				}
				else
				{
					Fit.Dom.Remove(btn.GetDomElement());
				}

				if (buttons.children.length === 0)
				{
					Fit.Dom.Remove(buttons);
					buttons = null;
				}

				setContentHeight();
				updatePosition();

				return false; // Break loop
			}
		});
	}

	/// <function container="Fit.Controls.Dialog" name="RemoveAllButtons" access="public">
	/// 	<description> Remove all buttons from dialog </description>
	/// 	<param name="dispose" type="boolean" default="undefined"> If defined, a value of True results in buttons being disposed (destroyed) </param>
	/// </function>
	this.RemoveAllButtons = function(dispose)
	{
		Fit.Validation.ExpectBoolean(dispose, true);

		if (buttons === null)
			return;

		Fit.Array.ForEach(Fit.Array.Copy(buttons.children), function(buttonElm) // Using Copy(..) since code modifies children collection
		{
			if (dispose === true)
			{
				buttonElm._internal.Instance.Dispose();
			}
			else
			{
				Fit.Dom.Remove(buttonElm);
			}
		});

		Fit.Dom.Remove(buttons);
		buttons = null;

		setContentHeight();
		updatePosition();
	}

	/// <function container="Fit.Controls.Dialog" name="IsOpen" access="public" returns="boolean">
	/// 	<description> Get flag indicating whether dialog is open or not </description>
	/// </function>
	this.IsOpen = function()
	{
		return Fit.Dom.IsRooted(dialog);
	}

	/// <function container="Fit.Controls.Dialog" name="BringToFront" access="public">
	/// 	<description> Bring draggable dialog to front </description>
	/// </function>
	this.BringToFront = function()
	{
		if (draggable !== null)
		{
			draggable.BringToFront();
		}
		else
		{
			Fit.Array.ForEach(document.querySelectorAll("div.FitUiControlDialog"), function(diaElm)
			{
				var dia = Fit.Controls.Find(diaElm.id);

				if (dia.Draggable() === true)
				{
					dialog.style.zIndex = Fit.DragDrop.Draggable.GetNextZindex();
					return false;
				}
			});
		}

		layer.style.zIndex = dialog.style.zIndex; // Ensure modal layer always remains exactly behind dialog with the same z-index value
	}

	/// <function container="Fit.Controls.Dialog" name="Open" access="public">
	/// 	<description> Open dialog </description>
	/// 	<param name="renderTarget" type="DOMElement" default="undefined">
	/// 		Optional render target which can be used to render dialog to specific
	/// 		container rather than the root of the body element. This allows for
	/// 		the dialog to inherit styles from the specified render target.
	/// 	</param>
	/// </function>
	this.Open = function(renderTarget)
	{
		Fit.Validation.ExpectDomElement(renderTarget, true);

		if (me.IsOpen() === true)
			return;

		me.BringToFront();

		if (modal === true)
		{
			layer.style.zIndex = dialog.style.zIndex; // Ensure modal layer always remains exactly behind dialog with the same z-index value
			Fit.Dom.Add(renderTarget || document.body, layer);
		}

		Fit.Dom.Add(renderTarget || document.body, dialog);

		setContentHeight();
		updatePosition();

		// Make dimensions and position adjust to dynamic content (added/removed/manipulated through DOM or Fit.Template).
		// Notice that mutation observer will not be triggered if title is set or buttons have been added,
		// and content is being updated using DOM, e.g. through GetContentDomElement() or an instance of
		// Fit.Template rendered to the content DOM element.
		// The reason for this can be found in setContentHeight() where a fixed height is being calculated,
		// based on whether title and/or button(s) have been added, and assigned to the content element of the
		// dialog. Therefore, in this case, changing the content of the content element through DOM (or an
		// instance of Fit.Template) will not change the dimension of the content element, and therefore not
		// trigger the mutation observer.
		// The external code using the dialog must call a public function which in turn calls setContentHeight(),
		// to have the dialog resize to its newly updated content - e.g. close and re-open the dialog, or update
		// the title like dia.Title(dia.Title()).
		// Making the observer monitor the entire dialog's DOM using the 'deep' flag would solve this problem in
		// most causes, although not when dimensions change using styling (e.g. .style.height).
		// Also, it would be more expensive, and could trigger height calculation even when not necessary, e.g.
		// when changing a selection in a drop down control which changes the DOM.
		mutationObserverId = Fit.Events.AddMutationObserver(dialog, function(elm)
		{
			setContentHeight();
			updatePosition();
		});

		resizeHandlerId = Fit.Events.AddHandler(window, "resize", function(e)
		{
			setContentHeight();
			updatePosition();
		});
	}

	/// <function container="Fit.Controls.Dialog" name="Close" access="public">
	/// 	<description> Close dialog </description>
	/// </function>
	this.Close = function()
	{
		if (me.IsOpen() === false)
			return;

		if (isClosing === true) // Guard against infinite loop if Dispose(), which calls Close(), is called from within an OnClose handler
			return;

		var canceled = false;
		isClosing = true;

		Fit.Array.ForEach(onCloseHandlers, function(cb)
		{
			if (cb(me) === false)
			{
				canceled = true;
				return false; // Break loop
			}
		});

		isClosing = false;

		if (canceled === true)
			return;

		if (me !== null) // me is null if dialog was disposed from within an OnClose handler
		{
			Fit.Events.RemoveMutationObserver(mutationObserverId);
			mutationObserverId = -1;

			Fit.Events.RemoveHandler(window, resizeHandlerId);
			resizeHandlerId = -1;

			Fit.Dom.Remove(dialog);

			if (layer !== null)
				Fit.Dom.Remove(layer);
		}
	}

	this.Render = function(toElement) // Override Render() on Fit.Controls.Component
	{
		me.Open(toElement);
	}

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		me.Close(); // Closes and triggers OnClose event, but only if dialog is open

		if (me === null) // me is null if dialog was disposed from within an OnClose handler
		{
			return;
		}

		if (layer !== null)
		{
			Fit.Dom.Remove(layer);
		}

		if (cmdMaximize !== null)
		{
			cmdMaximize.Dispose();
		}

		if (cmdDismiss !== null)
		{
			cmdDismiss.Dispose();
		}

		if (draggable !== null)
		{
			draggable.Dispose();
		}

		if (buttons !== null)
		{
			Fit.Array.ForEach(Fit.Array.Copy(buttons.children), function(buttonElm) // Using Copy(..) since Dispose() modifies children collection
			{
				buttonElm._internal.Instance.Dispose();
			});
		}

		if (mutationObserverId !== -1)
		{
			Fit.Events.RemoveMutationObserver(mutationObserverId);
		}

		if (resizeHandlerId !== -1)
		{
			Fit.Events.RemoveHandler(window, resizeHandlerId);
		}

		me = dialog = focusTrapStart = focusTrapEnd = titleContainer = titleText = titleButtons = cmdMaximize = cmdDismiss = content = iframe = resizer = buttons = modal = layer = draggable = suppressPositioning = width = minWidth = maxWidth = height = minHeight = maxHeight = mutationObserverId = resizeHandlerId = onDismissHandlers = onCloseHandlers = isClosing = null;

		base();
	});

	// ============================================
	// Events
	// ============================================

	/// <function container="Fit.Controls.DialogTypeDefs" name="DialogEventHandler">
	/// 	<description> Dialog event handler </description>
	/// 	<param name="sender" type="Fit.Controls.Dialog"> Instance of Dialog </param>
	/// </function>

	/// <function container="Fit.Controls.Dialog" name="OnDismiss" access="public">
	/// 	<description>
	/// 		Add event handler fired when dialog is being dismissed by the user.
	/// 		Action can be suppressed by returning False.
	/// 		Function receives one argument: Sender (Fit.Controls.Dialog)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DialogTypeDefs.DialogEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnDismiss = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onDismissHandlers, cb);
	}

	/// <function container="Fit.Controls.Dialog" name="OnClose" access="public">
	/// 	<description>
	/// 		Add event handler fired when dialog is closed or dismissed.
	/// 		Use this event to react to dialog being closed, no matter
	/// 		the cause. Use OnDismiss event to detect when user closed it.
	/// 		Action can be suppressed by returning False.
	/// 		Function receives one argument: Sender (Fit.Controls.Dialog)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DialogTypeDefs.DialogEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnClose = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onCloseHandlers, cb);
	}

	// ============================================
	// Protected
	// ============================================

	this._internal = (this._internal ? this._internal : {});

	this._internal.GetLayerElement = function()
	{
		return layer;
	}

	// ============================================
	// Private
	// ============================================

	function createContentElement()
	{
		var div = document.createElement("div");
		Fit.Dom.AddClass(div, "FitUiControlDialogContent");
		return div;
	}

	function createResizerElement()
	{
		// NOTICE: Resizing a dialog with an iframe:
		// Resizing might not work properly with iframes with content from a foreign domain,
		// since browsers won't dispatch mouse/touch events over foreign domains.
		// Some older browsers won't even dispatch the events over iframes with content
		// originating from the same domain as the one hosting the main page.
		// This becomes a problem if the user is able to quickly move the mouse over the
		// iframe while resizing, and the computer/browser is not fast enough to "catch up" with
		// the movement of the resize handle. Slowly resizing the dialog in this case works.
		// Possible solution:
		// If we need better support for resizing dialogs with iframes, consider using a timer
		// which on a regular basis (2-10 times per second) updates the dialog size based on the
		// pointer position. That way the missing invocation of OnTouchMove or OnMouseMove is mitigated.
		// It should also dispatch the logic found in OnTouchEnd, OnTouchCancel and OnMouseUp if the
		// timer detects that the pointer (mouse or finger) has been released. This can be determined
		// using: Fit.Events.GetPointerState().Buttons.Touch || Fit.Events.GetPointerState().Buttons.Primary
		// However, make sure this issue is resolved first: https://github.com/Jemt/Fit.UI/issues/153
		// The issue relates to the reliablity of Fit.Events.GetPointerState().Buttons.Primary since it
		// is unset OnMouseOut, which will be triggered if mouse/finger leaves resize handle.

		var resizer = document.createElement("div");
		Fit.Dom.AddClass(resizer, "FitUiControlDialogResizer");

		Fit.Events.AddHandler(resizer, (Fit.Browser.IsTouchEnabled() === true ? "touchstart" : "mousedown"), function(e)
		{
			if (me.Maximized() === true)
			{
				return; // Prevent resizing while maximized
			}

			var ev = Fit.Events.GetEvent(e);

			var focusedBeforeResize = Fit.Dom.GetFocused();
			var initPos = Fit.Events.GetPointerState().Coordinates.ViewPort;
			var initDim = { Width: me.GetDomElement().offsetWidth, Height: me.GetDomElement().offsetHeight };

			var moveHandler = null;		// OnMouseMove or OnTouchMove event ID
			var releaseHandler = null;	// OnMouseUp or OnTouchEnd event ID
			var cancelHandler = null;	// OnTouchCancel event ID (only set on touch devices)

			var cleanup = function()
			{
				Fit.Events.RemoveHandler(document, moveHandler);
				Fit.Events.RemoveHandler(document, releaseHandler);

				if (cancelHandler !== null)
				{
					Fit.Events.RemoveHandler(document, cancelHandler);
				}

				focusedBeforeResize.focus();
			};

			moveHandler = Fit.Events.AddHandler(document, (Fit.Browser.IsTouchEnabled() === true ? "touchmove" : "mousemove"), { passive: false /* https://developer.mozilla.org/en-US/docs/Web/API/EventTarget/addEventListener */ }, function(e)
			{
				if (me === null) // Unlikely, but theoretically possible that dialog gets disposed while being resized
				{
					cleanup();
					return;
				}

				var ev = Fit.Events.GetEvent(e);

				// Determine mouse/finger travel

				var pp = Fit.Events.GetPointerState().Coordinates.ViewPort;
				var left = pp.X - initPos.X;	// Positive = moved right, negative = moved left
				var top = pp.Y - initPos.Y;		// Positive = moved down, negative = moved up

				if (suppressPositioning === false) // Dialog remains centered - adjust amount of resizing needed ("double up")
				{
					left = left * 2;	// Double up when dialog is centered since both left and right side is resized
					top = top * 1.3;	// Approximate value to adjust resizing vertically - not 100% accurate - resize handle will not follow pointer precisely, but close enough - reason: dialog is not centered 50/50 vertically but 25/75, so a static value such as 1.3 is not accurate - the actual adjustment needed for accurate positioning is variable
				}

				// Calculate and apply new width and height

				var newWidth = initDim.Width + left;
				var newHeight = initDim.Height + top;

				if (newWidth > 0)
				{
					dialog.style.width = newWidth + "px";
				}

				if (newHeight > 0)
				{
					dialog.style.height = newHeight + "px";
					setContentHeight();
				}

				if (newWidth > 0 || newHeight > 0)
				{
					updatePosition();
				}

				// Stop page scrolling on iOS (required in both ontouchstart and ontouchmove handlers).
				// Ontouchmove must be registered with { passive: false } for this to work in browsers
				// which defaults to passive:true. This also prevents text selection on older browsers
				// not supporting user-select:none.

				Fit.Events.PreventDefault(ev);
			});

			releaseHandler = Fit.Events.AddHandler(document, (Fit.Browser.IsTouchEnabled() === true ? "touchend" : "mouseup"), function(e)
			{
				cleanup();
			});

			if (Fit.Browser.IsTouchEnabled() === true)
			{
				cancelHandler = Fit.Events.AddHandler(document, "touchcancel", function(e) // Can be triggered on iOS by swiping down notification panel or control center while resizing
				{
					cleanup();
				});
			}

			if (ev.type === "touchstart")
			{
				Fit.Events.PreventDefault(ev); // Stop page scrolling on iOS (required in both ontouchstart and ontouchmove handlers)
			}
		});

		return resizer;
	}

	function setContentHeight()
	{
		if (me.IsOpen() === false)
			return;

		// By default the Dialog component adjusts its height to the content.
		// But if Height/MinimumHeight/MaximumHeight is set, or dialog is maximized,
		// then make sure to adjust the height of the content element if title/buttons
		// have been added. If no title/buttons are added, the content element simply
		// adjusts to the height of the dialog.

		content.style.height = "";

		if ((buttons !== null || titleContainer !== null) && (me.Maximized() === true || me.Height().Value !== -1 || me.MinimumHeight().Value !== -1 || me.MaximumHeight().Value !== -1))
		{
			var dh = dialog.offsetHeight;
			var th = (titleContainer !== null ? titleContainer.offsetHeight : 0);
			var bh = (buttons !== null ? buttons.offsetHeight : 0);

			content.style.height = (dh - th - bh) + "px";
		}
	}

	function updatePosition()
	{
		if (suppressPositioning === true)
			return;

		if (me.IsOpen() === false)
			return;

		var elm = me.GetDomElement();

		// Updating the position may actually cause content to change dimensions,
		// which in turn triggers the Mutation Observer registered to monitor the
		// dialog element, which in turn triggers updatePosition() again.
		// So this may result in dialog bouncing around a few times, especially when
		// using tables with dynamic cell widths within the content area.
		// The reason for this is because when the dialog is pushed to the right side
		// of the viewport (when centered), it may be sqeezed if sufficient space is not
		// available for the content, which may happen when the dialog is configured to scale
		// with the content (default behaviour when no dimensions are set). This results in
		// the Mutation Observer being triggered which in turn causes updatePosition()
		// to be called again.
		// Resetting the position reduces the risk of this happening since it allows
		// for the content of the dialog to consume the space it needs, without
		// causing resizing of the content once the dialog is positioned further down.
		elm.style.left = "0px";
		elm.style.top = "0px";

		var dim = Fit.Browser.GetViewPortDimensions();
		var offsetTop = (dim.Height - elm.offsetHeight) * 0.25;	// Place dialog vertically with 25% of available space above dialog and remaining 75% space below dialog
		var offsetLeft = (dim.Width - elm.offsetWidth) * 0.5;	// Place dialog exactly in the middle horizontally

		// Positioning affects element dimensions on high density pixel displays
		// in Chrome which can be resolved by rounding down top and left values:
		// https://bugs.chromium.org/p/chromium/issues/detail?id=1316873
		offsetTop = Math.floor(offsetTop);
		offsetLeft = Math.floor(offsetLeft);

		if (offsetTop < 0) // Value becomes negative if dialog is higher than viewport
		{
			offsetTop = 0;
		}

		if (offsetLeft < 0) // Value becomes negative if dialog is wider than viewport
		{
			offsetLeft = 0;
		}

		elm.style.left = offsetLeft + "px";
		elm.style.top = offsetTop + "px";
	}

	function updateTitleButtons()
	{
		// Remove title buttons container if no longer needed

		if (titleButtons !== null && cmdMaximize === null && cmdDismiss === null)
		{
			Fit.Dom.Remove(titleButtons);
			titleButtons = null;

			if (me.Title() === "")
			{
				me.Title(null); // Remove title element - will remain enabled if Draggable is enabled
			}

			return;
		}

		// Ensure title bar and container for title buttons

		if (titleButtons === null && (cmdMaximize !== null || cmdDismiss !== null))
		{
			if (me.Title() === null)
				me.Title("");

			titleButtons = document.createElement("div");
			Fit.Dom.AddClass(titleButtons, "FitUiControlDialogTitleButtons");
			Fit.Dom.Add(titleContainer, titleButtons);
		}

		// Add/re-add to ensure proper order

		if (cmdMaximize !== null)
		{
			Fit.Dom.Add(titleButtons, cmdMaximize.GetDomElement());
		}

		if (cmdDismiss !== null)
		{
			Fit.Dom.Add(titleButtons, cmdDismiss.GetDomElement());
		}
	}

	init();
}

Fit.Controls.Dialog._internal = {};

/// <container name="Fit.Controls.DialogInterface">
/// 	Simple interface for controlling Prompt, Confirm, and Alert dialogs
/// </container>
/// <function container="Fit.Controls.DialogInterface" name="Dismiss" access="public">
/// 	<description> Dismiss dialog without taking any action </description>
/// </function>
/// <function container="Fit.Controls.DialogInterface" name="Confirm" access="public">
/// 	<description> Confirm dialog - equivalent to clicking the OK button </description>
/// </function>
/// <function container="Fit.Controls.DialogInterface" name="Cancel" access="public">
/// 	<description> Cancel dialog - equivalent to clicking the Cancel button, or the OK button on an Alert dialog </description>
/// </function>

Fit.Controls.Dialog._internal.BaseDialog = function(content, showCancel, cb)
{
	Fit.Validation.ExpectString(content);
	Fit.Validation.ExpectBoolean(showCancel);
	Fit.Validation.ExpectFunction(cb, true);

	// Create dialog

	var d = new Fit.Controls.Dialog();
	d.Content(content.replace(/\n/g, "<br>"));
	d.Modal(true);

	Fit.Dom.AddClass(d.GetDomElement(), "FitUiControlDialogBase");
	Fit.Dom.AddClass(d._internal.GetLayerElement(), "FitUiControlDialogBaseModalLayer");

	// Declare buttons

	var cmdOk = new Fit.Controls.Button();
	var cmdCancel = null;

	// Localization

	var localize = function()
	{
		var locale = Fit.Internationalization.GetLocale(d);

		cmdOk.Title(locale.Ok);

		if (cmdCancel !== null)
			cmdCancel.Title(locale.Cancel);
	};

	Fit.Internationalization.OnLocaleChanged(localize);

	// Destructor

	var dispose = function()
	{
		Fit.Internationalization.RemoveOnLocaleChanged(localize);
		d.Dispose();
	}

	// Configure and add buttons

	cmdOk.Icon("check");
	cmdOk.Type(Fit.Controls.ButtonType.Success);
	cmdOk.OnClick(function(sender)
	{
		dispose();

		if (Fit.Validation.IsSet(cb) === true)
			cb(true);
	});
	d.AddButton(cmdOk);

	if (showCancel === true)
	{
		cmdCancel = new Fit.Controls.Button();
		cmdCancel.Icon("ban");
		cmdCancel.Type(Fit.Controls.ButtonType.Danger);
		cmdCancel.OnClick(function(sender)
		{
			dispose();

			if (Fit.Validation.IsSet(cb) === true)
				cb(false);
		});
		d.AddButton(cmdCancel);
	}

	localize();

	// Open dialog

	return { Dialog: d, ConfirmButton: cmdOk, CancelButton: cmdCancel, Dismiss: function() { dispose(); } }; // NOTICE: CancelButton might be null !
}

/// <function container="Fit.Controls.Dialog" name="Alert" access="public" static="true" returns="Fit.Controls.DialogInterface">
/// 	<description> Display alert dialog </description>
/// 	<param name="content" type="string"> Content to display in alert dialog </param>
/// 	<param name="cb" type="function" default="undefined"> Optional callback function invoked when OK button is clicked </param>
/// </function>
Fit.Controls.Dialog.Alert = function(content, cb)
{
	Fit.Validation.ExpectString(content);
	Fit.Validation.ExpectFunction(cb, true);

	var baseDialog = Fit.Controls.Dialog._internal.BaseDialog(content, false, function(res)
	{
		if (Fit.Validation.IsSet(cb) === true)
			cb();
	});
	baseDialog.Dialog.Open();
	baseDialog.ConfirmButton.Focused(true);

	var diaApi =
	{
		Dismiss: function() { baseDialog.Dismiss(); },
		Confirm: function() { baseDialog.ConfirmButton.Click(); },
		Cancel: function() { baseDialog.ConfirmButton.Click(); } // An alert dialog has no cancel button so we trigger the OK button instead which in turn triggers the callback which is consistent with the behaviour of Prompt and Confirm
	};

	return diaApi;
}

/// <function container="Fit.Controls.DialogTypeDefs" name="ConfirmCallback">
/// 	<description> Confirmation dialog callback </description>
/// 	<param name="confirmed" type="boolean"> True if OK button is clicked, otherwise False </param>
/// </function>

/// <function container="Fit.Controls.Dialog" name="Confirm" access="public" static="true" returns="Fit.Controls.DialogInterface">
/// 	<description> Display confirmation dialog with OK and Cancel buttons </description>
/// 	<param name="content" type="string"> Content to display in confirmation dialog </param>
/// 	<param name="cb" type="Fit.Controls.DialogTypeDefs.ConfirmCallback">
/// 		Callback function invoked when a button is clicked.
/// 		True is passed to callback function when OK is clicked, otherwise False.
/// 	</param>
/// </function>
Fit.Controls.Dialog.Confirm = function(content, cb)
{
	Fit.Validation.ExpectString(content);
	Fit.Validation.ExpectFunction(cb);

	var baseDialog = Fit.Controls.Dialog._internal.BaseDialog(content, true, cb);
	baseDialog.Dialog.Open();
	baseDialog.ConfirmButton.Focused(true);

	var diaApi =
	{
		Dismiss: function() { baseDialog.Dismiss(); },
		Confirm: function() { baseDialog.ConfirmButton.Click(); },
		Cancel: function() { baseDialog.CancelButton.Click(); }
	};

	return diaApi;
}

/// <function container="Fit.Controls.DialogTypeDefs" name="PromptCallback">
/// 	<description> Prompt dialog callback </description>
/// 	<param name="value" type="string | null"> String value entered if OK button is clicked, Null if prompt is canceled </param>
/// </function>

/// <function container="Fit.Controls.Dialog" name="Prompt" access="public" static="true" returns="Fit.Controls.DialogInterface">
/// 	<description> Display prompt dialog that allows for user input </description>
/// 	<param name="content" type="string"> Content to display in prompt dialog </param>
/// 	<param name="defaultValue" type="string"> Default value in input field </param>
/// 	<param name="cb" type="Fit.Controls.DialogTypeDefs.PromptCallback" default="undefined">
/// 		Callback function invoked when OK or Cancel button is clicked.
/// 		Value entered in input field is passed, null if prompt is canceled.
/// 	</param>
/// </function>
Fit.Controls.Dialog.Prompt = function(content, defaultValue, cb)
{
	Fit.Validation.ExpectString(content);
	Fit.Validation.ExpectString(defaultValue);
	Fit.Validation.ExpectFunction(cb);

	var txt = new Fit.Controls.Input("FitUiControlDialogPrompt" + Fit.Data.CreateGuid());
	txt.Width(100, "%");
	txt.Value(defaultValue);

	var baseDialog = Fit.Controls.Dialog._internal.BaseDialog(content + "<br><br>", true, function(res)
	{
		// Notice: Dialog is disposed at this point!

		var val = txt.Value();
		txt.Dispose();
		txt = null;

		if (res === true) // OK
		{
			cb(val);
		}
		else // Cancel
		{
			cb(null);
		}
	});

	Fit.Events.AddHandler(baseDialog.Dialog.GetDomElement(), "keydown", function(e)
	{
		if (txt === null)
		{
			return; // Dialog closed - user pressed Space or Enter on OK or Cancel button
		}

		if (e.keyCode === 13 && txt.Focused() === true) // ENTER
		{
			// Prevent insertion of a line break if focus is returned to an editable
			// element when dialog is closed. This does not happen when pressing ENTER
			// on a focused button because it also invokes PreventDefault(e) internally.
			Fit.Events.PreventDefault(e);

			baseDialog.ConfirmButton.Click();
		}
		else if (e.keyCode === 27) // ESC
		{
			baseDialog.CancelButton.Click();
		}
	});

	Fit.Dom.Add(baseDialog.Dialog.GetContentDomElement(), txt.GetDomElement());

	baseDialog.Dialog.Open();
	txt.Focused(true);

	var diaApi =
	{
		Dismiss: function() { baseDialog.Dismiss(); },
		Confirm: function() { baseDialog.ConfirmButton.Click(); },
		Cancel: function() { baseDialog.CancelButton.Click(); }
	};

	return diaApi;
}

Fit.Internationalization.AddLocalization(Fit.Controls.Dialog,
{
	// Primary locales must be registered before country specific overrides.
	// Example order: en, en_US, en_GB, de, de_AT, etc.
	// All locales inherit from en. All country specific overrides inherit
	// from their general locale (e.g. de_AT inherits from de).
	// English (en) MUST be defined, and be defined first!

	"en":
	{
		"Ok": "OK",
		"Cancel": "Cancel"
	},
	"da":
	{
		"Ok": "OK",
		"Cancel": "Annuller"
	},
	"de":
	{
		"Ok" : "OK",
		"Cancel" : "Abbrechen"
	},
	"no":
	{
		"Ok" : "OK",
		"Cancel" : "Avbryt"
	}
});
/// <container name="Fit.Controls.DialogEditor" extends="Fit.Controls.Dialog">
/// 	Dialog containing HTML editor for rich text editing
/// </container>

/// <function container="Fit.Controls.DialogEditor" name="DialogEditor" access="public">
/// 	<description> Create instance of DialogEditor control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.DialogEditor = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.Dialog).Apply(ctlId);

	var me = this;
	var ed = null;
	var updatedConfig = null;

	function init()
	{
		Fit.Dom.AddClass(me.GetDomElement(), "FitUiControlDialogEditor");

		ed = new Fit.Controls.Input();
		ed.Width(100, "%");
		ed.Height(100, "%");
		ed.Render(me.GetContentDomElement());

		// Performance optimization:
		// The editor internally fires OnChange on every change which is expensive.
		// By configuring OnChange debouncing, we can greatly increase performance,
		// especially for large documents, which is what DialogEditor is intended for.
		// The only downside to this is that ControlBase will not be able to update IsDirty
		// and IsValid state in the control's data-dirty and data-valid DOM attributes.
		// Control will always fire OnChange immediately when the editor lose focus though.
		ed.DebounceOnChange(5000); // Only process OnChange every 5 seconds - we don't rely on OnChange or the data-dirty and data-valid attributes

		// me.Modal(true);
		// me.Maximizable(true);
		// me.Dismissible(true);
		// me.Resizable(true);
		// me.Draggable(true);

		me.Width(850);
		me.MinimumWidth(20, "em");
		me.MaximumWidth(100, "%");

		me.Height(550);
		me.MinimumHeight(15, "em");
		me.MaximumHeight(100, "%");
	}

	// ============================================
	// Public - overrides
	// ============================================

	this.Open = Fit.Core.CreateOverride(this.Open, function(toElement)
	{
		Fit.Validation.ExpectDomElement(toElement, true);

		if (ed.DesignMode() === false || updatedConfig !== null)
		{
			ed.DesignMode(true, updatedConfig || undefined);
			updatedConfig = null
		}

		base(toElement);
		ed.Focused(true);
	});

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		ed.Dispose();
		base();

		me = ed = updatedConfig = null;
	});

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.DialogEditor" name="Placeholder" access="public" returns="string">
	/// 	<description> Get/set value used as a placeholder to indicate expected input on supported browsers </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is set as placeholder </param>
	/// </function>
	this.Placeholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);
		return ed.Placeholder(val);
	}

	/// <function container="Fit.Controls.DialogEditor" name="CheckSpelling" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control should have spell checking enabled (default) or disabled </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, true enables spell checking while false disables it </param>
	/// </function>
	this.CheckSpelling = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);
		return ed.CheckSpelling(val);
	}

	/// <function container="Fit.Controls.DialogEditor" name="Value" access="public" returns="string">
	/// 	<description> Get/set editor control value </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is inserted into control </param>
	/// </function>
	this.Value = function(val)
	{
		Fit.Validation.ExpectString(val, true);
		return ed.Value(val);
	}

	/// <function container="Fit.Controls.DialogEditor" name="IsDirty" access="public" returns="boolean">
	/// 	<description> Get value indicating whether user has changed control value </description>
	/// </function>
	this.IsDirty = function()
	{
		return ed.IsDirty();
	}

	/// <function container="Fit.Controls.DialogEditor" name="Enabled" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is enabled or disabled </description>
	/// 	<param name="val" type="boolean" default="undefined">
	/// 		If defined, True enables control (default), False disables control.
	/// 	</param>
	/// </function>
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);
		return ed.Enabled(val);
	}

	/// <function container="Fit.Controls.DialogEditor" name="Focused" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether editor control has focus.
	/// 		Dialog must be open and visible for focus assignment to work.
	/// 	</description>
	/// 	<param name="value" type="boolean" default="undefined"> If defined, True assigns focus, False removes focus (blur) </param>
	/// </function>
	this.Focused = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);
		return ed.Focused(val);
	}

	// ============================================
	// Protected
	// ============================================

	this._internal = this._internal || {};
	this._internal.SetDesignModeConfig = function(config) // Allow Input control (DesignMode) to update editor within dialog (detached editing) - config object validated in DesignMode(..)
	{
		if (me.IsOpen() === true)
		{
			ed.DesignMode(true, config);
		}
		else
		{
			updatedConfig = config;
		}
	}

	init();
}
/// <container name="Fit.Controls.DropDown" extends="Fit.Controls.ControlBase">
/// 	Drop Down Menu control allowing for single and multi selection.
/// 	Supports data selection using any control extending from Fit.Controls.PickerBase.
/// 	This control is extending from Fit.Controls.ControlBase.
/// </container>

/// <function container="Fit.Controls.DropDown" name="DropDown" access="public">
/// 	<description> Create instance of DropDown control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.DropDown = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;								// Access to members from event handlers (where "this" may have a different meaning)
	var itemContainer = null;					// Container for selected items and input fields
	var itemCollection = {};					// Indexed collection for selected items (for fast lookup)
	var itemCollectionOrdered = [];				// Ordered item collection (item order can be changed using drag and drop)
	var itemDropZones = {};						// Indexed collection of dropzones used to enable item dragging/dropping
	var htmlAllowed = false;					// Flag indicating whether HTML is allowed (shown) in selected items
	var placeholder = "";						// Placeholder value displayed when no selection is made
	var arrow = null;							// Arrow button used to open/close drop down menu
	var txtPrimary = null;						// Primary input (search) field initially available
	var txtActive = null;						// Currently active input (search) field
	var txtEnabled = false;						// Flag indicating whether user can enter text
	var dropDownMenu = null;					// Drop down menu element
	var picker = null;							// Picker control within drop down menu
	var orgSelections = [];						// Original selection set using Value(..) function - used to determine whether control is dirty
	var invalidMessage = "Invalid selection";	// Mouse over text for invalid selections
	var invalidMessageChanged = false;			// Flag indicating whether built-in Invalid Selection Message has been overridden or not
	var initialFocus = true;					// Flag indicating first focus of control
	var maxHeight = { Value: 150, Unit: "px"};	// Picker max height (px)
	var maxWidth = { Value: -1, Unit: "px" };	// Picker max width (px)
	var prevValue = "";							// Previous input value - used to determine whether OnChange should be fired
	var focusAssigned = false;					// Boolean ensuring that control is only given focus when AddSelection is called, if user assigned focus to control
	var closeHandlers = [];						// Events (IDs) responsible for closing drop down when user clicks outside of control
	var dropZone = null;						// Active DropZone (drag and drop support)
	var isMobile = false;						// Flag indicating whether control is running on a mobile (touch) device
	var focusInputOnMobile = true;				// Flag indicating whether control should focus input fields (and potentially bring up a virtual keyboard) based on configuration, platform (computer vs touch) and where user initially clicked/touched DropDown to activate it
	var detectBoundaries = false;				// Flag indicating whether drop down menu should detect viewport collision and open upwards when needed
	var detectBoundariesRelToViewPort = false;	// Flag indicating whether drop down menu should be positioned relative to viewport (true) or scroll parent (false)
	var persistView = false;					// Flag indicating whether picker controls should remember and restore its scroll position and highlighted item when reopened
	var highlightFirst = false;					// Flag indicating whether picker controls should focus its first node automatically when opened
	var searchModeOnFocus = false;				// Flag indicating whether control goes into search mode when it is focused (search mode clears input field and displays "search.." placeholder)
	var onScrollElement = null;					// Holds control's scroll parent when drop down is open
	var onScrollEventId = -1;					// Holds ID of OnScroll event handler registered when drop down is open
	var cancelEventRegistrationOnOpen = null;	// Holds function reference which cancels event registration if no longer needed (related to onScrollElement and onScrollEventId)

	var onInputChangedHandlers = [];			// Invoked when input value is changed - takes two arguments (sender (this), text value)
	var onPasteHandlers = [];					// Invoked when a value is pasted - takes two arguments (sender (this), text value)
	var onOpenHandlers = [];					// Invoked when drop down is opened - takes one argument (sender (this))
	var onCloseHandlers = [];					// Invoked when drop down is closed - takes one argument (sender (this))

	// Picker - suppress events
	var suppressUpdateItemSelectionState = false;
	var suppressOnItemSelectionChanged = false;

	// Text selection mode
	var clearTextSelectionOnInputChange = false;
	var prevTextSelection = null;
	var textSelectionCallback = null;
	var cmdToggleTextMode = null;

	function init()
	{
		Fit.Internationalization.OnLocaleChanged(localize);
		localize();

		// Initial settings

		me._internal.Data("multiselect", "false");
		me._internal.Data("selectionmode", "visual");
		me._internal.Data("selectionmodetoggle", "false");
		me._internal.Data("open", "false");

		isMobile = Fit.Device.OptimizeForTouch;
		// if (Fit.Browser.GetInfo().IsMobile === true)
		// 	isMobile = true;

		// Create item container

		itemContainer = document.createElement("div");
		itemContainer.tabIndex = -1; // Make it focusable but not part of tab flow
		itemContainer.onclick = function(e) // Not triggered when user clicks arrow button - it has its own logic and suppresses event propagation
		{
			var target = Fit.Events.GetTarget(e);

			if (target.tagName !== "INPUT") // Focus input unless already focused (if input was clicked)
			{
				focusInputOnMobile = me.InputEnabled() === true; // Focus input on mobile only if input is enabled - otherwise we get a virtual keyboard presented which does nothing - user can still click/touch an input between selected items in Visual Selection Mode to place an item between two existing selections
				focusAssigned = true; // Clicking the item container causes blur to fire for input fields in drop down which changes focusAssigned to false - it must be true for focusInput(..) to assign focus
				focusInput(txtPrimary);
			}

			me.OpenDropDown();
		}
		Fit.Dom.AddClass(itemContainer, "FitUiControlDropDownItems");

		// Create arrow button used to open drop down

		arrow = document.createElement("i");
		Fit.Dom.AddClass(arrow, "fa");
		Fit.Dom.AddClass(arrow, "fa-chevron-down");
		arrow.tabIndex = -1; // Make it focusable but not part of tab flow
		arrow.onclick = function(e)
		{
			focusInputOnMobile = false; // On mobile, do not focus input fields when interacting with control, when DropDown was initially activated by clicking/touching arrow icon

			// Close drop down if already open
			if (me.IsDropDownOpen() === true)
			{
				me.CloseDropDown();

				if (isMobile === true)
				{
					me.Focused(false); // Force control to lose focus when closed on mobile, to have the OnBlur event fire
				}
			}
			else // DropDown is closed - open it
			{
				me.OpenDropDown();
			}

			if (isMobile === false)
			{
				focusAssigned = true; // Clicking the arrow causes blur to fire for input fields in drop down which changes focusAssigned to false - it must be true for focusInput(..) to assign focus
				focusInput(txtPrimary);
			}

			Fit.Events.StopPropagation(e); // Prevent event from reaching itemContainer.onclick which opens DropDown and might assign a different value to focusInputOnMobile
		}
		arrow.onfocus = function(e)
		{
			focusAssigned = true;
		}
		arrow.onblur = function(e)
		{
			if (me === null)
			{
				// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
				// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
				// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
				return;
			}

			focusAssigned = false;
		}

		// Create primary search textbox

		txtPrimary = createSearchField();
		txtActive = txtPrimary;

		// Create drop down menu

		dropDownMenu = document.createElement("div");
		if (Fit.Browser.GetBrowser() !== "MSIE" || Fit.Browser.GetVersion() >= 9) // OnMouseWheel event is buggy in IE8
		{
			dropDownMenu.onmousewheel = function(e) // Handler prevents scroll on page when scrolling picker
			{
				var ev = Fit.Events.GetEvent(e);

				dropDownMenu.firstChild.scrollTop -= ((ev.wheelDeltaY !== undefined) ? ev.wheelDeltaY : ev.wheelDelta); // Expecting PickerControl's container (firstChild) to be scrollable for this to work
				Fit.Events.PreventDefault(ev);
			}
		}
		Fit.Dom.AddClass(dropDownMenu, "FitUiControlDropDownPicker");
		dropDownMenu.style.display = "none"; // Considered closed by default (prevent OnClose from firing if CloseDropDown() is called on closed drop down)

		// Make drop down close when user clicks outside of control

		postpone(function() // Postpone to prevent drop down from closing if created and auto opened during OnClick/OnTouchStart (https://github.com/Jemt/Fit.UI/issues/170)
		{
			if (isMobile === false)
			{
				var eventId = Fit.Events.AddHandler(document, "click", function(e)
				{
					var target = Fit.Events.GetTarget(e);

					if (target !== document.documentElement && target !== document.body && Fit.Dom.IsRooted(target) === false)
					{
						return; // Do not close DropDown if target no longer exists - this may happen if something is removed within DropDown (e.g. an item in the WSDropDown's action menu)
					}

					if (me.IsDropDownOpen() === true && target !== me.GetDomElement() && Fit.Dom.Contained(me.GetDomElement(), target) === false)
					{
						me.CloseDropDown();
					}
				});
				Fit.Array.Add(closeHandlers, eventId);
			}
			else
			{
				// OnClick does not work reliably on mobile (at least not on iOS 9 and 10), so using touch events instead

				var coords = null;
				var eventId = -1;

				eventId = Fit.Events.AddHandler(document, "touchstart", function(e)
				{
					var target = Fit.Events.GetTarget(e);

					coords = null;

					if (target !== document.documentElement && target !== document.body && Fit.Dom.IsRooted(target) === false)
					{
						return; // Do not close DropDown if target no longer exists - this may happen if something is removed within DropDown (e.g. an item in the WSDropDown's action menu)
					}

					if (me.IsDropDownOpen() === true && target !== me.GetDomElement() && Fit.Dom.Contained(me.GetDomElement(), target) === false)
					{
						coords = Fit.Events.GetPointerState().Coordinates.Document;
					}
				});
				Fit.Array.Add(closeHandlers, eventId);

				eventId = Fit.Events.AddHandler(document, "touchend", function(e)
				{
					if (coords === null)
						return;

					// Determine whether user moved finger (e.g. to scroll page) in which case we do not want to close the menu

					var curCoords = Fit.Events.GetPointerState().Coordinates.Document;
					var moved = (Math.abs(coords.X - curCoords.X) > 10 || Math.abs(coords.Y - curCoords.Y) > 10); // Must be moved at least 10px left/right or up/down

					if (moved === false)
					{
						me.CloseDropDown();
						me.Focused(false);
					}
				});
				Fit.Array.Add(closeHandlers, eventId);
			}
		});

		// Make drop down close if focus is lost on mobile
		// which happens when onscreen keyboard is closed.

		me.OnBlur(function()
		{
			focusInputOnMobile = true; // Reset to initial value when focus is lost

			if (isMobile === true)
			{
				me.CloseDropDown();
			}
		});

		// Make drop down close if scroll parent is scrolled

		me.OnOpen(function()
		{
			cancelEventRegistrationOnOpen = postpone(function() // Postpone to avoid auto closing drop down if browser scrolls control into view when it receives focus
			{
				cancelEventRegistrationOnOpen = null;

				onScrollElement = Fit.Dom.GetScrollParent(me.GetDomElement());

				if (onScrollElement === document.documentElement)
				{
					onScrollElement = window;
				}

				if (onScrollElement !== null)
				{
					onScrollEventId = Fit.Events.AddHandler(onScrollElement, "scroll", function(e)
					{
						me.CloseDropDown();
					});
				}
			});
		});

		me.OnClose(function()
		{
			// Cancel postponed operation in case dropdown is opened and closed multiple times to
			// avoid multiple OnScroll event registrations (https://github.com/Jemt/Fit.UI/issues/195).
			if (cancelEventRegistrationOnOpen !== null)
			{
				cancelEventRegistrationOnOpen();
				cancelEventRegistrationOnOpen = null;
			}

			if (onScrollElement !== null)
			{
				Fit.Events.RemoveHandler(onScrollElement, onScrollEventId);

				onScrollElement = null;
				onScrollEventId = -1;
			}
		});

		// Suppress context menu (except for input fields)

		Fit.Events.AddHandler(me.GetDomElement(), "contextmenu", function(e)
		{
			if (Fit.Events.GetTarget(e).tagName !== "INPUT")
				return Fit.Events.PreventDefault(e);
		});

		// Text selection mode

		me.OnFocus(function()
		{
			if (me.TextSelectionMode() === true)
			{
				if (Fit.Browser.GetBrowser() === "MSIE" || Fit.Browser.GetBrowser() === "Edge")
				{
					txtPrimary.readOnly = false; // ReadOnly set to true to make text-overflow:ellipsis work
					postpone(function() { txtPrimary.blur(); txtPrimary.focus(); }); // Change to ReadOnly is not applied immediately unless blurred and re-focused
				}

				clearTextSelectionOnInputChange = true;

				//me.OpenDropDown(); // DISABLED - will also open control when gaining focus from tab navigation
			}
		});

		me.OnChange(function()
		{
			if (me.TextSelectionMode() === true)
			{
				updateTextSelection();
			}
		});

		me.OnBlur(function()
		{
			if (me.TextSelectionMode() === true)
			{
				updateTextSelection(); // Update when blurred in case user have entered a value

				if (Fit.Browser.GetBrowser() === "MSIE" || Fit.Browser.GetBrowser() === "Edge")
				{
					txtPrimary.readOnly = true; // ReadOnly set to true to make text-overflow:ellipsis work
				}
			}
		});

		// Support for SearchModeOnFocus

		me.OnFocus(function(sender)
		{
			if (searchModeOnFocus === true)
			{
				searchModeOnFocus = false; // Temporarily disable searchModeOnFocus to allow setInputEditing(..) (which is called by ClearInputForSearch(..)) to change editing state for txtPrimary
				me._internal.ClearInputForSearch(true); // True = keep DropDown open - do not auto close it
				searchModeOnFocus = true;
			}
		});

		me.OnBlur(function()
		{
			if (searchModeOnFocus === true)
			{
				searchModeOnFocus = false; // Temporarily disable searchModeOnFocus to allow setInputEditing(..) to change editing state for txtPrimary
				me._internal.UndoClearInputForSearch();
				searchModeOnFocus = true;
			}
		});

		me.OnChange(function()
		{
			// Determine whether value was changed by user or programmatically,
			// so placeholder is not set when value is assigned programmatically.
			// Control might not be focused on mobile if opened using arrow icon.
			// In this case we simply use the DropDown's opened state instead.
			var controlIsActive = me.Focused() === true || me.IsDropDownOpen() === true;

			if (searchModeOnFocus === true && controlIsActive === true)
			{
				if (me.GetSelections().length === 0)
				{
					// DropDown has a synthetic placeholder which is displayed
					// when no items are selected. Remove placeholder from input
					// field so we do not get two placeholders on top of each other.
					txtPrimary.placeholder = "";
				}
				else
				{
					// Display placeholder in input field when items are selected
					txtPrimary.placeholder = me.Placeholder();
				}
			}
		});

		// PickerBase - make picker aware of focused state of host control

		me.OnFocus(function()
		{
			if (picker !== null)
			{
				picker._internal.ReportFocused(true);
			}
		});

		me.OnBlur(function()
		{
			if (picker !== null)
			{
				picker._internal.ReportFocused(false);
			}
		});

		// Append elements to the DOM

		Fit.Dom.Add(itemContainer, txtPrimary);
		Fit.Dom.Add(itemContainer, arrow);
		me._internal.AddDomElement(itemContainer);
		me._internal.AddDomElement(dropDownMenu);

		me.AddCssClass("FitUiControlDropDown");
	}

	// ============================================
	// Public
	// ============================================

	// Error message

	/// <function container="Fit.Controls.DropDown" name="InvalidSelectionMessage" access="public" returns="string">
	/// 	<description> Get/set mouse over text shown for invalid selections </description>
	/// 	<param name="msg" type="string" default="undefined"> If defined, error message for invalid selections are set </param>
	/// </function>
	this.InvalidSelectionMessage = function(msg)
	{
		Fit.Validation.ExpectString(msg, true);

		if (Fit.Validation.IsSet(msg) === true)
		{
			invalidMessage = msg;
			invalidMessageChanged = true; // Make sure message is not changed if locale is changed on page

			updateInvalidMessageForSelectedItems();
		}

		return invalidMessage;
	}

	// Dimensions

	/// <function container="Fit.Controls.DropDown" name="DropDownMaxHeight" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set max height of drop down - returns object with Value (number) and Unit (string) properties </description>
	/// 	<param name="value" type="number" default="undefined"> If defined, max height is updated to specified value. A value of -1 forces picker to fit height to content. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="undefined"> If defined, max height is updated to specified CSS unit, otherwise px is assumed </param>
	/// </function>
	this.DropDownMaxHeight = function(value, unit)
	{
		Fit.Validation.ExpectNumber(value, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(value) === true)
		{
			maxHeight = { Value: value, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px")};

			if (picker !== null)
				picker.MaxHeight(maxHeight.Value, maxHeight.Unit);
		}

		return maxHeight;
	}

	/// <function container="Fit.Controls.DropDown" name="DropDownMaxWidth" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set max width of drop down - returns object with Value (number) and Unit (string) properties </description>
	/// 	<param name="value" type="number" default="undefined"> If defined, max width is updated to specified value. A value of -1 forces drop down to use control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="undefined"> If defined, max width is updated to specified CSS unit, otherwise px is assumed </param>
	/// </function>
	this.DropDownMaxWidth = function(value, unit)
	{
		Fit.Validation.ExpectNumber(value, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(value) === true)
		{
			maxWidth = { Value: value, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px")};

			if (value !== -1)
			{
				dropDownMenu.style.width = "auto"; // Adjust width to content - notice that optimizeDropDownPosition(..) and resetDropDownPosition() also manipulate the width property!
				dropDownMenu.style.maxWidth = maxWidth.Value + maxWidth.Unit;
			}
			else
			{
				dropDownMenu.style.width = (me.DetectBoundaries() === true ? dropDownMenu.style.width : ""); // Preserve width value if DetectBoundaries is enabled since it also modifies this property
				dropDownMenu.style.maxWidth = "";
			}
		}

		return maxWidth;
	}

	// ControlBase interface

	// See documentation on ControlBase
	this.Value = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		// Set

		if (Fit.Validation.IsSet(val) === true)
		{
			if (preserveDirtyState !== true)
			{
				orgSelections = [];
			}

			var fireChange = (itemCollectionOrdered.length > 0 || val !== ""); // Fire OnChange if current selections are cleared, and/or if new selections are set

			me._internal.ExecuteWithNoOnChange(function()
			{
				me.Clear();

				if (val !== "")
				{
					Fit.Array.ForEach(val.split(";"), function(item)
					{
						var info = item.split("=");

						if (info.length === 2) // Format: title1=val1;title2=val2;title3=val3
							me.AddSelection(decodeReserved(info[0]), decodeReserved(info[1]));
						else // Format: val1;val2;val3
							me.AddSelection(decodeReserved(info[0]), decodeReserved(info[0]));
					});
				}
			});

			if (preserveDirtyState !== true)
			{
				orgSelections = me.GetSelections();
			}

			if (fireChange === true)
				fireOnChange();
		}

		// Get

		var selections = me.GetSelections(); // Invalid selections excluded
		var value = "";

		Fit.Array.ForEach(selections, function(item)
		{
			value += ((value !== "") ? ";" : "") + encodeReserved(item.Title) + "=" + encodeReserved(item.Value);
		});

		return value;
	}

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		return (Fit.Core.IsEqual(orgSelections, me.GetSelections()) === false); // Invalid selections excluded
	}

	// See documentation on ControlBase
	this.Clear = function()
	{
		me.ClearSelections();
	}

	// See documentation on ControlBase
	this.Focused = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true)
			{
				var c = txtPrimary;
				var v = c.value;

				// Set focus
				c.focus();

				// Make cursor move to end
				c.value = "";
				c.value = v;
			}
			else if (val === false && Fit.Dom.Contained(me.GetDomElement(), Fit.Dom.GetFocused()) === true)
			{
				me.CloseDropDown();
				Fit.Dom.GetFocused().blur(); // Focused element could be txtActive (holds reference to currently focused input field) or arrow button
			}
		}

		return (txtActive === Fit.Dom.GetFocused() || arrow === Fit.Dom.GetFocused());
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		// This will destroy control - it will no longer work!

		if (picker !== null)
			picker.Destroy();

		if (Fit._internal.DropDown.Current === me)
		{
			Fit._internal.DropDown.Current = null;
		}

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		Fit.Array.ForEach(closeHandlers, function(eventId)
		{
			Fit.Events.RemoveHandler(document, eventId);
		});

		Fit.Array.ForEach(itemDropZones, function(key)
		{
			itemDropZones[key].Dispose();
		});

		if (onScrollElement !== null)
		{
			Fit.Events.RemoveHandler(onScrollElement, onScrollEventId);
		}

		me = itemContainer = itemCollection = itemCollectionOrdered = itemDropZones = htmlAllowed = placeholder = arrow = txtPrimary = txtActive = txtEnabled = dropDownMenu = picker = orgSelections = invalidMessage = invalidMessageChanged = initialFocus = maxHeight = maxWidth = prevValue = focusAssigned = closeHandlers = dropZone = isMobile = focusInputOnMobile = detectBoundaries = detectBoundariesRelToViewPort = persistView = highlightFirst = searchModeOnFocus = onScrollElement = onScrollEventId = cancelEventRegistrationOnOpen = onInputChangedHandlers = onPasteHandlers = onOpenHandlers = onCloseHandlers = suppressUpdateItemSelectionState = suppressOnItemSelectionChanged = clearTextSelectionOnInputChange = prevTextSelection = textSelectionCallback = cmdToggleTextMode = null;

		base();
	});

	// Misc. options

	/// <function container="Fit.Controls.DropDown" name="HtmlAllowed" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether HTML is allowed (shown) in selected items </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables support for HTML, False disables it </param>
	/// </function>
	this.HtmlAllowed = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && htmlAllowed !== val)
		{
			htmlAllowed = val;

			// Update view - RenameSelection(..) will enable/disable HTML depending on htmlAllowed (set above)

			Fit.Array.ForEach(itemCollectionOrdered, function(item)
			{
				me.RenameSelection(item.Value, item.Title);
			});
		}

		return htmlAllowed;
	}

	/// <function container="Fit.Controls.DropDown" name="Placeholder" access="public" returns="string">
	/// 	<description> Get/set value used as a placeholder on supported browsers, to indicate expected value or action </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is set as placeholder </param>
	/// </function>
	this.Placeholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			placeholder = val;
			updatePlaceholder(true);
		}

		return placeholder;
	}

	/// <function container="Fit.Controls.DropDownTypeDefs" name="SelectionToStringCallback" returns="string">
	/// 	<description> Callback responsible for constructing string value representing selected items </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.DropDown" name="TextSelectionMode" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set flag indicating whether to use Text Selection Mode (true) or Visual Selection Mode (false).
	/// 		Visual Selection Mode is the default way selected items are displayed, but it may result in control
	/// 		changing dimensions as items are added/removed. Text Selection Mode prevents this and gives the
	/// 		user a traditional DropDown control instead.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables Text Selection Mode, False disables it (Visual Selection Mode) </param>
	/// 	<param name="cb" type="Fit.Controls.DropDownTypeDefs.SelectionToStringCallback" default="undefined">
	/// 		If defined, function will be called with DropDown being passed as an argument when selection text
	/// 		needs to be updated. Function is expected to return a string representation of the selected items.
	/// 	</param>
	/// </function>
	this.TextSelectionMode = function(val, cb)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectFunction(cb, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && me.TextSelectionMode() === false)
			{
				textSelectionCallback = (cb ? cb : null);

				me.ClearInput(); // Clear any value entered by the user and fire OnInputChanged

				updateTextSelection();

				if (me.Focused() === false && (Fit.Browser.GetBrowser() === "MSIE" || Fit.Browser.GetBrowser() === "Edge"))
				{
					txtPrimary.readOnly = true; // Necessary to make text-overflow:ellipsis work in IE and Edge - disabled in OnFocus handler
				}
			}
			else if (val === false && me.TextSelectionMode() === true)
			{
				if (txtPrimary.value === prevTextSelection)
				{
					txtPrimary.value = "";
				}
				else
				{
					// User has changed value - clear and fire OnInputChanged.
					// This only happens if Text Selection Mode is being disabled
					// while control has focus and user have entered a value,
					// since selection text is always updated when control lose focus.
					me.ClearInput();
				}

				clearTextSelectionOnInputChange = false;
				prevTextSelection = null;
				textSelectionCallback = null;

				if (Fit.Browser.GetBrowser() === "MSIE" || Fit.Browser.GetBrowser() === "Edge")
				{
					txtPrimary.readOnly = false;
				}
			}

			me._internal.Data("selectionmode", (val === true ? "text" : "visual"));

			if (cmdToggleTextMode !== null)
			{
				if (val === true)
				{
					Fit.Dom.RemoveClass(cmdToggleTextMode, "fa-compress");
					Fit.Dom.AddClass(cmdToggleTextMode, "fa-expand");
				}
				else
				{
					{
						Fit.Dom.RemoveClass(cmdToggleTextMode, "fa-expand");
						Fit.Dom.AddClass(cmdToggleTextMode, "fa-compress");
					}
				}
			}

			me._internal.Repaint();
		}

		return (me._internal.Data("selectionmode") === "text");
	}

	/// <function container="Fit.Controls.DropDown" name="SelectionModeToggle" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control allow user to toggle Selection Mode (Visual or Text) </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables toggle button, False disables it </param>
	/// </function>
	this.SelectionModeToggle = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && cmdToggleTextMode === null)
			{
				cmdToggleTextMode = document.createElement("span");
				cmdToggleTextMode.onclick = function()
				{
					me.TextSelectionMode(!me.TextSelectionMode());
				}
				Fit.Dom.AddClass(cmdToggleTextMode, "fa");
				Fit.Dom.AddClass(cmdToggleTextMode, me.TextSelectionMode() === true ? "fa-expand" : "fa-compress");
				me._internal.AddDomElement(cmdToggleTextMode);
				me._internal.Data("selectionmodetoggle", "true");
			}
			else if (val === false && cmdToggleTextMode !== null)
			{
				me._internal.RemoveDomElement(cmdToggleTextMode);
				cmdToggleTextMode = null;
				me._internal.Data("selectionmodetoggle", "false");
			}
		}

		return (cmdToggleTextMode !== null);
	}

	/// <function container="Fit.Controls.DropDown" name="MultiSelectionMode" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether control allows for multiple selections </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables multi selection mode, False disables it </param>
	/// </function>
	this.MultiSelectionMode = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && me.MultiSelectionMode() !== val)
		{
			me.ClearSelections();
			me._internal.Data("multiselect", val.toString());
		}

		return (me._internal.Data("multiselect") === "true");
	}

	// Controlling selections

	/// <function container="Fit.Controls.DropDown" name="GetPicker" access="public" returns="Fit.Controls.PickerBase">
	/// 	<description> Get picker control used to add items to drop down control </description>
	/// </function>
	this.GetPicker = function()
	{
		return picker;
	}

	/// <function container="Fit.Controls.DropDown" name="SetPicker" access="public">
	/// 	<description> Set picker control used to add items to drop down control </description>
	/// 	<param name="pickerControl" type="Fit.Controls.PickerBase | null"> Picker control extending from PickerBase </param>
	/// </function>
	this.SetPicker = function(pickerControl)
	{
		Fit.Validation.ExpectInstance(pickerControl, Fit.Controls.PickerBase, true);

		if (pickerControl === picker)
			return; // Already active picker

		// Remove existing picker

		if (picker !== null)
			Fit.Dom.Remove(picker.GetDomElement());

		// Add picker

		if (!pickerControl)
		{
			picker = null;
			return;
		}

		pickerControl._internal.InitializePicker();
		pickerControl._internal.ReportFocused(me.Focused());

		picker = pickerControl;
		Fit.Dom.Add(dropDownMenu, picker.GetDomElement());

		// Allow picker to select items in case selections have already been set in drop down

		suppressOnItemSelectionChanged = true;
		picker.SetSelections(me.GetSelections()); // #167: This can be very slow for huge selections (thousands of selected nodes)
		suppressOnItemSelectionChanged = false;

		// Set picker MaxHeight

		picker.MaxHeight(maxHeight.Value, maxHeight.Unit);
		optimizeDropDownPosition(); // In case dropdown is already open and SetPicker was called async, e.g. initiated from OnOpen event. Function may change MaxHeight on Picker.

		// Persist view and initial focus

		picker.PersistView(persistView);
		picker.HighlightFirst(highlightFirst);

		// Make sure OnItemSelectionChanged is only registered once

		if (!picker._internal)
			picker._internal = {};

		if (picker._internal.HostControl)
			return;

		picker._internal.HostControl = me;

		// Register OnItemSelectionChanged handler which is used to
		// synchronize selections from picker control to drop down.

		var fireChangeEvent = false;

		picker.OnItemSelectionChanged(function(sender, eventArgs)
		{
			if (suppressOnItemSelectionChanged === true)
				return; // Skip - already processing OnItemSelectionChanged - may be invoked multiple times if e.g. switching selection in Single Selection Mode (existing item removed + new item selected)

			var txt = null;

			// Prevent this.AddSelection and this.RemoveSelection from calling
			// picker.UpdateItemSelection which in turn fires OnItemSelectionChanged, causing an infinite loop.
			suppressUpdateItemSelectionState = true;

			if (eventArgs.Selected === true && me.GetSelectionByValue(eventArgs.Value) === null) // Check whether node is already selected (PreSelection)
			{
				fireChangeEvent = true;

				// Changing a selection in the picker control may cause OnItemSelectionChanged to be fired multiple
				// times since an existing selection may first be deselected, followed by new item being selected.
				// In this case we suppress OnChange fired by RemoveSelection(..) and AddSelection(..), and instead
				// fire it when picker's OnItemSelectionComplete event is fired.
				me._internal.ExecuteWithNoOnChange(function() { me.AddSelection(eventArgs.Title, eventArgs.Value); });

				if (me.MultiSelectionMode() === false)
					me.CloseDropDown();
			}
			else if (eventArgs.Selected === false && me.GetSelectionByValue(eventArgs.Value) !== null)
			{
				fireChangeEvent = true;

				txt = txtActive; // RemoveSelection changes txtActive

				// Changing a selection in the picker control may cause OnItemSelectionChanged to be fired multiple
				// times since an existing selection may first be deselected, followed by new item being selected.
				// In this case we suppress OnChange fired by RemoveSelection(..) and AddSelection(..), and instead
				// fire it when picker's OnItemSelectionComplete event is fired.
				me._internal.ExecuteWithNoOnChange(function() { me.RemoveSelection(eventArgs.Value) });

				// Fix - if item removed was the last item, and txtActive
				// was one of the input fields belonging to that selection,
				// it will now have been removed.
				if (txt.parentElement.parentElement === null)
					txt = txtPrimary;
			}
			else
			{
				// User selected an item which is already selected

				// Only update input field if picker currently active is the one providing information about selection change.
				// Pickers might load and select nodes async. which allows for picker to be changed. More specifically we could
				// imagine a situation where Select All is triggered in a WSTreeView picker where nodes must first be fetched from
				// the server. While waiting for data, the user enters a search string which changes the picker control to WSListView.
				// Soon hereafter TreeView data is received and all nodes are selected, triggering OnItemSelectionChanged (here), in
				// which case we do not want the search value to be cleared away.
				if (picker === sender)
				{
					if (me.TextSelectionMode() === true)
					{
						updateTextSelection(); // Make sure any search value is removed and text selection is restored
					}
					else
					{
						me.ClearInput(); // Make sure any search value is removed
					}
				}
			}

			// DISABLED: Now handled using picker.OnFocus(..) handler further down - see https://github.com/Jemt/Fit.UI/issues/86 for details
			// if (eventArgs.ProgrammaticallyChanged === false && (isMobile === false || focusInputOnMobile === true))
			// {
			// 	focusAssigned = true; // Clicking the picker causes blur to fire for input fields in drop down which changes focusAssigned to false - it must be true for focusInput(..) to assign focus
			// 	focusInput(((txt !== null) ? txt : txtActive));
			// }

			suppressUpdateItemSelectionState = false;
		});

		picker.OnItemSelectionComplete(function(sender)
		{
			if (suppressOnItemSelectionChanged === true)
				return;

			// Picker may notify about selections that has already been
			// made due to PreSelections (nodes are selected when loaded).
			// Only fire OnChange if control value has actually changed.
			if (fireChangeEvent === true)
			{
				me._internal.FireOnChange();
				fireChangeEvent = false;
			}
		});

		picker.OnFocusIn(function(sender)
		{
			// Steal back focus - DropDown should remain the focused control
			focusAssigned = true; // Clicking the picker causes blur to fire for input fields in drop down which changes focusAssigned to false - it must be true for focusInput(..) to assign focus
			focusInput(txtActive);
		});
	}

	/// <function container="Fit.Controls.DropDown" name="AddSelection" access="public">
	/// 	<description> Add selection to control </description>
	/// 	<param name="title" type="string"> Item title </param>
	/// 	<param name="value" type="string"> Item value </param>
	/// 	<param name="valid" type="boolean" default="true">
	/// 		Flag indicating whether selection is valid or not. Invalid selections are highlighted and
	/// 		not included when selections are retrived using Value() function, and not considered when
	/// 		IsDirty() is called to determine whether control value has been changed by user.
	/// 		GetSelections(true) can be used to retrive all items, including invalid selections.
	/// 	</param>
	/// </function>
	this.AddSelection = function(title, value, valid)
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(valid, true);

		if (me.GetSelectionByValue(value) !== null)
			return;

		// Update picker control

		// Notice: suppressUpdateItemSelectionState is True if item was added by picker,
		// in which case picker.UpdateItemSelection(..) should not be called.
		// What happens is that picker fires drop down's OnItemSelectionChanged when an item is
		// selected, which sets suppressUpdateItemSelectionState to True, and calls AddSelection.
		// Failing to prevent picker.UpdateItemSelection(..) from being called in this case could
		// result in an infinite loop.
		if (picker !== null && suppressUpdateItemSelectionState === false)
		{
			// Notice: Picker fires OnItemSelectionChanged when picker.UpdateItemSelection(..) is invoked
			// below (other controls than DropDown may have registered an OnItemSelectionChanged
			// handler too). In this case we set suppressOnItemSelectionChanged to True, causing
			// drop down to do nothing in OnItemSelectionChanged handler when fired. Drop down's OnItemSelectionChanged
			// handler is responsible for handling items added/removed by picker, but in this case the change did not
			// come from the picker (since suppressUpdateItemSelectionState is False).
			// Failing to set this flag could result in an infinite loop.
			suppressOnItemSelectionChanged = true;

			var res = true;
			var error = null;

			try // Make sure we can set suppressOnItemSelectionChanged false again, so drop down remains in a functioning state
			{
				res = picker.UpdateItemSelection(value, true, me.Focused() === false);
			}
			catch (err) { error = err; }

			suppressOnItemSelectionChanged = false;

			if (error !== null)
				Fit.Validation.ThrowError(error);

			if (res === false)
				return; // Picker prevented selection from being added
		}

		// Clear selection if in Single Selection Mode

		if (me.MultiSelectionMode() === false)
		{
			var error = null;

			// ClearSelections() results in picker.UpdateItemSelection(..) being called if an item is currently selected,
			// eventually resulting in picker firing OnItemSelectionChanged and OnItemSelectionComplete. However, operation has
			// not completed yet, since new element has not yet been selected (done below). Skip events.
			suppressOnItemSelectionChanged = true;

			try // Make sure we can set suppressOnItemSelectionChanged false again, so drop down remains in a functioning state
			{
				me._internal.ExecuteWithNoOnChange(function() { me.ClearSelections(); });
			}
			catch (err) { error = err; }

			suppressOnItemSelectionChanged = false;

			if (error !== null)
				Fit.Validation.ThrowError(error);

			if (itemCollectionOrdered.length > 0) // A picker prevented selected item from being removed
				return;
		}

		// Create new item

		// Delete button
		var cmdDelete = document.createElement("i");
		Fit.Dom.AddClass(cmdDelete, "fa");
		Fit.Dom.AddClass(cmdDelete, "fa-times");
		cmdDelete.tabIndex = -1; // Prevents control from losing focus when clicking button - will not interfear with tab flow as -1 makes it focusable, but not part of tab flow
		cmdDelete.onclick = function(e) // OnClick fires after MouseUp
		{
			// Whether OnClick fires depends on what browser is being used. A couple of tests reveal this:
			// Chrome and IE: Does not fire onclick if an element is moved to another position in DOM (dragged and dropped).
			// Firefox: Does not fire onclick if element is moved, unless it is dropped in a new position in DOM (drag and drop)
			//  - opposite of Chrome and IE. Firefox does ignore minor movements though, for the sake of usability.

			me.RemoveSelection(decode(Fit.Dom.Data(cmdDelete.parentElement, "value")));

			// Always focus control when removing and moving items.
			// First of all it prevents OnFocus and OnBlur from firing
			// a lot of times if multiple elements are removed or moved,
			// secondly it's easier to ensure consistency across browsers
			// as various events are implemented differently, especially
			// when drag and drop comes into place.
			// RemoveSelection changes focus if control already has focus.
			// If three items are selected, and the first one is removed,
			// then focus is set to the first (left most) input field.
			// We always want txtPrimary to have focus after removing an
			// item using the mouse which is why focus is assigned below.
			// This is also the reason why the code in cmdDelete.OnMouseUp
			// alone is not sufficient. Focus must be assigned to txtPrimary
			// AFTER RemoveSelection(..) is invoked.
			focusAssigned = true;
			focusInput(txtPrimary);

			// Do not open drop down when an item is removed
			Fit.Events.StopPropagation(e);
		}
		cmdDelete.onmouseup = function() // MouseUp fires before OnClick
		{
			// OnClick is not fired in Firefox when mouse is moved (but not dropped in a new position),
			// which causes the item to remain intact and focused. Make sure proper element is focused in this case.
			focusAssigned = true;
			focusInput(txtPrimary);
		}

		// Item container (left input, title box, right input)
		var container = document.createElement("span");
		container.tabIndex = -1; // Make it focusable but not part of tab flow
		/*container.onclick = function(e)
		{
			focusInputOnMobile = true;
			focusAssigned = true;
			focusInput(txtPrimary);
		}*/

		// Input fields (left and right)
		var searchLeft = createSearchField();
		var searchRight = createSearchField();

		// Title box
		var item = document.createElement("span");
		Fit.Dom.Data(item, "value", encode(value)); // Value may contain special characters - encode to prevent value from breaking item

		// Set CSS class and mouse over for invalid selection
		if (valid === false)
		{
			Fit.Dom.Attribute(item, "title", invalidMessage);
			Fit.Dom.AddClass(item, "FitUiControlDropDownInvalid");
		}

		// Add title and delete button to title box
		var titleNode = me.HtmlAllowed() === false ? document.createTextNode(Fit.String.StripHtml(title)) : Fit.Dom.CreateElement(title);
		Fit.Dom.Add(item, titleNode);
		Fit.Dom.Add(item, cmdDelete);

		// Add elements to item container
		Fit.Dom.Add(container, searchLeft);
		Fit.Dom.Add(container, item);
		Fit.Dom.Add(container, searchRight);

		// Add new item

		var before = null;

		if (txtActive !== txtPrimary && me.MultiSelectionMode() === true && Fit.Dom.GetIndex(txtActive) === 0) // Left input
			before = txtActive.parentElement;
		else if (txtActive !== txtPrimary && me.MultiSelectionMode() === true && Fit.Dom.GetIndex(txtActive) === 2) // Right input
			before = txtActive.parentElement.nextSibling;
		else
			before = txtPrimary;

		itemContainer.insertBefore(container, before);

		var itemObject = createItemObject(title, value, valid !== false, item, titleNode); //convertItemElementToItemObject(item);
		itemCollection[itemObject.Value] = itemObject;
		itemCollectionOrdered.push(itemObject);

		// Clear input control value

		if (me.TextSelectionMode() === false)
		{
			me.ClearInput();
		}

		// Drag and Drop

		var drg = new Fit.DragDrop.Draggable(container, item);
		drg.OnDragStop(onDragStop);
		drg.OnDragging(onDragging);

		var drp = new Fit.DragDrop.DropZone(container);
		drp.OnDrop(onDrop);
		drp.OnEnter(onDropZoneEnter);
		drp.OnLeave(onDropZoneLeave);
		itemDropZones[value] = drp; // Keep reference so we can dispose it when item is removed

		// Focus input control

		if (me.MultiSelectionMode() === true)
		{
			// Make input to the right of newly added item active. This to make sure that
			// the next item added appears to the right of the previously added item.
			// If a left input field is selected, it remains focused, as the new element
			// is added in front of the left input field.
			if (txtActive !== txtPrimary && Fit.Dom.GetIndex(txtActive) === 2)
			{
				focusInput(searchRight);
			}
		}
		else
		{
			focusInput(txtPrimary);
		}

		// Fire OnChange event

		fireOnChange();
	}

	/// <container name="Fit.Controls.DropDownTypeDefs.DropDownItem">
	/// 	<description> DropDown item </description>
	/// 	<member name="Title" type="string"> Item title </member>
	/// 	<member name="Value" type="string"> Unique item value </member>
	/// 	<member name="Valid" type="boolean"> Value indicating whether item is a valid selection </member>
	/// </container>

	/// <function container="Fit.Controls.DropDown" name="GetSelections" access="public" returns="Fit.Controls.DropDownTypeDefs.DropDownItem[]">
	/// 	<description> Get selected items - returned array contain objects with Title (string), Value (string), and Valid (boolean) properties </description>
	/// 	<param name="includeInvalid" type="boolean" default="false"> Flag indicating whether invalid selection should be included or not </param>
	/// </function>
	this.GetSelections = function(includeInvalid)
	{
		Fit.Validation.ExpectBoolean(includeInvalid, true);

		var toReturn = [];

		Fit.Array.ForEach(itemCollectionOrdered, function(selection)
		{
			if (includeInvalid === true || selection.Valid === true)
			{
				Fit.Array.Add(toReturn, { Title: selection.Title, Value: selection.Value, Valid: selection.Valid });
			}
		});

		return toReturn;
	}

	/// <function container="Fit.Controls.DropDown" name="GetSelectionByValue" access="public" returns="Fit.Controls.DropDownTypeDefs.DropDownItem | null">
	/// 	<description> Get selected item by value - returns object with Title (string), Value (string), and Valid (boolean) properties if found, otherwise Null is returned </description>
	/// 	<param name="val" type="string"> Value of selected item to retrive </param>
	/// </function>
	this.GetSelectionByValue = function(val)
	{
		Fit.Validation.ExpectString(val);

		var selection = itemCollection[val] || null;

		if (selection !== null)
		{
			return { Title: selection.Title, Value: selection.Value, Valid: selection.Valid };
		}

		return null;
	}

	/// <function container="Fit.Controls.DropDown" name="GetHighlighted" access="public" returns="Fit.Controls.DropDownTypeDefs.DropDownItem | null">
	/// 	<description>
	/// 		Get item currently highlighted in picker control.
	/// 		Returns an object with Title (string), Value (string),
	/// 		and Valid (boolean) properties if found, otherwise Null.
	/// 	</description>
	/// </function>
	this.GetHighlighted = function()
	{
		if (picker !== null)
		{
			var hl = picker.GetHighlighted();
			return (hl !== null ? { Title: hl.Title, Value: hl.Value, Valid: true } : null);
		}

		return null;
	}

	/// <function container="Fit.Controls.DropDown" name="PersistView" access="public" returns="boolean">
	/// 	<description> Make DropDown restore scroll position and previously highlighted item when reopened </description>
	/// 	<param name="val" type="boolean" default="undefined"> If set, True enables feature, False disables it (default) </param>
	/// </function>
	this.PersistView = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== persistView)
		{
			persistView = val;

			if (me.GetPicker() !== null)
			{
				me.GetPicker().PersistView(val);
			}
		}

		return persistView;
	}

	/// <function container="Fit.Controls.DropDown" name="HighlightFirst" access="public" returns="boolean">
	/// 	<description> Make DropDown highlight first selectable item when opened </description>
	/// 	<param name="val" type="boolean" default="undefined"> If set, True enables feature, False disables it (default) </param>
	/// </function>
	this.HighlightFirst = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			highlightFirst = val;

			if (me.GetPicker() !== null)
			{
				me.GetPicker().HighlightFirst(val);
			}
		}

		return highlightFirst;
	}

	/// <function container="Fit.Controls.DropDown" name="RenameSelection" access="public">
	/// 	<description> Rename title of selected item by its value </description>
	/// 	<param name="val" type="string"> Value of selected item to rename </param>
	/// 	<param name="newTitle" type="string"> New item title </param>
	/// </function>
	this.RenameSelection = function(val, newTitle)
	{
		Fit.Validation.ExpectString(val);
		Fit.Validation.ExpectString(newTitle);

		var selection = itemCollection[val] || null;

		if (selection !== null)
		{
			var orgTitle = selection.Title;
			var newTitleNode = me.HtmlAllowed() === false ? document.createTextNode(Fit.String.StripHtml(newTitle)) : Fit.Dom.CreateElement(newTitle);

			Fit.Dom.Replace(selection.TitleNode, newTitleNode);
			selection.Title = newTitle;
			selection.TitleNode = newTitleNode;

			if (orgTitle !== newTitle)
			{
				fireOnChange();
			}
		}
	}

	/// <function container="Fit.Controls.DropDown" name="ClearSelections" access="public">
	/// 	<description> Clear selections </description>
	/// </function>
	this.ClearSelections = function()
	{
		var fireEvent = false;
		var wasFocused = focusAssigned; // Removing an input from DOM fires OnBlur which sets focusAssigned to False

		me._internal.ExecuteWithNoOnChange(function() // picker.UpdateItemSelection results in OnChange being fired
		{
			Fit.Array.ForEach(itemCollectionOrdered, function(selection)
			{
				if (picker !== null)
				{
					// Notice: Picker fires OnItemSelectionChanged when picker.UpdateItemSelection(..) is invoked
					// below (other controls than DropDown may have registered an OnItemSelectionChanged
					// handler too). In this case we set suppressOnItemSelectionChanged to True, causing
					// drop down to do nothing in OnItemSelectionChanged handler when fired. Drop down's OnItemSelectionChanged
					// handler is responsible for handling items added/removed by picker, but in this case the change did not
					// come from the picker.
					suppressOnItemSelectionChanged = true;

					var res = true;
					var error = null;

					try // Make sure we can set suppressOnItemSelectionChanged false again, so drop down remains in a functioning state
					{
						var res = picker.UpdateItemSelection(selection.Value, false, me.Focused() === false); // OnItemSelectionChanging and OnItemSelectionChanged are fired if picker recognizes item, causing it to be removed in drop down's OnItemSelectionChanged handler (unless canceled, in which case False is returned)
					}
					catch (err) { error = err; }

					suppressOnItemSelectionChanged = false;

					if (error !== null)
						Fit.Validation.ThrowError(error);

					if (res !== false && selection.DomElement.parentElement.parentElement !== null)
					{
						// Element was not removed (still rooted in DOM), because picker did not recognize
						// it (did not fire OnItemSelectionChanged) which would otherwise have triggered Drop
						// Down's OnItemSelectionChanged handler, which in turn would have called RemoveSelection.
						// And we know it did not cancel change since picker.UpdateItemSelection did not return False.
						// It is fine to remove item.
						Fit.Dom.Remove(selection.DomElement.parentElement);
					}

					if (res !== false)
						fireEvent = true;
				}
				else
				{
					Fit.Dom.Remove(selection.DomElement.parentElement);
					fireEvent = true;
				}
			});

			Fit.Array.ForEach(itemDropZones, function(key)
			{
				itemDropZones[key].Dispose();
			});

			itemCollection = {};
			itemCollectionOrdered = [];
			itemDropZones = {};
		});

		focusAssigned = wasFocused;

		focusInput(txtPrimary);

		if (fireEvent === true)
			fireOnChange();
	}

	/// <function container="Fit.Controls.DropDown" name="RemoveSelection" access="public">
	/// 	<description> Remove selected item by value </description>
	/// 	<param name="value" type="string"> Value of selected item to remove </param>
	/// </function>
	this.RemoveSelection = function(value)
	{
		Fit.Validation.ExpectString(value);

		// Update picker control.
		// Notice: suppressUpdateItemSelectionState is True if item was removed by picker,
		// in which case picker.UpdateItemSelection(..) should not be called.
		// What happens is that picker fires drop down's OnItemSelectionChanged when an item is
		// deselected, which sets suppressUpdateItemSelectionState to True, and calls RemoveSelection.
		// Failing to prevent picker.UpdateItemSelection(..) from being called in this case could
		// result in an infinite loop.
		if (picker !== null && suppressUpdateItemSelectionState === false)
		{
			// Notice: Picker fires OnItemSelectionChanged when picker.UpdateItemSelection(..) is invoked
			// below (other controls than DropDown may have registered an OnItemSelectionChanged
			// handler too). In this case we set suppressOnItemSelectionChanged to True, causing
			// drop down to do nothing in OnItemSelectionChanged handler when fired. Drop down's OnItemSelectionChanged
			// handler is responsible for handling items added/removed by picker, but in this case the change did not
			// come from the picker (since suppressUpdateItemSelectionState is False).
			// Failing to set this flag could result in an infinite loop.
			suppressOnItemSelectionChanged = true;

			var res = false;
			var error = null;

			try // Make sure we can set suppressOnItemSelectionChanged false again, so drop down remains in a functioning state
			{
				res = picker.UpdateItemSelection(value, false, me.Focused() === false);
			}
			catch (err) { error = err; }

			suppressOnItemSelectionChanged = false;

			if (error !== null)
				Fit.Validation.ThrowError(error);

			if (res === false)
				return; // Picker prevented selection from being removed
		}

		var found = null;
		var txt = null;

		var itemObject = itemCollection[value] || null;

		if (itemObject === null)
			return;

		if (me.MultiSelectionMode() === true && itemObject.DomElement.parentElement.nextSibling !== null && itemObject.DomElement.parentElement.nextSibling !== txtPrimary)
			txt = itemObject.DomElement.parentElement.nextSibling.children[0];

		found = itemObject.DomElement.parentElement;

		if (me.TextSelectionMode() === false)
		{
			// Do not assign focus in TextSelectionMode - only txtPrimary is visible,
			// and while modern browsers will just ignore focus assignment for hidden
			// inputs and keep focus where it is, IE8 will throw an error in a modal
			// alert dialog with the following message:
			// "ThrowError: Can't move focus to the control because it is invisible, not enabled, or of a type that does not accept the focus."
			// Focus is changed internally in DropDown when de-selecting items, in which case
			// one of the inputs belonging to one of the remaining hidden items will be assigned
			// focus and trigger the error because it is hidden.

			if (me.MultiSelectionMode() === false)
				focusInput(txtPrimary);
			else
				focusInput(((txt !== null) ? txt : txtPrimary)); // Place focus in front of item to the right of removed element
		}

		Fit.Dom.Remove(found);
		Fit.Array.Remove(itemCollectionOrdered, itemObject);
		delete itemCollection[value];

		itemDropZones[value].Dispose();
		delete itemDropZones[value];

		fireOnChange();
	}

	/// <container name="Fit.Controls.DropDownTypeDefs.UpdatedDropDownItem">
	/// 	<description> DropDown item </description>
	/// 	<member name="Title" type="string"> Updated item title </member>
	/// 	<member name="Value" type="string"> unique item value </member>
	/// 	<member name="Exists" type="boolean"> Value indicating whether item still exists or not </member>
	/// </container>

	/// <function container="Fit.Controls.DropDown" name="UpdateSelected" access="public" returns="Fit.Controls.DropDownTypeDefs.UpdatedDropDownItem[]">
	/// 	<description>
	/// 		Update title of selected items based on data in associated picker control.
	/// 		An array of updated items are returned. Each object has the following properties:
	/// 		 - Title: string (Updated title)
	/// 		 - Value: string (Unique item value)
	/// 		 - Exists: boolean (True if item still exists, False if not)
	/// 		This is useful if selections are stored in a database, and
	/// 		available items may have their titles changed over time. Invoking
	/// 		this function will ensure that the selection displayed to the user
	/// 		reflects the actual state of data in the picker control. Be aware
	/// 		that this function can only update selected items if a picker has been
	/// 		associated (see SetPicker(..)), and it contains the data from which
	/// 		selected items are to be updated.
	/// 		Items that no longer exists in picker's data will not automatically
	/// 		be removed.
	/// 	</description>
	/// </function>
	this.UpdateSelected = function()
	{
		if (me.GetPicker() === null)
		{
			return [];
		}

		var updated = [];
		var selections = me.GetSelections();

		Fit.Array.ForEach(selections, function(selected)
		{
			var pickerItem = me.GetPicker().GetItemByValue(selected.Value);

			if (pickerItem !== null)
			{
				me.RenameSelection(selected.Value, pickerItem.Title);

				Fit.Array.Add(updated, { Title: pickerItem.Title, Value: selected.Value, Exists: true });
			}
			else
			{
				Fit.Array.Add(updated, { Title: selected.Title, Value: selected.Value, Exists: false });
			}
		});

		// Update text representation if TextSelectionMode is enabled
		if (me.TextSelectionMode() === true)
		{
			me.TextSelectionMode(false);
			me.TextSelectionMode(true);
		}

		return updated;
	}

	// Controlling input field

	/// <function container="Fit.Controls.DropDown" name="ClearInput" access="public">
	/// 	<description> Clear text field </description>
	/// </function>
	this.ClearInput = function(input)
	{
		Fit.Validation.ExpectInstance(input, HTMLInputElement, true);

		var inp = ((Fit.Validation.IsSet(input) === true) ? input : txtActive);

		if (inp.value === "")
			return;

		inp.value = "";

		setInputEditing(inp, false);

		if (inp === txtActive)
			prevValue = "";

		fireOnInputChanged("");
	}

	/// <function container="Fit.Controls.DropDown" name="SetInputValue" access="public">
	/// 	<description>
	/// 		Set value of text field which is automatically cleared the first time control
	/// 		receives focus. Notice that this function should be called after AddSelection(..),
	/// 		since adding selections causes the value of the text field to be cleared.
	/// 	</description>
	/// 	<param name="val" type="string"> New value for text field </param>
	/// </function>
	this.SetInputValue = function(val)
	{
		Fit.Validation.ExpectString(val);

		var txt = txtActive;
		var fireOnChange = txt.value !== val;

		if (focusAssigned === false && txt !== txtPrimary)
		{
			// DropDown does not have focus, and the currently active input field
			// is not the primary one. Change it to the primary input field instead.

			// Clear active input field in case it has a value
			txtActive.value = "";
			setInputEditing(txtActive, false);

			txt = txtPrimary;
			txtActive = txtPrimary;
		}

		setInputEditing(txt, val !== "");

		txt.value = val;
		prevValue = val;

		if (fireOnChange === true)
		{
			fireOnInputChanged(txt.value);
		}
	}

	/// <function container="Fit.Controls.DropDown" name="GetInputValue" access="public" returns="string">
	/// 	<description> Get input value </description>
	/// </function>
	this.GetInputValue = function()
	{
		return txtActive.value;
	}

	/// <function container="Fit.Controls.DropDown" name="InputEnabled" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether input is enabled </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables input, False disables it </param>
	/// </function>
	this.InputEnabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			txtEnabled = val;
		}

		return txtEnabled;
	}

	/// <function container="Fit.Controls.DropDown" name="SearchModeOnFocus" access="public" returns="boolean">
	/// 	<description>
	/// 		Clear input and display "Search.." placeholder when control receives focus.
	/// 		If SearchModeOnFocus is enabled, InputEnabled will also be enabled. Disabling
	/// 		SearchModeOnFocus does not disable InputEnabled.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables search mode on focus, False disables it </param>
	/// </function>
	this.SearchModeOnFocus = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			searchModeOnFocus = val;

			if (me.InputEnabled() === false && val === true)
			{
				me.InputEnabled(true);
			}
		}

		return searchModeOnFocus;
	}

	// Controlling drop down menu

	/// <function container="Fit.Controls.DropDown" name="OpenDropDown" access="public">
	/// 	<description> Open drop down menu </description>
	/// </function>
	this.OpenDropDown = function()
	{
		if (dropDownMenu.style.display === "block")
			return;

		if (Fit._internal.DropDown.Current !== null && Fit._internal.DropDown.Current !== me)
			Fit._internal.DropDown.Current.CloseDropDown();

		if (searchModeOnFocus === false && me.TextSelectionMode() === false && txtActive === txtPrimary && me.GetInputValue() === "")
		{
			// Visual Selection Mode with SearchModeOnFocus (which keeps txtPrimary locked in place) disabled:

			// When placing the control in search mode temporarily (not "locked" by
			// SearchModeOnFocus) via me._internal.ClearInputForSearch() or by entering a value
			// in txtPrimary, the input control "word wraps" to a new line. When it lose focus,
			// it returns to its normal position if it is empty.
			// In Visual Selection Mode with SearchModeOnFocus disabled this poses a problem
			// since interacting with the picker control results in the input control losing
			// focus.
			// So, if the user clicks on an item in the picker control while the control
			// is temporarily in search mode (remember, SearchModeOnFocus is not enabled),
			// txtPrimary lose focus, the control's height is changed because the input
			// control returns to its normal position (it no longer "word wraps"), the picker
			// control moves up, and the mouse button is released on another element than the
			// one the user initially clicked on.
			// OnClick does not fire unless the mouse button is clicked and released
			// on the same element. So in this situation the picker control would not cause
			// the given item to be selected. The user would have to try again.
			// To avoid this, we exit search mode to force txtPrimary back into place when
			// re-opening the DropDown control.

			me._internal.UndoClearInputForSearch(); // Undo/exit search mode to return txtPrimary to its normal position
		}

		// Do this before displaying drop down to prevent dropdown with position:absolute
		// from changing height of document which may cause page to temporarily scroll, hence
		// result in incorrect measurement of control position in optimizeDropDownPosition().
		optimizeDropDownPosition(true);

		dropDownMenu.style.minWidth = me.GetDomElement().offsetWidth + "px"; // In case DropDownMaxWidth(..) is set - update every time drop down is opened in case viewport is resized and has changed control width
		dropDownMenu.style.display = "block";

		me._internal.Data("open", "true");

		//optimizeDropDownPosition();

		Fit._internal.DropDown.Current = me;

		fireOnDropDownOpen();

		// Scroll selected item into view in Single Selection Mode.
		// This needs to be done after fireOnDropDownOpen() is invoked
		// because it fires PickerBase.OnShow which in TreeView resets
		// the scroll position.
		// NOTICE: WSDropDown also calls RevealItemInView(..) when
		// root nodes have been populated.

		if (me.GetPicker() !== null && me.MultiSelectionMode() === false && me.GetSelections().length === 1)
		{
			me.GetPicker().RevealItemInView(me.GetSelections()[0].Value);
		}
	}

	/// <function container="Fit.Controls.DropDown" name="CloseDropDown" access="public">
	/// 	<description> Close drop down menu </description>
	/// </function>
	this.CloseDropDown = function()
	{
		if (dropDownMenu.style.display === "none")
			return;

		dropDownMenu.style.display = "none";
		resetDropDownPosition();

		me._internal.Data("open", "false");

		Fit._internal.DropDown.Current = null;

		fireOnDropDownClose();
	}

	/// <function container="Fit.Controls.DropDown" name="IsDropDownOpen" access="public" returns="boolean">
	/// 	<description> Get flag indicating whether drop down is open or not </description>
	/// </function>
	this.IsDropDownOpen = function()
	{
		return (dropDownMenu.style.display === "block");
	}

	/// <function container="Fit.Controls.DropDown" name="DetectBoundaries" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether boundary/collision detection is enabled or not (off by default).
	/// 		This may cause drop down to open upwards if sufficient space is not available below control.
	/// 		If control is contained in a scrollable parent, this will be considered the active viewport,
	/// 		and as such define the active boundaries - unless relativeToViewport is set to True, in which
	/// 		case the actual browser viewport will be used.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables collision detection, False disables it (default) </param>
	/// 	<param name="relativeToViewport" type="boolean" default="false">
	/// 		If defined, True results in viewport being considered the container to which available space is determined.
	/// 		This also results in DropDown menu being positioned with position:fixed, allowing it to escape a container
	/// 		with overflow (e.g. overflow:auto|hidden|scroll). Be aware though that this does not work reliably in combination
	/// 		with CSS animation and CSS transform as it creates a new stacking context to which position:fixed becomes relative.
	/// 		A value of False (default) results in available space being determined relative to the boundaries of the
	/// 		control's scroll parent. The DropDown menu will stay within its container and not overflow it.
	/// 	</param>
	/// </function>
	this.DetectBoundaries = function(val, relativeToViewport)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(relativeToViewport, true);

		// NOTICE: This feature is known to not work properly with iOS 9 and below:
		// https://github.com/Jemt/Fit.UI/issues/87

		if (Fit.Validation.IsSet(val) === true)
		{
			detectBoundaries = val;
			detectBoundariesRelToViewPort = relativeToViewport === true;
		}

		return detectBoundaries;
	}

	// ============================================
	// Events (OnChange defined on BaseControl)
	// ============================================

	/// <function container="Fit.Controls.DropDownTypeDefs" name="InputChangedEventHandler">
	/// 	<description> Input changed event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="value" type="string"> Input value </param>
	/// </function>

	/// <function container="Fit.Controls.DropDownTypeDefs" name="PasteEventHandler" returns="boolean | void">
	/// 	<description> Paste event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="value" type="string"> Value pasted into input field </param>
	/// </function>

	/// <function container="Fit.Controls.DropDownTypeDefs" name="InteractionEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.DropDown" name="OnInputChanged" access="public">
	/// 	<description>
	/// 		Add event handler fired when input value is changed.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.DropDown) and Value (string).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DropDownTypeDefs.InputChangedEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnInputChanged = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onInputChangedHandlers, cb);
	}

	/// <function container="Fit.Controls.DropDown" name="OnPaste" access="public">
	/// 	<description>
	/// 		Add event handler fired when text is pasted into input field.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.DropDown) and Value (string).
	/// 		Return False to cancel event and change, and prevent OnInputChanged from firing.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DropDownTypeDefs.PasteEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnPaste = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onPasteHandlers, cb);
	}

	/// <function container="Fit.Controls.DropDown" name="OnOpen" access="public">
	/// 	<description>
	/// 		Add event handler fired when drop down menu is opened.
	/// 		Function receives one argument: Sender (Fit.Controls.DropDown)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DropDownTypeDefs.InteractionEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnOpen = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onOpenHandlers, cb);
	}

	/// <function container="Fit.Controls.DropDown" name="OnClose" access="public">
	/// 	<description>
	/// 		Add event handler fired when drop down menu is closed.
	/// 		Function receives one argument: Sender (Fit.Controls.DropDown)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.DropDownTypeDefs.InteractionEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnClose = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onCloseHandlers, cb);
	}

	// ============================================
	// Protected
	// ============================================

	this._internal = (this._internal ? this._internal : {});

	this._internal.ClearInputForSearch = function(keepDropDownOpen) // Put input in search mode
	{
		Fit.Validation.ExpectBoolean(keepDropDownOpen, true);

		if (me.TextSelectionMode() === true)
		{
			forceFocusInput(txtPrimary);
			txtPrimary.value = "";
			updatePlaceholder(true);
		}
		else
		{
			forceFocusInput(txtPrimary);
			me.ClearInput();
			setInputEditing(txtPrimary, true);
		}

		// If DropDown is in Visual Selection Mode, the search field is now displayed
		// on a separate "line" (see code above where setInputEditing(..) is called).
		// When that input field lose focus, it will immediately return to its normal
		// state. This results in the control's height changing which in turn results
		// in the pulldown menu moving up. So if a user clicks an item in the pulldown
		// menu, the input lose focus, the position of the pulldown menu changes, and
		// nothing happens from the click because the picker uses a traditional onclick
		// event to register interactions, which doesn't fire unless the mouse button is released
		// on the same object it was pressed down on. We avoid this by closing the control.
		// We close it in both Text Selection Mode and Visual Selection Mode for consistency.
		if (keepDropDownOpen !== true)
		{
			me.CloseDropDown();
		}
	}

	this._internal.UndoClearInputForSearch = function() // Undo/exit search mode for input field
	{
		if (me.TextSelectionMode() === true)
		{
			updateTextSelection(); // Also removes placeholder by calling updatePlaceholder(true)
		}
		else
		{
			me.ClearInput();
			setInputEditing(txtPrimary, false);
		}
	}

	// Focus input field on mobile and keep it focused, even if DropDown was opened using arrow icon
	this._internal.ForceFocusMobile = function()
	{
		focusInputOnMobile = true;
		focusInput(txtPrimary);
	}

	// ============================================
	// Private
	// ============================================

	function createSearchField()
	{
		var txt = document.createElement("input");
		txt.autocomplete = "off";
		txt.spellcheck = false;

		txt.onpaste = function(e)
		{
			var orgValue = txt.value;

			postpone(function() // Timeout to queue event to have pasted value available
			{
				/*if (me.TextSelectionMode() === true && txt.value === prevTextSelection)
				{
					// User just copied selection text and pasted it again - ignore
					// so we don't fire OnInputChanged below and change prevValue which
					// will cause OnInputChanged to fire once again when control lose focus.
					return;
				}*/

				// DISABLED - does not make sense.
				// Pasting a value would not fire OnInputChanged, but starting to delete
				// the characters just pasted would. So pasting "ABC" would not fire anything, but
				// pressing backspace would fire OnInputChanged with the value "AB" - that is odd.
				/*if (txt.value === prevValue || txt.value === prevTextSelection)
				{
					return; // Skip event if user just copied text and pasted it again
				}*/

				prevValue = txt.value;
				var pastedValue = txt.value;

				setInputEditing(txt, true);

				if (fireOnPaste(txt.value) === true)
				{
					if (txt.value === pastedValue)
					{
						// No OnPaste handler altered input value by calling SetInputValue(..)

						fireOnInputChanged(txt.value);
					}
				}
				else
				{
					// Paste canceled - restore old value, unless OnPaste handler called SetInputValue with a different value

					if (txt.value === pastedValue)
					{
						txt.value = orgValue;
						prevValue = txt.value;
					}
				}
			});

			clearTextSelectionOnInputChange = false;
		}

		txt.ontouchstart = function(e) // Ontouchstart fires before onfocus which is needed - txt.onfocus uses focusInputOnMobile
		{
			focusInputOnMobile = true; // Always focus inputs on mobile if user initially activated control by clicking/touching an input field
		}

		txt.onfocus = function(e)
		{
			if (moveFocusToArrowOnMobile() === true) // Also found in focusInput(..) but is also necessary here in case focus was received using a tablet's virtual keyboard's "next/previous control" navigation
			{
				return; // Focused was moved to arrow
			}

			if (Fit.Dom.GetIndex(txt.parentElement) === itemCollectionOrdered.length - 1 && txt.nextSibling === null)
			{
				// Right input control in last selected item - focus txtPrimary instead so it word wraps alone when entering text
				txtPrimary.focus();
				return;
			}

			focusAssigned = true;

			if (initialFocus === true)
			{
				initialFocus = false;

				if (me.TextSelectionMode() === false)
					me.ClearInput(txtPrimary);
			}

			txtActive = txt;
			prevValue = (me.TextSelectionMode() === false ? txtActive.value : prevValue); // Do not assign text selection to prevValue - it should only hold changes made by user or by calls to SetInputValue(..)

			clearAllInputsButActive();
		}

		txt.onblur = function(e)
		{
			if (me === null)
			{
				// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
				// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
				// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
				return;
			}

			focusAssigned = false;

			if (txt.value === "")
			{
				setInputEditing(txt, false);
			}
		}

		txt.onkeydown = function(e) // Fires continuously for any key pressed - both characters and e.g backspace/delete/arrows etc. Key press may be canceled (change has not yet occured)
		{
			var ev = Fit.Events.GetEvent(e);

			txtActive = txt;
			clearAllInputsButActive();

			if (picker !== null)
			{
				if (me.IsDropDownOpen() === true)
				{
					var res = picker.HandleEvent(ev);

					if (res === false)
						return; // Picker returned False to prevent DropDown from also handling event
				}
			}

			if (me.TextSelectionMode() === true && clearTextSelectionOnInputChange === true && txtEnabled === true)
			{
				// Clear input in Text Selection Mode if user starts entering a value

				// Skip if key press is TAB, ENTER, ESC, LEFT, UP, RIGHT, or DOWN.
				// Also ignore key press if combined with modifier keys (except if SHIFT+Key for upper case letters, of course).
				var modKeys = Fit.Events.GetModifierKeys();
				if (Fit.Array.Contains([9, 13, 27, 37, 38, 39, 40], ev.keyCode) === false && (modKeys.Shift === false || modKeys.KeyDown !== 16) && modKeys.Ctrl === false && modKeys.Alt === false && modKeys.Meta === false)
				{
					// User is entering a value - clear input before character is applied to input

					clearTextSelectionOnInputChange = false;
					txt.value = ""; // No need to update prevValue - this is not the value set by the user - see onkeyup which updates prevValue

					return;
				}
			}

			if (ev.keyCode === 9) // Tab
			{
				if (me.TextSelectionMode() === true)
				{
					me.CloseDropDown();
					return;
				}

				if (ev.shiftKey === true) // Moving left
				{
					if (me.GetSelections().length === 0)
					{
						me.CloseDropDown();
						return; // Let browser handle navigation
					}
					else if (txt !== txtPrimary && txt.parentElement.previousSibling === null)
					{
						me.CloseDropDown();
						return; // Let browser handle navigation
					}

					moveToInput("Prev");
				}
				else // Moving right
				{
					if (txt === txtPrimary)
					{
						me.CloseDropDown();
						return; // Let browser handle navigation
					}

					moveToInput("Next");
				}

				return false;
			}
			else if (ev.keyCode === 38) // Arrow up
			{
				me.OpenDropDown(); // Make sure it is opened
			}
			else if (ev.keyCode === 40) // Arrow down
			{
				me.OpenDropDown(); // Make sure it is opened

				if (me.TextSelectionMode() === true) // && (Fit.Browser.GetBrowser() !== "MSIE" || Fit.Browser.GetVersion() !== 8)
				{
					// Set cursor at the beginning of text field.
					// Suppressing default behaviour to prevent cursor
					// from jumping to the end. SetCaretPosition(..)
					// is also necessary to make sure text value
					// does not remain selected (blue text selection) if
					// focus was assigned using tab navigation.
					Fit.Dom.SetCaretPosition(txtPrimary, 0);
					Fit.Events.PreventDefault(ev);
				}
			}
			else if (ev.keyCode === 37) // Arrow left
			{
				if (me.TextSelectionMode() === true)
					return;

				moveToInput("Prev");
			}
			else if (ev.keyCode === 39) // Arrow right
			{
				if (me.TextSelectionMode() === true)
					return;

				moveToInput("Next");
			}
			else if (ev.keyCode === 27) // Escape
			{
				if (Fit.Browser.GetInfo().Name === "MSIE")
					Fit.Events.PreventDefault(ev); // Do not clear input on IE

				me.CloseDropDown();
			}
			else if (ev.keyCode === 8) // Backspace - remove selection
			{
				if (me.TextSelectionMode() === true)
				{
					if (txtEnabled === false)
					{
						Fit.Events.PreventDefault(ev); // Prevent user from clearing input value
					}

					return;
				}

				if (txt.value.length === 0)
				{
					if (txt === txtPrimary)
					{
						me._internal.UndoClearInputForSearch();
					}

					if (Fit.Browser.GetInfo().Name === "MSIE")
						Fit.Events.PreventDefault(ev); // Do not navigate back on IE when backspace is pressed within input being removed

					var toRemove = null;

					if (txt !== txtPrimary && Fit.Dom.GetIndex(txt) === 2) // Right input
						toRemove = txt.parentElement;
					else if (txt !== txtPrimary && Fit.Dom.GetIndex(txt) === 0 && txt.parentElement.previousSibling !== null) // Left input
						toRemove = txt.parentElement.previousSibling;
					else if (txt.previousSibling !== null) // Primary input
						toRemove = txt.previousSibling;

					if (toRemove !== null)
					{
						me.RemoveSelection(decode(Fit.Dom.Data(toRemove.children[1], "value")));
					}
				}
			}
			else if (ev.keyCode === 46) // Delete - remove selection
			{
				if (me.TextSelectionMode() === true)
				{
					if (txtEnabled === false)
					{
						Fit.Events.PreventDefault(ev); // Prevent user from clearing input value
					}

					return;
				}

				if (txt.value.length === 0)
				{
					var toRemove = null;

					if (txt !== txtPrimary && Fit.Dom.GetIndex(txt) === 0) // Left input
						toRemove = txt.parentElement;
					else if (txt !== txtPrimary && Fit.Dom.GetIndex(txt) === 2 && txt.parentElement.nextSibling.tagName === "SPAN") // Right input
						toRemove = txt.parentElement.nextSibling;

					if (toRemove !== null)
					{
						me.RemoveSelection(decode(Fit.Dom.Data(toRemove.children[1], "value")));
					}
				}
			}
			else if (ev.keyCode === 13) // Enter - notice that item selection is handled by (delegated to) picker when Enter is pressed - picker.OnItemSelectionChanged receives selected item
			{
				// Prevent form submit
				Fit.Events.PreventDefault(e);
			}
			else
			{
				if (txtEnabled === false)
				{
					if (Fit.Events.GetModifierKeys().Meta === false && Fit.Events.GetModifierKeys().Ctrl === false)
					{
						// Suppress key press unless modifier key is being used (e.g. CTRL+F5 or CMD+R to reload page)
						Fit.Events.PreventDefault(ev);
					}

					return;
				}
			}
		}

		txt.onkeypress = function(e) // Fires continuously for real character keys (unless suppressed in OnKeyDown)
		{
			updatePlaceholder(true, true); // Make sure placeholder is removed immediately on keystroke

			setInputEditing(txt, true);
		}

		txt.onkeyup = function(e) // Fires only once when a key is released (unless suppressed in OnKeyDown)
		{
			var ev = Fit.Events.GetEvent(e);

			if (ev.keyCode !== 37 && ev.keyCode !== 38 && ev.keyCode !== 39 && ev.keyCode !== 40 && ev.keyCode !== 27 && ev.keyCode !== 9 && ev.keyCode !== 16 && ev.ctrlKey === false && ev.keyCode !== 17) // Do not fire change event for: arrow keys, escape, tab, shift, and on paste (never fires when CTRL is held down (ev.ctrlKey true) or released (ev.keyCode 17))
			{
				if (me.TextSelectionMode() === true && txt.value === prevTextSelection)
				{
					// Do not fire OnInputChanged. Input value is identical to text selection
					// so user probably pressed SPACE to toggle a selection (which doesn't change input value), or
					// pressed some other none-character key (e.g CMD/SUPER or ALT/OPTION) which did not alter input value.
					return;
				}

				if (txt.value !== prevValue /*&& (me.TextSelectionMode() === false || txt.value !== prevTextSelection)*/)
				{
					prevValue = txt.value;
					fireOnInputChanged(txt.value);
				}
			}
		}

		return txt;
	}

	function createItemObject(title, value, valid, domElement, titleNode)
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(valid);
		Fit.Validation.ExpectDomElement(domElement);
		Fit.Validation.ExpectNode(titleNode); // TextNode if HtmlAllowed() is false, DOMElement if HtmlAllowed() is true

		return { Title: title, Value: value, Valid: valid, DomElement: domElement, TitleNode: titleNode };
	}
	/*function convertItemElementToItemObject(itemElm)
	{
		Fit.Validation.ExpectDomElement(itemElm);
		return { Title: Fit.Dom.Text(itemElm), Value: decode(Fit.Dom.Data(itemElm, "value")), Valid: Fit.Dom.HasClass(itemElm, "FitUiControlDropDownInvalid") === false, DomElement: itemElm };
	}*/

	/*function getFirstSelectionElement()
	{
		if (itemCollectionOrdered.length > 0)
		{
			return itemCollectionOrdered[0].DomElement;
		}

		return null;
	}*/

	function moveToInput(direction) // direction = Next/Prev
	{
		Fit.Validation.ExpectString(direction);

		var newInput = null;

		if (direction === "Prev") // Moving left
		{
			if (txtActive !== txtPrimary && Fit.Dom.GetIndex(txtActive) === 0 && txtActive.parentElement.previousSibling !== null) // Left input has focus
				newInput = txtActive.parentElement.previousSibling.children[0];
			else if (txtActive !== txtPrimary && Fit.Dom.GetIndex(txtActive) === 2) // Right input has focus
				newInput = txtActive.parentElement.children[0];
			else if (txtActive.previousSibling !== null) // Primary input has focus
				newInput = txtActive.previousSibling.children[0];
		}
		else // Moving right
		{
			// TODO: Rename this! We have a private property in this class with the same name!
			var itemContainer = null; // Remains Null if last item's right input control has focus (unlikely since it will never be focused unless user manages to actually click it (5px wide)

			if (txtActive !== txtPrimary && Fit.Dom.GetIndex(txtActive) === 0) // Left input has focus
				itemContainer = txtActive.parentElement; // Use current item
			else if (txtActive !== txtPrimary && Fit.Dom.GetIndex(txtActive) === 2 && txtActive.parentElement.nextSibling.tagName === "SPAN") // Right input has focus
				itemContainer = txtActive.parentElement.nextSibling; // Use next item

			if (itemContainer !== null)
				newInput = itemContainer.children[2];
		}

		if (newInput === null) // May be null if no selections are available to move to
			return;

		// Move content
		/*newInput.value = txtActive.value;
		me.ClearInput();*/

		focusInput(newInput);
	}

	function moveFocusToArrowOnMobile()
	{
		// Move focus to arrow icon if on mobile and if user activated control in a way where bringing focus to
		// input fields does not make sense (search for focusInputOnMobile), or if control is configured with
		// TextSelectionMode(true) and InputEnabled(false) which doesn't allow user to place cursor between
		// selected items, nor write in the input field.
		if (isMobile === true && (focusInputOnMobile === false || (me.TextSelectionMode() === true && me.InputEnabled() === false)))
		{
			arrow.focus();	// Notice that arrow.onfocus will set focusAssigned
			return true;	// Indicate that focus was moved to arrow icon
		}

		return false; // Indicate that focus was NOT moved to arrow icon
	}

	function focusInput(input)
	{
		Fit.Validation.ExpectInstance(input, HTMLInputElement);

		txtActive = input;

		if (focusAssigned === true) // Only set focus if user initially assigned focus to control
		{
			if (moveFocusToArrowOnMobile() === false) // Returns false if focus was NOT moved to arrow icon
			{
				txtActive.focus(); // Notice: Input field's onfocus handler will move focus to txtPrimary if this is the last selection's right side input field - on mobile it might redirect focus to the arrow icon to avoid bringing up the virtual keyboard
			}
		}
	}

	function forceFocusInput(input)
	{
		Fit.Validation.ExpectInstance(input, HTMLInputElement);

		// By default focus is never assigned to input on mobile if user opened the
		// DropDown by clicking the arrow icon. ForceFocusInput allows us to ignore this
		// aspect. On desktop the input field always remains focused  as it automatically
		// steals back focus immediately after interacting with a picker control. See
		// picker.OnFocusIn handler registered in SetPicker(..)
		var orgFocusInputOnMobile = focusInputOnMobile;
		focusInputOnMobile = true;
		focusInput(input);
		focusInputOnMobile = orgFocusInputOnMobile;
	}


	function setInputEditing(input, val, keepStateOnParent) // Input being edited "word wraps" to separate line
	{
		Fit.Validation.ExpectInstance(input, HTMLInputElement);
		Fit.Validation.ExpectBoolean(val);
		Fit.Validation.ExpectBoolean(keepStateOnParent, true);

		// Do not change editing state for txtPrimary when SearchModeOnFocus is enabled.
		// In this case txtPrimary remains locked in editing mode so it remains visible,
		// even when changing focus within the control and when adding or removing items.
		if (searchModeOnFocus === true && input === txtPrimary)
		{
			return;
		}

		if (keepStateOnParent !== true)
		{
			Fit.Dom.Data(input.parentElement, "editing", val === true ? "true" : null);
		}

		Fit.Dom.Data(input, "editing", val === true ? "true" : null);

		if (input === txtPrimary)
		{
			// DropDown's placeholder is only displayed when no items are selected.
			// Therefore, use native placeholder for input if items are selected.
			txtPrimary.placeholder = val === true && me.GetSelections().length > 0 ? me.Placeholder() : "";
		}

		me._internal.Repaint();
	}

	function clearAllInputsButActive()
	{
		var inputs = itemContainer.getElementsByTagName("input");

		Fit.Array.ForEach(inputs, function(input)
		{
			if (input === txtActive)
				return;

			if (txtActive.parentElement === input.parentElement)
			{
				// Input is contained in a selected object whose other input is txtActive.
				// Do not remove editing state from parent element in this case - keep it!
				setInputEditing(input, false, true);
			}
			else
			{
				setInputEditing(input, false);
			}

			me.ClearInput(input);
		});

		me._internal.Repaint(); // Repaint - data attributes changed above
	}

	function optimizeDropDownPosition(force)
	{
		Fit.Validation.ExpectBoolean(force, true);

		if (detectBoundaries === false || (me.IsDropDownOpen() === false && force !== true))
			return;

		// Check visibility in case DropDown control is hidden while items are being selected.
		// This is a common approach to reduce expensive reflows when programmatically selecting
		// many items (e.g. using SelectAll in TreeView picker). If control is in fact hidden,
		// the picker container will keep its current position rather than being positioned
		// below/above DropDown as expected - that's acceptable. Re-opening DropDown will position
		// it properly. Without this fix, Fit.Dom.GetBoundingPosition(..) used below would return Null and
		// result in an error.
		if (Fit.Dom.IsVisible(me.GetDomElement()) === false)
			return;

		// DropDown menu is positioned above control if sufficient space is not available below it. It will always prefer
		// to open downwards if possible. The MaxHeight of the DropDown menu will be adjusted automatically to better fit.
		// In addition, if sufficient space is not available to the right of the control, the DropDown menu will open to
		// the left side instead. If sufficient space is not available on the left side either, it will choose the side with
		// the most space, and adjust the DropDown menu's MaxWidth to make it fit.

		// DropDown menu is positioned relative to itemContainer which is what makes up the DropDown UI - me.GetDomElement() is merely a container with no styling by default

		// Allow DropDown to reposition when items are added/remove which might affect height of control
		resetDropDownPosition();

		var spaceRequiredBelowControl = 100;							// Opens upwards if this amount of pixels is not available below control, and more space is available above control
		var spaceRequiredRightSide = getDropDownMaxWidthPixelValue();	// DropDownMaxWidth as px value - DropDown menu opens to the side that best accommodates the needed space - opening to the right is preferred
		var spacingToEdge = 10;											// Makes sure that DropDown menu has this amount of spacing (in pixels) to the edge of the viewport or scroll parent

		if (detectBoundariesRelToViewPort === false) // Detecting collisions against scroll parent
		{
			var posFromTopWithinContainer = -1;		// DropDown control's position from top within scrollable parent
			var posFromLeftWithinContainer = -1;	// DropDown control's position from left within scrollable parent
			var availableSpaceBelowControl = -1;	// Number of pixels available below control (from bottom side of control)
			var availableSpaceRightOfControl = -1;	// Number of pixels available to the right of control (from right side of control)
			var mostSpaceAboveControl = false;		// Flag indicating whether more space is available above control or not
			var mostSpaceLeftOfControl = false;		// Flag indicating whether more space is available to the left of control or not
			var condNotEnoughSpaceBelow = false;	// Flag indicating whether DropDown menu has insufficient space below control - opens upwards if this is true and mostSpaceAboveControl is true
			var condNotEnoughSpaceRightSide = false;// Flag indicating whether DropDown menu has insufficient space to the right of control - opens to the left if this is true and mostSpaceLeftOfControl is true

			var overflowParent = Fit.Dom.GetOverflowingParent(me.GetDomElement()); // Contrary to GetScrollParent(..), GetOverflowingParent(..) also take overflow:hidden into account

			if (overflowParent !== null) // Contained in custom scroll parent (not document)
			{
				var overflowParentPosition = Fit.Dom.GetBoundingPosition(overflowParent);
				var parentPosY = overflowParentPosition.Y + parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-top-width")); // Position inside scroll parent and inside its borders which are placed inside the box (box-sizing:border-box)
				var parentPosX = overflowParentPosition.X + parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-right-width")); // Position inside scroll parent and inside its borders which are placed inside the box (box-sizing:border-box)
				var parentInnerHeight = overflowParent.offsetHeight - parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-top-width")) - parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-bottom-width")); // Substract borders so we have the actual space available to the positioned element - borders are placed inside the box (box-sizing:border-box)
				var parentInnerWidth = overflowParent.offsetWidth - parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-left-width")) - parseInt(Fit.Dom.GetComputedStyle(overflowParent, "border-right-width")); // Substract borders so we have the actual space available to the positioned element - borders are placed inside the box (box-sizing:border-box)
				var controlPos = Fit.Dom.GetBoundingPosition(itemContainer);
				var parentScrollbars = Fit.Dom.GetScrollBars(overflowParent);

				posFromTopWithinContainer = controlPos.Y - parentPosY;
				posFromLeftWithinContainer = controlPos.X - parentPosX;
				availableSpaceBelowControl = parentInnerHeight - (posFromTopWithinContainer + itemContainer.offsetHeight) - parentScrollbars.Horizontal.Size;
				availableSpaceRightOfControl = parentInnerWidth - (posFromLeftWithinContainer + itemContainer.offsetWidth) - parentScrollbars.Vertical.Size;
			}
			else // Document itself is scroll parent
			{
				var controlPos = Fit.Dom.GetBoundingPosition(itemContainer);
				var controlPositionY = controlPos.Y;
				var controlPositionX = controlPos.X;
				var viewPortDimensions = Fit.Browser.GetViewPortDimensions(); // Returns { Width, Height } - actual space available (scrollbars are not included in these dimensions)
				var spaceBelowControl = viewPortDimensions.Height - (controlPositionY + itemContainer.offsetHeight); // Space available below control
				var spaceRightOfControl = viewPortDimensions.Width - (controlPositionX + itemContainer.offsetWidth); // Space available right of control

				posFromTopWithinContainer = controlPositionY;
				posFromLeftWithinContainer = controlPositionX;
				availableSpaceBelowControl = spaceBelowControl;
				availableSpaceRightOfControl = spaceRightOfControl;
			}

			mostSpaceAboveControl = posFromTopWithinContainer > availableSpaceBelowControl;
			mostSpaceLeftOfControl = posFromLeftWithinContainer > availableSpaceRightOfControl;
			condNotEnoughSpaceBelow = availableSpaceBelowControl - spacingToEdge < spaceRequiredBelowControl;
			condNotEnoughSpaceRightSide = spaceRequiredRightSide === -1 ? false /* DropDownMaxWidth not set - will not extend beyond width of control */ : availableSpaceRightOfControl - spacingToEdge < spaceRequiredRightSide;

			// Open upward or downward (default)

			var spaceAvailable = -1;

			if (condNotEnoughSpaceBelow === true && mostSpaceAboveControl === true) // Open upward
			{
				dropDownMenu.style.bottom = itemContainer.offsetHeight + "px";
				spaceAvailable = posFromTopWithinContainer - spacingToEdge;
			}
			else // Open like normal, downwards
			{
				spaceAvailable = availableSpaceBelowControl - spacingToEdge;
			}

			// Adjust DropDown menu's MaxHeight to make it fit

			if (picker !== null && (maxHeight.Unit !== "px" || maxHeight.Value > spaceAvailable))
			{
				// Reduce DropDown menu's max height.
				// CSS unit for MaxHeight is something like 'em' (so enforce pixel based
				// max height), or pixel based MaxHeight simply exceeds space available.

				picker.MaxHeight(spaceAvailable, "px");
			}

			if (picker !== null && Fit.Core.IsEqual(picker.MaxHeight(), maxHeight) === false && picker.MaxHeight().Value < maxHeight.Value)
			{
				// MaxHeight has previously been reduced by optimizeDropDownPosition and is now smaller than
				// the value initially configured for the DropDown menu. Increase its size again if sufficient space is available.

				picker.MaxHeight((maxHeight.Value < spaceAvailable ? maxHeight.Value : spaceAvailable), "px");
			}

			// Open left or right (default)

			if (condNotEnoughSpaceRightSide === true && mostSpaceLeftOfControl === true) // Open to the left
			{
				dropDownMenu.style.right = "0px";
				var spaceAvailableLeftSide = posFromLeftWithinContainer + itemContainer.offsetWidth - spacingToEdge;

				if (spaceAvailableLeftSide < spaceRequiredRightSide) // Adjust max-width if sufficient space is not available to the left side
				{
					dropDownMenu.style.maxWidth = spaceAvailableLeftSide + "px";
				}
			}
			else // Open like normal, to the right
			{
				var spaceAvailableRightSide = availableSpaceRightOfControl + itemContainer.offsetWidth - spacingToEdge;

				if (spaceAvailableRightSide < spaceRequiredRightSide) // Adjust max-width if sufficient space is not available to the right side
				{
					dropDownMenu.style.maxWidth = spaceAvailableRightSide + "px";
				}
			}
		}
		else // Position DropDown menu relative to viewport using position:fixed
		{
			// Position:fixed has the ability to escape overflow:hidden|auto|scroll,
			// but also results in the DropDown menu being "detached" from the
			// control, allowing the control to be scrolled away from the
			// DropDown menu - it sticks to the screen.
			// Also notice that position:fixed does not work reliable if control
			// is contained in a parent that uses CSS animation or transform,
			// as this creates a new stacking context to which position:fixed
			// becomes relative.

			var viewPortDimensions = Fit.Browser.GetViewPortDimensions();											// Returns { Width, Height } - actual space available (scrollbars are not included in these dimensions)
			var controlPositionY = Fit.Dom.GetBoundingPosition(itemContainer).Y;									// Position from top
			var controlPositionX = Fit.Dom.GetBoundingPosition(itemContainer).X;									// Position from left
			var spaceAboveControl = controlPositionY;																// Space available above control
			var spaceBelowControl = viewPortDimensions.Height - (controlPositionY + itemContainer.offsetHeight);	// Space available below control
			var spaceLeftOfControl = controlPositionX;																// Soace available left of control (from left side of control)
			var spaceRightOfControl = viewPortDimensions.Width - (controlPositionX + itemContainer.offsetWidth);	// Space available right of control (from right side of control)
			var mostSpaceAboveControl = spaceAboveControl > spaceBelowControl;										// True if there is more space available above control than below control
			var mostSpaceLeftOfControl = spaceRightOfControl < spaceLeftOfControl;									// True if there is more space available right of control than left of control
			var condNotEnoughSpaceBelow = spaceBelowControl - spacingToEdge < spaceRequiredBelowControl;
			var condNotEnoughSpaceRightSide = spaceRightOfControl + itemContainer.offsetWidth - spacingToEdge < spaceRequiredRightSide;

			dropDownMenu.style.position = "fixed";	// Using fixed positioning to escape containers with overflow:scroll|hidden|auto
			dropDownMenu.style.width = "auto";		// Picker by default has width:100% to assume the same width as the control, except if DropDownMaxWidth is set, in which case it is already "auto"

			// Open upward or downward (default)

			var spaceAvailable = -1;

			if (condNotEnoughSpaceBelow === true && mostSpaceAboveControl === true) // Open upward
			{
				// Handle situation where the control is contained in a parent with scroll
				// and the control has been partially scrolled out of view. In this case
				// we do not want to position the DropDown menu where the (now hidden)
				// top of the control is located in the viewport, but where the scrollable
				// container starts.
				// https://github.com/Jemt/Fit.UI/issues/51

				var scrollParent = Fit.Dom.GetScrollParent(me.GetDomElement());

				if (scrollParent !== Fit.Dom.GetScrollDocument())
				{
					// Control is positioned within a container with scroll.
					// Calculate position relative to viewport to determine
					// whether control has been scrolled out of view.

					var scrollParentPosY = Fit.Dom.GetBoundingPosition(scrollParent).Y;
					scrollParentPosY = scrollParentPosY + parseInt(Fit.Dom.GetComputedStyle(scrollParent, "border-top-width"));

					if (controlPositionY < scrollParentPosY)
					{
						// Relative to the viewport the control is positioned above the
						// scroll container which means it has been scrolled out of view.
						controlPositionY = scrollParentPosY;
					}
				}

				dropDownMenu.style.bottom = (viewPortDimensions.Height - controlPositionY) + "px";
				spaceAvailable = controlPositionY - spacingToEdge;
			}
			else // Open like normal, downwards
			{
				// Handle situation where the control is contained in a parent with scroll
				// and the control has been partially scrolled out of view. In this case
				// we do not want to position the DropDown menu where the (now hidden)
				// bottom of the control is located in the viewport, but where the scrollable
				// container ends.
				// https://github.com/Jemt/Fit.UI/issues/51 (see re-open comment oct. 30, 2020)

				var scrollParent = Fit.Dom.GetScrollParent(me.GetDomElement());
				var alternativeTopPos = -1;

				if (scrollParent !== Fit.Dom.GetScrollDocument())
				{
					// Control is positioned within a container with scroll.
					// Calculate position relative to viewport to determine
					// whether control has been scrolled out of view.

					var scrollParentBottomPosY = Fit.Dom.GetBoundingPosition(scrollParent).Y + scrollParent.offsetHeight - parseInt(Fit.Dom.GetComputedStyle(scrollParent, "border-bottom-width"));
					var controlBottomPosY = controlPositionY + itemContainer.offsetHeight;

					if (controlBottomPosY > scrollParentBottomPosY)
					{
						// Relative to the viewport the bottom of the control is positioned below the
						// bottom of the scroll container which means it has been scrolled out of view.
						alternativeTopPos = scrollParentBottomPosY;
					}
				}

				dropDownMenu.style.top = (alternativeTopPos !== -1 ? alternativeTopPos : controlPositionY + itemContainer.offsetHeight) + "px";
				spaceAvailable = spaceBelowControl - spacingToEdge;
			}

			if (picker !== null && (maxHeight.Unit !== "px" || maxHeight.Value > spaceAvailable))
			{
				// Reduce drop down menu's max height.
				// CSS unit for MaxHeight is something like 'em' (so enforce pixel based
				// max height), or pixel based MaxHeight simply exceeds space available.

				picker.MaxHeight(spaceAvailable, "px");
			}

			if (picker !== null && Fit.Core.IsEqual(picker.MaxHeight(), maxHeight) === false && picker.MaxHeight().Value < maxHeight.Value)
			{
				// MaxHeight has previously been reduced by optimizeDropDownPosition and is now smaller than
				// the value initially configured for the drop down. Increase its size again if sufficient space is available.

				picker.MaxHeight((maxHeight.Value < spaceAvailable ? maxHeight.Value : spaceAvailable), "px");
			}

			// Open left or right (defafult)

			if (condNotEnoughSpaceRightSide === true && mostSpaceLeftOfControl === true) // Open to the left
			{
				dropDownMenu.style.right = spaceRightOfControl + "px";
				var spaceAvailableLeftSide = controlPositionX + itemContainer.offsetWidth - spacingToEdge; // Space available to the left, from right side of control

				if (spaceAvailableLeftSide < spaceRequiredRightSide) // Adjust max-width if sufficient space is not available to the left side
				{
					dropDownMenu.style.maxWidth = spaceAvailableLeftSide + "px";
				}
			}
			else // Open like normal, to the right
			{
				dropDownMenu.style.left = controlPositionX + "px"; // Set in case document or containers have been scrolled
				var spaceAvailableRightSide = spaceRightOfControl + itemContainer.offsetWidth - spacingToEdge; // Space available to the right, from left side of control

				if (spaceAvailableRightSide < spaceRequiredRightSide) // Adjust max-width if sufficient space is not available to the right side
				{
					dropDownMenu.style.maxWidth = spaceAvailableRightSide + "px";
				}
			}
		}
	}

	function resetDropDownPosition()
	{
		// Reset changes made by optimizeDropDownPosition()
		dropDownMenu.style.position = "";
		dropDownMenu.style.width = (maxWidth.Value > -1 ? dropDownMenu.style.width : ""); // Preserve width if DropDownMaxWidth is enabled since it also modifies this property
		dropDownMenu.style.maxWidth = (maxWidth.Value > -1 ? maxWidth.Value + maxWidth.Unit : "");
		dropDownMenu.style.bottom = "";
		dropDownMenu.style.top = "";
		dropDownMenu.style.left = "";
		dropDownMenu.style.right = "";

		if (picker !== null) // Checking in case picker was removed while opened (unlikely though)
			picker.MaxHeight(maxHeight.Value, maxHeight.Unit);
	}

	function getDropDownMaxWidthPixelValue()
	{
		var maxWidthPixels = -1;

		if (maxWidth.Value !== -1)
		{
			if (maxWidth.Unit === "px")
			{
				maxWidthPixels = maxWidth.Value;
			}
			else
			{
				maxWidthPixels = parseInt(Fit.Dom.GetComputedStyle(dropDownMenu, "maxWidth")); // Get computed pixel value rather than e.g. 15% or 20em
			}
		}

		return maxWidthPixels;
	}

	function updatePlaceholder(force, willAssumeInputValue)
	{
		Fit.Validation.ExpectBoolean(force, true);
		Fit.Validation.ExpectBoolean(willAssumeInputValue, true);

		if (Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() < 10)
		{
			// Placeholder is not supported on IE8 as it currently relies on calc(..) in CSS.
			// Furthermore native support for placeholders (using the real placeholder attribute)
			// were introduced in IE10, so this also ensure consistent behaviour for all controls,
			// as e.g. Input and DatePicker uses the native placeholder implementation.
			return;
		}

		if (placeholder !== "" || force === true)
		{
			var setPlaceholder = placeholder !== "" && (me.TextSelectionMode() === true || itemCollectionOrdered.length === 0) && me.GetInputValue() === "" && willAssumeInputValue !== true;

			Fit.Dom.Data(itemContainer, "placeholder", setPlaceholder === true ? placeholder : null);
			Fit.Dom.Data(itemContainer, "placeholder-autoclear", setPlaceholder === true ? Fit.Browser.GetBrowser() === "MSIE" ? "true" : "false" : null);
		}
	}

	function onDragStop(draggable)
	{
		draggable.Reset();
	}

	function onDragging(draggable)
	{
		if (dropZone === null)
			return;

		var pointerPos = Fit.Events.GetPointerState().Coordinates.ViewPort;
		var elmPos = Fit.Dom.GetPosition(dropZone.GetDomElement(), true);

		if (elmPos.X + (dropZone.GetDomElement().offsetWidth / 2) < pointerPos.X) // Drop on right side
		{
			Fit.Dom.Data(dropZone.GetDomElement(), "dropping", "right");
		}
		else // Drop on left side
		{
			Fit.Dom.Data(dropZone.GetDomElement(), "dropping", "left");
		}

		me._internal.Repaint();
	}

	function onDrop(dropzone, draggable)
	{
		var fireChange = false;

		if (Fit.Dom.Data(dropzone.GetDomElement(), "dropping") === "right" && dropzone.GetDomElement() !== draggable.GetDomElement().previousSibling)
		{
			// Always focus control when removing and moving items.
			// First of all it prevents OnFocus and OnBlur from firing
			// a lot of times if multiple elements are removed or moved,
			// secondly it's easier to ensure consistency across browsers
			// as various events are implemented differently, especially
			// when drag and drop comes into place.
			focusAssigned = true;
			focusInput(txtPrimary);

			var draggableIndex = Fit.Dom.GetIndex(draggable.GetDomElement());
			var dropzoneIndex =  Fit.Dom.GetIndex(dropzone.GetDomElement());
			var itemObject = itemCollectionOrdered[draggableIndex];

			Fit.Array.RemoveAt(itemCollectionOrdered, draggableIndex);
			Fit.Array.Insert(itemCollectionOrdered, (draggableIndex < dropzoneIndex ? dropzoneIndex : dropzoneIndex + 1), itemObject);
			Fit.Dom.InsertAfter(dropzone.GetDomElement(), draggable.GetDomElement());

			fireChange = true;
		}
		else if (Fit.Dom.Data(dropzone.GetDomElement(), "dropping") === "left" && dropzone.GetDomElement().previousSibling !== draggable.GetDomElement())
		{
			// See comment in block above regarding focus
			focusAssigned = true;
			focusInput(txtPrimary);

			var draggableIndex = Fit.Dom.GetIndex(draggable.GetDomElement());
			var dropzoneIndex =  Fit.Dom.GetIndex(dropzone.GetDomElement());
			var itemObject = itemCollectionOrdered[draggableIndex];

			Fit.Array.RemoveAt(itemCollectionOrdered, draggableIndex);
			Fit.Array.Insert(itemCollectionOrdered, (draggableIndex < dropzoneIndex ? dropzoneIndex - 1 : dropzoneIndex), itemObject);
			Fit.Dom.InsertBefore(dropzone.GetDomElement(), draggable.GetDomElement());

			fireChange = true;
		}

		Fit.Dom.Data(dropZone.GetDomElement(), "dropping", null);
		dropZone = null;

		me._internal.Repaint();

		if (fireChange === true)
			fireOnChange();
	}

	function onDropZoneEnter(dropzone)
	{
		dropZone = dropzone;
	}

	function onDropZoneLeave(dropzone)
	{
		Fit.Dom.Data(dropzone.GetDomElement(), "dropping", null);
		dropZone = null;
		me._internal.Repaint();
	}

	function decodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/%3B/g, ";").replace(/%3D/g, "=") // Decode characters reserved for value format: title1=value1;title2=value2;etc..
	}

	function encodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/;/g, "%3B").replace(/=/g, "%3D") // Encode characters reserved for value format: title1=value1;title2=value2;etc..
	}

	function decode(str)
	{
		Fit.Validation.ExpectString(str);
		return decodeURIComponent(str);
	}

	function encode(str)
	{
		Fit.Validation.ExpectString(str);
		return encodeURIComponent(str);
	}

	function updateTextSelection()
	{
		// Add selected items to input field

		// Local flag controlling event behaviour - whether to fire OnInputChanged
		// when programmatically updating input with selection text or not.
		// Some developers want OnInputChanged fired always, while others only
		// expect it to fire when the user enters a value, or when SetInputValue(..)
		// is called from external code.
		// True		= Always fire OnInputChanged.
		// False	= Only fire when user enters a value or when SetInputValue(..) is
		//            called. Do not fire OnInputChanged when TextSelection is updated.
		var fireOnInputChangedEvent = false;

		// If fireOnInputChangedEvent is false:
		// Notice that in TextSelectionMode with fireOnInputChangedEvent=false the input field
		// handles two states: The value set by TextSelectionMode, and the value set by
		// either the user or a call to SetInputValue(..).
		// Therefore, even when a text selection is set, the input field will still be
		// considered empty in regards to how OnInputChanged fires.
		// So an actual value has to be entered for OnInputChanged to fire.
		// Clearing the text selection will not fire OnInputChanged since it is already
		// considered empty.

		if (fireOnInputChangedEvent === false && prevValue !== "")
		{
			// User entered a value which will now be removed and replaced.
			// Make sure OnInputChanged handlers are fired - this is the same
			// behaviour as in the ordinary visual selection mode
			// which also clears the input once a selection is made.
			me.ClearInput();
		}

		// Convert selected items to text

		var text = "";

		if (textSelectionCallback !== null)
		{
			text = textSelectionCallback(me);
		}
		else
		{
			var selections = me.GetSelections();

			Fit.Array.ForEach(selections, function(selection)
			{
				text += (text !== "" ? ", " : "") + Fit.String.StripHtml(selection.Title); // Selected items might contain HTML if HtmlAllowed() is true
			});

			if (me.MultiSelectionMode() === true && selections.length > 0)
			{
				text = "(" + selections.length + ") " + text;
			}
		}

		// Set new text selection

		if (fireOnInputChangedEvent === true)
		{
			//me.SetInputValue(text); // Notice: SetInputValue fires OnInputChanged event
			if (txtPrimary.value !== text)
			{
				txtPrimary.value = text;
				prevValue = text;
				fireOnInputChanged(text);
			}
		}
		else
		{
			txtPrimary.value = text;
		}

		prevTextSelection = (text !== "" ? text : null);

		// Set cursor position

		// Make cursor move to end and scroll it into view.
		// Unfortunately it will not make the field scroll to the right on IE
		// and Edge, and it seems very difficult (impossible?) to get working.
		// Other similar controls simply clear their selection when opened, and
		// restore it when closed, which might be the only way to ensure the
		// same experience across all browsers.
		/*if (me.Focused() === true)
		{
			txtPrimary.blur();
			txtPrimary.focus();
		}*/

		// Make cursor move to beginning of input field if focused
		if (Fit.Dom.GetFocused() === txtPrimary)
		{
			Fit.Dom.SetCaretPosition(txtPrimary, 0);
		}

		// Remove placeholder in case it was added using this._internal.ClearInputForSearch()

		updatePlaceholder(true);

		// Have TextSelection removed when input is changed

		clearTextSelectionOnInputChange = true;
	}

	function updateInvalidMessageForSelectedItems()
	{
		Fit.Array.ForEach(itemCollection, function(key)
		{
			var selection = itemCollection[key];

			if (selection.Valid === false)
			{
				Fit.Dom.Attribute(selection.DomElement, "title", invalidMessage);
			}
		});
	}

	function localize()
	{
		if (invalidMessageChanged === false)
		{
			var locale = Fit.Internationalization.GetLocale(me);
			invalidMessage = locale.Translations.InvalidSelection;

			updateInvalidMessageForSelectedItems();
		}
	}

	// Event dispatchers

	function fireOnInputChanged(val)
	{
		Fit.Validation.ExpectString(val);

		updatePlaceholder();

		Fit.Array.ForEach(onInputChangedHandlers, function(handler)
		{
			handler(me, val);
		});
	}

	function fireOnChange()
	{
		updatePlaceholder();
		optimizeDropDownPosition();
		me._internal.FireOnChange();
	}

	function fireOnPaste(val)
	{
		Fit.Validation.ExpectString(val);

		var proceed = true;

		Fit.Array.ForEach(onPasteHandlers, function(handler)
		{
			if (handler(me, val) === false)
				proceed = false; // Do not cancel loop, allow all handlers to fire
		});

		return proceed;
	}

	function fireOnDropDownOpen()
	{
		Fit.Array.ForEach(onOpenHandlers, function(handler)
		{
			handler(me);
		});

		if (picker !== null)
			picker._internal.FireOnShow();
	}

	function fireOnDropDownClose()
	{
		Fit.Array.ForEach(onCloseHandlers, function(handler)
		{
			handler(me);
		});

		if (picker !== null)
			picker._internal.FireOnHide();
	}

	function postpone(cb, timeout)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Validation.ExpectInteger(timeout, true)

		var toId = setTimeout(function()
		{
			if (me !== null) // Make sure control has not been disposed
			{
				cb();
			}
		}, timeout || 0);

		return function() { clearTimeout(toId); };
	}

	init();
}

Fit._internal.DropDown = {};
Fit._internal.DropDown.Current = null;

/*Fit.Controls.DropDown.Item = function()
{
	var title = null;
	var value = null;

	function init()
	{
		if (arguments.length === 1)
		{
			value = arguments[0];
		}
		else if (arguments.length === 2)
		{
			title = arguments[0];
			value = arguments[1];
		}

	}

	this.Title = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
			title = val;

		return title;
	}

	this.Value = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
			value = val;

		return value;
	}

	init();
}*/

/// <container name="Fit.Controls.PickerBase">
/// 	Class from which all Picker Controls extend.
/// 	Control developers must override: GetDomElement, Destroy.
/// 	Overriding the following functions is optional:
/// 	UpdateItemSelectionState, HandleEvent.
/// 	Picker Control must fire OnItemSelectionChanging and OnItemSelectionChanged when an item's
/// 	selection state is being changed, which is done by invoking
/// 	this._internal.FireOnItemSelectionChanging(title:string, value:string, currentSelectionState:boolean, programmaticallyChanged:boolean)
/// 	and
/// 	this._internal.FireOnItemSelectionChanged(title:string, value:string, newSelectionState:boolean, programmaticallyChanged:boolean).
/// 	Notice that FireOnItemSelectionChanging may return False, which must prevent item from being
/// 	selected, and at the same time prevent FireOnItemSelectionChanged from being called.
/// 	Changing an item selection may cause OnItemSelectionChanging and OnItemSelectionChanged to be
/// 	fired multiple times (e.g. if picker needs to first deselect one item before selecting another one).
/// 	Therefore PickerBase also features the OnItemSelectionComplete event which must be fired when related
/// 	changes complete, which is done by invoking this._internal.FireOnItemSelectionComplete().
/// 	OnItemSelectionComplete should only fire if a change was made (changes can be canceled using
/// 	OnItemSelectionChanging).
/// 	Picker control is also to invoke this._internal.FireOnFocusIn() if control gains focus, and
/// 	this._internal.FireOnFocusOut() if control loses focus.
/// </container>
Fit.Controls.PickerBase = function()
{
	// Private variables

	var me = this;

	var onShowHandlers = [];
	var onHideHandlers = [];
	var onChangingHandlers = [];
	var onChangeHandlers = [];
	var onCompleteHandlers = [];
	var onFocusHandlers = [];
	var onBlurHandlers = [];

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.PickerBase" name="MaxHeight" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set max height of control - returns object with Value (number) and Unit (string) properties </description>
	/// 	<param name="value" type="number" default="undefined"> If defined, max height is updated to specified value. A value of -1 forces picker to fit height to content. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="undefined"> If defined, max height is updated to specified CSS unit, otherwise px is assumed </param>
	/// </function>
	this.MaxHeight = function(value, unit)
	{
		Fit.Validation.ExpectNumber(value, true);
		Fit.Validation.ExpectStringValue(unit, true);

		var elm = me.GetDomElement();

		if (Fit.Validation.IsSet(value) === true)
		{
			if (value !== -1)
			{
				elm.style.maxHeight = value + ((Fit.Validation.IsSet(unit) === true) ? unit : "px");
			}
			else
			{
				elm.style.maxHeight = "";
			}
		}

		var res = { Value: -1, Unit: "px" }; // No maxHeight set, height adjusts to content

		if (elm.style.maxHeight !== "") // MaxHeight set
		{
			res.Value = parseFloat(elm.style.maxHeight);
			res.Unit = elm.style.maxHeight.replace(res.Value, "");
		}

		return res;
	}

	// ============================================
	// Events fired by host control
	// ============================================

	/// <function container="Fit.Controls.PickerBaseTypeDefs" name="BaseEventHandler">
	/// 	<description> Event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.PickerBase" name="OnShow" access="public">
	/// 	<description>
	/// 		Register event handler fired when picker control is shown in host control.
	/// 		The following argument is passed to event handler function: Sender (PickerBase).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.BaseEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnShow = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onShowHandlers, cb);
	}

	/// <function container="Fit.Controls.PickerBase" name="OnHide" access="public">
	/// 	<description>
	/// 		Register event handler fired when picker control is hidden in host control.
	/// 		The following argument is passed to event handler function: Sender (PickerBase).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.BaseEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnHide = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onHideHandlers, cb);
	}

	// ============================================
	// Events fired by picker control itself
	// ============================================

	/// <container name="Fit.Controls.PickerBaseTypeDefs.SelectionEventHandlerArguments">
	/// 	<description> Selection event handler arguments </description>
	/// 	<member name="Title" type="string"> Item title </member>
	/// 	<member name="Value" type="string"> Item value </member>
	/// 	<member name="Selected" type="boolean"> Flag indicating whether item is selected or not </member>
	/// </container>

	/// <function container="Fit.Controls.PickerBaseTypeDefs" name="SelectionChangingEventHandler" returns="boolean | void">
	/// 	<description> Selection changing event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.PickerBaseTypeDefs.SelectionEventHandlerArguments"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.PickerBaseTypeDefs" name="SelectionChangedEventHandler">
	/// 	<description> Selection changed event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.PickerBaseTypeDefs.SelectionEventHandlerArguments"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.PickerBase" name="OnItemSelectionChanging" access="public">
	/// 	<description>
	/// 		Register event handler fired when item selection is changing.
	/// 		Selection can be canceled by returning False.
	/// 		The following arguments are passed to event handler function:
	/// 		Sender (PickerBase), EventArgs (containing Title (string), Value (string), and Selected (boolean) properties).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.SelectionChangingEventHandler"> Event handler function </param>
	/// </function>
	this.OnItemSelectionChanging = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onChangingHandlers, cb);
	}

	/// <function container="Fit.Controls.PickerBase" name="OnItemSelectionChanged" access="public">
	/// 	<description>
	/// 		Register event handler fired when item selection is changed.
	/// 		This event may be fired multiple times when a selection is changed, e.g. in Single Selection Mode,
	/// 		where an existing selected item is deselected, followed by selection of new item.
	/// 		The following arguments are passed to event handler function:
	/// 		Sender (PickerBase), EventArgs (containing Title (string), Value (string), and Selected (boolean) properties).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.SelectionChangedEventHandler"> Event handler function </param>
	/// </function>
	this.OnItemSelectionChanged = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onChangeHandlers, cb);
	}

	/// <function container="Fit.Controls.PickerBase" name="OnItemSelectionComplete" access="public">
	/// 	<description> Register event handler invoked when a series of related item changes are completed </description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.BaseEventHandler">
	/// 		Event handler function which accepts Sender (PickerBase)
	/// 	</param>
	/// </function>
	this.OnItemSelectionComplete = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onCompleteHandlers, cb);
	}

	/// <function container="Fit.Controls.PickerBase" name="OnFocusIn" access="public">
	/// 	<description> Register OnFocusIn event handler which is invoked when picker gains focus </description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.BaseEventHandler">
	/// 		Event handler function which accepts Sender (PickerBase)
	/// 	</param>
	/// </function>
	this.OnFocusIn = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onFocusHandlers, cb);
	}

	/// <function container="Fit.Controls.PickerBase" name="OnFocusOut" access="public">
	/// 	<description> Register OnFocusOut event handler which is invoked when picker loses focus </description>
	/// 	<param name="cb" type="Fit.Controls.PickerBaseTypeDefs.BaseEventHandler">
	/// 		Event handler function which accepts Sender (PickerBase)
	/// 	</param>
	/// </function>
	this.OnFocusOut = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onBlurHandlers, cb);
	}

	// ============================================
	// For derivatives - control developers
	// ============================================

	/// <function container="Fit.Controls.PickerBase" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description>
	/// 		Overridden by control developers (required).
	/// 		Get DOMElement representing control.
	/// 	</description>
	/// </function>
	this.GetDomElement = function()
	{
		Fit.Validation.ThrowError("Function not implemented");
	}

	/// <function container="Fit.Controls.PickerBase" name="UpdateItemSelection" access="public">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control invokes this function when an item's selection state is changed from host control.
	/// 		Picker control is responsible for firing FireOnItemSelectionChanging and FireOnItemSelectionChanged,
	/// 		as demonstrated below, if the picker control contains the given item.
	///
	/// 		var item = getItem(value);
	/// 		if (item !== null)
	/// 		{
	/// 			&#160;&#160;&#160;&#160; if (this._internal.FireOnItemSelectionChanging(item.Title, item.Value, item.Selected, programmaticallyChanged) === false)
	/// 			&#160;&#160;&#160;&#160;&#160;&#160;&#160;&#160; return false;
	///
	/// 			&#160;&#160;&#160;&#160; item.SetSelected(selected);
	/// 			&#160;&#160;&#160;&#160; this._internal.FireOnItemSelectionChanged(item.Title, item.Value, item.Selected, programmaticallyChanged);
	/// 		}
	///
	/// 		Both events are fired by passing the given item's title, value, and current selection state.
	/// 		Be aware that host control may pass information about items not found in picker, e.g. when pasting
	/// 		items which may turn out not to be valid selections.
	/// 		Returning False from UpdateItemSelection will cancel the change.
	/// 	</description>
	/// 	<param name="value" type="string"> Item value </param>
	/// 	<param name="selected" type="boolean"> True if item was selected, False if item was deselected </param>
	/// 	<param name="programmaticallyChanged" type="boolean"> True if item was selected programmatically (not by user interaction), False otherwise </param>
	/// </function>
	this.UpdateItemSelection = function(value, selected, programmaticallyChanged) // Actually the current use of programmaticallyChanged is more like "control has focus" - used for performance optimizations
	{
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectBoolean(programmaticallyChanged);

		// Default implementation fires both events, even though specialized control may not know
		// anything about the given item selected/deselected. This is necessary in order to support
		// events out of the box without requiring developer to override function.

		// It's safe to assume that current selection state is equal to !selected since host control will
		// never call UpdateItemSelection with the current value of the given item, only the desired value.
		if (me._internal.FireOnItemSelectionChanging("", value, !selected, programmaticallyChanged) === false)
			return false;

		me._internal.FireOnItemSelectionChanged("", value, selected, programmaticallyChanged);
	}

	/// <container name="Fit.Controls.PickerBaseTypeDefs.Item">
	/// 	<description> Item information </description>
	/// 	<member name="Title" type="string"> Item title </member>
	/// 	<member name="Value" type="string"> Item value </member>
	/// </container>

	/// <function container="Fit.Controls.PickerBase" name="SetSelections" access="public">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control invokes this function when picker is assigned to host control, providing an array
	/// 		of items already selected. An item is an object with a Title (string) and Value (string) property set.
	/// 		If picker defines preselected items, firing OnItemSelectionChanged
	/// 		for these items, will update the host control appropriately.
	/// 	</description>
	/// 	<param name="items" type="Fit.Controls.PickerBaseTypeDefs.Item[]">
	/// 		Array containing selected items: {Title:string, Value:string}
	/// 	</param>
	/// </function>
	this.SetSelections = function(items)
	{
		Fit.Validation.ExpectArray(items);

		Fit.Array.ForEach(items, function(item)
		{
			Fit.Validation.ExpectString(item.Title);
			Fit.Validation.ExpectString(item.Value);
		});
	}

	/// <function container="Fit.Controls.PickerBase" name="GetItemByValue" access="public" returns="Fit.Controls.PickerBaseTypeDefs.Item | null">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control may invoke this function, for instance to update the title of selected items,
	/// 		to make sure these properly reflect the state of data displayed in the picker.
	/// 		Function returns Null when not implemented or when an item is not found. If found, an object
	/// 		with the following signature is returned: { Title: string, Value: string }.
	/// 	</description>
	/// 	<param name="val" type="string"> Value of item to retrieve </param>
	/// </function>
	this.GetItemByValue = function(val)
	{
		Fit.Validation.ExpectString(val);
		return null;
	}

	/// <function container="Fit.Controls.PickerBase" name="RevealItemInView" access="public">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control may invoke this function to reveal a selected item in
	/// 		the picker control. Often this means having the item scrolled into view.
	/// 	</description>
	/// 	<param name="val" type="string"> Value of item to reveal in view </param>
	/// </function>
	this.RevealItemInView = function(val)
	{
		Fit.Validation.ExpectString(val);
	}

	/// <function container="Fit.Controls.PickerBase" name="HandleEvent" access="public">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control dispatches keyboard events to this function to allow
	/// 		picker control to handle keyboard navigation with keys such as
	/// 		arrow up/down/left/right, enter, space, etc.
	/// 		Picker may return False to prevent host control from reacting to given event.
	/// 	</description>
	/// 	<param name="e" type="Event" default="undefined"> Keyboard event to process </param>
	/// </function>
	this.HandleEvent = function(e)
	{
		Fit.Validation.ExpectEvent(e, true);
	}

	/// <function container="Fit.Controls.PickerBase" name="GetHighlighted" access="public" returns="Fit.Controls.PickerBaseTypeDefs.Item | null">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		Host control or external code may invoke this function to obtain the currently
	/// 		highlighted item in the picker control.
	/// 		Function returns Null when not implemented or when no item is highlighted. If found,
	/// 		an object with the following signature is returned: { Title: string, Value: string }.
	/// 	</description>
	/// </function>
	this.GetHighlighted = function()
	{
		return null;
	}

	/// <function container="Fit.Controls.PickerBase" name="PersistView" access="public" returns="boolean">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		This function can be used to tell the picker control to persist (remember) its current state between interactions.
	/// 		For instance a TreeView control would remembers its scroll position and highlighted node, while a calendar would
	/// 		remember the previously selected year and month.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If set, True enables feature, False disables it (default) </param>
	/// </function>
	this.PersistView = function(val)
	{
		return false;
	}

	/// <function container="Fit.Controls.PickerBase" name="HighlightFirst" access="public" returns="boolean">
	/// 	<description>
	/// 		Overridden by control developers (optional).
	/// 		This function can be used to make the picker control automatically highlight the first item.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If set, True enables feature, False disables it (default) </param>
	/// </function>
	this.HighlightFirst = function()
	{
		return false;
	}

	/// <function container="Fit.Controls.PickerBase" name="Destroy" access="public">
	/// 	<description>
	/// 		Overridden by control developers (required).
	/// 		Destroys control to free up memory.
	/// 		Make sure to call Destroy() on PickerBase which can be done like so:
	/// 		this.Destroy = Fit.Core.CreateOverride(this.Destroy, function()
	/// 		{
	/// 			&#160;&#160;&#160;&#160; // Add control specific logic here
	/// 			&#160;&#160;&#160;&#160; base(); // Call Destroy on PickerBase
	/// 		});
	/// 	</description>
	/// </function>
	this.Destroy = function() // Must be overridden - remember to call base !
	{
		me = onShowHandlers = onHideHandlers = onChangingHandlers = onChangeHandlers = onCompleteHandlers = null;
	}

	// ============================================
	// Private
	// ============================================

	// Private members (must be public in order to be accessible to host control and controls extending from PickerBase)

	this._internal = (this._internal ? this._internal : {});

	this._internal.InitializePicker = function() // Called by Host Control when picker is assigned to it
	{
	}

	this._internal.ReportFocused = function(value) // Called by Host Control to tell picker when it is focused/blurred
	{
	}

	this._internal.FireOnShow = function() // Called by Host Control
	{
		Fit.Array.ForEach(onShowHandlers, function(handler)
		{
			handler(me);
		});
	},

	this._internal.FireOnHide = function() // Called by Host Control
	{
		Fit.Array.ForEach(onHideHandlers, function(handler)
		{
			handler(me);
		});
	},

	this._internal.FireOnItemSelectionChanging = function(title, value, selected, programmaticallyChanged) // Called by Picker Control
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectBoolean(programmaticallyChanged);

		var cancel = false;

		Fit.Array.ForEach(onChangingHandlers, function(handler)
		{
			if (handler(me, { Title: title, Value: value, Selected: selected, ProgrammaticallyChanged: programmaticallyChanged }) === false)
				cancel = true;
		});

		return !cancel;
	}

	this._internal.FireOnItemSelectionChanged = function(title, value, selected, programmaticallyChanged) // Called by Picker Control
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectBoolean(programmaticallyChanged);

		Fit.Array.ForEach(onChangeHandlers, function(handler)
		{
			handler(me, { Title: title, Value: value, Selected: selected, ProgrammaticallyChanged: programmaticallyChanged });
		});
	}

	this._internal.FireOnItemSelectionComplete = function() // Called by Picker Control
	{
		Fit.Array.ForEach(onCompleteHandlers, function(handler)
		{
			handler(me);
		});
	}

	this._internal.FireOnFocusIn = function() // Called by Picker Control
	{
		Fit.Array.ForEach(onFocusHandlers, function(handler)
		{
			handler(me);
		});
	}

	this._internal.FireOnFocusOut = function() // Called by Picker Control
	{
		Fit.Array.ForEach(onBlurHandlers, function(handler)
		{
			handler(me);
		});
	}
}

/// <container name="Fit.Controls.WSDropDown" extends="Fit.Controls.DropDown">
/// 	WebService enabled Drop Down Menu control allowing for single and multi selection.
/// 	Supports data selection using any control extending from Fit.Controls.PickerBase.
/// 	This control is extending from Fit.Controls.DropDown.
/// </container>

/// <function container="Fit.Controls.WSDropDown" name="WSDropDown" access="public">
/// 	<description> Create instance of WSDropDown control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.WSDropDown = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.DropDown).Apply(ctlId);

	var me = this;
	var list = null;
	var tree = null;
	var actionMenu = null;

	var search = "";
	var forceNewSearch = false;
	var hideLinesForFlatData = true;
	var dataRequested = false;		// Flag indicating whether TreeView data has been requested or not - determines whether a call to ensureTreeViewData() actually loads data or not
	var dataLoading = false;		// Flag indicating whether TreeView data is currently being loaded by WSDropDown internals (awaiting response) - will not be True when user expand nodes to load children, or when invoking e.g. dd.GetTreeView.Reload()
	//var nodesPopulated = false;	// Flag indicating whether TreeView root nodes have been populated - contrary to dataLoading this flag is set when a potentially partial portion of the data has been loaded
	var requestCount = 0;			// Counter to keep track of nodes for which data is currently being loaded, no matter how it was being loaded (via WSDropDown internals, programmatically on WSTreeView from external code, or by user expanding nodes)
	var onDataLoadedCallback = [];
	var suppressTreeOnOpen = false;
	var timeOut = null;
	var currentRequest = null;
	var classes = null;
	var autoUpdatedSelections = null; // Cached result from AutoUpdateSelected: [{ Title:string, Value:string, Exists:boolean }, ...]
	var useActionMenu = false;
	var useActionMenuForced = false;
	var useActionMenuAfterLoad = true;
	var useActionMenuSortingLocale = null;
	var useActionMenuHtmlAllowed = false;
	var treeViewEnabled = true;
	var orgPlaceholder = this.Placeholder;
	var customPlaceholderSet = false;
	var translations = null;

	var onRequestHandlers = [];
	var onResponseHandlers = [];
	var onAbortHandlers = [];
	//var onPopulatedHandlers = [];

	function init()
	{
		// Initialize self (drop down)

		me.OnInputChanged(function(sender, value)
		{
			if (me.Focused() === true) // Prevent search from being triggered when OnInputChanged fires after programmatically changing input value
				searchData(value);
		});

		// Get drop down open/close button element (arrow)

		var cmdOpen = me.GetDomElement().firstChild.children[me.GetDomElement().firstChild.children.length - 1];
		classes = cmdOpen.className;

		// Create ListView

		list = new Fit.Controls.WSListView(me.GetId() + "__WSListView");
		list.OnRequest(function(sender, eventArgs)
		{
			if (fireEventHandlers(onRequestHandlers, list, eventArgs) === false)
				return false;

			requestCount++;
			currentRequest = eventArgs.Request;
			cmdOpen.className = "fa fa-refresh fa-spin";
		});
		list.OnResponse(function(sender, eventArgs)
		{
			requestCount--;

			fireEventHandlers(onResponseHandlers, list, eventArgs);

			cmdOpen.className = classes;
			currentRequest = null;

			if (forceNewSearch === true)
			{
				forceNewSearch = false;

				eventArgs.Items = []; // Remove data, we do not want it to be populated to control
				searchData(me.GetInputValue());
			}
		});
		list.OnAbort(function(sender, eventArgs)
		{
			requestCount--;
		});
		list.OnItemSelectionChanging(function(sender, item)
		{
			// Prevent user from deselecting node which is read only in tree (not selectable)

			var node = tree.GetChild(item.Value, true);

			if (node !== null && node.Selectable() === false)
				return false;
		});
		list.OnItemSelectionChanged(function(sender, item)
		{
			// DropDown (base) automatically closes drop down in Single Selection Mode, but we
			// always want it to close drop down when using ListView to select data from a search query.
			// ListView is only used to select data from search results - TreeView is used to display
			// all data, no matter if it is received in a flat list or a hierarchy.
			if (item.Selected === true) // Only close when a selection is made - removing an item keeps drop down open
				me.CloseDropDown();
		});

		// Create TreeView

		tree = new Fit.Controls.WSTreeView(me.GetId() + "__WSTreeView");
		tree.Selectable(true); // Make nodes selectable by default when added
		tree.Width(100, "%");
		//tree.Lines(true); // DISABLED - lines do not scale with font size
		//tree.AllowDeselect(true); // DISABLED - AllowDeselect is True by default
		tree.OnRequest(function(sender, eventArgs)
		{
			if (fireEventHandlers(onRequestHandlers, tree, eventArgs) === false)
			{
				return false;
			}

			dataRequested = true;
			requestCount++;

			if (dataLoading === true)
			{
				cmdOpen.className = "fa fa-refresh fa-spin";
			}
		});
		tree.OnResponse(function(sender, eventArgs)
		{
			requestCount--;

			fireEventHandlers(onResponseHandlers, tree, eventArgs);

			if (requestCount === 0)
			{
				cmdOpen.className = classes;

				// Make sure to fire internal OnDataLoaded event in case a WebService operation
				// was invoked on DropDown while WSTreeView was loading data, e.g. triggered by
				// a user interaction such as expanding a node.
				// DropDown takes care of firing internal event OnDataLoaded if dataLoading is True,
				// in which case DropDown triggered the initial WebService operation and knows when
				// it has finished.
				if (dataLoading === false)
				{
					fireOnDataLoaded();
				}
			}
		});
		tree.OnAbort(function(sender, eventArgs)
		{
			requestCount--;

			fireEventHandlers(onAbortHandlers, tree, eventArgs);

			if (requestCount === 0)
			{
				cmdOpen.className = classes;

				if (dataLoading === false) // See comment to related code in OnResponse handler above
				{
					fireOnDataLoaded();
				}
			}
		});
		tree.OnPopulated(function(sender, eventArgs)
		{
			//nodesPopulated = true;

			// Helper lines

			if (hideLinesForFlatData === true && tree.Lines() === true) // Lines are off by default but might have been enabled like so: dd.GetTreeView().Lines(true)
			{
				// Disable helper lines if no children are contained

				var hasChildren = false;

				Fit.Array.ForEach(tree.GetChildren(), function(c)
				{
					if (c.GetChildren().length > 0)
					{
						hasChildren = true;
						return false;
					}
				});

				if (hasChildren === false)
				{
					tree.Lines(false);
					//tree.GetDomElement().style.marginLeft = "-2em";
				}

				hideLinesForFlatData = false;
			}

			// Scroll selected item into view in Single Selection Mode.
			// NOTICE: DropDown also calls RevealItemInView(..) when
			// dropdown is opened. But for WSDropDown there are (most often)
			// no nodes initially, so we make sure to call RevealItemInView(..)
			// below when root nodes have been loaded and populated.
			// We only want to do this for root nodes - otherwise the selected
			// item would constantly be scrolled into view while user expand nodes
			// with remote children.
			// This works even for nested items if they are returned in the initial
			// call for root nodes. It will not work if nodes are loaded programmatically
			// using e.g. WSDropDown.AutoUpdateSelected() or WSTreeView.ExpandAll() though,
			// if the selected item is returned in a secondary request for remote children.
			// To support this scenario we would need to carry information about whether
			// the request was made programmatically or triggered via a user interaction,
			// which would complicate things, so for now we are satisfied with the current
			// implementation.

			if (eventArgs.Node === null && me.MultiSelectionMode() === false && me.GetSelections().length === 1)
			{
				tree.RevealItemInView(me.GetSelections()[0].Value);
			}
		});
		tree.OnSelectAll(function(sender, eventArgs)
		{
			// Make sure focus is lost when SelectAll is invoked. Otherwise control will
			// reassign focus every time an item is added which is very expensive performance wise.
			// DISABLED: This is simply to annoying for the user as dropdown closes when Focused(false) is called.
			// Also, due to the implementation of DropDown.Focused(..), it only closes when control has focus
			// which it won't have if SelectAll is triggered using e.g. a ContextMenu, since the ContextMenu becomes
			// focused when clicked. So the inconsistency is also annoying. Furthermore automatically closing or
			// changing focus takes away control from the developer implementing the SelectAll behaviour.
			//me.Focused(false);

			// NOTICE: Selecting thousands of nodes using SelectAll may result in very poor performance
			// due to the large amount of DOM manipulation. Both the TreeView and DropDown goes through a full
			// state change for every single item being selected or deselected, so all events fire and reflows are
			// triggered for visible items.
			// To increase performance, temporarily hide TreeView and/or DropDown with display:none while SelectAll
			// is being performed. Switching to TextSelectionMode may also increase performance significantly.

			// Make sure TreeView is the active picker to have changes synchronized with
			// drop down control (in case SelectAll is triggered programmatically).
			if (me.GetPicker() !== tree)
				me.SetPicker(tree);
		});

		// Make sure synchronization events are wired immedately. These events ensure that changes to selected
		// items, done through the picker controls programmatically, are synchronized back to the DropDown control.
		me.SetPicker(list);
		me.SetPicker(tree);
		me.SetPicker(null); // Make sure no picker remains rooted as some controls (WSTreeView) loads data when rooted (using synthetic #rooted event)

		// Keep both pickers up to date when selections are changed. Otherwise only
		// the active picker receives changes immediately. WSTreeView.Reload(keepState = true)
		// is dependant upon selection state being up to date.
		// Imagine a scenario where WSTreeView.Reload(true) is called every time a selection
		// is changed in the DropDown control. If an item is removed while the WSListView picker
		// is active, WSTreeView will reload via OnChange, but since WSTreeView.Reload(..) is called
		// with keepState=true, it will restore the selection we just removed while WSListView was active.
		// The code below resolves this by ensuring all pickers are always current on selection state.

		var updateItemSelectionTree = tree.UpdateItemSelection;
		var updateItemSelectionList = list.UpdateItemSelection;

		var updateItemSelectionOverride = function(value, selected, programmaticallyChanged)
		{
			updateItemSelectionTree(value, selected, programmaticallyChanged);
			updateItemSelectionList(value, selected, programmaticallyChanged);
		};

		// UpdateItemSelection(..) is called by host control to update the picker's selections
		tree.UpdateItemSelection = updateItemSelectionOverride;
		list.UpdateItemSelection = updateItemSelectionOverride;

		// Create action menu

		var skipUpdateActionMenuOnChange = false;

		actionMenu = new Fit.Controls.ListView(me.GetId() + "__ActionsListView");
		actionMenu.HtmlAllowed(true);
		actionMenu.OnSelect(function(sender, item) // Using OnSelect instead of OnItemSelectionChanging since DropDown fires OnItemSelectionChanging when selection is changed, which would result in OnItemSelectionChanging being executed multiple times
		{
			if (item.Value === "SearchMore")
			{
				me._internal.ClearInputForSearch();
			}
			else if (item.Value === "ShowAll")
			{
				me._internal.UndoClearInputForSearch(); // In case user first picked SearchMore, entered a search value, changed their mind, and then selected ShowAll

				useActionMenuAfterLoad = false;

				me.SetPicker(tree);
				ensureTreeViewData();
			}
			else if (item.Value === "RemoveAll")
			{
				skipUpdateActionMenuOnChange = true;
				me.ClearSelections(); // Fires OnChange
				skipUpdateActionMenuOnChange = false;

				updateActionMenu(); // Update action menu (remove all "Remove: xyz" options)
			}
			else if (item.Value.indexOf("Remove:") === 0)
			{
				var dataValue = item.Value.replace("Remove:", "");

				// Find item below if using keyboard to remove item

				var above = null;
				var below = null;

				if (Fit.Browser.GetInfo().IsMobile === false && Fit.Events.GetPointerState().Buttons.Primary === false) // Do not find and highlight item below on touch devices, or if using the mouse
				{
					var items = actionMenu.GetItems();
					var stopNext = false;

					for (var i = 0 ; i < items.length ; i++)
					{
						if (items[i].Value.indexOf("Remove:") !== 0)
						{
							continue; // Skip items that do not remove individual selections
						}

						if (items[i].Value.replace("Remove:", "") === dataValue)
						{
							stopNext = true;
							continue;
						}

						if (stopNext === false)
						{
							above = items[i].Value;
						}
						else
						{
							below = items[i].Value;
							break;
						}
					}
				}

				// Remove item

				skipUpdateActionMenuOnChange = true;
				me.RemoveSelection(dataValue); // Fires OnChange
				skipUpdateActionMenuOnChange = false;

				// Update list of removable items

				actionMenu.RemoveItem(item.Value);

				if (me.GetSelections().length === 1)
				{
					updateActionMenu(); // Update action menu to get rid of "Remove all" item when only one item is left
				}

				// Highlight item below (or above) the item that was just removed

				if (above !== null || below !== null)
				{
					actionMenu.RevealItemInView(below !== null ? below : above);
				}
			}

			return false; // Prevent selection of behavioural item
		});

		me.OnChange(function(sender)
		{
			if (skipUpdateActionMenuOnChange === false)
			{
				updateActionMenu();
			}
		});

		// Misc

		me.OnOpen(function()
		{
			if (suppressTreeOnOpen === true)
			{
				suppressTreeOnOpen = false;
				return;
			}

			// Do not show action menu if the only option available is ShowAll.
			// In this case the user will not be able to select SeachMore, and
			// there is no selected items that can be removed from the control.
			// In this case we ignore useActionMenu === true, even when useActionMenuForced is true.
			var onlyTheShowAllOptionIsDisplayedInActionMenu = actionMenu.GetItems().length === 1 && actionMenu.HasItem("ShowAll") === true;

			if (useActionMenu === true && onlyTheShowAllOptionIsDisplayedInActionMenu === false)
			{
				me.SetPicker(actionMenu);
			}
			else
			{
				if (onlyTheShowAllOptionIsDisplayedInActionMenu === true)
				{
					useActionMenuAfterLoad = false;
				}

				if (treeViewEnabled === true)
				{
					me.SetPicker(tree);
					ensureTreeViewData();
				}
				else
				{
					list.RemoveItems(); // Do not show previous search results again
				}
			}
		});

		Fit.Internationalization.OnLocaleChanged(localize);
		localize();
	}

	// ============================================
	// Public
	// ============================================

	this.AddSelection = Fit.Core.CreateOverride(this.AddSelection, function(title, value, valid)
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(valid, true);

		base(title, value, valid);

		// Keep TreeView up to date with selection state

		if (me.GetPicker() !== tree) // DropDown (from which WSDropDown inherits) automatically updates the picker's selection state, but it can only do so for the active picker of course - make sure TreeView's selection state is kept up to date
		{
			// Selection just added might originate from a call to WSDropDown.Value(..), so the node might not have been loaded yet,
			// which is why we can't expect it to be available via tree.GetChild(value, true). Therefore we use tree.SetNodeSelection(..) to
			// update selection state since this will add the selection regardless of the node existing or not (using preselections).

			tree.SetNodeSelection(value, true);
		}
	});

	this.RemoveSelection = Fit.Core.CreateOverride(this.RemoveSelection, function(value)
	{
		Fit.Validation.ExpectString(value);

		base(value);

		// Keep TreeView up to date with selection state

		if (me.GetPicker() !== tree) // DropDown (from which WSDropDown inherits) automatically updates the picker's selection state, but it can only do so for the active picker of course - make sure TreeView's selection state is kept up to date
		{
			// Selection just remove might originate from a call to WSDropDown.Value(..), so the node might not have been loaded yet,
			// which is why we can't expect it to be available via tree.GetChild(value, true). Therefore we use tree.SetNodeSelection(..) to
			// update selection state since this will remove the selection regardless of the node existing or not (using preselections).

			tree.SetNodeSelection(value, false);
		}
	});

	this.Value = Fit.Core.CreateOverride(this.Value, function(value)
	{
		Fit.Validation.ExpectString(value, true);

		var val = base(value);

		// Keep TreeView up to date with selection state

		if (Fit.Validation.IsSet(value) === true && me.GetPicker() !== tree) // DropDown (from which WSDropDown inherits) automatically updates the picker's selection state, but it can only do so for the active picker of course - make sure TreeView's selection state is kept up to date
		{
			// Nodes just set might not have been loaded yet, so we can't expect them to be available via tree.GetChild(nodeValue, true).
			// Therefore we use tree.SetSelections(..) to update selection state since this will set the selections regardless of the
			// nodes existing or not (using preselections).

			tree.SetSelections(me.GetSelections());
		}

		return val;
	});

	this.ClearSelections = Fit.Core.CreateOverride(this.ClearSelections, function()
	{
		base();

		// Keep TreeView up to date with selection state

		if (me.GetPicker() !== tree) // DropDown (from which WSDropDown inherits) automatically updates the picker's selection state, but it can only do so for the active picker of course - make sure TreeView's selection state is kept up to date
		{
			tree.Clear();
		}
	});

	/// <function container="Fit.Controls.WSDropDownTypeDefs" name="AutoUpdateSelectedCallback">
	/// 	<description> AutoUpdateSelected callback </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="updatedItems" type="Fit.Controls.DropDownTypeDefs.UpdatedDropDownItem[]"> Updated items </param>
	/// </function>

	/// <function container="Fit.Controls.WSDropDown" name="AutoUpdateSelected" access="public">
	/// 	<description>
	/// 		Automatically update title of selected items based on data from WebService.
	/// 		Contrary to UpdateSelected(), AutoUpdateSelected() automatically loads all
	/// 		data from the associated WebService before updating the selected items, but
	/// 		only if one or more items are selected.
	/// 		The callback function is invoked when selected items have been updated.
	/// 		The following arguments are passed to function:
	/// 		 - Sender (WSDropDown)
	/// 		 - An array of updated items, each with a Title (string), Value (string), and Exists (boolean) property.
	/// 		Notice that items that no longer exists in picker's data, will NOT automatically be removed.
	/// 		To obtain all items with the most current state (both updated and unmodified selections), use;
	/// 		dropdown.AutoUpdateSelected(function(sender, updated) { console.log(&quot;All selected&quot;, dropdown.GetSelections()); });
	/// 		For additiona details see UpdateSelected().
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.AutoUpdateSelectedCallback" default="undefined">
	/// 		Optional callback function invoked when selected items have been updated
	/// 	</param>
	/// </function>
	this.AutoUpdateSelected = function(cb)
	{
		Fit.Validation.ExpectFunction(cb, true);

		if (me.GetSelections().length === 0) // Do not request data if no selections are made to be updated
		{
			if (Fit.Validation.IsSet(cb) === true)
			{
				cb(me, []);
			}

			return;
		}

		if (requestCount > 0)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.AutoUpdateSelected(cb); });
			return;
		}

		if (autoUpdatedSelections !== null)
		{
			if (Fit.Validation.IsSet(cb) === true)
			{
				cb(me, Fit.Core.Clone(autoUpdatedSelections)); // Clone to prevent changes to internal array and its data
			}

			return;
		}

		dataLoading = true;
		//nodesPopulated = false;

		var ensure = function()
		{
			tree.EnsureData(function(sender)
			{
				// Ensure all nodes in case only a subset is returned
				// when requesting root nodes via tree.Reload(..)

				// Picker must be set when calling UpdateSelected() on
				// DropDown from which WSDropDown inherits. Selected items
				// are updated based on data loaded by the picker control.
				me.SetPicker(tree);

				autoUpdatedSelections = me.UpdateSelected();

				dataLoading = false;

				// Switch to action menu if no data was received. Must
				// be called after dataLoading is set since function
				// calls updateActionMenu(..) which relies on this flag.
				showActionMenuIfNoDataReceivedOrOnFirstInteractionIfEnabled();

				if (Fit.Validation.IsSet(cb) === true)
				{
					cb(me, Fit.Core.Clone(autoUpdatedSelections)); // Clone to prevent changes to internal array and its data
				}

				fireOnDataLoaded();
			});
		}

		if (dataRequested === false)
		{
			// Notice that dataRequested may have been set to False in ClearData(..), which is why
			// we first call Reload to fetch initial data, since it also discards any existing nodes.

			tree.Reload(true, function(sender)
			{
				ensure(); // Get remaining nodes in case WebService did not return the entire hierarchy at once (Progressive Mode)
			});
		}
		else
		{
			// Some data has already been loaded - make sure we get the rest
			ensure();
		}
	}

	/// <function container="Fit.Controls.WSDropDown" name="Url" access="public" returns="string">
	/// 	<description>
	/// 		Get/set URL to WebService responsible for providing data to drop down.
	/// 		WebService must deliver data in the following JSON format:
	/// 		[
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 1", Value: "1001", Selectable: true, Selected: true, Children: [] },
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 2", Value: "1002", Selectable: false, Selected: false, Children: [] }
	/// 		]
	/// 		Only Value is required. Children is a collection of nodes with the same format as described above.
	/// 		HasChildren:boolean may be set to indicate that children are available server side and that WebService
	/// 		should be called to load these children when the given node is expanded.
	/// 	</description>
	/// 	<param name="wsUrl" type="string" default="undefined"> If defined, updates WebService URL (e.g. http://server/ws/data.asxm/GetData) </param>
	/// </function>
	this.Url = function(wsUrl)
	{
		Fit.Validation.ExpectString(wsUrl, true);

		if (Fit.Validation.IsSet(wsUrl) === true)
		{
			list.Url(wsUrl);
			tree.Url(wsUrl);
		}

		return list.Url();
	}

	/// <function container="Fit.Controls.WSDropDown" name="JsonpCallback" access="public" returns="string | null">
	/// 	<description>
	/// 		Get/set name of JSONP callback argument. Assigning a value will enable JSONP communication.
	/// 		Often this argument is simply &quot;callback&quot;. Passing Null disables JSONP communication again.
	/// 	</description>
	/// 	<param name="val" type="string | null" default="undefined"> If defined, enables JSONP and updates JSONP callback argument </param>
	/// </function>
	this.JsonpCallback = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true || val === null)
		{
			list.JsonpCallback(val);
			tree.JsonpCallback(val);
		}

		return list.JsonpCallback();
	}

	/// <function container="Fit.Controls.WSDropDownTypeDefs" name="DataCallback">
	/// 	<description> Event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.WSDropDown" name="ClearData" access="public">
	/// 	<description>
	/// 		Call this function to make control reload data when needed,
	/// 		ensuring that the user will see the most recent values available.
	/// 		Operation may be postponed if data is currently loading from WebService.
	/// 		Use callback to pick up execution once data has been cleared.
	/// 		Sender (Fit.Controls.WSDropDown) is passed to callback as an argument.
	/// 		To make sure data reloads immediate, please use ReloadData(..) instead.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.DataCallback" default="undefined">
	/// 		If defined, callback is invoked when data is cleared
	/// 	</param>
	/// </function>
	this.ClearData = function(cb)
	{
		clearData(true, true, cb);
	}

	/// <function container="Fit.Controls.WSDropDown" name="ReloadData" access="public">
	/// 	<description>
	/// 		Call this function to make control reload data immediately,
	/// 		ensuring that the user will see the most recent values available.
	/// 		Use callback to pick up execution once data has been loaded.
	/// 		Sender (Fit.Controls.WSDropDown) is passed to callback as an argument.
	/// 		To have data reload when needed (lazy loading), please use ClearData(..) instead.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.DataCallback" default="undefined">
	/// 		If defined, callback is invoked when data has been loaded
	/// 	</param>
	/// </function>
	this.ReloadData = function(cb)
	{
		clearData(false, false, function() // false 1 = prevent clearData(..) from reloading data, false 2 = prevent action menu from updating to avoid flickering - done when data has been reloaded
		{
			ensureTreeViewData(function()
			{
				updateActionMenu();
				cb && cb(me);
			});
		});
	}

	this.MultiSelectionMode = Fit.Core.CreateOverride(this.MultiSelectionMode, function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			tree.Selectable(true, val);
		}

		return base(val);
	});

	this.HtmlAllowed = Fit.Core.CreateOverride(this.HtmlAllowed, function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			tree.HtmlAllowed(val);
			list.HtmlAllowed(val);
			me.ActionMenuHtmlAllowed(val);
			base(val);
		}

		return base();
	});

	this.InputEnabled = Fit.Core.CreateOverride(this.InputEnabled, function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && base() !== val)
		{
			base(val);
			localize(); // Add/remove placeholder depending on whether input is enabled or not - also calls updateActionMenu() which will make sure SearchMore action is added/removed depending on whether input is enabled or not

			if (val === false && me.GetPicker() === list)
			{
				me.SetPicker(null); // Which picker to use is decided in the OnOpen handler
			}
		}

		return base();
	});

	/// <function container="Fit.Controls.WSDropDown" name="TreeViewEnabled" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether TreeView control is enabled or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables TreeView (default), False disables it </param>
	/// </function>
	this.TreeViewEnabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== treeViewEnabled)
		{
			treeViewEnabled = val;

			if (useActionMenuForced === false)
			{
				// Use action menu if there is no data to display, which makes it impossible to remove
				// selected items in TextSelectionMode - unless DropDown.SelectionModeToggle is enabled,
				// which lets the user switch to Visual Selection Mode with each item displaying a delete button.
				// But we want a consistent behaviour, so we use the action menu for both Text and Visual Selection Mode.

				if (val === true)
				{
					var isWaitingForData = dataRequested === false || dataLoading === true;
					useActionMenu = isWaitingForData === true || tree.GetChildren().length === 0;
				}
				else
				{
					useActionMenu = true; // User must search to retrieve available options
				}
			}

			if (val === false && me.GetPicker() === tree)
			{
				me.SetPicker(null); // Which picker to use is decided in the OnOpen handler
			}

			updateActionMenu(); // Update action menu to have ShowAll action added/removed depending on whether TreeView is enabled or not
		}

		return treeViewEnabled;
	}

	/// <function container="Fit.Controls.WSDropDown" name="ListViewEnabled" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set flag indicating whether searchable ListView is enabled or not.
	/// 		The value provided also determines the value for InputEnabled and vice versa.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables ListView and search capability (default), False disables it </param>
	/// </function>
	this.ListViewEnabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);
		return me.InputEnabled(val);
	}

	this.SearchModeOnFocus = Fit.Core.CreateOverride(this.SearchModeOnFocus, function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== base())
		{
			base(val);
			updateActionMenu(); // Add/remove search option depending on whether SearchModeOnFocus is enabled or not
		}

		return base();
	});

	this.Placeholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		customPlaceholderSet = true;
		return orgPlaceholder(val);
	}

	this.OpenDropDown = Fit.Core.CreateOverride(this.OpenDropDown, function()
	{
		if (me.InputEnabled() === true && me.GetInputValue() === "" && me.GetSelections().length === 0 && treeViewEnabled === false)
		{
			// Do not open DropDown - it will only contain "Search for more options"
			// when no items are currently selected and TreeView is disabled, and no
			// search value has been entered yet.
			// The control will display "Search.." (or a custom placeholder), making it
			// obvious what the user need to do to get data - no need to display the action menu.

			if (Fit.Browser.GetInfo().IsMobile === true)
			{
				// Focus input on mobile, even if DropDown was opened using
				// the arrow icon - this will bring up the virtual keyboard.
				me._internal.ForceFocusMobile();
			}

			return;
		}

		base();
	});

	/// <function container="Fit.Controls.WSDropDown" name="GetListView" access="public" returns="Fit.Controls.WSListView">
	/// 	<description> Get WSListView control used to display data in a flat list view </description>
	/// </function>
	this.GetListView = function()
	{
		return list;
	}

	/// <function container="Fit.Controls.WSDropDown" name="GetTreeView" access="public" returns="Fit.Controls.WSTreeView">
	/// 	<description> Get WSTreeView control used to display data in a hierarchical tree view </description>
	/// </function>
	this.GetTreeView = function()
	{
		return tree;
	}

	/// <function container="Fit.Controls.WSDropDown" name="UseActionMenu" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control uses the built-in action menu to ease addition and removal of items.
	/// 		If this property is not explicitly set, it will automatically be changed by the control depending on data and other settings.
	/// 		The action menu will be enabled if TreeViewEnabled is set to False, as it would otherwise not show anything unless the user
	/// 		enters a search value. If TreeViewEnabled is True but no data is provided to the TreeView control upon request, the action menu
	/// 		is also enabled.
	/// 		If the control does not have any selections, InputEnabled (or its alias ListViewEnabled) is True, and TreeViewEnabled is False,
	/// 		no picker will be displayed since the action menu would only display the &quot;Search for options&quot; item - but it should already
	/// 		be obvious to the user that searching is required due to the placeholder displaying &quot;Search..&quot; by default.
	/// 		Likewise, if TreeViewEnabled is True and InputEnabled (or its alias ListViewEnabled) is False, and no selections are made,
	/// 		the action menu would only display &quot;Show available options&quot;. In this case the TreeView will be displayed instead,
	/// 		even if UseActionMenu has explicitely been set to True.
	/// 		The behaviour described is in place to make sure the action menu is only displayed when it makes sense, since it introduces
	/// 		and extra step (click) required by the user to access data.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables the action menu, False disables it </param>
	/// </function>
	this.UseActionMenu = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			useActionMenuForced = true;

			if (val !== useActionMenu)
			{
				useActionMenu = val;

				if (val === true)
				{
					updateActionMenu();

					if (me.IsDropDownOpen() === true)
					{
						me.SetPicker(actionMenu);
					}
				}
				else
				{
					if (me.GetPicker() === actionMenu)
					{
						me.SetPicker(tree.GetChildren().length > 0 ? tree : list);
					}
				}
			}
		}

		return useActionMenu;
	}

	/// <function container="Fit.Controls.WSDropDown" name="ActionMenuSortingLocale" access="public" returns="string | null">
	/// 	<description>
	/// 		Get/set locale used to sort action menu alphabetically - returns null (default) if not enabled.
	/// 		Localized sorting is not supported on legacy browsers, which will fall back to sorting based on
	/// 		each character's position in the computer's character table.
	/// 	</description>
	/// 	<param name="val" type="string | null" default="undefined"> If defined, sorting is updated to use provided locale - null disables sorting </param>
	/// </function>
	this.ActionMenuSortingLocale = function(val)
	{
		Fit.Validation.ExpectString(val !== null ? val : "", true);

		if (Fit.Validation.IsSet(val) === true || val === null)
		{
			useActionMenuSortingLocale = val;
			updateActionMenu();
		}

		return useActionMenuSortingLocale;
	}

	/// <function container="Fit.Controls.WSDropDown" name="ActionMenuHtmlAllowed" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether HTML is allowed (shown) in selected items in action menu </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables support for HTML, False disables it </param>
	/// </function>
	this.ActionMenuHtmlAllowed = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== useActionMenuHtmlAllowed)
		{
			useActionMenuHtmlAllowed = val;
			updateActionMenu();
		}

		return useActionMenuHtmlAllowed;
	}

	/// <function container="Fit.Controls.WSDropDown" name="ResetActionMenu" access="public">
	/// 	<description>
	/// 		Reset action menu so it automatically determines whether to show up or not
	/// 		when DropDown control is opened/re-opened, based on rules outlined in the
	/// 		description for UseActionMenu(..).
	/// 		This is useful if calling ClearData(..) and one wants to make sure the TreeView
	/// 		data is immediately made visible once ready, rather than showing the action menu
	/// 		if it was previously shown.
	/// 	</description>
	/// </function>
	this.ResetActionMenu = function()
	{
		useActionMenuForced = false;
		useActionMenu = false;
		useActionMenuAfterLoad = true;
	}

	/// <function container="Fit.Controls.WSDropDown" name="UpdateActionMenu" access="public">
	/// 	<description>
	/// 		Update action menu to make it reflect availablity of data in TreeView
	/// 	</description>
	/// </function>
	this.UpdateActionMenu = function() // Comes in handy if TreeView data is reloaded/manipulated (accessible via GetTreeView()) which DropDown won't know about
	{
		updateActionMenu();
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		me.SetPicker(null);

		list.Destroy();
		tree.Destroy();
		actionMenu.Destroy();

		cancelSearch();

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		me = list = tree = actionMenu = search = forceNewSearch = hideLinesForFlatData = dataRequested = dataLoading /*= nodesPopulated*/ = requestCount = onDataLoadedCallback = suppressTreeOnOpen = timeOut = currentRequest = classes = autoUpdatedSelections = useActionMenu = useActionMenuForced = useActionMenuAfterLoad = useActionMenuSortingLocale = useActionMenuHtmlAllowed = treeViewEnabled = orgPlaceholder = customPlaceholderSet = translations = onRequestHandlers = onResponseHandlers = null;

		base();
	});

	// ============================================
	// Events
	// ============================================

	/// <container name="Fit.Controls.WSDropDownTypeDefs.RequestEventArgs">
	/// 	<description> Request event arguments </description>
	/// 	<member name="Sender" type="$TypeOfThis"> Instance of control </member>
	/// 	<member name="Picker" type="Fit.Controls.WSTreeView | Fit.Controls.WSListView"> Instance of picker control causing web service request </member>
	/// 	<member name="Node" type="Fit.Controls.TreeViewNode | null"> Instance of TreeViewNode for which chilren are being requested, Null if root nodes are being requested, or if WSListView triggered request </member>
	/// 	<member name="Search" type="string"> Search value if entered by user </member>
	/// 	<member name="Request" type="Fit.Http.JsonpRequest | Fit.Http.JsonRequest"> Instance of JsonpRequest or JsonRequest </member>
	/// </container>

	/// <container name="Fit.Controls.WSDropDownTypeDefs.ResponseEventArgs" extends="Fit.Controls.WSDropDownTypeDefs.RequestEventArgs">
	/// 	<description> Response event arguments </description>
	/// 	<member name="Data" type="Fit.Controls.WSListViewTypeDefs.JsonItem[] | Fit.Controls.WSTreeViewTypeDefs.JsonItem[]"> JSON data received from web service </member>
	/// </container>

	/// <container name="Fit.Controls.WSDropDownTypeDefs.AbortedRequestEventArgs" extends="Fit.Controls.WSDropDownTypeDefs.RequestEventArgs">
	/// 	<description> Aborted request event arguments </description>
	/// 	<member name="Data" type="null"> JSON data received from web service </member>
	/// </container>

	/// <function container="Fit.Controls.WSDropDownTypeDefs" name="CancelableRequestEventHandler">
	/// 	<description> Cancelable request event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSDropDownTypeDefs.RequestEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSDropDownTypeDefs" name="ResponseEventHandler">
	/// 	<description> Response event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSDropDownTypeDefs.ResponseEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSDropDownTypeDefs" name="RequestAbortedEventHandler">
	/// 	<description> Aborted request handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSDropDownTypeDefs.AbortedRequestEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSDropDown" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is being requested.
	/// 		Request can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSDropDown) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSDropDown instance
	/// 		 - Picker: Picker causing WebService data request (WSTreeView or WSListView instance)
	/// 		 - Node: Fit.Controls.TreeViewNode instance if requesting TreeView children, Null if requesting root nodes
	/// 		 - Search: Search value if entered by user
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.CancelableRequestEventHandler"> Event handler function </param>
	/// </function>
	this.OnRequest = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onRequestHandlers, cb);
	}

	/// <function container="Fit.Controls.WSDropDown" name="OnResponse" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is received,
	/// 		allowing for data transformation to occure before
	/// 		picker control is populated. Function receives two arguments:
	/// 		Sender (Fit.Controls.WSDropDown) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSDropDown instance
	/// 		 - Picker: Picker causing WebService data request (WSTreeView or WSListView instance)
	/// 		 - Node: Fit.Controls.TreeViewNode instance if requesting TreeView children, Null if requesting root nodes
	/// 		 - Search: Search value if entered by user
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Data: JSON data received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.ResponseEventHandler"> Event handler function </param>
	/// </function>
	this.OnResponse = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onResponseHandlers, cb);
	}

	/// <function container="Fit.Controls.WSDropDown" name="OnAbort" access="public">
	/// 	<description>
	/// 		Add event handler fired if data request is canceled.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSDropDown) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSDropDown instance
	/// 		 - Picker: Picker causing WebService data request (WSTreeView or WSListView instance)
	/// 		 - Node: Fit.Controls.TreeViewNode instance if requesting TreeView children, Null if requesting root nodes
	/// 		 - Search: Search value if entered by user
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Data: JSON data received from WebService (Null in this particular case)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSDropDownTypeDefs.RequestAbortedEventHandler"> Event handler function </param>
	/// </function>
	this.OnAbort = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onAbortHandlers, cb);
	}

	/*this.OnPopulated = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onPopulatedHandlers, cb);
	}*/

	// ============================================
	// Private
	// ============================================

	function ensureTreeViewData(cb)
	{
		Fit.Validation.ExpectFunction(cb, true);

		if (dataRequested === false)
		{
			dataLoading = true;
			//nodesPopulated = false;

			tree.Reload(true, function(sender)
			{
				dataLoading = false;

				// Switch to action menu if no data was received. Must
				// be called after dataLoading is set since function
				// calls updateActionMenu(..) which relies on this flag.
				showActionMenuIfNoDataReceivedOrOnFirstInteractionIfEnabled();

				if (Fit.Validation.IsSet(cb) === true)
				{
					cb(me);
				}

				fireOnDataLoaded();
			});
		}
	}

	function clearData(allowImmediateReload, refreshActionMenu, cb)
	{
		Fit.Validation.ExpectBoolean(allowImmediateReload);
		Fit.Validation.ExpectBoolean(refreshActionMenu);
		Fit.Validation.ExpectFunction(cb, true);

		// Postpone if WebService operation is currently running

		if (requestCount > 0)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.ClearData(cb); });
			return;
		}

		// Clear data/cache/state

		hideLinesForFlatData = true;	// Make TreeView hide helper lines if nodes received have no children
		dataRequested = false;			// Make data in TreeView reload via ensureTreeViewData() when DropDown is opened
		autoUpdatedSelections = null;	// Remove cached result from AutoUpdateSelected(..) used when multiple calls to the function is made

		// Update action menu

		// Update action menu in case "Show available options" has been disabled, which
		// will be the case if the previous request returned no data. We need it enabled if
		// action menu is enabled - otherwise the user won't be able to request updated data.
		refreshActionMenu && updateActionMenu();

		// Cancel pending search operation if scheduled

		cancelSearch();

		// Invoke callback

		if (Fit.Validation.IsSet(cb) === true)
		{
			cb(me);
		}

		// Immediately load TreeView data if DropDown is open and TreeView is active picker

		if (allowImmediateReload === true && me.IsDropDownOpen() === true && me.GetPicker() === tree)
		{
			ensureTreeViewData(); // Will not load anything if callback above triggered data load, e.g. by calling AutoUpdateSelected(..)
		}
	}

	function showActionMenuIfNoDataReceivedOrOnFirstInteractionIfEnabled()
	{
		// If no data is returned and DropDown is in TextSelectionMode, the user will
		// not be able to remove objects from the DropDown, unless SelectionModeToggle
		// is true. Therefore we allow for items to be removed using an action menu.
		// EDIT: Now displays action menu in both Visual and Text Selection Mode for consistency.

		if (useActionMenuForced === false)
		{
			useActionMenu = tree.GetChildren().length === 0;
		}

		if (/*me.TextSelectionMode() === true &&*/ useActionMenu === true)
		{
			updateActionMenu();

			if (useActionMenuAfterLoad === true || tree.GetChildren().length === 0)
			{
				me.SetPicker(actionMenu);
			}
		}
	}

	function searchData(value)
	{
		// Abort timer responsible for starting search request X milliseconds after user stops typing

		cancelSearch();

		// Schedule new search request if a WebService request is already in progress

		if (currentRequest !== null)
		{
			forceNewSearch = true;
			return;
		}

		// Schedule search to run X milliseconds after user stops typing

		// Search value is passed to WSDropDown event handler using EventArgs object. External code
		// is responsible for exposing it to the backend, which in turn must return matching items.
		search = value;

		if (search !== "")
		{
			list.RemoveItems(); // Remove data from previous search
			me.SetPicker(list);

			timeOut = setTimeout(function()
			{
				if (me.IsDropDownOpen() === false)
				{
					suppressTreeOnOpen = true; // Drop Down changes picker to TreeView when OnOpen fires - prevent this from happening
					me.OpenDropDown();
				}

				list.Reload();

				timeOut = null;
			}, 500);
		}
	}

	function cancelSearch()
	{
		if (timeOut !== null)
		{
			clearTimeout(timeOut);
			timeOut = null;
		}
	}

	function updateActionMenu()
	{
		// Do not update unless needed. Notice that useActionMenu can be changed while action menu is active
		// by calling ResetActionMenu(), e.g. during OnChange when removing an item using the action menu.
		// See https://github.com/Jemt/Fit.UI/issues/179
		if (useActionMenu === false && me.GetPicker() !== actionMenu)
		{
			return;
		}

		var searchIcon = "<span class='FitUiControlDropDownActionMenuItemIcon FitUiControlDropDownActionMenuItemIconSearch fa fa-search'></span> ";
		var showAllIcon = "<span class='FitUiControlDropDownActionMenuItemIcon FitUiControlDropDownActionMenuItemIconShowAll fa fa-sitemap'></span> ";
		var delIcon = "<span class='FitUiControlDropDownActionMenuItemIcon FitUiControlDropDownActionMenuItemIconDelete fa fa-times'></span> ";

		var selectedItems = me.GetSelections();
		var addRemoveAll = selectedItems.length > 1;

		actionMenu.RemoveItems();

		if (me.InputEnabled() === true && me.SearchModeOnFocus() === false)
		{
			actionMenu.AddItem(searchIcon + translations.SearchMore, "SearchMore");
		}

		if (treeViewEnabled === true)
		{
			var isWaitingForData = dataRequested === false || dataLoading === true;

			if (isWaitingForData === true || tree.GetChildren().length > 0)
			{
				actionMenu.AddItem(showAllIcon + translations.ShowAllOptions, "ShowAll");
			}
			else
			{
				actionMenu.AddItem(showAllIcon + "<i>" + translations.NoneAvailable + "</i>", "ShowAllNoneFound");
			}
		}

		if (addRemoveAll === true)
		{
			actionMenu.AddItem(delIcon + translations.RemoveAll, "RemoveAll");
		}

		if (useActionMenuSortingLocale !== null)
		{
			if ("".localeCompare)
			{
				selectedItems = selectedItems.sort(function(a, b)
				{
					return a.Title.localeCompare(b.Title, useActionMenuSortingLocale);
				});
			}
			else
			{
				selectedItems = selectedItems.sort(function(a, b)
				{
					var titleA = a.Title.toLowerCase();
					var titleB = b.Title.toLowerCase();

					return titleA === titleB ? 0 : titleA > titleB ? 1 : -1;
				});
			}
		}

		Fit.Array.ForEach(selectedItems, function(item)
		{
			actionMenu.AddItem((addRemoveAll === true ? " &nbsp; &nbsp; &nbsp; " : "") + delIcon + translations.Remove + " " + (useActionMenuHtmlAllowed === false ? Fit.String.StripHtml(item.Title) : item.Title), "Remove:" + item.Value); // HTML might be allowed in DropDown control but not in ActionMenu, so we strip it away to avoid HTML code in selections
		});
	}

	function localize()
	{
		var locale = Fit.Internationalization.GetLocale(me);
		translations = locale.Translations;

		updateActionMenu();

		if (customPlaceholderSet === false)
		{
			orgPlaceholder(me.InputEnabled() === true ? translations.Search : "");
		}
	}

	function onDataLoaded(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onDataLoadedCallback, cb);
	}

	function fireOnDataLoaded()
	{
		// Copied from WSTreeView.
		// Immediately clear collection. If multiple callbacks are registered,
		// chances are that only the first will run, and the remaining will be
		// re-scheduled again - so we need the collection to be cleared before
		// invoking callbacks.
		var orgOnDataLoadedCallback = onDataLoadedCallback;
		onDataLoadedCallback = [];

		Fit.Array.ForEach(orgOnDataLoadedCallback, function(cb)
		{
			cb();
		});
	}

	function fireEventHandlers(handlers, picker, eventArgs)
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			var data = null; // Remains Null for OnAbort event (no Children or Items provided)
			var searchValue = Fit.Core.InstanceOf(picker, Fit.Controls.WSListView) === true ? search : "";

			if (eventArgs.Children) // WSTreeView
			{
				data = eventArgs.Children;
			}
			else if (eventArgs.Items) // WSListView
			{
				data = eventArgs.Items;
			}

			var newArgs = { Sender: me, Picker: picker, Node: (eventArgs.Node ? eventArgs.Node : null), SelectAll: eventArgs.SelectAll, Search: searchValue, Data: data, Request: eventArgs.Request };

			if (cb(me, newArgs) === false)
				cancel = true; // Do not cancel loop though - all handlers must be fired!

			// Assign data back to eventArgs, in case a new array instance was created
			if (eventArgs.Children) // WSTreeView
				eventArgs.Children = newArgs.Data;
			else if (eventArgs.Items) // WSListView
				eventArgs.Items = newArgs.Data;

			if (newArgs.Request !== eventArgs.Request)
			{
				// Support for changing request instans to
				// take control over webservice communication.

				// Restrict to support for Fit.Http.Request or classes derived from this
				Fit.Validation.ExpectInstance(newArgs.Request, Fit.Http.Request);

				eventArgs.Request = newArgs.Request;
			}
		});

		return !cancel;
	}

	init();
}

;(function()
{
	var locale = {
		// Primary locales must be registered before country specific overrides.
		// Example order: en, en_US, en_GB, de, de_AT, etc.
		// All locales inherit from en. All country specific overrides inherit
		// from their primary locale (e.g. de_AT inherits from de).
		// English (en) MUST be defined!

		"en": // US
		{
			Translations:
			{
				InvalidSelection		: "Invalid selection",

				// WSDropDown
				Search					: "Search..",
				SearchMore				: "Search for options",
				ShowAllOptions			: "Show available options",
				RemoveAll				: "Remove all selected",
				Remove					: "Remove",
				NoneAvailable			: "List with options is empty"
			}
		},
		"da":
		{
			Translations:
			{
				InvalidSelection		: "Ugyldigt valg",

				// WSDropDown
				Search					: "Søg..",
				SearchMore				: "Søg efter valgmuligheder",
				ShowAllOptions			: "Vis tilgængelige valgmuligheder",
				RemoveAll				: "Fjern alle valgte",
				Remove					: "Fjern",
				NoneAvailable			: "Liste med valgmuligheder er tom"
			}
		},
		"de":
		{
			Translations:
			{
				InvalidSelection		: "Ungültige Auswahl",

				// WSDropDown
				Search					: "Suchen..",
				SearchMore				: "Nach Optionen suchen",
				ShowAllOptions			: "Verfügbare Optionen anzeigen",
				RemoveAll				: "Alle ausgewählten entfernen",
				Remove					: "Entfernen",
				NoneAvailable			: "Liste der Optionen ist leer"
			}
		},
		"no":
		{
			Translations:
			{
				InvalidSelection		: "Ugyldig valg",

				// WSDropDown
				Search					: "Søk..",
				SearchMore				: "Søk etter alternativer",
				ShowAllOptions			: "Vis alle tilgjengelige alternativer",
				RemoveAll				: "Fjern alle valgte",
				Remove					: "Fjern",
				NoneAvailable			: "Ingen alternativer tilgjengelig"
			}
		}
	}
	Fit.Internationalization.AddLocalization(Fit.Controls.DropDown, locale);
	Fit.Internationalization.AddLocalization(Fit.Controls.WSDropDown, locale);
})();
/// <container name="Fit.Controls.FilePicker" extends="Fit.Controls.ControlBase">
/// 	Control allowing for files to be selected locally and uploaded asynchronously.
/// 	Extending from Fit.Controls.ControlBase.
/// </container>

// http://fiddle.jshell.net/gho7Lmno/84/

/// <function container="Fit.Controls.FilePicker" name="FilePicker" access="public">
/// 	<description> Create instance of FilePicker control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.FilePicker = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;
	var button = null;
	var buttonTitleEnforced = null;
	var input = null;
	var width = { Value: -1, Unit: "px" }; // Differs from default value on ControlBase which is 200px - here a value of -1 indicates width:auto
	var url = null;
	var files = [];
	var autoUpload = false;

	// Legacy control
	var inputs = []; // Contains input controls when legacy mode is enabled (IE9 and older)
	var inputsByFileId = {};

	var dropZoneLabel = null;
	var dropZoneLabelEnforced = null;
	var dropZoneContainer = null;

	var onUploadHandlers = [];
	var onProgressHandlers = [];
	var onSuccessHandlers = [];
	var onFailureHandlers = [];
	var onCompletedHandlers = [];
	//var onAbortHandlers = [];

	function init()
	{
		createUploadField();

		if (inputs.length === 0) // Modern control
		{
			button = new Fit.Controls.Button("Button" + me.GetId());
			button.Icon("upload"); // files-o
			button.OnClick(function(sender) { input.click(); }); // Make sure Enter/Spacebar opens file dialog

			// Add file picker to button
			Fit.Dom.Add(button.GetDomElement(), input);
			me._internal.AddDomElement(button.GetDomElement());
		}

		me.AddCssClass("FitUiControlFilePicker");
		me.Width(-1); // Set width:auto - ControlBase uses width:200px by default
		me.Enabled(true);

		me._internal.Data("legacy", (inputs.length > 0).toString());
		me._internal.Data("dropzone", "false");

		Fit.Internationalization.OnLocaleChanged(localize);
		localize();
	}

	function createUploadField()
	{
		var inp = document.createElement("input");
		inp.type = "file";
		inp.name = "SelectedFiles" + me.GetId() + "[]"; // Used when uploading via postback - each file is sent separately as "SelectedFile" when uploading using the Upload(..) function
		inp.onchange = function(e) // e may be null if called programmatically from FilePicker.Clear()
		{
			// The modern control clears any previously selected files when a new selection is made.
			// The legacy control consists of multiple single picker controls, so it has to keep
			// previously selected files. This adds a bit more complexity to the legacy control since
			// we need to preserve some state information between every time OnChange is fired
			// and if Upload is called multiple times.

			if (inputs.length === 0) // Modern control
			{
				setValueFromFilesList(inp.files); // Also triggers OnChange and performs postback if AutoUpload is true
			}
			else // Legacy control
			{
				files = [];
				inputsByFileId = {};

				// Add selected files to internal collection

				Fit.Array.ForEach(Fit.Array.Copy(inputs), function(i)
				{
					// Remove empty input fields

					if (i.value === "")
					{
						if (me.MultiSelectionMode() === true) // Remove empty upload controls in Multi Selection Mode
						{
							Fit.Dom.Remove(i);
							Fit.Array.Remove(inputs, i);
						}

						return; // Skip file picker, no file selected
					}

					// file has been selected, create file information object

					var file = null;

					// Some files may have been uploaded earlier (e.g. if AutoUpload is enabled or if user
					// selects 3 files, triggers upload, selects another 5 files, and triggers upload again).
					// Make sure file information is preserved for files already processed.

					if (me.MultiSelectionMode() === true && i._file !== undefined && i._file.Processed === true)
					{
						file = i._file;
					}
					else
					{
						file = createFileInfo(i.value.replace(/^C:\\fakepath\\/, ""), "Unknown", -1, null);
					}

					i._file = file;

					// Add file information

					Fit.Array.Add(files, file);
					inputsByFileId[file.Id] = i;
				});

				// Make sure an empty upload control is always available in Multi Selection Mode, allowing for another file to be added
				if (me.MultiSelectionMode() === true)
					createUploadField();

				me._internal.FireOnChange();

				if (me.AutoUpload() === true)
					me.Upload();
			}
		}

		if (inp.files && !Fit.Controls.FilePicker.ForceLegacyMode) // Modern control
		{
			input = inp;
			input.tabIndex = -1;

			Fit.Events.AddHandler(input, "click", function(e)
			{
				// Do not trigger OnClick on button containing file picker.
				// It will cause file picker to open file dialog twice.
				Fit.Events.StopPropagation(e);
			});
		}
		else // Legacy control
		{
			Fit.Array.Add(inputs, inp);
			me._internal.AddDomElement(inp);
		}
	}

	function setValueFromFilesList(fileList)
	{
		Fit.Validation.ExpectInstance(fileList, FileList);

		// Add selected files to internal collection

		files = [];
		Fit.Array.ForEach(fileList, function(file)
		{
			Fit.Array.Add(files, createFileInfo(file.name, file.type, file.size, file));

			// Dropzone always allow for multiple files to be dropped - handle Single Selection Mode here
			if (me.MultiSelectionMode() === false)
				return false; // Break loop
		});

		// Fire OnChange and trigger upload if configured to upload automatically

		me._internal.FireOnChange();

		if (me.AutoUpload() === true)
			me.Upload();
	}

	// ============================================
	// Public - overrides
	// ============================================

	// See documentation on ControlBase
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (Fit.Validation.IsSet(focus) === true)
		{
			if (inputs.length === 0) // Modern control
			{
				if (focus === true)
					input.focus();
				else
					input.blur();
			}
			else // Legacy control
			{
				if (focus === true)
					inputs[0].focus();
				else
					inputs[0].blur();
			}
		}

		return (Fit.Dom.GetFocused() === input);
	}

	// See documentation on ControlBase
	this.Value = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			Fit.Validation.ThrowError("SecurityError: Due to security restrictions, the value of an upload control cannot be set");
		}

		var strFiles = "";

		Fit.Array.ForEach(files, function(file)
		{
			strFiles += ((strFiles !== "") ? ";" : "") + file.Filename;
		});

		return strFiles;
	}

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		return (files.length > 0);
	}

	// See documentation on ControlBase
	this.Clear = function()
	{
		if (files.length === 0)
			return;

		if (inputs.length === 0) // Modern control
		{
			input.value = "";
			input.onchange(null);
		}
		else // Legacy control
		{
			// Remove all file pickers in Legacy Mode - clearing them is not possible in Legacy IE
			Fit.Array.ForEach(Fit.Array.Copy(inputs), function(i)
			{
				Fit.Dom.Remove(i);
				Fit.Array.Remove(inputs, i);
			});

			createUploadField();
			inputs[0].onchange(null);
		}
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		// This will destroy control - it will no longer work!

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		if (button !== null) // Modern control
		{
			button.Dispose();
		}

		me = button = buttonTitleEnforced = input = width = url = files = autoUpload = inputs = inputsByFileId = dropZoneLabel = dropZoneLabelEnforced = dropZoneContainer = onUploadHandlers = onProgressHandlers = onSuccessHandlers = onFailureHandlers = onCompletedHandlers = null; // onAbortHandlers
		base();
	});

	/// <function container="Fit.Controls.FilePicker" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description>
	/// 		Fit.Controls.ControlBase.Width override:
	/// 		Get/set control width - returns object with Value and Unit properties.
	/// 		This implementation differs from ControlBase.Width as passing a value of -1 resets
	/// 		the control width to fit its content, rather than assuming a fixed default width.
	/// 	</description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control width is updated to specified value. A value of -1 resets control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control width is updated to specified CSS unit </param>
	/// </function>
	this.Width = Fit.Core.CreateOverride(this.Width, function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1) // Fixed width
			{
				width = base(val, unit);

				if (inputs.length === 0 && me.ShowDropZone() === false)
					button.Width(100, "%");
			}
			else // Adjust width to content size
			{
				width = { Value: -1, Unit: "px" }; // Any changes to this line must be dublicated to line declaring the width variable !
				me.GetDomElement().style.width = "auto";

				if (inputs.length === 0 && me.ShowDropZone() === false)
					button.Width(-1);
			}
		}

		return width;
	});

	// See documentation on ControlBase
	this.Height = Fit.Core.CreateOverride(this.Height, function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true && inputs.length === 0) // Modern control - not supported in Legacy Mode
		{
			base(val, unit);

			if (me.ShowDropZone() === false)
			{
				if (val > -1) // Fixed height
					button.Height(100, "%");
				else // Adjust height to content size
					button.Height(-1);
			}
		}

		return base();
	});

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.FilePicker" name="Url" access="public" returns="string">
	/// 	<description>
	/// 		Get/set URL to which files are uploaded when FilePicker.Upload() is called.
	/// 		Multiple files will be uploaded using POST over individual HTTP connections,
	/// 		and each file will be accessible from the POST data collection using the SelectedFile key.
	/// 	</description>
	/// 	<param name="url" type="string" default="undefined"> If defined, upload URL is set to specified value </param>
	/// </function>
	this.Url = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true)

		if (Fit.Validation.IsSet(val) === true)
		{
			url = val;
		}

		return url;
	}

	/// <function container="Fit.Controls.FilePicker" name="ButtonText" access="public" returns="string">
	/// 	<description> Get/set button text </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, button text is set to specified value </param>
	/// </function>
	this.ButtonText = function(val)
	{
		Fit.Validation.ExpectString(val, true)

		if (Fit.Validation.IsSet(val) === true)
		{
			buttonTitleEnforced = val;

			if (button !== null) // Modern control
			{
				button.Title(val);
				Fit.Dom.Add(button.GetDomElement(), input);
			}
		}

		return (button !== null ? button.Title() : buttonTitleEnforced || ""); // Button is null in legacy mode
	}

	// DEPRECATED
	this.Title = function(val)
	{
		Fit.Browser.LogDeprecated("Use of deprecated function Title(..) on instance of Fit.Controls.FilePicker - please use ButtonText(..) instead");
		return me.ButtonText(val);
	}

	/// <function container="Fit.Controls.FilePicker" name="DropZoneText" access="public" returns="string">
	/// 	<description> Get/set drop zone text </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, drop zone text is set to specified value </param>
	/// </function>
	this.DropZoneText = function(val)
	{
		Fit.Validation.ExpectString(val, true)

		if (Fit.Validation.IsSet(val) === true)
		{
			dropZoneLabelEnforced = val;

			if (dropZoneLabel !== null)
				dropZoneLabel.innerHTML = val;
		}

		return (dropZoneLabel !== null ? dropZoneLabel.innerHTML : dropZoneLabelEnforced || "");
	}

	/// <function container="Fit.Controls.FilePicker" name="GetFiles" access="public" returns="Fit.Controls.FilePickerTypeDefs.File[]">
	/// 	<description>
	/// 		Get collection of selected files. Each file is represented as an object with the following members:
	/// 		 - Filename:string (Name of selected file)
	/// 		 - Type:string (Mime type for selected file)
	/// 		 - Size:integer (File size in bytes)
	/// 		 - Id:string (Unique file ID)
	/// 		 - Processed:boolean (Flag indicating whether file has been uploaded, or is currently being uploaded)
	/// 		 - Progress:integer (Value from 0-100 indicating upload progress, a value of -1 when not uploading/uploaded)
	/// 		 - FileObject:File (Native JS File object representing selected file)
	/// 		 - GetImagePreview:function (Returns an HTMLImageElement with a preview for supported file types)
	/// 		 - ServerResponse:string (Response from server after successful file upload, otherwise Null)
	/// 		NOTICE: The following properties/functions are not available in Legacy Mode: Type, Size, FileObject, GetImagePreview().
	/// 	</description>
	/// </function>
	this.GetFiles = function()
	{
		return files;
	}

	/// <function container="Fit.Controls.FilePicker" name="MultiSelectionMode" access="public" returns="boolean">
	/// 	<description> Get/set flag indicating whether control allows for multiple selections </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables multi selection mode, False disables it </param>
	/// </function>
	this.MultiSelectionMode = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true)

		if (Fit.Validation.IsSet(val) === true)
		{
			if (inputs.length === 0) // Modern control
			{
				if (val === true && me.MultiSelectionMode() === false)
				{
					input.multiple = "multiple";
				}
				else if (val === false && me.MultiSelectionMode() === true)
				{
					if (files.length > 1)
						me.Clear();

					input.multiple = "";
				}
			}
			else // Legacy control
			{
				me.Clear();
			}

			me._internal.Data("multiple", val.toString());

			localize();
		}

		return (me._internal.Data("multiple") === "true");
	}

	/// <function container="Fit.Controls.FilePicker" name="Enabled" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control is enabled or not.
	/// 		Any files added to the control prior to being disabled, will
	/// 		not be included with a traditional postback to the server.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables control, False disables it </param>
	/// </function>
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== me.Enabled())
		{
			me._internal.Data("enabled", val.toString());

			if (val === false)
			{
				me.Focused(false);
			}

			if (inputs.length === 0) // Modern control
			{
				input.disabled = !val;
				button.Enabled(val);
			}
			else // Legacy control
			{
				Fit.Array.ForEach(inputs, function(inp)
				{
					inp.disabled = !val;

					if (me.MultiSelectionMode() === true && val === true && inp._file !== undefined && inp._file.Processed === true) // Input controls previously uploaded should remain disabled
						inp.disabled = true;
				});
			}

			me._internal.UpdateInternalState();
			me._internal.Repaint();
		}

		return (me._internal.Data("enabled") === "true");
	}

	/// <function container="Fit.Controls.FilePicker" name="ShowDropZone" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is displayed as a drop zone on supported browsers or not </description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables drop zone, False disables it (default) </param>
	/// </function>
	this.ShowDropZone = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && inputs.length === 0) // Only available for modern control
		{
			if (val === true && dropZoneLabel === null)
			{
				dropZoneLabel = Fit.Dom.CreateElement("<div></div>");
				dropZoneContainer = Fit.Dom.CreateElement("<div></div>");

				/*me.GetDomElement().ondragenter = function(e)
				{
					me._internal.Data("dropping", "true");
				}*/

				me.GetDomElement().ondragleave = function(e)
				{
					me._internal.Data("dropping", "false");
				}

				// OnDragOver must be overridden and suppressed to avoid files being opened in browser when dropped
				me.GetDomElement().ondragover = function(e)
				{
					var ev = Fit.Events.GetEvent(e);
					Fit.Events.PreventDefault(ev); // Prevent files from opening in browser

					// OnDragEnter and OnDragLeave is not sufficient to update dropping state since user may enter dropzone (triggers OnDragEnter),
					// then hover button (triggers OnDragLeave), and then hover dropzone again which unfortunately doesn't trigger OnDragEnter again.
					me._internal.Data("dropping", "true");
				}

				me.GetDomElement().ondrop = function(e)
				{
					var ev = Fit.Events.GetEvent(e);
					Fit.Events.PreventDefault(ev); // Prevent files from opening in browser

					setValueFromFilesList(ev.dataTransfer.files); // Also triggers OnChange and performs postback if AutoUpload is true
					me._internal.Data("dropping", "false");
				}

				button.Width(-1);
				button.Height(-1);

				Fit.Dom.Add(dropZoneContainer, dropZoneLabel);
				Fit.Dom.Add(dropZoneContainer, button.GetDomElement());
				Fit.Dom.Add(me.GetDomElement(), dropZoneContainer);

				localize();

				me._internal.Data("dropping", "false");
				me._internal.Data("dropzone", "true");
			}
			else if (val === false && dropZoneLabel !== null)
			{
				//me.GetDomElement().ondragenter = null;
				me.GetDomElement().ondragleave = null;
				me.GetDomElement().ondragover = null;
				me.GetDomElement().ondrop = null;

				button.Width(me.Width().Value, me.Width().Unit);
				button.Height(me.Height().Value, me.Height().Unit);

				Fit.Dom.Remove(dropZoneContainer);
				Fit.Dom.Add(me.GetDomElement(), button.GetDomElement());

				dropZoneLabel = null;
				dropZoneContainer = null;

				me._internal.Data("dropping", null);
				me._internal.Data("dropzone", "false");
			}
		}

		return (dropZoneLabel !== null);
	}

	/// <function container="Fit.Controls.FilePicker" name="IsLegacyModeEnabled" access="public" returns="boolean">
	/// 	<description> Get value indicating whether control is in legacy mode (old fashion upload control) </description>
	/// </function>
	this.IsLegacyModeEnabled = function()
	{
		return (inputs.length > 0);
	}

	/// <function container="Fit.Controls.FilePicker" name="AutoUpload" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control automatically starts upload process when files are selected </description>
	/// 	<param name="val" type="boolean" default="undefined"> If specified, True enables auto upload, False disables it (default) </param>
	/// </function>
	this.AutoUpload = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			autoUpload = val;
		}

		return autoUpload;
	}

	/// <function container="Fit.Controls.FilePicker" name="Upload" access="public">
	/// 	<description>
	/// 		Upload selected files. Each file will be uploaded using POST over individual HTTP connections,
	/// 		and each file will be accessible from the POST data collection using the SelectedFile key.
	/// 		Use the OnProgress event to monitor the upload process, or use the OnCompleted event
	/// 		to be notified when all files have been fully processed.
	/// 	</description>
	/// 	<param name="skip" type="string[]" default="undefined">
	/// 		Optional argument allowing some of the selected files to be skipped during
	/// 		upload. The argument is a string array with the names of the files to skip.
	/// 	</param>
	/// </function>
	this.Upload = function(skip)
	{
		Fit.Validation.ExpectTypeArray(skip, Fit.Validation.ExpectStringValue, true);

		if (url === null)
			Fit.Validation.ThrowError("Unable to upload file(s), no URL has been specified");

		if (me.Enabled() === false) // Legacy control cannot upload data when disabled - this ensures consistency
			return;

		var filesToUpload = [];

		Fit.Array.ForEach(files, function(file)
		{
			if (Fit.Validation.IsSet(skip) === false || Fit.Array.Contains(skip, file.Filename) === false)
			{
				if (file.Processed === true) // Skip previously uploaded files
					return;

				Fit.Array.Add(filesToUpload, file);
			}
		});

		if (filesToUpload.length === 0)
			return;

		if (fireEvent(onUploadHandlers) === false)
			return;

		var completed = [];

		Fit.Array.ForEach(filesToUpload, function(file)
		{
			file.Processed = true;

			if (inputs.length === 0) // Modern control
			{
				var data = new FormData();
				data.append("SelectedFile", file.FileObject);

				var req = new XMLHttpRequest();
				Fit.Events.AddHandler(req.upload, "progress", function(e)
				{
					var ev = Fit.Events.GetEvent(e);
					fireEvent(onProgressHandlers, file, ((ev.loaded > 0 && ev.total > 0) ? Math.floor((ev.loaded / ev.total) * 100) : 0));
				});
				Fit.Events.AddHandler(req, "readystatechange", function(e)
				{
					// Using readystatechange rather than req.upload.onload (success) and req.upload.onerror (failure).
					// Server response is not available when req.upload.onload fires: Fit.Events.AddHandler(req.upload, "load", function(e) { /*..*/ });
					// OnError only fires when network communication fails (e.g. network unplugged or server process is restarted), not when an error
					// occur server side (500 Internal Server Error). Using readystatechange we can catch both communication and server errors.

					if (req.readyState === 4)
					{
						if (req.status === 200) // Success
						{
							file.ServerResponse = req.responseText;

							Fit.Array.Add(completed, file);
							fireEvent(onSuccessHandlers, file);
						}
						else // Failure
						{
							Fit.Array.Add(completed, file);
							fireEvent(onFailureHandlers, file);
						}

						if (completed.length === filesToUpload.length)
							fireEvent(onCompletedHandlers);
					}
				});
				/*Fit.Events.AddHandler(req.upload, "abort", function(e)
				{
					fireEvent(onAbortHandlers, file);
				});*/

				req.open("POST", url);
				req.send(data);
			}
			else // Legacy control
			{
				var enforcedOnModernBrowser = (Fit.Browser.GetInfo().Name !== "MSIE" || Fit.Browser.GetInfo().Version > 8);

				var picker = inputsByFileId[file.Id];

				var iFrame = null;
				var form = null;

				var progress = 0;
				var interval = null;

				// Create hidden iFrame used to upload current file asynchronously

				iFrame = document.createElement("iframe");
				iFrame.name = "iFrame" + Fit.Data.CreateGuid();
				iFrame.style.display = "none";

				if (enforcedOnModernBrowser === true)
				{
					// When Legacy Mode is enforced in modern browsers, the OnLoad handler MUST be registered
					// AFTER rooting iFrame in DOM, to prevent WebKit/Chrome from firing OnLoad multiple times.
					Fit.Dom.InsertAfter(picker, iFrame);
				}

				Fit.Events.AddHandler(iFrame, "load", function(e)
				{
					// Read server response

					try // Will throw an error if uploading to foreign domain, in which case ServerResponse will remain Null
					{
						file.ServerResponse = iFrame.contentDocument.body.innerHTML;
					}
					catch (err)
					{
						Fit.Browser.Log("Unable to read server response, most likely due to violation of Same-Origin Policy");
					}

					// Clean up

					Fit.Dom.Remove(iFrame);
					clearInterval(interval);

					// Fire events

					Fit.Array.Add(completed, file);

					fireEvent(onProgressHandlers, file, 100);
					fireEvent(onSuccessHandlers, file);

					if (completed.length === filesToUpload.length)
						fireEvent(onCompletedHandlers);
				});

				if (enforcedOnModernBrowser === false)
				{
					// On IE8 the OnLoad handler MUST be registered BEFORE
					// rooting iFrame in DOM, otherwise it will not be fired.
					Fit.Dom.InsertAfter(picker, iFrame);
				}

				// Create form used to upload current file - data is posted to hidden iFrame created above

				form = document.createElement("form");
				form.action = url;
				form.method = "POST";
				form.setAttribute("enctype", "multipart/form-data"); // Must be registered using setAttribute() for upload to work in IE8 //form.enctype="multipart/form-data";
				form.target = iFrame.name;

				// Add form to page, and file picker to form.
				// Having multiple forms on the same page might break apps expecting only one form element, so it is
				// immediately removed from page once submitted, and picker is returned to its prior location in the DOM.
				Fit.Dom.InsertAfter(picker, form);
				form.appendChild(picker);

				// Post data - start file upload
				form.submit();

				// Immediately remove form element after submit, as it might confuse apps expecting
				// only one form element on page. Picker is returned to its prior location in the DOM.
				Fit.Dom.InsertBefore(iFrame, picker);
				Fit.Dom.Remove(form);

				// Simulate progress in legacy mode which does not support the progress event

				interval = setInterval(function()
				{
					if (me === null)
					{
						// Control was disposed while uploading file
						clearInterval(interval);
						return;
					}

					progress = progress + 5;
					fireEvent(onProgressHandlers, file, progress);

					if (progress === 90) // Halt progress on 90% and wait for file to fully complete
						clearInterval(interval);

				}, 1000);

				if (me.MultiSelectionMode() === true)
					picker.disabled = true;
			}
		});
	}

	function localize()
	{
		if (inputs.length > 0)
			return; // Legacy mode - nothing to localize

		var locale = Fit.Internationalization.GetLocale(me);

		if (buttonTitleEnforced === null)
		{
			var buttonTitle = (me.MultiSelectionMode() === true ? locale.SelectFiles : locale.SelectFile);
			button.Title(buttonTitle);
		}

		if (dropZoneLabel !== null)
		{
			var dzText = (dropZoneLabelEnforced !== null ? dropZoneLabelEnforced : (me.MultiSelectionMode() === true ? locale.DropFiles : locale.DropFile));
			dropZoneLabel.innerHTML = dzText;
		}
	}

	function getImagePreview(file) // file is an instance of File (native JS object type) - returns Image instance if preview can be created, otherwise Null
	{
		if (inputs.length > 0)
			return null; // Legacy control

		if (file.type === "image/png" || file.type === "image/jpg" || file.type === "image/jpeg" || file.type === "image/svg" || file.type === "image/gif")
		{
			var img = new Image();

			setTimeout(function() // Postpone to allow external code to register an onload handler on the image, which should be registered before assigning image source (src)
			{
				var fr = new FileReader();
				fr.onload = function(e)
				{
					var ev = Fit.Events.GetEvent(e);
					img.src = ev.target.result;
				}
				fr.readAsDataURL(file);
			}, 0);

			return img;
		}

		return null;
	}

	function fireEvent(handlers, file, progress)
	{
		var canceled = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (Fit.Validation.IsSet(file) === true) // OnSuccess/OnFailure/OnProcess
			{
				var eventArgs = cloneFileInfo(file);
				eventArgs.Progress = 100;

				if (Fit.Validation.IsSet(progress) === true)
					eventArgs.Progress = progress;

				if (cb(me, eventArgs) === false)
					canceled = true;
			}
			else // OnUpload/OnCompleted
			{
				if (cb(me) === false)
					canceled = true;
			}
		});

		return !canceled;
	}

	function createFileInfo(filename, type, size, fileObject)
	{
		Fit.Validation.ExpectString(filename);
		Fit.Validation.ExpectString(type);
		Fit.Validation.ExpectNumber(size);
		Fit.Validation.ExpectInstance(fileObject, File, true);

		// IMPORTANT: Make sure changes to this object is also made to object returned by cloneFileInfo(..)
		return { Filename: filename, Type: type, Size: size, Id: Fit.Data.CreateGuid(), Processed: false, Progress: -1, FileObject: fileObject || null, GetImagePreview: function() { return getImagePreview(fileObject); }, ServerResponse: null };
	}

	function cloneFileInfo(file) // Object as created by createFileInfo(..)
	{
		// We cannot use Fit.Core.Clone(..) since this is not a simple JSON object (DOM input field contained).
		// Also notice that the clone's Input and GetImagePreview properties are references (shared with original object).

		return { Filename: file.Filename, Type: file.Type, Size: file.Size, Id: file.Id, Processed: file.Processed, Progress: file.Progress, FileObject: file.FileObject, GetImagePreview: file.GetImagePreview, ServerResponse: file.ServerResponse };
	}

	// ============================================
	// Events
	// ============================================

	/// <function container="Fit.Controls.FilePickerTypeDefs" name="GetImagePreview" returns="HTMLImageElement | null">
	/// 	<description> Returns image preview for supported file types, Null for unsupported file types, and Null on browsers not supporting the File API </description>
	/// </function>

	/// <container name="Fit.Controls.FilePickerTypeDefs.File">
	/// 	<description> File information </description>
	/// 	<member name="Filename" type="string"> File name </member>
	/// 	<member name="Type" type="string"> Mime type - returns Unknown on browsers not supporting the File API </member>
	/// 	<member name="Size" type="integer"> File size in bytes - returns -1 on browsers not supporting the File API </member>
	/// 	<member name="Id" type="string"> Unique file ID </member>
	/// 	<member name="Processed" type="boolean"> Flag indicating whether file has been uploaded, or is currently being uploaded, with a value of True </member>
	/// 	<member name="Progress" type="integer"> Value from 0-100 indicating upload progress, a value of -1 when not uploading/uploaded </member>
	/// 	<member name="FileObject" type="File | null"> Native JS File object representing file data - returns Null on browsers not supporting the File API </member>
	/// 	<member name="GetImagePreview" type="Fit.Controls.FilePickerTypeDefs.GetImagePreview"> Get image preview for supported file types - returns Null on browsers not supporting the File API </member>
	/// 	<member name="ServerResponse" type="string | null"> Response from server after successful file upload, otherwise Null </member>
	/// </container>

	/// <function container="Fit.Controls.FilePickerTypeDefs" name="CancelableUploadEventHandler" returns="boolean | void">
	/// 	<description> Cancelable upload event handler </description>
	/// 	<param name="sender" type="Fit.Controls.FilePicker"> Instance of FilePicker </param>
	/// </function>

	/// <function container="Fit.Controls.FilePickerTypeDefs" name="CompletedEventHandler">
	/// 	<description> Completed event handler </description>
	/// 	<param name="sender" type="Fit.Controls.FilePicker"> Instance of FilePicker </param>
	/// </function>

	/// <function container="Fit.Controls.FilePickerTypeDefs" name="ProgressEventHandler">
	/// 	<description> Progress event handler </description>
	/// 	<param name="sender" type="Fit.Controls.FilePicker"> Instance of FilePicker </param>
	/// 	<param name="eventArgs" type="Fit.Controls.FilePickerTypeDefs.File"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.FilePicker" name="OnUpload" access="public">
	/// 	<description>
	/// 		Add event handler fired when upload is started.
	/// 		Operation can be canceled by returning False.
	/// 		Function receives one argument: Sender (Fit.Controls.FilePicker).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.FilePickerTypeDefs.CancelableUploadEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnUpload = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onUploadHandlers, cb);
	}

	/// <function container="Fit.Controls.FilePicker" name="OnProgress" access="public">
	/// 	<description>
	/// 		Add event handler fired when the upload process for a given file is progressing.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.FlePicker) and EventArgs object.
	/// 		EventArgs object contains the following members:
	/// 		 - Filename:string (Name of given file)
	/// 		 - Type:string (Mime type for given file)
	/// 		 - Size:integer (File size in bytes)
	/// 		 - Id:string (Unique file ID)
	/// 		 - Processed:boolean (Flag indicating whether file has been uploaded, or is currently being uploaded)
	/// 		 - Progress:integer (A value from 0-100 indicating how many percent of the file has been uploaded)
	/// 		 - FileObject:File (Native JS File object representing given file)
	/// 		 - GetImagePreview:function (Returns an HTMLImageElement with a preview for supported file types)
	/// 		 - ServerResponse:string (Response from server after successful file upload, otherwise Null)
	/// 		Be aware that Type and Size cannot be determined in Legacy Mode, and that FileObject in this
	/// 		case will be Null. GetImagePreview() will also return Null in Legacy Mode.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.FilePickerTypeDefs.ProgressEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnProgress = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onProgressHandlers, cb);
	}

	/// <function container="Fit.Controls.FilePicker" name="OnSuccess" access="public">
	/// 	<description>
	/// 		Add event handler fired when a file has successfully been uploaded.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.FlePicker) and EventArgs object.
	/// 		EventArgs object contains the following members:
	/// 		 - Filename:string (Name of given file)
	/// 		 - Type:string (Mime type for given file)
	/// 		 - Size:integer (File size in bytes)
	/// 		 - Id:string (Unique file ID)
	/// 		 - Processed:boolean (Flag indicating whether file has been uploaded, or is currently being uploaded)
	/// 		 - Progress:integer (A value from 0-100 indicating how many percent of the file has been uploaded)
	/// 		 - FileObject:File (Native JS File object representing given file)
	/// 		 - GetImagePreview:function (Returns an HTMLImageElement with a preview for supported file types)
	/// 		 - ServerResponse:string (Response from server after successful file upload, otherwise Null)
	/// 		Be aware that Type and Size cannot be determined in Legacy Mode, and that FileObject in this
	/// 		case will be Null. GetImagePreview() will also return Null in Legacy Mode.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.FilePickerTypeDefs.ProgressEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSuccess = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSuccessHandlers, cb);
	}

	/// <function container="Fit.Controls.FilePicker" name="OnFailure" access="public">
	/// 	<description>
	/// 		Add event handler fired for a given file if the upload process failed.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.FlePicker) and EventArgs object.
	/// 		EventArgs object contains the following members:
	/// 		 - Filename:string (Name of given file)
	/// 		 - Type:string (Mime type for given file)
	/// 		 - Size:integer (File size in bytes)
	/// 		 - Id:string (Unique file ID)
	/// 		 - Processed:boolean (Flag indicating whether file has been uploaded, or is currently being uploaded)
	/// 		 - Progress:integer (A value from 0-100 indicating how many percent of the file has been uploaded)
	/// 		 - FileObject:File (Native JS File object representing given file)
	/// 		 - GetImagePreview:function (Returns an HTMLImageElement with a preview for supported file types)
	/// 		 - ServerResponse:string (Response from server after successful file upload, otherwise Null)
	/// 		Be aware that Type and Size cannot be determined in Legacy Mode, and that FileObject in this
	/// 		case will be Null. GetImagePreview() will also return Null in Legacy Mode.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.FilePickerTypeDefs.ProgressEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnFailure = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onFailureHandlers, cb);
	}

	/// <function container="Fit.Controls.FilePicker" name="OnCompleted" access="public">
	/// 	<description>
	/// 		Add event handler fired when all files selected have been fully processed.
	/// 		Be aware that this event fires even if some files were not uploaded successfully.
	/// 		At this point files returned from GetFiles() contains a ServerResponse:string property
	/// 		containing the response from the server. This property remains Null in case of errors.
	/// 		Function receives one argument: Sender (Fit.Controls.FlePicker).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.FilePickerTypeDefs.CompletedEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnCompleted = function(cb) // Fires when all files have been processed, even if files have failed!
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onCompletedHandlers, cb);
	}

	/*this.OnAbort = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onAbortHandlers, cb);
	}*/

	init();
}

Fit.Internationalization.AddLocalization(Fit.Controls.FilePicker,
{
	// Primary locales must be registered before country specific overrides.
	// Example order: en, en_GB, de, de_AT, etc.
	// All locales inherit from en. All country specific overrides inherit
	// from their general locale (e.g. de_AT inherits from de).
	// English (en) MUST be defined, and be defined first!

	"en":
	{
		"SelectFile": "Select file",
		"SelectFiles": "Select files",
		"DropFile": "Drag and drop file here, or",
		"DropFiles": "Drag and drop files here, or"
	},
	"da":
	{
		"SelectFile": "Vælg fil",
		"SelectFiles": "Vælg filer",
		"DropFile": "Træk og slip fil her, eller",
		"DropFiles": "Træk og slip filer her, eller"
	},
	"de":
	{
		"SelectFile": "Datei aussuchen",
		"SelectFiles": "Dateien auswählen",
		"DropFile": "Zieh deine Datei hierher und lege sie ab, oder",
		"DropFiles": "Zieh deine Dateien hierher und lege sie ab, oder"
	},
	"no":
	{
		"SelectFile": "Velg fil",
		"SelectFiles": "Velg filer",
		"DropFile": "Dra og slipp fil her, eller",
		"DropFiles": "Dra og slipp filer her, eller"
	}
});

/// <container name="Fit.Controls.Input" extends="Fit.Controls.ControlBase">
/// 	Input control which allows for one or multiple lines of
/// 	text, and features a Design Mode for rich HTML content.
/// 	Extending from Fit.Controls.ControlBase.
/// </container>

/// <function container="Fit.Controls.Input" name="Input" access="public">
/// 	<description> Create instance of Input control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.Input = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;
	var orgVal = "";			// Holds initial value used to determine IsDirty state
	var preVal = "";			// Holds latest change made by user - used to determine whether OnChange needs to be fired
	var input = null;
	var changeObserverId = -1;	// Holds interval ID to observer function for controls (e.g. color picker) not continuously firing OnChange when value is changed
	var cmdResize = null;
	var designEditor = null;
	var designEditorDom = null; // DOM elements within CKEditor which we rely on - some <div> elements become <span> elements in older browsers
	/*{
		OuterContainer: null,	// <div class="cke">
		InnerContainer: null,	//     <div class="cke_inner">
		Top: null,				//         <span class="cke_top">
		Content: null,			//         <div class="cke_contents">
		Editable: null,			//             <div class="cke_editable">
		Bottom: null			//         <span class="cke_bottom">
	}*/
	var designEditorDirty = false;
	var designEditorDirtyPending = false;
	var designEditorConfig = null;
	var designEditorReloadConfig = null;
	var designEditorRestoreButtonState = null;
	var designEditorSuppressPaste = false;
	var designEditorSuppressOnResize = false;
	var designEditorMustReloadWhenReady = false;
	var designEditorMustDisposeWhenReady = false;
	var designEditorUpdateSizeDebouncer = -1;
	var designEditorHeightMonitorId = -1;
	var designEditorActiveToolbarPanel = null; // { DomElement: HTMLElement, UnlockFocusStateIfEmojiPanelIsClosed: function, CloseEmojiPanel: function }
	var designEditorDetached = null; // { IsActive: boolean, GetValue: function, SetVisible: function, SetEnabled: function, Focus: function, Reload: function, Open: function, Close: function, Dispose: function }
	var designEditorClearPlaceholder = true;
	var designEditorCleanEditableDom = false;
	var designEditorGlobalKeyDownEventId = -1;
	var designEditorGlobalKeyUpEventId = -1;
	//var htmlWrappedInParagraph = false;
	var wasAutoChangedToMultiLineMode = false; // Used to revert to single line if multi line was automatically enabled along with DesignMode(true), Maximizable(true), or Resizable(true)
	var minimizeHeight = -1;
	var maximizeHeight = -1;
	var minMaxUnit = null;
	var maximizeHeightConfigured = -1;
	var resizable = Fit.Controls.InputResizing.Disabled;
	var nativeResizableAvailable = false; // Updated in init()
	var mutationObserverId = -1;		// Specific to DesignMode
	var rootedEventId = -1;				// Specific to DesignMode
	var createWhenReadyIntervalId = -1;	// Specific to DesignMode
	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);
	var debounceOnChangeTimeout = -1;
	var debouncedOnChange = null;
	var imageBlobUrls = [];				// Specific to DesignMode
	var locale = null;

	// ============================================
	// Init
	// ============================================

	function init()
	{
		input = document.createElement("input");
		input.type = "text";
		input.autocomplete = "off";
		input.spellcheck = true;
		input.onkeyup = function()
		{
			if (designEditorClearPlaceholder === true)
			{
				designEditorClearPlaceholder = false;	// Prevent additional calls to updateDesignEditorPlaceholder - it retrieves editor value which is expensive
				updateDesignEditorPlaceholder(true);	// Clear placeholder
			}

			if (debounceOnChangeTimeout === -1)
			{
				fireOnChange();
			}
			else
			{
				if (debouncedOnChange === null)
				{
					debouncedOnChange = Fit.Core.CreateDebouncer(fireOnChange, debounceOnChangeTimeout);
				}

				debouncedOnChange.Invoke();
			}

			if (me.Maximizable() === true)
			{
				// Scroll to bottom if nearby, to make sure text does not collide with maximize button.
				// Extra padding-bottom is added inside control to allow for spacing between text and maximize button.

				var scrollContainer = designEditorDom && designEditorDom.Editable || input;
				var autoScrollToBottom = scrollContainer.scrollTop + scrollContainer.clientHeight > scrollContainer.scrollHeight - 15; // True when at bottom or very close (15px buffer)

				if (autoScrollToBottom === true)
				{
					scrollContainer.scrollTop += 99;
				}
			}
		}
		input.onchange = function() // OnKeyUp does not catch changes by mouse (e.g. paste or moving selected text)
		{
			if (me === null)
			{
				// Fix for Chrome which fires OnChange and OnBlur (in both capturering and bubbling phase)
				// if control has focus while being removed from DOM, e.g. if used in a dialog closed using ESC.
				// More details here: https://bugs.chromium.org/p/chromium/issues/detail?id=866242
				return;
			}

			input.onkeyup();
		}
		me._internal.AddDomElement(input);

		me.AddCssClass("FitUiControlInput");

		me._internal.Data("multiline", "false");
		me._internal.Data("maximizable", "false");
		me._internal.Data("maximized", "false");
		me._internal.Data("resizable", resizable.toLowerCase());
		me._internal.Data("resized", "false");
		me._internal.Data("designmode", "false");

		Fit.Internationalization.OnLocaleChanged(localize);
		localize();

		me.OnBlur(function(sender)
		{
			hideToolbarInDesignMode(); // Hide toolbar if configured to do so

			// Due to CKEditor and plugins allowing for inconsistency between what is being
			// pushed via OnChange and the editor's actual value, we ensure that the latest
			// and actual value is pushed via Input.OnChange when the control lose focus.
			// See related bug report for CKEditor here: https://github.com/ckeditor/ckeditor4/issues/4856

			if (debouncedOnChange !== null)
			{
				debouncedOnChange.Cancel(); // Do not trigger fireOnChange twice (below) if currently scheduled for execution
			}

			fireOnChange(); // Only fires OnChange if value has actually changed
		});

		me.OnFocus(function(sender)
		{
			restoreHiddenToolbarInDesignEditor();	// Make toolbar appear if currently hidden
			//updateDesignEditorPlaceholder(true);	// Clear placeholder text

			if (me.Type() === "Color") // Color picker does not continuously fire OnChange when changing color - fix that using an observer function
			{
				changeObserverId = setInterval(function()
				{
					input.onkeyup();
				}, 100);
			}
		});
		me.OnBlur(function(sender)
		{
			restoreDesignEditorButtons();			// Restore (enable) editor's toolbar buttons in case they were temporarily disabled
			updateDesignEditorPlaceholder();		// Show placeholder text if control value is empty

			if (changeObserverId !== -1)
			{
				clearInterval(changeObserverId);
				changeObserverId = -1;
			}
		});

		Fit.Events.AddHandler(me.GetDomElement(), "paste", true, function(e)
		{
			if (me.DesignMode() === true && designEditorSuppressPaste === true)
			{
				Fit.Events.Stop(e);
			}
		});

		// Suppress CKEditor's ContextMenu to open the browser's own ContextMenu,
		// unless right-clicking in a table, in which case we need access to the table tools.
		Fit.Events.AddHandler(me.GetDomElement(), "contextmenu", true, function(e) // Capture phase (true argument) not supported by IE8 - too bad, IE8 users will have to use the browser's Edit menu at the top to cut/copy/paste
		{
			if (me.DesignMode() === true)
			{
				var ev = Fit.Events.GetEvent(e);
				var target = Fit.Events.GetTarget(ev);

				if (isTable(target) === false)
				{
					Fit.Events.StopPropagation(ev); // Suppress CKEditor's context menu (required by the table plugin)
				}
			}
		});
		Fit.Events.AddHandler(me.GetDomElement(), "keydown", true, function(e) // Capture phase (true argument) not supported by IE8
		{
			if (me.DesignMode() === true)
			{
				var ev = Fit.Events.GetEvent(e);

				if ((ev.shiftKey === true && ev.keyCode === 121) || ev.keyCode === 93) // SHIFT + F10 or Windows ContextMenu key
				{
					Fit.Events.StopPropagation(ev); // Suppress CKEditor's context menu (required by the table plugin)
				}
			}
		});

		try
		{
			// We rely on the .buttons property to optimize resizing for textarea (MultiLine mode).
			// The MouseEvent class might not be available on older browsers or might throw an exception when constructing.
			nativeResizableAvailable = window.MouseEvent && new MouseEvent("mousemove", {}).buttons !== undefined || false;
		}
		catch (err) {}
	}

	// ============================================
	// Public - overrides
	// ============================================

	// See documentation on ControlBase
	this.Visible = Fit.Core.CreateOverride(this.Visible, function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) && designEditorDetached !== null)
		{
			designEditorDetached.SetVisible(val);
		}

		return base(val);
	});

	// See documentation on ControlBase
	this.Enabled = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== me.Enabled())
		{
			me._internal.Data("enabled", val === true ? "true" : "false");

			if (val === false)
			{
				me.Focused(false);
			}

			input.disabled = val === false;

			if (designModeEnabledAndReady() === true) // ReadOnly mode will be set when instance is ready, if not ready at this time
			{
				designEditor.setReadOnly(input.disabled);

				// Set tabindex to allow or disallow focus. Unfortunately there is no editor API for changing the tabindex.
				// Preventing focus is only possible by nullifying DOM attribute (these does not work: delete elm.tabIndex; elm.tabIndex = null|undefined|-1).
				Fit.Dom.Attribute(designEditorDom.Editable, "tabindex", input.disabled === true ? null : "0");

				// Prevent control from losing focus when HTML editor is initialized,
				// e.g. if Design Mode is enabled when ordinary input control gains focus.
				// This also prevents control from losing focus if toolbar is clicked without
				// hitting a button. A value of -1 makes it focusable, but keeps it out of
				// tab flow (keyboard navigation). Also set when DesignMode(true) is called.
				Fit.Dom.Attribute(me.GetDomElement(), "tabindex", input.disabled !== true && me.DesignMode() === true ? "-1" : null); // Remove tabindex used to prevent control from losing focus when clicking toolbar buttons, as it will allow control to gain focus when clicked using the mouse
			}

			if (designEditorDetached !== null)
			{
				designEditorDetached.SetEnabled(val);
			}

			me._internal.UpdateInternalState();
			me._internal.Repaint();
		}

		return me._internal.Data("enabled") === "true";
	}

	// See documentation on ControlBase
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (designEditorDetached !== null && designEditorDetached.IsActive === true)
		{
			if (focus === true)
			{
				designEditorDetached.Focus();
			}
			else if (focus === false)
			{
				Fit.Browser.Debug("WARNING: Unable to remove focus from Input control '" + me.GetId() + "' when modal detached editor is open");
			}

			return me.Visible(); // Always considered focused if detached editor is open and control (along with detached editor) is visible
			//return designEditorDetached.GetFocused();
		}

		var elm = input;

		if (me.DesignMode() === true)
		{
			if (designModeEnabledAndReady() === true)
			{
				elm = designEditor; // Notice: designEditor is an instance of CKEditor, not a DOM element, but it does expose a focus() function
			}
			else
			{
				elm = me.GetDomElement(); // Editor not loaded yet - focus control container temporarily - focus is later moved to editable area once instanceReady handler is invoked
			}
		}

		if (Fit.Validation.IsSet(focus) === true)
		{
			if (focus === true)
			{
				if (Fit._internal.Controls.Input.ActiveEditorForDialog === me)
				{
					// Remove flag used to auto close editor dialog, in case Focused(false)
					// was called followed by Focused(true), while editor dialog was loading.
					delete Fit._internal.Controls.Input.ActiveDialogForEditorCanceled;
				}

				elm.focus();
			}
			else // Remove focus
			{
				if (designModeEnabledAndReady() === true)
				{
					if (Fit._internal.Controls.Input.ActiveEditorForDialog === me)
					{
						if (Fit._internal.Controls.Input.ActiveDialogForEditor !== null)
						{
							// A dialog (e.g. link or image dialog) is currently open, and will now be closed

							// Hide dialog - fires dialog's OnHide event and returns focus to editor
							Fit._internal.Controls.Input.ActiveDialogForEditor.hide();

							// CKEditor instance has no blur() function, so we call blur() on DOM element currently focused within CKEditor
							Fit.Dom.GetFocused().blur();

							// Fire OnBlur manually as blur() above didn't trigger this, as it normally
							// would. The call to the dialog's hide() function fires its OnHide event
							// which disables the focus lock, but does so asynchronously, which is
							// why OnBlur does not fire via ControlBase's onfocusout handler.
							me._internal.FireOnBlur();
						}
						else
						{
							// A dialog (e.g. link or image dialog) is currently loading. This situation
							// can be triggered for debugging purposes by adding the following code in the
							// beforeCommandExec event handler:
							// setTimeout(function() { me.Focused(false); }, 0);
							// Alternatively register an onwheel/onscroll handler on the document that
							// removes focus from the control, and quickly scroll the document while the
							// dialog is loading. Use network throttling to increase the load time of the
							// dialog if necessary.

							// Make dialog close automatically when loaded and shown - handled in dialog's OnShow event handler
							Fit._internal.Controls.Input.ActiveDialogForEditorCanceled = true;

							// CKEditor instance has no blur() function, so we call blur() on DOM element currently focused within CKEditor.
							// Notice that OnBlur does not fire immediately (focus state is locked), but does so when dialog's OnHide event fires (async).
							// While we could fire it immediately and prevent it from firing when the dialog's OnHide event fires, it would prevent
							// developers from using the OnBlur event to dispose a control in Design Mode, since CKEditor fails when being disposed
							// while dialogs are open. Focused() will return False after the call to blur() below though - as expected.
							Fit.Dom.GetFocused().blur();
						}
					}
					else
					{
						if (designEditorActiveToolbarPanel !== null)
						{
							designEditorActiveToolbarPanel.CloseEmojiPanel(); // Returns focus to editor and nullifies designEditorActiveToolbarPanel
						}

						// Make sure this control is focused so that one control instance can not
						// be used to accidentially remove focus from another control instance.
						if (Fit.Dom.Contained(me.GetDomElement(), Fit.Dom.GetFocused()) === true)
						{
							// CKEditor instance has no blur() function, so we call blur() on DOM element currently focused within CKEditor
							Fit.Dom.GetFocused().blur();
						}
					}
				}
				else
				{
					elm.blur();
				}
			}
		}

		// Guard against disposed control in case Focused(false) was called and an OnBlur handler disposed the control.
		// As the code further up shows, we call me._internal.FireOnBlur() if a dialog is currently open. This results
		// in OnBlur firing immediately, while normally it happens asynchronously due to how OnFocus and OnBlur is handled
		// in ControlBase, in which case we do not need to worry that the control might be disposed. It happens "later".
		// The situation can easily arise if an OnScroll handler is reponsible for removing focus from a control,
		// and if that control also has an OnBlur handler registered which disposes the control. Scrolling with a
		// dialog open will then trigger the situation which we guard against here.
		if (me === null)
		{
			return false;
		}

		if (me.DesignMode() === true)
		{
			// If a dialog is open and it belongs to this control instance, and focus is found within dialog, then control is considered having focus.
			// However, if <body> is focused while dialog is open, control is also considered to have focus, since dialog temporarily assigns focus to
			// <body> when tabbing between elements within the dialog. This seems safe as no other control can be considered focused if <body> has focus.
			// We also consider the control focused if an associated dialog (modal) is currently loading (Fit._internal.Controls.Input.ActiveDialogForEditor is null).
			if (Fit._internal.Controls.Input.ActiveEditorForDialog === me && (Fit._internal.Controls.Input.ActiveDialogForEditor === null || Fit.Dom.Contained(Fit._internal.Controls.Input.ActiveDialogForEditor.getElement().$, Fit.Dom.GetFocused()) === true || Fit.Dom.GetFocused() === document.body))
				return true;

			// If a toolbar dialog/callout is open and contains the element currently having focus, then control is considered having focus.
			// If the dialog/callout contains an iframe in which an element has focus, then the iframe is considered focused in the main window.
			if (designEditorActiveToolbarPanel !== null && Fit.Dom.Contained(designEditorActiveToolbarPanel.DomElement, Fit.Dom.GetFocused()) === true)
				return true;

			return Fit.Dom.GetFocused() === me.GetDomElement() || Fit.Dom.Contained(me.GetDomElement(), Fit.Dom.GetFocused());
		}

		return (Fit.Dom.GetFocused() === elm);
	}

	// See documentation on ControlBase
	this.Value = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			val = me.Type() === "Color" ? val.toUpperCase() : val;

			var fireOnChange = (me.Value() !== val);

			orgVal = (preserveDirtyState !== true ? val : orgVal);
			preVal = val;
			designEditorDirty = designEditorDirtyPending === true ? true : false;
			designEditorDirtyPending = false;

			/*if (val.indexOf("<p>") === 0)
				htmlWrappedInParagraph = true; // Indicates that val is comparable with value from CKEditor which wraps content in paragraphs*/

			if (designModeEnabledAndReady() === true)
			{
				// NOTICE: Invalid HTML is removed, so an all invalid HTML string will be discarded
				// by the editor, resulting in the editor's getData() function returning an empty string.

				// Calling setData(..) fires CKEditor's onchange event which in turn fires
				// Input's OnChange event. Suppress OnChange which is fired further down.
				me._internal.ExecuteWithNoOnChange(function()
				{
					designEditorCleanEditableDom = true;
					CKEDITOR.instances[me.GetId() + "_DesignMode"].setData(val);
					designEditorCleanEditableDom = false;
				});

				updateDesignEditorPlaceholder();
				updateDesignEditorSize(); // In case auto grow is enabled, in which case editor must adjust its height to its new content
			}
			else
			{
				input.value = val;
			}

			// Notice: Identical logic is NOT found in DesignMode(true, config) as with RevokeExternalBlobUrlsOnDispose below.
			// When the RevokeUnreferencedBlobUrlsOnValueSet mechanism is in play, the control has already been used as an HTML editor
			// before, as we are expecting the control to be re(-used) to manipulate different values. In this case we already have
			// designEditorConfig available, although it could theoretically be changed over time so RevokeUnreferencedBlobUrlsOnValueSet
			// is sometimes enabled and sometimes not - but we don't care to support poor design like this:
			// input.DesignMode(true, configWithRevokeUnreferencedBlobUrlsOnValueSetDISBLED);
			// input.Value("New HTML value");
			// input.DesignMode(true, configWithRevokeUnreferencedBlobUrlsOnValueSetENABLED); // This will not clean up image blobs no longer referenced
			if (designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.RevokeUnreferencedBlobUrlsOnValueSet === true)
			{
				// Remove image blobs from memory when a new value is set, unless some (or all)
				// of these image blobs are still referenced in the new value, of course.
				// This is useful if an editor instance is being (re-)used to modify different values.
				// NOTICE: There is a major memory leak in CKEditor related to bulk pasting images
				// from the file system, and the last image pasted always remains in memory:
				// https://github.com/ckeditor/ckeditor4/issues/5124

				var blobUrlsReferenced = Fit.String.ParseImageBlobUrls(val);
				var newImageBlobUrls = [];

				Fit.Array.ForEach(imageBlobUrls, function(blobUrl)
				{
					if (Fit.Array.Contains(blobUrlsReferenced, blobUrl) === true)
					{
						newImageBlobUrls.push(blobUrl); // Keep - still referenced in new value
					}
					else
					{
						URL.revokeObjectURL(blobUrl); // Revoke - no longer referenced in new value
					}
				});

				imageBlobUrls = newImageBlobUrls;
			}

			// Notice: Identical logic found in DesignMode(true, config)!
			if (designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.RevokeExternalBlobUrlsOnDispose === true)
			{
				// Keep track of image blobs added via Value(..) so we can dispose of them automatically.
				// When RevokeExternalBlobUrlsOnDispose is True it basically means that the Input control
				// is allowed (and expected) to take control over memory management for these blobs
				// based on the rule set in RevokeBlobUrlsOnDispose.
				// This code is also found in DesignMode(true, config) since images might be added before
				// DesignMode is enabled, in which case we do not yet have the editor configuration needed
				// to determine the desired behaviour.
				// NOTICE: There is a major memory leak in CKEditor related to bulk pasting images
				// from the file system, and the last image pasted always remains in memory:
				// https://github.com/ckeditor/ckeditor4/issues/5124

				var blobUrls = Fit.String.ParseImageBlobUrls(val);

				Fit.Array.ForEach(blobUrls, function(blobUrl)
				{
					if (Fit.Array.Contains(imageBlobUrls, blobUrl) === false)
					{
						Fit.Array.Add(imageBlobUrls, blobUrl);
					}
				});
			}

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		if (designModeEnabledAndReady() === true)
		{
			// If user has not changed value, then return the value initially set.
			// CKEditor may change (optimize) HTML when applied, but we always want
			// the value initially set when no changes have been made by the user.
			// See additional comments regarding this in the IsDirty() implementation.
			if (designEditorDirty === false)
			{
				return orgVal;
			}

			var curVal = CKEDITOR.instances[me.GetId() + "_DesignMode"].getData();

			// Remove extra line break added by htmlwriter plugin at the end: <p>Hello world</p>\n
			curVal = curVal.replace(/<\/p>\n$/, "</p>");

			// Remove empty class attribute on <img> tags which may be temporarily set when selecting
			// an image using the dragresize plugin. This plugin adds a CSS class (ckimgrsz) to the image
			// tag while being selected, although the class name is removed when calling getData() above.
			// However, the empty class attribute is useless, so we remove it. It also results in IsDirty()
			// returning True while the image is selected if we keep it. Actually the class attribute should
			// never have been returned since the allowedContent option does not allow it - might be a minor bug.
			curVal = curVal.replace(/(<img.*?) class=""(.*?>)/, "$1$2"); // Not using /g switch as only one image can be selected

			return curVal;
		}

		return me.Type() === "Color" ? input.value.toUpperCase() : input.value;
	}

	// See documentation on ControlBase
	this.UserValue = Fit.Core.CreateOverride(this.UserValue, function(val)
	{
		if (Fit.Validation.IsSet(val) === true && me.DesignMode() === true)
		{
			designEditorDirtyPending = true;
		}

		return base(val);
	});

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		if (me.DesignMode() === true)
		{
			// Never do value comparison in DesignMode.
			// A value such as "Hello world" could have been provided,
			// which by CKEditor would be returned as "<p>Hello world</p>".
			// A value such as '<p style="text-align: center;">Hello</p>' could
			// also have been set, which by CKEditor would be optimized to
			// '<p style="text-align:center">Hello</p>' via ACF (Advanced Content Filter):
			// https://ckeditor.com/docs/ckeditor4/latest/guide/dev_advanced_content_filter.html
			// Furthermore invalid HTML is removed while valid HTML is kept.
			// All this makes it very difficult to reliably determine dirty state
			// by comparing values. Therefore, if the user changed anything by interacting
			// with the editor, or UserValue(..) was called, always consider the value dirty.

			// Another positive of avoiding value comparison to determine dirty state
			// is that retrieving the value from CKEditor is fairly expensive.

			return designEditorDirty;
		}

		return (orgVal !== me.Value());
	}

	// See documentation on ControlBase
	this.Clear = function()
	{
		me.Value("");
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		// This will destroy control - it will no longer work!

		if (me.DesignMode() === true && designModeEnabledAndReady() === false) // DesignMode is enabled but editor is not done loading/initializing
		{
			// WARNING: This has the potential to leak memory if editor never loads and resumes task of disposing control!
			designEditorMustDisposeWhenReady = true;

			// Editor was disposed while loading/initializing.
			// Postpone destruction of control to make sure we can clean up resources
			// reliably once editor is ready. We know that CKEditor does not dispose properly
			// unless fully loaded (it may leave an instance on the global CKEDITOR object or even fail).
			Fit.Browser.Debug("WARNING: Attempting to dispose Input control '" + me.GetId() + "' while initializing DesignMode! Control will be disposed later.");

			// Do not keep control in user interface when disposed.
			// Mount control in document root and move it off screen (outside visible
			// viewport area). CKEditor will fail initialization if not mounted in DOM.
			me.Render(document.body);
			me.GetDomElement().style.position = "fixed";
			me.GetDomElement().style.left = "0px";
			me.GetDomElement().style.bottom = "-100px";
			me.GetDomElement().style.maxHeight = "100px";

			// Detect memory leak
			/* setTimeout(function()
			{
				if (me !== null)
				{
					Fit.Browser.Debug("WARNING: Input in DesignMode was not properly disposed in time - potential memory leak detected");
				}
			}, 5000); // Usually the load time for an editor is barely measurable, so 5 seconds seems sufficient */

			return;
		}

		var curVal = designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.RevokeBlobUrlsOnDispose === "UnreferencedOnly" ? me.Value() : null;

		if (Fit._internal.Controls.Input.ActiveEditorForDialog === me)
		{
			if (Fit._internal.Controls.Input.ActiveDialogForEditor === null)
			{
				// Dialog is currently loading.
				// CKEditor will throw an error if disposed while a dialog (e.g. the link dialog) is loading,
				// leaving a modal layer on the page behind, making it unusable. This may happen if disposed
				// from e.g. a DOM event handler, a mutation observer, a timer, or an AJAX request. The input control
				// itself does not fire any events while the dialog is loading which could trigger this situation, so
				// this can only happen from "external code".

				// WARNING: This has the potential to leak memory if dialog never loads and resumes task of disposing control!
				Fit._internal.Controls.Input.ActiveEditorForDialogDestroyed = designEditor;
				Fit.Dom.Remove(me.GetDomElement());

				// Detect memory leak
				/* setTimeout(function()
				{
					if (me !== null)
					{
						Fit.Browser.Log("WARNING: Input in DesignMode was not properly disposed in time - potential memory leak detected");
					}
				}, 5000); // Usually the load time for a dialog is barely measurable, so 5 seconds seems sufficient */

				return;
			}
			else
			{
				Fit._internal.Controls.Input.ActiveDialogForEditor.hide(); // Fires dialog's OnHide event
			}
		}

		if (designEditor !== null)
		{
			// Destroying editor also fires OnHide event for any dialog currently open, which will clean up
			// Fit._internal.Controls.Input.ActiveEditorForDialog;
			// Fit._internal.Controls.Input.ActiveEditorForDialogDestroyed;
			// Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed;
			// Fit._internal.Controls.Input.ActiveDialogForEditor;
			// Fit._internal.Controls.Input.ActiveDialogForEditorCanceled;

			destroyDesignEditorInstance(); // Destroys editor and stops related mutation observers, timers, etc.
		}

		if (changeObserverId !== -1)
		{
			clearInterval(changeObserverId);
		}

		Fit.Internationalization.RemoveOnLocaleChanged(localize);

		/*if (designEditorUpdateSizeDebouncer !== -1)
		{
			clearTimeout(designEditorUpdateSizeDebouncer);
		}

		if (mutationObserverId !== -1)
		{
			Fit.Events.RemoveMutationObserver(mutationObserverId);
		}

		if (rootedEventId !== -1)
		{
			Fit.Events.RemoveHandler(me.GetDomElement(), rootedEventId);
		}

		if (createWhenReadyIntervalId !== -1)
		{
			clearInterval(createWhenReadyIntervalId);
		}*/

		if (debouncedOnChange !== null)
		{
			debouncedOnChange.Cancel();
		}

		if (designEditorConfig === null || !designEditorConfig.Plugins || !designEditorConfig.Plugins.Images || !designEditorConfig.Plugins.Images.RevokeBlobUrlsOnDispose || designEditorConfig.Plugins.Images.RevokeBlobUrlsOnDispose === "All")
		{
			Fit.Array.ForEach(imageBlobUrls, function(imageUrl)
			{
				URL.revokeObjectURL(imageUrl);
			});
		}
		else // UnreferencedOnly
		{
			Fit.Array.ForEach(imageBlobUrls, function(imageUrl)
			{
				if (curVal.match(new RegExp("img.*src=([\"'])" + imageUrl + "\\1", "i")) === null)
				{
					URL.revokeObjectURL(imageUrl);
				}
			});
		}

		me = orgVal = preVal = input = changeObserverId = cmdResize = designEditor = designEditorDom = designEditorDirty = designEditorDirtyPending = designEditorConfig = designEditorReloadConfig = designEditorRestoreButtonState = designEditorSuppressPaste = designEditorSuppressOnResize = designEditorMustReloadWhenReady = designEditorMustDisposeWhenReady = designEditorUpdateSizeDebouncer = designEditorHeightMonitorId = designEditorActiveToolbarPanel = designEditorDetached = designEditorClearPlaceholder = designEditorCleanEditableDom = designEditorGlobalKeyDownEventId = designEditorGlobalKeyUpEventId /*= htmlWrappedInParagraph*/ = wasAutoChangedToMultiLineMode = minimizeHeight = maximizeHeight = minMaxUnit = maximizeHeightConfigured = resizable = nativeResizableAvailable = mutationObserverId = rootedEventId = createWhenReadyIntervalId = isIe8 = debounceOnChangeTimeout = debouncedOnChange = imageBlobUrls = locale = null;

		base();
	});

	// See documentation on ControlBase
	this.Width = Fit.Core.CreateOverride(this.Width, function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			me._internal.Data("resized", "false");

			base(val, unit);
			updateDesignEditorSize();
		}

		return base();
	});

	// See documentation on ControlBase
	this.Height = Fit.Core.CreateOverride(this.Height, function(val, unit, suppressMinMax)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);
		Fit.Validation.ExpectBoolean(suppressMinMax, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			// Restore/minimize control if currently maximized
			if (me.Maximizable() === true && suppressMinMax !== true)
			{
				me.Maximized(false);
			}

			me._internal.Data("resized", "false");
			me._internal.Data("autogrow", me.DesignMode() === true ? "false" : null);

			var autoGrowEnabled = false;
			if (val === -1 && designModeEnabledAndReady() === true) // Enable auto grow if editor is loaded and ready - otherwise enabled in instanceReady handler
			{
				// A value of -1 is used to reset control height (assume default height).
				// In DesignMode we want the control height to adjust to the content of the editor in this case.
				// The editor's ability to adjust to the HTML content is handled in updateDesignEditorSize() below.
				// Auto grow can also be enabled using configuration object passed to DesignMode(true, config).
				me._internal.Data("autogrow", "true"); // Make control container adjust to editor's height
				autoGrowEnabled = true;
			}

			// Enable support for relative height if editor is loaded and ready - otherwise enabled in instanceReady handler
			enableDesignEditorHeightMonitor(val !== -1 && unit === "%" && designModeEnabledAndReady() === true);

			var hideToolbarAgain = false;
			if (isToolbarHiddenInDesignEditor() === true)
			{
				// If in DesignMode, temporarily restore toolbar to allow update to height.
				// When toolbar is hidden, a fixed height is set on the editable area which
				// prevent changes to control height.
				restoreHiddenToolbarInDesignEditor();
				hideToolbarAgain = true;
			}

			var h = base(val, unit);
			updateDesignEditorSize();

			if (hideToolbarAgain === true)
			{
				hideToolbarInDesignMode();
			}

			// Calculate new maximize height if control is maximizable
			if (me.Maximizable() === true && suppressMinMax !== true)
			{
				minimizeHeight = h.Value;
				maximizeHeight = (maximizeHeightConfigured !== -1 ? maximizeHeightConfigured : (minimizeHeight !== -1 ? minimizeHeight * 2 : 300));
				minMaxUnit = h.Unit;
			}

			if (autoGrowEnabled === true) // Repaint in case auto grow was enabled above
			{
				repaint();
			}
		}

		return base();
	});

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.Input" name="Placeholder" access="public" returns="string">
	/// 	<description> Get/set value used as a placeholder to indicate expected input on supported browsers </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, value is set as placeholder </param>
	/// </function>
	this.Placeholder = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			input.placeholder = val;
			updateDesignEditorPlaceholder();
		}

		return (input.placeholder ? input.placeholder : "");
	}

	/// <function container="Fit.Controls.Input" name="CheckSpelling" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control should have spell checking enabled (default) or disabled </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, true enables spell checking while false disables it </param>
	/// </function>
	this.CheckSpelling = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val !== input.spellcheck)
			{
				input.spellcheck = val;

				if (me.DesignMode() === true)
				{
					reloadEditor();
				}
			}
		}

		return input.spellcheck;
	}

	/// <function container="Fit.Controls.Input" name="Type" access="public" returns="Fit.Controls.InputType">
	/// 	<description> Get/set input type (e.g. Text, Password, Email, etc.) </description>
	/// 	<param name="val" type="Fit.Controls.InputType" default="undefined"> If defined, input type is changed to specified value </param>
	/// </function>
	this.Type = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (Fit.Validation.IsSet(Fit.Controls.InputType[val]) === false || val === Fit.Controls.InputType.Unknown)
				Fit.Validation.ThrowError("Unsupported input type specified - use e.g. Fit.Controls.InputType.Text");

			if (val === Fit.Controls.InputType.Textarea)
			{
				me.MultiLine(true);
			}
			else
			{
				me.MultiLine(false);

				if (val === Fit.Controls.InputType.Color)
					input.type = "color";
				else if (val === Fit.Controls.InputType.Date)
					input.type = "date";
				else if (val === Fit.Controls.InputType.DateTime)
					input.type = "datetime";
				else if (val === Fit.Controls.InputType.Email)
					input.type = "email";
				else if (val === Fit.Controls.InputType.Month)
					input.type = "month";
				else if (val === Fit.Controls.InputType.Number)
					input.type = "number";
				else if (val === Fit.Controls.InputType.Password)
					input.type = "password";
				else if (val === Fit.Controls.InputType.PhoneNumber)
					input.type = "tel";
				else if (val === Fit.Controls.InputType.Text)
					input.type = "text";
				else if (val === Fit.Controls.InputType.Time)
					input.type = "time";
				else if (val === Fit.Controls.InputType.Week)
					input.type = "week";
			}
		}

		if (me.MultiLine() === true || me.DesignMode() === true)
			return Fit.Controls.InputType.Textarea;
		else if (input.type === "color")
			return Fit.Controls.InputType.Color;
		else if (input.type === "date")
			return Fit.Controls.InputType.Date;
		else if (input.type === "datetime")
			return Fit.Controls.InputType.DateTime;
		else if (input.type === "email")
			return Fit.Controls.InputType.Email;
		else if (input.type === "month")
			return Fit.Controls.InputType.Month;
		else if (input.type === "number")
			return Fit.Controls.InputType.Number;
		else if (input.type === "password")
			return Fit.Controls.InputType.Password;
		else if (input.type === "tel")
			return Fit.Controls.InputType.PhoneNumber;
		else if (input.type === "text")
			return Fit.Controls.InputType.Text;
		else if (input.type === "time")
			return Fit.Controls.InputType.Time;
		else if (input.type === "week")
			return Fit.Controls.InputType.Week;

		return Fit.Controls.InputType.Unknown; // Only happens if someone changed the type to an unsupported value through the DOM (e.g. hidden or checkbox)
	}

	/// <function container="Fit.Controls.Input" name="MultiLine" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is in Multi Line mode (textarea) </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables Multi Line mode, False disables it </param>
	/// </function>
	this.MultiLine = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (me.DesignMode() === true && designModeEnabledAndReady() === false)
			{
				console.error("MultiLine(boolean) is not allowed for Input control '" + me.GetId() + "' while DesignMode editor is initializing!");
				return false; // Return current un-modified state - Input control is not in MultiLine mode when DesignMode is enabled
			}

			if (me.DesignMode() === true)
				me.DesignMode(false);

			if (val === true && wasAutoChangedToMultiLineMode === true)
			{
				wasAutoChangedToMultiLineMode = false;
			}

			if (val === true && input.tagName === "INPUT")
			{
				var focused = me.Focused();

				var oldInput = input;
				me._internal.RemoveDomElement(oldInput);

				input = document.createElement("textarea");
				input.value = oldInput.value;
				input.spellcheck = oldInput.spellcheck;
				input.placeholder = oldInput.placeholder;
				input.disabled = oldInput.disabled;
				input.onkeyup = oldInput.onkeyup;
				input.onchange = oldInput.onchange;
				me._internal.AddDomElement(input);

				if (nativeResizableAvailable === true)
				{
					Fit.Events.AddHandler(input, "mousemove", function(e)
					{
						var ev = Fit.Events.GetEvent(e);

						if (ev.buttons !== 1) // The .buttons property does not exist in older browsers (see nativeResizableAvailable)
						{
							return; // Skip - primary button not held down - not resizing
						}

						if (me.Resizable() !== Fit.Controls.InputResizing.Disabled && (input.style.width !== "" || input.style.height !== "")) // Textarea was resized
						{
							me._internal.Data("resized", "true");
						}
					});
				}

				if (focused === true)
					input.focus();

				me._internal.Data("multiline", "true");
				repaint();
			}
			else if (val === false && input.tagName === "TEXTAREA")
			{
				var focused = me.Focused();

				var oldInput = input;
				me._internal.RemoveDomElement(oldInput);

				if (cmdResize !== null)
				{
					me._internal.RemoveDomElement(cmdResize);
					cmdResize = null;

					me._internal.Data("maximized", "false");
					me._internal.Data("maximizable", "false");
					repaint();
				}
				else if (resizable !== Fit.Controls.InputResizing.Disabled)
				{
					resizable = Fit.Controls.InputResizing.Disabled;
					me._internal.Data("resizable", resizable.toLowerCase());
					me._internal.Data("resized", "false");
				}

				input = document.createElement("input");
				input.autocomplete = "off";
				input.type = "text";
				input.value = oldInput.value;
				input.spellcheck = oldInput.spellcheck;
				input.placeholder = oldInput.placeholder;
				input.disabled = oldInput.disabled;
				input.onkeyup = oldInput.onkeyup;
				input.onchange = oldInput.onchange;
				me._internal.AddDomElement(input);

				me.Height(-1);

				if (focused === true)
					input.focus();

				wasAutoChangedToMultiLineMode = false;

				me._internal.Data("multiline", "false");
				repaint();
			}
		}

		return (input.tagName === "TEXTAREA" && me.DesignMode() === false);
	}

	/// <function container="Fit.Controls.Input" name="Resizable" access="public" returns="Fit.Controls.InputResizing">
	/// 	<description>
	/// 		Get/set value indicating whether control is resizable on supported
	/// 		(modern) browsers. Making control resizable will disable Maximizable.
	/// 	</description>
	/// 	<param name="val" type="Fit.Controls.InputResizing" default="undefined">
	/// 		If defined, determines whether control resizes, and in what direction(s).
	/// 	</param>
	/// </function>
	this.Resizable = function(val)
	{
		Fit.Validation.ExpectStringValue(val, true);

		if (val === Fit.Controls.InputResizing.Enabled || val === Fit.Controls.InputResizing.Disabled || val === Fit.Controls.InputResizing.Horizontal || val === Fit.Controls.InputResizing.Vertical)
		{
			if (val !== resizable)
			{
				if (val !== Fit.Controls.InputResizing.Disabled) // Resizing enabled
				{
					if (me.Maximizable() === true)
					{
						me.Maximizable(false);
						//Fit.Browser.Log("Maximizable disabled as Resizable was enabled!");
					}

					if (me.MultiLine() === false && me.DesignMode() === false)
					{
						me.MultiLine(true);
						wasAutoChangedToMultiLineMode = true;
					}
				}
				else // Resizing disabled
				{
					// Reset custom width/height set by user

					var w = me.Width();
					me.Width(w.Value, w.Unit);

					var h = me.Height();
					me.Height(h.Value, h.Unit);
				}

				resizable = val;
				me._internal.Data("resizable", val.toLowerCase());

				if (val === Fit.Controls.InputResizing.Disabled)
				{
					me._internal.Data("resized", "false");

					input.style.width = "";
					input.style.height = "";
					input.style.margin = ""; // Chrome adds some odd margin when textarea is resized
				}

				revertToSingleLineIfNecessary();

				if (me.DesignMode() === true)
				{
					reloadEditor();
				}
			}
		}

		return resizable;
	}

	/// <function container="Fit.Controls.Input" name="Maximizable" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control is maximizable.
	/// 		Making control maximizable will disable Resizable.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables maximize button, False disables it </param>
	/// 	<param name="heightMax" type="number" default="undefined">
	/// 		If defined, this becomes the height of the input control when maximized.
	/// 		The value is considered the same unit set using Height(..) which defaults to px.
	/// 		If not set, the value assumes twice the height set using Height(..).
	/// 	</param>
	/// </function>
	this.Maximizable = function(val, heightMax)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectNumber(heightMax, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && cmdResize === null)
			{
				if (me.Resizable() !== Fit.Controls.InputResizing.Disabled)
				{
					me.Resizable(Fit.Controls.InputResizing.Disabled);
					//Fit.Browser.Log("Resizable disabled as Maximizable was enabled!");
				}

				if (me.MultiLine() === false && me.DesignMode() === false)
				{
					me.MultiLine(true);
					wasAutoChangedToMultiLineMode = true;
				}

				// Determine height to use when maximizing and minimizing

				var h = me.Height();

				minimizeHeight = h.Value;
				maximizeHeight = ((Fit.Validation.IsSet(heightMax) === true) ? heightMax : ((minimizeHeight !== -1) ? minimizeHeight * 2 : 300));
				minMaxUnit = h.Unit;
				maximizeHeightConfigured = heightMax || -1;

				// Create maximize/minimize button

				cmdResize = document.createElement("span");
				cmdResize.tabIndex = -1; // Allow button to temporarily gain focus so control does not fire OnBlur
				cmdResize.onclick = function()
				{
					me.Maximized(!me.Maximized());
					me.Focused(true);
				}
				Fit.Dom.AddClass(cmdResize, "fa");
				Fit.Dom.AddClass(cmdResize, "fa-chevron-down");
				me._internal.AddDomElement(cmdResize);

				// Update UI

				me._internal.Data("maximizable", "true");
				repaint();
			}
			else if (val === false && cmdResize !== null)
			{
				me._internal.RemoveDomElement(cmdResize);
				cmdResize = null;

				me.Height(minimizeHeight, minMaxUnit);
				minimizeHeight = -1;
				maximizeHeight = -1;
				minMaxUnit = null;

				me._internal.Data("maximizable", "false"); // Also set in MultiLine(..)

				revertToSingleLineIfNecessary();

				repaint();
			}
			else if (val === true && cmdResize !== null && Fit.Validation.IsSet(heightMax) === true)
			{
				// Already enabled - just update maximize height
				maximizeHeight = heightMax !== -1 ? heightMax : minimizeHeight * 2;
				maximizeHeightConfigured = heightMax;
			}
		}

		return (cmdResize !== null);
	}

	/// <function container="Fit.Controls.Input" name="Maximized" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether control is maximized </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True maximizes control, False minimizes it </param>
	/// </function>
	this.Maximized = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		var autoGrowEnabled = me.Height().Value === -1 && me.DesignMode() === true;

		if (Fit.Validation.IsSet(val) === true && me.Maximizable() === true && autoGrowEnabled === false)
		{
			if (val === true && Fit.Dom.HasClass(cmdResize, "fa-chevron-up") === false)
			{
				me.Height(maximizeHeight, minMaxUnit, true);
				Fit.Dom.RemoveClass(cmdResize, "fa-chevron-down");
				Fit.Dom.AddClass(cmdResize, "fa-chevron-up");

				me._internal.Data("maximized", "true");
				repaint();
			}
			else if (val === false && Fit.Dom.HasClass(cmdResize, "fa-chevron-down") === false)
			{
				me.Height(minimizeHeight, minMaxUnit, true);
				Fit.Dom.RemoveClass(cmdResize, "fa-chevron-up");
				Fit.Dom.AddClass(cmdResize, "fa-chevron-down");

				me._internal.Data("maximized", "false"); // Also set in MultiLine(..)
				repaint();
			}
		}

		return (cmdResize !== null && Fit.Dom.HasClass(cmdResize, "fa-chevron-up") === true);
	}

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigPluginsImagesConfig">
	/// 	<description> Configuration for image plugins </description>
	/// 	<member name="Enabled" type="boolean"> Flag indicating whether to enable image plugins or not (defaults to False) </member>
	/// 	<member name="EmbedType" type="'base64' | 'blob'" default="undefined">
	/// 		How to store and embed images. Base64 (default) is persistent while blob is temporary
	/// 		and must be extracted from memory and uploaded/stored to be permanantly persisted.
	/// 		References to blobs can be parsed from the HTML value produced by the editor.
	/// 	</member>
	/// 	<member name="RevokeBlobUrlsOnDispose" type="'All' | 'UnreferencedOnly'" default="undefined">
	/// 		This option is in effect when EmbedType is blob.
	/// 		Dispose images from blob storage (revoke blob URLs) added though image plugins when control is disposed.
	/// 		If "UnreferencedOnly" is specified, the component using Fit.UI's input control will be responsible for
	/// 		disposing referenced blobs. Failing to do so may cause a memory leak. Defaults to All.
	/// 	</member>
	/// 	<member name="RevokeExternalBlobUrlsOnDispose" type="boolean" default="undefined">
	/// 		This option is in effect when EmbedType is blob.
	/// 		Dispose images from blob storage (revoke blob URLs) added through Value(..)
	/// 		function when control is disposed. Basically ownership of these blobs are handed
	/// 		over to the control for the duration of its life time.
	/// 		These images are furthermore subject to the rule set in RevokeBlobUrlsOnDispose.
	/// 		Defaults to False.
	/// 	</member>
	/// 	<member name="RevokeUnreferencedBlobUrlsOnValueSet" type="boolean" default="undefined">
	/// 		This option is in effect when EmbedType is blob.
	/// 		Dispose images from blob storage (revoke blob URLs) when value is changed with Value(..),
	/// 		but keep any images still referenced in new value. This is useful if an editor instance
	/// 		is being used to modify different HTML values over time.
	/// 	</member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigPluginsTablesConfig">
	/// 	<description> Configuration for table plugins </description>
	/// 	<member name="Enabled" type="boolean"> Flag indicating whether to enable table plugins or not (defaults to False) </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigPlugins">
	/// 	<description> Additional plugins enabled in DesignMode </description>
	/// 	<member name="Emojis" type="boolean" default="undefined"> Plugin(s) related to emoji support (defaults to False) </member>
	/// 	<member name="Images" type="Fit.Controls.InputTypeDefs.DesignModeConfigPluginsImagesConfig" default="undefined"> Plugin(s) related to support for images (not enabled by default) </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigToolbar">
	/// 	<description> Toolbar buttons enabled in DesignMode </description>
	/// 	<member name="Formatting" type="boolean" default="undefined"> Enable text formatting (bold, italic, underline) (defaults to True) </member>
	/// 	<member name="Justify" type="boolean" default="undefined"> Enable text alignment (defaults to True) </member>
	/// 	<member name="Lists" type="boolean" default="undefined"> Enable ordered and unordered lists with indentation (defaults to True) </member>
	/// 	<member name="Links" type="boolean" default="undefined"> Enable links (defaults to True) </member>
	/// 	<member name="Emojis" type="boolean" default="undefined"> Enable emoji button (defaults to False) </member>
	/// 	<member name="Images" type="boolean" default="undefined"> Enable image button (defaults to false) </member>
	/// 	<member name="Tables" type="boolean" default="undefined"> Enable table button (defaults to false) </member>
	/// 	<member name="Detach" type="boolean" default="undefined"> Enable detach button (defaults to false) </member>
	/// 	<member name="Position" type="'Top' | 'Bottom'" default="undefined"> Toolbar position (defaults to Top) </member>
	/// 	<member name="Sticky" type="boolean" default="undefined"> Make toolbar stick to edge of scroll container on supported browsers when scrolling (defaults to False) </member>
	/// 	<member name="HideWhenInactive" type="boolean" default="undefined"> Hide toolbar when control is inactive (defaults to False) </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigInfoPanel">
	/// 	<description> Information panel at the top or bottom of the editor, depending on the location of the toolbar </description>
	/// 	<member name="Text" type="string" default="undefined"> Text to display </member>
	/// 	<member name="Alignment" type="'Left' | 'Center' | 'Right'" default="undefined"> Text alignment - defaults to Center </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeTagsOnRequestEventHandlerArgs">
	/// 	<description> Request handler event arguments </description>
	/// 	<member name="Sender" type="Fit.Controls.Input"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonRequest | Fit.Http.JsonpRequest"> Instance of JsonRequest or JsonpRequest </member>
	/// 	<member name="Query" type="{ Marker: string, Query: string }"> Query information </member>
	/// </container>
	/// <function container="Fit.Controls.InputTypeDefs" name="DesignModeTagsOnRequest" returns="boolean | void">
	/// 	<description> Cancelable request event handler </description>
	/// 	<param name="sender" type="Fit.Controls.Input"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnRequestEventHandlerArgs"> Event arguments </param>
	/// </function>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponseJsonTag">
	/// 	<description> JSON object representing tag </description>
	/// 	<member name="Value" type="string"> Unique value </member>
	/// 	<member name="Title" type="string"> Title </member>
	/// 	<member name="Icon" type="string" default="undefined"> Optional URL to icon/image </member>
	/// 	<member name="Url" type="string" default="undefined"> Optional URL to associate with tag </member>
	/// 	<member name="Data" type="string" default="undefined"> Optional data to associate with tag </member>
	/// 	<member name="Context" type="string" default="undefined"> Optional context information to associate with tag </member>
	/// </container>
	/// <container name="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponseEventHandlerArgs">
	/// 	<description> Response handler event arguments </description>
	/// 	<member name="Sender" type="Fit.Controls.Input"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonRequest | Fit.Http.JsonpRequest"> Instance of JsonRequest or JsonpRequest </member>
	/// 	<member name="Query" type="{ Marker: string, Query: string }"> Query information </member>
	/// 	<member name="Tags" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponseJsonTag[]"> Tags received from WebService </member>
	/// </container>
	/// <function container="Fit.Controls.InputTypeDefs" name="DesignModeTagsOnResponse">
	/// 	<description> Response event handler </description>
	/// 	<param name="sender" type="Fit.Controls.Input"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponseEventHandlerArgs"> Event arguments </param>
	/// </function>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeTagsTagCreatorReturnType">
	/// 	<description> JSON object representing tag to be inserted into editor </description>
	/// 	<member name="Title" type="string"> Tag title </member>
	/// 	<member name="Value" type="string"> Tag value (ID) </member>
	/// 	<member name="Type" type="string"> Tag type (marker) </member>
	/// 	<member name="Url" type="string" default="undefined"> Optional tag URL </member>
	/// 	<member name="Data" type="string" default="undefined"> Optional tag data </member>
	/// 	<member name="Context" type="string" default="undefined"> Optional tag context </member>
	/// </container>
	/// <container name="Fit.Controls.InputTypeDefs.DesignModeTagsTagCreatorCallbackArgs">
	/// 	<description> TagCreator event arguments </description>
	/// 	<member name="Sender" type="Fit.Controls.Input"> Instance of control </member>
	/// 	<member name="QueryMarker" type="string"> Query marker </member>
	/// 	<member name="Tag" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponseJsonTag"> Tag received from WebService </member>
	/// </container>
	/// <function container="Fit.Controls.InputTypeDefs" name="DesignModeTagsTagCreator" returns="Fit.Controls.InputTypeDefs.DesignModeTagsTagCreatorReturnType | null | void">
	/// 	<description>
	/// 		Function producing JSON object representing tag to be inserted into editor.
	/// 		Returning nothing or Null results in default tag being inserted into editor.
	/// 	</description>
	/// 	<param name="sender" type="Fit.Controls.Input"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.InputTypeDefs.DesignModeTagsTagCreatorCallbackArgs"> Event arguments </param>
	/// </function>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfigTags">
	/// 	<description> Configuration for tags in DesignMode </description>
	/// 	<member name="Triggers" type="{ Marker: string, MinimumCharacters?: integer, DebounceQuery?: integer, Pattern?: RegExp }[]">
	/// 		Markers triggering tags request and context menu.
	/// 		Notice that Pattern, if specified, must include the marker for match to occur,
	/// 		as well as specifying the minimum amount of characters - e.g. /^@[a-z]{3,}$/
	/// 	</member>
	/// 	<member name="QueryUrl" type="string">
	/// 		URL to request data from. Endpoint receives the following payload:
	/// 		{ Marker: "@", Query: "search" }
	///
	/// 		Data is expected to be returned in the following format:
	/// 		[
	/// 		    { Value: "t-1", Title: "Tag 1", Icon: "images/img1.jpeg", Url: "show/1", Data: "..." },
	/// 		    { Value: "t-2", Title: "Tag 2", Icon: "images/img2.jpeg", Url: "show/2", Data: "..." }, ...
	/// 		]
	///
	/// 		The Value and Title properties are required. The Icon property is optional and must specify the path to an image.
	/// 		The Url property is optional and must specify a path to a related page/resource.
	/// 		The Data property is optional and allows for additional data to be associated with the tag.
	/// 		To hold multiple values, consider using a base64 encoded JSON object:
	/// 		btoa(JSON.stringify({ creationDate: new Date(), active: true }))
	///
	/// 		The data eventuelly results in a tag being added to the editor with the following format:
	/// 		<a data-tag-type="@" data-tag-id="unique id 1" data-tag-data="..." data-tag-context="..." href="show/1">Tag name 1</a>
	/// 		The data-tag-data and data-tag-context attributes are only declared if the corresponding Data and Context properties are defined in data.
	/// 	</member>
	/// 	<member name="JsonpCallback" type="string" default="undefined"> Name of URL parameter receiving name of JSONP callback function (only for JSONP services) </member>
	/// 	<member name="JsonpTimeout" type="integer" default="undefined"> Number of milliseconds to allow JSONP request to wait for a response before aborting (only for JSONP services) </member>
	/// 	<member name="OnRequest" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnRequest" default="undefined">
	/// 		Event handler invoked when tags are requested. Request may be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.Input) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.Input instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Query: Contains query information in its Marker and Query property
	/// 	</member>
	/// 	<member name="OnResponse" type="Fit.Controls.InputTypeDefs.DesignModeTagsOnResponse" default="undefined">
	/// 		Event handler invoked when tags data is received, allowing for data transformation.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.Input) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.Input instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Query: Contains query information in its Marker and Query property
	/// 		 - Tags: JSON tags array received from WebService
	/// 	</member>
	/// 	<member name="TagCreator" type="Fit.Controls.InputTypeDefs.DesignModeTagsTagCreator" default="undefined">
	/// 		Callback invoked when a tag is being inserted into editor, allowing
	/// 		for customization to the title and attributes associated with the tag.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.Input) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.Input instance
	/// 		 - QueryMarker: String containing query marker
	/// 		 - Tag: JSON tag received from WebService
	/// 	</member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeAutoGrow">
	/// 	<description> Auto grow configuration </description>
	/// 	<member name="Enabled" type="boolean"> Flag indicating whether auto grow feature is enabled or not - on by default if no height is set, or if Height(-1) is set </member>
	/// 	<member name="MinimumHeight" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Minimum height of editable area </member>
	/// 	<member name="MaximumHeight" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Maximum height of editable area </member>
	/// 	<member name="PreventResizeBeyondMaximumHeight" type="boolean" default="undefined"> Prevent user from resizing editor beyond maximum height (see MaximumHeight property - defaults to False) </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeDetachable">
	/// 	<description> Detachable configuration </description>
	/// 	<member name="Title" type="string" default="undefined"> Dialog title </member>
	/// 	<member name="Maximizable" type="boolean" default="undefined"> Flag indicating whether dialog is maximizable </member>
	/// 	<member name="Maximized" type="boolean" default="undefined"> Flag indicating whether dialog is initially maximized </member>
	/// 	<member name="Draggable" type="boolean" default="undefined"> Flag indicating whether dialog is draggable </member>
	/// 	<member name="Resizable" type="boolean" default="undefined"> Flag indicating whether dialog is resizable </member>
	/// 	<member name="Width" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Dialog width </member>
	/// 	<member name="MinimumWidth" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Minimum width of dialog </member>
	/// 	<member name="MaximumWidth" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Maximum Width of dialog </member>
	/// 	<member name="Height" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Dialog height </member>
	/// 	<member name="MinimumHeight" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Minimum height of dialog </member>
	/// 	<member name="MaximumHeight" type="{ Value: number, Unit?: Fit.TypeDefs.CssUnit }" default="undefined"> Maximum height of dialog </member>
	/// </container>

	/// <container name="Fit.Controls.InputTypeDefs.DesignModeConfig">
	/// 	<description> Configuration for DesignMode </description>
	/// 	<member name="Plugins" type="Fit.Controls.InputTypeDefs.DesignModeConfigPlugins" default="undefined"> Plugins configuration </member>
	/// 	<member name="Toolbar" type="Fit.Controls.InputTypeDefs.DesignModeConfigToolbar" default="undefined"> Toolbar configuration </member>
	/// 	<member name="InfoPanel" type="Fit.Controls.InputTypeDefs.DesignModeConfigInfoPanel" default="undefined"> Information panel configuration </member>
	/// 	<member name="Tags" type="Fit.Controls.InputTypeDefs.DesignModeConfigTags" default="undefined"> Tags configuration </member>
	/// 	<member name="AutoGrow" type="Fit.Controls.InputTypeDefs.DesignModeAutoGrow" default="undefined"> Auto grow configuration </member>
	/// 	<member name="Detachable" type="Fit.Controls.InputTypeDefs.DesignModeDetachable" default="undefined"> Detachable configuration </member>
	/// </container>

	/// <function container="Fit.Controls.Input" name="DesignMode" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether control is in Design Mode allowing for rich HTML content.
	/// 		Notice that this control type requires dimensions (Width/Height) to be specified in pixels.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables Design Mode, False disables it </param>
	/// 	<param name="editorConfig" type="Fit.Controls.InputTypeDefs.DesignModeConfig" default="undefined">
	/// 		If provided and DesignMode is enabled, configuration is applied when editor is created.
	/// 	</param>
	/// </function>
	this.DesignMode = function(val, editorConfig)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectObject(editorConfig, true);
		Fit.Validation.ExpectObject((editorConfig || {}).Plugins, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Plugins || {}).Emojis, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Plugins || {}).Images, true);
		Fit.Validation.ExpectBoolean((((editorConfig || {}).Plugins || {}).Images || {}).Enabled, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Plugins || {}).Images || {}).EmbedType, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Plugins || {}).Images || {}).RevokeBlobUrlsOnDispose, true);
		Fit.Validation.ExpectBoolean((((editorConfig || {}).Plugins || {}).Images || {}).RevokeExternalBlobUrlsOnDispose, true);
		Fit.Validation.ExpectObject((editorConfig || {}).Toolbar, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Formatting, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Justify, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Lists, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Links, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Emojis, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Images, true);
		Fit.Validation.ExpectStringValue(((editorConfig || {}).Toolbar || {}).Position, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Sticky, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).HideWhenInactive, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Toolbar || {}).Detach, true);
		Fit.Validation.ExpectObject((editorConfig || {}).InfoPanel, true);
		Fit.Validation.ExpectString(((editorConfig || {}).InfoPanel || {}).Text, true);
		Fit.Validation.ExpectString(((editorConfig || {}).InfoPanel || {}).Alignment, true);
		Fit.Validation.ExpectObject((editorConfig || {}).Tags, true);
		Fit.Validation.ExpectObject((editorConfig || {}).AutoGrow, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).AutoGrow || {}).Enabled, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).AutoGrow || {}).MinimumHeight, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).AutoGrow || {}).MinimumHeight || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).AutoGrow || {}).MinimumHeight || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).AutoGrow || {}).MaximumHeight, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).AutoGrow || {}).MaximumHeight || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).AutoGrow || {}).MaximumHeight || {}).Unit, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).AutoGrow || {}).PreventResizeBeyondMaximumHeight, true);
		Fit.Validation.ExpectObject((editorConfig || {}).Detachable, true);
		Fit.Validation.ExpectString(((editorConfig || {}).Detachable || {}).Title, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Detachable || {}).Maximizable, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Detachable || {}).Maximized, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Detachable || {}).Draggable, true);
		Fit.Validation.ExpectBoolean(((editorConfig || {}).Detachable || {}).Resizable, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).Width, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).Width || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).Width || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).MinimumWidth, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).MinimumWidth || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).MinimumWidth || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).MaximumWidth, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).MaximumWidth || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).MaximumWidth || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).Height, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).Height || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).Height || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).MinimumHeight, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).MinimumHeight || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).MinimumHeight || {}).Unit, true);
		Fit.Validation.ExpectObject(((editorConfig || {}).Detachable || {}).MaximumHeight, true);
		Fit.Validation.ExpectNumber((((editorConfig || {}).Detachable || {}).MaximumHeight || {}).Value, true);
		Fit.Validation.ExpectStringValue((((editorConfig || {}).Detachable || {}).MaximumHeight || {}).Unit, true);

		if (editorConfig && editorConfig.Tags)
		{
			Fit.Validation.ExpectTypeArray(editorConfig.Tags.Triggers, function(trigger)
			{
				Fit.Validation.ExpectStringValue(trigger.Marker);
				Fit.Validation.ExpectInteger(trigger.MinimumCharacters, true);
				Fit.Validation.ExpectInteger(trigger.DebounceQuery, true);
				Fit.Validation.ExpectInstance(trigger.Pattern, RegExp, true);
			});
			Fit.Validation.ExpectStringValue(editorConfig.Tags.QueryUrl);
			Fit.Validation.ExpectStringValue(editorConfig.Tags.JsonpCallback, true);
			Fit.Validation.ExpectInteger(editorConfig.Tags.JsonpTimeout, true);
			Fit.Validation.ExpectFunction(editorConfig.Tags.OnRequest, true);
			Fit.Validation.ExpectFunction(editorConfig.Tags.OnResponse, true);
			Fit.Validation.ExpectFunction(editorConfig.Tags.TagCreator, true);
		}

		if (Fit.Validation.IsSet(val) === true)
		{
			var designMode = (me._internal.Data("designmode") === "true");

			if (Fit._internal.Controls.Input.ActiveEditorForDialog === me && Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed === true)
				designMode = false; // Not considered in Design Mode if scheduled to be disabled (postponed because a dialog is currently loading)

			if ((val === true && designMode === false) || (val === true && Fit.Validation.IsSet(editorConfig) === true && Fit.Core.IsEqual(editorConfig, designEditorConfig) === false))
			{
				var configUpdated = designMode === true; // Already in DesignMode which means editorConfig was changed

				if (Fit._internal.Controls.Input.ActiveEditorForDialog === me)
				{
					// Control is actually already in Design Mode, but waiting
					// for dialog to finish loading, so DesignMode can be disabled (scheduled).
					// Remove flag responsible for disabling DesignMode so it remains an editor.
					delete Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed;

					if (configUpdated === false)
					{
						return true;
					}
				}

				if (configUpdated === true)
				{
					reloadEditor(false, Fit.Core.Clone(editorConfig)); // Clone to prevent external code from making changes later
					return true;
				}

				if (Fit.Validation.IsSet(editorConfig) === true)
				{
					designEditorConfig = Fit.Core.Clone(editorConfig); // Clone to prevent external code from making changes later
				}

				// Notice: Identical logic found in Value(..)!
				if (designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.RevokeExternalBlobUrlsOnDispose === true)
				{
					// Keep track of image blobs added via Value(..) so we can dispose of them automatically.
					// When RevokeExternalBlobUrlsOnDispose is True it basically means that the Input control
					// is allowed (and expected) to take control over memory management for these blobs
					// based on the rule set in RevokeBlobUrlsOnDispose.
					// This code is also found in Value(..) since images might be added after editor has been created.

					var blobUrls = Fit.String.ParseImageBlobUrls(me.Value());

					Fit.Array.ForEach(blobUrls, function(blobUrl)
					{
						if (Fit.Array.Contains(imageBlobUrls, blobUrl) === false)
						{
							Fit.Array.Add(imageBlobUrls, blobUrl);
						}
					});
				}

				if (me.MultiLine() === false)
				{
					me.MultiLine(true);
					wasAutoChangedToMultiLineMode = true;
				}

				me._internal.Data("designmode", "true");
				me._internal.Data("toolbar", designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.HideWhenInactive === true ? "false" : "true");
				me._internal.Data("toolbar-position", designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.Position === "Bottom" ? "bottom" : "top");
				me._internal.Data("toolbar-sticky", designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.Sticky === true ? "true" : "false");

				// Prevent control from losing focus when HTML editor is initialized,
				// e.g. if Design Mode is enabled when ordinary input control gains focus.
				// This also prevents control from losing focus if toolbar is clicked without
				// hitting a button. A value of -1 makes it focusable, but keeps it out of
				// tab flow (keyboard navigation). Also set when Enabled(true) is called.
				me.GetDomElement().tabIndex = -1; // TabIndex is removed if DesignMode is disabled (DesignMode(false)) or if control is disabled (Enabled(false))

				if (me.Focused() === true)
				{
					// Move focus from input to control's outer container (tabindex
					// set above) to keep focus while editor is loading/initializing.
					// Focus is moved to editor once initialization is complete - see
					// instanceReady handler.
					me.GetDomElement().focus();
				}

				input.id = me.GetId() + "_DesignMode";

				if (window.CKEDITOR === undefined)
				{
					window.CKEDITOR = null;

					Fit.Loader.LoadScript(Fit.GetUrl() + "/Resources/CKEditor/ckeditor.js?cacheKey=" + Fit.GetVersion().Version, function(src) // Using Fit.GetUrl() rather than Fit.GetPath() to allow editor to be used on e.g. JSFiddle (Cross-Origin Resource Sharing policy)
					{
						// WARNING: Control could potentially have been disposed at this point, but
						// we still need to finalize the configuration of CKEditor which is global.

						// Prevent CKEditor from automatically converting editable elements to inline editors.
						// https://ckeditor.com/docs/ckeditor4/latest/api/CKEDITOR.html#cfg-disableAutoInline
						CKEDITOR.disableAutoInline = true;

						if (Fit.Validation.IsSet(Fit._internal.Controls.Input.Editor.Skin) === true)
						{
							CKEDITOR.config.skin = Fit._internal.Controls.Input.Editor.Skin;
						}

						CKEDITOR.on("instanceReady", function(ev)
						{
							// Do not produce XHTML self-closing tags such as <br /> and <img src="img.jpg" />
							// https://ckeditor.com/docs/ckeditor4/latest/features/output_format.html
							// NOTICE: The htmlwriter plugin is required for this to work!
							// Output produced is now both HTML4 and HTML5 compatible, but is not valid
							// XHTML anymore! Self-closing tags are allowed in HTML5 but not valid in HTML4.
							ev.editor.dataProcessor.writer.selfClosingEnd = ">"; // Defaults to ' />'
						});

						// Register OnShow and OnHide event handlers when a dialog is opened for the first time.
						// IMPORTANT: These event handlers are shared by all input control instances in Design Mode,
						// so we cannot use 'me' to access the current control for which a dialog is opened.
						// Naturally 'me' will always be a reference to the first control that opened a given dialog.
						CKEDITOR.on("dialogDefinition", function(e) // OnDialogDefinition fires only once
						{
							var dialogName = e.data.name;
							var dialogDef = e.data.definition;

							if (dialogName === "table")
							{
								// Remove default table width (500px).
								// Allow table width to adjust to content.
								dialogDef.getContents("info").get("txtWidth").default = "";
							}

							var dialog = dialogDef.dialog;

							dialog.on("show", function(ev)
							{
								if (Fit._internal.Controls.Input.ActiveDialogForEditorCanceled)
								{
									// Focused(false) was called on control while dialog was loading - close dialog

									if (Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() < 9)
									{
										// CKEditor uses setTimeout(..) to focus an input field in the dialog, but if the dialog is
										// closed immediately, that input field will be removed from DOM along with the dialog of course,
										// which in IE8 results in an error:
										// "Can't move focus to the control because it is invisible, not enabled, or of a type that does not accept the focus."
										// Other browsers simply ignore the request to focus a control that is no longer found in DOM.
										setTimeout(function()
										{
											ev.sender.hide(); // Fires OnHide
										}, 100);
									}
									else
									{
										ev.sender.hide(); // Fires OnHide
									}

									return;
								}

								if (Fit._internal.Controls.Input.ActiveEditorForDialog === undefined)
									return; // Control was disposed while waiting for dialog to load and open

								Fit.Dom.Data(ev.sender.getElement().$, "skin", CKEDITOR.config.skin); // Add e.g. data-skin="bootstrapck" to dialog - used in Input.css

								// Keep instance to dialog so we can close it if e.g. Focused(false) is invoked
								Fit._internal.Controls.Input.ActiveDialogForEditor = ev.sender;

								if (Fit._internal.Controls.Input.ActiveEditorForDialogDestroyed)
								{
									// Dispose() was called on control while dialog was loading.
									// Since destroying editor while a dialog is loading would cause
									// an error in CKEditor, the operation has been postponed til dialog's
									// OnShow event fires, and the dialog is ready.
									setTimeout(function()
									{
										// Dispose() calls destroy() on editor which closes dialog and causes the dialog's OnHide event to fire.
										// Dispose() uses Fit._internal.Controls.Input.ActiveDialogForEditor, which is why it is set above, before
										// checking whether control has been destroyed (scheduled for destruction).
										Fit._internal.Controls.Input.ActiveEditorForDialog.Dispose();
									}, 0); // Postponed - CKEditor throws an error if destroyed from OnShow event handler

									return;
								}

								if (Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed)
								{
									setTimeout(function()
									{
										delete Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed;

										// DesignMode(false) calls destroy() on editor which closes dialog and causes the dialog's OnHide event to fire.
										Fit._internal.Controls.Input.ActiveEditorForDialog.DesignMode(false);
									}, 0); // Postponed - CKEditor throws an error if destroyed from OnShow event handler

									return;
								}

								// Allow light dismissable panels/callouts to prevent close/dismiss
								// when interacting with editor dialogs hosted outside of these panels/callouts,
								// by detecting the presence of the data-disable-light-dismiss="true" attribute.

								var ckeDialogElement = this.getElement().$;
								Fit.Dom.Data(ckeDialogElement, "disable-light-dismiss", "true");

								var bgModalLayer = document.querySelector("div.cke_dialog_background_cover"); // Shared among instances
								if (bgModalLayer !== null) // Better safe than sorry
								{
									Fit.Dom.Data(bgModalLayer, "disable-light-dismiss", "true");
								}

								// Reduce pollution of document root

								if (Fit._internal.ControlBase.ReduceDocumentRootPollution === true)
								{
									// Move dialog to control - otherwise placed in the root of the document where it pollutes,
									// and makes it impossible to interact with the dialog in light dismissable panels and callouts.
									// Dialog is placed alongside control and not within the control's container, to prevent Fit.UI
									// styling from affecting the dialog.
									// DISABLED: It breaks file picker controls in dialogs which are hosted in iframes.
									// When an iframe is re-rooted in DOM it reloads, and any dynamically created content is lost.
									// We will have to increase the z-index to make sure dialogs open on top of modal layers.
									// EDIT 2021-08-20: Enabled again. The base64image plugin has now been altered so it no longer
									// uses CKEditor's built-in file picker which is wrapped in an iFrame. Therefore the dialog can
									// once again be mounted next to the Input control.

									var ckeDialogElement = this.getElement().$;
									Fit.Dom.InsertAfter(Fit._internal.Controls.Input.ActiveEditorForDialog.GetDomElement(), ckeDialogElement);

									// 2nd+ time dialog is opened it remains invisible - make it appear and position it
									ckeDialogElement.style.display = !CKEDITOR.env.ie || CKEDITOR.env.edge ? "flex" : ""; // https://github.com/ckeditor/ckeditor4/blob/8b208d05d1338d046cdc8f971c9faf21604dd75d/plugins/dialog/plugin.js#L152
									this.layout(); // 'this' is the dialog instance - layout() positions dialog

									// Temporarily move modal background layer next to control to ensure it is part of same
									// stacking context as control and dialog. Otherwise it might stay on top of a panel containing
									// the editor, if that panel has position:fixed (which creates a separate stacking context)
									// and a lower z-index than the background layer.
									// Be aware that the background layer also has position:fixed so it will "escape" a parent
									// container that also has position:fixed, still making it stretch from the upper left corner
									// of the screen to the lower right corner of the screen. However, if a parent is using
									// CSS transform, it will prevent the background layer from escaping, instead positioning it
									// and making it stretch from the upper left corner of the container using transform, to the
									// lower right corner of that container.
									// Also be aware that the use of transform will cause minor problems when moving/dragging dialog,
									// although that is not related to remounting of the background layer.
									var bgModalLayer = document.querySelector("div.cke_dialog_background_cover");
									if (bgModalLayer !== null) // Better safe than sorry
									{
										Fit.Dom.InsertAfter(Fit._internal.Controls.Input.ActiveEditorForDialog.GetDomElement(), bgModalLayer);
									}

									if (ev.sender.definition.title === "Image")
									{
										// Hide image resize handles placed in the root of the document.
										// When the modal background layer above is rooted next to the control,
										// it becomes impossible to ensure the resize handles remains hidden behind
										// the background layer, since the control may be part of a stacking context
										// different from the one containing the image resize handles (document root).
										// It would require dynamic z-index values to achieve this. Therefore the
										// resize handles are temporarily hidden instead.

										var imageResizeHandlersContainer = document.querySelector("#ckimgrsz");
										if (imageResizeHandlersContainer !== null) // Better safe than sorry
										{
											imageResizeHandlersContainer.style.display = "none";
										}
									}
								}
							});

							dialog.on("hide", function(ev) // Fires when user closes dialog, or when hide() is called on dialog, or if destroy() is called on editor instance from Dispose() or DesignMode(false)
							{
								var inputControl = Fit._internal.Controls.Input.ActiveEditorForDialog;
								var showCanceledDueToBlur = Fit._internal.Controls.Input.ActiveDialogForEditorCanceled === true;

								// Clean up global references accessible while dialog is open
								delete Fit._internal.Controls.Input.ActiveEditorForDialog;
								delete Fit._internal.Controls.Input.ActiveEditorForDialogDestroyed;
								delete Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed;
								delete Fit._internal.Controls.Input.ActiveDialogForEditor;
								delete Fit._internal.Controls.Input.ActiveDialogForEditorCanceled;

								if (Fit._internal.ControlBase.ReduceDocumentRootPollution === true)
								{
									// Return modal background layer to document root - it was temporarily moved
									// next to control to ensure it works properly with current stacking context.
									// See comments regarding this in dialog "show" handler registered above.
									// The background layer will not work for other editor instances if not moved back.
									var bgModalLayer = document.querySelector("div.cke_dialog_background_cover");
									if (bgModalLayer !== null) // Better safe than sorry
									{
										Fit.Dom.Add(document.body, bgModalLayer);
									}

									// Allow image resize handlers to show up again (hidden in "show" handler registered above)
									if (ev.sender.definition.title === "Image")
									{
										var imageResizeHandlersContainer = document.querySelector("#ckimgrsz");
										if (imageResizeHandlersContainer !== null) // Better safe than sorry
										{
											imageResizeHandlersContainer.style.display = "";
										}
									}
								}

								// Disable focus lock - let ControlBase handle OnFocus and OnBlur automatically again.
								// This is done postponed since unlocking it immediately will cause OnFocus to fire when
								// dialog returns focus to the editor.
								setTimeout(function()
								{
									if (inputControl.GetDomElement() === null)
										return; // Control was disposed - OnHide was fired because destroy() was called on editor instance from Dispose()

									inputControl._internal.FocusStateLocked(false);

									if (showCanceledDueToBlur === true)
									{
										// Undo focus which dialog returned to editor.
										// ControlBase fires OnBlur because focus state was unlocked above.
										Fit.Dom.GetFocused().blur();
									}
								}, 0);
							});
						});

						if (me === null)
							return; // Control was disposed while waiting for jQuery UI to load

						if (me.DesignMode() === false)
							return; // DesignMode was disabled while waiting for resources to load

						createEditor();
					});
				}
				else if (window.CKEDITOR === null)
				{
					if (createWhenReadyIntervalId === -1) // Make sure DesignMode has not been enabled multiple times - e.g. DesignMode(true); DesignMode(false); DesignMode(true); - in which case an interval timer may already be "waiting" for CKEditor resources to finish loading
					{
						createWhenReadyIntervalId = setInterval(function()
						{
							/*if (me === null)
							{
								// Control was disposed while waiting for CKEditor to finish loading
								clearInterval(iId);
								return;
							}*/

							if (window.CKEDITOR !== null)
							{
								clearInterval(createWhenReadyIntervalId);
								createWhenReadyIntervalId = -1;

								// Create editor if still in DesignMode (might have been disabled while waiting for
								// CKEditor resources to finish loading), and if editor has not already been created.
								// Editor may already exist if control had DesignMode enabled, then disabled, and then
								// enabled once again.
								// If the control is the first one to enabled DesignMode, it will start loading CKEditor
								// resources and postpone editor creation until resources have finished loading.
								// When disabled and re-enabled, the control will realize that resources are being loaded,
								// and postpone editor creation once again, this time using the interval timer here.
								// When resources are loaded, it will create the editor instances, and when the interval
								// timer here executes, it will also create the editor instance, unless we prevent it by
								// making sure only to do it if designEditor is null. Without this check we might experience
								// the following warning in the browser console, when editor is being created on the same
								// textarea control multiple times:
								// [CKEDITOR] Error code: editor-element-conflict. {editorName: "64992ea4-bd01-4081-b606-aa9ff23f417b_DesignMode"}
								// [CKEDITOR] For more information about this error go to https://ckeditor.com/docs/ckeditor4/latest/guide/dev_errors.html#editor-element-conflict
								if (me.DesignMode() === true && designEditor === null)
								{
									createEditor();
								}
							}
						}, 500);
					}
				}
				else
				{
					createEditor();
				}

				if (me.Resizable() !== Fit.Controls.InputResizing.Disabled) // Undo any resizing done in ordinary MultiLine mode
				{
					Fit.Dom.Data(me.GetDomElement(), "resized", "false");

					input.style.width = "";
					input.style.height = "";
					input.style.margin = ""; // Chrome adds some odd margin when textarea is resized
				}

				var enableAutoGrow = me.Height().Value === -1 || (designEditorConfig !== null && designEditorConfig.AutoGrow && designEditorConfig.AutoGrow.Enabled === true);

				if (enableAutoGrow === true && me.Maximizable() === true)
				{
					// Maximize button is disabled when auto grow is enabled, but we make sure to "minimize" control
					// so maximize button returns to its initial state, in case control was maximized prior to enabling
					// DesignMode with auto grow. Otherwise the button indicates that the control is maximized, and so
					// does calls to Maximized() which will incorrectly return True.
					me.Maximized(false);
				}

				repaint();
			}
			else if (val === false && designMode === true)
			{
				if (designModeEnabledAndReady() === false)
				{
					console.error("DesignMode(false) is not allowed for Input control '" + me.GetId() + "' while DesignMode editor is initializing!");
					return true; // Return current un-modified state - DesignMode remains enabled
				}

				var focused = me.Focused();

				if (focused === true) // Make sure focus is preserved when editor is destroyed
				{
					me.GetDomElement().focus();
				}

				if (Fit._internal.Controls.Input.ActiveEditorForDialog === me)
				{
					if (Fit._internal.Controls.Input.ActiveDialogForEditor !== null)
					{
						Fit._internal.Controls.Input.ActiveDialogForEditor.hide(); // Fires dialog's OnHide event
					}
					else
					{
						// Dialog is still loading - calling designEditor.destroy() below will cause an error,
						// leaving a modal layer on the page behind, making it unusable. This may happen if Design Mode is disabled
						// from e.g. a DOM event handler, a mutation observer, a timer, or an AJAX request. The input control
						// itself does not fire any events while the dialog is loading which could trigger this situation, so
						// this can only happen from "external code".

						// WARNING: This has the potential to leak memory if dialog never loads and resumes task of destroying control!
						Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed = true;

						// Detect memory leak
						/* setTimeout(function()
						{
							if (me !== null && me.DesignMode() === false && Fit._internal.Controls.Input.ActiveEditorForDialog === me)
							{
								Fit.Browser.Log("WARNING: Input in DesignMode was not properly disposed in time - potential memory leak detected");
							}
						}, 5000); // Usually the load time for a dialog is barely measurable, so 5 seconds seems sufficient */

						return true; // Return current un-modified state - DesignMode remains enabled until dialog is done loading
					}
				}

				if (designEditorActiveToolbarPanel !== null)
				{
					designEditorActiveToolbarPanel.CloseEmojiPanel();
				}

				// Destroy editor - content is automatically synchronized to input control.
				// Calling destroy() fires OnHide for any dialog currently open, which in turn
				// disables locked focus state and returns focus to the control.
				destroyDesignEditorInstance();

				me._internal.Data("designmode", "false");
				Fit.Dom.Data(me.GetDomElement(), "resized", "false");

				// Remove DesignMode specific data attributes
				me._internal.Data("autogrow", null);
				me._internal.Data("toolbar", null);
				me._internal.Data("toolbar-position", null);
				me._internal.Data("toolbar-sticky", null);

				revertToSingleLineIfNecessary();
				if (focused === true)
				{
					// On IE8 input.focus() does not work if input field is switched to a traditional input field,
					// or if input field is hidden/invisible. It's just not reliable and not worth it. Remove focus
					// from the control in IE8 when DesignMode is disabled and preserve focus in every other browser.

					if (isIe8 === false)
					{
						input.focus();
					}
					else
					{
						me.GetDomElement().blur(); // Control container was given focus further up - this will fire OnBlur as expected
					}
				}

				// Remove tabindex used to prevent control from losing focus when clicking toolbar buttons
				Fit.Dom.Attribute(me.GetDomElement(), "tabindex", null);

				repaint();
			}
		}

		return (me._internal.Data("designmode") === "true");
	}

	/// <function container="Fit.Controls.Input" name="DebounceOnChange" access="public" returns="integer">
	/// 	<description>
	/// 		Get/set number of milliseconds used to postpone onchange event.
	/// 		Every new keystroke/change resets the timer. Debouncing can
	/// 		improve performance when working with large amounts of data.
	/// 	</description>
	/// 	<param name="timeout" type="integer" default="undefined"> If defined, timeout value (milliseconds) is updated - a value of -1 disables debouncing </param>
	/// </function>
	this.DebounceOnChange = function(timeout)
	{
		Fit.Validation.ExpectInteger(timeout, true);

		if (Fit.Validation.IsSet(timeout) === true && timeout !== debounceOnChangeTimeout)
		{
			debounceOnChangeTimeout = timeout;

			if (debouncedOnChange !== null)
			{
				debouncedOnChange.Flush();
				debouncedOnChange = null; // Re-created when needed with new timeout value
			}
		}

		return debounceOnChangeTimeout;
	}

	// ============================================
	// Protected
	// ============================================

	this._internal = (this._internal ? this._internal : {});

	this._internal.DesignModeEnabledAndReady = function()
	{
		return designModeEnabledAndReady();
	}

	// ============================================
	// Private
	// ============================================

	function createEditor()
	{
		// NOTICE: CKEDITOR requires input control to be rooted in DOM.
		// Creating the editor when Render(..) is called is not the solution, since the programmer
		// may call GetDomElement() instead and root the element at any given time which is out of our control.
		// It may be possible to temporarily root the control and make it invisible while the control
		// is being created, and remove it from the DOM when instanceReady is fired. However, since creating
		// the editor is an asynchronous operation, we need to detect whether the element has been rooted
		// elsewhere when instanceCreated is fired, and only remove it from the DOM if this is not the case.
		// This problem needs to be solved some other time as it may spawn other problems, such as determining
		// the size of objects while being invisible. The CKEditor team may also solve the bug in an update.
		if (Fit.Dom.IsRooted(me.GetDomElement()) === false)
		{
			//Fit.Validation.ThrowError("Control must be appended/rendered to DOM before DesignMode can be initialized");

			var retry = function()
			{
				if (Fit.Dom.IsRooted(me.GetDomElement()) === true)
				{
					if (me.DesignMode() === true)
					{
						createEditor();
					}

					return true;
				}

				// Return False to indicate that we still need to keep retrying (still in DesignMode).
				// Otherwie return True to indicate success - retrying is no longer relevant.
				return (me.DesignMode() === true ? false : true);
			};

			setTimeout(function() // Queue to allow control to be rooted
			{
				if (me === null)
				{
					return; // Control was disposed
				}

				if (retry() === false)
				{
					// Still not rooted - add observer to create editor instance once control is rooted

					rootedEventId = Fit.Events.AddHandler(me.GetDomElement(), "#rooted", function(e)
					{
						if (retry() === true || me.DesignMode() === false)
						{
							Fit.Events.RemoveHandler(me.GetDomElement(), rootedEventId);
							rootedEventId = -1;
						}
					});
				}
			}, 0);

			return;
		}

		var langSupport = ["da", "de", "en", "no"];
		var localeCode = Fit.Internationalization.Locale().length === 2 ? Fit.Internationalization.Locale() : Fit.Internationalization.Locale().substring(0, 2);
		var lang = Fit.Array.Contains(langSupport, localeCode) === true ? localeCode : "en";
		var plugins = [];
		var toolbar = [];
		var mentions = [];

		var config = designEditorConfig || {};

		// Enable additional plugins not compiled into CKEditor by default

		if ((config.Plugins && config.Plugins.Emojis === true) || (config.Toolbar && config.Toolbar.Emojis === true))
		{
			Fit.Array.Add(plugins, "emoji");
		}

		if (designModeEnableImagePlugin() === true)
		{
			if (config.Toolbar && config.Toolbar.Images === true)
			{
				Fit.Array.Add(plugins, "base64image");
			}

			plugins = Fit.Array.Merge(plugins, ["base64imagepaste", "dragresize"]);
		}

		Fit.Array.Add(plugins, "custombuttons");

		// Add toolbar buttons

		if (!config.Toolbar || config.Toolbar.Formatting !== false)
		{
			Fit.Array.Add(toolbar,
			{
				name: "BasicFormatting",
				items: [ "Bold", "Italic", "Underline" ]
			});
		}

		if (!config.Toolbar || config.Toolbar.Justify !== false)
		{
			Fit.Array.Add(toolbar,
			{
				name: "Justify",
				items: [ "JustifyLeft", "JustifyCenter", "JustifyRight" ]
			});
		}

		if (!config.Toolbar || config.Toolbar.Lists !== false)
		{
			Fit.Array.Add(toolbar,
			{
				name: "Lists",
				items: [ "NumberedList", "BulletedList", "Indent", "Outdent" ]
			});
		}

		if (!config.Toolbar || config.Toolbar.Links !== false)
		{
			Fit.Array.Add(toolbar,
			{
				name: "Links",
				items: [ "Link", "Unlink" ]
			});
		}

		if (config.Toolbar)
		{
			var insert = [];

			if (config.Toolbar.Emojis === true)
			{
				Fit.Array.Add(insert, "EmojiPanel");
			}

			if (config.Toolbar.Images === true)
			{
				Fit.Array.Add(insert, "base64image");
			}

			if (config.Toolbar.Tables === true)
			{
				Fit.Array.Add(insert, "Table");
			}

			if (insert.length > 0)
			{
				Fit.Array.Add(toolbar,
				{
					name: "Insert",
					items: insert
				});
			}

			var customButtons = [];
			var customToolbarGroups = [];

			if (config.Toolbar.Detach === true)
			{
				Fit.Array.Add(customButtons,
				{
					Label: locale.Detach,
					Command: "Detach",
					Icon: Fit.GetUrl() + "/Controls/Input/" + (window.devicePixelRatio === 2 ? "maximize-highres.png" : "maximize.png"),
					Callback: function(args)
					{
						//console.log("Command " + args.Command.name + " executed", args);
						openDetachedDesignEditor();
					}
				});

				/*Fit.Array.Add(customButtons,
				{
					Label: "Testing 1-2-3",
					Command: "TestButton",
					Icon: "/files/images/Bird.png",
					Callback: function(args)
					{
						alert("Hello world");
					}
				});*/

				Fit.Array.Add(customToolbarGroups,
				{
					name: "DetachableEditor",
					items: ["Detach"/*, "TestButton"*/]
				});
			}

			toolbar = Fit.Array.Merge(toolbar, customToolbarGroups);
		}

		// Configure tags/mentions plugin

		if (config.Tags)
		{
			var requestAwaiting = null;

			var createEventArgs = function(marker, query, request) // EventsArgs for OnRequest and OnResponse
			{
				return { Sender: me, Query: { Marker: marker, Query: query }, Request: request };
			};

			Fit.Array.ForEach(config.Tags.Triggers, function(trigger)
			{
				var mention =
				{
					marker: trigger.Marker,
					minChars: trigger.MinimumCharacters || 0,
					pattern: trigger.Pattern,
					throttle: 0, // Throttling is not debouncing - it merely ensures that no more than 1 request is made every X milliseconds when value is changed (defaults to 200ms) - real debouncing implemented further down, which reduce and cancel network calls as user types - also a work around for https://github.com/ckeditor/ckeditor4/issues/5036
					feed: function(args, resolve)
					{
						// WebService is expected to return tag items in an array like so:
						// [ { Title: string, Value: string, Icon?: string, Url?: string, Data?: string }, { ... }, ... ]

						var req = null;

						if (config.Tags.JsonpCallback)
						{
							req = new Fit.Http.JsonpRequest(config.Tags.QueryUrl, config.Tags.JsonpCallback);
							config.Tags.JsonpTimeout && req.Timeout(config.Tags.JsonpTimeout);
							req.SetParameter("Marker", args.marker);
							req.SetParameter("Query", args.query);
						}
						else
						{
							req = new Fit.Http.JsonRequest(config.Tags.QueryUrl);
							req.SetData({ Marker: args.marker, Query: args.query });
						}

						if (config.Tags.OnRequest)
						{
							var eventArgs = createEventArgs(args.marker, args.query, req);

							if (config.Tags.OnRequest(me, eventArgs) === false)
							{
								resolve([]);
								return;
							}

							if (eventArgs.Request !== req)
							{
								// Support for changing request instans to
								// take control over webservice communication.

								// Restrict to support for Fit.Http.Request or classes derived from this
								Fit.Validation.ExpectInstance(eventArgs.Request, Fit.Http.Request);

								req = eventArgs.Request;
							}
						}

						var processDataAndResolve = function(items)
						{
							if (config.Tags.OnResponse) // OnResponse is allowed to manipulate tags
							{
								var eventArgs = Fit.Core.Merge(createEventArgs(args.marker, args.query, req), { Tags: items });
								config.Tags.OnResponse(me, eventArgs);

								items = eventArgs.Tags; // In case OnResponse event handler assigned new collection
							}

							Fit.Array.ForEach(items, function(item)
							{
								// Set properties required by mentions plugin
								item.id = item.Value;
								item.name = item.Title;
							});

							resolve(items); // Opens context menu immediately if array contain elements, unless user managed to add a space after the search value while waiting for a response, in which case the context menu will not be opened

							if (items.length > 0)
							{
								// Calling resolve(..) above immediately opens the context menu from which a tag can be selected

								// Get the autocomplete context menu currently open. There can be only one
								// such menu open at any time. Each editor can declare multiple autocomplete
								// context menus since each tag marker is associated with its own context menu.
								var ctm = document.querySelector("ul.cke_autocomplete_opened");

								if (ctm !== null) // Null if user managed to enter a space after tag search value, before response was received - in this case resolve(..) above will not open the context menu
								{
									// Allow light dismissable panels/callouts to prevent close/dismiss
									// when interacting with tags context menu hosted outside of these panels/callouts,
									// by detecting the presence of the data-disable-light-dismiss="true" attribute.
									Fit.Dom.Data(ctm, "disable-light-dismiss", "true");

									if (Fit._internal.ControlBase.ReduceDocumentRootPollution === true)
									{
										// Tags context menu is placed in the root of the document where
										// it pollutes the global scope. Move it next to the Fit.UI control.
										// We do not mount it within the Fit.UI control as it could cause Fit.UI styles
										// to take effect on the context menu.

										// Has position:absolute by default, but this may be affected by a positioned
										// container (offsetParent), so we change it to position:fixed. Downside:
										// It no longer sticks to the editor when scrolling. However, if a container has CSS
										// transform set, the context menu's position will be affected and become inaccurate.
										ctm.style.position = "fixed";
										Fit.Dom.InsertAfter(me.GetDomElement(), ctm);
									}
								}
							}
						};

						if (Fit.Core.InstanceOf(req, Fit.Http.JsonpRequest) === true)
						{
							req.OnSuccess(function(sender)
							{
								var response = req.GetResponse();
								var items = ((response instanceof Array) ? response : []);

								processDataAndResolve(items);
							});

							req.OnTimeout(function(sender)
							{
								resolve([]);
								Fit.Validation.ThrowError("Unable to get tags - request did not return data in time (JSONP timeout reached)");
							});
						}
						else
						{
							req.OnSuccess(function(sender)
							{
								var response = req.GetResponseJson();
								var items = ((response instanceof Array) ? response : []);

								processDataAndResolve(items);
							});

							req.OnFailure(function(sender)
							{
								resolve([]);
								Fit.Validation.ThrowError("Unable to get tags - request failed with HTTP Status code " + req.GetHttpStatus());
							});
						}

						if (requestAwaiting !== null)
						{
							requestAwaiting.Abort();
						}

						requestAwaiting = req;
						req.Start();
					},
					itemTemplate: function(item) // Item must define "name" and "id" properties - the {name} placeholder is replaced by "@" + the value of the "name" property - to get rid of "@" simply use an alternative property such as nameWithoutTag:"Some username"
					{
						if (item.Icon)
						{
							var imageWidth = "24px";
							var spacingWidth = "5px";
							var spanWidthCss = isIe8 === true ? "width: 165px" : "width: calc(100% - " + imageWidth + " - " + spacingWidth + ")"; // We are using fixed dimensions on IE8 (see Input.css)

							return '<li data-id="' + item.Value + '"><img src="' + item.Icon + '" style="width: ' + imageWidth + '; height: ' + imageWidth + '; border-radius: ' + imageWidth + '; vertical-align: middle" alt=""><span style="display: inline-block; box-sizing: border-box; ' + spanWidthCss + '; white-space: nowrap; overflow: hidden; text-overflow: ellipsis; vertical-align: middle; margin-left: ' + spacingWidth + '">' + item.Title + '</span></li>';
						}
						else
						{
							return '<li data-id="' + item.Value + '"><span style="display: inline-block; box-sizing: border-box; width: 100%">' + item.Title + '</span></li>';
						}
					},
					outputTemplate: function(item)
					{
						// IMPORTANT: Output produced must respect ACF (Advanced Content Filter).
						// So the tag produced must be allowed, and any attributes contained must be allowed.

						var alternativeItem = null;

						if (config.Tags.TagCreator)
						{
							var callbackArgs = { Sender: me, QueryMarker: trigger.Marker, Tag: Fit.Core.Clone(item) };
							alternativeItem = config.Tags.TagCreator(me, callbackArgs) || null;
						}

						// Function should return a link for tags to "just work". Returning a <span> requires the span to be whitelisted in
						// extraAllowedContent configuration, but even then the editor will continue writing text within the <span> element,
						// rather than next to it. So one would expect something like: We will assign <span data-tag-type="@" ..>@James Bond</span> to this mission.
						// But what we get instead is something like: We will assign <span data-tag-type="@" ..>@James Bond to this mission</span>.
						// The same happens to link tags if the href attribute is removed, which is why we always add it, even when no URL is defined.

						if (alternativeItem !== null)
						{
							return '<a data-tag-type="' + (alternativeItem.Type || trigger.Marker) + '" data-tag-id="' + (alternativeItem.Value || item.Value) + '"' + (alternativeItem.Data || item.Data ? ' data-tag-data="' + (alternativeItem.Data || item.Data) + '"' : '') + (alternativeItem.Context || item.Context ? ' data-tag-context="' + (alternativeItem.Context || item.Context) + '"' : '') + (alternativeItem.Url || item.Url ? ' href="' + (alternativeItem.Url || item.Url) + '"' : 'href=""') + '>' + (alternativeItem.Title || (trigger.Marker + item.Title)) + '</a>&nbsp;';
						}
						else
						{
							return '<a data-tag-type="' + trigger.Marker + '" data-tag-id="' + item.Value + '"' + (item.Data ? ' data-tag-data="' + item.Data + '"' : '') + (item.Context ? ' data-tag-context="' + item.Context + '"' : '') + (item.Url ? ' href="' + item.Url + '"' : 'href=""') + '>' + trigger.Marker + item.Title + '</a>&nbsp;';
						}
					}
				};

				if (trigger.DebounceQuery !== 0) // A value of 0 (zero) disables debouncing
				{
					// Wrap feed handler in debounce function so that every time it gets invoked, it cancels the previous invocation
					mention.feed = Fit.Core.CreateDebouncer(mention.feed, trigger.DebounceQuery || 300).Invoke;
				}

				Fit.Array.Add(mentions, mention);
			});
		}

		var onImageAdded = function(args)
		{
			if (args.type === "blob")
			{
				// For a list of blobs in Chrome see: chrome://blob-internals/
				// Be aware that garbage is NOT being collected unless needed, so
				// don't expect the list to update immediately. Garbage collection
				// can be triggered in Dev Tools > Memory: click the trash can icon.
				// Make sure to garbage collect from the tab/window running Fit.UI,
				// NOT from the tab/window listing blobs!
				// Use https://jsfiddle.net/ute87p1m/6/ to test garbage collection.

				imageBlobUrls.push(args.url);
			}

			/*// Image data can be retrieved from a blob like this:
			if (img.src.indexOf("blob:") === 0)
			{
				var r = new Fit.Http.Request(img.src); // E.g. "blob:http://localhost:8080/0c5aa2ae-f2ea-414a-af42-53047959ad1b"
				r.RequestProperties({ responseType: "blob" });
				r.OnSuccess(function(sender)
				{
					var blob = sender.GetResponse();

					var reader = new FileReader();
					reader.onload = function(ev)
					{
						var base64 = ev.target.result;
						console.log(base64);
					};
					reader.readAsDataURL(blob);
				});
				r.Start();
			}*/
		};

		// How disallowedContent works is described here: https://ckeditor.com/docs/ckeditor4/latest/guide/dev_disallowed_content.html
		// How the rules work is covered in more details here: https://ckeditor.com/docs/ckeditor4/latest/guide/dev_allowed_content_rules.html
		// Format in short: element[allowed/disallowed attributes]{allowed/disallowed styles}(allowed/disallowed classes)
		// Allowed and disallowed elements, attributes, styles, and classes can be revealed runtime using:
		//     CKEDITOR.instances['<Instance ID>'].filter.allowedContent
		//     CKEDITOR.instances['<Instance ID>'].filter.disallowedContent
		// Use the following code to check whether given content is allowed or not based on allowedContent and disallowedContent:
		//     CKEDITOR.instances['<Instance ID>'].filter.check("element[attributes]{styles}(classes)");
		// Be aware that disallowing e.g. border* works even though e.g. check("td{border}") returns true, incorrectly indicating that it is allowed.
		// Being more specific using e.g. check("td{border-style}") causes it to return false as expected, correctly indicating that it is not allowed.
		var disallowedContent = "";

		// Undo from allowedContent in table plugin: https://github.com/ckeditor/ckeditor4/blob/4df6984595e3b73de61cd2a1b1a7ec823f9cfbdc/plugins/table/plugin.js#L21C21-L21C102
		//disallowedContent += (disallowedContent !== "" ? ";" : "") + "table[align,cellpadding,cellspacing]{*}"; // Remove all attributes except border and summary

		// Undo from allowedContent in tabletools plugin: https://github.com/ckeditor/ckeditor4/blob/4df6984595e3b73de61cd2a1b1a7ec823f9cfbdc/plugins/tabletools/plugin.js#L820
		// and from table plugin: https://github.com/ckeditor/ckeditor4/blob/4df6984595e3b73de61cd2a1b1a7ec823f9cfbdc/plugins/table/plugin.js#L24C1-L25C1
		//disallowedContent += (disallowedContent !== "" ? ";" : "") + "td th{background-color,border*,height,vertical-align,white-space,width}"; // Remove all CSS properties except text-align - retains all attributes (colspan, rowspan)

		designEditor = CKEDITOR.replace(me.GetId() + "_DesignMode",
		{
			toolbarLocation: designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.Position === "Bottom" ? "bottom" : "top",
			uiColor: Fit._internal.Controls.Input.Editor.Skin === "moono-lisa" || Fit._internal.Controls.Input.Editor.Skin === null ? "#FFFFFF" : undefined,
			//allowedContent: true, // http://docs.ckeditor.com/#!/guide/dev_allowed_content_rules and http://docs.ckeditor.com/#!/api/CKEDITOR.config-cfg-allowedContent
			extraAllowedContent: "a[data-tag-type,data-tag-id,data-tag-data,data-tag-context]", // https://ckeditor.com/docs/ckeditor4/latest/api/CKEDITOR_config.html#cfg-extraAllowedContent
			disallowedContent: disallowedContent,
			language: lang,
			disableNativeSpellChecker: me.CheckSpelling() === false,
			readOnly: me.Enabled() === false,
			tabIndex: me.Enabled() === false ? -1 : 0,
			title: "",
			width: "100%", // Assume width of container
			height: me.Height().Value > -1 ? me.Height().Value + me.Height().Unit : "100%", // Height of content area - toolbar and bottom panel takes up additional space - once editor is loaded, the outer dimensions are accurately set using updateDesignEditorSize() - a height of 100% enables auto grow
			startupFocus: me.Focused() === true ? "end" : false, // Doesn't work when editor is hidden while initializing to avoid flickering - focus is set again when editor is made visible in instanceReady handler, at which point it will place cursor at the end of the editor if startupFocus is set to "end"
			extraPlugins: plugins.join(","),
			clipboard_handleImages: false, // Disable native support for image pasting - allow base64imagepaste plugin to handle image data if loaded
			base64image: // Custom property used by base64image plugin if loaded
			{
				storage: designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.EmbedType === "blob" ? "blob" : "base64", // "base64" (default) or "blob" - base64 will always be provided by browsers not supporting blob storage
				onImageAdded: onImageAdded
			},
			base64imagepaste: // Custom property used by base64imagepaste plugin if loaded - notice that IE has native support for image pasting as base64 so plugin is not triggered in IE
			{
				storage: designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.EmbedType === "blob" ? "blob" : "base64", // "base64" (default) or "blob" - base64 will always be provided by browsers not supporting blob storage
				onImageAdded: onImageAdded
			},
			resize_enabled: resizable !== Fit.Controls.InputResizing.Disabled,
			resize_dir: resizable === Fit.Controls.InputResizing.Enabled ? "both" : resizable === Fit.Controls.InputResizing.Vertical ? "vertical" : resizable === Fit.Controls.InputResizing.Horizontal ? "horizontal" : "none", // Specific to resize plugin (horizontal | vertical | both - https://ckeditor.com/docs/ckeditor4/latest/features/resize.html)
			toolbar: toolbar,
			removeButtons: "", // Set to empty string to prevent CKEditor from removing buttons such as Underline
			customButtons: customButtons,
			mentions: mentions,
			emoji_minChars: 9999, // Impossible requirement to number of search characters to "disable" emoji auto complete menu - we cannot make it work properly with light dismissable panels/callouts since we have no event available for registering the data-disable-light-dismiss="true" attribute, and it's not very useful in any case
			on:
			{
				instanceReady: function()
				{
					designEditorDom = // Object assignment will make designModeEnabledAndReady() return True, so it must be assigned immediately
					{
						OuterContainer: designEditor.container.$,
						InnerContainer: designEditor.container.$.querySelector(".cke_inner"),
						Top: designEditor.container.$.querySelector(".cke_top"), // Null if toolbar is placed at the bottom
						Content: designEditor.container.$.querySelector(".cke_contents"),
						Editable: designEditor.container.$.querySelector(".cke_editable"),
						Bottom: designEditor.container.$.querySelector(".cke_bottom") // Only exist if editor is resizable or toolbar is placed at the bottom
					}

					// Make sure expected DOM elements are present, so we do not have to perform null checks anywhere else in the code.
					// Notice that designEditorDom.Top is null if toolbar is placed at the bottom, and designEditorDom.Bottom is null
					// unless toolbar is placed at the bottom, or editor is resizable, which adds a resize handled in the lower right corner.
					if (designEditorDom.InnerContainer === null || designEditorDom.Content === null || designEditorDom.Editable === null || (designEditorDom.Top === null && designEditorDom.Bottom === null))
					{
						throw "One or more editor DOM elements are missing"; // This should only happen if CKEditor changes its DOM structure
					}

					if (designEditorMustDisposeWhenReady === true)
					{
						Fit.Browser.Debug("WARNING: Input control '" + me.GetId() + "' was disposed while initializing DesignMode - now resuming disposal");
						me.Dispose();
						return;
					}

					if (designEditorMustReloadWhenReady === true)
					{
						Fit.Browser.Debug("WARNING: Editor for Input control '" + me.GetId() + "' finished loading, but properties affecting editor has changed while initializing - reloading to adjust to changes");
						reloadEditor(true);
						return;
					}

					// Make links in editor clickable in combination with CTRL/META/SHIFT
					var mouseOver = false;
					Fit.Events.AddHandler(designEditorDom.Editable, "mouseover", function(e)
					{
						mouseOver = true;
						Fit.Dom.Data(designEditorDom.Editable, "command-button-active", (e.ctrlKey === true || e.metaKey === true || e.shiftKey === true) && "true" || null);
					});
					Fit.Events.AddHandler(designEditorDom.Editable, "mouseout", function(e)
					{
						mouseOver = false;
						Fit.Dom.Data(designEditorDom.Editable, "command-button-active", null);
					});
					designEditorGlobalKeyDownEventId = Fit.Events.AddHandler(document, "keydown", function(e)
					{
						mouseOver && Fit.Dom.Data(designEditorDom.Editable, "command-button-active", (e.ctrlKey === true || e.metaKey === true || e.shiftKey === true) && "true" || null);
					});
					designEditorGlobalKeyUpEventId = Fit.Events.AddHandler(document, "keyup", function(e)
					{
						mouseOver && Fit.Dom.Data(designEditorDom.Editable, "command-button-active", null);
					});
					Fit.Events.AddHandler(designEditorDom.Editable, "mousedown", function(e) // Using OnMouseDown to make sure click is registered before control gains focus if not already focused (using me.Focused() in event handler)
					{
						var target = Fit.Events.GetTarget(e);

						// Notice that target.href is computed - it is always a fully qualified URL. For an empty href attribute the href property
						// will point to the current page URL, and for a relative URL in the href attribute the href property is combined with the
						// current page URL, also forming a fully qualified URL.

						if (target.tagName === "A" && Fit.Dom.Attribute(target, "href") !== "")
						{
							if (e.ctrlKey === true || e.metaKey === true)
							{
								window.open(target.href);
							}
							else if (e.shiftKey === true)
							{
								window.open(target.href, "_blank");
							}
							else
							{
								var editorInactive = me.Focused() === false && designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.HideWhenInactive === true;

								if (me.Enabled() === false || editorInactive === true)
								{
									var popupCode = Fit.Dom.Data(target, "cke-pa-onclick");

									if (popupCode !== null) // Popup window link - example code: window.open(this.href, "name", "options"); return false;
									{
										popupCode = popupCode.replace("this.href", "'" + target.href + "'"); // Insert link URL
										popupCode = popupCode.replace("return false;", ""); // Remove return statement which is illegal in eval(..)

										eval(popupCode);
									}
									else
									{
										window.open(target.href, editorInactive === true ? "_blank" : Fit.Dom.Attribute(target, "target") || "_self");
									}
								}
							}
						}
					});

					removeCkeSavedSrcAttributesFromDesignEditor();

					updateDesignEditorPlaceholder(); // Show/hide placeholder - value might have been set/removed while initializing editor

					Fit.Dom.Data(designEditorDom.OuterContainer, "skin", CKEDITOR.config.skin); // Add e.g. data-skin="bootstrapck" to editor - used in Input.css

					// Enabled state might have been changed while loading.
					// Unfortunately there is no API for changing the tabIndex.
					// Similar logic is found in Enabled(..) function.
					designEditor.setReadOnly(me.Enabled() === false);
					Fit.Dom.Attribute(designEditorDom.Editable, "tabindex", me.Enabled() === false ? null : "0"); // Preventing focus is only possible by nullifying DOM attribute (these does not work: delete elm.tabIndex; elm.tabIndex = null|undefined|-1)

					if (me.Height().Value === -1 || (designEditorConfig !== null && designEditorConfig.AutoGrow && designEditorConfig.AutoGrow.Enabled === true))
					{
						// Enable auto grow - this is done late to allow an initial static height on the outer control which prevents "flickering" while loading

						me.Height(-1); // Make sure auto grow is enabled since it is unlikely external code has done so explicitely by calling Height(-1)

						// Make necessary adjustments to editor DOM for auto grow's min/max height to work

						var editableDiv = designEditorDom.Editable;
						editableDiv.style.minHeight = designEditorConfig !== null && designEditorConfig.AutoGrow && designEditorConfig.AutoGrow.MinimumHeight ? designEditorConfig.AutoGrow.MinimumHeight.Value + (designEditorConfig.AutoGrow.MinimumHeight.Unit || "px") : ""; // NOTICE: Minimum height of editable area, not control
						editableDiv.style.maxHeight = designEditorConfig !== null && designEditorConfig.AutoGrow && designEditorConfig.AutoGrow.MaximumHeight ? designEditorConfig.AutoGrow.MaximumHeight.Value + (designEditorConfig.AutoGrow.MaximumHeight.Unit || "px") : ""; // NOTICE: Maximum height of editable area, not control

						// Restrict resizing:
						// Make sure user cannot resize editor beyond max height of editable area.
						// Editable area will not "follow" since it is restricted using maxHeight set above.
						// If we do not want resizing to be restricted, then unset minHeight and maxHeight set
						// above, when resizing occur (see CKEditor's resize event handler).
						if (editableDiv.style.maxHeight !== "" && designEditorConfig.AutoGrow.PreventResizeBeyondMaximumHeight === true)
						{
							var contents = designEditorDom.Content;
							contents.style.maxHeight = editableDiv.style.maxHeight;
						}
					}

					if (me.Height().Value !== -1 && me.Height().Unit === "%")
					{
						enableDesignEditorHeightMonitor(true); // Enable support for relative height
					}

					if (designEditorConfig !== null && designEditorConfig.InfoPanel && designEditorConfig.InfoPanel.Text)
					{
						var infoPanel = document.createElement("div");
						infoPanel.className = "FitUiControlInputInfoPanel";
						infoPanel.innerHTML = designEditorConfig.InfoPanel.Text;
						infoPanel.style.cssText = "text-align: " + (designEditorConfig.InfoPanel.Alignment ? designEditorConfig.InfoPanel.Alignment.toLowerCase() : "center");

						if (designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.Position === "Bottom")
						{
							Fit.Dom.InsertBefore(designEditorDom.Content, infoPanel);
						}
						else
						{
							Fit.Dom.InsertAfter(designEditorDom.Content, infoPanel);
						}
					}

					// Sticky toolbar - compensate for padding in scroll parent

					if (me._internal.Data("toolbar-sticky") === "true")
					{
						var toolbarContainer = designEditorDom.Top || designEditorDom.Bottom;

						if (Fit.Dom.GetComputedStyle(toolbarContainer, "position") === "sticky") // False on non-supported browsers as position:sticky is applied via the @supports CSS rule
						{
							var scrollParent = Fit.Dom.GetScrollParent(me.GetDomElement());

							if (scrollParent !== null) // In case editor is hosted in a container with position:fixed with overflow:hidden, in which case Fit.Dom.GetScrollParent(..) returns null
							{
								var toolbarPosition = me._internal.Data("toolbar-position"); // top | bottom

								if (toolbarPosition === "top")
								{
									var paddingOffset = Fit.Dom.GetComputedStyle(scrollParent, "padding-top"); // E.g. "28px"
									toolbarContainer.style.top = paddingOffset !== "0px" ? "-" + paddingOffset : "";
								}
								else
								{
									var paddingOffset = Fit.Dom.GetComputedStyle(scrollParent, "padding-bottom"); // E.g. "28px"
									toolbarContainer.style.bottom = paddingOffset !== "0px" ? "-" + paddingOffset : "";
								}
							}
						}
					}

					// Register necessary events with emoji panel when opened

					var emojiButton = designEditor.container.$.querySelector("a.cke_button__emojipanel");

					if (emojiButton !== null) // Better safe than sorry
					{
						Fit.Events.AddHandler(emojiButton, "click", function(e)
						{
							// Make sure OnFocus fires before locking focus state

							if (me.Focused() === false)
							{
								// Control not focused - make sure OnFocus fires when emoji button is clicked,
								// and make sure ControlBase internally considers itself focused, so there is
								// no risk of OnFocus being fired twice without OnBlur firing in between,
								// when focus state is unlocked, and focus is perhaps re-assigned to another
								// DOM element within the control, which will be the case if the design editor
								// is switched back to an ordinary input field (e.g. using DesignMode(false)).
								me.Focused(true);
							}

							// Prevent control from firing OnBlur when emoji dialog is opened.
							// Notice that locking the focus state will also prevent OnFocus
							// from being fired automatically.
							me._internal.FocusStateLocked(true);

							setTimeout(function() // Postpone - emoji panel is made visible after click event
							{
								// Allow light dismissable panels/callouts to prevent close/dismiss
								// when interacting with emoji widget hosted outside of panels/callouts,
								// by detecting the presence of a data-disable-light-dismiss="true" attribute.
								var emojiPanel = document.querySelector("div.cke_emoji-panel"); // Shared among instances

								if (emojiPanel !== null) // Better safe than sorry
								{
									Fit.Dom.Data(emojiPanel, "disable-light-dismiss", "true");

									emojiPanel._associatedFitUiControl = me;

									designEditorActiveToolbarPanel =
									{
										DomElement: emojiPanel,
										UnlockFocusStateIfEmojiPanelIsClosed: function() // Function called regularly via interval timer while emoji panel is open to make sure focus state is unlocked when emoji panel is closed, e.g. by pressing ESC, clicking outside of emoji panel, or by choosing an emoji
										{
											if (designModeEnabledAndReady() === false /* No longer in DesignMode */ || Fit.Dom.IsVisible(emojiPanel) === false /* Emoji panel closed */ || emojiPanel._associatedFitUiControl !== me /* Emoji panel now opened from another editor */)
											{
												designEditorActiveToolbarPanel = null;

												// Disable focus lock - let ControlBase handle OnFocus and OnBlur automatically again
												me._internal.FocusStateLocked(false);

												// Fire OnBlur in case user changed focus while emoji panel was open.
												// OnBlur does not fire automatically when focus state is locked.
												if (me.Focused() === false)
												{
													me._internal.FireOnBlur();
												}
											}
										},
										CloseEmojiPanel: function()
										{
											if (emojiPanel._associatedFitUiControl === me && Fit.Dom.IsVisible(emojiPanel) === true && Fit.Dom.Contained(emojiPanel, Fit.Dom.GetFocused()) === true)
											{
												designEditor.focus();
												designEditorActiveToolbarPanel.UnlockFocusStateIfEmojiPanelIsClosed();
											}
										}
									}
								}

								// Hide status bar in emoji dialog
								var emojiFrame = emojiPanel.querySelector("iframe");
								var emojiContent = emojiFrame && emojiFrame.contentDocument;
								var emojiContentBlock = emojiContent && emojiContent.querySelector(".cke_emoji-outer_emoji_block");
								var emojiContentStatus = emojiContent && emojiContent.querySelector(".cke_emoji-status_bar");
								emojiContentBlock && (emojiContentBlock.style.height = "220px");
								emojiContentStatus && (emojiContentStatus.style.display = "none");

								var checkClosedId = setInterval(function()
								{
									// Invoke cleanup function regularly to make sure
									// focus lock is relased when emoji panel is closed,
									// and to fire OnBlur if another control was focused
									// while emoji panel was open.

									if (me === null)
									{
										clearInterval(checkClosedId);
										return;
									}

									if (designEditorActiveToolbarPanel !== null)
									{
										designEditorActiveToolbarPanel.UnlockFocusStateIfEmojiPanelIsClosed(); // Nullfies designEditorActiveToolbarPanel if emoji panel is closed
									}

									if (designEditorActiveToolbarPanel === null)
									{
										clearInterval(checkClosedId);
									}
								}, 250);
							}, 0);
						});
					}

					// DISABLED: Doesn't work! Emoji panel contains an iFrame. When it is re-mounted
					// in DOM, the iframe reloads, and dynamically added content is lost. Also, this makes
					// CKEditor throw errors and the dialog never appears.
					/*if (Fit._internal.ControlBase.ReduceDocumentRootPollution === true)
					{
						// Move emoji dialog to control - otherwise placed in the root of the document where it pollutes,
						// and makes it impossible to interact with the dialog in light dismissable panels and callouts.
						// Dialog is placed alongside control and not within the control's container, to prevent Fit.UI
						// styling from affecting the dialog.
						if (config.Toolbar && config.Toolbar.Emojis === true)
						{
							var emojiButton = designEditor.container.$.querySelector("a.cke_button__emojipanel");

							if (emojiButton !== null)
							{
								Fit.Events.AddHandler(emojiButton, "click", function(e)
								{
									setTimeout(function() // Postpone - made visible after click event
									{
										var emojiPanel = document.querySelector("div.cke_emoji-panel:not([style*='display: none'])");

										if (emojiPanel !== null)
										{
											Fit.Dom.InsertAfter(me.GetDomElement(), emojiPanel);
										}
									}, 0);
								});
							}
						}
					}*/

					// Remove buggy cut/copy/paste operations from ContextMenu.
					// Related issue: https://github.com/ckeditor/ckeditor4/issues/469
					// NOTICE: ContextMenu will not show up when empty - it won't
					// even open the browser's own ContextMenu in this case. We work
					// around this by suppressing CKEditor's ContextMenu using the
					// "contextmenu" event registered in init().
					designEditor.removeMenuItem("cut");
					designEditor.removeMenuItem("copy");
					designEditor.removeMenuItem("paste");

					// Make editor assume configured width and height.
					// Notice that using config.width and config.height
					// (https://ckeditor.com/docs/ckeditor4/latest/features/size.html)
					// results in editor becoming too high since the toolbar height is not
					// substracted. This problem does not occur when using updateDesignEditorSize().
					updateDesignEditorSize();

					if (me.Focused() === false)
					{
						// Hide editor toolbar if configured to do so
						hideToolbarInDesignMode();
					}
					/*else
					{
						// Remove placeholder if initially focused
						updateDesignEditorPlaceholder(true);
					}*/

					// Make editor visible - postpone to allow editor to first calculate auto grow height
					// so the user will not see the chrome (borders) of the editor increase its height.
					setTimeout(function()
					{
						if (me === null) // Might have been disposed while loading, e.g as part of "warm up" to load resources
						{
							return;
						}

						designEditorDom.OuterContainer.style.visibility = "visible";

						// Because editor is hidden while initializing, startupFocus
						// (https://ckeditor.com/docs/ckeditor4/latest/api/CKEDITOR_config.html#cfg-startupFocus)
						// won't be able to place focus in the editor. We resolve this by assigning focus again
						// once editor is visible (visibility set above). Because startupFocus was set, it will
						// place focus at the end of the editor as expected.
						if (me.Focused() === true)
						{
							designEditor.focus(); // Won't work on iOS as assigning focus must be the result of a direct user interaction, which this is not since it is postponed using setTimeout(..) and async. loading of the editor
						}
					}, 0);
				},
				change: function() // CKEditor bug: not fired in Opera 12 (possibly other old versions as well)
				{
					if (designEditorCleanEditableDom === true)
					{
						removeCkeSavedSrcAttributesFromDesignEditor();
					}

					if (me._internal.FireOnChangeSuppressed === true)
					{
						// Do not process event - it has been fired by CKEditor when HTML
						// value was initially assigned in Value(..) which happend through
						// me._internal.ExecuteWithNoOnChange(function() { .. }).
						// See Value(..) implementation for details.
						return;
					}

					// Assume value was changed by user if control has focus
					if (designEditorDirty === false && me.Focused() === true)
					{
						designEditorDirty = true;
					}

					input.onkeyup();
				},
				resize: function() // Fires when size is changed (except via auto grow), not just when resized using resize handle in lower right cornor
				{
					if (designEditorSuppressOnResize === false) // Only set data-resized="true" when resized using resize handle
					{
						designEditorDom.Editable.style.height = ""; // Disable fixed height set if toolbar was hidden/displayed at some point (see hideToolbarInDesignMode() and restoreHiddenToolbarInDesignEditor())

						// Disable Min/Max height configured with auto grow feature so user can resize it freely, unless PreventResizeBeyondMaximumHeight is enabled
						if (designEditorConfig !== null && designEditorConfig.AutoGrow && designEditorConfig.AutoGrow.Enabled === true && designEditorConfig.AutoGrow.PreventResizeBeyondMaximumHeight !== true)
						{
							var editableDiv = designEditorDom.Editable;
							editableDiv.style.minHeight = "";
							editableDiv.style.maxHeight = "";

							var contents = designEditorDom.Content;
							contents.style.maxHeight = "";
						}

						me._internal.Data("resized", "true");
						repaint();
					}
				},
				selectionChange: function(ev)
				{
					var elm = ev.data.selection.getStartElement().$;

					// Allow light dismissable panels/callouts to prevent close/dismiss
					// when interacting with image resize handles hosted outside of panels/callouts,
					// by detecting the presence of a data-disable-light-dismiss="true" attribute.

					if (elm.tagName === "IMG")
					{
						setTimeout(function() // Postpone - wait for image resize plugin to add image resize handles
						{
							var imageResizeHandlesContainer = document.querySelector("#ckimgrsz");
							if (imageResizeHandlesContainer !== null) // Better safe than sorry
							{
								Fit.Dom.Data(imageResizeHandlesContainer, "disable-light-dismiss", "true");
							}
						}, 0);
					}

					// Disable/enable toolbar buttons, depending on whether a tag/mention is selected

					if (elm.tagName === "A" && Fit.Dom.Data(elm, "tag-id") !== null)
					{
						// Notice that selectionChange handler is invoked while editor is loading if control was given initial focus.
						// But at this point the toolbar buttons are not yet available to be disabled, so disableDesignEditorButtons()
						// won't work. However, as soon as the editor is done loading, focus is re-assigned to the editable area
						// which will trigger selectionChange handler once again, at which point designModeEnabledAndReady() returns true.
						if (designModeEnabledAndReady() === true)
						{
							designEditorSuppressPaste = true;
							setTimeout(function() // Postpone - otherwise we won't be able to temporarily disable some of the buttons (https://jsfiddle.net/ymv56znq/14/)
							{
								disableDesignEditorButtons();
							}, 0);
						}
					}
					else
					{
						designEditorSuppressPaste = false;
						restoreDesignEditorButtons();
					}
				},
				doubleclick: function(ev)
				{
					// Suppress link dialog when double clicking. User must use link
					// button instead which triggers beforeCommandExec below - it creates
					// a focus lock to prevent control from losing focus and firing OnBlur.
					if (ev.data.element.$.tagName === "A")
					{
						ev.cancel();
						return;
					}

					// Suppress link dialog for tags (similar code found in beforeCommandExec handler below)
					// DISABLED: No longer needed since link dialog is now suppressed for all links (see code above)
					/*if (Fit.Dom.Data(ev.data.element.$, "tag-id") !== null)
					{
						ev.cancel();
						return;
					}*/
				},
				paste: function(ev)
				{
					var html = ev.data.dataValue;

					// Fix line breaks in image alt attributes generated by Word (at least on Mac).
					// Word automatically decodes the nature of an image, and adds a suitable description
					// such as "Girl walking a dog\n\nDescription automatically generated". Notice line breaks.
					var imageTags = html.match(/<img [^>]+?>/g);
					var updatedImageTags = [];
					Fit.Array.ForEach(imageTags || [], function(imageTag)
					{
						var updatedImageTag = imageTag.replace(/\n/g, " ");
						html = html.replace(imageTag, updatedImageTag);
						updatedImageTags.push(updatedImageTag);
					});

					// Image paste plugin does not handle pasting of mixed content (text and images).
					// For Word specifically, this results in images being added as inline base64 images.
					// When copying text and images from browsers, external image references are added instead.
					// Convert base64 images from Word into image blobs on supported browsers if blob storage is enabled.
					if (designEditorConfig !== null && designEditorConfig.Plugins && designEditorConfig.Plugins.Images && designEditorConfig.Plugins.Images.EmbedType === "blob" && window.URL && window.URL.createObjectURL)
					{
						var dataUrlRegEx = /<img.*? src=(["'])(data:image\/[a-z]+;base64,[A-Za-z0-9+\/=]+)\1/; // [ 0 = Full match, 1 = quote type, 2 = Base64 image data URL ][] | null (https://regex101.com/r/r7GhkO/1)

						Fit.Array.ForEach(updatedImageTags, function(img)
						{
							var imageTagWithDataUrl = img.match(dataUrlRegEx);

							if (imageTagWithDataUrl !== null)
							{
								// Convert images synchronously - we need them immediately while paste event is
								// running, so we can manipulate data before it is inserted into CKEditor. If this
								// proves to perform poorly, consider moving the logic to the afterInsertHtml handler.
								// However, we would need to prevent changes to the editor while waiting for conversion to finish.
								Fit.Core.Base64ToBlob(imageTagWithDataUrl[2], null /* get mime type from base64 data */, false /* not async */, function(result)
								{
									if (result.Blob !== null) // Null on failure
									{
										var newImageDataUrl = URL.createObjectURL(result.Blob);
										imageBlobUrls.push(newImageDataUrl);
										html = html.replace(imageTagWithDataUrl[2], newImageDataUrl);
									}
									else
									{
										Fit.Browser.Log("Error converting base64 image to blob: " + result.Error);
									}
								});
							}
						});
					}

					ev.data.dataValue = html;

					// Prevent pasting (especially images) into tags.
					// OnPaste is suppressed using an OnPaste handler in capture phase, which will prevent the operation entirely
					// on supported browsers. On legacy browsers we handle this by invoking undo on the editor instance instead.
					//var path = ev.editor.elementPath(); // Null if dialog button is triggered without placing text cursor in editor first
					//if (Fit.Dom.Data(path.lastElement.$, "tag-id") !== null)
					if (designEditorSuppressPaste === true) // Also handled in a native OnPaste event handler (capture phase) for supported browsers, which suppresses the event entirely
					{
						setTimeout(function() // Postpone - allow editor to create snapshot
						{
							ev.editor.execCommand("undo"); // Undo change - paste event cannot be canceled, as it has already happened
						}, 0);
						return;
					}
				},
				afterInsertHtml: function(ev)
				{
					removeCkeSavedSrcAttributesFromDesignEditor();
				},
				beforeCommandExec: function(ev)
				{
					// Suppress any command (formatting, link dialog etc.) for tags (similar code found in doubleclick handler above).
					// Commmands can be triggered in multiple ways, e.g. using toolbar buttons, using keyboard shortcuts, and programmatically.
					var path = ev.editor.elementPath(); // Null if dialog button is triggered without placing text cursor in editor first
					if (path === null && ev.editor.getData().indexOf("<p><a data-tag-id=") === 0)
					{
						// Text cursor has not been placed in editor, but a command such as Bold or "insert image"
						// has been triggered, and editor content starts with a tag. This results in command being
						// applied to the tag, which we do not want. Usually this is prevented by the toolbar being
						// disabled when a tag is selected (see selectionChange event handler further up), but that
						// is not the case when the user has not yet placed the cursor in the editor.
						ev.cancel();
						return;
					}
					else if (path !== null && Fit.Dom.Data(path.lastElement.$, "tag-id") !== null && ev.data.name !== "undo") // Allow undo within tag, in case user typed something by mistake
					{
						// Cursor is currently placed in a tag - do not allow formatting
						ev.cancel();
						return;
					}

					if (ev && ev.data && ev.data.command && ev.data.command.dialogName)
					{
						// Command triggered was a dialog

						// IE9-IE11 does not fire OnFocus when user clicks a dialog button directly,
						// without placing the text cursor in the editing area first. To avoid this
						// problem, we simply ignore dialog commands if control does not already
						// have focus. We target all versions of IE for consistency.
						if (me.Focused() === false && Fit.Browser.GetBrowser() === "MSIE")
						{
							ev.cancel();
							return;
						}

						// Prevent multiple control instances from opening a dialog at the same time.
						// This is very unlikely to happen, as it requires the second dialog to be
						// triggered programmatically, since a modal layer is immediately placed on top
						// of the page when clicking a button that opens a dialog, preventing additional
						// interaction with editors.
						// Naturally conflicting CSS causing the modal layer to remain hidden could
						// allow the user to trigger multiple dialogs. Better safe than sorry.
						if (Fit._internal.Controls.Input.ActiveEditorForDialog)
						{
							ev.cancel();
							return;
						}

						// Make sure OnFocus fires before locking focus state

						if (me.Focused() === false)
						{
							// Control not focused - make sure OnFocus fires when a button is clicked,
							// and make sure ControlBase internally considers itself focused, so there is
							// no risk of OnFocus being fired twice without OnBlur firing in between,
							// when focus state is unlocked, and focus is perhaps re-assigned to another
							// DOM element within the control, which will be the case if the design editor
							// is switched back to an ordinary input field (e.g. using DesignMode(false)).
							me.Focused(true);
						}

						// Prevent control from firing OnBlur when dialogs are opened.
						// Notice that locking the focus state will also prevent OnFocus
						// from being fired automatically.
						me._internal.FocusStateLocked(true);

						// Make control available to global dialog event handlers which
						// cannot access individual control instances otherwise.

						Fit._internal.Controls.Input.ActiveEditorForDialog = me;	// Editor instance is needed when OnHide event is fired for dialog on global CKEditor instance
						Fit._internal.Controls.Input.ActiveDialogForEditor = null;	// Dialog instance associated with editor will be set when dialog's OnShow event fires
					}
				},
				menuShow: function(ev) // Fires when CKEditor's ContextMenu is opened, and when any sub menus are opened
				{
					me._internal.FocusStateLocked(true);

					if (designEditor.contextMenu.onHide === undefined) // Configure ContextMenu on first use - ContextMenu is shared amoung all editor instances
					{
						var ctxElm = ev.data[0].element.$;

						Fit.Events.AddHandler(ctxElm, "mousedown", true, function(e) // Capture phase (true argument) not supported by IE8 - too bad
						{
							Fit.Events.Stop(e); // Do not trigger click on ContextMenu's border as it triggers onHide and moves focus to <body> rather than editor
						});

						var returningFocus = false;

						designEditor.contextMenu.onHide = function()
						{
							// Quirks and noteworthy details related to CKEditor's ContextMenu and its onHide callback:
							// - ContextMenu is an iFrame which holds focus while open, and fires onHide when it lose focus.
							//   It's actually multiple iframes - one for the root items, and one for the sub items.
							// - ContextMenu is still visible when OnHide fires, and it still holds focus if an element within was clicked with the mouse or selected with the keyboard.
							// - When an item in the ContextMenu is triggered, it returns focus to the editor after OnHide has fired.
							// - When ContextMenu is dismissed (by clicking outside of ContextMenu or by pressing ESC), it moves focus to <body>, which causes Input.Onblur to fire unless suppressed.
							//   Once hidden, focus is moved to the element clicked with the mouse, or to the editor if the ContextMenu was dismissed using ESC.
							// - If the user clicks on the border of the ContextMenu, it doesn't return focus to the editor after temporarily focusing <body>.
							// - The onHide callback fires one time when ContextMenu is dismissed but twice when an item is selected/clicked (see me._internal.FocusStateLocked() check).
							// - The onHide callback is invoked before changes are made in editor, and before dialogs are opened.
							// - Manually re-focusing the editor (me.Focused(true)) during the execution of onHide sometimes result in immediate invocation of onHide again (mitigated using returningFocus check).
							// - ContextMenu is shared amoung all instances of CKEditor.
							// - Sometimes the first right-click triggers the browser's own ContextMenu rather than CKEditor's ContextMenu.
							// - The onHide callback is undocumented API - perhaps for good reasons.

							if (me._internal.FocusStateLocked() === false || returningFocus === true) // Skip redundant invocation - sometimes fired twice (see quirks documented above)
							{
								return;
							}

							var focusedElement = Fit.Dom.GetFocused();
							var contextMenuHasFocus = focusedElement.tagName === "IFRAME" && focusedElement.className.indexOf("cke_panel_frame") !== -1;

							if (contextMenuHasFocus === true) // ContextMenu has focus if use triggered one of the items
							{
								// Make sure focus is returned before focus state is unlocked - otherwise OnFocus will be fired when editor finish modification and returns focus to editor.
								// Calling Focused(true) when triggering a ContextMenu item sometimes causes onHide to fire again immediately - this is the case if triggering e.g. Table properties, but not when triggering e.g. Paste or Delete cell.
								returningFocus = true;
								me.Focused(true);
								returningFocus = false;
							}
							else // ContextMenu does not have focus - user probably clicked in editor or outside of editor to dismiss it
							{
								if (Fit.Dom.GetFocused() === document.body) // ContextMenu returns focus to body when dismissed, even when clicking in editor
								{
									me.Focused(true); // Return focus to editor to prevent Input.OnBlur from firing - once onHide has finished, CKEditor will focus the area clicked - and contrary to Focused(true) above, this does not cause onHide to immediately fire again
								}
								else // If something else (e.g. another control) has been given focus, then don't steal back focus, but fire OnBlur instead
								{
									me._internal.FireOnBlur();
								}
							}

							me._internal.FocusStateLocked(false);
						};

						designEditor.contextMenu.onHide._fitUiCallback = true;
					}
					else if (designEditor.contextMenu.onHide._fitUiCallback !== true) // Make sure we detect if another plugin starts using the onHide callback
					{
						throw "Unexpected use of ContextMenu.onHide event!";
					}
				}
			}
		});
	}

	function disableDesignEditorButtons() // Might be called multiple times, e.g. if navigating from one tag/mention to another - buttons must be disabled every time since CKEditor itself re-enable buttons when navigating elements in editor
	{
		var preserveButtonState = designEditorRestoreButtonState === null;

		if (preserveButtonState === true)
		{
			designEditorRestoreButtonState = {};
		}

		Fit.Array.ForEach(designEditor.toolbar, function(toolbarGroup)
		{
			var items = toolbarGroup.items;

			Fit.Array.ForEach(toolbarGroup.items, function(item)
			{
				if (item.command) // Buttons have a command identifier which can be used to resolve the actual command instance
				{
					var cmd = designEditor.getCommand(item.command);

					if (preserveButtonState === true && cmd.state !== CKEDITOR.TRISTATE_DISABLED) // https://ckeditor.com/docs/ckeditor4/latest/api/CKEDITOR_command.html#property-state
					{
						designEditorRestoreButtonState[item.command] = true;
					}

					cmd.disable();
				}
				else if (item.setState) // MenuButtons allow for direct manipulation of enabled/disabled state
				{
					if (preserveButtonState === true && item.getState() !== CKEDITOR.TRISTATE_DISABLED) // https://ckeditor.com/docs/ckeditor4/latest/api/CKEDITOR_command.html#property-state
					{
						designEditorRestoreButtonState[item.name] = item;
					}

					item.setState(CKEDITOR.TRISTATE_DISABLED);
				}
			});
		});
	}

	function restoreDesignEditorButtons()
	{
		if (designEditorRestoreButtonState !== null)
		{
			Fit.Array.ForEach(designEditorRestoreButtonState, function(commandKey)
			{
				if (designEditorRestoreButtonState[commandKey] === true) // Command button
				{
					var cmd = designEditor.getCommand(commandKey);
					cmd.enable();
				}
				else // MenuButton
				{
					designEditorRestoreButtonState[commandKey].setState(CKEDITOR.TRISTATE_OFF); // Enabled but not highlighted/activated like e.g. a bold button would be when selecting bold text
				}
			});

			designEditorRestoreButtonState = null;
		}
	};

	function updateDesignEditorSize()
	{
		if (me.DesignMode() === true && designEditorHeightMonitorId === -1)
		{
			// Postpone if editor is not ready yet

			if (designEditorUpdateSizeDebouncer !== -1)
			{
				clearTimeout(designEditorUpdateSizeDebouncer);
				designEditorUpdateSizeDebouncer = -1;
			}

			if (designModeEnabledAndReady() === false)
			{
				// Postpone, editor is not ready yet.
				// This may happen when editor is created and Width(..) is
				// immediately set after creating and mounting the control.
				// https://github.com/Jemt/Fit.UI/issues/34
				// This is a problem because CKEditor uses setTimeout(..) to for instance
				// allow early registration of events, and because resources are loaded
				// in an async. manner.
				designEditorUpdateSizeDebouncer = setTimeout(function() // Timer is stopped if control is disposed
				{
					designEditorUpdateSizeDebouncer = -1;
					updateDesignEditorSize();
				}, 100);

				return;
			}

			// Postpone update to editor size if control is currently hidden or not
			// rooted in DOM, in which case designEditor.resize(..) will throw an error.

			if (mutationObserverId !== -1) // Cancel any mutation observer previously registered
			{
				Fit.Events.RemoveMutationObserver(mutationObserverId);
				mutationObserverId = -1;
			}

			if (Fit.Dom.IsVisible(me.GetDomElement()) === false) // Hidden (e.g. display:none or not rooted in DOM)
			{
				// Mutation observer is triggered when element changes, including when rooted, in which case
				// width and height becomes measurable, and changes to dimensions also trigger mutation observer.
				mutationObserverId = Fit.Events.AddMutationObserver(me.GetDomElement(), function(elm)
				{
					if (Fit.Dom.IsVisible(me.GetDomElement()) === true)
					{
						disconnect();
						mutationObserverId = -1;

						updateDesignEditorSize(); // Does nothing if DesignMode is no longer enabled
					}
				});

				return;
			}

			//var w = me.Width();
			var h = me.Height();

			// If editor is configured with AutoGrow enabled and toolbar is configured with HideWhenInactive,
			// then editor won't be able to adjust its height when not focused, since a fixed height is applied
			// to the editable area while the toolbar is hidden. Therefore, temporarily show the toolbar, update
			// the editor size, and then hide the toolbar again.
			var showHideToolbar = me.Focused() === false;

			// Default control width is 200px (defined in Styles.css).
			// NOTICE: resize does not work reliably when editor is hidden, e.g. behind a tab with display:none.
			// The height set will not have the height of the toolbar substracted since the height can not be
			// determined for hidden objects, so the editor will become larger than the value set (height specified + toolbar height).
			// http://docs.ckeditor.com/#!/api/CKEDITOR.editor-method-resize
			designEditorSuppressOnResize = true;
			showHideToolbar && restoreHiddenToolbarInDesignEditor(true); // Does nothing unless HideWhenInactive is enabled - true argument prevents call back to updateDesignEditorSize again, hence preventing a "maximum call stack exceeded" error
			designEditor.resize("100%", h.Value > -1 ? h.Value + h.Unit : "100%"); // A height of 100% allow editor to automatically adjust the height of the editor's content area to the height of its content (data-autogrow="true" must be set to make control container adjust to its content as well)
			showHideToolbar && hideToolbarInDesignMode(true); // Does nothing unless HideWhenInactive is enabled - true argument prevents call back to updateDesignEditorSize again, hence preventing a "maximum call stack exceeded" error
			designEditorSuppressOnResize = false;
		}
	}

	function enableDesignEditorHeightMonitor(enable)
	{
		Fit.Validation.ExpectBoolean(enable);

		if (enable === true && designEditorHeightMonitorId === -1)
		{
			// Temporary fixed height might be set (see hideToolbarInDesignMode()).
			// Remove it - it will prevent editor from obtaining the height of the control container.
			designEditorDom.Editable.style.height = "";

			var toolbarContainer = designEditorDom.Top || designEditorDom.Bottom; // Top is null if editor is placed at the bottom

			var prevHeight = -1;
			var prevToolbarVisible = toolbarContainer.style.display === "";

			designEditorHeightMonitorId = setInterval(function()
			{
				var newHeight = me.GetDomElement().offsetHeight; // Returns 0 if element is not visible
				var newToolbarVisible = toolbarContainer.style.display === "";

				if (newHeight > 0 && (newHeight !== prevHeight || newToolbarVisible !== prevToolbarVisible))
				{
					prevHeight = newHeight;
					prevToolbarVisible = newToolbarVisible;

					designEditorSuppressOnResize = true;
					designEditor.resize("100%", newHeight + "px"); // Assume full width and height of control container - height:100% does not achieve this, so we apply height in pixels
					designEditorSuppressOnResize = false;
				}
			}, 250);
		}
		else if (enable === false && designEditorHeightMonitorId !== -1)
		{
			clearInterval(designEditorHeightMonitorId);
			designEditorHeightMonitorId = -1;
		}
	}

	function isToolbarHiddenInDesignEditor() // Returns True if editor is fully loaded and toolbar is hidden
	{
		var toolbarContainer = designModeEnabledAndReady() === true ? designEditorDom.Top || designEditorDom.Bottom : null; // Top is null if editor is placed at the bottom
		return (toolbarContainer !== null && toolbarContainer.style.display === "none");
	}

	function hideToolbarInDesignMode(suppressUpdateEditorSize)
	{
		Fit.Validation.ExpectBoolean(suppressUpdateEditorSize, true);

		if (designModeEnabledAndReady() === true && designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.HideWhenInactive === true)
		{
			var toolbarContainer = designEditorDom.Top || designEditorDom.Bottom; // Top is null if editor is placed at the bottom

			if (toolbarContainer.style.display === "none")
			{
				return; // Already hidden
			}

			// Prevent editor from increasing its height when toolbar is shown.
			// This is not ideal. We use the top/bottom's (toolbar's) height but it might change
			// if window is resized, which will cause buttons to "word wrap". But that is
			// acceptable. In this case the editor might change dimensions when toolbar is
			// shown and static height on content area is removed in OnFocus handler registered
			// in init().

			var updateSize = false;

			if (designEditorHeightMonitorId === -1) // Do not apply temporary fixed height if height monitor is running - in this case height will be adjusted as needed
			{
				var content = designEditorDom.Editable;
				content.style.height = toolbarContainer.offsetHeight + content.offsetHeight + "px";

				updateSize = true;
			}

			// Hide toolbar

			toolbarContainer.style.display = "none";

			me._internal.Data("toolbar", "false");

			// Make editable area adjust to take up space previously consumed by toolbar
			updateSize === true && suppressUpdateEditorSize !== true && updateDesignEditorSize();
		}
	}

	function restoreHiddenToolbarInDesignEditor(suppressUpdateEditorSize)
	{
		Fit.Validation.ExpectBoolean(suppressUpdateEditorSize, true);

		if (designModeEnabledAndReady() === true && designEditorConfig !== null && designEditorConfig.Toolbar && designEditorConfig.Toolbar.HideWhenInactive === true)
		{
			// Toolbar has been initially hidden - make it appear again

			var toolbarContainer = designEditorDom.Top || designEditorDom.Bottom; // Top is null if editor is placed at the bottom

			if (toolbarContainer.style.display === "")
			{
				return; // Already restored - no longer hidden
			}

			toolbarContainer.style.display = "";

			// Hiding the toolbar will reduce the height of the editor since the toolbar takes up place when shown.
			// Displaying the toolbar again later will naturally increase the editor's height again. To avoid this,
			// a fixed height is applied to the editable area when toolbar is hidden, so the editor remains the same
			// height. This fixed height is removed once the toolbar is shown again.
			// However, if the editor has been resized, then we need to use the resized height of the editor and reduce
			// the height of the editable area, so the toolbar can fit within the editor without increasing its height.
			// We must keep the size set by the user.

			if (me._internal.Data("resized") === "false")
			{
				// Remove fixed height from editable area. When toolbar is
				// shown again, editor will assume its normal height again.
				var content = designEditorDom.Editable;
				content.style.height = "";
			}
			else
			{
				// User has changed size of editor. Reduce height of editable area so that
				// the toolbar can fit within the editor without increasing the height of it.
				var content = designEditorDom.Editable;
				content.style.height = (content.offsetHeight - toolbarContainer.offsetHeight) + "px";
			}

			me._internal.Data("toolbar", "true");

			if (designEditorHeightMonitorId === -1)
			{
				// Update size of editable area in case auto grow is not enabled, in which case
				// toolbar will now have taken up space outside of control's container (overflowing).
				// Make editable area fit control container again.
				suppressUpdateEditorSize !== true && updateDesignEditorSize();
			}
			else
			{
				// Restart height monitor to force update to editor height.
				// Toolbar buttons caused editor to increase its height so it no longer
				// fits within the control container - the editor exceeds the boundaries.
				enableDesignEditorHeightMonitor(false);
				enableDesignEditorHeightMonitor(true);
			}
		}
	}

	function updateDesignEditorPlaceholder(clearPlaceholder)
	{
		Fit.Validation.ExpectBoolean(clearPlaceholder, true);

		if (designModeEnabledAndReady() === true)
		{
			if (Fit.Browser.GetBrowser() === "MSIE" && Fit.Browser.GetVersion() < 10)
			{
				// Native support for placeholders (using the real placeholder attribute) was
				// introduced in IE10, so we want to ensure consistent behaviour for all controls,
				// as e.g. Input and DatePicker uses the native placeholder implementation.
				return;
			}

			// WARNING: Retrieving value from editor is expensive! Do not
			// call updateDesignEditorPlaceholder() too often (e.g. OnChange).
			// Simply make sure placeholder is updated OnFocus and OnBlur.

			var val = clearPlaceholder !== true && me.Value() === "" ? me.Placeholder() : "";
			Fit.Dom.Data(designEditorDom.Editable, "placeholder", val || null);

			if (val !== "")
			{
				designEditorClearPlaceholder = true;
			}
		}
	}

	function openDetachedDesignEditor()
	{
		me._internal.FocusStateLocked(true);

		// Re-use previously created detached editor

		if (designEditorDetached !== null)
		{
			designEditorDetached.Open();
			return;
		}

		// Create dialog editor and buttons

		var de = new Fit.Controls.DialogEditor();
		var cmdOk = new Fit.Controls.Button();
		var cmdCancel = new Fit.Controls.Button();

		// Configure dialog

		var setSettings = function(preserveCustomSizeAndPosition)
		{
			// Editor configuration

			// Configure detached editor like original, but with a few required changes.
			// We need to make sure image blobs are handled properly, that detached editor
			// cannot created another detached editor, that the toolbar is initially visible
			// at the top of the dialog, and that auto grow is disabled.

			var deConfig = Fit.Core.Clone(designEditorConfig || {});

			if (designModeEnableImagePlugin() === true)
			{
				deConfig = Fit.Core.Merge(deConfig, // Override image plugin configuration
				{
					Plugins:
					{
						Images:
						{
							Enabled: true,
							EmbedType: deConfig.Plugins && deConfig.Plugins.Images && deConfig.Plugins.Images.EmbedType,

							// Image blobs added in detached editor must always be disposed if no longer referenced.
							// Furthermore we make sure image blobs originating from main editor are never disposed.
							// When detached editor is closed, images transfered from detached editor to main editor
							// are added to the main editor's index over image blobs so that the main editor becomes
							// responsible for the memory management of these.
							// If detached editor is closed without transfering changes (canceled), all images found
							// in the detached editor, which are not referenced in the main editor, are disposed.
							// See OnClick handlers for OK and Cancel buttons.

							RevokeBlobUrlsOnDispose: "UnreferencedOnly",	// Make dialog editor preserve newly added (and still referenced) image blobs when disposed
							RevokeExternalBlobUrlsOnDispose: false			// Make dialog editor preserve images blobs initially added from main editor
						}
					}
				});
			}

			deConfig.Toolbar = deConfig.Toolbar || {};
			deConfig.Toolbar.Detach = false;
			deConfig.Toolbar.Position = "Top";
			deConfig.Toolbar.Sticky = false;
			deConfig.Toolbar.HideWhenInactive = false;

			delete deConfig.AutoGrow;

			// Dialog configuration - apply default values for properties not defined

			var detachConfig = deConfig.Detachable || {};
			delete deConfig.Detachable;

			detachConfig = Fit.Core.Merge(detachConfig, // Apply default values - existing properties are preserved (Fit.Core.MergeOverwriteBehaviour.Never)
			{
				Title: "",
				Maximizable: true,
				Maximized: false,
				Draggable: true,
				Resizable: true,
				Width: detachConfig.Width ? detachConfig.Width : { Value: 850, Unit: "px" },
				MinimumWidth: detachConfig.MinimumWidth ? detachConfig.MinimumWidth : { Value: 20, Unit: "em" },
				MaximumWidth: detachConfig.MaximumWidth ? detachConfig.MaximumWidth : { Value: 100, Unit: "%" },
				Height: detachConfig.Height ? detachConfig.Height : { Value: 550, Unit: "px" },
				MinimumHeight: detachConfig.MinimumHeight ? detachConfig.MinimumHeight : { Value: 12, Unit: "em" },
				MaximumHeight: detachConfig.MaximumHeight ? detachConfig.MaximumHeight : { Value: 100, Unit: "%" }
			}, Fit.Core.MergeOverwriteBehaviour.Never);

			// Set dialog settings

			de.Title(detachConfig.Title);
			de.Modal(true);
			de.Draggable(detachConfig.Draggable);
			de.Resizable(detachConfig.Resizable);
			de.Maximizable(detachConfig.Maximizable);
			de.Maximized(detachConfig.Maximized);
			preserveCustomSizeAndPosition === false && de.Width(detachConfig.Width.Value, detachConfig.Width.Unit || "px");
			preserveCustomSizeAndPosition === false && de.Height(detachConfig.Height.Value, detachConfig.Height.Unit || "px");
			de.MinimumWidth(detachConfig.MinimumWidth.Value, detachConfig.MinimumWidth.Unit || "px");
			de.MinimumHeight(detachConfig.MinimumHeight.Value, detachConfig.MinimumHeight.Unit || "px");
			de.MaximumWidth(detachConfig.MaximumWidth.Value, detachConfig.MaximumWidth.Unit || "px");
			de.MaximumHeight(detachConfig.MaximumHeight.Value, detachConfig.MaximumHeight.Unit || "px");

			preserveCustomSizeAndPosition === false && de.Reset(); // Reset custom size (resized) and position (dragged)

			// Set dialog editor settings

			de.CheckSpelling(me.CheckSpelling());
			de._internal.SetDesignModeConfig(deConfig);
		};

		// Localization support

		var localizeDetachedEditor = function()
		{
			// Editor itself is already localized, so we just need to
			// localize the dialog. The locale variable will already have
			// been updated by the OnLocaleChanged handler registered in init().

			cmdOk.Title(locale.Ok);
			cmdCancel.Title(locale.Cancel);
		};
		Fit.Internationalization.OnLocaleChanged(localizeDetachedEditor);

		// Commit changes when pressing CTRL + S (Windows) or CMD + S (Mac)

		Fit.Events.AddHandler(de.GetDomElement(), "keydown", function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			if ((ev.ctrlKey === true || ev.metaKey === true) && ev.keyCode === 83) // CTRL/CMD + S
			{
				cmdOk.Click();
				Fit.Events.PreventDefault(ev);
			}
		});

		// Expose detached editor API

		designEditorDetached =
		{
			IsActive: false,

			GetValue: function()
			{
				return de.Value();
			},

			SetVisible: function(val)
			{
				// Focus state remains locked when toggling visibility.
				// We merely make sure to invoke OnBlur and OnFocus events.
				// Focus lock is only released when detached editor is closed
				// or if control is disposed.

				if (val === false && de.IsOpen() === true)
				{
					de.Close();
					me._internal.FireOnBlur();
				}
				else if (val === true && de.IsOpen() === false)
				{
					de.Open(); // Automatically brings focus to editor
					me._internal.FireOnFocus();
				}
			},

			SetEnabled: function(val)
			{
				if (val === false && de.Enabled() === true)
				{
					de.Enabled(false);
					cmdOk.Enabled(false);
					cmdCancel.Focused(true);
				}
				else if (val === true && de.Enabled() === false)
				{
					de.Enabled(true);
					cmdOk.Enabled(true);
					de.Focused(true);
				}
			},

			Focus: function()
			{
				if (de.Enabled() === true)
				{
					de.Focused(true);
				}
				else
				{
					cmdCancel.Focused(true);
				}
			},

			// GetFocused: function()
			// {
			// 	return de.Focused() === true /* also returns true if e.g. link/image dialog is open */
			// 		|| cmdOk.Focused() === true || cmdCancel.Focused() === true;
			// },

			Reload: function()
			{
				setSettings(true); // True argument = preserve custom size (resized) and position (dragged)
			},

			Open: function()
			{
				designEditorDetached.IsActive = true;
				de.Value(me.Value());
				setSettings(false); // False argument = reset custom size (resized) and position (dragged)
				de.Open();
			},

			Close: function()
			{
				designEditorDetached.IsActive = false;
				de.Close();
			},

			Dispose: function()
			{
				Fit.Internationalization.RemoveOnLocaleChanged(localizeDetachedEditor);
				de.Dispose(); // Will also dispose associated buttons
				designEditorDetached = null;
			}
		};

		cmdOk.Title(locale.Ok);
		cmdOk.Icon("check");
		cmdOk.Type(Fit.Controls.ButtonType.Success);
		cmdOk.OnClick(function(sender)
		{
			var referencedBlobUrls = Fit.String.ParseImageBlobUrls(de.Value());
			Fit.Array.ForEach(referencedBlobUrls, function(blobUrl)
			{
				if (Fit.Array.Contains(imageBlobUrls, blobUrl) === false)
				{
					Fit.Array.Add(imageBlobUrls, blobUrl);
				}
			});

			me.Value(de.Value());

			designEditorDetached.Close();

			me.Focused(true);
			me._internal.FocusStateLocked(false);
		});
		de.AddButton(cmdOk);

		cmdCancel.Title(locale.Cancel);
		cmdCancel.Icon("ban");
		cmdCancel.Type(Fit.Controls.ButtonType.Danger);
		cmdCancel.OnClick(function(sender)
		{
			var closeDialog = function()
			{
				var referencedBlobUrls = Fit.String.ParseImageBlobUrls(de.Value());
				Fit.Array.ForEach(referencedBlobUrls, function(blobUrl)
				{
					if (Fit.Array.Contains(imageBlobUrls, blobUrl) === false) // Only remove images added in dialog editor
					{
						URL.revokeObjectURL(blobUrl);
					}
				});

				var enabled = me.Enabled();
				designEditorDetached.Close(); // Close first so me.Focused(true) below does not redirect focus to detached editor

				if (enabled === true) // Return focus to control if it is still enabled - if not, do not return focus and fire OnBlur
				{
					me.Focused(true);
					me._internal.FocusStateLocked(false);
				}
				else
				{
					me._internal.FocusStateLocked(false);
					me._internal.FireOnBlur();
				}
			};

			if (de.Value() !== me.Value())
			{
				Fit.Controls.Dialog.Confirm(locale.CancelConfirmTitle + "<br><br>" + locale.CancelConfirmDescription, function(res)
				{
					if (res === true)
					{
						closeDialog();
					}
					else
					{
						cmdCancel.Focused(true);
					}
				});
			}
			else
			{
				closeDialog();
			}
		});
		de.AddButton(cmdCancel);

		designEditorDetached.Open();
	}

	function removeCkeSavedSrcAttributesFromDesignEditor() // Editor must be fully initialized when calling this function (designEditorDom is required)
	{
		// Remove data-cke-saved-src attributes from images - they contain a duplicate
		// value of the src attribute which might be very large when pasting base64 images.
		// Investigating the use of data-cke-saved-src (https://github.com/ckeditor/ckeditor4/search?q=cke-saved-src)
		// reveals that the data-cke-saved-src attribute is preferred over the src attribute
		// which makes sense if an image is temporarily replaced by a place holder, e.g. when uploading.
		// But we do not rely on such functionality, so the src attribute alone should be sufficient.
		// Also see related bug report: https://github.com/ckeditor/ckeditor4/issues/5151
		var imageTags = designEditorDom.Editable.querySelectorAll('img[data-cke-saved-src]');
		Fit.Array.ForEach(imageTags, function(img)
		{
			Fit.Dom.Data(img, "cke-saved-src", null);
		});
	}

	function isTable(element)
	{
		Fit.Validation.ExpectElement(element);

		var isTable = false;
		var elm = element;

		while (elm !== me.GetDomElement())
		{
			if (elm.tagName === "TABLE")
			{
				isTable = true;
				break;
			}

			elm = elm.parentElement;
		}

		return isTable;
	}

	function revertToSingleLineIfNecessary()
	{
		if (wasAutoChangedToMultiLineMode === true && me.Maximizable() === false && me.Resizable() === Fit.Controls.InputResizing.Disabled && me.DesignMode() === false)
		{
			me.MultiLine(false); // Changes wasAutoChangedToMultiLineMode to false
		}
	}

	function fireOnChange()
	{
		var newVal = me.Value();
		var compareValue = me.Type() === "Color" ? preVal.toUpperCase() : preVal; // Value() returns uppercase value for color picker - preVal might be in lower case if assigned before input type was changed

		if (newVal !== compareValue)
		{
			// DISABLED: No longer necessary with the introduction of designEditorDirty which ensures
			// that we get the initial value set from Value(), unless changed by the user using the editor.
			/*if (designEditor !== null && htmlWrappedInParagraph === false) // A value not wrapped in paragraph(s) was assigned to HTML editor
			{
				// Do not trigger OnChange if the only difference is that CKEditor
				// wrapped the value initially assigned to control in a paragraph.
				// Only changes made programmatically through the Input control's API
				// or by the user should be pushed.
				// This approach is not perfect unfortunately. For instance CKEditor
				// trims the value, so assigning " hello world" or " <p>Hello world</p>"
				// to the control will result in OnChange firing if fireOnChange() is called.

				var newValWithoutParagraph = newVal.replace(/^<p>/, "").replace(/<\/p>$/, ""); // Remove <p> and </p> at the beginning and end

				if (newValWithoutParagraph === preVal)
				{
					return; // Do not fire OnChange
				}
			}*/

			preVal = newVal;
			me._internal.FireOnChange();
		}
	}

	function reloadEditor(force, reloadConfig)
	{
		Fit.Validation.ExpectBoolean(force, true);
		Fit.Validation.ExpectObject(reloadConfig, true); // Not validated further, as it has already been validated in DesignMode(..)

		if (force !== true && (designModeEnabledAndReady() === false || designEditorMustReloadWhenReady === true))
		{
			// Attempting to reload editor while initializing - postpone until editor is fully loaded,
			// since we cannot guarantee reliable behavior with CKEditor if it's disposed while loading.
			designEditorMustReloadWhenReady = true;
			designEditorReloadConfig = reloadConfig || designEditorReloadConfig;
			return;
		}

		designEditorMustReloadWhenReady = false;

		// Disabling DesignMode brings it back to input or textarea mode.
		// If reverting to input mode, Height is reset, so we need to preserve that.

		// NOTICE: Custom width/height set using resize handle is not preserved when editor is reloaded

		var height = me.Height();
		var currentWasAutoChangedToMultiLineMode = wasAutoChangedToMultiLineMode; // DesignMode(false) will result in wasAutoChangedToMultiLineMode being set to false if DesignMode(true) changed the control to MultiLine mode

		// Prevent detached editor from being closed when reloading, e.g. if CheckSpelling is changed.
		// Editor will also be reloaded if a different editor configuration is passed to DesignMode(true, updatedConfig).
		var detachedEditor = null;
		if (designEditorDetached !== null)
		{
			detachedEditor = designEditorDetached;
			designEditorDetached = null; // Prevent me.DesignMode(false), which in turn calls destroyDesignEditorInstance(), from closing detached editor dialog
		}

		me.DesignMode(false);
		me.DesignMode(true, reloadConfig || designEditorReloadConfig || undefined); // Use reloadConfig if set (and if reload was not postponed) or use designEditorReloadConfig if reload was postponed with updated editor config
		designEditorReloadConfig = null;

		if (detachedEditor !== null)
		{
			designEditorDetached = detachedEditor;
			designEditorDetached.Reload(); // Reload detached editor to reflect any changes made to configuration
		}

		me.Height(height.Value, height.Unit);
		wasAutoChangedToMultiLineMode = currentWasAutoChangedToMultiLineMode;
	}

	function destroyDesignEditorInstance()
	{
		// Destroying editor also fires OnHide event for any dialog currently open, which will clean up:
		// Fit._internal.Controls.Input.ActiveEditorForDialog;
		// Fit._internal.Controls.Input.ActiveEditorForDialogDestroyed;
		// Fit._internal.Controls.Input.ActiveEditorForDialogDisabledPostponed;
		// Fit._internal.Controls.Input.ActiveDialogForEditor;
		// Fit._internal.Controls.Input.ActiveDialogForEditorCanceled;

		// Calling destroy() fires OnHide for any dialog currently open, which
		// in turn disables locked focus state and returns focus to the control.

		designEditor.destroy();

		if (designEditorDetached !== null)
		{
			designEditorDetached.Dispose();
		}

		if (designEditorGlobalKeyDownEventId !== -1)
		{
			Fit.Events.RemoveHandler(document, designEditorGlobalKeyDownEventId);
		}

		if (designEditorGlobalKeyUpEventId !== -1)
		{
			Fit.Events.RemoveHandler(document, designEditorGlobalKeyUpEventId);
		}

		designEditor = null;
		designEditorDom = null;
		//designEditorDirty = false; // Do NOT reset this! We need to preserve dirty state in case DesignMode is reloaded!
		designEditorDirtyPending = false;
		//designEditorConfig = null; // Do NOT nullify this! We need it, in case DesignMode is toggled!
		//designEditorReloadConfig = null; // Do NOT nullify this! We need it, in case DesignMode is reloaded!
		designEditorRestoreButtonState = null;
		designEditorSuppressPaste = false;
		designEditorSuppressOnResize = false;
		designEditorMustReloadWhenReady = false;
		designEditorMustDisposeWhenReady = false;
		designEditorActiveToolbarPanel = null;
		designEditorDetached = null;
		designEditorClearPlaceholder = true;
		designEditorCleanEditableDom = false;
		designEditorGlobalKeyDownEventId = -1;
		designEditorGlobalKeyUpEventId = -1;

		if (designEditorUpdateSizeDebouncer !== -1)
		{
			clearTimeout(designEditorUpdateSizeDebouncer);
			designEditorUpdateSizeDebouncer = -1;
		}

		if (designEditorHeightMonitorId !== -1)
		{
			clearInterval(designEditorHeightMonitorId);
			designEditorHeightMonitorId = -1;
		}

		if (mutationObserverId !== -1)
		{
			Fit.Events.RemoveMutationObserver(mutationObserverId);
			mutationObserverId = -1;
		}

		if (rootedEventId !== -1)
		{
			Fit.Events.RemoveHandler(me.GetDomElement(), rootedEventId);
			rootedEventId = -1;
		}

		if (createWhenReadyIntervalId !== -1)
		{
			clearInterval(createWhenReadyIntervalId);
			createWhenReadyIntervalId = -1;
		}
	}

	function designModeEnabledAndReady()
	{
		return designEditorDom !== null; // Editor is fully loaded when editor DOM is made available
	}

	function designModeEnableImagePlugin()
	{
		var config = designEditorConfig || {};
		var enableImagePlugin = (config.Plugins && config.Plugins.Images && config.Plugins.Images.Enabled === true) || (config.Toolbar && config.Toolbar.Images === true) || false;

		// Force enable image support if images are contained in value - otherwise editor will remove them
		if (enableImagePlugin === false && designEditorDetached !== null && designEditorDetached.GetValue().indexOf("<img ") > -1)
		{
			enableImagePlugin = true;
		}
		if (enableImagePlugin === false && me.Value().indexOf("<img ") > -1)
		{
			enableImagePlugin = true;
		}

		return enableImagePlugin;
	}

	function localize()
	{
		locale = Fit.Internationalization.GetLocale(me);

		if (me.DesignMode() === true)
		{
			// Prevent reloadEditor() from reloading detached editor.
			// It will automatically reload when locale is changed.
			// Without this guard the editor would reload twice.
			var detachedEditor = null;
			if (designEditorDetached !== null)
			{
				detachedEditor = designEditorDetached;
				designEditorDetached = null; // Prevent reloadEditor() from reloading detached editor
			}

			// Re-create editor with new language
			reloadEditor();

			if (detachedEditor !== null)
			{
				designEditorDetached = detachedEditor;
			}
		}
	}

	function repaint()
	{
		if (isIe8 === true)
		{
			me.AddCssClass("FitUi_Non_Existing_Input_Class");
			me.RemoveCssClass("FitUi_Non_Existing_Input_Class");
		}
	}

	init();
}

/// <container name="Fit.Controls.InputType">
/// 	Enum values determining input type
/// </container>
Fit.Controls.InputType =
{
	/// <member container="Fit.Controls.InputType" name="Textarea" access="public" static="true" type="string" default="Textarea">
	/// 	<description> Multi line input field </description>
	/// </member>
	Textarea: "Textarea",

	/// <member container="Fit.Controls.InputType" name="Color" access="public" static="true" type="string" default="Color">
	/// 	<description> Input control useful for entering a color </description>
	/// </member>
	Color: "Color",

	/// <member container="Fit.Controls.InputType" name="Date" access="public" static="true" type="string" default="Date">
	/// 	<description> Input control useful for entering a date </description>
	/// </member>
	Date: "Date",

	/// <member container="Fit.Controls.InputType" name="DateTime" access="public" static="true" type="string" default="DateTime">
	/// 	<description> Input control useful for entering a date and time </description>
	/// </member>
	DateTime: "DateTime",

	/// <member container="Fit.Controls.InputType" name="Email" access="public" static="true" type="string" default="Email">
	/// 	<description> Input control useful for entering an e-mail address </description>
	/// </member>
	Email: "Email",

	/// <member container="Fit.Controls.InputType" name="Month" access="public" static="true" type="string" default="Month">
	/// 	<description> Input control useful for entering a month </description>
	/// </member>
	Month: "Month",

	/// <member container="Fit.Controls.InputType" name="Number" access="public" static="true" type="string" default="Number">
	/// 	<description> Input control useful for entering a number </description>
	/// </member>
	Number: "Number",

	/// <member container="Fit.Controls.InputType" name="Password" access="public" static="true" type="string" default="Password">
	/// 	<description> Input control useful for entering a password (characters are masked) </description>
	/// </member>
	Password: "Password",

	/// <member container="Fit.Controls.InputType" name="PhoneNumber" access="public" static="true" type="string" default="PhoneNumber">
	/// 	<description> Input control useful for entering a phone number </description>
	/// </member>
	PhoneNumber: "PhoneNumber",

	/// <member container="Fit.Controls.InputType" name="Text" access="public" static="true" type="string" default="Text">
	/// 	<description> Input control useful for entering ordinary text </description>
	/// </member>
	Text: "Text",

	/// <member container="Fit.Controls.InputType" name="Time" access="public" static="true" type="string" default="Time">
	/// 	<description> Input control useful for entering time </description>
	/// </member>
	Time: "Time",

	/// <member container="Fit.Controls.InputType" name="Week" access="public" static="true" type="string" default="Week">
	/// 	<description> Input control useful for entering a week number </description>
	/// </member>
	Week: "Week",

	Unknown: "Unknown"
}

Fit.Controls.Input.Type = Fit.Controls.InputType; // Backward compatibility

/// <container name="Fit._internal.Controls.Input">
/// 	Allows for manipulating control (appearance, features, and behaviour).
/// 	Features are NOT guaranteed to be backward compatible, and incorrect use might break control!
/// </container>
Fit._internal.Controls.Input = {};

/// <container name="Fit._internal.Controls.Input.Editor">
/// 	Internal settings related to HTML Editor (Design Mode)
/// </container>
Fit._internal.Controls.Input.Editor =
{
	/// <member container="Fit._internal.Controls.Input.Editor" name="Skin" access="public" static="true" type="'bootstrapck' | 'moono-lisa' | null">
	/// 	<description> Skin used with DesignMode - must be set before an editor is created and cannot be changed for each individual control </description>
	/// </member>
	Skin: null // Notice: CKEditor does not support multiple different skins on the same page - do not change value once an editor has been created
};

/// <container name="Fit.Controls.InputResizing">
/// 	<description> Resizing options </description>
/// 	<member name="Enabled" access="public" static="true" type="string" default="Enabled"> Allow for resizing both vertically and horizontally </member>
/// 	<member name="Disabled" access="public" static="true" type="string" default="Disabled"> Do not allow resizing </member>
/// 	<member name="Horizontal" access="public" static="true" type="string" default="Horizontal"> Allow for horizontal resizing </member>
/// 	<member name="Vertical" access="public" static="true" type="string" default="Vertical"> Allow for vertical resizing </member>
/// </container>
Fit.Controls.InputResizing = // Enums must exist runtime
{
	Enabled: "Enabled",
	Disabled: "Disabled",
	Horizontal: "Horizontal",
	Vertical: "Vertical"
};
Fit.Internationalization.AddLocalization(Fit.Controls.Input,
{
	// Primary locales must be registered before country specific overrides.
	// Example order: en, en_GB, de, de_AT, etc.
	// All locales inherit from en. All country specific overrides inherit
	// from their general locale (e.g. de_AT inherits from de).
	// English (en) MUST be defined, and be defined first!

	"en":
	{
		"Detach": "Open in dialog",
		"Ok": "OK",
		"Cancel": "Cancel",
		"CancelConfirmTitle": "Discard changes?",
		"CancelConfirmDescription": "Are you sure you want to discard the changes?",

		// HTML editor - table plugins
		"TableTools": "Table tools",
		"CreateTable": "Create table",
		"NumberOfColumns": "Number of columns",
		"NumberOfRows": "Number of rows",
		"CreateTableTooLarge": "Table is getting too large - reduce number of columns or rows",
		"InsertRowAbove": "Insert row above",
		"InsertRowBelow": "Insert row below",
		"DeleteRow": "Delete row",
		"InsertColumnBefore": "Insert column before",
		"InsertColumnAfter": "Insert column after",
		"DeleteColumn": "Delete column",
		"DeleteTable": "Delete table"
	},
	"da":
	{
		"Detach": "Åben i dialog",
		"Ok": "OK",
		"Cancel": "Annullér",
		"CancelConfirmTitle": "Annullér ændringer?",
		"CancelConfirmDescription": "Er du sikker på at du vil annullere ændringerne?",

		// HTML editor - table plugins
		"TableTools": "Tabelværktøjer",
		"CreateTable": "Opret tabel",
		"NumberOfColumns": "Antal kolonner",
		"NumberOfRows": "Antal rækker",
		"CreateTableTooLarge": "Tabellen bliver for stor - reducer antal kolonner eller rækker",
		"InsertRowAbove": "Indsæt række over",
		"InsertRowBelow": "Indsæt række under",
		"DeleteRow": "Slet række",
		"InsertColumnBefore": "Indsæt kolonne før",
		"InsertColumnAfter": "Indsæt kolonne efter",
		"DeleteColumn": "Slet kolonne",
		"DeleteTable": "Slet tabel"
	},
	"de":
	{
		"Detach": "Im Dialog öffnen",
		"Ok": "OK",
		"Cancel": "Abbrechen",
		"CancelConfirmTitle": "Änderungen verwerfen?",
		"CancelConfirmDescription": "Möchten Sie die Änderungen wirklich verwerfen?",

		// HTML editor - table plugins
		"TableTools": "Tabellenwerkzeuge",
		"CreateTable": "Tabelle erstellen",
		"NumberOfColumns": "Anzahl der Spalten",
		"NumberOfRows": "Anzahl der Zeilen",
		"CreateTableTooLarge": "Tabelle wird zu groß - reduzieren Sie die Anzahl der Spalten oder Zeilen",
		"InsertRowAbove": "Zeile oben einfügen",
		"InsertRowBelow": "Zeile unten einfügen",
		"DeleteRow": "Zeile löschen",
		"InsertColumnBefore": "Spalte davor einfügen",
		"InsertColumnAfter": "Spalte danach einfügen",
		"DeleteColumn": "Spalte löschen",
		"DeleteTable": "Tabelle löschen"
	},
	"no":
	{
		"Detach": "Åpne i dialog",
		"Ok": "OK",
		"Cancel": "Avbryt",
		"CancelConfirmTitle": "Avbryt endringer?",
		"CancelConfirmDescription": "Er du sikker på at du vil avbryte endringene?",

		// HTML editor - table plugins
		"TableTools": "Tabellverktøy",
		"CreateTable": "Opprett tabell",
		"NumberOfColumns": "Antall kolonner",
		"NumberOfRows": "Antall rader",
		"CreateTableTooLarge": "Tabellen blir for stor - reduser antall kolonner eller rader",
		"InsertRowAbove": "Sett inn rad over",
		"InsertRowBelow": "Sett inn rad under",
		"DeleteRow": "Slett rad",
		"InsertColumnBefore": "Sett inn kolonne før",
		"InsertColumnAfter": "Sett inn kolonne etter",
		"DeleteColumn": "Slett kolonne",
		"DeleteTable": "Slett tabell"
	}
});

/// <container name="Fit.Controls.ListView" extends="Fit.Controls.PickerBase;Fit.Controls.Component">
/// 	Picker control which allows for entries
/// 	to be selected in the DropDown control.
/// </container>

/// <function container="Fit.Controls.ListView" name="ListView" access="public">
/// 	<description> Create instance of ListView control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.ListView = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.PickerBase).Apply();
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	var me = this;
	var list = me.GetDomElement();
	var active = null;
	var persistView = false;
	var scrollPositionTop = 0;
	var highlightFirst = false;
	var firstWasHighlighted = false;
	var htmlAllowed = false;
	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);

	var onSelectHandlers = [];
	var onSelectedHandlers = [];

	function init()
	{
		list.tabIndex = "0";
		Fit.Dom.AddClass(list, "FitUiControlListView");

		me.OnShow(function()
		{
			if (persistView === false) // Reset selection and scroll position
			{
				if (highlightFirst === true)
				{
					focusFirstItem();
				}
				else
				{
					setActive(null);
				}

				list.scrollTop = 0;
			}
			else // View persisted
			{
				if (highlightFirst === true && firstWasHighlighted === false)
				{
					focusFirstItem();
					firstWasHighlighted = true;
				}

				list.scrollTop = scrollPositionTop;
			}
		});

		list.onscroll = function(e)
		{
			// Preserve scroll position which is lost if picker control is removed from DOM
			scrollPositionTop = list.scrollTop;
		};

		list.onclick = function(e)
		{
			var ev = Fit.Events.GetEvent(e);
			var elm = Fit.Events.GetTarget(e);

			if (elm === list)
				return;

			while (elm.parentElement !== list)
				elm = elm.parentElement;

			setActive(elm, true); // true = do not scroll into view - it already is

			// Fire OnChanging and OnChange events

			var item = convertItemElementToObject(elm);
			var selectionCanceled = fireOnSelectHandlers(item) === false;

			// Notice: We always pass False as current selection state to OnItemSelectionChanging since ListView does
			// not keep track of selection state. In theory item could very well already be selected in host control.
			// Event handlers should not trust boolean to reveal selection in host control, only in picker.
			if (selectionCanceled === false && me._internal.FireOnItemSelectionChanging(item.Title, item.Value, false, false) === true)
			{
				fireOnSelectedHandlers(item);

				me._internal.FireOnItemSelectionChanged(item.Title, item.Value, true, false);
				me._internal.FireOnItemSelectionComplete();
			}
		}

		list.onfocus = function(e)
		{
			me._internal.FireOnFocusIn(); // Fire OnFocusIn on PickerBase

			// Skip if this was a mouse click, leave handling to OnClick which fires after OnFocus.
			// Notice that Fit.Events.GetPointerState() is used to determine whether mouse button
			// is active since this information is not accessible through Event instance passed in OnFocus.
			if (Fit.Events.GetPointerState().Buttons.Primary === true)
				return;

			// Implemented Tab and Shift+Tab support below rather than using built-in support to prevent
			// the need for an OnFocus event handler on every single item contained, which would otherwise
			// be necessary to intercept focus and make element active.

			if (Fit.Events.GetModifierKeys().Shift === true) // Shift+Tab: Navigate up/backwards
			{
				if (list.children.length > 0)
					setActive(list.children[list.children.length - 1]);
			}
			else // Tab: Navigate down/forward
			{
				if (list.children.length > 0)
					setActive(list.children[0]);
			}
		}

		list.onblur = function(e)
		{
			setActive(null);
			me._internal.FireOnFocusOut(); // Fire OnFocusOut on PickerBase
		}

		list.onkeydown = function(e)
		{
			var ev = Fit.Events.GetEvent(e);

			// Handle Tab navigation manually to prevent the need to
			// implement a onfocus handler on every single item, responsible
			// for selecting an item when it gains focus.

			if (ev.keyCode === 9 && ev.shiftKey === false) // Tab
			{
				if (active !== list.children[list.children.length - 1])
				{
					moveDown();

					// Prevent native tab navigation from moving focus to next focusable element
					Fit.Events.PreventDefault(ev);
					return false;
				}
			}
			else if (ev.keyCode === 9 && ev.shiftKey === true) // Shift + Tab
			{
				if (active !== list.firstChild)
				{
					moveUp();

					// Prevent native tab navigation from moving focus to any focusable element above
					Fit.Events.PreventDefault(ev);
					return false;
				}
			}
			else
			{
				me.HandleEvent(ev); // Handle arrow up/down and enter
			}
		}
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.ListView" name="HtmlAllowed" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether HTML is allowed (shown) in ListView items </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables support for HTML, False disables it </param>
	/// </function>
	this.HtmlAllowed = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && htmlAllowed !== val)
		{
			htmlAllowed = val;

			// Update view

			Fit.Array.ForEach(list.children, function(entry)
			{
				if (val === false)
				{
					Fit.Dom.Text(entry, Fit.String.StripHtml(entry._orgTitle));
				}
				else
				{
					entry.innerHTML = entry._orgTitle;
				}
			});
		}

		return htmlAllowed;
	}

	/// <function container="Fit.Controls.ListView" name="AddItem" access="public">
	/// 	<description> Add item to ListView </description>
	/// 	<param name="title" type="string"> Item title </param>
	/// 	<param name="value" type="string"> Item value </param>
	/// </function>
	this.AddItem = function(title, value)
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);

		var entry = document.createElement("div");
		entry._orgTitle = title;

		if (me.HtmlAllowed() === false)
		{
			Fit.Dom.Text(entry, Fit.String.StripHtml(title));
		}
		else
		{
			entry.innerHTML = title;
		}

		Fit.Dom.Data(entry, "value", encode(value));
		Fit.Dom.Data(entry, "active", "false");

		list.appendChild(entry);
	}

	/// <container name="Fit.Controls.ListViewTypeDefs.ListViewItem">
	/// 	<description> ListView item </description>
	/// 	<member name="Title" type="string"> Item title </member>
	/// 	<member name="Value" type="string"> Unique item value </member>
	/// </container>

	/// <function container="Fit.Controls.ListView" name="GetItem" access="public" returns="Fit.Controls.ListViewTypeDefs.ListViewItem | null">
	/// 	<description> Get item by value - returns object with Title (string) and Value (string) properties if found, otherwise Null </description>
	/// 	<param name="value" type="string"> Value of item to retrieve </param>
	/// </function>
	this.GetItem = function(value)
	{
		Fit.Validation.ExpectString(value);

		var itemElm = getItemElement(value);

		if (itemElm !== null)
		{
			return convertItemElementToObject(itemElm);
		}

		return null;
	}

	/// <function container="Fit.Controls.ListView" name="GetItems" access="public" returns="Fit.Controls.ListViewTypeDefs.ListViewItem[]">
	/// 	<description> Get all items - returns array containing objects with Title (string) and Value (string) properties </description>
	/// </function>
	this.GetItems = function()
	{
		var items = [];

		Fit.Array.ForEach(list.children, function(child)
		{
			Fit.Array.Add(items, convertItemElementToObject(child));
		});

		return items;
	}

	/// <function container="Fit.Controls.ListView" name="HasItem" access="public" returns="boolean">
	/// 	<description> Returns value indicating whether control contains item with specified value </description>
	/// 	<param name="value" type="string"> Value of item to check for </param>
	/// </function>
	this.HasItem = function(value)
	{
		Fit.Validation.ExpectString(value);
		return getItemElement(value) !== null;
	}

	/// <function container="Fit.Controls.ListView" name="RemoveItem" access="public">
	/// 	<description> Remove item from ListView </description>
	/// 	<param name="value" type="string"> Value of item to remove </param>
	/// </function>
	this.RemoveItem = function(value)
	{
		Fit.Validation.ExpectString(value);

		var item = getItemElement(value);

		if (item !== null)
		{
			Fit.Dom.Remove(item);

			if (item === active)
			{
				active = null;
			}
		}
	}

	/// <function container="Fit.Controls.ListView" name="RemoveItems" access="public">
	/// 	<description> Remove all items from ListView </description>
	/// </function>
	this.RemoveItems = function()
	{
		list.innerHTML = "";
		setActive(null);
	}

	// ============================================
	// PickerBase interface
	// ============================================

	this.GetItemByValue = function(val)
	{
		Fit.Validation.ExpectString(val);
		return me.GetItem(val);
	}

	this.RevealItemInView = function(val)
	{
		Fit.Validation.ExpectString(val);

		var itemElm = getItemElement(val);

		if (itemElm !== null)
		{
			setActive(itemElm);
		}
	}

	this.HandleEvent = function(e)
	{
		Fit.Validation.ExpectEvent(e, true);

		var ev = Fit.Events.GetEvent(e);

		if (ev.type === "keydown")
		{
			if (ev.keyCode === 38) // arrow up
			{
				moveUp();

				// Prevent scrollable div from scrolling up
				Fit.Events.PreventDefault(ev);
			}
			else if (ev.keyCode === 40) // arrow down
			{
				moveDown();

				// Prevent scrollable div from scrolling down
				Fit.Events.PreventDefault(ev);
			}
			else if (ev.keyCode === 13) // enter
			{
				if (active === null && list.children.length === 1)
					moveDown(); // Select first item if no item is selected

				if (active !== null)
				{
					var item = convertItemElementToObject(active);
					var selectionCanceled = fireOnSelectHandlers(item) === false;

					// Notice: We always pass False as current selection state to OnItemSelectionChanging since ListView does
					// not keep track of selection state. In theory item could very well already be selected in host control.
					// Event handlers should not trust boolean to reveal selection in host control, only in picker.
					if (selectionCanceled === false && me._internal.FireOnItemSelectionChanging(item.Title, item.Value, false, false) === true)
					{
						fireOnSelectedHandlers(item);

						me._internal.FireOnItemSelectionChanged(item.Title, item.Value, true, false);
						me._internal.FireOnItemSelectionComplete();
					}
				}

				// Prevent form submit
				Fit.Events.PreventDefault(ev);
			}
		}
	}

	this.GetHighlighted = function()
	{
		if (active !== null)
		{
			return convertItemElementToObject(active);
		}

		return null;
	}

	this.PersistView = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== persistView)
		{
			persistView = val;
			scrollPositionTop = 0;
		}

		return persistView;
	}

	this.HighlightFirst = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val !== highlightFirst)
			{
				highlightFirst = val;
				firstWasHighlighted = false;
			}

			// Allow external code to force focus first item if picker is visible.
			// Usually first item is automatically highlighted when the host control
			// is opened (see OnShow handler in init()).
			if (val === true && Fit.Dom.IsVisible(me.GetDomElement()) === true)
			{
				focusFirstItem();
				firstWasHighlighted = true;
			}
		}

		return highlightFirst;
	}

	/// <function container="Fit.Controls.ListViewTypeDefs" name="OnSelectEventHandler" returns="boolean | void">
	/// 	<description> OnSelect event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="item" type="{ Title: string, Value: string }"> Selected item </param>
	/// </function>

	/// <function container="Fit.Controls.ListView" name="OnSelect" access="public">
	/// 	<description>
	/// 		Register event handler fired when item is being selected.
	/// 		Selection can be canceled by returning False.
	/// 		The following arguments are passed to event handler function:
	/// 		Sender (ListView) and Item (with Title (string) and Value (string) properties).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ListViewTypeDefs.OnSelectEventHandler"> Event handler function </param>
	/// </function>
	this.OnSelect = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectHandlers, cb);
	}

	/// <function container="Fit.Controls.ListViewTypeDefs" name="OnSelectedEventHandler">
	/// 	<description> OnSelected event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="item" type="{ Title: string, Value: string }"> Selected item </param>
	/// </function>

	/// <function container="Fit.Controls.ListView" name="OnSelected" access="public">
	/// 	<description>
	/// 		Register event handler fired when item is selected.
	/// 		The following arguments are passed to event handler function:
	/// 		Sender (ListView) and Item (with Title (string) and Value (string) properties).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.ListViewTypeDefs.OnSelectedEventHandler"> Event handler function </param>
	/// </function>
	this.OnSelected = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectedHandlers, cb);
	}

	this.Destroy = Fit.Core.CreateOverride(this.Destroy, function(calledInternally)
	{
		Fit.Validation.ExpectBoolean(calledInternally, true);

		// This will destroy control - it will no longer work!

		if (calledInternally !== true)
		{
			me.Dispose(true); // Component.Dispose()
		}

		base(); // PickerBase.Destroy()
	});

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function(calledInternally)
	{
		Fit.Validation.ExpectBoolean(calledInternally, true);

		base(); // Component.Dispose()

		if (calledInternally !== true)
		{
			me.Destroy(true); // PickerBase.Destroy()
		}

		me = list = active = persistView = scrollPositionTop = highlightFirst = firstWasHighlighted = htmlAllowed = isIe8 = onSelectHandlers = onSelectedHandlers = null;
	});

	// ============================================
	// Protected
	// ============================================

	this._internal = (this._internal ? this._internal : {});

	this._internal.FocusFirstItem = function() // Similar implementation in Fit.Controls.TreeView._internal.FocusFirstNode()
	{
		// If picker is still visible (it might have been hidden if user closed
		// host control while data was being loaded/populated async.) then focus first item.
		if (Fit.Dom.IsVisible(me.GetDomElement()) === true)
		{
			focusFirstItem();
			firstWasHighlighted = true;
		}
	}

	// ============================================
	// Private
	// ============================================

	function getItemElement(value)
	{
		Fit.Validation.ExpectString(value);

		var found = null;

		Fit.Array.ForEach(list.children, function(child)
		{
			if (decode(Fit.Dom.Data(child, "value")) === value)
			{
				found = child;
				return false;
			}
		});

		return found;
	}

	function convertItemElementToObject(elm)
	{
		Fit.Validation.ExpectDomElement(elm);
		return { Title: elm._orgTitle, Value: decode(Fit.Dom.Data(elm, "value")) };
	}

	function setActive(elm, suppressScrollIntoView)
	{
		Fit.Validation.ExpectDomElement(elm, true);
		Fit.Validation.ExpectBoolean(suppressScrollIntoView, true);

		if (active !== null)
			Fit.Dom.Data(active, "active", "false");

		active = (elm ? elm : null);

		if (active !== null)
		{
			Fit.Dom.Data(active, "active", "true");

			if (suppressScrollIntoView !== true)
			{
				list.scrollTop = active.offsetHeight * Fit.Dom.GetIndex(active); // Alternative to active.scrollIntoView(true) which unfortunately also scrolls main view
				repaint();
			}
		}
	}

	function focusFirstItem()
	{
		if (list.children.length > 0)
		{
			setActive(list.children[0]);
		}
	}

	function moveUp()
	{
		if (list.children.length === 0)
			return;

		// Select item

		if (active === null) // Select first entry if no selection is made
		{
			setActive(list.firstChild);
		}
		else if (active.previousSibling !== null) // Select previous entry if available
		{
			setActive(active.previousSibling);
		}
	}

	function moveDown()
	{
		if (list.children.length === 0)
			return;

		// Select item

		if (active === null) // Select first entry if no selection is made
		{
			setActive(list.firstChild);
		}
		else if (active.nextSibling !== null) // Select next entry if available
		{
			setActive(active.nextSibling);
		}
	}

	function fireOnSelectHandlers(item)
	{
		Fit.Validation.ExpectObject(item);
		Fit.Validation.ExpectString(item.Title);
		Fit.Validation.ExpectString(item.Value);

		var selectionCanceled = false;

		Fit.Array.ForEach(onSelectHandlers, function(handler)
		{
			if (handler(me, item) === false)
			{
				selectionCanceled = true;
			}
		});

		return selectionCanceled === true ? false : true; // Return False if canceled
	}

	function fireOnSelectedHandlers(item)
	{
		Fit.Validation.ExpectObject(item);
		Fit.Validation.ExpectString(item.Title);
		Fit.Validation.ExpectString(item.Value);

		Fit.Array.ForEach(onSelectedHandlers, function(handler)
		{
			handler(me, item);
		});
	}

	function repaint()
	{
		if (isIe8 === true)
		{
			Fit.Dom.AddClass(list, "FitUi_Non_Existing_ListView_Class");
			Fit.Dom.RemoveClass(list, "FitUi_Non_Existing_ListView_Class");
		}
	}

	function decode(str)
	{
		Fit.Validation.ExpectString(str);
		return decodeURIComponent(str);
	}

	function encode(str)
	{
		Fit.Validation.ExpectString(str);
		return encodeURIComponent(str);
	}

	init();
}

/// <container name="Fit.Controls.WSListView" extends="Fit.Controls.ListView">
/// 	WebService enabled picker control which allows for entries
/// 	to be selected in the DropDown control.
/// </container>

/// <function container="Fit.Controls.WSListView" name="WSListView" access="public">
/// 	<description> Create instance of WSListView control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.WSListView = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.ListView).Apply(ctlId);

	var me = this;
	var url = null;
	var jsonpCallback = null;
	var dataLoading = false;
	var onDataLoadedCallback = [];
	var onRequestHandlers = [];
	var onResponseHandlers = [];
	var onAbortHandlers = [];
	var onPopulatedHandlers = [];

	function init()
	{
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.WSListView" name="Url" access="public" returns="string">
	/// 	<description>
	/// 		Get/set URL to WebService responsible for providing data to control.
	/// 		WebService must deliver data in the following JSON format:
	/// 		[
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 1", Value: "1001", Selectable: true, Children: [] },
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 2", Value: "1002", Selectable: false, Children: [] }
	/// 		]
	/// 		Only Value is required. Children is a collection of items with the same format as described above.
	/// 		Be aware that items are treated as a flat list, even when hierarchically structured using the Children property.
	/// 		Items with Selectable set to False will simply be ignored (not shown) while children will still be added.
	/// 	</description>
	/// 	<param name="wsUrl" type="string" default="undefined"> If defined, updates WebService URL (e.g. http://server/ws/data.asxm/GetItems) </param>
	/// </function>
	this.Url = function(wsUrl)
	{
		Fit.Validation.ExpectString(wsUrl, true);

		if (Fit.Validation.IsSet(wsUrl) === true)
		{
			url = wsUrl;
		}

		return url;
	}

	/// <function container="Fit.Controls.WSListView" name="JsonpCallback" access="public" returns="string | null">
	/// 	<description>
	/// 		Get/set name of JSONP callback argument. Assigning a value will enable JSONP communication.
	/// 		Often this argument is simply &quot;callback&quot;. Passing Null disables JSONP communication again.
	/// 	</description>
	/// 	<param name="val" type="string | null" default="undefined"> If defined, enables JSONP and updates JSONP callback argument </param>
	/// </function>
	this.JsonpCallback = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true || val === null)
		{
			jsonpCallback = val;
		}

		return jsonpCallback;
	}

	/// <function container="Fit.Controls.WSListViewTypeDefs" name="ReloadCallback">
	/// 	<description> Reload callback </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.WSListView" name="Reload" access="public">
	/// 	<description> Load/reload data from WebService </description>
	/// 	<param name="cb" type="Fit.Controls.WSListViewTypeDefs.ReloadCallback" default="undefined">
	/// 		If defined, callback function is invoked when data has been loaded
	/// 		and populated - takes Sender (Fit.Controls.WSListView) as an argument.
	/// 	</param>
	/// </function>
	this.Reload = function(cb)
	{
		Fit.Validation.ExpectFunction(cb, true);

		if (dataLoading === true)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.Reload(cb); });
			return;
		}

		dataLoading = true;

		getData(function() // Callback is invoked when nodes are populated, but before OnPopulated is fired by getData(..)
		{
			dataLoading = false;

			if (Fit.Validation.IsSet(cb) === true)
			{
				cb(me);
			}

			fireOnDataLoaded();
		});
	}

	// See documentation on PickerBase
	this.Destroy = Fit.Core.CreateOverride(this.Destroy, function()
	{
		// This will destroy control - it will no longer work!

		me = url = jsonpCallback = dataLoading = onDataLoadedCallback = onRequestHandlers = onResponseHandlers = onAbortHandlers = onPopulatedHandlers = null;
		base();
	});

	// ============================================
	// Events
	// ============================================

	/// <container name="Fit.Controls.WSListViewTypeDefs.EventHandlerArgs">
	/// 	<description> Event handler arguments </description>
	/// 	<member name="Sender" type="$TypeOfThis"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonRequest | Fit.Http.JsonpRequest"> Instance of JsonRequest or JsonpRequest </member>
	/// </container>

	/// <container name="Fit.Controls.WSListViewTypeDefs.JsonItem">
	/// 	<description> JSON list item </description>
	/// 	<member name="Value" type="string"> Unique value </member>
	/// 	<member name="Title" type="string" default="undefined"> Title - using Value if not defined </member>
	/// 	<member name="Selectable" type="boolean" default="undefined"> Value indicating whether item can be selected or not - selectable by default </member>
	/// </container>

	/// <container name="Fit.Controls.WSListViewTypeDefs.DataHandlerEventArgs" extends="Fit.Controls.WSListViewTypeDefs.EventHandlerArgs">
	/// 	<description> Data event handler arguments </description>
	/// 	<member name="Items" type="Fit.Controls.WSListViewTypeDefs.JsonItem[]"> JSON list items </member>
	/// </container>

	/// <container name="Fit.Controls.WSListViewTypeDefs.AbortHandlerEventArgs" extends="Fit.Controls.WSListViewTypeDefs.EventHandlerArgs">
	/// 	<description> Abort event handler arguments </description>
	/// 	<member name="Items" type="null"> JSON list items </member>
	/// </container>

	/// <function container="Fit.Controls.WSListViewTypeDefs" name="CancelableRequestEventHandler" returns="boolean | void">
	/// 	<description> Request event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSListViewTypeDefs.EventHandlerArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSListViewTypeDefs" name="DataEventHandler">
	/// 	<description> Response event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSListViewTypeDefs.DataHandlerEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSListViewTypeDefs" name="AbortEventHandler">
	/// 	<description> Abort event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSListViewTypeDefs.AbortHandlerEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSListView" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is being requested.
	/// 		Request can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSListView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSListView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSListViewTypeDefs.CancelableRequestEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnRequest = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onRequestHandlers, cb);
	}

	/// <function container="Fit.Controls.WSListView" name="OnResponse" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is received,
	/// 		allowing for data transformation to occure before
	/// 		ListView is populated. Function receives two arguments:
	/// 		Sender (Fit.Controls.WSListView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSListView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Items: JSON items received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSListViewTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnResponse = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onResponseHandlers, cb);
	}

	/// <function container="Fit.Controls.WSListView" name="OnAbort" access="public">
	/// 	<description>
	/// 		Add event handler fired if data request is canceled.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSListView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSListView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Items: JSON items received from WebService (Null in this particular case)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSListViewTypeDefs.AbortEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnAbort = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onAbortHandlers, cb);
	}

	/// <function container="Fit.Controls.WSListView" name="OnPopulated" access="public">
	/// 	<description>
	/// 		Add event handler fired when ListView has been populated with items.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSListView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSListView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Items: JSON items received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSListViewTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnPopulated = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onPopulatedHandlers, cb);
	}

	// ============================================
	// Private
	// ============================================

	function getData(cb)
	{
		Fit.Validation.ExpectFunction(cb, true);

		if (url === null)
			Fit.Validation.ThrowError("Unable to get data, no WebService URL has been specified");

		// Determine request method

		var request = null;

		if (jsonpCallback !== null)
		{
			request = new Fit.Http.JsonpRequest(url, jsonpCallback);
		}
		else
		{
			request = new Fit.Http.JsonRequest(url)
		}

		// Fire OnRequest

		var eventArgs = { Sender: me, Request: request, Items: null };

		if (fireEventHandlers(onRequestHandlers, eventArgs) === false)
			return;

		if (eventArgs.Request !== request)
		{
			// Support for changing request instans to
			// take control over webservice communication.

			// Restrict to support for Fit.Http.Request or classes derived from this
			Fit.Validation.ExpectInstance(eventArgs.Request, Fit.Http.Request);

			request = eventArgs.Request;
		}

		// Define request callbacks

		var onSuccess = function(data)
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			// Fire OnResponse

			eventArgs.Items = ((data instanceof Array) ? data : []);
			fireEventHandlers(onResponseHandlers, eventArgs);

			// Populate ListView

			me.RemoveItems();

			Fit.Array.ForEach(eventArgs.Items, function(item)
			{
				populate(item);
			});

			// Highlight first item

			if (me.PersistView() === true)
			{
				// Reset PersistView to make sure it does not reuse state
				// for newly loaded items, in case item list is changed.
				me.PersistView(false);
				me.PersistView(true);
			}

			if (me.HighlightFirst() === true)
			{
				// Reset HighlightFirst to make sure it takes effect again
				// for newly loaded items, in case item list is changed.
				me.HighlightFirst(false);
				me.HighlightFirst(true);

				me._internal.FocusFirstItem();
			}

			// Invoke callback

			if (Fit.Validation.IsSet(cb) === true)
			{
				cb();
			}

			// Fire OnPopulated

			fireEventHandlers(onPopulatedHandlers, eventArgs);
		}

		var onFailure = function(httpStatusCode)
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			Fit.Validation.ThrowError("Unable to get data - request failed with HTTP Status code " + httpStatusCode)
		};

		var onAbort = function()
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			fireEventHandlers(onAbortHandlers, eventArgs);
		}

		// Register request callbacks

		if (Fit.Core.InstanceOf(request, Fit.Http.JsonpRequest) === false)
		{
			request.OnSuccess(function(req)
			{
				var data = request.GetResponseJson();
				onSuccess(data);
			});

			request.OnFailure(function(req)
			{
				onFailure(request.GetHttpStatus());
			});

			request.OnAbort(function(req)
			{
				onAbort();
			});
		}
		else
		{
			request.OnSuccess(function(req)
			{
				var children = req.GetResponse();
				onSuccess(children);
			});

			request.OnTimeout(function()
			{
				onFailure("UNKNOWN (JSONP)");
			});
		}

		// Invoke request

		request.Start();
	}

	function onDataLoaded(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onDataLoadedCallback, cb);
	}

	function fireOnDataLoaded()
	{
		// Copied from WSTreeView.
		// Immediately clear collection. If multiple callbacks are registered,
		// chances are that only the first will run, and the remaining will be
		// re-scheduled again - so we need the collection to be cleared before
		// invoking callbacks.
		var orgOnDataLoadedCallback = onDataLoadedCallback;
		onDataLoadedCallback = [];

		Fit.Array.ForEach(orgOnDataLoadedCallback, function(cb)
		{
			cb();
		});
	}

	function populate(jsonItem)
	{
		Fit.Validation.ExpectIsSet(jsonItem);
		Fit.Validation.ExpectString(jsonItem.Value);
		Fit.Validation.ExpectString(jsonItem.Title, true);
		Fit.Validation.ExpectBoolean(jsonItem.Selectable, true);
		Fit.Validation.ExpectArray(jsonItem.Children, true);

		if (jsonItem.Selectable !== false)
			me.AddItem((jsonItem.Title ? jsonItem.Title : jsonItem.Value), jsonItem.Value);

		if (jsonItem.Children)
		{
			Fit.Array.ForEach(jsonItem.Children, function(c)
			{
				populate(c);
			});
		}
	}

	function fireEventHandlers(handlers, eventArgs)
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (cb(me, eventArgs) === false)
				cancel = true; // Do not cancel loop - all handlers must be fired!
		});

		return !cancel;
	}

	init();
}

/// <container name="Fit.Controls.ProgressBar" extends="Fit.Controls.Component">
/// 	ProgressBar control useful for indicating progress.
/// </container>

/// <function container="Fit.Controls.ProgressBar" name="ProgressBar" access="public">
/// 	<description> Create instance of ProgressBar control </description>
/// 	<param name="controlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.ProgressBar = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	// Internals

	var me = this;
	var id = me.GetId();
	var element = me.GetDomElement();
	var status = null;
	var title = "";
	var width = { Value: 200, Unit: "px" }; // Any changes to this line must be dublicated to Width(..)
	var onProgressHandlers = [];

	function init()
	{
		Fit.Dom.AddClass(element, "FitUiControl");
		Fit.Dom.AddClass(element, "FitUiControlProgressBar");

		status = document.createElement("div");
		status.style.width = "0%";
		Fit.Dom.AddClass(status, "FitUiControlProgressBarStatus");
		Fit.Dom.Add(element, status);

		title = document.createElement("span");
		Fit.Dom.Add(status, title);
	}

	/// <function container="Fit.Controls.ProgressBar" name="Title" access="public" returns="string">
	/// 	<description> Get/set title in progress bar </description>
	/// 	<param name="val" type="string" default="undefined"> If specified, title will be set to specified value </param>
	/// </function>
	this.Title = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			title.innerHTML = val;
		}

		return title.innerHTML;
	}

	/// <function container="Fit.Controls.ProgressBar" name="Width" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control width - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control width is updated to specified value. A value of -1 resets control width. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control width is updated to specified CSS unit </param>
	/// </function>
	this.Width = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				width = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				element.style.width = width.Value + width.Unit;
			}
			else
			{
				width = { Value: 200, Unit: "px" }; // Any changes to this line must be dublicated to line declaring the width variable !
				element.style.width = "";
			}
		}

		return width;
	}

	/// <function container="Fit.Controls.ProgressBar" name="Progress" access="public" returns="integer">
	/// 	<description> Get/set progress - a value between 0 and 100 </description>
	/// 	<param name="val" type="integer" default="undefined"> If defined, progress is set to specified value (0-100) </param>
	/// </function>
	this.Progress = function(val)
	{
		Fit.Validation.ExpectInteger(val, true);

		if (Fit.Validation.IsSet(val) === true && val >= 0 && val <= 100 && status.style.width !== val + "%")
		{
			status.style.width = val + "%";

			// Fire OnProgress event

			Fit.Array.ForEach(onProgressHandlers, function(cb)
			{
				cb(me);
			});
		}

		return parseInt(status.style.width);
	}

	/// <function container="Fit.Controls.ProgressBarTypeDefs" name="ProgressEventHandler">
	/// 	<description> Progress event handler </description>
	/// 	<param name="sender" type="Fit.Controls.ProgressBar"> Instance of ProgressBar </param>
	/// </function>

	/// <function container="Fit.Controls.ProgressBar" name="OnProgress" access="public">
	/// 	<description> Set callback function invoked when progress is changed </description>
	/// 	<param name="cb" type="Fit.Controls.ProgressBarTypeDefs.ProgressEventHandler">
	/// 		Event handler invoked when progress is changed - takes progress bar instance as argument
	/// 	</param>
	/// </function>
	this.OnProgress = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onProgressHandlers, cb);
	}

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		me = id = element = status = title = width = onProgressHandlers = null;
		base();
	});

	init();
}

/// <container name="Fit.Controls.SoftLog" extends="Fit.Controls.Component">
/// 	The SoftLog is an alternative to the browser's console log which allows for debugging on devices without developer tools.
/// </container>

/// <function container="Fit.Controls.SoftLog" name="SoftLog" access="public">
/// 	<description> Create instance of SoftLog control </description>
/// 	<param name="controlId" type="string" default="undefined">
/// 		Unique control ID that can be used to access control using Fit.Controls.Find(..)
/// 	</param>
/// </function>
Fit.Controls.SoftLog = function(controlId)
{
	Fit.Validation.ExpectStringValue(controlId, true);
	Fit.Core.Extend(this, Fit.Controls.Component).Apply(controlId);

	var me = this;
	var txt = null;
	var maxEntries = 100;
	var height = { Value: -1, Unit: "px" } // Any changes to this line must be dublicated to Height(..)
	var orgConsoleLog = null;
	var exceptionEventHandlerId = -1;

	function init()
	{
		var dom = me.GetDomElement();

		Fit.Dom.AddClass(dom, "FitUiControl");
		Fit.Dom.AddClass(dom, "FitUiControlSoftLog");

		txt = document.createElement("textarea");
		txt.readOnly = true;
		Fit.Dom.Add(dom, txt);
	}

	/// <function container="Fit.Controls.SoftLog" name="CatchUncaughtExceptions" access="public" returns="boolean">
	/// 	<description> Get/set value determining whether to catch unhandled exceptions </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables exception logging, False disables it </param>
	/// </function>
	this.CatchUncaughtExceptions = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && exceptionEventHandlerId === -1)
			{
				exceptionEventHandlerId = Fit.Events.AddHandler(window, "error", function(e) // IE10+ is required (https://developer.mozilla.org/en-US/docs/Web/API/ErrorEvent)
				{
					var ev = Fit.Events.GetEvent(e);
					me.Log(ev.message + " (" + ev.filename + ":" + ev.lineno + ")");
				});
			}
			else if (val === false && exceptionEventHandlerId !== -1)
			{
				Fit.Events.RemoveHandler(window, exceptionEventHandlerId);
				exceptionEventHandlerId = -1;
			}
		}

		return exceptionEventHandlerId !== -1;
	}

	/// <function container="Fit.Controls.SoftLog" name="InterceptConsoleLog" access="public" returns="boolean">
	/// 	<description> Get/set value determining whether to intercept console.log(..) </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables console.log(..) interception, False disables it </param>
	/// </function>
	this.InterceptConsoleLog = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true && orgConsoleLog === null)
			{
				orgConsoleLog = window.console.log;

				window.console.log = function()
				{
					orgConsoleLog.apply(this, arguments);

					Fit.Array.ForEach(arguments, function(arg)
					{
						if (Fit.Array.Contains(["string", "number", "function", "boolean"], typeof(arg)))
						{
							me.Log(arg.toString());
						}
						else if (arg === null || arg === undefined)
						{
							me.Log(arg + "");
						}
						else if (arg.tagName && arg.outerHTML)
						{
							me.Log(arg.cloneNode(false).outerHTML);
						}
						else
						{
							me.Log(JSON.stringify(arg));
						}
					});
				}
			}
			else if (val === false && orgConsoleLog !== null)
			{
				window.console.log = orgConsoleLog;
				orgConsoleLog = null;
			}
		}

		return orgConsoleLog !== null;
	}

	/// <function container="Fit.Controls.SoftLog" name="MaxEntries" access="public" returns="integer">
	/// 	<description> Get/set number of log entries preserved </description>
	/// 	<param name="val" type="integer" default="undefined"> If defined, changes number of log entries preserved </param>
	/// </function>
	this.MaxEntries = function(val)
	{
		Fit.Validation.ExpectInteger(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > 0)
			{
				maxEntries = val;
				trimLog(false);
			}
			else
			{
				maxEntries = -1;
			}
		}

		return maxEntries;
	}

	/// <function container="Fit.Controls.SoftLog" name="Height" access="public" returns="Fit.TypeDefs.CssValue">
	/// 	<description> Get/set control height - returns object with Value and Unit properties </description>
	/// 	<param name="val" type="number" default="undefined"> If defined, control height is updated to specified value. A value of -1 resets control height. </param>
	/// 	<param name="unit" type="Fit.TypeDefs.CssUnit" default="px"> If defined, control height is updated to specified CSS unit </param>
	/// </function>
	this.Height = function(val, unit)
	{
		Fit.Validation.ExpectNumber(val, true);
		Fit.Validation.ExpectStringValue(unit, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val > -1)
			{
				height = { Value: val, Unit: ((Fit.Validation.IsSet(unit) === true) ? unit : "px") };
				txt.style.height = height.Value + height.Unit;
			}
			else
			{
				height = { Value: -1, Unit: "px" }; // Any changes to this line must be dublicated to line declaring the height variable !
				txt.style.height = "";
			}
		}

		return height;
	}

	/// <function container="Fit.Controls.SoftLog" name="Log" access="public">
	/// 	<description> Log message </description>
	/// 	<param name="msg" type="string"> Text message to log </param>
	/// </function>
	this.Log = function(msg)
	{
		Fit.Validation.ExpectString(msg);

		var dt = new Date();
		var ms = dt.getMilliseconds();
		var time = Fit.Date.Format(dt, "hh:mm:ss") + "." + (ms === 0 ? "000" : ms < 10 ? "00" + ms : ms < 100 ? "0" + ms : ms);

		trimLog(true);

		txt.value += (txt.value !== "" ? "\n" : "") + time + ": " + msg.replace(/\r/g, "").replace(/\n/g, " ");
		txt.scrollLeft = 0;
		txt.scrollTop = 999999;
	}

	/// <function container="Fit.Controls.SoftLog" name="Clear" access="public">
	/// 	<description> Clear log </description>
	/// </function>
	this.Clear = function()
	{
		txt.value = "";
	}

	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		if (orgConsoleLog !== null)
		{
			window.console.log = orgConsoleLog;
		}

		if (exceptionEventHandlerId !== -1)
		{
			Fit.Events.RemoveHandler(window, exceptionEventHandlerId);
		}

		me = txt = maxEntries = height = orgConsoleLog = exceptionEventHandlerId = null;

		base();
	});

	function trimLog(leaveRoomForNewLogEntry)
	{
		Fit.Validation.ExpectBoolean(leaveRoomForNewLogEntry);

		if (maxEntries > 0)
		{
			var messages = txt.value !== "" ? txt.value.split("\n") : [];

			if (messages.length >= maxEntries)
			{
				messages = maxEntries === 1 && leaveRoomForNewLogEntry === true ? [] : messages.slice((maxEntries - (leaveRoomForNewLogEntry === true ? 1 : 0)) * -1);
			}

			txt.value = messages.join("\n");
		}
	}

	init();
}
/// <container name="Fit.Controls.TreeView" extends="Fit.Controls.PickerBase;Fit.Controls.ControlBase">
/// 	TreeView control allowing data to be listed in a structured manner.
/// 	Extending from Fit.Controls.PickerBase.
/// 	Extending from Fit.Controls.ControlBase.
///
/// 	Performance considerations (for huge amounts of data):
///
/// 	1) Selectable(..) is used to transform how nodes allow selections
/// 	   (disabled/single select/multi select/select all). This requires the function to
/// 	   recursively modify all nodes contained to make sure they are configured identically.
/// 	   However, this also happens when AddChild(node) is called, to make sure
/// 	   nodes added at any time is configured in accordance with TreeView configuration.
/// 	   Selectable(..) should therefore be called before adding nodes to prevent
/// 	   an extra recursive operation on all nodes contained.
///
/// 	2) Selected(nodes) performs better than Value("val1;val2;val3")
///
/// 	3) RemoveChild(node) performance is non-linear, relative to the amount of children contained.
/// 	   The function recursively iterates children to find selected nodes to deselect them, to
/// 	   make sure TreeView is updated accordingly.
///
/// 	4) GetChild("val1", true) is faster at finding one specific node, compared to recursively
/// 	   iterating the result from GetChildren(), since internal children collections are indexed.
///
/// 	5) Be aware that some operations (e.g. AddChild, Expand/Collapse, Select/Deselect) forces
/// 	   Internet Explorer 8 to repaint tree to work around render bugs. Repainting can be minimized
/// 	   greately by populating root nodes before adding them to the TreeView instance.
/// 	   However, be aware that this comes with the performance penalty mentioned in article 1 (AddChild).
/// 	   It is likely that repainting does not pose a major performance problem, though.
/// </container>

/// <function container="Fit.Controls.TreeView" name="TreeView" access="public">
/// 	<description> Create instance of TreeView control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.TreeView = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.PickerBase).Apply();
	Fit.Core.Extend(this, Fit.Controls.ControlBase).Apply(ctlId);

	var me = this;
	var rootContainer = null;		// UL element
	var rootNode = null;			// Fit.Controls.TreeViewNode instance
	var focusInEventId = -1;		// ID for OnFocus event handler registered at the document level

	var keyNavigationEnabled = true;

	var selectable = false;
	var multiSelect = false;
	var showSelectAll = false; // TBD: Never implemented - can be achieved using ContextMenu. Remove or implement?
	var allowDeselect = true;
	var revealExpandedNodes = false;
	var persistView = false;
	var scrollPosition = { X: 0, Y: 0 };
	var highlightFirst = false;
	var firstWasHighlighted = false;

	var selected = createInternalCollection();
	var selectedOrg = [];

	var ctx = null; // Context menu

	var onSelectHandlers = [];
	var onSelectedHandlers = [];
	var onToggleHandlers = [];
	var onToggledHandlers = [];
	var onSelectAllHandlers = [];
	var onSelectAllCompleteHandlers = [];
	var onContextMenuHandlers = [];

	var forceClear = false;
	var isIe8 = (Fit.Browser.GetInfo().Name === "MSIE" && Fit.Browser.GetInfo().Version === 8);

	// ============================================
	// Init
	// ============================================

	function init()
	{
		// Initial settings

		me.AddCssClass("FitUiControlTreeView");
		me._internal.Data("selectable", "false");
		me._internal.Data("multiselect", "false");
		me._internal.Data("wordwrap", "false");
		me._internal.Data("picker", "false");
		me._internal.Data("htmlallowed", "false");

		// Create internal root node to hold children

		rootContainer = document.createElement("ul");
		me._internal.AddDomElement(rootContainer);

		rootNode = new Fit.Controls.TreeViewNode(" ", "TREEVIEW_ROOT_NODE");
		rootNode.GetDomElement().tabIndex = -1;
		rootContainer.appendChild(rootNode.GetDomElement());

		// TreeView Node Interface:
		// The Tree View Node Interface allow nodes to synchronize their selection state to the TreeView.
		// That way the TreeView never has to recursively iterate a potentially huge tree to determine
		// which nodes are selected, to clear all nodes, etc.
		// One might wonder why the TreeView does not maintain the internal selected collection itself
		//  - after all, all event handling is performed by the TreeView, so it knows when a node is
		//    selected. The reason for this, of course, is that nodes can be selected programmatically as well.
		// The TreeView Node Interface also ensures that once a node is attached to the TreeView, any children added to that
		// particular node is configured accordingly with the existing nodes in the TreeView (Selectable, Checkbox, ShowSelectAll).

		var treeViewNodeInterface =
		{
			Select: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

				// Deselect selected node if Multi Selection is not enabled
				if (multiSelect === false && selected.length > 0)
					executeWithNoOnChange(function() { selected[0].Selected(false); });

				// Add node to internal selection
				Fit.Array.Add(selected, node);
			},
			Deselect: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

				// Remove node from internal selection
				Fit.Array.Remove(selected, node);
			},
			Selected: function(val)
			{
				Fit.Validation.ExpectArray(val, true);
				return me.Selected(val); // Fires OnChange
			},
			IsSelectable: function()
			{
				return selectable;
			},
			IsMultiSelect: function()
			{
				return multiSelect;
			},
			ShowSelectAll: function()
			{
				return showSelectAll;
			},
			Repaint: function()
			{
				repaint();
			},
			FireToggle: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
				return fireEventHandlers(onToggleHandlers, node);
			},
			FireToggled: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
				return fireEventHandlers(onToggledHandlers, node);
			},
			FireSelect: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

				// FireSelect is invoked prior to node change.
				// If TreeView is in Single Selection Mode, and a non-selectable node is already selected,
				// new selection is canceled by returning False, to keep existing selection.
				// However, we never want to cancel change if TreeView.Clear() is invoked, in which case forceClear is true.
				if (forceClear === false && multiSelect === false && selected.length > 0 && selected[0].Selectable() === false)
					return false;

				return fireEventHandlers(onSelectHandlers, node);
			},
			FireSelected: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
				fireEventHandlers(onSelectedHandlers, node);
			},
			FireOnChange: function()
			{
				me._internal.FireOnChange();
			},
			GetTreeView: function()
			{
				return me;
			},
			Dissociate: function(node)
			{
				Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

				if (node === activeNode)
				{
					Fit.Dom.Data(activeNode.GetDomElement(), "active", null);
					activeNode = null;
				}
			}
		}

		rootNode.GetDomElement()._internal.TreeView = treeViewNodeInterface;

		// Event handling

		rootNode.GetDomElement().onkeydown = function(e)
		{
			var ev = Fit.Events.GetEvent(e);
			var elm = Fit.Events.GetTarget(e);

			if (keyNavigationEnabled === false)
				return;

			//{ PickerControl support - START

			// If used as picker control, make sure first node is selected on initial key press (activeNode has not yet been set)
			if (isPicker === true && isNodeSet(activeNode) === false)
			{
				focusFirstNode(); // Sets activeNode to first child
				return;
			}

			// If onkeydown was not fired from a list item (which usually has focus),
			// it was most likely fired from a host control through HandleEvent(..).
			// In this case activeNode will be set.
			if (elm.tagName !== "LI" && isPicker === true && isNodeSet(activeNode) === true)
				elm = activeNode.GetDomElement();

			//} // PickerControl support - END

			// Variable elm above is almost always a <li> node element since we make sure only these elements are focused
			// when node is clicked or keyboard navigation occure. However, IF the node title for some strange reason contains an element
			// that gains focus (e.g. an input control), that element could be contained in the elm variable when onkeydown is fired.
			// We already make sure that <a> link elements do not gain focus which is more likely to be used as node title, so there
			// is almost no chance the elm variable will ever contain anything else but the <li> node element. But better safe than sorry.
			if (elm.tagName !== "LI")
				return;

			// Element is root node if user clicked next to a child node within root node element
			if (elm === rootNode.GetDomElement())
			{
				if (rootNode.GetChildren().length > 0)
					focusNode(rootNode.GetChildren()[0]); // Select first node

				Fit.Events.PreventDefault(ev);
				return;
			}

			var node = elm._internal.Node;

			// Handle context menu

			if (ev.keyCode === 93 || (Fit.Events.GetModifierKeys().Shift === true && ev.keyCode === 121)) // Context menu button or Shift+F10
			{
				var label = node.GetDomElement().querySelector("span");

				var pos = Fit.Dom.GetPosition(label);
				var scrollContainers = Fit.Dom.GetScrollPosition(label); // TreeView may be placed in container(s) with scroll
				var scrollDocument = Fit.Dom.GetScrollPosition(document.body); // Page may have been scrolled

				pos.X = (pos.X - scrollContainers.X) + scrollDocument.X + label.offsetWidth - 10;
				pos.Y = (pos.Y - scrollContainers.Y) + scrollDocument.Y + label.offsetHeight - 5;

				openContextMenu(node, pos);

				Fit.Events.PreventDefault(ev);
				return;
			}

			// Handle navigation links

			if (ev.keyCode === 13) // Enter
			{
				if (node.Selectable() === true && me.Enabled() === true)
				{
					node.Selected(true);
				}
				else
				{
					// Navigate link contained in item

					var links = node.GetDomElement().getElementsByTagName("a");

					if (links.length === 1)
					{
						if (links[0].click) // Some browsers, e.g. Safari 5, does not support click()
						{
							links[0].click();
						}
						else
						{
							var clickEvent = document.createEvent("MouseEvent");
							clickEvent.initEvent("click", true, true);
							links[0].dispatchEvent(clickEvent);
						}
					}
				}

				Fit.Events.PreventDefault(ev);
				return;
			}

			// Toggle selection

			if (ev.keyCode === 32) // Spacebar (toggle selection)
			{
				if (node.Selectable() === true && me.Enabled() === true)
					toggleNodeSelection(node);

				Fit.Events.PreventDefault(ev);
				return;
			}

			// Determine which node to focus next on keyboard navigation (arrow keys)

			var next = null;

			if (ev.keyCode === 37) // Left (collapse or move to parent)
			{
				// Collapse node if expanded - if not expanded, jump to parent node

				if (node.Expanded() === true)
				{
					node.Expanded(false);
				}
				else
				{
					next = node.GetParent();
				}

				Fit.Events.PreventDefault(ev);
			}
			else if (ev.keyCode === 39) // Right (expand)
			{
				// Expand node if not already expanded, and children are contained

				if (node.Expanded() === false && node.GetChildren().length > 0)
					node.Expanded(true);

				Fit.Events.PreventDefault(ev);
			}
			else if (ev.keyCode === 38) // Up
			{
				// Move selection up
				next = getNodeAbove(node);

				Fit.Events.PreventDefault(ev);
			}
			else if (ev.keyCode === 40) // Down
			{
				// Move selection down
				next = getNodeBelow(node);

				Fit.Events.PreventDefault(ev);
			}

			// Focus next node

			if (next !== null)
				focusNode(next);
		}

		rootNode.GetDomElement().onclick = function(e)
		{
			var elm = Fit.Events.GetTarget(e);

			// If element was a node's checkbox, cancel change to selection,
			// and let node handle whether it should be checked or not.
			if (elm.tagName === "INPUT" && elm.parentElement.tagName === "LI")
				elm.checked = !elm.checked;

			var nodeElm = ((elm.tagName === "LI") ? elm : Fit.Dom.GetParentOfType(elm, "LI"));
			var node = nodeElm._internal.Node;

			// Focus node
			focusNode(node);

			// Toggle node if expand button was clicked
			if (elm === node.GetDomElement().children[0])
				node.Expanded(!node.Expanded());

			// Toggle selection or perform auto postback if node is selectable
			if (elm !== node.GetDomElement().children[0] && node.Selectable() === true)
			{
				toggleNodeSelection(node);
			}
		}

		// Clicking the TreeView's scrollbar causes it to lose focus to a parent container if it is focusable (has tabindex).
		// IE not affected though. We detect when user clicks on the TreeView's scrollable container and then reclaims focus
		// when OnFocus fires at the document root level. Since OnFocus is handled in the capture phase at the document
		// root level, it happens early enough to prevent the control from firing its OnBlur event.
		if (Fit.Browser.GetBrowser() !== "MSIE")
		{
			var reclaimFocus = false;

			Fit.Events.AddHandler(me.GetDomElement(), "mousedown", true, function(e)
			{
				if (Fit.Events.GetTarget(e) === me.GetDomElement()) // True if TreeView container was clicked, including its scrollbar
				{
					// Only reclaim focus if a container further up the hierarchy is focusable (has tabindex)

					var parent = me.GetDomElement();
					while ((parent = parent.parentElement) !== null)
					{
						if (Fit.Dom.Attribute(parent, "tabindex") !== null) // We cannot check against parent.tabIndex as it defaults to -1 when not set
						{
							reclaimFocus = true;
							break;
						}
					}
				}
			});

			focusInEventId = Fit.Events.AddHandler(document, "focus", true, function(e)
			{
				if (reclaimFocus === true)
				{
					reclaimFocus = false; // MUST be set immediately as code below changes focus and fires OnFocus for the document yet again

					// Fire PickerBase.OnFocusIn event - this causes DropDown to steal back focus - search for picker.OnFocusIn(..)
					// DISABLED - depends on the knowledge of DropDown specific behaviour which we should not rely on! Now focusing
					// root node instead (see below), which in turn also results in invocation of PickerBase.OnFocusIn since TreeView
					// invokes this whenever it gains focus - search for this.OnFocus(function(sender)...
					//me._internal.FireOnFocusIn();

					// Focus was lost to an outer container with tabindex. Move focus back to control.
					// We do not use me.Focused(true) since it returns focus to a previously focused
					// node, which will cause scroll position to change.
					// This will also fire the PickerBase.OnFocusIn event since TreeView fires this event
					// whenever it gains focus. This allows e.g. DropDown to yet again move focus to its
					// primary input field.
					rootNode.GetDomElement().focus();
				}
			});
		}

		Fit.Events.AddHandler(me.GetDomElement(), "contextmenu", function(e) { return Fit.Events.PreventDefault(e); }); // Disable context menu
		Fit.Events.AddHandler(me.GetDomElement(), "mouseup", function(e) // Select All feature
		{
			if (Fit.Events.GetPointerState().Buttons.Secondary === true) // Right click
			{
				var target = Fit.Events.GetTarget(e);

				if (target !== me.GetDomElement()) // Skip if right clicking TreeView container (possible if padding is applied)
				{
					var node = ((target.tagName === "LI") ? target._internal.Node : Fit.Dom.GetParentOfType(target, "li")._internal.Node);
					openContextMenu(node);
				}

				return Fit.Events.PreventDefault(e);
			}
		});

		var touchTimeout = -1;
		Fit.Events.AddHandler(me.GetDomElement(), "touchstart", function(e)
		{
			var target = Fit.Events.GetTarget(e);

			if (target !== me.GetDomElement()) // Skip if touching TreeView container (possible if padding is applied)
			{
				touchTimeout = setTimeout(function()
				{
					if (me === null)
					{
						return; // Control was disposed while holding down finger to trigger ContextMenu (unlikely though)
					}

					var node = ((target.tagName === "LI") ? target._internal.Node : Fit.Dom.GetParentOfType(target, "li")._internal.Node);
					openContextMenu(node);

					touchTimeout = -1;
				}, 500);
			}
		});
		Fit.Events.AddHandler(me.GetDomElement(), "touchend", function(e)
		{
			var elm = Fit.Events.GetTarget(e);

			if (touchTimeout !== -1)
			{
				clearTimeout(touchTimeout);
				touchTimeout = -1;
			}
		});

		Fit.Events.AddHandler(me.GetDomElement(), "scroll", function(e)
		{
			// Preserve scroll position which is lost if picker control is removed from DOM
			scrollPosition.X = me.GetDomElement().scrollLeft;
			scrollPosition.Y = me.GetDomElement().scrollTop;
		});
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.TreeView" name="WordWrap" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether Word Wrapping is enabled </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables Word Wrapping, False disables it </param>
	/// </function>
	this.WordWrap = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			me._internal.Data("wordwrap", val.toString()); // Actual word-wrapping is achieved using CSS
			repaint();
		}

		return (me._internal.Data("wordwrap") === "true");
	}

	/// <function container="Fit.Controls.TreeView" name="HtmlAllowed" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether HTML is allowed (shown) in TreeView nodes </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables support for HTML, False disables it </param>
	/// </function>
	this.HtmlAllowed = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			me._internal.Data("htmlallowed", val.toString());

			// Update view for all nodes recursively

			var nodes = rootNode.GetChildren();

			Fit.Array.ForEach(nodes, function(node)
			{
				node.HtmlAllowed(val, true); // Recursive
			});
		}

		return (me._internal.Data("htmlallowed") === "true");
	}

	/// <function container="Fit.Controls.TreeView" name="Selectable" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether user can change selection state of nodes.
	/// 		This affects all contained nodes. To configure nodes
	/// 		individually, use Selectable(..) function on node instances.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables node selection, False disables it </param>
	/// 	<param name="multi" type="boolean" default="false"> If defined, True enables node multi selection, False disables it </param>
	/* 	<param name="showSelAll" type="boolean" default="false"> If defined, True enables Select All checkbox, False disables it </param> */
	/// </function>
	this.Selectable = function(val, multi, showSelAll)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(multi, true);
		Fit.Validation.ExpectBoolean(showSelAll, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			selectable = val;
			multiSelect = ((multi === true) ? true : false);
			showSelectAll = ((showSelAll === true) ? true : false);
			//allowDeselect = multi; // DISABLED - Selectable(..) should not change this behaviour!

			me._internal.Data("selectable", selectable.toString());
			me._internal.Data("multiselect", multiSelect.toString());

			var fireOnChange = (selected.length > 0);

			selected = createInternalCollection();

			// Deselect all children and make sure they are configured identically

			if (me.Value() !== "")
			{
				// Help developers - it might be confusing that changing Selectable state also clears selection
				Fit.Browser.Debug("TreeView: Selectable changed - Selections will be cleared");
			}

			executeWithNoOnChange(function() // Prevent n.Selected(false) from firering OnChange event
			{
				executeRecursively(rootNode, function(n)
				{
					if (n === rootNode)
						return; // Skip root node itself

					n.Selectable(selectable, multiSelect, showSelectAll);
					n.Selected(false);
				});
			});

			repaint();

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		return selectable;
	}

	/// <function container="Fit.Controls.TreeView" name="Selected" access="public" returns="Fit.Controls.TreeViewNode[]">
	/// 	<description> Get/set selected nodes </description>
	/// 	<param name="val" type="Fit.Controls.TreeViewNode[]" default="undefined"> If defined, provided nodes are selected </param>
	/// </function>
	this.Selected = function(val)
	{
		Fit.Validation.ExpectArray(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			selectedOrg = [];
			var fireOnChange = (selected.length > 0);

			executeWithNoOnChange(function() // Prevent node.Selected(true) from firering OnChange event
			{
				me.Clear();

				Fit.Array.ForEach(val, function(n)
				{
					Fit.Validation.ExpectInstance(n, Fit.Controls.TreeViewNode);

					var node = ((n.GetTreeView() === me) ? n : me.GetChild(n.Value(), true)); // Try GetChild(..) in case node was constructed, but with a valid value

					if (node === null)
						Fit.Validation.ThrowError("Node is not associated with this TreeView, unable to change selection");

					Fit.Array.Add(selectedOrg, node);
					node.Selected(true); // Adds node to internal selected collection through TreeViewNodeInterface
					fireOnChange = true;
				});
			});

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		return Fit.Array.Copy(selected); // Copy to prevent changes to internal selection array

		/*var copy = Fit.Array.Copy(selected); // Copy to prevent changes to internal selection array
		copy.toString = selected.toString;

		return copy;*/
	}

	/// <function container="Fit.Controls.TreeView" name="SelectAll" access="public">
	/// 	<description> Select all nodes </description>
	/// 	<param name="selected" type="boolean"> Value indicating whether to select or deselect nodes </param>
	/// 	<param name="selectAllNode" type="Fit.Controls.TreeViewNode" default="undefined">
	/// 		If specified, children under given node is selected/deselected recursively.
	/// 		If not specified, all nodes contained in TreeView will be selected/deselected.
	/// 	</param>
	/// </function>
	this.SelectAll = function(selected, selectAllNode/*, suppressOnSelectAllCompletedCallback*/) // suppressOnSelectAllCompletedCallback is for internal use only!
	{
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectInstance(selectAllNode, Fit.Controls.TreeViewNode, true);

		// NOTICE: Selecting thousands of nodes using SelectAll may result in very poor performance
		// due to the large amount of DOM manipulation. The control goes through a full state change
		// for every single item being selected or deselected, so all events fire and reflows are
		// triggered for visible items.
		// To increase performance, temporarily hide TreeView with display:none while SelectAll is being performed.

		var node = (selectAllNode ? selectAllNode : null); // Null = select all nodes, Set = select only children under passed node

		// Fire OnSelectAll event

		if (me._internal.FireOnSelectAll(selected, node) === false)
			return; // Event handler canceled event

		// Change selection and expand (all nodes)

		var changed = false;

		executeWithNoOnChange(function() // Prevent OnChange from firing every time a node's selection state is changed
		{
			var nodes = ((node !== null) ? node.GetChildren() : rootNode.GetChildren());

			Fit.Array.Recurse(nodes, "GetChildren", function(child)
			{
				if (child.Selectable() === true)
				{
					if (child.Selected() !== selected)
					{
						changed = true;
						child.Selected(selected);
					}
				}

				//child.Expanded(true); // DISABLED - This hurts performance significantly for large TreeViews (10-15.000 nodes) as a huge amounts of DOM elements needs to be pushed to render tree, e.g. when opening/closing dropdown hosting TreeView
			});
		});

		//if (suppressOnSelectAllCompletedCallback !== true)
		//{
		me._internal.FireOnSelectAllComplete(selected, node);
		//}

		if (changed === true)
			me._internal.FireOnChange();
	}

	/// <function container="Fit.Controls.TreeView" name="ExpandAll" access="public">
	/// 	<description> Expand all nodes, optionally to a maximum depth </description>
	/// 	<param name="maxDepth" type="integer" default="undefined"> Optional maximum depth to expand nodes </param>
	/// </function>
	this.ExpandAll = function(maxDepth) // Overridden by WSTreeView
	{
		Fit.Validation.ExpectInteger(maxDepth, true);

		Fit.Array.CustomRecurse(me.GetChildren(), function(node)
		{
			node.Expanded(true);
			return (node.GetLevel() + 1 < (maxDepth || 99999) ? node.GetChildren() : null);
		});
	}

	/// <function container="Fit.Controls.TreeView" name="CollapseAll" access="public">
	/// 	<description> Collapse all nodes, optionally to a maximum depth </description>
	/// 	<param name="maxDepth" type="integer" default="undefined"> Optional maximum depth to collapse nodes </param>
	/// </function>
	this.CollapseAll = function(maxDepth) // Overridden by WSTreeView
	{
		Fit.Validation.ExpectInteger(maxDepth, true);

		Fit.Array.CustomRecurse(me.GetChildren(), function(node)
		{
			node.Expanded(false);
			return (node.GetLevel() + 1 < (maxDepth || 99999) ? node.GetChildren() : null);
		});
	}

	/// <function container="Fit.Controls.TreeView" name="AllowDeselect" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether user is allowed to deselect nodes.
	/// 		By default the user is allowed to deselect nodes.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, changes behaviour to specified value </param>
	/// </function>
	this.AllowDeselect = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			allowDeselect = val;
		}

		return allowDeselect;
	}

	// See documentation on ControlBase
	this.Value = function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		// Set

		if (Fit.Validation.IsSet(val) === true)
		{
			selectedOrg = (preserveDirtyState !== true ? [] : selectedOrg);
			var fireOnChange = (selected.length > 0);

			executeWithNoOnChange(function()
			{
				me.Clear();

				var values = val.split(";");

				Fit.Array.ForEach(values, function(nodeValue)
				{
					var nodeVal = decodeReserved(((nodeValue.indexOf("=") === -1) ? nodeValue : nodeValue.split("=")[1]));
					var child = me.GetChild(nodeVal, true);

					if (child !== null)
					{
						if (preserveDirtyState !== true)
						{
							Fit.Array.Add(selectedOrg, child);
						}

						child.Selected(true);
						fireOnChange = true;
					}
				});
			});

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		// Get

		var nodes = me.Selected();
		var value = "";

		Fit.Array.ForEach(nodes, function(node)
		{
			value += ((value !== "") ? ";" : "") + encodeReserved(node.Title()) + "=" + encodeReserved(node.Value());
		});

		return value;
	}

	// See documentation on ControlBase
	this.IsDirty = function()
	{
		if (selected.length !== selectedOrg.length)
			return true;

		var dirty = false;
		Fit.Array.ForEach(selectedOrg, function(node)
		{
			if (Fit.Array.Contains(selected, node) === false)
			{
				dirty = true;
				return false;
			}
		});
		return dirty;
	}

	// See documentation on ControlBase
	this.Focused = function(focus)
	{
		Fit.Validation.ExpectBoolean(focus, true);

		if (Fit.Validation.IsSet(focus) === true)
		{
			var focused = getNodeFocused();

			if (focus === true && focused === null)
			{
				var nodes = rootNode.GetChildren();

				if (nodes.length > 0)
					nodes[0].Focused(true);
			}
			else if (focus === false && focused !== null)
			{
				focused.Focused(false);
			}
		}

		return (getNodeFocused() !== null);
	}

	/// <function container="Fit.Controls.TreeView" name="Lines" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether helper lines are shown.
	/// 		Notice that helper lines cause node items to obtain a fixed
	/// 		line height of 20px, making it unsuitable for large fonts.
	/// 	</description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables helper lines, False disables them </param>
	/// </function>
	this.Lines = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true)
			{
				me.AddCssClass("FitUiControlTreeViewLines");
				me.AddCssClass("FitUiControlTreeViewDotLines"); // Optional, render dotted lines instead of solid lines
			}
			else
			{
				me.RemoveCssClass("FitUiControlTreeViewLines");
				me.RemoveCssClass("FitUiControlTreeViewDotLines");
			}

			repaint();
		}

		return me.HasCssClass("FitUiControlTreeViewLines");
	}

	/// <function container="Fit.Controls.TreeView" name="ContextMenu" access="public" returns="Fit.Controls.ContextMenu">
	/// 	<description> Get/set instance of ContextMenu control triggered when right clicking nodes in TreeView </description>
	/// 	<param name="contextMenu" type="Fit.Controls.ContextMenu | null"> If defined, assignes ContextMenu control to TreeView </param>
	/// </function>
	this.ContextMenu = function(contextMenu)
	{
		Fit.Validation.ExpectInstance(contextMenu, Fit.Controls.ContextMenu, true);

		if (contextMenu !== undefined) // Accept null to disable context menu
		{
			ctx = contextMenu;
		}

		return ctx;
	}

	/// <function container="Fit.Controls.TreeView" name="Clear" access="public">
	/// 	<description>
	/// 		Fit.Controls.ControlBase.Clear override:
	/// 		Clear control value.
	/// 		Override allows for non-selectable nodes to keep their selection state.
	/// 		This is useful if TreeView has been configured to preselect some non-selectable
	/// 		nodes, hence preventing the user from removing these selections. In that case the
	/// 		desired functionality of the Clear function could be to preserve these preselections.
	/// 		If called with no arguments, all selections are cleared.
	/// 	</description>
	/// 	<param name="preserveNonSelectable" type="boolean" default="false">
	/// 		True causes selection state of non-selectable nodes to be preserved, False do not
	/// 	</param>
	/// </function>
	this.Clear = function(preserveNonSelectable)
	{
		if (selected.length === 0)
			return;

		forceClear = (preserveNonSelectable !== true);

		// Deselecting items causes them to be removed from internal selected collection.
		// Copying array to avoid breaking ForEach which doesn't like collection being modified while iterated.
		var sel = Fit.Array.Copy(selected);

		executeWithNoOnChange(function() // Prevent node.Selected(false) from firering OnChange
		{
			Fit.Array.ForEach(sel, function(node)
			{
				if (preserveNonSelectable !== true || node.Selectable() === true)
					node.Selected(false); // Removes node from internal selected collection through TreeViewNodeInterface
			});
		});

		forceClear = false;

		me._internal.FireOnChange();
	}

	/// <function container="Fit.Controls.TreeView" name="AddChild" access="public">
	/// 	<description> Add node to TreeView </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to add </param>
	/// 	<param name="atIndex" type="integer" default="undefined"> Optional index at which node is added </param>
	/// </function>
	this.AddChild = function(node, atIndex)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectInteger(atIndex, true);
		rootNode.AddChild(node, atIndex);
	}

	/// <function container="Fit.Controls.TreeView" name="RemoveChild" access="public">
	/// 	<description> Remove node from TreeView - this does not result in OnSelect and OnSelected being fired for selected nodes </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to remove </param>
	/// </function>
	this.RemoveChild = function(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		rootNode.RemoveChild(node);
	}

	/// <function container="Fit.Controls.TreeView" name="RemoveAllChildren" access="public">
	/// 	<description> Remove all nodes contained in TreeView - this does not result in OnSelect and OnSelected being fired for selected nodes </description>
	/// 	<param name="dispose" type="boolean" default="false"> Set True to dispose nodes </param>
	/// </function>
	this.RemoveAllChildren = function(dispose)
	{
		Fit.Validation.ExpectBoolean(dispose, true);

		var fireOnChange = (selected.length > 0);

		// Clear internal selections by creating new collection rather than calling me.Clear().
		// This is done to prevent OnSelect (and OnItemSelectionChanging), OnSelected (and
		// OnItemSelectionChanged), and OnChange from firing if selections were made.
		// In fact, we want to preserve selection state for removed nodes in case it is
		// needed later, e.g. if moved to another TreeView.
		selected = createInternalCollection();

		executeWithNoOnChange(function()
		{
			var children = rootNode.GetChildren();

			Fit.Array.ForEach(children, function(child)
			{
				if (dispose === true)
					child.Dispose();
				else
					rootNode.RemoveChild(child);
			});
		});

		if (fireOnChange === true)
			me._internal.FireOnChange();
	}

	/// <function container="Fit.Controls.TreeView" name="GetChild" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get node by value - returns Null if not found </description>
	/// 	<param name="val" type="string"> Node value </param>
	/// 	<param name="recursive" type="boolean" default="false"> If defined, True enables recursive search </param>
	/// </function>
	this.GetChild = function(val, recursive)
	{
		Fit.Validation.ExpectString(val);
		Fit.Validation.ExpectBoolean(recursive, true);

		return rootNode.GetChild(val, recursive);
	}

	/// <function container="Fit.Controls.TreeView" name="GetChildren" access="public" returns="Fit.Controls.TreeViewNode[]">
	/// 	<description> Get all children </description>
	/// </function>
	this.GetChildren = function()
	{
		return rootNode.GetChildren();
	}

	/// <function container="Fit.Controls.TreeView" name="GetAllNodes" access="public" returns="Fit.Controls.TreeViewNode[]">
	/// 	<description> Get all nodes across all children and their children, in a flat structure </description>
	/// </function>
	this.GetAllNodes = function()
	{
		var nodes = [];
		executeRecursively(rootNode, function(n)
		{
			if (n === rootNode)
				return;

			Fit.Array.Add(nodes, n);
		});
		return nodes;
	}

	/// <function container="Fit.Controls.TreeView" name="GetNodeFocused" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get node currently having focus - returns Null if no node has focus </description>
	/// </function>
	this.GetNodeFocused = function()
	{
		return getNodeFocused();
	}

	/// <function container="Fit.Controls.TreeView" name="GetActiveNode" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get active (highlighted or focused) node - returns Null if no node is currently active </description>
	/// </function>
	this.GetActiveNode = function()
	{
		return isPicker ? activeNode : getNodeFocused();
	}

	/// <function container="Fit.Controls.TreeView" name="SetActiveNode" access="public">
	/// 	<description> Set active (highlighted or focused) node </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to set active in TreeView </param>
	/// </function>
	this.SetActiveNode = function(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		if (node.GetTreeView() === me)
		{
			focusNode(node); // Will highlight node if TreeView is used as picker, otherwise focus node
		}
	}

	/// <function container="Fit.Controls.TreeView" name="GetNodeAbove" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get node above specified node - returns Null if no node is above the specified one </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to get node above </param>
	/// </function>
	this.GetNodeAbove = function(node)
	{
		return getNodeAbove(node, true);
	}

	/// <function container="Fit.Controls.TreeView" name="GetNodeBelow" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get node below specified node - returns Null if no node is below the specified one </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to get node below </param>
	/// </function>
	this.GetNodeBelow = function(node)
	{
		return getNodeBelow(node, true, true);
	}

	/// <function container="Fit.Controls.TreeView" name="KeyboardNavigation" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether keyboard navigation is enabled </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables keyboard navigation, False disables it </param>
	/// </function>
	this.KeyboardNavigation = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			keyNavigationEnabled = val;
		}

		return keyNavigationEnabled;
	}

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function(calledInternally)
	{
		Fit.Validation.ExpectBoolean(calledInternally, true);

		// This will destroy control - it will no longer work!

		if (focusInEventId !== -1)
		{
			Fit.Events.RemoveHandler(document, focusInEventId);
		}

		me._internal.ExecuteWithNoOnChange(function() // Prevent Dispose() on nodes from firing OnChange when they are removed from hierarchy
		{
			rootNode.Dispose();
		});

		if (ctx !== null)
		{
			ctx.Dispose(); //ctx.Hide();
		}

		base();

		if (calledInternally !== true)
		{
			me.Destroy(true); // PickerBase.Destroy()
		}

		me = rootContainer = rootNode = focusInEventId = keyNavigationEnabled = selectable = multiSelect = showSelectAll = allowDeselect = revealExpandedNodes = persistView = scrollPosition = highlightFirst = firstWasHighlighted = selected = selectedOrg = ctx = onSelectHandlers = onSelectedHandlers = onToggleHandlers = onToggledHandlers = onSelectAllHandlers = onSelectAllCompleteHandlers = onContextMenuHandlers = forceClear = isIe8 = isPicker = activeNode = hostFocused = null;
	});

	// ============================================
	// Events (OnChange defined on BaseControl)
	// ============================================

	/// <container name="Fit.Controls.TreeViewTypeDefs.SelectionEventHandlerArgs">
	/// 	<description> Selection event handler arguments </description>
	/// 	<member name="Node" type="Fit.Controls.TreeViewNode | null">
	/// 		Instance of TreeViewNode - Null if Select All was triggered for root nodes (all nodes)
	/// 	</member>
	/// 	<member name="Selected" type="boolean"> Value indicating new selection state </member>
	/// </container>

	/// <function container="Fit.Controls.TreeViewTypeDefs" name="CancelableNodeEventHandler" returns="boolean | void">
	/// 	<description> Cancelable node event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Instance of TreeViewNode </param>
	/// </function>

	/// <function container="Fit.Controls.TreeViewTypeDefs" name="NodeEventHandler">
	/// 	<description> Node event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Instance of TreeViewNode </param>
	/// </function>

	/// <function container="Fit.Controls.TreeViewTypeDefs" name="CancelableSelectionEventHandler" returns="boolean | void">
	/// 	<description> Cancelable select all event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.TreeViewTypeDefs.SelectionEventHandlerArgs">
	/// 		Event handler arguments
	/// 	</param>
	/// </function>

	/// <function container="Fit.Controls.TreeViewTypeDefs" name="SelectionCompleteEventHandler">
	/// 	<description> Select all complete event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.TreeViewTypeDefs.SelectionEventHandlerArgs">
	/// 		Event handler arguments
	/// 	</param>
	/// </function>

	/// <function container="Fit.Controls.TreeView" name="OnSelect" access="public">
	/// 	<description>
	/// 		Add event handler fired when node is being selected or deselected.
	/// 		Selection can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and Node (Fit.Controls.TreeViewNode).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.CancelableNodeEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSelect = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnSelected" access="public">
	/// 	<description>
	/// 		Add event handler fired when node is selected or deselected.
	/// 		Selection can not be canceled. Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and Node (Fit.Controls.TreeViewNode).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.NodeEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSelected = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectedHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnToggle" access="public">
	/// 	<description>
	/// 		Add event handler fired when node is being expanded or collapsed.
	/// 		Toggle can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and Node (Fit.Controls.TreeViewNode).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.CancelableNodeEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnToggle = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onToggleHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnToggled" access="public">
	/// 	<description>
	/// 		Add event handler fired when node is expanded or collapsed.
	/// 		Toggle can not be canceled. Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and Node (Fit.Controls.TreeViewNode).
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.NodeEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnToggled = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onToggledHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnSelectAll" access="public">
	/// 	<description>
	/// 		Add event handler fired when Select All is used for a given node.
	/// 		This event can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Node: Fit.Controls.TreeViewNode instance - Null if Select All was triggered for root nodes (all nodes)
	/// 		 - Selected: Boolean value indicating new selection state
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.CancelableSelectionEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSelectAll = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectAllHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnSelectAllComplete" access="public">
	/// 	<description>
	/// 		Add event handler fired when Select All operation has completed.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Node: Fit.Controls.TreeViewNode instance - Null if Select All was triggered for root nodes (all nodes)
	/// 		 - Selected: Boolean value indicating new selection state
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.SelectionCompleteEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnSelectAllComplete = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onSelectAllCompleteHandlers, cb);
	}

	/// <function container="Fit.Controls.TreeView" name="OnContextMenu" access="public">
	/// 	<description>
	/// 		Add event handler fired before context menu is shown.
	/// 		This event can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.TreeView) and Node (Fit.Controls.TreeViewNode).
	/// 		Use Sender.ContextMenu() to obtain a reference to the context menu.
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.TreeViewTypeDefs.CancelableNodeEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnContextMenu = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onContextMenuHandlers, cb);
	}

	// ============================================
	// PickerBase interface
	// ============================================

	var isPicker = false;
	var activeNode = null;
	var hostFocused = false;

	this.OnShow(function(sender)
	{
		if (persistView === false) // Reset selection and scroll position
		{
			if (highlightFirst === true)
			{
				focusFirstNode();
			}
			else
			{
				unsetActiveNode();
			}

			me.GetDomElement().scrollTop = 0;
			me.GetDomElement().scrollLeft = 0;
		}
		else // View persisted
		{
			if (highlightFirst === true && firstWasHighlighted === false)
			{
				focusFirstNode();
				firstWasHighlighted = true;
			}

			me.GetDomElement().scrollTop = scrollPosition.Y;
			me.GetDomElement().scrollLeft = scrollPosition.X;
		}
	});

	this.OnSelect(function(sender, node)
	{
		// Handlers may return False which will prevent node from being selected, and OnSelected from being fired
		return me._internal.FireOnItemSelectionChanging(node.Title(), node.Value(), node.Selected(), hostFocused === false);
	});

	this.OnSelected(function(sender, node)
	{
		me._internal.FireOnItemSelectionChanged(node.Title(), node.Value(), node.Selected(), hostFocused === false);
	});

	this.OnChange(function(sender)
	{
		me._internal.FireOnItemSelectionComplete();
	});

	this.OnFocus(function(sender)
	{
		me._internal.FireOnFocusIn();
	});

	this.OnBlur(function(sender)
	{
		me._internal.FireOnFocusOut();
	});

	this.SetSelections = function(items)
	{
		Fit.Validation.ExpectArray(items);

		var vals = [];
		var fireOnChange = false;

		Fit.Array.ForEach(items, function(item)
		{
			Fit.Array.Add(vals, item.Value);
		});

		// Set new selections provided by host control

		executeWithNoOnChange(function()
		{
			// Deselect nodes not found in items passed (vals array)

			Fit.Array.ForEach(Fit.Array.Copy(selected), function(selectedNode) // Copying array to prevent "Collection modified" error - deselecting nodes causes them to be removed from array
			{
				// Notice that non-selectable (read only) nodes always keep their selection state
				if (Fit.Array.Contains(vals, selectedNode.Value()) === false && selectedNode.Selectable() === true)
				{
					selectedNode.Selected(false);
					fireOnChange = true;
				}
			});

			// Select nodes from items argument not already selected

			Fit.Array.ForEach(vals, function(nodeVal)
			{
				var child = me.GetChild(nodeVal, true); // Based on indexed lookup - fast!

				// Notice: Items passed may contain nodes that were already selected, hence checking node's
				// Selected state to prevent OnChange from firing if items provided matched TreeView's current selection.
				if (child !== null && child.Selected() === false)
				{
					child.Selected(true);
					fireOnChange = true;
				}
			});
		});

		// Fire OnChange

		if (fireOnChange === true)
			me._internal.FireOnChange();
	}

	this.GetItemByValue = function(val)
	{
		Fit.Validation.ExpectString(val);

		var node = me.GetChild(val, true);

		if (node !== null)
		{
			return { Title: node.Title(), Value: node.Value() };
		}

		return null;
	}

	this.RevealItemInView = function(val)
	{
		Fit.Validation.ExpectString(val);

		var node = me.GetChild(val, true);

		if (node !== null)
		{
			// Expand parent nodes if selected node is a child node.
			// But only expand nodes automatically the first time to
			// allow user to collapse nodes and have them remain collapsed.
			if (revealExpandedNodes === false)
			{
				revealExpandedNodes = true;

				var parent = node;
				while ((parent = parent.GetParent()) !== null)
				{
					parent.Expanded(true);
				}
			}

			focusNode(node);
		}
	}

	this.GetHighlighted = function()
	{
		if (isNodeSet(activeNode) === true)
		{
			return { Title: activeNode.Title(), Value: activeNode.Value() };
		}

		return null;
	}

	this.PersistView = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true && val !== persistView)
		{
			persistView = val;
			scrollPosition.X = 0;
			scrollPosition.Y = 0;
		}

		return persistView;
	}

	this.HighlightFirst = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val !== highlightFirst)
			{
				highlightFirst = val;
				firstWasHighlighted = false;
			}

			// Allow external code to force focus first item if picker is visible.
			// Usually first item is automatically highlighted when the host control
			// is opened (see OnShow handler in init()).
			if (val === true && Fit.Dom.IsVisible(me.GetDomElement()) === true)
			{
				focusFirstNode();
				firstWasHighlighted = true;
			}
		}

		return highlightFirst;
	}

	this.UpdateItemSelection = function(itemValue, selected, programmaticallyChanged)
	{
		Fit.Validation.ExpectString(itemValue);
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectBoolean(programmaticallyChanged);

		var node = me.GetChild(itemValue, true);

		if (node !== null && node.Selected() !== selected)
		{
			if (node.Selectable() === false)
				return false; // Cancel, node is not selectable

			node.Selected(selected); // Fires OnSelect (which fires OnItemSelectionChanging) and OnSelected (which fires OnItemSelectionChanged)

			if (node.Selected() !== selected)
				return false; // An event handler has canceled change, node's selection state was not updated - return false to prevent host control from adding item
		}
	}

	this._internal = (this._internal ? this._internal : {});

	this._internal.InitializePicker = function() // Override function from PickerBase
	{
		isPicker = true;
		me._internal.Data("picker", "true");
	}

	this._internal.ReportFocused = function(focused) // Override function from PickerBase
	{
		Fit.Validation.ExpectBoolean(focused);
		hostFocused = focused;
	}

	this._internal.FireOnSelectAll = function(selected, node) // Make function available to derivatives
	{
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode, true);

		return fireEventHandlers(onSelectAllHandlers, { Selected: selected, Node: node || null });
	}

	this._internal.FireOnSelectAllComplete = function(selected, node) // Make function available to derivatives
	{
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode, true);

		fireEventHandlers(onSelectAllCompleteHandlers, { Selected: selected, Node: node || null });
	}

	this._internal.FocusFirstNode = function() // Similar implementation in Fit.Controls.ListView._internal.FocusFirstItem()
	{
		// If picker is still visible (it might have been hidden if user closed
		// host control while data was being loaded/populated async.) then focus first node.
		if (Fit.Dom.IsVisible(me.GetDomElement()) === true)
		{
			focusFirstNode();
			firstWasHighlighted = true;
		}
	}

    this.HandleEvent = function(e)
    {
		Fit.Validation.ExpectEvent(e, true);

		var ev = Fit.Events.GetEvent(e);

		if (ev.type === "keydown")
		{
			rootNode.GetDomElement().onkeydown(e);

			if (ev.keyCode === 37 || ev.keyCode === 39) // Left/Right
				return false; // Suppress, left/right is reserved for expanding/collapsing nodes
			if (ev.keyCode === 32) // Space
				return false; // Suppress, space is reserved for selecting nodes
		}
    }

	this.Destroy = Fit.Core.CreateOverride(this.Destroy, function(calledInternally)
	{
		Fit.Validation.ExpectBoolean(calledInternally, true);

		// This will destroy control - it will no longer work!

		if (calledInternally !== true)
		{
			me.Dispose(true); // Component.Dispose()
		}

		base(); // PickerBase.Destroy()
	});

	// ============================================
	// Private
	// ============================================

	function executeWithNoOnChange(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		me._internal.ExecuteWithNoOnChange(cb);
	}

	function executeRecursively(node, cb)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectFunction(cb);

		if (cb(node) === false)
			return false;

		Fit.Array.ForEach(node.GetChildren(), function(child)
		{
			if (executeRecursively(child, cb) === false)
				return false;
		});
	}

	function createInternalCollection()
	{
		var selected = [];
		return selected;
	}

	function repaint()
	{
		// In some cases, IE8 does not repaint Treeview properly.
		// This often happens when modifying the TreeView programmatically.
		// Calling this function resolves the problem.

		if (isIe8 === true)
		{
			me.AddCssClass("FitUi_Non_Existing_TreeView_Class");
			me.RemoveCssClass("FitUi_Non_Existing_TreeView_Class");
		}
	}

	function toggleNodeSelection(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		if (node.Selected() === true && allowDeselect === false)
			return;

		// TreeViewNodeInterface now takes care of deselecting an existing node if in Single Selection Mode,
		// but prevents selection change if currently selected node is non-selectable (read only).
		// See Select and FireSelect functions on TreeViewNodeInterface for details.
		// Node itself is responsible for firing OnChange which also happens through TreeViewNodeInterface.
		node.Selected(!node.Selected());
	}

	function focusNode(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		// Only set node active if it is visible (not hidden by collapsed parent, not
		// hidden by closed DropDown control when used as picker, not hidden using
		// styles such as display:none, and not disconnected from (not rooted in) DOM).
		// RevealItemInView() may be triggered asynchronously by WSDropDown when loading
		// nodes, in which case user may have closed dropdown while waiting for data,
		// resulting in node no longer being visible to focus.
		// We use offsetParent further down which is only available for elements in
		// the render tree - "visible" elements. Furthermore scroll position cannot be
		// determined when control is not visible (part of render tree).
		if (Fit.Dom.IsVisible(node.GetDomElement()) === false)
		{
			if (isPicker === true)
			{
				// Picker is invisible, but we would still like active node to
				// be set so keyboard navigation uses this as offset, rather than
				// starting from the top again. Therefore, if node passed is visible
				// in the hierarcy (not hidden behind collapsed nodes), then use it.

				nodeIsVisibleInHierarchy = true;

				var parent = node;
				while ((parent = parent.GetParent()) !== null)
				{
					if (parent.Expanded() === false)
					{
						nodeIsVisibleInHierarchy = false;
						break;
					}
				}

				if (nodeIsVisibleInHierarchy === true)
				{
					if (isNodeSet(activeNode) === true)
					{
						Fit.Dom.Data(activeNode.GetDomElement(), "active", null);
					}

					activeNode = node;
					Fit.Dom.Data(activeNode.GetDomElement(), "active", "true");
				}
			}

			return;
		}

		if (isPicker === true)
		{
			// Since host control has focus, we use the "active" data attribute
			// to indicate that the node has focus, even though it has not,
			// and keep a reference to the "pseudo focused" node in activeNode.

			if (isNodeSet(activeNode) === true)
				Fit.Dom.Data(activeNode.GetDomElement(), "active", null);

			activeNode = node;
			Fit.Dom.Data(activeNode.GetDomElement(), "active", "true");

			// Scroll active node into view if necessary.
			// This is a bit expensive, especially if we navigate up the
			// tree from 5+ nested levels, and hold down the arrow key.

			// Get node position within control

			// Get position relative to offset parent (parent node) which has position:relative
			var nodePositionWithinControl = { X: activeNode.GetDomElement().offsetLeft, Y: activeNode.GetDomElement().offsetTop };

			// Get node position all the way up to the div.FitUiControl control container which has position:relative
			var offsetParent = activeNode.GetDomElement().offsetParent;
			while (offsetParent !== me.GetDomElement())
			{
				nodePositionWithinControl.X += offsetParent.offsetLeft;
				nodePositionWithinControl.Y += offsetParent.offsetTop;

				offsetParent = offsetParent.offsetParent;
			}

			// Get node height

			var nodeHeightWithoutChildren = -1;

			var childrenContainer = ((activeNode.GetChildren().length > 0) ? activeNode.GetChildren()[0].GetDomElement().parentElement : null);

			if (childrenContainer !== null)
				childrenContainer.style.display = "none";

			nodeHeightWithoutChildren = activeNode.GetDomElement().offsetHeight;

			if (childrenContainer !== null)
				childrenContainer.style.display = "";

			// Scroll item into view

			// Vertical scroll
			if (nodePositionWithinControl.Y > (me.GetDomElement().scrollTop + me.GetDomElement().clientHeight - nodeHeightWithoutChildren) || nodePositionWithinControl.Y < me.GetDomElement().scrollTop)
				me.GetDomElement().scrollTop = Math.ceil(nodePositionWithinControl.Y - (me.GetDomElement().clientHeight / 2)); // Horizontal center

			// Horizontal scroll
			if (activeNode.GetLevel() > 2)
				me.GetDomElement().scrollLeft = Math.ceil(nodePositionWithinControl.X);
			else
				me.GetDomElement().scrollLeft = 0;

			repaint();
		}
		else
		{
			node.Focused(true);
		}
	}

	function focusFirstNode() // Focus first node in TreeView
	{
		if (rootNode.GetChildren().length > 0)
		{
			focusNode(rootNode.GetChildren()[0]);
		}
	}

	function unsetActiveNode() // Unset picker selection
	{
		if (isNodeSet(activeNode) === true)
		{
			Fit.Dom.Data(activeNode.GetDomElement(), "active", null);
			activeNode = null;
		}
	}

	function isNodeSet(node) // Used to check whether activeNode is set and that it has not been disposed
	{
		// TODO: It does not seem that isNodeSet(..) serves a purpose anymore.
		// Since the introduction of Dissociate on treeViewNodeInterface, activeNode
		// is nullified whenever a node is either removed or disposed. Theoretically
		// a disposed node could be set via SetActiveNode(..), but we don't need to
		// guard against incorrect usecases like that.
		// The use of isNodeSet(activeNode) can probably be replaced by: activeNode !== null
		return (node !== null && node.GetDomElement() !== null); // GetDomElement() returns null if node has been disposed
	}

	function getNodeFocused()
	{
		// Returns focused node or null - active node is not actually focused when TreeView is used as picker control but merely highlighted
		return ((Fit.Dom.GetFocused() && Fit.Dom.GetFocused().tagName === "LI" && Fit.Dom.GetFocused()._internal && Fit.Dom.Contained(rootContainer, Fit.Dom.GetFocused()) === true) ? Fit.Dom.GetFocused()._internal.Node : null);
	}

	function getNodeAbove(node, noLastOnExpand)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectBoolean(noLastOnExpand, true);

		// Get parent node
		var parent = node.GetParent();
		parent = ((parent !== null) ? parent : rootNode);

		// Find current node's position in parent node
		var children = parent.GetChildren();
		var idx = Fit.Array.GetIndex(children, node);

		// Select node above current node, within same parent
		var next = ((idx > 0) ? children[idx-1] : null);

		if (noLastOnExpand === true)
			return next;

		// Now make sure the last node in a hierarchy
		// of expanded nodes gets selected.

		if (next !== null)
		{
			while (next.Expanded() === true)
			{
				children = next.GetChildren();
				next = children[children.length - 1];
			}
		}
		else
		{
			// Current node is top node within parent - select parent
			next = node.GetParent();
		}

		return next;
	}

	function getNodeBelow(node, noFirstOnExpand, noSkipToParent)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectBoolean(noFirstOnExpand, true);
		Fit.Validation.ExpectBoolean(noSkipToParent, true);

		if (node.Expanded() === true && noFirstOnExpand !== true) // Select first child if current node is expanded
		{
			var children = node.GetChildren();
			return children[0];
		}
		else // Get node below current node within same level (parent)
		{
			// Get parent node

			var parent = node.GetParent();
			parent = ((parent !== null) ? parent : rootNode);

			// Find current node's position in parent node
			var children = parent.GetChildren();
			var idx = Fit.Array.GetIndex(children, node);

			// Select node below current node, within same parent
			var next = ((children.length - 1 >= idx + 1) ? children[idx+1] : null);

			if (noSkipToParent === true || next !== null || node.GetParent() === null) // Found, or last element within root node (in which case next variable is Null)
				return next;

			// No more nodes within parent - select node below parent

			return getNodeBelow(parent, true);
		}
	}

	function openContextMenu(node, pos) // pos is optional
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		if (ctx === null)
			return;

		if (fireEventHandlers(onContextMenuHandlers, node) === false)
			return;

		if (Fit.Validation.IsSet(pos) === true)
			ctx.Show(pos.X, pos.Y);
		else
			ctx.Show();
	}

	function fireEventHandlers(handlers, evObj)
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (cb(me, evObj) === false)
				cancel = true; // Do NOT cancel loop though! All handlers must be fired!
		});

		return !cancel;
	}

	function decodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/%3B/g, ";").replace(/%3D/g, "="); // Decode characters reserved for value format
	}

	function encodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/;/g, "%3B").replace(/=/g, "%3D"); // Encode characters reserved for value format
	}

	init();
}

/// <container name="Fit.Controls.TreeViewNodeOptions">
/// 	<description> Options for TreeViewNode </description>
/// 	<member name="HtmlAllowed" type="boolean" default="undefined"> Value indicating whether HTML is allowed (shown) in TreeView node </member>
/// </container>

/// <function container="Fit.Controls.TreeViewNode" name="TreeViewNode" access="public">
/// 	<description> Create instance of TreeViewNode </description>
/// 	<param name="displayTitle" type="string"> Node title </param>
/// 	<param name="nodeValue" type="string"> Node value </param>
/// 	<param name="options" type="Fit.Controls.TreeViewNodeOptions" default="undefined"> Optional options </param>
/// </function>
Fit.Controls.TreeViewNode = function(displayTitle, nodeValue, options)
{
	Fit.Validation.ExpectString(displayTitle);
	Fit.Validation.ExpectString(nodeValue);
	Fit.Validation.ExpectObject(options, true);
	Fit.Validation.ExpectBoolean((options || {}).HtmlAllowed, true);

	var me = this;
	var nodeTitle = null;
	var elmLi = null;
	var elmUl = null;
	var cmdToggle = null;
	var chkSelect = null;
	var lblTitle = null;
	var childrenIndexed = {};
	var childrenArray = [];
	var lastChild = null;
	var behavioralNodeCallback = null;
	var htmlAllowed = options && options.HtmlAllowed === true || false;

	// ============================================
	// Init
	// ============================================

	function init()
	{
		elmLi = document.createElement("li");
		cmdToggle = document.createElement("div");
		lblTitle = document.createElement("span");

		me.Title(displayTitle);
		Fit.Dom.Data(elmLi, "value", encode(nodeValue));

		Fit.Dom.Data(elmLi, "state", "static");
		Fit.Dom.Data(elmLi, "selectable", "false");
		Fit.Dom.Data(elmLi, "selected", "false");
		Fit.Dom.Data(elmLi, "last", "false");

		elmLi.tabIndex = 0; // Make focusable

		elmLi.appendChild(cmdToggle);
		elmLi.appendChild(lblTitle);

		elmLi._internal = { Node: me, TreeView: null, TreeViewConfigOverrides: { Selectable: undefined, ShowCheckbox: undefined, ShowSelectAll: undefined } };
	}

	// ============================================
	// Public
	// ============================================

	/// <function container="Fit.Controls.TreeViewNode" name="HtmlAllowed" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether HTML is allowed (shown) in TreeView node </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables support for HTML, False disables it </param>
	/// 	<param name="recursive" type="boolean" default="undefined"> If defined, True applies change recursively to children </param>
	/// </function>
	this.HtmlAllowed = function(val, recursive)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(recursive, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			htmlAllowed = val;

			// Update view - optionally update children recursively

			me.Title(me.Title());

			if (recursive === true)
			{
				Fit.Array.ForEach(childrenArray, function(child)
				{
					child.HtmlAllowed(val, true);
				});
			}
		}

		return htmlAllowed;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Title" access="public" returns="string">
	/// 	<description> Get/set node title </description>
	/// 	<param name="val" type="string" default="undefined"> If defined, node title is updated </param>
	/// </function>
	this.Title = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			nodeTitle = val;

			if (htmlAllowed === false)
			{
				Fit.Dom.Text(lblTitle, Fit.String.StripHtml(val));
			}
			else
			{
				lblTitle.innerHTML = val;

				// Make sure any contained links do not receive focus when navigating TreeView with Tab/Shift+Tab
				Fit.Array.ForEach(lblTitle.getElementsByTagName("a"), function(link) { link.tabIndex = -1; });
			}
		}

		return nodeTitle;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Value" access="public" returns="string">
	/// 	<description> Get node value </description>
	/// </function>
	this.Value = function()
	{
		return nodeValue; // Read only - value has been used on parent node as index (childrenIndexed)
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Expanded" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether node is expanded </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True expands node, False collapses it </param>
	/// </function>
	this.Expanded = function(expand)
	{
		Fit.Validation.ExpectBoolean(expand, true);

		if (Fit.Validation.IsSet(expand) === true && Fit.Dom.Data(elmLi, "state") !== "static" && me.Expanded() !== expand)
		{
			var tv = elmLi._internal.TreeView;

			// Fire OnToggle event

			if (tv !== null && tv.FireToggle(me) === false)
				return (Fit.Dom.Data(elmLi, "state") === "expanded");

			// Update state

			Fit.Dom.Data(elmLi, "state", ((expand === true) ? "expanded" : "collapsed"));

			// Repaint and fire OnToggled event

			if (tv !== null)
			{
				tv.Repaint();
				tv.FireToggled(me);
			}
		}

		return (Fit.Dom.Data(elmLi, "state") === "expanded");
	}

	/// <function container="Fit.Controls.TreeViewTypeDefs" name="CancelableBehavioralNodeCallback" returns="boolean | void">
	/// 	<description> Behavioral node callback </description>
	/// 	<param name="sender" type="Fit.Controls.TreeViewNode"> Instance of TreeViewNode </param>
	/// 	<param name="selected" type="boolean"> Value indicating new selection state </param>
	/// </function>

	/// <function container="Fit.Controls.TreeViewNode" name="SetBehavioralNodeCallback" access="public">
	/// 	<description>
	/// 		Set callback invoked when node is selected.
	/// 		A behavioral node is not considered data, so selecting it will not change
	/// 		the control value. Since the node is not considered data, it will not trigger
	/// 		the OnSelect and OnSelected TreeView events either.
	/// 		Callback receives two arguments:
	/// 		Sender (Fit.Controls.TreeViewNode) and Selected (boolean value indicating new selection state).
	/// 		Callback may cancel changed selection state by returning False.
	/// 	</description>
	/// 	<param name="func" type="Fit.Controls.TreeViewTypeDefs.CancelableBehavioralNodeCallback | null">
	/// 		Callback function invoked when node is selected - Null disables behavioral state
	/// 	</param>
	/// </function>
	this.SetBehavioralNodeCallback = function(func)
	{
		Fit.Validation.ExpectFunction(func !== null ? func : function() {});
		behavioralNodeCallback = func;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="IsBehavioralNode" access="public" returns="boolean">
	/// 	<description> Returns True if this is a behavioral node, otherwise False - see SetBehavioralNodeCallback for more details </description>
	/// </function>
	this.IsBehavioralNode = function()
	{
		return behavioralNodeCallback !== null;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Selectable" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether user can change node selection state </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True enables node selection, False disables it </param>
	/// 	<param name="showCheckbox" type="boolean" default="false"> If defined, True adds a selection checkbox, False removes it </param>
	/* 	<param name="showSelectAll" type="boolean" default="false"> If defined, True adds a Select All checkbox useful for selecting all children, False removes it </param> */
	/// </function>
	this.Selectable = function(val, showCheckbox, showSelectAll)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(showCheckbox, true);
		Fit.Validation.ExpectBoolean(showSelectAll, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			// Prevent node from obtaining configuration from TreeView if explicitly set
			// before adding node to TreeView. See AddChild(..) for TreeViewConfigOverrides usage.
			elmLi._internal.TreeViewConfigOverrides.Selectable = val;
			elmLi._internal.TreeViewConfigOverrides.ShowCheckbox = showCheckbox;
			elmLi._internal.TreeViewConfigOverrides.ShowSelectAll = showSelectAll;

			Fit.Dom.Data(elmLi, "selectable", val.toString());

			if (showCheckbox === true && chkSelect === null)
			{
				chkSelect = document.createElement("input");
				chkSelect.type = "checkbox";
				chkSelect.tabIndex = "-1";
				chkSelect.checked = me.Selected();

				// Alternatively use Fit.UI CheckBox (will most likely need changes to CSS to adjust position and size)
				// Also search for chkSelect._FitControl to find related changes if this is enabled at some point!
				/*var chk = new Fit.Controls.CheckBox(Fit.Data.CreateGuid());
				chkSelect = chk.GetDomElement();
				chk.OnChange(function(sender)
				{
					chkSelect.checked = chk.Checked();
				});
				chkSelect._FitControl = chk;*/

				Fit.Dom.InsertBefore(lblTitle, chkSelect);
			}
			else if (showCheckbox === false && chkSelect !== null)
			{
				Fit.Dom.Remove(chkSelect);
				chkSelect = null;
			}

			if (chkSelect !== null)
			{
				chkSelect.disabled = ((val === false) ? true : false);
				//chkSelect._FitControl.Enabled(val); // Related to alternative use of Fit.UI CheckBox
			}
		}

		return (Fit.Dom.Data(elmLi, "selectable") === "true");
	}

	/// <function container="Fit.Controls.TreeViewNode" name="HasCheckbox" access="public" returns="boolean">
	/// 	<description> Get value indicating whether node has its selection checkbox enabled </description>
	/// </function>
	this.HasCheckbox = function()
	{
		return (chkSelect !== null);
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Selected" access="public" returns="boolean">
	/// 	<description>
	/// 		Get/set value indicating whether node is selected.
	/// 		If node is selected, it will automatically be made
	/// 		selectable, if not already done so.
	/// 	</description>
	/// 	<param name="select" type="boolean" default="undefined"> If defined, True selects node, False deselects it </param>
	/// </function>
	this.Selected = function(select, suppressOnChange) // suppressOnChange is for internal use only!
	{
		Fit.Validation.ExpectBoolean(select, true);

		// Notice:
		// Nodes are responsible for keeping the TreeView current
		// in regards to selected nodes, through the TreeViewNodeInterface.
		// If this approach was not used, TreeView would have to
		// iterate recursively over all nodes to determine which
		// nodes are selected, which would be too expensive for
		// huge tree views containing thousands of nodes.

		if (Fit.Validation.IsSet(select) === true && me.Selected() !== select)
		{
			var tv = elmLi._internal.TreeView;

			if (behavioralNodeCallback !== null)
			{
				if (behavioralNodeCallback(me, !me.Selected()) === false)
				{
					return (Fit.Dom.Data(elmLi, "selected") === "true");
				}
			}

			// Fire OnSelect event

			if (behavioralNodeCallback === null)
			{
				if (tv !== null && tv.FireSelect(me) === false)
				{
					return (Fit.Dom.Data(elmLi, "selected") === "true");
				}
			}

			var wasSelected = me.Selected();

			// Update state

			Fit.Dom.Data(elmLi, "selected", select.toString());

			if (chkSelect !== null)
				chkSelect.checked = select;

			// Repaint

			if (tv !== null)
				tv.Repaint();

			// Synchronize selection to TreeView (unless this was a behavioral node (not a data node))

			if (behavioralNodeCallback === null && tv !== null)
			{
				if (select === true && wasSelected === false)
				{
					tv.Select(me);
					tv.FireSelected(me);

					if (suppressOnChange !== true)
						tv.FireOnChange();
				}
				else if (select === false && wasSelected === true)
				{
					tv.Deselect(me);
					tv.FireSelected(me);

					if (suppressOnChange !== true)
						tv.FireOnChange();
				}
			}
		}

		return (Fit.Dom.Data(elmLi, "selected") === "true");
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Focused" access="public" returns="boolean">
	/// 	<description> Get/set value indicating whether node has focus </description>
	/// 	<param name="val" type="boolean" default="undefined"> If defined, True assigns focus, False removes it (blur) </param>
	/// </function>
	this.Focused = function(val)
	{
		Fit.Validation.ExpectBoolean(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === true)
			{
				if (Fit.Dom.IsVisible(elmLi) === true)
				{
					elmLi.focus();
				}
			}
			else if (Fit.Dom.GetFocused() === elmLi)
			{
				elmLi.blur();
			}
		}

		return (Fit.Dom.GetFocused() === elmLi);
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetParent" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get parent node - returns Null if node has no parent </description>
	/// </function>
	this.GetParent = function()
	{
		if (!elmLi.parentElement)
			return null; // Not rooted in another node yet
			if (!elmLi.parentElement.parentElement._internal || !elmLi.parentElement.parentElement._internal.TreeView) // Notice: _internal may have been set by e.g. Fit.Events.AddHandler
			return null; // Rooted, but not in another node - most likely rooted in TreeView UL container
		if (elmLi.parentElement.parentElement._internal.Node.Value() === "TREEVIEW_ROOT_NODE")
			return null; // Indicate top by returning Null when root node is reached

		return elmLi.parentElement.parentElement._internal.Node;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetTreeView" access="public" returns="Fit.Controls.TreeView | null">
	/// 	<description> Returns TreeView if associated, otherwise Null </description>
	/// </function>
	this.GetTreeView = function()
	{
		if (elmLi._internal.TreeView !== null)
			return elmLi._internal.TreeView.GetTreeView();

		return null;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetLevel" access="public" returns="integer">
	/// 	<description> Get node depth in current hierarchy - root node is considered level 0 </description>
	/// </function>
	this.GetLevel = function()
	{
		var level = 0;
		var node = me;

		while ((node = node.GetParent()) !== null) // Parent is Null when no more nodes are found, and when TREEVIEW_ROOT_NODE is reached for nodes rooted in TreeView
		{
			level++;
		}

		return level;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetIndex" access="public" returns="integer">
	/// 	<description> Get node index (position in parent node or TreeView) - returns -1 if node has not been added yet </description>
	/// </function>
	this.GetIndex = function()
	{
		if (me.GetTreeView() === null)
		{
			return -1;
		}

		return Fit.Dom.GetIndex(me.GetDomElement());
	}

	/// <function container="Fit.Controls.TreeViewNode" name="AddChild" access="public">
	/// 	<description> Add child node </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to add </param>
	/// 	<param name="atIndex" type="integer" default="undefined"> Optional index at which node is added </param>
	/// </function>
	this.AddChild = function(node, atIndex)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectInteger(atIndex, true);

		// Remove node from existing parent if already rooted.
		// This is important to make sure that TreeView, from which
		// node is moved, is updated to reflect the change, and
		// to ensure that node (and its children) is no longer capable
		// of updating the old TreeView through its TreeViewNodeInterface.

		if (node.GetParent() !== null)
			node.GetParent().RemoveChild(node);
		if (node.GetTreeView() !== null)
			node.GetTreeView().RemoveChild(node);

		// Register child node.
		// This is done prior to firing any events so that event handlers
		// can rely on node being fully accessible through DOM (rooted).

		// Create child container (<ul>) if not already added
		if (elmUl === null)
		{
			elmUl = document.createElement("ul");
			elmLi.appendChild(elmUl);

			Fit.Dom.Data(elmLi, "state", "collapsed");
		}

		if (Fit.Validation.IsSet(atIndex) === false || lastChild === null || atIndex > lastChild.GetIndex())
		{
			// Make sure last node is marked as such, allowing for specialized CSS behaviour
			if (lastChild !== null)
				Fit.Dom.Data(lastChild.GetDomElement(), "last", "false");
			Fit.Dom.Data(node.GetDomElement(), "last", "true");
			lastChild = node;
		}

		// Add child to DOM and internal collections
		if (lastChild === node)
		{
			elmUl.appendChild(node.GetDomElement());
			childrenIndexed[node.Value()] = node;
			Fit.Array.Add(childrenArray, node);
		}
		else
		{
			Fit.Dom.InsertAt(elmUl, atIndex, node.GetDomElement());
			childrenIndexed[node.Value()] = node;
			Fit.Array.Insert(childrenArray, atIndex, node);
		}

		// Changing data attribute above requires repaint in IE8
		// for dynamically added nodes to render helper lines properly.
		if (elmLi._internal.TreeView !== null)
			elmLi._internal.TreeView.Repaint();

		// Behavioral node support

		if (node.IsBehavioralNode() === true)
		{
			executeRecursively(node, function(n)
			{
				n.GetDomElement()._internal.TreeView = elmLi._internal.TreeView;
			});

			return; // Skip remaining wiring - behavioral nodes are not considered data, so it should not fire OnSelect, OnSelected, and OnChange
		}

		// Configure TreeView association and synchronize selection state

		var tv = elmLi._internal.TreeView;

		if (tv !== null)
		{
			var selected = [];
			var fireOnChange = false;

			executeRecursively(node, function(n)
			{
				//n.GetDomElement()._internal.TreeView = null; // Avoid conflicts if node currently belongs to another TreeView (e.g. drag and drop)

				if (n.Selected() === true)
				{
					// Deselect node which allows us to reselect it later and have all
					// relevant events fired with appropriate current selection state set
					// (OnSelect with Selected = false, and OnSelected with Selected = true).
					Fit.Array.Add(selected, n);
					n.Selected(false); // Does not fire any events since TreeViewNodeInterface is not set yet
				}

				n.GetDomElement()._internal.TreeView = tv;
				var ovr = n.GetDomElement()._internal.TreeViewConfigOverrides;

				// Configure node like TreeView, unless node has been explicitly configured (see TreeViewConfigOverrides)
				n.Selectable(((Fit.Validation.IsSet(ovr.Selectable) === true) ? ovr.Selectable : tv.IsSelectable()), ((Fit.Validation.IsSet(ovr.ShowCheckbox) === true) ? ovr.ShowCheckbox : tv.IsMultiSelect()), ((Fit.Validation.IsSet(ovr.ShowSelectAll) === true) ? ovr.ShowSelectAll : tv.ShowSelectAll()));
			});

			// Re-select nodes initially selected

			Fit.Array.ForEach(selected, function(node)
			{
				// Notice that TreeViewNodeInterface is temporarily removed to prevent
				// node.Selected(true) from firing events - we do not want OnChange to be
				// fired multiple times, so we fire only OnSelect and OnSelected below.
				// DISABLED: This is garbage! Functions such as node.GetTreeView() or node.Expanded(true)
				// will not work properly with this approach from e.g. an OnSelected event handler! Fixed
				// by passing an argument to node.Selected(..) instead which suppresses the OnChange event.
				/*node.GetDomElement()._internal.TreeView = null;

				if (tv.FireSelect(node) === true)
				{
					node.Selected(true);
					tv.Select(node);
					tv.FireSelected(node);
					fireOnChange = true;
				}

				node.GetDomElement()._internal.TreeView = tv;*/

				var wasSelected = node.Selected();
				var newSelection = node.Selected(true, true); // Second True is an internal argument suppressing firing of the OnChange event

				if (wasSelected !== newSelection)
				{
					fireOnChange = true;
				}
			});

			// if (tv.GetTreeView().Enabled() === false)
			// {
			// 	tv.GetTreeView().Enabled(true);
			// 	tv.GetTreeView().Enabled(false);
			// }

			// Fire OnChange if selections were made

			if (fireOnChange === true)
				tv.FireOnChange();
		}
	}

	/// <function container="Fit.Controls.TreeViewNode" name="RemoveChild" access="public">
	/// 	<description> Remove child node - this does not result in TreeView.OnSelect and TreeView.OnSelected being fired for selected nodes </description>
	/// 	<param name="node" type="Fit.Controls.TreeViewNode"> Node to remove </param>
	/// </function>
	this.RemoveChild = function(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		if (node.GetParent() === me || (me.Value() === "TREEVIEW_ROOT_NODE" && me.GetTreeView().GetChild(node.Value()) !== null))
		{
			// Deconfigure TreeView association and synchronize selection state

			var tv = node.GetDomElement()._internal.TreeView;

			if (tv !== null)
			{
				// Deselecting selected nodes in TreeView

				var fireOnChange = false;

				executeRecursively(node, function(n)
				{
					if (n.Selected() === true)
					{
						// Node itself remains selected, this only removes node from TreeView's internal selection collection.
						// Notice that OnSelect and OnSelected is not fired since the node is not actually deselected, only removed from TreeView.
						tv.Deselect(n);

						fireOnChange = true;
					}

					n.GetDomElement()._internal.TreeView = null; // Make sure removed node cannot update TreeView anymore
				});

				tv.Dissociate(node);

				if (fireOnChange === true)
					tv.FireOnChange();
			}

			// Remove from DOM

			elmUl.removeChild(node.GetDomElement());
			delete childrenIndexed[node.Value()];
			Fit.Array.Remove(childrenArray, node);

			if (childrenArray.length === 0)
			{
				elmLi.removeChild(elmUl);
				elmUl = null;
				Fit.Dom.Data(elmLi, "state", "static");
			}

			// Update last node

			if (node === lastChild)
			{
				lastChild = null;
				Fit.Dom.Data(node.GetDomElement(), "last", "false");

				if (childrenArray.length > 0)
				{
					lastChild = childrenArray[childrenArray.length - 1];
					Fit.Dom.Data(lastChild.GetDomElement(), "last", "true");
				}
			}

			// Make sure helper lines are repainted on IE8
			if (elmLi._internal.TreeView !== null)
				elmLi._internal.TreeView.Repaint();
		}
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetChild" access="public" returns="Fit.Controls.TreeViewNode | null">
	/// 	<description> Get node by value - returns Null if not found </description>
	/// 	<param name="val" type="string"> Node value </param>
	/// 	<param name="recursive" type="boolean" default="false"> If defined, True enables recursive search </param>
	/// </function>
	this.GetChild = function(val, recursive)
	{
		Fit.Validation.ExpectString(val);
		Fit.Validation.ExpectBoolean(recursive, true);

		var node = childrenIndexed[val];

		if (node === undefined && recursive === true)
		{
			var found = null;
			Fit.Array.ForEach(childrenArray, function(child)
			{
				found = child.GetChild(val, recursive);

				if (found !== null)
					return false;
			});
			return found;
		}

		return ((node !== undefined) ? node : null);
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetChildren" access="public" returns="Fit.Controls.TreeViewNode[]">
	/// 	<description> Get all children </description>
	/// </function>
	this.GetChildren = function()
	{
		return Fit.Array.Copy(childrenArray); // Copy to prevent changes to internal children array
	}

	/// <function container="Fit.Controls.TreeViewNode" name="Dispose" access="public">
	/// 	<description> Destroys object to free up memory </description>
	/// </function>
	this.Dispose = function()
	{
		// This will destroy node - it will no longer work!

		// Dispose children
		Fit.Array.ForEach(Fit.Array.Copy(childrenArray), function(child) { child.Dispose(); });

		// Remove from DOM

		var parentNode = me.GetParent();

		if (parentNode !== null)
		{
			parentNode.RemoveChild(me);
		}
		else
		{
			if (me.GetTreeView() !== null)
			{
				me.GetTreeView().RemoveChild(me);
			}
			else
			{
				Fit.Dom.Remove(elmLi);
			}
		}

		// Dispose private members
		me = nodeTitle = elmLi = elmUl = cmdToggle = chkSelect = lblTitle = childrenIndexed = childrenArray = lastChild = behavioralNodeCallback = htmlAllowed = null;
	}

	/// <function container="Fit.Controls.TreeViewNode" name="GetDomElement" access="public" returns="DOMElement">
	/// 	<description> Get DOMElement representing node </description>
	/// </function>
	this.GetDomElement = function()
	{
		return elmLi;
	}

	// Private

	function executeRecursively(node, cb)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectFunction(cb);

		if (cb(node) === false)
			return false;

		Fit.Array.ForEach(node.GetChildren(), function(child)
		{
			if (executeRecursively(child, cb) === false)
				return false;
		});
	}

	/*function decode(str)
	{
		Fit.Validation.ExpectString(str);
		return decodeURIComponent(str); // Turn string back to original value
	}*/

	function encode(str)
	{
		Fit.Validation.ExpectString(str);
		return encodeURIComponent(str); // Allow value to be stored in a HTML Data Attribute
	}

	init();
}

Fit.Controls.TreeView.Node = Fit.Controls.TreeViewNode; // Backward compatibility

/// <container name="Fit.Controls.WSTreeView" extends="Fit.Controls.TreeView">
/// 	TreeView control allowing data from a
/// 	WebService to be listed in a structured manner.
/// 	Extending from Fit.Controls.TreeView.
///
/// 	Notice: WSTreeView works a bit differently from TreeView.
/// 	Nodes are loaded on-demand, meaning when Selected(..) or Value(..)
/// 	is called to set selections, nodes not loaded yet are stored internally as
/// 	preselections. Nodes not loaded yet will not have OnSelect, OnSelected,
/// 	and any associated events fired, until they are actually loaded.
/// 	But they will be returned when Selected() or Value() is called (getters).
/// 	OnChange, however, will always be fired when selections are changed,
/// 	no matter if nodes are loaded or not.
/// </container>

/// <function container="Fit.Controls.WSTreeView" name="WSTreeView" access="public">
/// 	<description> Create instance of WSTreeView control </description>
/// 	<param name="ctlId" type="string" default="undefined"> Unique control ID that can be used to access control using Fit.Controls.Find(..) </param>
/// </function>
Fit.Controls.WSTreeView = function(ctlId)
{
	Fit.Validation.ExpectStringValue(ctlId, true);
	Fit.Core.Extend(this, Fit.Controls.TreeView).Apply(ctlId);

	var me = this;
	var url = null;
	var jsonpCallback = null;
	var rootNode = null;
	var preSelected = {};
	var orgSelected = [];
	var loadDataOnInit = true;
	var dataLoading = false;			// True when requesting data via Reload, EnsureData, and SelectAll
	var dataReloading = false;			// True when requesting new data via Reload
	var dataReloadingEnableFunc = null;	// Callback function when data is reloading and control has been disabled, and later needs to be re-enabled via this callback
	var nodesLoading = 0;				// Value is above zero when nodes are being loaded when expanded
	var recursiveNodeLoadCount = -1;	// Value used by recursivelyLoadAllNodes(..) to keep track of progress when chain loading nodes
	var onDataLoadedCallback = [];
	var selectAllMode = Fit.Controls.WSTreeViewSelectAllMode.Progressively;
	var onRequestHandlers = [];
	var onResponseHandlers = [];
	var onAbortHandlers = [];
	var onPopulatedHandlers = [];
	var baseSelected = this.Selected;	// Used by Selected(..), Value(), and SetSelections(..)

	// Support for ExpandAll which may trigger WebService requests to load data.
	// We therefore need to keep some state to resume ExpandAll when nodes are received,
	// which is done using the WSTreeView's OnPopulated event further down.
	var expandCollapseAllMode = -1;			// 0 = Collapse, 1 = Expand
	var expandCollapseAllMaxDepth = 99999;	// The maximum depth to expand/collapse

	function init()
	{
		rootNode = me.GetDomElement().firstChild.firstChild._internal.Node;
		rootNode.GetDomElement()._internal.WSHasChildren = true; // Support for nodeFullyLoaded(..)

		var rootedEventId = -1;
		rootedEventId = Fit.Events.AddHandler(me.GetDomElement(), "#rooted", function(elm)
		{
			if (loadDataOnInit === true)
			{
				me.Reload(true);
			}

			Fit.Events.RemoveHandler(me.GetDomElement(), rootedEventId);
		});

		me.OnToggle(function(sender, node)
		{
			// Load node data when expanded

			if (node.IsBehavioralNode() === true)
				return; // Do not request data for behavioral node

			if (node.Expanded() === true) // Node is currently expanded and will now become collapsed
				return;

			if (dataReloading === true && node.GetDomElement()._internal.WSHasChildren === true && node.GetDomElement()._internal.WSDone !== true)
			{
				// All data is currently being reloaded, so loading remote children for a node
				// which will be replaced when reload operation completes does not make sense,
				// and it will fail because the node will be disposed and replaced by the new set
				// of nodes from the reload operation.
				return false;
			}

			if (dataLoading === true && node.GetDomElement()._internal.WSHasChildren === true && node.GetDomElement()._internal.WSDone !== true)
			{
				// Node contains remote children that has not yet been loaded, and
				// data is currently being loaded, e.g. by EnsureData(..), Reload(..),
				// or SelectAll(..). Adding request to process queue.
				onDataLoaded(function() { node.Expanded(true); });
				return false;
			}

			if (node.GetDomElement()._internal.WSLoading === true)
			{
				// Data for this node is currently being loaded.
				// Return False to cancel toggle to prevent user from being
				// able to expand node (and re-trigger data load) multiple times.
				return false;
			}

			var dataStartedLoading = loadNodeData(node, function(nodePopulated) // Returns False if there are no remote nodes to load, or if request is suppressed using an OnRequest handler
			{
				// Children now loaded and populated

				nodesLoading--;

				if (nodeDisposedOrDetached(node) === false)
				{
					delete node.GetDomElement()._internal.WSLoading;
					node.Expanded(true); // Unfortunately causes both OnToggle and OnToggled to be fired, although OnToggle has already been fired once (canceled below)
				}

				if (nodesLoading === 0)
				{
					// Reload, SelectAll, or EnsureData may have been called while
					// node was loading - make sure they are resumed from process queue.
					fireOnDataLoadedToResumeProcessQueue();
				}
			});

			if (dataStartedLoading === true)
			{
				// Prevent e.g. EnsureData, Reload, and SelectAll from loading while current node
				// is being loaded and populated.
				// We cannot use the dataLoading variable for this though, as it would also prevent the
				// user from expanding (and requesting data for) multiple nodes simultaneously since this OnToggle
				// handler prevents the user from expanding nodes if dataLoading is True (set by EnsureData/Reload/etc).
				// Instead we use nodesLoading which is incremented and decremented when node data is requested and populated.
				// EnsureData(..), Reload(..) and other functions fetching data checks this variable, and
				// if its value is above zero, the functions will reschedule and run after all requested
				// nodes are done loading.
				nodesLoading++;

				node.GetDomElement()._internal.WSLoading = true;
				return false; // Cancel toggle, will be "resumed" (triggered again in loadNodeData callback above) when data is loaded
			}

			if (dataStartedLoading === false && node.GetDomElement()._internal.WSHasChildren === true && node.GetChildren().length === 1 && node.GetChildren()[0].Value() === "__" && node.GetChildren()[0].Title() === "__")
			{
				// Node has remote children and no pre-populated children, but data load
				// was not started, meaning it was suppressed using an OnRequest handler.
				// Prevent node from expanding since it will reveal the place holder node making
				// parent node expandable.
				return false;
			}
		});

		//{ SelectAll - Mode: Instantly

		// Instant Mode - how it works:
		// Node on which Select All is triggered will have all its children removed (in OnResponse handler)
		// and replaced by the new data returned. This is to prevent that any children already loaded is
		// expanded and causing separate requests. Instant Mode is quaranteed to only trigger
		// one request and fire OnChange only once, provided that no nodes returned has HasChildren set to True
		// which WILL trigger additional requests and fire OnChange multiple times.
		// Instant Mode provides better performance over Progressive Mode since the latter will constantly
		// occupy the JS thread on and off, as nodes are received from multiple requests, making the browser less responsive.
		// However, Instant Mode loads data by expanding nodes while Progressive Mode can do this without affecting the UI.
		// Progressive Mode is handled in SelectAll function override.
		// TBD: Can/should we change implementation so it uses the new node loader mechanism which Progressive Mode uses,
		// so SelectAll in Instant Mode does not expand nodes, but merely loads children?

		me.OnSelectAll(function(sender, eventArgs)
		{
			if (selectAllMode === Fit.Controls.WSTreeViewSelectAllMode.Progressively)
				return; // Progressive Mode is handled in WSTreeView.SelectAll function override

			var node = ((eventArgs.Node !== null) ? eventArgs.Node : rootNode);
			var internal = node.GetDomElement()._internal;

			// Check whether all nodes have already been loaded

			if (nodeFullyLoaded(node) === true)
				return; // Everything is already loaded, let TreeView.SelectAll take care of selecting and expanding nodes

			// Some (or all) children missing - load now

			// Flags set below ensures that Select All is resumed in OnRequest and OnPopulated event handlers
			internal.WSSelectAll = true; // Used when firing OnRequest, OnResponse, and OnPopulated events which expose an eventArgs.SelectAll boolean
			internal.WSCheckedState = eventArgs.Selected;

			if (node === rootNode) // Load root nodes
			{
				setTimeout(function() // Postpone call to Reload to allow SelectAll to fire all OnSelectAll listeners before requesting data, causing OnRequest to fire
				{
					me.Reload(true);
				}, 0);
			}
			else
			{
				// Clear WebService state in case node has already been expanded
				delete internal.WSDone;
				delete internal.WSLoading;

				// Make node load children after OnSelectAll is canceled below, by returning False
				node.Expanded(false);
				setTimeout(function() { node.Expanded(true); }, 0);
			}

			// Cancel SelectAll, it will cause any existing children to be selected, and in turn fire OnChange.
			// Children will be selected once the complete hierarchy of children is received and populated (see OnResponse and OnPopulated below).
			return false;
		});

		me.OnResponse(function(sender, eventArgs)
		{
			if (selectAllMode === Fit.Controls.WSTreeViewSelectAllMode.Progressively)
				return;

			// Remove any existing children (subtree may have been partially loaded by user).
			// WebService is expected to return entire record set capable of replacing partially loaded data.

			var node = ((eventArgs.Node !== null) ? eventArgs.Node : rootNode);

			if (eventArgs.SelectAll === true) // Select All triggered request
			{
				me._internal.ExecuteWithNoOnChange(function() // Prevent checked nodes from firing OnChange when removed, fires in OnPopulated event handler
				{
					// Deselect children first (recursively) - may need to be synchronized with drop down control
					Fit.Array.Recurse(node.GetChildren(), "GetChildren", function(n)
					{
						n.Selected(false); // NOTICE: In case n.Selectable() returns False, we rely on WebService to return node with the same Selectable and Selected state as it has now
					});

					// Remove nodes - will be replaced in OnPopulated event handler
					Fit.Array.ForEach(node.GetChildren(), function(child)
					{
						node.RemoveChild(child);
					});
				});
			}
		});

		me.OnPopulated(function(sender, eventArgs)
		{
			if (selectAllMode === Fit.Controls.WSTreeViewSelectAllMode.Progressively)
				return;

			var node = ((eventArgs.Node !== null) ? eventArgs.Node : rootNode);
			var selected = node.GetDomElement()._internal.WSCheckedState;

			if (eventArgs.SelectAll === true) // Select All triggered request
			{
				// Select/deselect and expand all nodes

				var changed = false;

				me._internal.ExecuteWithNoOnChange(function()
				{
					if (node !== rootNode && node.Selectable() === true && node.Selected() !== selected)
					{
						node.Selected(selected);
						changed = true;
					}

					Fit.Array.Recurse(node.GetChildren(), "GetChildren", function(child)
					{
						// Support HasChildren:true, even in Instant Mode

						var childInternal = child.GetDomElement()._internal;

						if (childInternal.WSHasChildren === true && childInternal.WSDone !== true) // Node has children server side which have not loaded yet
						{
							Fit.Browser.Log("NOTICE: WebService did not provide all data in one single HTTP request - an additional request is required to load data for node with value '" + child.Value() + "'");
							childInternal.WSSelectAll = true; // Used when firing OnRequest, OnResponse, and OnPopulated events which expose an eventArgs.SelectAll boolean
							childInternal.WSCheckedState = selected;
						}

						// Select node if selectable

						if (child.Selectable() === true && child.Selected() !== selected)
						{
							child.Selected(selected);
							changed = true;
						}

						// Expand node to load children (HasChildren)

						child.Expanded(true);
					});
				});

				if (changed === true)
					me._internal.FireOnChange();
			}
		});

		//}

		// ExpandAll(..) support.
		// Using the OnPopulated event to progressively expand nodes is more
		// complicated than just loading all nodes first with recursivelyLoadAllNodes(..),
		// and via its callback, which is fired when all nodes are loaded, expand all the nodes.
		// But the approach using OnPopulated to resume the ExpandAll operation has the advantage
		// that the user can visually see the progress as nodes expand and their children start loading.
		me.OnPopulated(function(sender, eventArgs)
		{
			if (expandCollapseAllMode === -1)
				return;

			var node = eventArgs.Node;

			if (node === null)
				return; // Root node populated

			if (node.GetLevel() + 1 < expandCollapseAllMaxDepth)
			{
				expandCollapseNodesRecursively(node.GetChildren(), expandCollapseAllMode, expandCollapseAllMaxDepth);
			}
		});
	}

	/// <function container="Fit.Controls.WSTreeView" name="Url" access="public" returns="string">
	/// 	<description>
	/// 		Get/set URL to WebService responsible for providing data to TreeView.
	/// 		WebService must deliver data in the following JSON format:
	/// 		[
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 1", Value: "1001", Selectable: true, Selected: true, Children: [] },
	/// 			&#160;&#160;&#160;&#160; { Title: "Test 2", Value: "1002", Selectable: false, Selected: false, Children: [] }
	/// 		]
	/// 		Only Value is required. Children is a collection of nodes with the same format as described above.
	/// 		HasChildren:boolean may be set to indicate that children are available server side and that WebService
	/// 		should be called to load these children when the given node is expanded.
	/// 		Additionally Expanded:boolean can be set to initially display node as expanded.
	/// 	</description>
	/// 	<param name="wsUrl" type="string" default="undefined"> If defined, updates WebService URL (e.g. http://server/ws/data.asxm/GetNodes) </param>
	/// </function>
	this.Url = function(wsUrl)
	{
		Fit.Validation.ExpectString(wsUrl, true);

		if (Fit.Validation.IsSet(wsUrl) === true)
		{
			url = wsUrl;
		}

		return url;
	}

	/// <function container="Fit.Controls.WSTreeView" name="JsonpCallback" access="public" returns="string | null">
	/// 	<description>
	/// 		Get/set name of JSONP callback argument. Assigning a value will enable JSONP communication.
	/// 		Often this argument is simply &quot;callback&quot;. Passing Null disables JSONP communication again.
	/// 	</description>
	/// 	<param name="val" type="string | null" default="undefined"> If defined, enables JSONP and updates JSONP callback argument </param>
	/// </function>
	this.JsonpCallback = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true || val === null)
		{
			jsonpCallback = val;
		}

		return jsonpCallback;
	}

	/// <function container="Fit.Controls.WSTreeView" name="SelectAllMode" access="public" returns="Fit.Controls.WSTreeViewSelectAllMode">
	/// 	<description>
	/// 		Get/set flag indicating whether WebService returns the complete hierarchy when
	/// 		Select All is triggered (Instantly), or loads data for each level individually
	/// 		when TreeView automatically expands all nodes (Progressively - chain loading).
	/// 	</description>
	/// 	<param name="val" type="Fit.Controls.WSTreeViewSelectAllMode" default="undefined"> If defined, behaviour is set to specified mode </param>
	/// </function>
	this.SelectAllMode = function(val)
	{
		Fit.Validation.ExpectString(val, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (val === Fit.Controls.WSTreeViewSelectAllMode.Progressively || val === Fit.Controls.WSTreeViewSelectAllMode.Instantly)
				selectAllMode = val;
		}

		return selectAllMode;
	}

	// See documentation on TreeView
	this.SelectAll = Fit.Core.CreateOverride(this.SelectAll, function(select, selectAllNode)
	{
		Fit.Validation.ExpectBoolean(select);
		Fit.Validation.ExpectInstance(selectAllNode, Fit.Controls.TreeViewNode, true);

		// NOTICE: Selecting thousands of nodes using SelectAll may result in very poor performance
		// due to the large amount of DOM manipulation. The control goes through a full state change
		// for every single item being selected or de-selected, so all events fire and reflows are
		// triggered for visible items.
		// To increase performance, temporarily hide TreeView with display:none while SelectAll is being performed.

		// TBD: Add support for completed callback (?).
		// EnsureData, which is also based on the new node loader mechanism
		// (loadNodeData and recursivelyLoadAllNodes) keeps track of progress
		// and knows when all nodes have been loaded, so we could easily
		// add support for a callback or an event such as OnSelectAllComplete.
		// We already have an event called OnSelectAll which fires prior to
		// selection changes.
		// We can implement this when/if needed at some point. See EnsureData
		// for inspiration.

		if (selectAllMode === Fit.Controls.WSTreeViewSelectAllMode.Instantly)
		{
			// Fit.Controls.WSTreeViewSelectAllMode.Instantly Mode is handled with
			// OnSelectAll, OnResponse, and OnPopulated handlers registered in init()
			// when TreeView.SelectAll (base) is invoked below.
			base(select, selectAllNode);
			return;
		}

		// Progressive Mode - how it works:
		// Nodes are loaded progressively (chain loaded) from WebService.
		// Nodes with children not loaded yet will be loaded and populated, while
		// nodes already populated will remain (which is the opposite of Instant Mode which removes
		// children already loaded to make sure all data is received in just one request).
		// WebService may return multiple levels of nodes, or even the complete hierarchy, but Progressive Mode
		// will never be able to quarantee that only one request is made, since the node triggering Select All
		// may already have multiple children loaded with HasChildren set to True, which will spawn individual
		// requests.
		// To select all nodes and ensure only one request is made, set SelectAllMode
		// to Fit.Controls.WSTreeViewSelectAllMode.Instantly and make sure the server
		// returns the entire children hierarchy for a given target node.

		if (dataLoading === true || nodesLoading > 0)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.SelectAll(select, selectAllNode); });
			return;
		}

		var internal = (selectAllNode || rootNode).GetDomElement()._internal;
		internal.WSSelectAll = true; // Used when firing OnRequest, OnResponse, and OnPopulated events which expose an eventArgs.SelectAll boolean

		var baseSelectAll = base; // Reference to original SelectAll function on TreeView class
		var exec = function()
		{
			dataLoading = true;

			recursivelyLoadAllNodes(selectAllNode || null, function() // Fired when all nodes have been loaded
			{
				/*//baseSelectAll(select, selectAllNode);
				baseSelectAll(select, selectAllNode, true); // Passing True to suppress firing of OnSelectAllComplete event (internal argument)
				dataLoading = false;
				me._internal.FireOnSelectAllComplete(select, selectAllNode);
				fireOnDataLoadedToResumeProcessQueue();*/

				// This callback passed to recursivelyLoadAllNodes(..) is fired BEFORE OnPopulated
				// is fired, so if SelectAll results in multiple nodes loading data async, we may experience
				// events firing like so:
				//  - OnPopulated(sender, node1) fired
				//  - OnPopulated(sender, node2) fired
				//  - OnPopulated(sender, node3) fired
				//  - SelectAll(sender, nodeA) fired
				//  - SelectAllComplete(sender, nodeA) fired
				//  - OnPopulated(sender, node4) fired
				// Notice how the last node will have OnPopulated called after OnSelect and OnSelectComplete,
				// while all the other nodes had it called prior to that. Such inconsistency is no good. And
				// since SelectAll is async anyways, we might as well just queue the execution of SelectAll
				// using setTimeout(..) allowing for the last OnPopulated event to fire first.
				// This way we can also let baseSelectAll(..) handle execution of both OnSelectAll and OnSelectAllComplete.
				// https://github.com/Jemt/Fit.UI/issues/84

				dataLoading = false;
				fireOnDataLoadedToResumeProcessQueue();
				setTimeout(function() { baseSelectAll(select, selectAllNode); }, 0); // Allow OnPopulated to fire first in recursivelyLoadAllNodes => loadNodeData => getData
			}/*,
			function(nodePopulated) // Fired for every node loaded, except if node has been disposed or detached while loading
			{
				//nodePopulated.Expanded(true); // DISABLED: This hurts performance significantly for large TreeViews (10-15.000 nodes) as a huge amounts of DOM elements needs to be pushed to render tree, e.g. when opening/closing dropdown hosting TreeView
			}*/);
		};

		if (loadDataOnInit === true) // No data loaded yet
		{
			me.Reload(true, function(sender) // Will change loadDataOnInit to False
			{
				exec();
			});
		}
		else // Data already (partially) loaded - often just the root nodes
		{
			exec();
		}
	});

	// See documentation on TreeView
	this.RemoveAllChildren = Fit.Core.CreateOverride(this.RemoveAllChildren, function(dispose)
	{
		Fit.Validation.ExpectBoolean(dispose, true);

		preSelected = {}; // Clear preselections to avoid auto selection of nodes added later
		base(dispose);
	});

	// See documentation on TreeView
	this.Selectable = Fit.Core.CreateOverride(this.Selectable, function(val, multi, showSelAll)
	{
		Fit.Validation.ExpectBoolean(val, true);
		Fit.Validation.ExpectBoolean(multi, true);
		Fit.Validation.ExpectBoolean(showSelAll, true);

		if (Fit.Validation.IsSet(val) === true)
		{
			if (Fit.Array.Count(preSelected) > 0)
			{
				// Help developers - it might be confusing that changing Selectable state also clears selection
				Fit.Browser.Debug("WSTreeView: Selectable changed - PreSelections will be cleared");
			}

			preSelected = {}; // Clear preselections to ensure same behaviour as TreeView.Selectable(..) which clear selections - avoid auto selection if nodes are loaded later
		}

		return base(val, multi, showSelAll);
	});

	// See documentation on ControlBase
	this.Clear = Fit.Core.CreateOverride(this.Clear, function()
	{
		var fireChange = (me.Selected().length > 0); // Selected() return nodes loaded as well as preselections

		preSelected = {};

		me._internal.ExecuteWithNoOnChange(function() { base(); });

		if (fireChange === true)
			me._internal.FireOnChange();
	});

	/// <function container="Fit.Controls.WSTreeViewTypeDefs" name="ReloadCallback">
	/// 	<description> Reload callback </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// </function>

	/// <function container="Fit.Controls.WSTreeView" name="Reload" access="public">
	/// 	<description> Reload data from WebService </description>
	/// 	<param name="keepState" type="boolean" default="undefined">
	/// 		If defined, True will preserve selections, expanded state, and focus state, False will not (default)
	/// 	</param>
	/// 	<param name="cb" type="Fit.Controls.WSTreeViewTypeDefs.ReloadCallback" default="undefined">
	/// 		If defined, callback function is invoked when root nodes have been loaded
	/// 		and populated - takes Sender (Fit.Controls.WSTreeView) as an argument.
	/// 	</param>
	/// </function>
	this.Reload = function(a, b) // Correct signature: Reload(boolean, function)
	{
		// Backward compatibility - temporary support for deprecated Reload signature: Reload([callback])

		var keepState = undefined;
		var cb = undefined;

		if (typeof(a) === "boolean")
		{
			// Correct signature used: Reload([bool[, callback]])

			keepState = a;
			cb = b;
		}
		else if (typeof(a) === "function")
		{
			// Deprecated signature used: Reload([callback])

			keepState = b;
			cb = a;

			Fit.Browser.Log("WARNING: Using deprecated function signature for WSTreeView.Reload(callback) which will be removed in the future - please use Reload(boolean, callback) instead");
		}

		// End of backward compatibility for deprecated Reload signature

		Fit.Validation.ExpectBoolean(keepState, true);
		Fit.Validation.ExpectFunction(cb, true);

		// Disable control when new data is requested so old data cannot be selected

		if (me.Enabled() === true && dataReloadingEnableFunc === null)
		{
			dataReloadingEnableFunc = me._internal.DisableAndKeepFocus(); // Only preserves focus if control is already focused, of course
			dataReloading = true;
		}

		// Postpone operation if currently loading data

		if (dataLoading === true || nodesLoading > 0)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.Reload(keepState, cb); });
			return;
		}

		// Load data

		dataLoading = true;
		dataReloading = true;

		getData(null, keepState === true, function(node, eventArgs) // Notice: node argument is null when requesting root nodes
		{
			// Preserve expanded state

			var expanded = {};

			if (keepState === true)
			{
				Fit.Array.ForEach(me.GetAllNodes(), function(node)
				{
					if (node.Expanded() === true)
					{
						expanded[node.Value()] = true;
					}
				});
			}

			// Get focused or highlighted node

			var focusedNode = keepState === true && me.GetNodeFocused() !== null ? me.GetNodeFocused().Value() : null;		// Focused node (real focus) - when WSTreeView is used as a separate control
			var highlightedNode = keepState === true && me.GetHighlighted() !== null ? me.GetHighlighted().Value : null;	// Highlighted noded - when WSTreeView is used as a picker control

			// Preserve selection if instructed to.
			// Nodes will automatatically be selected again using preselections
			// once data has been populated. This happens in the getData(..) function.

			var selected = me.Selected();
			var newPreselection = {};

			if (keepState === true)
			{
				Fit.Array.ForEach(selected, function(node)
				{
					newPreselection[node.Value()] = createPreSelectionNode(node.Title(), node.Value());
				});
			}
			else
			{
				// Clear control if selection is not preserved. This ensures that events such as
				// OnSelect and OnSelected (PickerBase) and OnChange fires for nodes currently selected,
				// which in turn updates a host control using WSTreeView as a picker control.
				me.Clear();
			}

			// Preserve scroll position in case user scrolled view while data was loading. Calling node.Focused(true)
			// or SetActiveNode(..) further down will not only focus or highlight node, but also scroll it into view.
			var scrollbars = Fit.Dom.GetScrollBars(me.GetDomElement());
			var scrollParent = scrollbars.Vertical.Enabled === true || scrollbars.Horizontal.Enabled === true ? me.GetDomElement() : Fit.Dom.GetScrollParent(me.GetDomElement());
			var scrollPositionToRestore = keepState === true ? { Top: scrollParent.scrollTop, Left: scrollParent.scrollLeft } : null;

			// Remove nodes
			me._internal.ExecuteWithNoOnChange(function()
			{
				me.RemoveAllChildren(true); // True to dispose objects - also clears selections, including preselections
			});

			preSelected = newPreselection;

			// Restore expanded state (createNodeFromJson reads Expanded property from JSON children).
			// NOTICE: Nodes with remote children (HasChildren is true) will NOT be expanded, even if
			// expanded prior to calling Reload(true, ..)! These nodes must be expanded using TreeViewNode.Expanded(true)
			// to load data, and it will cause data to load async. which will result in additional updates to the UI - and one
			// of the intentions with keepState is to reduce such "flickering" in the user interface. If one really wants
			// to preserve expanded state for nodes with remote children, then enrich the request to the server with information
			// about what children are expanded so their children can be included in the data response, and set HasChildren to False.
			if (keepState === true)
			{
				Fit.Array.Recurse(eventArgs.Children, "Children", function(jsonChild)
				{
					if (jsonChild.HasChildren === true)
					{
						return; // Skip node with remote children - see reason in comment above
					}

					jsonChild.Expanded = expanded[jsonChild.Value] === true;
				});
			}

			// Populate nodes
			Fit.Array.ForEach(eventArgs.Children, function(jsonChild)
			{
				me.AddChild(createNodeFromJson(jsonChild));
			});

			// Re-enable control
			if (dataReloadingEnableFunc !== null)
			{
				dataReloadingEnableFunc();
				dataReloadingEnableFunc = null;
			}

			// Focus or highlight previously focused/highlighted node
			if (focusedNode !== null && me.Focused() === true)
			{
				var nodeToFocus = me.GetChild(focusedNode, true);

				if (nodeToFocus !== null)
				{
					nodeToFocus.Focused(true);
				}
			}
			else if (highlightedNode !== null)
			{
				var nodeToActivate = me.GetChild(highlightedNode, true);

				if (nodeToActivate !== null)
				{
					me.SetActiveNode(nodeToActivate);
				}
			}

			// Restore scroll position
			if (scrollPositionToRestore !== null)
			{
				scrollParent.scrollTop = scrollPositionToRestore.Top;
				scrollParent.scrollLeft = scrollPositionToRestore.Left;
			}

			dataLoading = false;
			dataReloading = false;

			rootNode.GetDomElement()._internal.WSDone = true;

			/*if (Fit.Validation.IsSet(cb) === true)
			{
				cb(me); // Fires when nodes are populated (above), but before OnPopulated is fired by getData(..)
			}*/

			fireOnDataLoadedToResumeProcessQueue();

			if (Fit.Validation.IsSet(cb) === true)
			{
				setTimeout(function() { cb(me); }, 0); // Allow OnPopulated to fire first in recursivelyLoadAllNodes => loadNodeData => getData - see WSTreeView.SelectAll(..) for details
			}
		});

		loadDataOnInit = false;
	}

	/// <function container="Fit.Controls.WSTreeView" name="EnsureData" access="public">
	/// 	<description>
	/// 		Ensure all data from WebService.
	/// 		Contrary to Reload(..), this function does not clear selected
	/// 		values, or remove nodes already loaded - it merely loads data
	/// 		not already loaded.
	/// 	</description>
	/// 	<param name="callback" type="Fit.Controls.WSTreeViewTypeDefs.ReloadCallback" default="undefined">
	/// 		If defined, callback function is invoked when all nodes have been loaded
	/// 		and populated - takes Sender (Fit.Controls.WSTreeView) as an argument.
	/// 	</param>
	/// </function>
	this.EnsureData = function(cb)
	{
		Fit.Validation.ExpectFunction(cb, true);

		if (dataLoading === true || nodesLoading > 0)
		{
			// Data is currently loading - postpone by adding request to process queue
			onDataLoaded(function() { me.EnsureData(cb); });
			return;
		}

		var exec = function()
		{
			dataLoading = true;

			recursivelyLoadAllNodes(null, function() // Fired when all nodes have been loaded
			{
				dataLoading = false;

				/*if (Fit.Validation.IsSet(cb) === true)
				{
					cb(me);
				}*/

				fireOnDataLoadedToResumeProcessQueue();

				if (Fit.Validation.IsSet(cb) === true)
				{
					setTimeout(function() { cb(me); }, 0); // Allow OnPopulated to fire first in recursivelyLoadAllNodes => loadNodeData => getData - see WSTreeView.SelectAll(..) for details
				}
			});
		};

		if (me.GetChildren().length === 0) // No nodes in TreeView - we check against node count instead of loadDataOnInit since we want to be able to re-ensure nodes in case they we initially loaded and later removed
		{
			me.Reload(true, function(sender) // Will change loadDataOnInit to False
			{
				exec();
			});
		}
		else // Data already (partially) loaded - often just the root nodes
		{
			exec();
		}
	}

	// See documentation on TreeView
	this.ExpandAll = function(maxDepth)
	{
		Fit.Validation.ExpectInteger(maxDepth, true);
		setExpandedStateForAllNodes(1, maxDepth);
	}

	// See documentation on TreeView
	this.CollapseAll = function(maxDepth)
	{
		Fit.Validation.ExpectInteger(maxDepth, true);
		setExpandedStateForAllNodes(0, maxDepth);
	}

	// See documentation on ControlBase
	this.Value = Fit.Core.CreateOverride(this.Value, function(val, preserveDirtyState)
	{
		Fit.Validation.ExpectString(val, true);
		Fit.Validation.ExpectBoolean(preserveDirtyState, true);

		// Setter

		if (Fit.Validation.IsSet(val) === true)
		{
			orgSelected = (preserveDirtyState !== true ? [] : orgSelected);
			preSelected = {};
			var fireOnChange = (me.Selected().length > 0); // Selected() return nodes already loaded and preselections

			me._internal.ExecuteWithNoOnChange(function()
			{
				// Select nodes already loaded

				base(val);
				var selected = baseSelected(); // Get selection just made as node objects, which can be iterated using ForEach

				if (selected.length > 0)
					fireOnChange = true;

				// Update orgSelected used to determine dirty state

				if (preserveDirtyState !== true)
				{
					Fit.Array.ForEach(selected, function(node)
					{
						Fit.Array.Add(orgSelected, node.Value());
					});
				}

				// Add nodes not loaded yet to preselections

				var values = val !== "" ? val.split(";") : [];

				Fit.Array.ForEach(values, function(nodeVal)
				{
					var info = nodeVal.split("=");
					var preSel = { Title: null, Value: null };

					if (info.length === 1)
					{
						preSel.Title = "[pre-selection]";
						preSel.Value = decodeReserved(info[0]);
					}
					else
					{
						preSel.Title = decodeReserved(info[0]);
						preSel.Value = decodeReserved(info[1]);
					}

					if (me.GetChild(preSel.Value, true) === null)
					{
						preSelected[preSel.Value] = createPreSelectionNode(preSel.Title, preSel.Value);

						if (preserveDirtyState !== true)
						{
							Fit.Array.Add(orgSelected, preSel.Value);
						}

						fireOnChange = true;
					}
				});
			});

			if (fireOnChange === true)
				me._internal.FireOnChange();
		}

		// Getter

		var nodes = me.Selected();
		var value = "";

		Fit.Array.ForEach(nodes, function(node)
		{
			value += ((value !== "") ? ";" : "") + encodeReserved(node.Title()) + "=" + encodeReserved(node.Value());
		});

		return value;
	});

	/// <function container="Fit.Controls.WSTreeView" name="SetNodeSelection" access="public">
	/// 	<description>
	/// 		Allows for a node's selection state to be set even if node has not been loaded yet
	/// 	</description>
	/// 	<param name="value" type="string"> Node value </param>
	/// 	<param name="selected" type="boolean"> Node selection state </param>
	/// </function>
	this.SetNodeSelection = function(value, selected)
	{
		Fit.Validation.ExpectString(value);
		Fit.Validation.ExpectBoolean(selected);

		var node = me.GetChild(value, true);

		if (node !== null)
		{
			node.Selected(selected); // Will fire OnChange
		}
		else
		{
			if (selected === true && preSelected[value] === undefined)
			{
				preSelected[value] = createPreSelectionNode("[pre-selection]", value);
				me._internal.FireOnChange();
			}
			else if (selected === false && preSelected[value] !== undefined)
			{
				delete preSelected[value];
				me._internal.FireOnChange();
			}
		}
	}

	/// <function container="Fit.Controls.WSTreeView" name="Selected" access="public" returns="Fit.Controls.TreeViewNode[]">
	/// 	<description>
	/// 		Fit.Controls.TreeView.Selected override:
	/// 		Get/set selected nodes.
	/// 		Notice for getter: Nodes not loaded yet (preselections) are NOT associated with TreeView.
	/// 		They are there to indicate a selection that has not yet occurred since the data has not yet been loaded.
	/// 		Changing properties of these nodes (e.g. node.Expanded(boolean) or node.Selected(boolean)) will not have any effect on the TreeView.
	/// 		Such nodes can be identified using GetTreeView(): if (node.GetTreeView() === null) console.log("This is a preselection node");
	/// 		The following functions can be used to get node information: node.Title() and node.Value().
	/// 	</description>
	/// 	<param name="val" type="Fit.Controls.TreeViewNode[]" default="undefined"> If defined, provided nodes are selected </param>
	/// </function>
	this.Selected = function(val)
	{
		Fit.Validation.ExpectInstanceArray(val, Fit.Controls.TreeViewNode, true);

		// Setter

		if (Fit.Validation.IsSet(val) === true)
		{
			// Exclude nodes not belonging to TreeView - trying to select them
			// will cause an error to be thrown from TreeView.Selected (base).
			// Nodes excluded are added as preselections instead.

			var toSelect = [];

			Fit.Array.ForEach(val, function(n)
			{
				var node = ((n.GetTreeView() === me) ? n : me.GetChild(n.Value(), true)); // Try GetChild(..) in case node was constructed, but with a value of an existing node

				if (node !== null)
					Fit.Array.Add(toSelect, node);
			});

			// Select nodes

			orgSelected = [];
			preSelected = {};
			var fireOnChange = (me.Selected().length > 0); // Selected() return nodes already loaded and preselections

			// Select nodes already loaded

			var selected = null;
			me._internal.ExecuteWithNoOnChange(function() { selected = baseSelected(toSelect); });

			if (selected.length > 0)
				fireOnChange = true;

			// Update orgSelected used to determine dirty state

			Fit.Array.ForEach(selected, function(node)
			{
				Fit.Array.Add(orgSelected, node.Value());
			});

			// Add nodes not loaded yet to preselections

			Fit.Array.ForEach(val, function(node)
			{
				if (me.GetChild(node.Value(), true) === null)
				{
					preSelected[node.Value()] = createPreSelectionNode(node.Title(), node.Value());
					Fit.Array.Add(orgSelected, node.Value());
					fireOnChange = true;
				}
			});

			// Fire OnChange

			if (fireOnChange === true)
			{
				me._internal.FireOnChange();
			}
		}

		// Getter

		var nodes = baseSelected();

		// Add preselections - they are considered part of value
		Fit.Array.ForEach(preSelected, function(preSelVal)
		{
			Fit.Array.Add(nodes, preSelected[preSelVal]);
		});

		return nodes;
	}

	// See documentation on ControlBase
	this.IsDirty = Fit.Core.CreateOverride(this.IsDirty, function()
	{
		// Get nodes currently selected

		var valuesSelected = [];

		Fit.Array.ForEach(me.Selected(), function(selected)
		{
			Fit.Array.Add(valuesSelected, selected.Value());
		});

		// Determine dirty state

		if (valuesSelected.length !== orgSelected.length)
			return true;

		var dirty = false;
		Fit.Array.ForEach(orgSelected, function(val)
		{
			if (Fit.Array.Contains(valuesSelected, val) === false)
			{
				dirty = true;
				return false;
			}
		});
		return dirty;
	});

	// See documentation on ControlBase
	this.Dispose = Fit.Core.CreateOverride(this.Dispose, function()
	{
		me = url = jsonpCallback = preSelected = orgSelected = loadDataOnInit = dataLoading = dataReloading = dataReloadingEnableFunc = nodesLoading = recursiveNodeLoadCount = onDataLoadedCallback = onRequestHandlers = onResponseHandlers = onPopulatedHandlers = baseSelected = expandCollapseAllMode = expandCollapseAllMaxDepth = null;

		base();
	});

	// ============================================
	// PickerBase interface
	// ============================================

	/*this.OnShow(function(sender) // Event defined on PickerBase
	{
		if (loadDataOnInit === true)
		{
			me.Reload();
			loadDataOnInit = false;
		}
	});*/

	// See documentation on PickerBase
	this.SetSelections = Fit.Core.CreateOverride(this.SetSelections, function(items)
	{
		Fit.Validation.ExpectArray(items);

		var fireOnChange = (me.Selected().length > 0); // me.Selected() return nodes already loaded and preselections

		// Make sure calls to Value() or Selected() does not return old preselections for events fired

		preSelected = {};

		// Call SetSelections(..) on base class to make sure nodes already loaded is immediately selected.

		me._internal.ExecuteWithNoOnChange(function() { base(items); });

		if (me.Selected().length > 0)
			fireOnChange = true;

		// Set preselections for nodes not loaded yet

		// Add all values to preselections - preselections for nodes already loaded is removed further down
		Fit.Array.ForEach(items, function(item)
		{
			Fit.Validation.ExpectString(item.Title);
			Fit.Validation.ExpectString(item.Value);

			preSelected[item.Value] = createPreSelectionNode(item.Title, item.Value);
		});

		// Remove values from preselections if already loaded
		Fit.Array.ForEach(baseSelected(), function(selected)
		{
			delete preSelected[selected.Value()];
		});

		if (Fit.Core.IsEqual(preSelected, {}) === false)
			fireOnChange = true;

		// Fire OnChange

		if (fireOnChange === true)
			me._internal.FireOnChange();
	});

	// See documentation on PickerBase
	this.UpdateItemSelection = Fit.Core.CreateOverride(this.UpdateItemSelection, function(itemValue, selected, programmaticallyChanged)
	{
		Fit.Validation.ExpectString(itemValue);
		Fit.Validation.ExpectBoolean(selected);
		Fit.Validation.ExpectBoolean(programmaticallyChanged);

		if (me.GetChild(itemValue, true) === null) // Node not loaded yet, update preselections
		{
			// Since the node is not loaded yet, OnSelect, OnItemSelectionChanging,
			// OnSelected, and OnItemSelectionChanged are not fired - they fire once the node is loaded.

			if (selected === true && preSelected[itemValue] === undefined)
			{
				preSelected[itemValue] = createPreSelectionNode("[pre-selection]", itemValue);
				me._internal.FireOnChange();
			}
			else if (selected === false && preSelected[itemValue] !== undefined)
			{
				delete preSelected[itemValue];
				me._internal.FireOnChange();
			}
		}
		else // Update nodes already loaded
		{
			if (base(itemValue, selected, programmaticallyChanged) === false) // Fires OnSelect, OnItemSelectionChanging, OnSelected, OnItemSelectionChanged, and OnChange
				return false;
		}
	});

	// ============================================
	// Events
	// ============================================

	/// <container name="Fit.Controls.WSTreeViewTypeDefs.EventHandlerArgs">
	/// 	<description> Request handler event arguments </description>
	/// 	<member name="Sender" type="$TypeOfThis"> Instance of control </member>
	/// 	<member name="Request" type="Fit.Http.JsonRequest | Fit.Http.JsonpRequest"> Instance of JsonRequest or JsonpRequest </member>
	/// 	<member name="Node" type="Fit.Controls.TreeViewNode"> Instance of TreeViewNode </member>
	/// </container>

	/// <container name="Fit.Controls.WSTreeViewTypeDefs.JsonItem">
	/// 	<description> JSON object representing node </description>
	/// 	<member name="Value" type="string"> Unique value </member>
	/// 	<member name="Title" type="string" default="undefined"> Title - using Value if not defined </member>
	/// 	<member name="Selected" type="boolean" default="undefined"> Value indicating whether item is initially selected - not selected by default </member>
	/// 	<member name="Selectable" type="boolean" default="undefined"> Value indicating whether item can be selected or not - not selectable by default </member>
	/// 	<member name="HasChildren" type="boolean" default="undefined"> Set True to trigger an additional request to web service to retrieve children for this item when it is expanded </member>
	/// 	<member name="Children" type="Fit.Controls.WSTreeViewTypeDefs.JsonItem[]" default="undefined"> Children </member>
	/// 	<member name="Expanded" type="boolean" default="undefined"> Value indicating whether item is initially expanded to reveal children - not expanded by default </member>
	/// 	<member name="Supplementary" type="{[key:string]: string}" default="undefined"> Associative string array to carry additional information </member>
	/// </container>

	/// <container name="Fit.Controls.WSTreeViewTypeDefs.DataHandlerEventArgs" extends="Fit.Controls.WSTreeViewTypeDefs.EventHandlerArgs">
	/// 	<description> Data event handler arguments </description>
	/// 	<member name="Children" type="Fit.Controls.WSTreeViewTypeDefs.JsonItem[]"> JSON objects representing nodes </member>
	/// </container>

	/// <container name="Fit.Controls.WSTreeViewTypeDefs.AbortHandlerEventArgs" extends="Fit.Controls.WSTreeViewTypeDefs.EventHandlerArgs">
	/// 	<description> Abort event handler arguments </description>
	/// 	<member name="Children" type="null"> JSON objects representing nodes </member>
	/// </container>

	/// <function container="Fit.Controls.WSTreeViewTypeDefs" name="CancelableRequestEventHandler" returns="boolean | void">
	/// 	<description> Cancelable request event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSTreeViewTypeDefs.EventHandlerArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSTreeViewTypeDefs" name="DataEventHandler">
	/// 	<description> Data event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSTreeViewTypeDefs.DataHandlerEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSTreeViewTypeDefs" name="AbortEventHandler">
	/// 	<description> Abort event handler </description>
	/// 	<param name="sender" type="$TypeOfThis"> Instance of control </param>
	/// 	<param name="eventArgs" type="Fit.Controls.WSTreeViewTypeDefs.AbortHandlerEventArgs"> Event arguments </param>
	/// </function>

	/// <function container="Fit.Controls.WSTreeView" name="OnRequest" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is being requested.
	/// 		Request can be canceled by returning False.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSTreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSTreeView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Node: Fit.Controls.TreeViewNode instance
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSTreeViewTypeDefs.CancelableRequestEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnRequest = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onRequestHandlers, cb);
	}

	/// <function container="Fit.Controls.WSTreeView" name="OnResponse" access="public">
	/// 	<description>
	/// 		Add event handler fired when data is received,
	/// 		allowing for data transformation to occure before
	/// 		TreeView is populated. Function receives two arguments:
	/// 		Sender (Fit.Controls.WSTreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSTreeView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Node: Fit.Controls.TreeViewNode instance to be populated
	/// 		 - Children: JSON nodes received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSTreeViewTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnResponse = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onResponseHandlers, cb);
	}

	/// <function container="Fit.Controls.WSTreeView" name="OnAbort" access="public">
	/// 	<description>
	/// 		Add event handler fired if data request is canceled.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSTreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSTreeView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Node: Fit.Controls.TreeViewNode instance to be populated
	/// 		 - Children: JSON nodes received from WebService (Null in this particular case)
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSTreeViewTypeDefs.AbortEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnAbort = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onAbortHandlers, cb);
	}

	/// <function container="Fit.Controls.WSTreeView" name="OnPopulated" access="public">
	/// 	<description>
	/// 		Add event handler fired when TreeView has been populated with nodes.
	/// 		Node is not populated and event is not fired though if node is disposed
	/// 		or detached from TreeView while data is loading from WebService.
	/// 		Function receives two arguments:
	/// 		Sender (Fit.Controls.WSTreeView) and EventArgs object.
	/// 		EventArgs object contains the following properties:
	/// 		 - Sender: Fit.Controls.WSTreeView instance
	/// 		 - Request: Fit.Http.JsonpRequest or Fit.Http.JsonRequest instance
	/// 		 - Node: Fit.Controls.TreeViewNode instance now populated with children
	/// 		 - Children: JSON nodes received from WebService
	/// 	</description>
	/// 	<param name="cb" type="Fit.Controls.WSTreeViewTypeDefs.DataEventHandler">
	/// 		Event handler function
	/// 	</param>
	/// </function>
	this.OnPopulated = function(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onPopulatedHandlers, cb);
	}

	// ============================================
	// Private
	// ============================================

	// Several functions end up calling getData(..) which is a bit confusing.
	// The following link has a drawing that clarifies how this works: https://github.com/Jemt/Fit.UI/issues/83
	function getData(node, forcePersist, cb)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode, true); // Node is null when requesting root nodes
		Fit.Validation.ExpectBoolean(forcePersist);
		Fit.Validation.ExpectFunction(cb);

		if (url === null)
			Fit.Validation.ThrowError("Unable to get data, no WebService URL has been specified");

		// Determine request method

		var request = null;

		if (jsonpCallback !== null)
		{
			request = new Fit.Http.JsonpRequest(url, jsonpCallback);
		}
		else
		{
			request = new Fit.Http.JsonRequest(url)
		}

		// Fire OnRequest

		var eventArgs = { Sender: null, Node: null, Request: null, Children: null };
		eventArgs.Sender = me;
		eventArgs.Node = node;
		eventArgs.Request = request;
		eventArgs.SelectAll = (node || rootNode).GetDomElement()._internal.WSSelectAll === true;

		if (fireEventHandlers(onRequestHandlers, eventArgs) === false)
			return false;

		if (eventArgs.Request !== request)
		{
			// Support for changing request instans to
			// take control over webservice communication.

			// Restrict to support for Fit.Http.Request or classes derived from this
			Fit.Validation.ExpectInstance(eventArgs.Request, Fit.Http.Request);

			request = eventArgs.Request;
		}

		// Define request callbacks

		var onSuccess = function(children)
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			// Fire OnResponse

			eventArgs.Children = ((children instanceof Array) ? children : []);
			fireEventHandlers(onResponseHandlers, eventArgs);

			// Fire getData callback

			// WARNING: node might have been disposed - WebService communication is async!
			cb(node, eventArgs); // Callback is responsible for populating TreeView

			// Remove loading indicator

			if (request._loadingIndicator !== undefined) // Loading indicator not set when requesting root nodes
			{
				Fit.Dom.Remove(request._loadingIndicator);
				delete request._loadingIndicator;
			}

			// Highlight first node

			if (node === null) // Only do this for root nodes
			{
				if (me.PersistView() === true && forcePersist === false)
				{
					// Reset PersistView to make sure it does not reuse state
					// for newly loaded nodes, in case node list is changed.
					me.PersistView(false);
					me.PersistView(true);
				}

				if (me.HighlightFirst() === true)
				{
					// Reset HighlightFirst to make sure it takes effect again
					// for newly loaded nodes, in case node list is changed.
					me.HighlightFirst(false);
					me.HighlightFirst(true);

					me._internal.FocusFirstNode();
				}
			}

			// Select nodes found in preselections

			var hasBeenDisposedOrDetached = node !== null && nodeDisposedOrDetached(node);

			if (hasBeenDisposedOrDetached === false && Fit.Core.IsEqual(preSelected, {}) === false) // Prevent nodes from being iterated if no preselections are found
			{
				// Notice: OnChange is not fired! It has already been fired when PreSelections were added.
				// They are also included when e.g. WSTreeView.Value() is called to obtain current selections.
				// However, OnSelect (which fires OnItemSelectionChanging) and OnSelected (which fires OnItemSelectionChanged)
				// fires when node.selected(boolean) is called below.
				me._internal.ExecuteWithNoOnChange(function()
				{
					Fit.Array.Recurse(((node !== null) ? node.GetChildren() : me.GetChildren()), "GetChildren", function(n)
					{
						if (preSelected[n.Value()] !== undefined)
						{
							delete preSelected[n.Value()];
							n.Selected(true);
						}
					});
				});
			}

			// Fire OnPopulated.
			// Contrary to OnRequest, OnResponse, and OnAbort, OnPopulated does not
			// fire if node has been disposed or detached, in which case WSTreeView
			// will not have populated the node.
			// OnRequest/OnResponse/OnAbort handlers can check whether a given node
			// has been disposed or detached like this:
			// var disposed = node.GetDomElement() === null;
			// var detached = !disposed && node.GetTreeView() === null;
			// var moved = !disposed && node.GetTreeView() !== myTreeViewInstance;
			// var disposedOrDetached = node.GetDomElement() === null || node.GetTreeView() === null;

			if (hasBeenDisposedOrDetached === false)
			{
				fireEventHandlers(onPopulatedHandlers, eventArgs);
			}
		}

		var onFailure = function(httpStatusCode)
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			Fit.Validation.ThrowError("Unable to get children for " + ((node !== null) ? "node '" + node.Title() + "'" : "root level") + " - request failed with HTTP Status code " + httpStatusCode)
		}

		var onAbort = function()
		{
			if (me === null)
				return; // Control was disposed while waiting for data to be loaded

			if (request._loadingIndicator !== undefined) // Loading indicator not set when requesting root nodes
			{
				Fit.Dom.Remove(request._loadingIndicator);
				delete request._loadingIndicator;
			}

			if (node !== null) // Null when requesting root nodes
			{
				if (nodeDisposedOrDetached(node) === false)
				{
					delete node.GetDomElement()._internal.WSLoading;
				}
			}

			fireEventHandlers(onAbortHandlers, eventArgs);
		};

		// Register request callbacks

		if (Fit.Core.InstanceOf(request, Fit.Http.JsonpRequest) === false)
		{
			request.OnSuccess(function(req)
			{
				var children = request.GetResponseJson();
				onSuccess(children);
			});

			request.OnFailure(function(req)
			{
				onFailure(request.GetHttpStatus());
			});

			request.OnAbort(function(req)
			{
				onAbort();
			});
		}
		else
		{
			request.OnSuccess(function(req)
			{
				var children = req.GetResponse();
				onSuccess(children);
			});

			request.OnTimeout(function()
			{
				onFailure("UNKNOWN (JSONP)");
			});
		}

		// Display loading indicator

		if (node !== null) // Null when requesting root nodes
		{
			var li = node.GetDomElement();

			request._loadingIndicator = document.createElement("i");
			Fit.Dom.AddClass(request._loadingIndicator, "FitUiControlLoadingIndicator");
			Fit.Dom.Add(li, request._loadingIndicator);
		}

		// Invoke request

		request.Start();

		return true;
	}

	function onDataLoaded(cb)
	{
		Fit.Validation.ExpectFunction(cb);
		Fit.Array.Add(onDataLoadedCallback, cb);
	}

	function fireOnDataLoadedToResumeProcessQueue()
	{
		// Immediately clear collection. If multiple callbacks are registered,
		// chances are that only the first will run, and the remaining will be
		// re-scheduled again - so we need the collection to be cleared before
		// invoking callbacks.
		var orgOnDataLoadedCallback = onDataLoadedCallback;
		onDataLoadedCallback = [];

		Fit.Array.ForEach(orgOnDataLoadedCallback, function(cb)
		{
			cb();
		});
	}

	function createNodeFromJson(jsonNode)
	{
		Fit.Validation.ExpectIsSet(jsonNode);

		// Convert JSON to TreeView node, including all contained children

		var child = new Fit.Controls.TreeViewNode((jsonNode.Title ? jsonNode.Title : jsonNode.Value), jsonNode.Value, { HtmlAllowed: me.HtmlAllowed() });

		if (jsonNode.Selectable !== undefined)
			child.Selectable((jsonNode.Selectable === true)); // Node will obtain Selectable state from TreeView unless explicitly set here

		if (jsonNode.Selected !== undefined)
			child.Selected((jsonNode.Selected === true)); // Notice, will not cause various events to fire since TreeViewNodeInterface is not assigned at this point

		if (jsonNode.Children instanceof Array)
		{
			Fit.Array.ForEach(jsonNode.Children, function(c)
			{
				child.AddChild(createNodeFromJson(c));
			});
		}

		if (jsonNode.Expanded !== undefined)
			child.Expanded((jsonNode.Expanded === true)); // Notice, will not cause various events to fire since TreeViewNodeInterface is not assigned at this point

		// Set internal flag indicating whether node has children server side

		child.GetDomElement()._internal.WSHasChildren = (jsonNode.HasChildren === true);

		// Add fake child to node if HasChildren indicates that there are more children
		// server side. Adding the fake child ensures that the node can be expanded, which
		// allow us to use OnToggle to load the children at this point.
		// However, the current node may already have some of its children added,
		// so only add the fake child if no children are found.

		if (jsonNode.HasChildren === true && child.GetChildren().length === 0)
		{
			// Make node expandable by attaching a place holder node
			// which is automatically removed when node is expanded.
			var expanderNode = new Fit.Controls.TreeViewNode("__", "__");
			expanderNode.Selectable(false); // Prevent it from being selected when using SelectAll
			child.AddChild(expanderNode);
		}

		// Add supplementary information provided by WebService

		child.Supplementary = {};
		var reserved = new Array("Title", "Value", "Children", "HasChildren", "Selected", "Selectable", "Expanded");

		Fit.Array.ForEach(jsonNode, function(key)
		{
			if (Fit.Array.Contains(reserved, key) === false)
				child.Supplementary[key] = jsonNode[key];
		});

		return child;
	}

	function createPreSelectionNode(title, value)
	{
		Fit.Validation.ExpectString(title);
		Fit.Validation.ExpectString(value);

		var node = new Fit.Controls.TreeViewNode(title, value);
		node.Selectable(true);
		node.Selected(true);

		return node;
	}

	function nodeFullyLoaded(node) // Check whether all children has been loaded
	{
		var internal = node.GetDomElement()._internal;

		if (internal.WSSelectAll === true)
			return true; // This node previously caused SelectAll

		// Check whether node was part of hierarchy loaded using Select All

		var parent = node.GetParent();

		while (parent !== null)
		{
			if (parent.GetDomElement()._internal.WSSelectAll === true)
				return true;

			parent = parent.GetParent();
		}

		// Check whether all children are loaded (usually manually by expanding nodes)

		var fullyLoaded = true;

		Fit.Array.Recurse([node], "GetChildren", function(child)
		{
			var childInternal = child.GetDomElement()._internal;

			if (childInternal.WSHasChildren === true && childInternal.WSDone !== true)
			{
				fullyLoaded = false;
				return false; // Break loop
			}
		});

		return fullyLoaded;
	}

	// Check whether a node has been disposed or detached from TreeView.
	// We use this function when loading nodes async, to make sure the
	// nodes are still in working order when data is received, which will
	// not be the case for a node that has been disposed (destroyed).
	// Also, a node that has been detached should no longer affect this
	// TreeView instance.
	function nodeDisposedOrDetached(node)
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);

		// Return True if node is either disposed or no longer attached to TreeView.
		// GetTreeView() returns Null if node has been removed from TreeView, or a
		// TreeView instance different from this one, if attached to another TreeView.
		// NOTICE: Moving a node currently being loaded will not work properly.
		// It will not have its _internal state updated to reflect its proper
		// state. If this is needed, search for calls to nodeDisposedOrDetached(..),
		// and make sure the node is updated if only detached, but do NOT process
		// it any further (e.g. expand it), nor let it affect state within the initial
		// TreeView from which it was moved. And make sure to test it properly!
		return node.GetDomElement() === null || node.GetTreeView() !== me;
	}

	// Loads node data and returns True if data is being loaded, False if no data needs to be loaded.
	// Does not load nodes recursively! Use recursivelyLoadAllNodes(..) for that!
	function loadNodeData(node, cb) // Populated node is passed to callback
	{
		Fit.Validation.ExpectInstance(node, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectFunction(cb, true);

		if (node.GetDomElement()._internal.WSDone === true)
		{
			return false; // Data has already been loaded
		}

		if (node.GetDomElement()._internal.WSHasChildren === false)
		{
			return false; // No remote children to load
		}

		// Get data

		var canceled = !getData(node, false, function(n, eventArgs) // Callback fired when data is ready
		{
			if (nodeDisposedOrDetached(node) === false)
			{
				// Remove place holder child which served the purpose of making the node expandable

				var expanderNode = node.GetChild("__");

				if (expanderNode !== null) // Does not exist if node was partially pre-populated server side
				{
					node.RemoveChild(expanderNode);
				}

				// Populate node

				Fit.Array.ForEach(eventArgs.Children, function(c)
				{
					node.AddChild(createNodeFromJson(c));
				});

				node.GetDomElement()._internal.WSDone = true;
			}

			// Invoke callback

			if (Fit.Validation.IsSet(cb) === true)
			{
				// WARNING: node might have been disposed - WebService communication is async!
				cb(node);
			}
		});

		return canceled === false;
	}

	// Load all children for given node recursively.
	// No arguments are passed to cbComplete - populated node is passed to cbProgress.
	function recursivelyLoadAllNodes(targetNode, cbComplete, cbProgress, isSubCall)
	{
		Fit.Validation.ExpectInstance(targetNode, Fit.Controls.TreeViewNode, true);
		Fit.Validation.ExpectFunction(cbComplete, true);
		Fit.Validation.ExpectFunction(cbProgress, true);
		Fit.Validation.ExpectBoolean(isSubCall, true);

		if (isSubCall !== true)
		{
			recursiveNodeLoadCount = 0;
		}

		var targetNodeInternal = (targetNode || rootNode).GetDomElement()._internal;
		var nodes = null;

		if (targetNode === null)
		{
			nodes = me.GetChildren();
		}
		else
		{
			nodes = targetNode.GetChildren();
		}

		var anythingToLoad = false;

		Fit.Array.ForEach(nodes, function(node)
		{
			if (node.IsBehavioralNode() === true)
				return; // Do not request data for behavioral node

			if (targetNodeInternal.WSSelectAll === true)
				node.GetDomElement()._internal.WSSelectAll = true; // Used when firing OnRequest, OnResponse, and OnPopulated events which expose an eventArgs.SelectAll boolean

			var dataStartedLoading = loadNodeData(node, function(n) // True if node has remote children that will be loaded, False if no remote children is available to be loaded
			{
				var hasBeenDisposed = nodeDisposedOrDetached(node);

				if (hasBeenDisposed === false && Fit.Validation.IsSet(cbProgress) === true)
				{
					cbProgress(node);
				}

				recursiveNodeLoadCount--;

				if (hasBeenDisposed === false)
				{
					recursivelyLoadAllNodes(node, cbComplete, cbProgress, true);
				}

				if (recursiveNodeLoadCount === 0)
				{
					if (Fit.Validation.IsSet(cbComplete) === true)
					{
						cbComplete();
					}
				}
			});

			if (dataStartedLoading === true)
			{
				anythingToLoad = true;
				recursiveNodeLoadCount++;
			}
			else
			{
				// No remote children to load, but node may have existing children that could have remote children

				if (Fit.Validation.IsSet(cbProgress) === true)
				{
					cbProgress(node);
				}

				if (recursivelyLoadAllNodes(node, cbComplete, cbProgress, true) === true)
				{
					anythingToLoad = true;
				}
			}
		});

		if (isSubCall !== true && anythingToLoad === false && Fit.Validation.IsSet(cbComplete) === true)
		{
			cbComplete();
		}

		return anythingToLoad;
	}

	// Progressively expand nodes. Nodes containing remote children will keep
	// expanding when nodes are loaded and populated, which is handled by
	// an OnToggle handler registered in init().
	function setExpandedStateForAllNodes(expandMode, maxDepth)
	{
		Fit.Validation.ExpectInteger(expandMode); // 0 = Collapse, 1 = Expand
		Fit.Validation.ExpectInteger(maxDepth, true);

		if (dataLoading === true || nodesLoading > 0)
		{
			// Data is currently loading - postpone by adding request to process queue.
			// This also applies when collapsing nodes, since the operation currently loading
			// data may be ExpandAll or SelectAll (which also expand nodes), which should be allowed
			// to finish first.
			onDataLoaded(function() { setExpandedStateForAllNodes(expandMode, maxDepth); });
			return;
		}

		// Save state to allow operation to be resumed as nodes are progressively
		// loaded and populated. State is used in OnPopulated handler registered in init().

		expandCollapseAllMode = -1;
		expandCollapseAllMaxDepth = 99999;

		if (Fit.Validation.IsSet(maxDepth) === true)
		{
			if (maxDepth < 1)
				return;

			expandCollapseAllMaxDepth = maxDepth;
		}

		expandCollapseAllMode = expandMode;

		// Load data and start expanding/collapsing

		if (loadDataOnInit === true && expandCollapseAllMode === 1)
		{
			me.Reload(true, function(sender)
			{
				expandCollapseNodesRecursively(me.GetChildren(), expandCollapseAllMode, expandCollapseAllMaxDepth);
			});
		}
		else // Data already loaded
		{
			expandCollapseNodesRecursively(me.GetChildren(), expandCollapseAllMode, expandCollapseAllMaxDepth);
		}
	}

	function expandCollapseNodesRecursively(nodes, expandCollapseMode, maxDepth)
	{
		Fit.Validation.ExpectInstanceArray(nodes, Fit.Controls.TreeViewNode);
		Fit.Validation.ExpectInteger(expandCollapseMode);
		Fit.Validation.ExpectInteger(maxDepth);

		Fit.Array.CustomRecurse(nodes, function(node)
		{
			if (expandCollapseMode === 1)
				node.Expanded(true);
			else if (expandCollapseMode === 0)
				node.Expanded(false);

			return (node.GetLevel() + 1 < maxDepth ? node.GetChildren() : null);
		});
	}

	function fireEventHandlers(handlers, eventArgs)
	{
		var cancel = false;

		Fit.Array.ForEach(handlers, function(cb)
		{
			if (cb(me, eventArgs) === false)
				cancel = true; // Do NOT cancel loop though! All handlers must be fired!
		});

		return !cancel;
	}

	function decodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/%3B/g, ";").replace(/%3D/g, "="); // Decode characters reserved for value format
	}

	function encodeReserved(str)
	{
		Fit.Validation.ExpectString(str);
		return str.replace(/;/g, "%3B").replace(/=/g, "%3D"); // Encode characters reserved for value format
	}

	init();
}

/// <container name="Fit.Controls.WSTreeViewSelectAllMode">
/// 	Enum indicating how data is loaded from WebService when using the Select All feature
/// </container>
Fit.Controls.WSTreeViewSelectAllMode =
{
	/// <member container="Fit.Controls.WSTreeViewSelectAllMode" name="Progressively" access="public" static="true" type="string" default="Progressively">
	/// 	<description>
	/// 		Chain load children by progressively expanding them as they are loaded.
	/// 		This may result in several HTTP requests to WebService, and OnChange will
	/// 		fire for every node expanded if children are available server side.
	/// 	</description>
	/// </member>
	Progressively: "Progressively",

	/// <member container="Fit.Controls.WSTreeViewSelectAllMode" name="Instantly" access="public" static="true" type="string" default="Instantly">
	/// 	<description>
	/// 		Load all children at once (WebService is expected to return the complete hierarchy in one single request).
	/// 		This approach will provide better performance as it does not fire OnChange for every child expanded,
	/// 		and only sends one HTTP request to WebService.
	/// 	</description>
	/// </member>
	Instantly: "Instantly"
};

Fit.Controls.WSTreeView.SelectAllMode = Fit.Controls.WSTreeViewSelectAllMode; // Backward compatibility

Fit.Validation.Enabled(true);
return Fit;})();

	return fitInstance;
});

